OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Jūs sukūrėte naują lentą {board}",
    "{user} has created a new board {board}" : "{user} sukūrė naują lentą {board}",
    "You have deleted the board {board}" : "Jūs ištrynėte lentą {board}",
    "{user} has deleted the board {board}" : "{user} ištrynė lentą {board}",
    "You have restored the board {board}" : "Jūs atkūrėte lentą {board}",
    "{user} has restored the board {board}" : "{user} atkūrė lentą {board}",
    "You have shared the board {board} with {acl}" : "Jūs pradėjote bendrinti lentą {board} su {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} pradėjo bendrinti lentą {board} su {acl}",
    "You have removed {acl} from the board {board}" : "Jūs pašalinote {acl} iš lentos {board}",
    "{user} has removed {acl} from the board {board}" : "{user} pašalino {acl} iš lentos {board}",
    "You have renamed the board {before} to {board}" : "Jūs pervadinote lentą {before} į {board}",
    "{user} has renamed the board {before} to {board}" : "{user} pervadino lentą {before} į {board}",
    "You have archived the board {board}" : "Jūs užarchyvavote lentą {board}",
    "{user} has archived the board {before}" : "{user} užarchyvavo lentą {before}",
    "You have unarchived the board {board}" : "Jūs išarchyvavote lentą {board}",
    "{user} has unarchived the board {before}" : "{user} išarchyvavo lentą {before}",
    "You have created a new list {stack} on board {board}" : "Jūs lentoje {board} sukūrėte naują sąrašą {stack}",
    "{user} has created a new list {stack} on board {board}" : "{user} lentoje {board} sukūrė naują sąrašą {stack}",
    "You have renamed list {before} to {stack} on board {board}" : "Jūs lentoje {board} pervadinote sąrašą {before} į {stack}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} lentoje {board} pervadino sąrašą {before} į {stack}",
    "You have deleted list {stack} on board {board}" : "Jūs lentoje {board} ištrynėte sąrašą {stack}",
    "{user} has deleted list {stack} on board {board}" : "{user} lentoje {board} ištrynė sąrašą {stack}",
    "You have created card {card} in list {stack} on board {board}" : "Jūs lentoje {board}, sąraše {stack} sukūrėte kortelę {card}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} lentoje {board}, sąraše {stack} sukūrė kortelę {card}",
    "You have deleted card {card} in list {stack} on board {board}" : "Jūs lentoje {board}, sąraše {stack} ištrynėte kortelę {card}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} lentoje {board}, sąraše {stack} ištrynė kortelę {card} ",
    "You have renamed the card {before} to {card}" : "Jūs pervadinote kortelę {before} į {card}",
    "{user} has renamed the card {before} to {card}" : "{user} pervadino kortelę {before} į {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Jūs lentoje {board}, sąraše {stack} į kortelę {card} pridėjote aprašą",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} lentoje {board}, sąraše {stack} į kortelę {card} pridėjo aprašą",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Jūs lentoje {board}, sąraše {stack} atnaujinote kortelės {card} aprašą",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} lentoje {board}, sąraše {stack} atnaujino kortelės {card} aprašą",
    "You have archived card {card} in list {stack} on board {board}" : "Jūs lentoje {board}, sąraše {stack} užarchyvavote kortelę {card}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} lentoje {board}, sąraše {stack} užarchyvavo kortelę {card}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Jūs lentoje {board}, sąraše {stack} išarchyvavote kortelę {card}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} lentoje {board}, sąraše {stack} išarchyvavo kortelę {card}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Pažymėjote {card} kortelę kaip atliktą {stack} sąraše {board} lentoje",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} pažymėjo {card} kortelę kaip atliktą {stack} sąraše {board} lentoje",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Pažymėjote {card} kortelę kaip neatšauktą {stack} sąraše {board}lentoje",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} pažymėjo {card} kortelę kaip neatliktą {stack} sąraše {board} lentoje",
    "You have removed the due date of card {card}" : "Jūs pašalinote galutinį kortelės {card} terminą",
    "{user} has removed the due date of card {card}" : "{user} pašalino galutinį kortelės {card} terminą",
    "You have set the due date of card {card} to {after}" : "Jūs nustatėte galutinį kortelės {card} terminą į {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} nustatė galutinį kortelės {card} terminą į {after}",
    "You have updated the due date of card {card} to {after}" : "Jūs atnaujinote galutinį kortelės {card} terminą į {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} atnaujino galutinį kortelės {card} terminą į {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Jūs lentoje {board}, sąraše {stack} į kortelę {card} pridėjote žymą {label}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} lentoje {board}, sąraše {stack} į kortelę {card} pridėjo žymą {label}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Jūs lentoje {board}, sąraše {stack} iš kortelės {card} pašalinote žymą {label}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} lentoje {board}, sąraše {stack} iš kortelės {card} pašalino žymą {label}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Jūs lentoje {board} prie kortelės {card} priskyrėte {assigneduser}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} lentoje {board} prie kortelės {card} priskyrė {assigneduser}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Jūs pašalinote priskyrimą {assigneduser} iš kortelės {card}, esančios lentoje {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} pašalino {assigneduser} priskyrimą {card} ties {board} lenta",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Jūs perkėlėte kortelę {card} iš sąrašo {stackBefore} į sąrašą {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} perkėlė kortelę {card} iš sąrašo {stackBefore} į sąrašą {stack}",
    "You have added the attachment {attachment} to card {card}" : "Jūs pridėjote priedą {attachment} į kortelę {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} pridėjo priedą {attachment} į kortelę {card}",
    "You have updated the attachment {attachment} on card {card}" : "Jūs atnaujinote priedą {attachment} kortelėje {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} atnaujino priedą {attachment} kortelėje {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Jūs ištrynėte priedą {attachment} iš kortelės {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} ištrynė priedą {attachment} iš kortelės {card}",
    "You have restored the attachment {attachment} to card {card}" : "Jūs atkūrėte priedą {attachment} į kortelę {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} atkūrė priedą {attachment} į kortelę {card}",
    "You have commented on card {card}" : "Jūs pakomentavote kortelėje {card}",
    "{user} has commented on card {card}" : "{user} pakomentavo kortelėje {card}",
    "Deck" : "Darbai",
    "Changes in the <strong>Deck app</strong>" : "Pakeitimai <strong>Darbų programėlėje</strong>",
    "A <strong>board, list or card</strong> was changed" : "Buvo pakeista <strong>lentelė, sąrašas arba kortelė</strong>",
    "A <strong>comment</strong> was created on a card" : "Kortelėje buvo sukurtas <strong>komentaras</strong>",
    "A <strong>card description</strong> has been changed" : "Buvo pakeistas <strong>kortelės aprašymas</strong>",
    "The file was uploaded" : "Failas buvo įkeltas",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Įkeliamas failas viršija upload_max_filesize direktyvą, esančią faile php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Įkeltas failas viršija MAX_FILE_SIZE direktyvą, kuri buvo nurodyta HTML formoje",
    "The file was only partially uploaded" : "Failas buvo tik dalinai įkeltas",
    "No file was uploaded" : "Nebuvo įkeltas joks failas",
    "Missing a temporary folder" : "Trūksta laikinojo aplanko",
    "Could not write file to disk" : "Nepavyko įrašyti failą į diską",
    "A PHP extension stopped the file upload" : "PHP plėtinys sustabdė failo įkėlimą",
    "No file uploaded or file size exceeds maximum of %s" : "Neįkeltas joks failas arba failo dydis viršija maksimalų %s dydį",
    "Invalid file type. Only JSON files are allowed." : "Neteisingas failo tipas. Leidžiami tik JSON failai.",
    "Invalid JSON data" : "Netinkami JSON duomenys",
    "Failed to import board" : "Nepavyko importuoti lentos",
    "Cards due today" : "Šiandien baigiamos kortelės",
    "Cards due tomorrow" : "Rytoj baigiamos kortelės",
    "Upcoming cards" : "Artėjančios kortelės",
    "Load more" : "Įkelti daugiau",
    "Welcome to Nextcloud Deck!" : "Sveiki atvykę į „Nextcloud Darbai“!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Kortelę \"%s\" ties \"%s\" priskyrė jums %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} priskyrė jums {deck-card} kortelę iš {deck-board}.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Kortelės „%1$s“ galiojimo laikas baigėsi %2$s.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Kortelė {deck-card}, esanti lentoje {deck-board} pasiekė savo galutinio termino datą.",
    "%s has mentioned you in a comment on \"%s\"." : "%s paminėjo jus komentare ties \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} paminėjo jus komentare ties „{deck-card}“.",
    "The board \"%s\" has been shared with you by %s." : "Lentą \"%s\" su jumis bendrina %s.",
    "{user} has shared {deck-board} with you." : "{user} pasidalino {deck-board} su jumis.",
    "The remote board %1$s has been shared with you by %2$s" : "%1$s nuotolinę lentą su jumis bendrino %2$s",
    "Deck board" : "Darbų lenta",
    "Owned by %1$s" : "Savininkas %1$s",
    "Deck boards, cards and comments" : "Darbų lentos, kortelės ir komentarai",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Iš %1$s, %2$s/%3$s, savininkas %4$s",
    "Create a new deck card" : "Sukurti naują darbų kortelę",
    "Card comments" : "Kortelės komentarai",
    "%s on %s" : "%s ant %s",
    "Deck boards and cards" : "Darbų lentos ir kortelės",
    "No data was provided to create an attachment." : "Priedo sukūrimui nebuvo pateikta jokių duomenų.",
    "Finished" : "Baigta",
    "To review" : "Reikia peržiūrėti",
    "Action needed" : "Reikalingas veiksmas",
    "Later" : "Vėliau",
    "copy" : "kopija",
    "Read more inside" : "Skaitykite daugiau viduje",
    "Custom lists - click to rename!" : "Pasirinktiniai sąrašai – spustelėkite, jei norite pervadinti!",
    "To Do" : "Reikia atlikti",
    "In Progress" : "Vykdoma",
    "Done" : "Atlikta",
    "1. Open to learn more about boards and cards" : "1. Atidarykite, kad sužinotumėte daugiau apie lentas ir korteles",
    "2. Drag cards left and right, up and down" : "2. Vilkite korteles į kairę ir į dešinę, aukštyn ir žemyn",
    "3. Apply rich formatting and link content" : "3. Taikykite raiškūjį formatavimą ir susiekite turinį",
    "4. Share, comment and collaborate!" : "4. Bendrinkite, komentuokite ir bendradarbiaukite!",
    "Create your first card!" : "Sukurkite savo pirmąją kortelę!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Šiame komentare yra daugiau nei %s simbolių.\nPridėta kaip priedas prie kortelės su %s vardu .\nPasiekiama URL adresu: %s.",
    "Attachments" : "Priedai",
    "File" : "Failas",
    "date" : "data",
    "Card not found" : "Kortelė nerasta",
    "Path is already shared with this card" : "Kelias jau yra bendrinamas su šia kortele",
    "Invalid date, date format must be YYYY-MM-DD" : "Netinkama data, datos formatas privalo būti MMMM-mm-dd",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Visos jums priklausančios lentos, įskaitant krūvas, korteles, etiketes, užduotis ir komentarus",
    "Personal planning and team project organization" : "Asmeninis planavimas ir komandinių projektų organizavimas",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "„Darbai “– tai „kanban“ stiliaus organizavimo įrankis, skirtas asmeniniam planavimui ir komandų projektų organizavimui, integruotas su „Nextcloud“.\n\n\n- 📥 Pridėkite užduotis į korteles ir išdėstykite jas tvarkingai\n- 📄 Įrašykite papildomas pastabas „Markdown“ formatu\n- 🔖 Priskirkite žymes, kad viską būtų dar lengviau tvarkyti\n- 👥 Dalinkitės su komanda, draugais ar šeima\n- 📎 Pridėkite failus ir įterpkite juos į savo „Markdown“ aprašymą\n- 💬 Diskutuokite su savo komanda naudodami komentarus\n- ⚡ Sekite pokyčius veiklos sraute\n- 🚀 Sutvarkykite savo projektą",
    "Add board" : "Pridėti lentą",
    "Card details" : "Išsamiau apie kortelę",
    "Select the board to link to a project" : "Pasirinkite lentą, kurią susieti su projektu",
    "Search by board title" : "Ieškoti pagal lentos pavadinimą",
    "Select board" : "Pasirinkti lentą",
    "Move/copy card" : "Perkelti/kopijuoti kortelę",
    "Select a board" : "Pasirinkite lentą",
    "No lists available" : "Nėra sąrašų",
    "Select a list" : "Pasirinkite sąrašą",
    "Move card" : "Perkelti kortelę",
    "Copy card" : "Kopijuoti kortelę",
    "Select the card to link to a project" : "Pasirinkite kortelę, kurią susieti su projektu",
    "Link to card" : "Susieti su kortele",
    "Select a card" : "Pasirinkti kortelę",
    "Cancel" : "Atsisakyti",
    "This board is read only" : "Ši lenta yra tik skaitymui",
    "Drop your files to upload" : "Tempkite failus, norėdami juos išsiųsti",
    "File already exists" : "Failas jau yra",
    "A file with the name {filename} already exists." : "Failas, pavadinimu {filename}, jau yra.",
    "Do you want to overwrite it?" : "Ar norite jį perrašyti?",
    "Overwrite file" : "Perrašyti failą",
    "Keep existing file" : "Palikti esamą failą",
    "Add card" : "Pridėti kortelę",
    "Archived cards" : "Archyvuotos kortelės",
    "Add list" : "Pridėti sąrašą",
    "List name" : "Sąrašo pavadinimas",
    "Clear search" : "Išvalyti paiešką",
    "Active filters" : "Aktyvūs filtrai",
    "Apply filter" : "Taikyti filtrą",
    "Filter by tag" : "Filtruoti pagal žymą",
    "Filter by assigned user" : "Filtruoti pagal priskirtą vartotoją",
    "Unassigned" : "Nepriskirta",
    "Filter by status" : "Filtruoti pagal būseną",
    "Open and completed" : "Atidaryta ir atlikta",
    "Open" : "Atverti",
    "Completed" : "Užbaigta",
    "Filter by due date" : "Filtruoti pagal galutinio termino datą",
    "Overdue" : "Praėjęs galutinis terminas",
    "Next 24 hours" : "Artimiausios 24 val.",
    "Next 7 days" : "Artimiausios 7 dienos",
    "Next 30 days" : "Artimiausios 30 dienų",
    "No due date" : "Be galutinio termino",
    "Clear filter" : "Išvalyti filtrą",
    "View Modes" : "Peržiūros režimai",
    "Toggle View Modes" : "Išskleisti peržiūros režimus",
    "Kanban view" : "Kanban rodinys",
    "Gantt view" : "Gantt rodinys",
    "Hide archived cards" : "Slėpti archyvuotas korteles",
    "Show archived cards" : "Rodyti archyvuotas korteles",
    "Toggle compact mode" : "Perjungti kompaktišką veikseną",
    "Hide card cover images" : "Slėpti kortelės viršelio vaizdus",
    "Show card cover images" : "Rodyti kortelės viršelio vaizdus",
    "Open details" : "Atverti išsamesnę informaciją",
    "Details" : "Išsamiau",
    "Deck settings" : "Darbų nustatymai",
    "General" : "Bendra",
    "Use bigger card view" : "Naudoti didesnį kortelių rodinį",
    "Appearance" : "Išvaizda",
    "Show card ID badge" : "Rodyti kortelės ID ženklelį",
    "Show boards in calendar/tasks" : "Rodyti darbus kalendoriuje/užduotyse",
    "Admin settings" : "Administratoriaus nustatymai",
    "Limit board creation to some groups" : "Apriboti lentų kūrimą tik kai kurioms grupėms",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Vartotojai, nepriklausantys šioms grupėms, negalės kurti savo lentų, bet vis tiek galės dirbti su lentomis, kurios su jais buvo bendrinamos.",
    "Enable federation" : "Įjungti federaciją",
    "Board actions" : "Veiksmai su lentomis",
    "Scroll sideways" : "Slinkti į šoną",
    "Navigate between cards" : "Naršyti tarp kortelių",
    "Close card details" : "Užverti kortelės duomenis",
    "Search" : "Ieškoti",
    "Show card filters" : "Rodyti kortelių filtrus",
    "Clear card filters" : "Išvalyti kortelių filtrus",
    "Show those shortcuts" : "Rodyti tuos sparčiuosius klavišus",
    "Card actions" : "Veiksmai su kortelėmis",
    "Open card details" : "Atverti kortelės duomenis",
    "Edit the card title" : "Redaguoti kortelės pavadinimą",
    "Assign yourself to the current card" : "Priskirkite save dabartinei kortelei",
    "Archive/unarchive the current card" : "Archyvuoti / išarchyvuoti dabartinę kortelę",
    "Mark card as completed/not completed" : "Pažymėti kortelę kaip užpildytą / neužpildytą",
    "Open card menu" : "Atidarykite kortelės meniu",
    "Currently present people" : "Šiuo metu esami žmonės",
    "Supported prefixes: {prefixes}. Wrap phrases in double quotes." : "Palaikomi priešdėliai: {prefixes}. Frazes įrašykite tarp dvigubų kabučių.",
    "Search cards" : "Ieškoti kortelių",
    "Loading board" : "Įkeliama lenta",
    "Board not found" : "Lenta nerasta",
    "Create a new list to add cards to this board" : "Norėdami pridėti korteles į šią lentą, sukurkite naują sąrašą",
    "Sharing" : "Bendrinimas",
    "Tags" : "Žymos",
    "Deleted items" : "Ištrinti elementai",
    "Activity" : "Veikla",
    "Deleted lists" : "Ištrinti sąrašai",
    "Undo" : "Atšaukti",
    "Deleted cards" : "Ištrintos kortelės",
    "Hours" : "Valandos",
    "Days" : "Dienos",
    "Weeks" : "Savaitės",
    "Months" : "Mėnesiai",
    "No cards yet" : "Kol kas kortelių nėra",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Nustatykite kortelių pradžios ir pabaigos datas, kad jas būtų galima matyti Gantt diagramoje",
    "Cards without dates ({count})" : "Kortelės be datos ({count})",
    "Failed to create share with {displayName}" : "Nepavyko sukurti bendrinimo su {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Ar tikrai norite {title} lentą perkelti {user}?",
    "Transfer the board." : "Perkelti lentą.",
    "Transfer" : "Persiųsti",
    "The board has been transferred to {user}" : "Lenta buvo perkelta {user}",
    "Failed to transfer the board to {user}" : "Nepavyko perkelti lentos {user}",
    "Board owner" : "Lentos savininkas",
    "(Group)" : "(Grupė)",
    "(Team)" : "(Komanda)",
    "(remote)" : "(nuotolinis)",
    "Can edit" : "Gali redaguoti",
    "Can share" : "Gali bendrinti",
    "Can manage" : "Gali tvarkyti",
    "Owner" : "Savininkas",
    "Delete" : "Ištrinti",
    "List deleted" : "Sąrašas ištrintas",
    "Edit list title" : "Redaguoti sąrašo pavadinimą",
    "Archive all cards" : "Archyvuoti visas korteles",
    "Unarchive all cards" : "Išarchyvuoti visas korteles",
    "Do not set cards as \"done\"" : "Nenustatyti korteles kaip „atlikta“",
    "Set cards as \"done\"" : "Nustatyti korteles kaip „atlikta“",
    "Delete list" : "Ištrinti sąrašą",
    "Archive all cards in this list" : "Archyvuoti visas korteles šiame sąraše",
    "Unarchive all cards in this list" : "Išarchyvuoti visas korteles šiame sąraše",
    "Add a new card" : "Pridėti naują kortelę",
    "Card name" : "Kortelės pavadinimas",
    "title and color value must be provided" : "privalo būti pateiktos pavadinimo ir spalvos reikšmės",
    "Edit" : "Taisyti",
    "Add a new tag" : "Pridėti naują žymą",
    "Search boards" : "Ieškoti lentų",
    "Board name" : "Lentos pavadinimas",
    "Members" : "Nariai",
    "Assign to users/groups/team" : "Priskirti vartotojams/grupėms/komandoms",
    "Assign a user to this card…" : "Priskirti šiai kortelei vartotoją…",
    "Select a user to assign to this card…" : "Pasirinkite vartotoją, kurį norite priskirti šiai kortelei…",
    "File to share" : "Failas, kurį bendrinti",
    "Invalid path selected" : "Pasirinktas neteisingas kelias",
    "Upload new files" : "Išsiųsti naujus failus",
    "Share from Files" : "Bendrinti iš „Failai“",
    "Pending share" : "Laukiama bendrinimo",
    "Add this attachment" : "Pridėti šį priedą",
    "Show in Files" : "Rodyti failuose",
    "Download" : "Atsisiųsti",
    "Remove attachment" : "Šalinti priedą",
    "Delete Attachment" : "Ištrinti priedą",
    "Restore Attachment" : "Atkurti priedą",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Atnaujinta: {modifiedDate} ⸱ Sukurta {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Atnaujinta: {modifiedDate} \nSukurta: {createdDate} \nSukūrė: {owner} ",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Atnaujinta: {modifiedDate} \nSukurta: {createdDate}",
    "The title cannot be empty." : "Pavadinimas negali būti tuščias.",
    "Cannot close unsaved card!" : "Neįmanoma uždaryti neišsaugotos kortelės!",
    "Open in sidebar view" : "Atverti šoninės juostos rodinyje",
    "Open in bigger view" : "Atverti didesniame rodinyje",
    "Comments" : "Komentarai",
    "Failed to load comments" : "Nepavyko įkelti komentarų",
    "No comments yet. Begin the discussion!" : "Kol kas komentarų nėra. Pradėkite diskusiją!",
    "The comment cannot be empty." : "Komentaras negali būti tuščias.",
    "The comment cannot be longer than 1000 characters." : "Komentaras negali būti ilgesnis nei 1000 simbolių.",
    "Submit" : "Pateikti",
    "Created:" : "Sukurta:",
    "In reply to" : "Atsakant į",
    "Cancel reply" : "Atšaukti atsakymą",
    "Reply" : "Atsakyti",
    "Update" : "Atnaujinti",
    "Assign dependent cards" : "Priskirti priklausomas korteles",
    "Mark as done" : "Žymėti kaip atliktą",
    "Mark as not done" : "Žymėti kaip neatliktą",
    "Remove dependency" : "Pašalinkite priklausomybę",
    "Add dependent card" : "Pridėkite priklausomą kortelę",
    "Assign a dependent card…" : "Priskirti priklausomą kortelę…",
    "Select a dependent card…" : "Pasirinkite priklausomą kortelę…",
    "Write a description …" : "Parašykite aprašą…",
    "The description has been changed by another user." : "Aprašymą pakeitė kitas vartotojas.",
    "Could not save description" : "Nepavyko išsaugoti aprašymo",
    "Description" : "Aprašas",
    "(Unsaved)" : "(Neįrašyta)",
    "(Saving…)" : "(Įrašoma…)",
    "Formatting help" : "Formatavimo pagalba",
    "Edit description" : "Taisyti aprašą",
    "View description" : "Rodyti aprašą",
    "Add Attachment" : "Pridėti priedą",
    "Choose attachment" : "Pasirinkite priedą",
    "Select Date" : "Pasirinkti datą",
    "Later today – {timeLocale}" : "Šiandien vėliau – {timeLocale}",
    "Set due date for later today" : "Nustatyti terminą vėliau šiandien",
    "Tomorrow – {timeLocale}" : "Rytoj – {timeLocale}",
    "Set due date for tomorrow" : "Nustatyti terminą rytojui",
    "This weekend – {timeLocale}" : "Šį savaitgalį – {timeLocale}",
    "Set due date for this weekend" : "Nustatyti terminą šiam savaitgaliui",
    "Next week – {timeLocale}" : "Kitą savaitę – {timeLocale}",
    "Set due date for next week" : "Nustatyti terminą kitai savaitei",
    "Assign a due date to this card…" : "Nustatyti šios kortelės įvykdymo datą…",
    "Set a due date" : "Nustatyti galutinį terminą",
    "Add due date" : "Pridėti galutinį terminą",
    "Choose a date" : "Pasirinkite datą",
    "Remove due date" : "Šalinti galutinio termino datą",
    "Due at:" : "Terminas:",
    "Not done" : "Nepadaryta",
    "Unarchive card" : "Išarchyvuoti kortelę",
    "Archive card" : "Archyvuoti kortelę",
    "Assign a start date to this card…" : "Priskirkite šiai kortelei pradžios datą…",
    "Set a start date" : "Nustatyti pradžios datą",
    "Add start date" : "Pridėti pradžios datą",
    "Remove start date" : "Pašalinti pradžios datą",
    "Assign a tag to this card…" : "Priskirti šiai kortelei žymą…",
    "Select or create a tag…" : "Pasirinkite arba sukurkite žymą…",
    "Create a new tag:" : "Sukurkite naują žymą:",
    "(group)" : "(grupė)",
    "{count} comments, {unread} unread" : "Komentarų: {count}, neskaityta: {unread}",
    "Todo items" : "Atliktini veiksmai",
    "Edit card title" : "Redaguoti kortelės pavadinimą",
    "Open link" : "Atverti nuorodą",
    "Card deleted" : "Kortelė ištrinta",
    "Change card color" : "Keisti kortelės spalvą",
    "Edit title" : "Taisyti pavadinimą",
    "Assign to me" : "Priskirti sau",
    "Unassign myself" : "Atšaukti savo priskyrimą",
    "Delete card" : "Ištrinti kortelę",
    "All boards" : "Visos lentos",
    "Archived boards" : "Archyvuotos lentos",
    "Shared with you" : "Bendrinama su jumis",
    "Cancel edit" : "Atsisakyti redagavimo",
    "Save board" : "Išsaugoti lentą",
    "Board {0} deleted" : "Lenta {0} ištrinta",
    "All cards" : "Visos kortelės",
    "Only assigned cards" : "Tik priskirtos kortelės",
    "No reminder" : "Jokio priminimo",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Ar tikrai norite ištrinti {title} lentą? Tai ištrins visus šios lentos duomenis, įskaitant archyvuotas korteles.",
    "Delete the board?" : "Ištrinti lentą?",
    "Are you sure you want to leave the board {title}?" : "Ar tikrai norite palikti {title} lentą?",
    "Leave the board?" : "Išeiti iš lentos?",
    "Leave" : "Išeiti",
    "Failed to leave the board" : "Nepavyko išeiti iš lentos",
    "Exporting board..." : "Eksportuojama lenta...",
    "An error occurred" : "Įvyko klaida",
    "Board details" : "Išsamiau apie lentą",
    "Edit board" : "Redaguoti lentą",
    "Clone board" : "Dubliuoti lentą",
    "Unarchive board" : "Išarchyvuoti lentą",
    "Archive board" : "Archyvuoti lentą",
    "Export board" : "Eksportuoti lentą",
    "Turn on due date reminders" : "Įjungti priminimus apie galutinio termino datą",
    "Turn off due date reminders" : "Išjungti priminimus apie galutinio termino datą",
    "Remove as default board" : "Pašalinti kaip numatytąją lentą",
    "Set as default board" : "Nustatyti kaip numatytąją lentą",
    "Due date reminders" : "Priminimai apie galutinio termino datą",
    "Assigned cards" : "Priskirtos kortelės",
    "No notifications" : "Pranešimų nėra",
    "Delete board" : "Ištrinti lentą",
    "Leave board" : "Išeiti iš lentos",
    "Importing board..." : "Importuojama lenta...",
    "Board imported successfully" : "Lenta sėkmingai importuota",
    "Import board" : "Importuoti lentą",
    "Clone {boardTitle}" : "Klonuoti {boardTitle}",
    "Clone cards" : "Dubliuoti korteles",
    "Clone assignments" : "Klonuoti užduotis",
    "Clone labels" : "Klonuoti etiketes",
    "Clone due dates" : "Klonuoti galutinį terminą",
    "Advanced options" : "Išplėstinės parinktys",
    "Move all cards to the first list" : "Perkelti visas korteles į pirmąjį sąrašą",
    "Restore archived cards" : "Atkurti archyvuotas korteles",
    "Clone" : "Dubliuoti",
    "Export {boardTitle}" : "Eksportuoti {boardTitle}",
    "Export as JSON" : "Eksportuoti kaip JSON",
    "Export as CSV" : "Eksportuoti kaip CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Pastaba: importuojant atgal į „Darbų“ programėlę palaikomas tik JSON formatas.",
    "Export" : "Eksportuoti",
    "Loading filtered view" : "Įkeliama filtruota peržiūra",
    "Search for {searchQuery} in other boards" : "Ieškoti {searchQuery} kitose lentose",
    "Search for {searchQuery} in all boards" : "Ieškoti {searchQuery} visose lentose",
    "No results found" : "Nerasta jokių rezultatų",
    "Deck board {name}\n* Last modified on {lastMod}" : "Darbų lenta {name}\n* Paskutinį kartą redaguota {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Sukurta  {created}\n* Paskutinį kartą pakeista  {lastMod}\n* {nbAttachments} priedai\n* {nbComments} komentarai",
    "{nbCards} cards" : "{nbCards} kortelės",
    "Due on {date}" : "Galutinis terminas {date}",
    "{stack} in {board}" : "{stack} lentoje {board}",
    "Click to expand description" : "Spustelėkite norėdami išskleisti aprašą",
    "Click to expand comment" : "Spustelėkite norėdami išskleisti komentarą",
    "Create card" : "Sukurti kortelę",
    "Create a new card" : "Sukurti naują kortelę",
    "Card title" : "Kortelės pavadinimas",
    "Creating the new card …" : "Kuriama nauja kortelė…",
    "Card \"{card}\" was added to \"{board}\"" : "Kortelė „{card}“ buvo pridėta į lentą „{board}“",
    "Open card" : "Atverti kortelę",
    "Close" : "Užverti",
    "No upcoming cards" : "Artėjančių kortelių nėra",
    "upcoming cards today" : "artėjančios šiandienai kortelės",
    "upcoming cards tomorrow" : "artėjančios rytdienai kortelės",
    "upcoming cards" : "artėjančios kortelės",
    "New card" : "Nauja kortelė",
    "Link to a board" : "Susieti su lenta",
    "Link to a card" : "Susieti su kortele",
    "Create a card" : "Sukurti kortelę",
    "Message from {author} in {conversationName}" : "{author} žinutės {conversationName}",
    "Something went wrong" : "Kažkas nutiko",
    "Failed to upload {name}" : "Nepavyko įkelti {name}",
    "Maximum file size of {size} exceeded" : "Viršytas maksimalus {size} failo dydis",
    "Assigned users" : "Priskirti vartotojai",
    "Due date" : "Galutinio termino data",
    "Created" : "Sukurta",
    "Modified" : "Pakeistas",
    "Error creating the share" : "Klaida kuriant bendrinimą",
    "Share with a Deck card" : "Pasidalinti su Darbų kortele",
    "Share {file} with a Deck card" : "Pasidalinti {file} su Darbų kortele",
    "Share" : "Bendrinti",
    "The card \"%s\" on \"%s\" has reached its due date." : "Kortelė „%s“, esanti lentoje „%s“, pasiekė savo galutinį terminą.",
    "Share board with a user, group or team …" : "Bendrinti lentą su vartotoju, grupe ar komanda…",
    "Searching for users, groups and teams …" : "Ieškoti vartotojų, grupių ar komandų…",
    "No participants found" : "Nerasta jokių dalyvių",
    "Save" : "Įrašyti",
    "seconds ago" : "prieš keletą sekundžių",
    "Today" : "Šiandien",
    "Tomorrow" : "Rytoj",
    "No due" : "Be galutinio termino"
},
"nplurals=4; plural=(n % 10 == 1 && (n % 100 > 19 || n % 100 < 11) ? 0 : (n % 10 >= 2 && n % 10 <=9) && (n % 100 > 19 || n % 100 < 11) ? 1 : n % 1 != 0 ? 2: 3);");
