OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "As creadu una lavagna noa {board}",
    "{user} has created a new board {board}" : "{user} at creadu una lavagna noa {board}",
    "You have deleted the board {board}" : "As cantzelladu sa lavagna {board}",
    "{user} has deleted the board {board}" : "{user} at cantzelladu sa lavagna {board}",
    "You have restored the board {board}" : "As ripristinadu sa lavagna {board}",
    "{user} has restored the board {board}" : "{user} at ripristinadu sa lavagna {board}",
    "You have shared the board {board} with {acl}" : "As cumpartzidu sa lavagna {board} cun {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} at cumpartzidu sa lavagna {board} cun {acl}",
    "You have removed {acl} from the board {board}" : "Ci nd'as bogadu {acl} dae sa lavagna {board}",
    "{user} has removed {acl} from the board {board}" : "{user} ci nd'at bogadu {acl} dae sa lavagna {board}",
    "You have renamed the board {before} to {board}" : "As torradu a numenare sa lavagna {before} comente {board}",
    "{user} has renamed the board {before} to {board}" : "{user} at torradu a numenare {before} comente {board}",
    "You have archived the board {board}" : "As archiviadu sa lavagna {board}",
    "{user} has archived the board {before}" : "{user} at archiviadu sa lavagna {before}",
    "You have unarchived the board {board}" : "As annulladu s'archiviatzione de sa lavagna {board}",
    "{user} has unarchived the board {before}" : "{user} at annulladu s'archiviatzione de sa lavagna {before}",
    "You have created a new list {stack} on board {board}" : "As creadu un'elencu nou {stack} in sa lavagna {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} at creadu un'elencu nou {stack} in sa lavagna {board}",
    "You have renamed list {before} to {stack} on board {board}" : "As torradu a numenare s'elencu {before} comente {stack} in sa {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} at torradu a numenare s'elencu {before} comente {stack} sulla lavagna {board}",
    "You have deleted list {stack} on board {board}" : "As cantzelladu s'elencu {stack} in sa lavagna {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} at cantzelladu s'elencu {stack} in sa lavagna {board}",
    "You have created card {card} in list {stack} on board {board}" : "As creadu un'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} at creadu un'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "As cantzelladu s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} at cantzelladu s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "You have renamed the card {before} to {card}" : "As torradu a numenare s'ischeda {before} comente {card}",
    "{user} has renamed the card {before} to {card}" : "{user} at torradu a numenare sa carta {before} comente {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "As agiuntu una descritzione a s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} at agiuntu una descritzione a s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "As agiornadu sa descritzione de s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} at agiornadu sa descritzione de s'ischedas {card} in s'elencu {stack} in sa lavagna {board}",
    "You have archived card {card} in list {stack} on board {board}" : "As archiviadu s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} at archiviadu s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "As annulladu s'archiviatzione de s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} at annulladu s'archiviatzione de s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "You have removed the due date of card {card}" : "Ci nd'as bogadu sa data de iscadèntzia dae s'ischeda {card}",
    "{user} has removed the due date of card {card}" : "{user} ci nd'at bogadu sa data de iscadèntzia dae s'ischeda {card}",
    "You have set the due date of card {card} to {after}" : "As cunfiguradu sa data de iscadèntzia de s'ischeda {card} a su {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} at cunfiguradu sa data de iscadèntzia de sa tèssera {card} a su {after}",
    "You have updated the due date of card {card} to {after}" : "As agiornadu sa data de iscadèntzia de s'ischeda {card} a su {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} as agiornadu sa data de iscadèntzia de s'ischeda {card} a su {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "As agiuntu s'eticheta {label} a s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} at agiuntu s'eticheta {label} a s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Ci nd'as bogadu {label} dae s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} ci nd'at bogadu s'eticheta {label} dae s'ischeda {card} in s'elencu {stack} in sa lavagna {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "As assignadu {assigneduser} a s'ischeda {card} in sa lavagna {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} at assignadu {assigneduser} a s'ischeda {card} in sa lavagna {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "As annulladu s'assignatzione {assigneduser} dae s'ischeda {card} in sa lavagna {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} at annulladu s'assignatzione {assigneduser} dae s'ischeda {card} in sa lavagna {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "As tramudadu s'ischeda {card} dae s'elencu {stackBefore} a {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} at tramudadu s'ischeda {card} dae s'ischeda {stackBefore} a {stack}",
    "You have added the attachment {attachment} to card {card}" : "As agiuntu s'alligongiadu {attachment} a s'ischeda {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} at agiuntu s'alligongiadu {attachment} a s'ischeda {card}",
    "You have updated the attachment {attachment} on card {card}" : "As agiornadu s'alligongiadu {attachment} a s'ischeda {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} at agiornadu s'alligongiadu {attachment} a s'ischeda {card}",
    "You have deleted the attachment {attachment} from card {card}" : "As cantzelladu s'alligongiadu {attachment} dae s'ischeda {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} at eliminadu s'alligongiadu {attachment} dae s'ischeda {card}",
    "You have restored the attachment {attachment} to card {card}" : "As ripristinadu s'alligongiadu {attachment} in s'ischeda {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} at ripristinadu s'alligongiadu {attachment} in s'ischeda {card}",
    "You have commented on card {card}" : "As cummentadu in s'ischeda {card}",
    "{user} has commented on card {card}" : "{user} at cummentadu in s'ischeda {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Càmbios in <strong> s'aplicatzione Deck</strong>",
    "A <strong>comment</strong> was created on a card" : "Unu <strong>cummentu</strong> est istadu creadu in s'ischeda",
    "The file was uploaded" : "S'archìviu est istadu carrigadu",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "S'archìviu carrigadu superat sa diretiva upload_max_filesize in php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "S'archìviu carrigadu superat sa diretiva MAX_FILE_SIZE ispetzificada in su mòdulu HTML",
    "The file was only partially uploaded" : "Isceti una parte de s'archìviu est carrigada",
    "No file was uploaded" : "Perunu archìviu carrigadu",
    "Missing a temporary folder" : "Mancat una cartella temporànea",
    "Could not write file to disk" : "No at fatu a iscrìere s'archìviu in su discu",
    "A PHP extension stopped the file upload" : "Un'estensione PHP at firmadu su carrigamentu de s'archìviu",
    "No file uploaded or file size exceeds maximum of %s" : "Perun'archìviu carrigadu o sa mannària de s'archìviu superat su màssimu de %s",
    "Upcoming cards" : "Ischedas imbenientes",
    "Load more" : "Càrriga àteru",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "S'ischeda \"%s\" in \"%s\" s'est istadu assignadu dae %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} t'at assignadu s'ischeda {deck-card} in {deck-board}.",
    "The card {deck-card} on {deck-board} has reached its due date." : "S'ischeda {deck-card} in {deck-board} est cròmpida a sa data de iscadèntzia.",
    "%s has mentioned you in a comment on \"%s\"." : "%s t'at mentovadu in unu cummentu \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} t'at mentovadu in unu cummentu in {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Sa lavagna \"%s\" est istada cumpartzida cun tegus dae %s.",
    "{user} has shared {deck-board} with you." : "{user} at cumpartzidu {deck-board} cun tegus.",
    "Deck board" : "Tabella in Deck",
    "Card comments" : "Cummentos de is ischedas",
    "%s on %s" : "%s in %s",
    "No data was provided to create an attachment." : "Perunu datu frunidu pro creare un'alligongiadu",
    "Finished" : "Agabbadu",
    "To review" : "De revisionare",
    "Action needed" : "Bisòngiat un'atzione",
    "Later" : "Luego",
    "copy" : "còpia",
    "To Do" : "De fàghere",
    "Done" : "Fatu",
    "Attachments" : "Alligongiados",
    "File" : "Archìviu",
    "Card not found" : "Ischeda no agatada",
    "Path is already shared with this card" : "Su caminu est giai cumpartzidu cun cust'ischeda",
    "Invalid date, date format must be YYYY-MM-DD" : "Data non bàlida, su formadu de sa data depet èssere AAAA-MM-GG",
    "Personal planning and team project organization" : "Pranificatzione personale e organizatzione de su progetu de iscuadra",
    "Add board" : "Agiunghe lavagna",
    "Card details" : "Detàllios ischeda",
    "Select the board to link to a project" : "Seletziona lavagna de collegare a unu progetu",
    "Search by board title" : "Chirca dae su tìtulu de sa lavagna",
    "Select board" : "Seletziona lavagna",
    "Select a board" : "Seletziona una lavagna",
    "No lists available" : "Perunu elencu a disponimentu",
    "Select a list" : "Seletziona un'elencu",
    "Move card" : "Tràmuda ischeda",
    "Select the card to link to a project" : "Seletzioan s'ischeda de collegare a unu progetu",
    "Link to card" : "Collega a un'ischeda",
    "Select a card" : "Seletziona un'ischeda",
    "Cancel" : "Annulla",
    "This board is read only" : "Custa lavagna est isceti pro lèghere",
    "Drop your files to upload" : "Lassa is archìvios pro ddos carrigare",
    "File already exists" : "Custu archìviu ddoe est giai",
    "A file with the name {filename} already exists." : "B'at giai un'archìviu cun su nùmene de {filename}.",
    "Do you want to overwrite it?" : "Ddu cheres subraiscrìere? ",
    "Overwrite file" : "Subraiscrie archìviu",
    "Keep existing file" : "Mantene s'archìviu chi ddoe est",
    "Add card" : "Agiunghe ischeda",
    "Archived cards" : "Ischedas archiviadas",
    "Add list" : "Agiunghe elencu",
    "List name" : "Nùmene elencu",
    "Apply filter" : "Àplica filtru",
    "Filter by tag" : "Filtra pro etichetas",
    "Filter by assigned user" : "Filtra pro utente assignadu",
    "Unassigned" : "No assignadu",
    "Open" : "Aberi",
    "Completed" : "Cumpletadu",
    "Filter by due date" : "Filtra pro data de iscadèntzia",
    "Overdue" : "Iscadèntzia",
    "Next 24 hours" : "24 oras imbenientes",
    "Next 7 days" : "7 dies imbenientes",
    "Next 30 days" : "30 dies imbenientes",
    "No due date" : "Peruna iscadèntzia",
    "Clear filter" : "Lìmpia filtru",
    "Hide archived cards" : "Cua is ischedas archiviadas",
    "Show archived cards" : "Mustra ischedas archiviadas",
    "Toggle compact mode" : "Càmbia a modalidade cumpata",
    "Open details" : "Aberi is detàllios",
    "Details" : "Detàllios",
    "Use bigger card view" : "Imprea bista cun ischedas prus mannas",
    "Appearance" : "Aspetu",
    "Show boards in calendar/tasks" : "Mustra lavagnas in calendàriu/fainas",
    "Search" : "Chirca",
    "Loading board" : "Carrighende lavagna",
    "Board not found" : "Lavagna no agatada",
    "Create a new list to add cards to this board" : "Crea un'elencu nou de agiùnghere ischedas a custa lavagna",
    "Sharing" : "Cumpartzidura",
    "Tags" : "Etichetas",
    "Deleted items" : "Temas cantzellados",
    "Activity" : "Atividade",
    "Deleted lists" : "Elencos cantzellados",
    "Undo" : "Annulla",
    "Deleted cards" : "Ischeda cantzellada",
    "Weeks" : "Chidas",
    "Months" : "Meses",
    "No cards yet" : "Perun'ischeda ancora",
    "Failed to create share with {displayName}" : "No at fatu a creare cumpartzidura cun {displayName}",
    "Transfer" : "Tràmuda",
    "Board owner" : "Mere lavagna",
    "(Group)" : "(Grupu)",
    "(remote)" : "(remotu)",
    "Can edit" : "Faghet a modificare",
    "Can share" : "Faghet a cumpartzire",
    "Can manage" : "Faghet a gestire",
    "Owner" : "Mere",
    "Delete" : "Cantzella",
    "List deleted" : "Elencu cantzelladu",
    "Archive all cards" : "Archìvia totu is ischedas",
    "Delete list" : "Cantzella elencu",
    "Archive all cards in this list" : "Archìvia totu is ischedas in cust'elencu",
    "Add a new card" : "Agiùnghe un'ischeda noa",
    "Card name" : "Nùmene de s'ischeda",
    "title and color value must be provided" : "tocat de frunire su tìtulu e su balore de su colore",
    "Edit" : "Modìfica ",
    "Add a new tag" : "Agiunghe un'eticheta noa",
    "Board name" : "Nùmene de sa lavagna",
    "Members" : "Membros",
    "Assign a user to this card…" : "Assigna utente a cust'ischeda...",
    "File to share" : "Archìviu de cumpartzire",
    "Invalid path selected" : "Percursu seletzionadu non bàlidu",
    "Upload new files" : "Càrriga archìvios noos",
    "Share from Files" : "Cumpartzi dae Archìvios",
    "Add this attachment" : "Agiunghe custu alligongiadu",
    "Show in Files" : "Mustra in Archìvios",
    "Download" : "Iscàrriga",
    "Delete Attachment" : "Cantzella alligongiadu",
    "Restore Attachment" : "Riprìstina alligongiadu",
    "The title cannot be empty." : "Su tìtulu non podet èssere bòidu",
    "Open in sidebar view" : "Aberi in s'istanca laterale",
    "Open in bigger view" : "Aberi in una bista prus ampra",
    "Comments" : "Cummentos",
    "Failed to load comments" : "No at fatu a carrigare is cummentos",
    "No comments yet. Begin the discussion!" : "Perunu cummentu ancora. Cumintzat sa chistionada!",
    "The comment cannot be empty." : "Su cummentu non podet èssere bòidu",
    "The comment cannot be longer than 1000 characters." : "Su cummentu non podet èssere prus longu de 1000 caràteres.",
    "Submit" : "Imbia",
    "Created:" : "Creadu:",
    "In reply to" : "Rispondende a ",
    "Cancel reply" : "Annulla risposta",
    "Reply" : "Risponde",
    "Update" : "Agiorna",
    "Mark as done" : "Marca comente cumpletadu",
    "Write a description …" : "Iscrie una descritzione ...",
    "Description" : "Descritzione",
    "(Unsaved)" : "(Non sarvada)",
    "(Saving…)" : "(Sarbende…)",
    "Formatting help" : "Ghia pro sa formatatzione",
    "Edit description" : "Modìfica descritzione",
    "View description" : "Visualiza descritzione",
    "Add Attachment" : "Agiunghe alligongiadu",
    "Choose attachment" : "Sèbera un'alligongiadu",
    "Select Date" : "Seletziona data",
    "Set a due date" : "Cunfigura data de iscadèntzia",
    "Remove due date" : "Boga s'iscadèntzia",
    "Unarchive card" : "Ischeda no archiviada",
    "Archive card" : "Archìviu no archiviadu",
    "Assign a tag to this card…" : "Assigna un'eticheta a cust'ischeda...",
    "Create a new tag:" : "Crea un'eticheta noa:",
    "(group)" : "(grupu)",
    "{count} comments, {unread} unread" : "{count} cummentos, {unread} non lèghidos",
    "Card deleted" : "Ischeda cantzellada",
    "Edit title" : "Modìfica tìtulu",
    "Assign to me" : "Assigna a mie",
    "Unassign myself" : "Annulla s'assignatzione a mie",
    "Delete card" : "Cantzella ischeda",
    "All boards" : "Totu is lavagnas",
    "Archived boards" : "Lavagnas archiviadas",
    "Shared with you" : "Cumpartzidu cun tegus",
    "Cancel edit" : "Annulla modìfica",
    "Board {0} deleted" : "Lavagna {0} cantzellada",
    "All cards" : "Totu is ischedas",
    "Only assigned cards" : "Isceti ischedas assignadas",
    "No reminder" : "Perunu apuntu",
    "Delete the board?" : "Cheres cantzellare sa lavagna?",
    "An error occurred" : "B'at àpidu un'errore",
    "Board details" : "Detàllios lavagna",
    "Edit board" : "Modìfica lavagna",
    "Clone board" : "Clona lavagna",
    "Unarchive board" : "Annulla s'archiviatzione de sa lavagna",
    "Archive board" : "Archìvia lavagna",
    "Turn on due date reminders" : "Allughe is notìficas pro ammentare is iscadèntzias",
    "Turn off due date reminders" : "Istuda is notìficas pro ammentare is iscadèntzias",
    "Due date reminders" : "Notìficas pro ammentare is iscadèntzias",
    "Assigned cards" : "Ischedas assignadas",
    "No notifications" : "Peruna notìfica",
    "Delete board" : "Cantzella lavagna",
    "Clone cards" : "Clona ischeda",
    "Advanced options" : "Sèberos avantzados",
    "Clone" : "Clona",
    "Export" : "Esporta",
    "Loading filtered view" : "Carrigamentu de sa bista cun su filtru",
    "Search for {searchQuery} in all boards" : "Chirca {searchQuery} in totu is lavagnas",
    "No results found" : "Perunu resurtadu agatadu",
    "{stack} in {board}" : "{stack} in {board}",
    "Create card" : "Crea ischeda",
    "Create a new card" : "Crea un'ischeda noa",
    "Card title" : "Tìtulu ischeda",
    "Creating the new card …" : "Creende s'ischeda noa ...",
    "Card \"{card}\" was added to \"{board}\"" : "S'ischeda \"{card}\" est istada agiunta a \"{board}\"",
    "Open card" : "Aberi ischeda",
    "Close" : "Serra",
    "No upcoming cards" : "Peruna ischeda abarrada",
    "upcoming cards" : "ischedas abarradas",
    "New card" : "Carta noa",
    "Link to a board" : "Collega a una tabella",
    "Link to a card" : "Collega a un'ischeda",
    "Create a card" : "Crea un'ischeda",
    "Message from {author} in {conversationName}" : "Messàgiu de {author} in {conversationName}",
    "Something went wrong" : "Ddoe at àpidu un'errore",
    "Failed to upload {name}" : "No at fatu a agiornare {name}",
    "Maximum file size of {size} exceeded" : "Mannària màssima de s'archìviu de {size} superada",
    "Due date" : "Data de fine",
    "Created" : "Creadu",
    "Modified" : "Modificadu",
    "Error creating the share" : "Errore in sa creatzione de sa cumpatzidura",
    "Share with a Deck card" : "Cumpartzi cun un'ischeda deck",
    "Share {file} with a Deck card" : "Cumpartzi {file} cun un'ischeda de deck",
    "Share" : "Cumpartzi",
    "The card \"%s\" on \"%s\" has reached its due date." : "S'ischeda \"%s\" in \"%s\" est cròmpida a sa data de iscadèntzia.",
    "No participants found" : "Perunu partetzipante agatadu",
    "Save" : "Sarva",
    "seconds ago" : "segundos a immoe",
    "Today" : "Oe",
    "Tomorrow" : "Cras",
    "No due" : "Peruna iscadèntzia"
},
"nplurals=2; plural=(n != 1);");
