<?php

declare(strict_types=1);

namespace Psalm\Node\Name;

use PhpParser\Node\Name\FullyQualified;
use Psalm\Node\VirtualNode;

final class VirtualFullyQualified extends FullyQualified implements VirtualNode
{

}
