OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Креиравте нова табла {board}",
    "{user} has created a new board {board}" : "{user} креирање нова табла {board}",
    "You have deleted the board {board}" : "Избришавте табла {board}",
    "{user} has deleted the board {board}" : "{user} избриша табла {board}",
    "You have restored the board {board}" : "Вративте табла {board}",
    "{user} has restored the board {board}" : "{user} врати табла {board}",
    "You have shared the board {board} with {acl}" : "Ја споделивте таблата {board} со {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} ја сподели таблата {board} со {acl}",
    "You have removed {acl} from the board {board}" : "Го избришавте {acl} од таблата {board}",
    "{user} has removed {acl} from the board {board}" : "{user} го избриша {acl} од таблата {board}",
    "You have renamed the board {before} to {board}" : "Ја преименувавте таблата {before} во {board}",
    "{user} has renamed the board {before} to {board}" : "{user} ја преименуваше таблата {before} во {board}",
    "You have archived the board {board}" : "Ја архивиравте таблата {board}",
    "{user} has archived the board {before}" : "{user} ја архивираше таблата {before}",
    "You have unarchived the board {board}" : "Ја вративте од архива таблата {board}",
    "{user} has unarchived the board {before}" : "{user} ја врати од архива таблата {before}",
    "You have created a new list {stack} on board {board}" : "Креиравте нова листа {stack} на таблата {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} креирање нова листа {stack} на таблата {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Ја преименувавте листа {before} во {stack} на таблата {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} ја преименуваше листата {before} во {stack} на таблата {board}",
    "You have deleted list {stack} on board {board}" : "Ја избришавте листата {stack} од таблата {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} ја избриша листата {stack} од таблата {board}",
    "You have created card {card} in list {stack} on board {board}" : "Креиравте картица {card} во листата {stack} на таблата {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} креираше картица {card} во листата {stack} на таблата {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Избришавте картица {card} во листата {stack} на таблата {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} избриша картица {card} во листата {stack} на таблата {board}",
    "You have renamed the card {before} to {card}" : "Ја преименувавте картицата {before} во {card}",
    "{user} has renamed the card {before} to {card}" : "{user} ја преименуваше картицата {before} во {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Додадовте опис на картицата {card} во листата {stack} на таблата {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} додаде опис на картицата {card} во листата {stack} на таблата {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Го ажуриравте описот на картицата {card} во листата {stack} на таблата {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} го ажурираше описот на картицата {card} во листата {stack} на таблата {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Ја архивиравте картицата {card} во листата {stack} на таблата {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} ја архивираше картицата {card} во листата {stack} на таблата {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Ја вративте од архива картицата {card} во листата {stack} на таблата {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} ја врати од архива картицата {card} во листата {stack} на таблата {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Ја означивте картицата {card} како завршена во листата {stack} на таблата {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} ја означи картичката {card} како завршена во листата {stack} на таблата {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Ја означивте картицата {card} како не-завршена во листата {stack} на таблата {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} ја означи картичката {card} како не-завршена во листата {stack} на таблата {board}",
    "You have removed the due date of card {card}" : "Го избришавте датумот на истекување на картицата {card}",
    "{user} has removed the due date of card {card}" : "{user} го избриша датумот на истекување на картицата {card}",
    "You have set the due date of card {card} to {after}" : "Поставивте датум на истекување на картицата {card} до {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} постави датум на истекување на картицата {card} до {after}",
    "You have updated the due date of card {card} to {after}" : "Го ажуриравте датумот на истекување на картицата {card} до {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} го ажурираше датумот на истекување на картицата {card} до {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Додадовте ознака {label} на картицата {card} во листата {stack} на таблата {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} додаде ознака {label} на картицата {card} во листата {stack} на таблата {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Ја избришавте ознаката {label} од картицата {card} во листата {stack} на таблата {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} ја избриша ознаката {label} од картицата {card} во листата {stack} на таблата {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Го додадовте {assigneduser} во картицата {card} на таблата {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} го додада {assigneduser} во картицата {card} на таблата {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Го отстранивте {assigneduser} од картицата {card} на таблата {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} го отстрани {assigneduser} од картицата {card} на таблата {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Ја преместивте картицата {card} од листата {stackBefore} во {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} ја премести картицата {card} од листата {stackBefore} во {stack}",
    "You have added the attachment {attachment} to card {card}" : "Додадовте прилог {attachment} на картицата {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} додадте прилог {attachment} на картицата {card}",
    "You have updated the attachment {attachment} on card {card}" : "Го ажуриравте прилогот {attachment} на картицата {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} го ажурираше прилогот {attachment} на картицата {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Го избришавте прилогот {attachment} од картицата {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} го избриша прилогот {attachment} од картицата {card}",
    "You have restored the attachment {attachment} to card {card}" : "Го вративте прилогот {attachment} на картицата {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} го врати прилогот {attachment} на картицата {card}",
    "You have commented on card {card}" : "Коментиравте на картицата {card}",
    "{user} has commented on card {card}" : "{user} коментирање на картицата {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Промени во <strong>апликацијата Deck</strong>",
    "A <strong>board, list or card</strong> was changed" : "Променета е <strong>табла, листа или картица</strong>",
    "A <strong>comment</strong> was created on a card" : "<strong>Коментар</strong> е креиран на картица",
    "A <strong>card description</strong> has been changed" : "Променет е <strong>опис на картица</strong> ",
    "The file was uploaded" : "Датотеката е прикачена",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Подигнатата датотека ја надминува upload_max_filesize директивата во php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Големината на датотеката ја надминува MAX_FILE_SIZE директивата која беше специфицирана во HTML формата",
    "The file was only partially uploaded" : "Датотеката беше само делумно прикачена",
    "No file was uploaded" : "Нема прикачено дадотека",
    "Missing a temporary folder" : "Недостасува привремена папка",
    "Could not write file to disk" : "Неможе да се запишува на дискот",
    "A PHP extension stopped the file upload" : "PHP додаток го стопираше прикачувањето на датотеката",
    "No file uploaded or file size exceeds maximum of %s" : "Нема прикачена дадотека или големината го надмминува максимумот од %s",
    "Invalid file type. Only JSON files are allowed." : "Невалиден тип на датотека. Дозволени се само JSON датотеки.",
    "Invalid JSON data" : "Невалидни JSON податоци",
    "Failed to import board" : "Неуспешен увоз на табла",
    "Cards due today" : "Картици со рок до денес",
    "Cards due tomorrow" : "Картици со рок до утре",
    "Upcoming cards" : "Престојни картици",
    "Load more" : "Вчитај повеќе",
    "Welcome to Nextcloud Deck!" : "Добредојдовте во Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Картицата \"%s\" на \"%s\" ти е доделена од %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} ти ја додели картицата {deck-card} на {deck-board}.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Картицата \"%1$s\" во \"%2$s\" го достигна датумот на истекување.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Картицата {deck-card} на {deck-board} го достигна рокот.",
    "%s has mentioned you in a comment on \"%s\"." : "%s те спомна во коментар на \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} те спомна во коментар на {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Таблата \"%s\" ја сподли со тебе %s.",
    "{user} has shared {deck-board} with you." : "{user} сподели {deck-board} со вас.",
    "The remote board %1$s has been shared with you by %2$s" : "Далечинска табла %1$s е споделена со тебе од %2$s",
    "Deck board" : "Табла",
    "Owned by %1$s" : "Сопственик %1$s",
    "Deck boards, cards and comments" : "Табли, картици и коментари",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Од %1$s, во %2$s/%3$s, сопственик %4$s",
    "Create a new deck card" : "Креирај нова картица",
    "Card comments" : "Коментари на картица",
    "%s on %s" : "%s на %s",
    "Deck boards and cards" : "Табли и картици",
    "No data was provided to create an attachment." : "Нема податоци за креирање на прилог.",
    "Finished" : "Завршено",
    "To review" : "На ревизија",
    "Action needed" : "Потребна е акција",
    "Later" : "Покасно",
    "copy" : "копирај",
    "Read more inside" : "Прочитај повеќе",
    "Custom lists - click to rename!" : "Прилагодени листи – кликнете за преименување!",
    "To Do" : "За правење",
    "In Progress" : "Во тек",
    "Done" : "Готово",
    "1. Open to learn more about boards and cards" : "1. Отворете за да дознаете повеќе за таблите и картичките",
    "2. Drag cards left and right, up and down" : "2. Влечете ги картичките лево и десно, горе и долу",
    "3. Apply rich formatting and link content" : "3. Применете богато форматирање и поврзете содржина",
    "4. Share, comment and collaborate!" : "4. Споделувајте, коментирајте и соработувајте!",
    "Create your first card!" : "Креирајте ја вашата прва картичка!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Коментарот има повеќе од %s карактери.\nДодаден е како пролог на картицата со име %s.\nДостапен е на линк: %s.",
    "Attachments" : "Прилози",
    "File" : "Датотека",
    "date" : "датум",
    "Card not found" : "Картицата не е пронајдена",
    "Path is already shared with this card" : "Патеката веќе е споделена со оваа картица",
    "Invalid date, date format must be YYYY-MM-DD" : "Невалиден датум, форматот мора да биде ГГГГ-ММ-ДД",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Сите табли во ваша сопственост, вклучувајќи ги колоните, картичките, ознаките, задолжените лица и коментарите.",
    "Personal planning and team project organization" : "Персонален планер и тимски проект организер",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck е алатка за организација во стил на kanban, наменета за лични планови и организација на проекти за тимови интегрирани со Nextcloud.\n\n\n📥 Додај ги задачите на картички и подреди ги\n📄 Запиши дополнителни белешки во Markdown\n🔖 Додели етикети за уште подобра организација\n👥 Сподели со твојот тим, пријатели или семејство\n📎 Прикачи датотеки и вметни ги во Markdown описот\n💬 Дискутирај со твојот тим преку коментари\n⚡ Следи ги промените во активностите\n🚀 Организирај го твојот проект",
    "Add board" : "Додади табла",
    "Card details" : "Детали за картица",
    "Select the board to link to a project" : "Избери табла за поврзување со проект",
    "Search by board title" : "Барај по име на табла",
    "Select board" : "Избери табла",
    "Move/copy card" : "Премести/копирај картица",
    "Select a board" : "Избери табла",
    "No lists available" : "Нема достапни листи",
    "Select a list" : "Избери листа",
    "Move card" : "Премести картица",
    "Copy card" : "Копирај картица",
    "Select the card to link to a project" : "Избери картица за поврзување со проект",
    "Link to card" : "Линк до картица",
    "Select a card" : "Избери картица",
    "Cancel" : "Откажи",
    "This board is read only" : "Оваа табла е само за читање",
    "Drop your files to upload" : "Повлечи датотеки за да прикачите",
    "File already exists" : "Датотека веќе постои",
    "A file with the name {filename} already exists." : "Датотека со име {filename} веќе постои.",
    "Do you want to overwrite it?" : "Дали сакате да го пребришете?",
    "Overwrite file" : "Пребриши датотека",
    "Keep existing file" : "Зачувај ја постоечката датотека",
    "Add card" : "Додади картица",
    "Archived cards" : "Архивирани картици",
    "Add list" : "Додади листа",
    "List name" : "Име на листа",
    "Clear search" : "Исчисти пребарување",
    "Active filters" : "Активни филтри",
    "Apply filter" : "Додади филтер",
    "Filter by tag" : "Филтрирај по ознака",
    "Filter by assigned user" : "Филтрирај по назначени корисници",
    "Unassigned" : "Неназначени",
    "Filter by status" : "Филтрирај по статус",
    "Open and completed" : "Отворени и завршени",
    "Open" : "Отвори",
    "Completed" : "Завршено",
    "Filter by due date" : "Филтрирај по краен рок",
    "Overdue" : "Заостанати",
    "Next 24 hours" : "Следни 24 часа",
    "Next 7 days" : "Следни 7 дена",
    "Next 30 days" : "Следни 30 дена",
    "No due date" : "Нема краен рок",
    "Clear filter" : "Исчисти филтри",
    "View Modes" : "Режими на приказ",
    "Toggle View Modes" : "Промени режими на приказ",
    "Kanban view" : "Канбан преглед",
    "Gantt view" : "Гантов дијаграм",
    "Hide archived cards" : "Сокриј ги архивираните картици",
    "Show archived cards" : "Прикажи ги архивираните картици",
    "Toggle compact mode" : "Вклучи компактен мод",
    "Hide card cover images" : "Сокриј насловни слики на картици",
    "Show card cover images" : "Прикажи насловни слики на картици",
    "Open details" : "Отвори детали",
    "Details" : "Детали",
    "Deck settings" : "Deck параметри",
    "General" : "Општо",
    "Use bigger card view" : "Користи поголем преглед на картици",
    "Appearance" : "Изглед",
    "Show card ID badge" : "Прикажи ID на картиците",
    "Show boards in calendar/tasks" : "Прикажи ги таблите во календарнот",
    "Admin settings" : "Административни параметри",
    "Limit board creation to some groups" : "Ограничи го креирањето на нови табли само на овие групи",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Корисниците кој што не се во овие групи нема да можат да прават нови таби, но ќе можат да работат на таблите кој ќе бидат споделени со нив.",
    "Enable federation" : "Овозможи федерација",
    "Board actions" : "Акции за табла",
    "Scroll sideways" : "Лизгај странично",
    "Navigate between cards" : "Навигација помеѓу картиците",
    "Close card details" : "Затвори детали на картица",
    "Search" : "Барај",
    "Show card filters" : "Прикажи филтри за картици",
    "Clear card filters" : "Исчисти филтри за картици",
    "Show those shortcuts" : "Покажи ги овие кратенки",
    "Card actions" : "Акции за картици",
    "Open card details" : "Отвори детали на картица",
    "Edit the card title" : "Измени наслов на картица",
    "Assign yourself to the current card" : "Доделете се себеси на тековната картица",
    "Archive/unarchive the current card" : "Архивирај/одархивирај тековната картица",
    "Mark card as completed/not completed" : "Означи ја картицата како завршена/незавршена",
    "Open card menu" : "Отвори мени на картица",
    "Currently present people" : "Моментално приситни корисници",
    "Loading board" : "Вчитување на табла",
    "Board not found" : "Таблата не е пронајдена",
    "Create a new list to add cards to this board" : "Додадете нова листа за да додадете картици на таблата =",
    "Sharing" : "Споделување",
    "Tags" : "Ознаки",
    "Deleted items" : "Избришани работи",
    "Activity" : "Активност",
    "Deleted lists" : "Избришани листи",
    "Undo" : "Врати",
    "Deleted cards" : "Избришани картици",
    "Hours" : "Часови",
    "Days" : "Дена",
    "Weeks" : "Недели",
    "Months" : "Месеци",
    "No cards yet" : "Сеуште нема картици",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Постави почетен датум и краен датум на картиците за да ги гледате во Гантов дијаграм",
    "Cards without dates ({count})" : "Картици без датуми ({count})",
    "Failed to create share with {displayName}" : "Неможе да се сподели со {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Дали сте сигурни дека сакате да ја префрлите таблата {title} на {user}?",
    "Transfer the board." : "Префрли табла.",
    "Transfer" : "Трансфер",
    "The board has been transferred to {user}" : "Таблата е префрлена на {user}",
    "Failed to transfer the board to {user}" : "Неуспешно префрлање на таблата на {user}",
    "Board owner" : "Сопственик на таблата",
    "(Group)" : "(Група)",
    "(Team)" : "(Тим)",
    "(remote)" : "(далечински)",
    "Can edit" : "Може да уредува",
    "Can share" : "Can share",
    "Can manage" : "Може да ја менаџира",
    "Owner" : "Сопственик",
    "Delete" : "Избриши",
    "List deleted" : "Листата е избришана",
    "Edit list title" : "Удери наслов на листата",
    "Archive all cards" : "Архивирај ги сите картици",
    "Unarchive all cards" : "Врати ги од архива сите картици",
    "Do not set cards as \"done\"" : "Не ги означувај како \"готови\"",
    "Set cards as \"done\"" : "Означи картици како \"готови\"",
    "Delete list" : "Избриши листа",
    "Archive all cards in this list" : "Архивирај ги сите картици во листата",
    "Unarchive all cards in this list" : "Врати ги од архива сите картици во оваа листа",
    "Add a new card" : "Додади нова картица",
    "Card name" : "Име на картицата",
    "title and color value must be provided" : "Мора да се внесе наслов и боја",
    "Edit" : "Уреди",
    "Add a new tag" : "Додади нова ознака",
    "Board name" : "Име на табла",
    "Members" : "Членови",
    "Assign to users/groups/team" : "Додели на корисници/групи/тимови",
    "Assign a user to this card…" : "Додели корисник на оваа картица...",
    "Select a user to assign to this card…" : "Избери на кого да се додели оваа картица…",
    "File to share" : "Датотека за споделување",
    "Invalid path selected" : "Избрана невалидна патека",
    "Upload new files" : "Прикачи нови датотеки",
    "Share from Files" : "Сподели од датотеките",
    "Pending share" : "Споделување на чекање",
    "Add this attachment" : "Додади го овој прилог",
    "Show in Files" : "Прикажи во датотеките",
    "Download" : "Преземи",
    "Remove attachment" : "Отстрани прилог",
    "Delete Attachment" : "Избриши прилог",
    "Restore Attachment" : "Врати прилог",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Променето: {modifiedDate} ⸱ Креирано: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Променето: {modifiedDate}\nКреирано: {createdDate}\nКреирано од: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Промемето: {modifiedDate}\nКреирано: {createdDate}",
    "The title cannot be empty." : "Насловот неможе да биде празен.",
    "Cannot close unsaved card!" : "Неможе да се затвори незачувана картица!",
    "Open in sidebar view" : "Отвори страничен поглед",
    "Open in bigger view" : "Отвори на голем екран",
    "Comments" : "Коментари",
    "Failed to load comments" : "Неуспешно вчитување на коментари",
    "No comments yet. Begin the discussion!" : "Сеуште нема коментари. Започни дискусија!",
    "The comment cannot be empty." : "Коментарот неможе да биде празен. =",
    "The comment cannot be longer than 1000 characters." : "Коментарот неможе да биде поголем од 1000 карактери.",
    "Submit" : "Испрати",
    "Created:" : "Создадено:",
    "In reply to" : "Како одговор на",
    "Cancel reply" : "Откажи одговор",
    "Reply" : "Одговор",
    "Update" : "Ажурирај",
    "Assign dependent cards" : "Додели зависни картици",
    "Mark as done" : "Означи како готово",
    "Mark as not done" : "Означи како не-готово",
    "Remove dependency" : "Отстрани зависност",
    "Add dependent card" : "Додели зависна картица",
    "Assign a dependent card…" : "Доделување зависна картица...",
    "Select a dependent card…" : "Избери зависна картица...",
    "Write a description …" : "Напиши опис ...",
    "The description has been changed by another user." : "Описот е променет од друг корисник.",
    "Could not save description" : "Неможе да се зачува описот",
    "Description" : "Опис",
    "(Unsaved)" : "(Незачувано)",
    "(Saving…)" : "(Снимање…)",
    "Formatting help" : "Помош за форматирање",
    "Edit description" : "Измени опис",
    "View description" : "Види опис",
    "Add Attachment" : "Додади прилог",
    "Choose attachment" : "Избери прилог",
    "Select Date" : "Избери датум",
    "Later today – {timeLocale}" : "Денес покасно – {timeLocale}",
    "Set due date for later today" : "Постави краен рок за денес подоцна",
    "Tomorrow – {timeLocale}" : "Утре – {timeLocale}",
    "Set due date for tomorrow" : "Постави краен рок за утре",
    "This weekend – {timeLocale}" : "Овој викенд – {timeLocale}",
    "Set due date for this weekend" : "Постави краен рок за овој викенд",
    "Next week – {timeLocale}" : "Следна недела – {timeLocale}",
    "Set due date for next week" : "Постави краен рок за следната недела",
    "Assign a due date to this card…" : "Додели рок за оваа картица…",
    "Set a due date" : "Постави краен рок",
    "Add due date" : "Додади краен рок",
    "Choose a date" : "Избери датум",
    "Remove due date" : "Отстрани краен рок",
    "Due at:" : "Краен рок:",
    "Not done" : "Не е завршено",
    "Unarchive card" : "Врати картица од архива",
    "Archive card" : "Архивирај картица",
    "Assign a start date to this card…" : "Додели почетен датум за оваа картица…",
    "Set a start date" : "Постави почетен датум",
    "Add start date" : "Додади почетен датум",
    "Remove start date" : "Острани почетен датум",
    "Assign a tag to this card…" : "Додади ознака на оваа картица...",
    "Select or create a tag…" : "Избери или креирај ознака...",
    "Create a new tag:" : "Направи нова ознака:",
    "(group)" : "(group)",
    "{count} comments, {unread} unread" : "{count} коментари, {unread} непрочитани",
    "Todo items" : "Задачи",
    "Edit card title" : "Измени наслов на картица",
    "Open link" : "Отвори линк",
    "Card deleted" : "Картицата е избришана",
    "Change card color" : "Промени ја бојата на картицата",
    "Edit title" : "Удери наслов",
    "Assign to me" : "Доделени мене",
    "Unassign myself" : "Откажи се",
    "Delete card" : "Избриши картица",
    "All boards" : "Сите табли",
    "Archived boards" : "Архивирани табли",
    "Shared with you" : "Споделено со тебе",
    "Cancel edit" : "Откажи ажурирање",
    "Save board" : "Зачувај табла",
    "Board {0} deleted" : "Таблата {0} е избришана",
    "All cards" : "Сите картици",
    "Only assigned cards" : "Само доделени картици",
    "No reminder" : "Нема потсетник",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Дали сте сигурени дека сакате да ја избришете оваа табла {title}? Ова ќе ги избрише и сите податоци во таблата и архивираните картици.",
    "Delete the board?" : "Бришење на таблата?",
    "Are you sure you want to leave the board {title}?" : "Дали си сигурен дека сакаш да ја напуштиш таблата {title}?",
    "Leave the board?" : "Напушти ја таблата?",
    "Leave" : "Напушти",
    "Failed to leave the board" : "Неуспешено напуштање на табла",
    "Exporting board..." : "Извоз на табла...",
    "An error occurred" : "Настана грешка",
    "Board details" : "Детали за таблата",
    "Edit board" : "Измени табла",
    "Clone board" : "Клонирај табла",
    "Unarchive board" : "Врати табла од архива",
    "Archive board" : "Архивирај табла",
    "Export board" : "Извези табла",
    "Turn on due date reminders" : "Вклучи потсетници за крајните рокови",
    "Turn off due date reminders" : "Исклучи потсетници за крајните рокови",
    "Remove as default board" : "Острани стандардна табла",
    "Set as default board" : "Постави како стандардна табла",
    "Due date reminders" : "Потсетници за крајните рокови",
    "Assigned cards" : "Доделени картици",
    "No notifications" : "Нема известувања",
    "Delete board" : "Избриши табла",
    "Leave board" : "Напушти табла",
    "Importing board..." : "Увезување табла...",
    "Board imported successfully" : "Таблата е успешно увезена",
    "Import board" : "Увези табла",
    "Clone {boardTitle}" : "Клонирај {boardTitle}",
    "Clone cards" : "Клонирај картици",
    "Clone assignments" : "Клонирај задачи",
    "Clone labels" : "Клонирај ознаки",
    "Clone due dates" : "Клонирај крајни рокови",
    "Advanced options" : "Напредни опции",
    "Move all cards to the first list" : "Помести ги сите картици во првата листа",
    "Restore archived cards" : "Врати архивирани картици",
    "Clone" : "Клонирај",
    "Export {boardTitle}" : "Извези {boardTitle}",
    "Export as JSON" : "Извези како JSON",
    "Export as CSV" : "Извези како CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Забелешка: Поддржан е само JSON формат за увоз назад во апликацијата Deck.",
    "Export" : "Извези",
    "Loading filtered view" : "Вчитување на филтриран поглед",
    "Search for {searchQuery} in other boards" : "Барај {searchQuery} во други табли",
    "Search for {searchQuery} in all boards" : "Барај {searchQuery} во сите табли",
    "No results found" : "Нема пронајдено резултати",
    "Deck board {name}\n* Last modified on {lastMod}" : "Табла {name}\n* Последна промена на {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Креирана на {created}\n* Последна промена на {lastMod}\n* {nbAttachments} прилози\n* {nbComments} коментари",
    "{nbCards} cards" : "{nbCards} картици",
    "Due on {date}" : "Истекува на {date}",
    "{stack} in {board}" : "{stack} во {board}",
    "Click to expand description" : "Кликнни за проширување на опис",
    "Click to expand comment" : "Кликнни за проширување на коментарот",
    "Create card" : "Креирајте картица",
    "Create a new card" : "Креирајте нова картица",
    "Card title" : "Наслов на картицата",
    "Creating the new card …" : "Креирање нова картица ...",
    "Card \"{card}\" was added to \"{board}\"" : "Картицата \"{card}\" е додадена во \"{board}\"",
    "Open card" : "Отвори картица",
    "Close" : "Затвори",
    "No upcoming cards" : "Нема престојни картици",
    "upcoming cards today" : "престојни картици за денес",
    "upcoming cards tomorrow" : "престојни картици за утре",
    "upcoming cards" : "престојни картици",
    "New card" : "Нова картица",
    "Link to a board" : "Линк до табла",
    "Link to a card" : "Линк до картица",
    "Create a card" : "Креирајте картица",
    "Message from {author} in {conversationName}" : "Порака од {author} во {conversationName}",
    "Something went wrong" : "Нешто не е во ред",
    "Failed to upload {name}" : "Неуспешно прикачување {name}",
    "Maximum file size of {size} exceeded" : "Максималната големина на датотека од {size} е достигната",
    "Assigned users" : "Доделени корисници",
    "Due date" : "До датум",
    "Created" : "Креирано",
    "Modified" : "Изменето",
    "Error creating the share" : "Грешка при креирање на споделување",
    "Share with a Deck card" : "Споделено со Deck картица",
    "Share {file} with a Deck card" : "Сподели {file} со Deck картица",
    "Share" : "Сподели",
    "The card \"%s\" on \"%s\" has reached its due date." : "Картицата \"%s\" на \"%s\" го достигна датумот на истекување.",
    "Share board with a user, group or team …" : "Сподели табла со корисник, група или тим...",
    "Searching for users, groups and teams …" : "Пребарување на корисници, групи или тимови...",
    "No participants found" : "Не се пронајдени учесници",
    "Save" : "Зачувај",
    "seconds ago" : "пред неколку секунди",
    "Today" : "Денес",
    "Tomorrow" : "Утре",
    "No due" : "Не истекува"
},
"nplurals=2; plural=(n % 10 == 1 && n % 100 != 11) ? 0 : 1;");
