OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Bạn đã tạo một bảng mới {board}",
    "{user} has created a new board {board}" : "{user} đã tạo một bảng mới {board}",
    "You have deleted the board {board}" : "Bạn đã xóa một bảng {board}",
    "{user} has deleted the board {board}" : "{user} đã xóa một bảng {board}",
    "You have restored the board {board}" : "Bạn đã khôi phục lại bảng {board}",
    "{user} has restored the board {board}" : "{user} đã khôi phục bảng {board}",
    "You have shared the board {board} with {acl}" : "Bạn đã chia sẽ bảng {board} với {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} đã chia sẽ bảng {board} cho {acl}",
    "You have removed {acl} from the board {board}" : "Bạn đã xoá {acl} khỏi bảng {board}",
    "{user} has removed {acl} from the board {board}" : "{user} đã xoá {acl} khỏi bảng {board}",
    "You have renamed the board {before} to {board}" : "Bạn đã đổi tên bảng {before} thành {board}",
    "{user} has renamed the board {before} to {board}" : "{user} đã đổi tên bảng {before} thành {board}",
    "You have archived the board {board}" : "Bạn đã lưu trữ bảng {board}",
    "{user} has archived the board {before}" : "{user} đã lưu trữ bảng {before}",
    "You have unarchived the board {board}" : "Bạn đã gỡ lưu trữ bảng {board}",
    "{user} has unarchived the board {before}" : "{user} đã gỡ lưu trữ bảng {before}",
    "You have created a new list {stack} on board {board}" : "Bạn đã tạo một danh sách mới là {stack} tại bảng {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} đã tạo một danh sách mới {stack} tại bảng {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Bạn đã đổi tên danh sách {before} thành {stack} tại bảng {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} đã đổi tên danh sách {before} thành {stack} tại bảng {board}",
    "You have deleted list {stack} on board {board}" : "Bạn đã xóa danh sách {stack} tại bảng {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} đã xóa danh sách {stack} tại bảng {board}",
    "You have created card {card} in list {stack} on board {board}" : "Bạn đã tạo thẻ {card} trong danh sách {stack} tại bảng {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} đã tạo thẻ {card} trong danh sách {stack} tại bảng {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Bạn đã xóa thẻ {card} trong danh sách {stack} tại bảng {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} đã xóa thẻ {card} trong danh sách {stack} tại bảng {board}",
    "You have renamed the card {before} to {card}" : "Bạn đã đổi tên thẻ {before} thành {card}",
    "{user} has renamed the card {before} to {card}" : "{user} đã đổi tên thẻ {before} thành {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Bạn đã thêm mô tả vào thẻ {card} trong danh sách {stack} tại bảng {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} đã thêm mô tả vào thẻ {card} trong danh sách {stack} tại bảng {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Bạn đã cập nhật mô tả của thẻ {card} trong danh sách {stack} tại bảng {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} đã cập nhật mô tả của thẻ {card} trong danh sách {stack} tại bảng {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Bạn đã lưu trữ thẻ {card} trong danh sách {stack} tại bảng {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} đã lưu trữ thẻ {card} trong danh sách {stack} tại bảng {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Bạn đã gỡ lưu trữ thẻ {card} trong danh sách {stack} tại bảng {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} đã gỡ lưu trữ thẻ {card} trong danh sách {stack} tại bảng {board} ",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Bạn đánh dấu thẻ {card} là đã hoàn thành trong danh sách {stack} tại bảng {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} đã đánh dấu thẻ {card} là đã hoàn thành trong danh sách {stack} tại bảng {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Bạn đã đánh dấu thẻ {card} là chưa hoàn thành trong danh sách {stack} tại bảng {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} đã đánh dấu thẻ {card} là chưa hoàn thành trong danh sách {stack} tại bảng {board}",
    "You have removed the due date of card {card}" : "Bạn đã gỡ ngày đến hạn tại thẻ {card}",
    "{user} has removed the due date of card {card}" : "{user} đã gỡ ngày đến hạn tại thẻ {card}",
    "You have set the due date of card {card} to {after}" : "Bạn đã đặt lại ngày đến hạn tại thẻ {card} thành {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} đã đặt lại ngày hết hạn tại thẻ {card} là {after}",
    "You have updated the due date of card {card} to {after}" : "Bạn đã cập nhật ngày đến hạn tại thẻ {card} thành {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} đã cập nhật ngày đến hạn tại thẻ {card} thành {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Bạn đã thêm nhãn dán {label} vào thẻ {card} trong danh sách {stack} tại bảng {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} đã thêm nhãn dán {label} vào thẻ {card} trong danh sách {stack} tại bảng {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Bạn đã xóa nhãn dán {label} từ thẻ {card} trong danh sách {stack} tại bảng {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} đã xóa nhãn dán {label} từ thẻ {card} trong danh sách {stack} tại bảng {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Bạn đã giao việc cho {assigneduser} ở thẻ {card} tại bảng {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} đã giao việc {assigneduser} ở thẻ {card} tại bảng {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Bạn đã gỡ phân việc cho {assigneduser} từ thẻ {card} tại bảng {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} đã gỡ phân việc cho {assigneduser} khỏi thẻ {card} tại bảng {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Bạn đã di chuyển thẻ {card} từ danh sách {stackBefore} đến {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} đã di chuyển thẻ {card} từ danh sách {stackBefore} đến {stack}",
    "You have added the attachment {attachment} to card {card}" : "Bạn đã thêm tệp đính kèm {attachment} vào thẻ {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} đã thêm tệp đính kèm {attachment} vào thẻ {card}",
    "You have updated the attachment {attachment} on card {card}" : "Bạn đã sửa đổi tệp đính kèm {attachment} tại thẻ {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} đã sửa tệp đính kèm {attachment} tại thẻ {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Bạn đã xóa tệp đính kèm {attachment} ở thẻ {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} đã xóa tệp đính kèm {attachment} tại thẻ {card}",
    "You have restored the attachment {attachment} to card {card}" : "Bạn đã khôi phục tệp đính kèm {attachment} vào thẻ {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} đã khôi phục tệp đính kèm {attachment} vào thẻ {card}",
    "You have commented on card {card}" : "Bạn đã bình luận tại thẻ {card}",
    "{user} has commented on card {card}" : "{user} đã bình luận về thẻ {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Những thay đổi trong ứng dụng <strong>Deck</strong>",
    "A <strong>board, list or card</strong> was changed" : "Một <strong>bảng, danh sách hoặc thẻ</strong> đã bị thay đổi",
    "A <strong>comment</strong> was created on a card" : "Một <strong>bình luận</strong> được viết tại thẻ",
    "A <strong>card description</strong> has been changed" : "Một <strong>thẻ mô tả</strong> đã được sửa lại",
    "The file was uploaded" : "Tệp tin đã được tải lên",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Tập tin tải lên đã vượt quá upload_max_filesize giới hạn trong hệ thống",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Tập được tải lên vượt quá MAX_FILE_SIZE đã được giới hạn trong biểu mẫu HTML",
    "The file was only partially uploaded" : "Tệp đang chỉ được tải lên một phần",
    "No file was uploaded" : "Chưa có tệp nào dược tải lên",
    "Missing a temporary folder" : "Thiếu một thư mục tạm",
    "Could not write file to disk" : "Không thể ghi tệp lên hệ thống",
    "A PHP extension stopped the file upload" : "Một chức năng mở rộng của PHP đã dừng tải tệp lên",
    "No file uploaded or file size exceeds maximum of %s" : "Chưa có tệp nào được tải lên hoặc kích thước tệp vượt quá giới hạn của %s",
    "Invalid file type. Only JSON files are allowed." : "Loại tệp không hợp lệ. Chỉ tệp JSON được cho phép.",
    "Invalid JSON data" : "Dữ liệu JSON không hợp lệ",
    "Failed to import board" : "Nhập dữ liệu bảng thất bại",
    "Cards due today" : "Thẻ hết hạn hôm nay",
    "Cards due tomorrow" : "Những thẻ hết thời hạn vào ngày mai.",
    "Upcoming cards" : "Thẻ sắp đến hạn",
    "Load more" : "Xem thêm",
    "Welcome to Nextcloud Deck!" : "Chào mừng tới Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Thẻ \"%s\" tại \"%s\" đã được phân công cho bạn bởi  %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} đã giao thẻ nhiệm vụ {deck-card} tại bảng {deck-board} cho bạn.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Thẻ {deck-card} tại {deck-board} đã đến hạn.",
    "%s has mentioned you in a comment on \"%s\"." : "%s đã nhắc đến bạn trong một bình luận tại \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} đã nhắc đến bạn trong một bình luận tại thẻ {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Bảng \"%s\" đã được chia sẽ cho bạn từ %s .",
    "{user} has shared {deck-board} with you." : "{user} đã chia sẻ bảng {deck-board} cho bạn.",
    "Deck board" : "Bảng Deck",
    "Owned by %1$s" : "Sở hữu bởi %1$s",
    "Deck boards, cards and comments" : "Các bảng Deck, thẻ và bình luận",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Từ %1$s, tại %2$s/%3$s, thuộc sở hữu của %4$s",
    "Create a new deck card" : "Tạo một thẻ Deck mới",
    "Card comments" : "Những bình luận của thẻ",
    "%s on %s" : "%s tại %s",
    "Deck boards and cards" : "Bảng Deck và các thẻ",
    "No data was provided to create an attachment." : "Không có dữ liệu nào được cung cấp để tạo tệp đính kèm.",
    "Finished" : "Hoàn thành",
    "To review" : "Xem thử",
    "Action needed" : "Cần thực hiện",
    "Later" : "Để sau",
    "copy" : "sao chép",
    "Read more inside" : "Đọc thêm chi tiết",
    "Custom lists - click to rename!" : "Danh sách tùy chỉnh - nhấn để đổi tên!",
    "To Do" : "Cần làm",
    "In Progress" : "Đang thực hiện",
    "Done" : "Hoàn tất",
    "1. Open to learn more about boards and cards" : "1. Mở để tìm hiểu thêm về bảng và thẻ",
    "2. Drag cards left and right, up and down" : "2. Kéo thẻ sang trái và phải, lên và xuống",
    "3. Apply rich formatting and link content" : "3. Áp dụng định dạng và liên kết nội dung",
    "4. Share, comment and collaborate!" : "4. Chia sẻ, bình luận và collaborate!",
    "Create your first card!" : "Tạo thẻ đầu tiên của bạn ngay!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Bình luận này có hơn %s ký tự.\nĐã một tệp đính kèm vào thẻ với tên %s.\nCó thể truy cập tại URL: %s.",
    "Attachments" : "Đính kèm",
    "File" : "Tệp Tin",
    "Card not found" : "Không tìm thấy thẻ",
    "Path is already shared with this card" : "Đường dẫn đã được chia sẻ với thẻ này",
    "Invalid date, date format must be YYYY-MM-DD" : "Định dạng ngày không hợp lệ, phải là định dạng YYYY-MM-DD",
    "Personal planning and team project organization" : "Kế hoạch cá nhân và tổ chức nhóm dự án",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck được gọi là Kế Hoạch, và là một công cụ tổ chức theo phong cách dễ hiểu nhằm mục đích để lập kế hoạch cá nhân hoặc dự án của tổ chức cho các tính năng nhóm được tích hợp với Nextcloud.\n\n- 📥 Thêm các nhiệm vụ của bạn vào thẻ và sắp xếp chúng theo thứ tự\n- 📄 Ghi chú thêm trong định dạng Markdown\n- 🔖 Gán nhãn dán để tổ chức tốt hơn\n- 👥 Chia sẻ với nhóm, bạn bè hoặc gia đình của bạn\n- 📎 Đính kèm tệp và nhúng chúng vào mô tả Markdown của bạn\n- 💬 Thảo luận với nhóm của bạn bằng cách bình luận\n- ⚡ Theo dõi các thay đổi trong luồng hoạt động\n- 🚀 Dự án tổ chức của bạn",
    "Add board" : "Thêm bảng",
    "Card details" : "Chi tiết thẻ",
    "Select the board to link to a project" : "Chọn bảng để liên kết với một dự án",
    "Search by board title" : "Tìm kiếm bằng từ khoá tiêu đề của bảng",
    "Select board" : "Chọn bảng",
    "Move/copy card" : "Chuyển/Sao chép thẻ",
    "Select a board" : "Chọn một bảng",
    "No lists available" : "Không có danh sách nào có sẵn",
    "Select a list" : "Chọn một danh sách",
    "Move card" : "Di chuyển thẻ",
    "Copy card" : "Sao chép thẻ",
    "Select the card to link to a project" : "Chọn thẻ để liên kết với một dự án.",
    "Link to card" : "Liên kết đến thẻ",
    "Select a card" : "Chọn một thẻ",
    "Cancel" : "Hủy bỏ",
    "This board is read only" : "Bảng này chỉ có thể đọc",
    "Drop your files to upload" : "Thả tệp của bạn để tải lên",
    "File already exists" : "Tệp đã tồn tại",
    "A file with the name {filename} already exists." : "Một tệp tin có tên {filename} đã tồn tại.",
    "Do you want to overwrite it?" : "Bạn có muốn ghi đè lên nó không?",
    "Overwrite file" : "Ghi đè tập tin",
    "Keep existing file" : "Giữ tệp hiện có",
    "Add card" : "Thêm tác vụ",
    "Archived cards" : "Thẻ đã lưu trữ",
    "Add list" : "Thêm danh sách",
    "List name" : "Danh sách tên",
    "Active filters" : "Kích hoạt bộ lọc",
    "Apply filter" : "Áp dụng bộ lọc",
    "Filter by tag" : "Lọc theo nhãn dán",
    "Filter by assigned user" : "Lọc theo người được phân công",
    "Unassigned" : "Chưa được phân công",
    "Filter by status" : "Lọc theo trạng thái",
    "Open and completed" : "Đã mở và hoàn thành",
    "Open" : "Mở",
    "Completed" : "Hoàn thành",
    "Filter by due date" : "Lọc theo thời hạn",
    "Overdue" : "Quá hạn",
    "Next 24 hours" : "Trong vòng 24 giờ tới",
    "Next 7 days" : "7 ngày tiếp theo",
    "Next 30 days" : "30 ngày tiếp theo",
    "No due date" : "Không có thời hạn",
    "Clear filter" : "Làm sạch bộ lọc",
    "View Modes" : "Chế độ xem",
    "Toggle View Modes" : "Chế độ xem chuyển đổi",
    "Hide archived cards" : "Ẩn tác vụ đã lưu trữ",
    "Show archived cards" : "Hiển thị các thẻ đã lưu trữ",
    "Toggle compact mode" : "Chuyển đổi chế độ thu gọn",
    "Hide card cover images" : "Ẩn hình ảnh bìa thẻ",
    "Show card cover images" : "Hiển thị hình ảnh bìa thẻ",
    "Open details" : "Mở chi tiết",
    "Details" : "Thông tin",
    "Deck settings" : "Thiết lập Kế Hoạch",
    "General" : "Cài đặt chung",
    "Use bigger card view" : "Sử dụng giao diện thẻ lớn hơn",
    "Appearance" : "Giao diện",
    "Show card ID badge" : "Hiển thị ID thẻ nhận dạng",
    "Show boards in calendar/tasks" : "Hiển thị boards trong lịch/công việc",
    "Limit board creation to some groups" : "Giới hạn việc tạo bảng cho một số nhóm",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Người dùng nằm ngoài nhóm đó sẽ không thể tạo bảng của riêng họ, nhưng vẫn có thể làm việc trên các bảng đã được chia sẻ với họ.",
    "Board actions" : "Hành động cho Bảng",
    "Scroll sideways" : "Cuộn chuột ngang",
    "Navigate between cards" : "Di chuyển giữa các thẻ",
    "Close card details" : "Đóng chi tiết thẻ",
    "Search" : "Tìm kiếm",
    "Show card filters" : "Hiển thị bộ lọc thẻ",
    "Clear card filters" : "Xóa bộ lọc thẻ",
    "Card actions" : "Hành động thẻ",
    "Open card details" : "Mở chi tiết thẻ",
    "Edit the card title" : "Chỉnh sửa tiêu đề thẻ",
    "Assign yourself to the current card" : "Gán cho bản thân thẻ hiện tại",
    "Archive/unarchive the current card" : "Đóng gói/mở gói thẻ hiện tại",
    "Mark card as completed/not completed" : "Đánh dấu thẻ là đã hoàn thành/chưa hoàn thành",
    "Open card menu" : "Mở menu thẻ",
    "Currently present people" : "Hiện tại có những người hiện diện",
    "Loading board" : "Đang tải bảng",
    "Board not found" : "Bảng không tìm thấy",
    "Create a new list to add cards to this board" : "Tạo một danh sách mới để thêm thẻ vào bảng này",
    "Sharing" : "Đang chia sẽ",
    "Tags" : "Thẻ",
    "Deleted items" : "Đã xóa",
    "Activity" : "Hoạt động",
    "Deleted lists" : "Danh sách đã bị xóa",
    "Undo" : "Hoàn tác",
    "Deleted cards" : "Thẻ đã xóa",
    "Hours" : "Giờ",
    "Days" : "Ngày",
    "Weeks" : "Tuần",
    "Months" : "Tháng",
    "No cards yet" : "Hiện tại chưa có thẻ nào",
    "Failed to create share with {displayName}" : "Không thể tạo chia sẻ với {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Bạn có chắc chắn muốn chuyển bảng {title} cho {user} không?",
    "Transfer the board." : "Chuyển bảng.",
    "Transfer" : "Truyền",
    "The board has been transferred to {user}" : "Bảng đã được chuyển cho {user}",
    "Failed to transfer the board to {user}" : "Không thể chuyển bảng cho {user}",
    "Board owner" : "Chủ sở hữu bảng",
    "(Group)" : "(Nhóm)",
    "(Team)" : "(Đội)",
    "(remote)" : "(từ xa)",
    "Can edit" : "Có thể chỉnh sửa",
    "Can share" : "Can share",
    "Can manage" : "Có thể quản lý",
    "Owner" : "Chủ",
    "Delete" : "Xóa",
    "List deleted" : "Danh sách đã bị xóa",
    "Edit list title" : "Chỉnh sửa tiêu đề danh sách",
    "Archive all cards" : "Lưu trữ tất cả các thẻ",
    "Unarchive all cards" : "Giải nén tất cả các thẻ",
    "Delete list" : "Xoá danh sách",
    "Archive all cards in this list" : "Lưu trữ tất cả các thẻ trong danh sách này",
    "Unarchive all cards in this list" : "Giải nén tất cả các thẻ trong danh sách này.",
    "Add a new card" : "Thêm một tác vụ mới",
    "Card name" : "Tên thẻ",
    "title and color value must be provided" : "tiều đề và giá trị màu phải được cung cấp",
    "Edit" : "Chỉnh sửa",
    "Add a new tag" : "Thêm một thẻ mới",
    "Board name" : "Tên bảng",
    "Members" : "Thành viên",
    "Assign to users/groups/team" : "Gán cho người dùng/nhóm/đội",
    "Assign a user to this card…" : "Gán một người dùng cho thẻ này...",
    "Select a user to assign to this card…" : "Chọn một người dùng để gán cho thẻ này…",
    "File to share" : "Tệp để chia sẻ",
    "Invalid path selected" : "‎Đường dẫn không hợp lệ được chọn‎",
    "Upload new files" : "Tải lên các tệp mới",
    "Share from Files" : "Chia sẻ từ Thư mục",
    "Pending share" : "Chia sẻ đang chờ xử lý",
    "Add this attachment" : "Thêm tệp đính kèm này",
    "Show in Files" : "Hiển thị trong Tệp",
    "Download" : "Tải xuống",
    "Remove attachment" : "Xóa tệp đính kèm",
    "Delete Attachment" : "Xóa tệp đính kèm",
    "Restore Attachment" : "Khôi Phục Tệp Đính Kèm",
    "The title cannot be empty." : "Tiêu đề không thể để trống.",
    "Open in sidebar view" : "Mở trong chế độ xem thanh bên",
    "Open in bigger view" : "Mở trong chế độ xem lớn hơn",
    "Comments" : "Các bình luận",
    "Failed to load comments" : "Không thể tải bình luận",
    "No comments yet. Begin the discussion!" : "Chưa có bình luận. Bắt đầu cuộc thảo luận!",
    "The comment cannot be empty." : "Bình luận không thể để trống.",
    "The comment cannot be longer than 1000 characters." : "Bình luận không thể dài hơn 1000 ký tự.",
    "Submit" : "Gửi",
    "Created:" : "Đã tạo:",
    "In reply to" : "Trả lời",
    "Cancel reply" : "Hủy bỏ trả lời",
    "Reply" : "Trả l",
    "Update" : "Cập nhật",
    "Mark as done" : "Đánh dấu là đã hoàn thành",
    "Mark as not done" : "Đánh dấu là chưa hoàn thành",
    "Write a description …" : "Viết một mô tả ...",
    "Description" : "Mô tả",
    "(Unsaved)" : "(Chưa Lưu)",
    "(Saving…)" : "(Đang Lưu…)",
    "Formatting help" : "Định dạng trợ giúp",
    "Edit description" : "Chỉnh sửa mô tả",
    "View description" : "Xem mô tả",
    "Add Attachment" : "Thêm Đính kèm",
    "Choose attachment" : "Chọn tệp đính kèm",
    "Select Date" : "Chọn ngày",
    "Later today – {timeLocale}" : "Sau này - {timeLocale}",
    "Set due date for later today" : "Đặt ngày đáo hạn cho sau hôm nay",
    "Tomorrow – {timeLocale}" : "Ngày mai - {timeLocale}",
    "Set due date for tomorrow" : "Đặt ngày đáo hạn cho ngày mai",
    "This weekend – {timeLocale}" : "Cuối tuần này - {timeLocale}",
    "Set due date for this weekend" : "Đặt ngày hết hạn cho cuối tuần này",
    "Next week – {timeLocale}" : "Tuần sau - {timeLocale}",
    "Set due date for next week" : "Đặt ngày hết hạn cho tuần sau",
    "Assign a due date to this card…" : "Gán một ngày đến hạn cho thẻ này...",
    "Set a due date" : "Đặt một ngày hết hạn",
    "Add due date" : "Thêm ngày đáo hạn",
    "Choose a date" : "Chọn một ngày",
    "Remove due date" : "Xóa thời hạn",
    "Due at:" : "Đến lúc:",
    "Not done" : "Chưa hoàn thành",
    "Unarchive card" : "Thẻ chưa lưu trữ",
    "Archive card" : "Lưu trữ thẻ",
    "Assign a tag to this card…" : "Gán một \"thẻ phân biệt\" cho thẻ này ...",
    "Select or create a tag…" : "Chọn hoặc tạo một thẻ…",
    "Create a new tag:" : "Tạo một thẻ phân biệt mới:",
    "(group)" : "(nhóm)",
    "{count} comments, {unread} unread" : "{count} bình luận, {unread} chưa đọc",
    "Todo items" : "Các mục công việc cần làm",
    "Edit card title" : "Chỉnh sửa tiêu đề thẻ",
    "Open link" : "Mở liên kết",
    "Card deleted" : "Thẻ đã bị xóa",
    "Edit title" : "Chỉnh sửa tiêu đề",
    "Assign to me" : "Phân công cho tôi",
    "Unassign myself" : "Hủy bỏ phân công cho tôi",
    "Delete card" : "Xoá thẻ",
    "All boards" : "Tất cả các bảng",
    "Archived boards" : "Dự án đã lưu trữ",
    "Shared with you" : "Đã chia sẻ với bạn",
    "Cancel edit" : "Hủy chỉnh sửa",
    "Save board" : "Lưu bảng",
    "Board {0} deleted" : "Bảng {0} đã bị xóa",
    "All cards" : "Tất cả các thẻ",
    "Only assigned cards" : "Chỉ những thẻ được gán",
    "No reminder" : "Không có nhắc hẹn",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Bạn có chắc chắn muốn xóa bảng {title} không? Điều này sẽ xóa tất cả dữ liệu của bảng này bao gồm cả các thẻ đã được lưu trữ.",
    "Delete the board?" : "Xóa bảng?",
    "Exporting board..." : "Đang xuất dữ liệu bảng...",
    "An error occurred" : "Có lỗi đã xảy ra",
    "Board details" : "Thông tin dự án",
    "Edit board" : "Chỉnh sửa dự án",
    "Clone board" : "Tạo bản sao bảng",
    "Unarchive board" : "Bỏ lưu trữ dự án",
    "Archive board" : "Lưu trữ dự án",
    "Export board" : "Xuất dữ liệu bảng",
    "Turn on due date reminders" : "Bật nhắc nhở ngày đáo hạn",
    "Turn off due date reminders" : "Tắt nhắc nhở ngày đáo hạn",
    "Due date reminders" : "Nhắc nhở ngày hết hạn",
    "Assigned cards" : "Thẻ được giao",
    "No notifications" : "Không có thông báo",
    "Delete board" : "Xóa dự án",
    "Importing board..." : "Đang nhập dữ liệu bảng...",
    "Board imported successfully" : "Nhập dữ liệu bảng thành công",
    "Import board" : "Nhập dữ liệu bảng",
    "Clone {boardTitle}" : "Nhân đôi {boardTitle}",
    "Clone cards" : "Tạo bản sao thẻ",
    "Clone assignments" : "Tạo bản sao công việc",
    "Clone labels" : "Tạo bản sao nhãn dán",
    "Clone due dates" : "Tạo bản sao ngày hết hạn",
    "Advanced options" : "Tùy chọn nâng cao",
    "Move all cards to the first list" : "Chuyển tất cả thẻ vào đầu danh sách",
    "Restore archived cards" : "Khôi phục thẻ đã lưu trữ",
    "Clone" : "Tạo bản sao",
    "Export {boardTitle}" : "Xuất dữ liệu bảng {boardTitle}",
    "Export as JSON" : "Xuất dữ liệu dưới dạng JSON",
    "Export as CSV" : "Xuất dưới dạng CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Lưu ý: Chỉ hỗ trợ định dạng JSON để nhập lại dữ liệu vào ứng dụng Deck.",
    "Export" : "Xuất ra",
    "Loading filtered view" : "Đang tải kết quả đã lọc",
    "Search for {searchQuery} in all boards" : "Tìm kiếm {searchQuery} trong tất cả các bảng",
    "No results found" : "Không tìm thấy kết quả",
    "Deck board {name}\n* Last modified on {lastMod}" : "Bảng Deck có tên {name}\n* Được sửa lần cuối vào lúc {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Được tạo vào {created}\n* Được sửa đổi lần cuối vào {lastMod}\n* {nbAttachments} tệp đính kèm\n* {nbComments} bình luận",
    "{nbCards} cards" : "{nbCards} thẻ",
    "Due on {date}" : "Đến hạn vào lúc {date}",
    "{stack} in {board}" : "{stack} tại bảng {board}",
    "Click to expand description" : "Nhấp để xem thêm mô tả",
    "Click to expand comment" : "Nhấp để xem thêm bình luận",
    "Create card" : "Tạo thẻ",
    "Create a new card" : "Tạo một thẻ mới",
    "Card title" : "Tiêu đề thẻ",
    "Creating the new card …" : "Đang tạo thẻ mới ...",
    "Card \"{card}\" was added to \"{board}\"" : "Thẻ \"{card}\" đã được thêm vào bảng \"{board}\"",
    "Open card" : "Mở thẻ",
    "Close" : "Đóng",
    "No upcoming cards" : "Không có thẻ hết hạn sắp tới",
    "upcoming cards today" : "thẻ sắp hết hạn hôm nay",
    "upcoming cards tomorrow" : "thẻ sắp hết hạn vào ngày mai",
    "upcoming cards" : "thẻ sắp đến hạn",
    "New card" : "Thẻ mới",
    "Link to a board" : "Liên kết đến một bảng",
    "Link to a card" : "Liên kết đến một thẻ",
    "Create a card" : "Tạo thẻ",
    "Message from {author} in {conversationName}" : "Tin nhắn từ {author} tại cuộc trò chuyện {conversationName}",
    "Something went wrong" : "Có lỗi đã xảy ra",
    "Failed to upload {name}" : "Tải lên {name} thất bại",
    "Maximum file size of {size} exceeded" : "Đã vượt quá kích thước {size} tối đa tập tin",
    "Assigned users" : "Người dùng được phân công",
    "Due date" : "Thời hạn",
    "Created" : "Đã tạo",
    "Modified" : "Thay đổi",
    "Error creating the share" : "Lỗi khi đang chia sẻ",
    "Share with a Deck card" : "Chia sẻ đến một thẻ Deck",
    "Share {file} with a Deck card" : "Chia sẻ {file} đến một thẻ Deck",
    "Share" : "Chia sẻ",
    "The card \"%s\" on \"%s\" has reached its due date." : "Thẻ \"%s\" tại \"%s\" đã đến hạn.",
    "Share board with a user, group or team …" : "Chia sẻ bảng với người dùng, nhóm hoặc đội ...",
    "Searching for users, groups and teams …" : "Đang tìm kiếm người dùng, nhóm và đội ...",
    "No participants found" : "Không tìm thấy người tham gia",
    "Save" : "Lưu",
    "seconds ago" : "vài giây trước",
    "Today" : "Hôm nay",
    "Tomorrow" : "Ngày mai",
    "No due" : "Không có ngày hết hạn"
},
"nplurals=1; plural=0;");
