<?php

// autoload_files.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    '88254829cb0eed057c30eaabb6d8edc4' => $vendorDir . '/amphp/amp/src/functions.php',
    '429ae5f14a13a9076791c19422e10996' => $vendorDir . '/amphp/amp/src/Future/functions.php',
    'c8601a4144b50a7b548da082c89c4dc1' => $vendorDir . '/amphp/amp/src/Internal/functions.php',
    '3da389f428d8ee50333e4391c3f45046' => $vendorDir . '/amphp/serialization/src/functions.php',
    'bcb7d4fc55f4b1a7e10f5806723e9892' => $vendorDir . '/amphp/sync/src/functions.php',
    '6c2681bc13923b80242cc286856cab22' => $vendorDir . '/amphp/byte-stream/src/functions.php',
    '4ee21bf8253a2272056aefb41f7f0116' => $vendorDir . '/amphp/byte-stream/src/Internal/functions.php',
    '0e6d7bf4a5811bfa5cf40c5ccd6fae6a' => $vendorDir . '/symfony/polyfill-mbstring/bootstrap.php',
    '107d0b55c0ad2d07d8643394552ac029' => $vendorDir . '/amphp/process/src/functions.php',
    '6e3fae29631ef280660b3cdad06f25a8' => $vendorDir . '/symfony/deprecation-contracts/function.php',
    '320cde22f66dd4f5d3fd621d3e88b98f' => $vendorDir . '/symfony/polyfill-ctype/bootstrap.php',
    '445532134d762b3cbc25500cac266092' => $vendorDir . '/daverandom/libdns/src/functions.php',
    'f4c761ca07639428acc28ba36643243e' => $vendorDir . '/amphp/dns/src/functions.php',
    '8825ede83f2f289127722d4e842cf7e8' => $vendorDir . '/symfony/polyfill-intl-grapheme/bootstrap.php',
    'e69f7f6ee287b969198c3c9d6777bd38' => $vendorDir . '/symfony/polyfill-intl-normalizer/bootstrap.php',
    'd4e415514e4352172d58f02433fa50e4' => $vendorDir . '/amphp/socket/src/functions.php',
    '1c2dcb9d6851a7abaae89f9586ddd460' => $vendorDir . '/amphp/socket/src/Internal/functions.php',
    '7fec264e4908bd2976476a6241c3f4d0' => $vendorDir . '/amphp/socket/src/SocketAddress/functions.php',
    'b6b991a57620e2fb6b2f66f03fe9ddc2' => $vendorDir . '/symfony/string/Resources/functions.php',
    '5d92623b88cbe7c988a6aca6fa79ebce' => $vendorDir . '/amphp/parallel/src/Context/functions.php',
    'c20d640a0b06382af5bbe7e0c7475adf' => $vendorDir . '/amphp/parallel/src/Context/Internal/functions.php',
    '42a785d14bae29c606457755ffd1653b' => $vendorDir . '/amphp/parallel/src/Ipc/functions.php',
    '805558626a43dc52f2afc82368f8d62e' => $vendorDir . '/amphp/parallel/src/Worker/functions.php',
);
