OC.L10N.register(
    "deck",
    {
    "The file was uploaded" : "Ulac afaylu yettwaznen",
    "The file was only partially uploaded" : "Afaylu, cwiṭ kan i yettwaznen segs",
    "No file was uploaded" : "Ulac afaylu i d-yettwasulin",
    "Missing a temporary folder" : "Ixuṣ ukaram akudan",
    "Load more" : "Sali-d ugar",
    "Finished" : "Immed",
    "Later" : "Ticki",
    "copy" : "nɣel",
    "To Do" : "Ad yettwaxdem",
    "Done" : "Immed",
    "Attachments" : "Ticeqqufin",
    "File" : "Afaylu",
    "Cancel" : "Sefsex",
    "Clear search" : "Sfeḍ anadi",
    "Active filters" : "Imzizdigen urmiden",
    "Open" : "Ldi",
    "Completed" : "Yemmed",
    "Open details" : "Ldi talqayt",
    "Details" : "Talqayt",
    "General" : "Amatu",
    "Appearance" : "Udem",
    "Search" : "Nadi",
    "Sharing" : "Beṭṭu",
    "Tags" : "Tibzimin",
    "Activity" : "Armud",
    "Undo" : "Sefsex",
    "Hours" : "Isragen",
    "Days" : "Ussan",
    "Transfer" : "Seḍfeṛ",
    "Owner" : "Bab",
    "Delete" : "Kkes",
    "Edit" : "Ẓreg",
    "Members" : "Imedrawen",
    "Download" : "Sider",
    "Comments" : "Commentaires",
    "Submit" : "Azen",
    "Cancel reply" : "Semmet tiririt.",
    "Reply" : "Err",
    "Update" : "Leqqem",
    "Description" : "Aglam",
    "Open link" : "Nɣel aseɣwen",
    "Cancel edit" : "Sefsex aseẓreg",
    "An error occurred" : "Teḍra-d tuccḍa",
    "No notifications" : "Ulac tisezmal",
    "Export" : "Sifeḍ",
    "No results found" : "Ur d-yufi kra seg unadi",
    "Close" : "Mdel",
    "Something went wrong" : "Yella wayen yeḍran",
    "Created" : "Yettwarna.",
    "Modified" : "Yettwabeddel",
    "Share" : "Bḍu",
    "Save" : "Sekles",
    "seconds ago" : "Tasinin aya",
    "Today" : "Ass-a"
},
"nplurals=2; plural=(n != 1);");
