"use strict";(self.webpackChunkdeck=self.webpackChunkdeck||[]).push([["src_store_dashboard_js"],{60650(e,s,a){a.d(s,{default:()=>i});var t=a(85471),r=a(95353),d=a(45699),c=a(63814);t.default.use(r.default);const n=new class{url(e){return(0,c.KT)("apps/deck/api/v1.0/".concat(e))}get(e){return d.Ay.get(this.url("overview/".concat(e)),{headers:{"OCS-APIRequest":"true"}}).then((e=>Promise.resolve(e.data.ocs.data)),(e=>Promise.reject(e))).catch((e=>Promise.reject(e)))}},i={state:{assignedCards:[]},getters:{assignedCardsDashboard:e=>e.assignedCards},mutations:{setAssignedCards(e,s){e.assignedCards=s}},actions:{async loadUpcoming(e){let{commit:s}=e;s("setAssignedCards",await n.get("upcoming"))}}}}}]);
//# sourceMappingURL=deck-src_store_dashboard_js.js.map?v=9e89cedfc76d736dd648