OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Létrehozta az új {board} táblát",
    "{user} has created a new board {board}" : "{user} létrehozta az új {board} táblát",
    "You have deleted the board {board}" : "Törölte a(z) {board} táblát",
    "{user} has deleted the board {board}" : "{user} törölte a(z) {board} táblát",
    "You have restored the board {board}" : "Visszaállította a(z) {board} táblát",
    "{user} has restored the board {board}" : "{user} visszaállította a(z) {board} táblát",
    "You have shared the board {board} with {acl}" : "Megosztotta a(z) {board} táblát a következővel: {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} megosztotta a(z) {board} táblát a következővel: {acl}",
    "You have removed {acl} from the board {board}" : "Eltávolította a következőt a(z) {board} táblától: {acl}",
    "{user} has removed {acl} from the board {board}" : "{user} eltávolította a következőt a(z) {board} táblától: {acl}",
    "You have renamed the board {before} to {board}" : "Átnevezte a(z) {before} táblát erre: {board}",
    "{user} has renamed the board {before} to {board}" : "{user} átnevezte a(z) {before} táblát erre: {board}",
    "You have archived the board {board}" : "Archiválta a(z) {board} táblát",
    "{user} has archived the board {before}" : "{user} archiválta a(z) {before} táblát",
    "You have unarchived the board {board}" : "Visszavonta a(z) {board} tábla archiválását",
    "{user} has unarchived the board {before}" : "{user} visszavonta a(z) {before} tábla archiválását",
    "You have created a new list {stack} on board {board}" : "Létrehozta az új {stack} listát a(z) {board} táblán",
    "{user} has created a new list {stack} on board {board}" : "{user} létrehozta az új {stack} listát a(z) {board} táblán",
    "You have renamed list {before} to {stack} on board {board}" : "Átnevezte a(z) {board} tábla {before} listát erre: {stack}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} átnevezte a(z) {board} tábla {before} listáját erre: {stack}",
    "You have deleted list {stack} on board {board}" : "Törölte a(z) {stack} listát a(z) {board} tábláról",
    "{user} has deleted list {stack} on board {board}" : "{user} törölte a(z) {stack} listát a(z) {board} tábláról",
    "You have created card {card} in list {stack} on board {board}" : "Létrehozta a(z) {card} kártyát a(z) {stack} listában, a(z) {board} táblán",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} létrehozta a(z) {card} kártyát a(z) {stack} listában, a(z) {board} táblán",
    "You have deleted card {card} in list {stack} on board {board}" : "Törölte a(z) {card} kártyát a(z) {stack} listából, a(z) {board} táblán",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} törölte a(z) {card} kártyát a(z) {stack} listában, a(z) {board} táblán",
    "You have renamed the card {before} to {card}" : "Átnevezte a(z) {before} kártyát erre: {card}",
    "{user} has renamed the card {before} to {card}" : "{user} átnevezte a(z) {before} kártyát erre: {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Leírást adott hozzá a(z) {card} kártyához a(z) {stack} listában, a(z) {board} táblán",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} leírást adott hozzá a(z) {card} kártyához a(z) {stack} listában, a(z) {board} táblán",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Frissítette a(z) {card} kártya leírását a(z) {stack} listában, a(z) {board} táblán",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} frissítette a(z) {card} kártya leírását a(z) {stack} listában, a(z) {board} táblán",
    "You have archived card {card} in list {stack} on board {board}" : "Archiválta a(z) {card} kártyát a(z) {stack} listában, a(z) {board} táblán",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} archiválta a(z) {card} kártyát a(z) {stack} listában, a(z) {board} táblán",
    "You have unarchived card {card} in list {stack} on board {board}" : "Visszavonta a(z) {card} kártya archiválását a(z) {stack} listában, a(z) {board} táblán",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} visszavonta a(z) {card} kártya archiválását a(z) {stack} listában, a(z) {board} táblán",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Késznek jelölte a(z) {card} kártyát a(z) {stack} listában, a(z) {board} táblán",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} késznek jelölte a(z) {card} kártyát a(z) {stack} listában, a(z) {board} táblán",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Visszavonta a(z) {card} kártya késznek jelölését a(z) {stack} listában, a(z) {board} táblán",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} visszavonta a(z) {card} kártya késznek jelölését a(z) {stack} listában, a(z) {board} táblán",
    "You have removed the due date of card {card}" : "Eltávolította a(z) {card} kártya esedékességét",
    "{user} has removed the due date of card {card}" : "{user} eltávolította a(z) {card} kártya esedékességét",
    "You have set the due date of card {card} to {after}" : "Erre állította a(z) {card} kártya határidejét: {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} erre állította a(z) {card} kártya határidejét: {after}",
    "You have updated the due date of card {card} to {after}" : "Frissítette a(z) {card} kártya esedékességét erre: {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} frissítette a(z) {card} kártya esedékességét erre: {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Hozzáadta a(z) {label} címkét a(z) {card} kártyához, a(z) {stack} listában, a(z) {board} táblán",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} hozzáadta a(z) {label} címkét a(z) {card} kártyához, a(z) {stack} listában, a(z) {board} táblán",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Eltávolította a(z) {label} címkét a(z) {card} kártyáról, a(z) {stack} listában, a(z) {board} táblán",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} eltávolította a(z) {label} címkét a(z) {card} kártyáról, a(z) {stack} listában, a(z) {board} táblán",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Hozzárendelte a(z) {card} kártyát a(z) {board} táblán a következőhöz: {assigneduser}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} hozzárendelte a(z) {card} kártyát a(z) {board} táblán a következőhöz: {assigneduser}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Eltávolította a(z) {card} kártyát a(z) {board} táblán a következőtől: {assigneduser}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} eltávolította a(z) {card} kártyát a(z) {board} táblán a következőtől: {assigneduser}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Áthelyezte a(z) {card} kártyát a(z) {stackBefore} listából a(z) {stack} listába",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} áthelyezte a(z) {card} kártyát a(z) {stackBefore} listából a(z) {stack} listába",
    "You have added the attachment {attachment} to card {card}" : "Hozzáadta a(z) {attachment} mellékletet a(z) {card} kártyához",
    "{user} has added the attachment {attachment} to card {card}" : "{user} hozzáadta a(z) {attachment} mellékletet a(z) {card} kártyához",
    "You have updated the attachment {attachment} on card {card}" : "Frissítette a(z) {attachment} mellékletet a(z) {card} kártyánál",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} frissítette a(z) {attachment} mellékletet a(z) {card} kártyán",
    "You have deleted the attachment {attachment} from card {card}" : "Eltávolította a(z) {attachment} mellékletet a(z) {card} kártyáról",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} eltávolította a(z) {attachment} mellékletet a(z) {card} kártyáról",
    "You have restored the attachment {attachment} to card {card}" : "Visszaállította a(z) {attachment} mellékletet a(z) {card} kártyánál",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} visszaállította a(z) {attachment} mellékletet a(z) {card} kártyánál",
    "You have commented on card {card}" : "Hozzászólt a(z) {card} kártyához",
    "{user} has commented on card {card}" : "{user} hozzászólt a(z) {card} kártyához",
    "Deck" : "Kártyák",
    "Changes in the <strong>Deck app</strong>" : "Változások a <strong>Kártyák alkalmazásban</strong>",
    "A <strong>board, list or card</strong> was changed" : "Egy <strong>tábla, lista vagy kártya</strong> megváltozott",
    "A <strong>comment</strong> was created on a card" : "Egy <strong>hozzászólás</strong> lett létrehozva egy kártyán",
    "A <strong>card description</strong> has been changed" : "Egy <strong>kártyaleírás</strong> megváltozott",
    "The file was uploaded" : "A fájl fel lett töltve",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "A feltöltött fájl meghaladja a php.ini-ben szereplő upload_max_filesize direktívában megadott méretet",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "A feltöltött fájl meghaladja a HTML űrlapon megadott MAX_FILE_SIZE direktívában meghatározott méretet",
    "The file was only partially uploaded" : "A fájl csak részlegesen lett feltöltve",
    "No file was uploaded" : "Nincs feltöltött fájl",
    "Missing a temporary folder" : "Átmeneti mappa hiányzik",
    "Could not write file to disk" : "Nem lehet a fájlt lemezre írni",
    "A PHP extension stopped the file upload" : "A PHP kiterjesztés megállította a fájl feltöltését",
    "No file uploaded or file size exceeds maximum of %s" : "Nincs fájl feltöltve, vagy a fájl meghaladja a maximumot: %s",
    "Invalid file type. Only JSON files are allowed." : "Érvénytelen fájltípus. Csak a JSON fájlok engedélyezettek.",
    "Invalid JSON data" : "Érvénytelen JSON adatok",
    "Failed to import board" : "A tábla importálása sikertelen",
    "Cards due today" : "Kártya határideje: ma",
    "Cards due tomorrow" : "Kártya határideje: holnap",
    "Upcoming cards" : "Közelgő kártyák",
    "Load more" : "Továbbiak betöltése",
    "Welcome to Nextcloud Deck!" : "Üdvözli a Nextcloud Kártyák!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "A(z) „%s” kártyát a(z) „%s” táblán %s hozzárendelte Önhöz.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} hozzárendelte Önhöz a(z) {deck-board} táblán lévő {deck-card} kártyát.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "A(z) „%1$s” kártya a(z) „%2$s” táblán elérte a határidejét.",
    "The card {deck-card} on {deck-board} has reached its due date." : "A(z) {deck-board} táblán lévő {deck-card} kártya elérte a határidejét.",
    "%s has mentioned you in a comment on \"%s\"." : "%s megemlítette egy hozzászólásban ennél: „%s”.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} megemlítette egy hozzászólásban a(z) {deck-card} kártyán.",
    "The board \"%s\" has been shared with you by %s." : "A(z) „%s” táblát %s osztotta meg Önnel.",
    "{user} has shared {deck-board} with you." : "{user} megosztotta Önnel a(z) {deck-board} táblát.",
    "The remote board %1$s has been shared with you by %2$s" : "A(z) %1$s távoli táblát %2$s osztotta meg Önnel.",
    "Deck board" : "Kártyatábla",
    "Owned by %1$s" : "Tulajdonos: %1$s",
    "Deck boards, cards and comments" : "Kártyatáblák, kártyák és megjegyzések",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "A következőtől: %1$s, ebben: %2$s/%3$s, tulajdonos: %4$s",
    "Create a new deck card" : "Új kártya létrehozása",
    "Card comments" : "Kártya hozzászólásai",
    "%s on %s" : "%s ezen: %s",
    "Deck boards and cards" : "Kártyatáblák és kártyák",
    "No data was provided to create an attachment." : "Nincsenek megadva adatok a melléklet létrehozásához.",
    "Finished" : "Kész",
    "To review" : "Áttekintendő",
    "Action needed" : "Művelet szükséges",
    "Later" : "Később",
    "copy" : "másolat",
    "Read more inside" : "Továbbolvasás",
    "Custom lists - click to rename!" : "Egyéni listák – kattintson az átnevezéshez.",
    "To Do" : "Teendő",
    "In Progress" : "Folyamatban",
    "Done" : "Kész",
    "1. Open to learn more about boards and cards" : "1. Nyissa meg, hogy többet tudjon meg a táblákról és kártyákról",
    "2. Drag cards left and right, up and down" : "2. Húzza a kártyákat balra és jobbra, fel és le",
    "3. Apply rich formatting and link content" : "3. Alkalmazzon gazdag formázást és hivatkozzon tartalomra",
    "4. Share, comment and collaborate!" : "4. Osszon meg, szóljon hozzá és dolgozzanak együtt.",
    "Create your first card!" : "Hozza létre az első kártyáját.",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Ez a megjegyzés több mint %s karakter hosszú.\nMellékletként lett hozzáadva a(z) %s nevű kártyához.\nEzen a webcímen érhető el: %s.",
    "Attachments" : "Mellékletek",
    "File" : "Fájl",
    "date" : "dátum",
    "Card not found" : "A kártya nem található",
    "Path is already shared with this card" : "Az útvonal már meg van osztva ezzel a kártyával",
    "Invalid date, date format must be YYYY-MM-DD" : "Érvénytelen dátum, a dátumnak YYYY-MM-DD formátumúnak kell lennie",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Az összes tábla, amelynek Ön a tulajdonosa, beleértve a listákat, kártyákat, címkéket, hozzárendeléseket és megjegyzéseket",
    "Personal planning and team project organization" : "Személyes tervezés és csapatos projektszervezés",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "A Kártyák egy kanban-stílusú szervezőeszköz, amely a személyes tervezést és a csapatok projektszervezését célozza, a Nextcloudba integrálva.\n\n\n- 📥 Saját feladatok kártyákhoz adása, és azok sorrendezése\n- 📄 További jegyzetek leírása Markdownban\n- 🔖 Címkék hozzárendelése a még jobb rendszerezés miatt\n- 👥 Megosztás a csapattal, barátokkal vagy családdal\n- 📎 Fájlok hozzáadása mellékletként, vagy beágyazás a Markdown leírásba\n- 💬 Megbeszélés a csapattal hozzászólások használatával\n- ⚡ A változások követése a tevékenységnaplóban\n- 🚀 Rendszerezze a projektjét",
    "Add board" : "Tábla hozzáadása",
    "Card details" : "Kártya részletei",
    "Select the board to link to a project" : "Válasszon ki egy táblát, amely egy projektre fog hivatkozni",
    "Search by board title" : "Keresés táblacím szerint",
    "Select board" : "Válasszon táblát",
    "Move/copy card" : "Kártya áthelyezése/másolása",
    "Select a board" : "Válasszon egy táblát",
    "No lists available" : "Nincs elérhető lista",
    "Select a list" : "Válasszon listát",
    "Move card" : "Kártya áthelyezése",
    "Copy card" : "Kártya másolása",
    "Select the card to link to a project" : "Válasszon ki egy kártyát, amely egy projektre fog hivatkozni",
    "Link to card" : "Hivatkozás egy kártyára",
    "Select a card" : "Válasszon egy kártyát",
    "Cancel" : "Mégse",
    "This board is read only" : "Ez a tábla csak olvasható",
    "Drop your files to upload" : "Dobja ide a fájljait a feltöltéshez",
    "File already exists" : "A fájl már létezik",
    "A file with the name {filename} already exists." : "{filename} nevű fájl már létezik.",
    "Do you want to overwrite it?" : "Felülírja?",
    "Overwrite file" : "Fájl felülírása",
    "Keep existing file" : "Meglévő fájl megtartása",
    "Add card" : "Kártya hozzáadása",
    "Archived cards" : "Archivált kártyák",
    "Add list" : "Lista hozzáadása",
    "List name" : "Listanév",
    "Clear search" : "Keresés törlése",
    "Active filters" : "Aktív szűrők",
    "Apply filter" : "Szűrő alkalmazása",
    "Filter by tag" : "Szűrés címke szerint",
    "Filter by assigned user" : "Szűrés hozzárendelt felhasználó szerint",
    "Unassigned" : "Nem hozzárendelt",
    "Filter by status" : "Szűrés állapot szerint",
    "Open and completed" : "Megnyitva és befejezve",
    "Open" : "Megnyitás",
    "Completed" : "Teljesített",
    "Filter by due date" : "Szűrés határidő szerint",
    "Overdue" : "Lejárt",
    "Next 24 hours" : "Következő 24 óra",
    "Next 7 days" : "Következő 7 nap",
    "Next 30 days" : "Következő 30 nap",
    "No due date" : "Nincs határidő",
    "Clear filter" : "Szűrő törlése",
    "View Modes" : "Módok megtekintése",
    "Toggle View Modes" : "Nézetmódváltás",
    "Kanban view" : "Kanban nézet",
    "Gantt view" : "Gantt nézet",
    "Hide archived cards" : "Archivált kártyák elrejtése",
    "Show archived cards" : "Archivált kártyák megjelenítése",
    "Toggle compact mode" : "Kompakt mód be/ki",
    "Hide card cover images" : "Kártyaborítóképek elrejtése",
    "Show card cover images" : "Kártyaborítóképek megjelenítése",
    "Open details" : "Részletek megnyitása",
    "Details" : "Részletek",
    "Deck settings" : "Kártyák beállításai",
    "General" : "Általános",
    "Use bigger card view" : "Nagyobb kártyanézet használata",
    "Appearance" : "Megjelenés",
    "Show card ID badge" : "Kártyaazonosító jelvény megjelenítése",
    "Show boards in calendar/tasks" : "Táblák megjelenítése a naptárak/teendők között",
    "Admin settings" : "Rendszergazdai beállítások",
    "Limit board creation to some groups" : "A táblalétrehozás bizonyos csoportokra korlátozása",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Az ezeken a csoportokon kívüli felhasználók nem fogják tudni létrehozni a saját tábláikat, de dolgozhatnak azokon a táblákon, melyek meg lettek velük osztva.",
    "Enable federation" : "Föderáció engedélyezése",
    "Board actions" : "Táblaműveletek",
    "Scroll sideways" : "Görgetés oldalra",
    "Navigate between cards" : "Navigáció a kártyák közt",
    "Close card details" : "Kártya részleteinek bezárása",
    "Search" : "Keresés",
    "Show card filters" : "Kártyaszűrők megjelenítése",
    "Clear card filters" : "Kártyaszűrők törlése",
    "Show those shortcuts" : "Gyorsbillentyűk megjelenítése",
    "Card actions" : "Kártyaműveletek",
    "Open card details" : "Kártya részleteinek megnyitása",
    "Edit the card title" : "Kártya címének szerkesztése",
    "Assign yourself to the current card" : "Saját maga hozzárendelése a jelenlegi kártyához",
    "Archive/unarchive the current card" : "Jelenlegi kártya archiválása be/ki",
    "Mark card as completed/not completed" : "Jelenlegi kártya késznek jelölése be/ki",
    "Open card menu" : "Kártyamenü megnyitása",
    "Currently present people" : "Jelenleg jelenlévő emberek",
    "Loading board" : "Tábla betöltése",
    "Board not found" : "A tábla nem található",
    "Create a new list to add cards to this board" : "Hozzon létre egy új listát a kártyák ehhez a táblához való hozzáadásához",
    "Sharing" : "Megosztás",
    "Tags" : "Címkék",
    "Deleted items" : "Törölt elemek",
    "Activity" : "Tevékenység",
    "Deleted lists" : "Törölt listák",
    "Undo" : "Visszavonás",
    "Deleted cards" : "Törölt kártyák",
    "Hours" : "Óra",
    "Days" : "Napok",
    "Weeks" : "Hetek",
    "Months" : "Hónapok",
    "No cards yet" : "Még nincsenek kártyák",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Állítson be kezdődátumot és határidőt a kártyákon, hogy lássa őket a Gantt-diagramon",
    "Cards without dates ({count})" : "Dátum nélküli kártyák ({count})",
    "Failed to create share with {displayName}" : "Nem lehet létrehozni a következő megosztást: {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Biztos, hogy átadja a(z) {board} tábla tulajdonjogát {user} számára?",
    "Transfer the board." : "A tábla átadása.",
    "Transfer" : "Átadás",
    "The board has been transferred to {user}" : "A tábla átadása {user} számára sikeres",
    "Failed to transfer the board to {user}" : "A tábla átadása {user} számára sikertelen",
    "Board owner" : "Tábla tulajdonosa",
    "(Group)" : "(Csoport)",
    "(Team)" : "(Csapat)",
    "(remote)" : "(távoli)",
    "Can edit" : "Szerkesztheti",
    "Can share" : "Megoszthatja",
    "Can manage" : "Kezelheti",
    "Owner" : "Tulajdonos",
    "Delete" : "Törlés",
    "List deleted" : "Lista törölve",
    "Edit list title" : "Listacím szerkesztése",
    "Archive all cards" : "Az összes kártya archiválása",
    "Unarchive all cards" : "Az összes kártya archiválásának visszavonása",
    "Do not set cards as \"done\"" : "Ne állítsa be a kártyákat „készként”",
    "Set cards as \"done\"" : "Kártyák beállítása „készként”",
    "Delete list" : "Lista törlése",
    "Archive all cards in this list" : "Az összes kártya archiválása ebben a listában",
    "Unarchive all cards in this list" : "Az összes kártya archiválásának visszavonása ebben a listában",
    "Add a new card" : "Új kártya hozzáadása",
    "Card name" : "Kártya neve",
    "title and color value must be provided" : "a cím és szín értékét meg kell adni",
    "Edit" : "Szerkesztés",
    "Add a new tag" : "Új címke hozzáadása",
    "Board name" : "Tábla neve",
    "Members" : "Tagok",
    "Assign to users/groups/team" : "Felhasználóhoz/csoporthoz/csapathoz rendelés",
    "Assign a user to this card…" : "Felhasználó rendelése ehhez a kártyához…",
    "Select a user to assign to this card…" : "Válasszon kártyához rendelendő felhasználót…",
    "File to share" : "Fájl megosztása",
    "Invalid path selected" : "Érvénytelen útvonal kiválasztva",
    "Upload new files" : "Új fájlok feltöltése",
    "Share from Files" : "Megosztás a Fájlokból",
    "Pending share" : "Függőben lévő megosztás",
    "Add this attachment" : "E melléklet hozzáadása",
    "Show in Files" : "Megjelenítés a Fájlokban",
    "Download" : "Letöltés",
    "Remove attachment" : "Melléklet eltávolítása",
    "Delete Attachment" : "Melléklet törlése",
    "Restore Attachment" : "Melléklet visszaállítása",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Módosítva: {modifiedDate} ⸱ Létrehozva: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Módosítva: {modifiedDate}\nLétrehozva: {createdDate}\nLétrehozta: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Módosítva: {modifiedDate}\nLétrehozva: {createdDate}",
    "The title cannot be empty." : "A cím nem lehet üres.",
    "Cannot close unsaved card!" : "A mentetlen kártya nem zárható be!",
    "Open in sidebar view" : "Oldalsáv nézet megnyitása",
    "Open in bigger view" : "Megtekintés nagyobb nézetben",
    "Comments" : "Hozzászólások",
    "Failed to load comments" : "A hozzászólások betöltése sikertelen",
    "No comments yet. Begin the discussion!" : "Még nincsenek hozzászólások. Kezdje el a beszélgetést!",
    "The comment cannot be empty." : "A hozzászólás nem lehet üres.",
    "The comment cannot be longer than 1000 characters." : "A megjegyzés nem lehet 1000 karakternél hosszabb.",
    "Submit" : "Beküldés",
    "Created:" : "Létrehozva:",
    "In reply to" : "Válaszként erre:",
    "Cancel reply" : "Válasz elvetése",
    "Reply" : "Válasz",
    "Update" : "Frissítés",
    "Assign dependent cards" : "Függő kártyák hozzárendelése",
    "Mark as done" : "Megjelölés készként",
    "Mark as not done" : "Megjelölés nem készként",
    "Remove dependency" : "Függőség eltávolítása",
    "Add dependent card" : "Függő kártya hozzáadása",
    "Assign a dependent card…" : "Függő kártya hozzárendelése…",
    "Select a dependent card…" : "Válasszon függő kártyát…",
    "Write a description …" : "Leírás megadása…",
    "The description has been changed by another user." : "A leírást egy másik felhasználó megváltoztatta.",
    "Could not save description" : "A leírás mentése sikertelen",
    "Description" : "Leírás",
    "(Unsaved)" : "(Nem mentett)",
    "(Saving…)" : "(Mentés…)",
    "Formatting help" : "Formázási súgó",
    "Edit description" : "Leírás szerkesztése",
    "View description" : "Leírás megtekintése",
    "Add Attachment" : "Melléklet hozzáadása",
    "Choose attachment" : "Válasszon mellékletet",
    "Select Date" : "Dátum kiválasztása",
    "Later today – {timeLocale}" : "Később a mai nap – {timeLocale}",
    "Set due date for later today" : "Határidő beállítása a mai napon későbbre",
    "Tomorrow – {timeLocale}" : "Holnap – {timeLocale}",
    "Set due date for tomorrow" : "Határidő beállítása holnapra",
    "This weekend – {timeLocale}" : "Ez a hétvége – {timeLocale}",
    "Set due date for this weekend" : "Határidő beállítása erre a hétvégére",
    "Next week – {timeLocale}" : "Következő hét – {timeLocale}",
    "Set due date for next week" : "Határidő beállítása a következő hétre",
    "Assign a due date to this card…" : "Határidő rendelése ehhez a kártyához…",
    "Set a due date" : "Határidő beállítása",
    "Add due date" : "Határidő hozzáadása",
    "Choose a date" : "Válasszon dátumot",
    "Remove due date" : "Határidő eltávolítása",
    "Due at:" : "Határidő:",
    "Not done" : "Nincs kész",
    "Unarchive card" : "Kártya archiválásának visszavonása",
    "Archive card" : "Kártya archiválása",
    "Assign a start date to this card…" : "Kezdési dátum rendelése ehhez a kártyához…",
    "Set a start date" : "Kezdési dátum beállítása",
    "Add start date" : "Kezdési dátum hozzáadása",
    "Remove start date" : "Kezdési dátum eltávolítása",
    "Assign a tag to this card…" : "Címke rendelése ehhez a kártyához…",
    "Select or create a tag…" : "Válasszon vagy hozzon létre egy címkét…",
    "Create a new tag:" : "Új címke létrehozása:",
    "(group)" : "(csoport)",
    "{count} comments, {unread} unread" : "{count} hozzászólás, {unread} olvasatlan",
    "Todo items" : "Teendő elemek",
    "Edit card title" : "Kártyacím szerkesztése",
    "Open link" : "Hivatkozás megnyitása",
    "Card deleted" : "Kártya törölve",
    "Change card color" : "Kártya színének módosítása",
    "Edit title" : "Cím szerkesztése",
    "Assign to me" : "Hozzám rendelés",
    "Unassign myself" : "Saját magam hozzárendelésének eltávolítása",
    "Delete card" : "Kártya törlése",
    "All boards" : "Az összes tábla",
    "Archived boards" : "Archivált táblák",
    "Shared with you" : "Megosztva Önnel",
    "Cancel edit" : "Szerkesztés elvetése",
    "Save board" : "Tábla mentése",
    "Board {0} deleted" : "A(z) {0} tábla törölve",
    "All cards" : "Összes kártya",
    "Only assigned cards" : "Csak hozzárendelt kártyák",
    "No reminder" : "Nincs emlékeztető",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Biztos, hogy törli a(z) {title} táblát? Ez törölni fogja a tábla összes adatát, köztük az archivált kártyákat is.",
    "Delete the board?" : "Törli a táblát?",
    "Are you sure you want to leave the board {title}?" : "Biztos, hogy elhagyja a(z) {title} táblát?",
    "Leave the board?" : "Elhagyja a táblát?",
    "Leave" : "Elhagyás",
    "Failed to leave the board" : "Nem sikerült elhagyni a táblát",
    "Exporting board..." : "Tábla exportálása…",
    "An error occurred" : "Hiba történt",
    "Board details" : "Tábla részletei",
    "Edit board" : "Tábla szerkesztése",
    "Clone board" : "Tábla klónozása",
    "Unarchive board" : "Tábla archiválásának visszavonása",
    "Archive board" : "Tábla archiválása",
    "Export board" : "Tábla exportálása",
    "Turn on due date reminders" : "Határidő emlékeztető beállítása",
    "Turn off due date reminders" : "Határidő emlékeztető kikapcsolása",
    "Remove as default board" : "Alapértelmezett tábla eltávolítása",
    "Set as default board" : "Beállítás alapértelmezett táblaként",
    "Due date reminders" : "Határidő emlékeztetők",
    "Assigned cards" : "Hozzárendelt kártyák",
    "No notifications" : "Nincsenek értesítések",
    "Delete board" : "Tábla törlése",
    "Leave board" : "Tábla elhagyása",
    "Importing board..." : "Tábla importálása…",
    "Board imported successfully" : "Tábla sikeresen importálva",
    "Import board" : "Tábla importálása",
    "Clone {boardTitle}" : "A(z) {boardTitle} klónozása",
    "Clone cards" : "Kártyák klónozása",
    "Clone assignments" : "Hozzárendelések klónozása",
    "Clone labels" : "Címkék klónozása",
    "Clone due dates" : "Határidők klónozása",
    "Advanced options" : "Speciális beállítások",
    "Move all cards to the first list" : "Összes kártya áthelyezése az első listára",
    "Restore archived cards" : "Archivált kártyák helyreállítása",
    "Clone" : "Klónozás",
    "Export {boardTitle}" : "A(z) {boardTitle} exportálása",
    "Export as JSON" : "Exportálás JSON-ként",
    "Export as CSV" : "Exportálás CSV-ként",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Megjegyzés: Csak a JSON formátum támogatott a Kártyák alkalmazásba való importáláskor.",
    "Export" : "Exportálás",
    "Loading filtered view" : "Szűrt nézet betöltése",
    "Search for {searchQuery} in other boards" : "Keresés a(z) {searchQuery} kifejezésre a többi táblában",
    "Search for {searchQuery} in all boards" : "Keresés a(z) {searchQuery} kifejezésre az összes táblában",
    "No results found" : "Nincs találat",
    "Deck board {name}\n* Last modified on {lastMod}" : "{name} kártyatábla\n* Legutóbb módosítva: {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Létrehozva: {created}\n* Utoljára módosítva: {lastMod}\n* {nbAttachments} melléklet\n* {nbComments} megjegyzés",
    "{nbCards} cards" : "{nbCards} kártya",
    "Due on {date}" : "Határidő: {date}",
    "{stack} in {board}" : "{stack} itt: {board}",
    "Click to expand description" : "Kattintson a leírás kibontásához",
    "Click to expand comment" : "Kattintson a megjegyzés kibontásához",
    "Create card" : "Kártya létrehozása",
    "Create a new card" : "Új kártya létrehozása",
    "Card title" : "Kártya címe",
    "Creating the new card …" : "Új kártya létrehozása…",
    "Card \"{card}\" was added to \"{board}\"" : "A(z) „{card}” kártya hozzáadva a(z) „{board}” táblához",
    "Open card" : "Kártya megnyitása",
    "Close" : "Bezárás",
    "No upcoming cards" : "Nincsenek közelgő kártyák",
    "upcoming cards today" : "közelgő kártyák a mai napon",
    "upcoming cards tomorrow" : "közelgő kártyák holnap",
    "upcoming cards" : "közelgő kártyák",
    "New card" : "Új kártya",
    "Link to a board" : "Hivatkozás egy táblához",
    "Link to a card" : "Hivatkozás egy kártyához",
    "Create a card" : "Kártya létrehozása",
    "Message from {author} in {conversationName}" : "Üzenet a(z) {conversationName} beszélgetésben tőle: {author}",
    "Something went wrong" : "Valami hiba történt",
    "Failed to upload {name}" : "Feltöltés sikertelen: {name}",
    "Maximum file size of {size} exceeded" : "A legnagyobb fájlméret ({size}) túllépve",
    "Assigned users" : "Hozzárendelt felhasználók",
    "Due date" : "Határidő",
    "Created" : "Létrehozva",
    "Modified" : "Módosítva",
    "Error creating the share" : "Megosztás létrehozása sikertelen",
    "Share with a Deck card" : "Megosztás kártyával",
    "Share {file} with a Deck card" : "A(z) {file} megosztása egy kártyával",
    "Share" : "Megosztás",
    "The card \"%s\" on \"%s\" has reached its due date." : "A(z) „%s” kártya a(z) „%s” táblán elérte a határidejét.",
    "Share board with a user, group or team …" : "Tábla megosztása felhasználóval, csoporttal vagy csapattal…",
    "Searching for users, groups and teams …" : "Felhasználók, csoportok és csapatok keresése…",
    "No participants found" : "Nem találhatók résztvevők",
    "Save" : "Mentés",
    "seconds ago" : "másodperce",
    "Today" : "Ma",
    "Tomorrow" : "Holnap",
    "No due" : "Nincs határidő"
},
"nplurals=2; plural=(n != 1);");
