OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Yeni {board} panosunu oluşturdunuz",
    "{user} has created a new board {board}" : "{user}, yeni {board} panosunu oluşturdu",
    "You have deleted the board {board}" : "{board} panosunu sildiniz",
    "{user} has deleted the board {board}" : "{user}, {board} panosunu sildi",
    "You have restored the board {board}" : "{board} panosunu geri yüklediniz",
    "{user} has restored the board {board}" : "{user}, {board} panosunu geri yükledi",
    "You have shared the board {board} with {acl}" : "{board} panosunu {acl} ile paylaştınız",
    "{user} has shared the board {board} with {acl}" : "{user}, {board} panosunu {acl} ile paylaştı",
    "You have removed {acl} from the board {board}" : "{board} panosunun {acl} ile paylaşımını kaldırdınız",
    "{user} has removed {acl} from the board {board}" : "{user}, {board} panosunun {acl} ile paylaşımını kaldırdı",
    "You have renamed the board {before} to {board}" : "{before} panosunun adını {board} olarak değiştirdiniz",
    "{user} has renamed the board {before} to {board}" : "{user}, {before} panosunun adını {board} olarak değiştirdi",
    "You have archived the board {board}" : "{board} panosunu arşive kaldırdınız",
    "{user} has archived the board {before}" : "{user}, {board} panosunu arşive kaldırdı",
    "You have unarchived the board {board}" : "{board} panosunu arşivden çıkardınız",
    "{user} has unarchived the board {before}" : "{user}, {board} panosunu arşivden çıkardı",
    "You have created a new list {stack} on board {board}" : "{board} panosunda yeni {stack} listesini oluşturdunuz",
    "{user} has created a new list {stack} on board {board}" : "{user}, {board} panosunda yeni {stack} listesini oluşturdu",
    "You have renamed list {before} to {stack} on board {board}" : "{board} panosundaki {before} listesinin adını {stack} olarak değiştirdiniz",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user}, {board} panosundaki {before} listesinin adını {stack} olarak değiştirdi",
    "You have deleted list {stack} on board {board}" : "{board} panosundaki {stack} listesini sildiniz",
    "{user} has deleted list {stack} on board {board}" : "{user}, {board} panosundaki {stack} listesini sildi",
    "You have created card {card} in list {stack} on board {board}" : "{board} panosundaki {stack} listesinde {card} kartını oluşturdunuz",
    "{user} has created card {card} in list {stack} on board {board}" : "{user}, {board} panosundaki {stack} listesinde {card} kartını oluşturdu",
    "You have deleted card {card} in list {stack} on board {board}" : "{board} panosunun {stack} listesindeki {card} kartını sildiniz",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user}, {board} panosunun {stack} listesindeki {card} kartını sildi",
    "You have renamed the card {before} to {card}" : "{before} kartının adını {card} olarak değiştirdiniz",
    "{user} has renamed the card {before} to {card}" : "{user}, {before} kartının adını {card} olarak değiştirdi",
    "You have added a description to card {card} in list {stack} on board {board}" : "{board} panosunun {stack} listesindeki {card} kartına bir açıklama eklediniz",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user}, {board} panosunun {stack} listesindeki {card} kartına bir açıklama ekledi",
    "You have updated the description of card {card} in list {stack} on board {board}" : "{board} panosunun {stack} listesindeki {card} kartının açıklamasını güncellediniz",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user}, {board} panosunun {stack} listesindeki {card} kartının açıklamasını güncelledi",
    "You have archived card {card} in list {stack} on board {board}" : "{board} panosunun {stack} listesindeki {card} kartını arşive kaldırdınız",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user}, {board} panosunun {stack} listesindeki {card} kartını arşive kaldırdı",
    "You have unarchived card {card} in list {stack} on board {board}" : "{board} panosunun {stack} listesindeki {card} kartını arşivden çıkardınız",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user}, {board} panosunun {stack} listesindeki {card} kartını arşivden çıkardı",
    "You have marked the card {card} as done in list {stack} on board {board}" : "{board} panosunun {stack} listesindeki {card} kartını tamamlanmış olarak işaretlediniz",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user}, {board} panosunun {stack} listesindeki {card} kartını tamamlanmış olarak işaretledi",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "{board} panosunun {stack} listesindeki {card} kartını tamamlanmamış olarak işaretlediniz",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user}, {board} panosunun {stack} listesindeki {card} kartını tamamlanmamış olarak işaretledi",
    "You have removed the due date of card {card}" : "{card} kartının bitiş tarihini kaldırdınız",
    "{user} has removed the due date of card {card}" : "{user}, {card} kartının bitiş tarihini kaldırdı",
    "You have set the due date of card {card} to {after}" : "{card} kartının bitiş tarihini {after} olarak ayarladınız",
    "{user} has set the due date of card {card} to {after}" : "{user}, {card} kartının bitiş tarihini {after} olarak ayarladı",
    "You have updated the due date of card {card} to {after}" : "{card} kartının bitiş tarihini {after} olarak güncellediniz",
    "{user} has updated the due date of card {card} to {after}" : "{user}, {card} kartının bitiş tarihini {after} olarak güncelledi",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "{board} panosunun {stack} listesindeki {card} kartına {label} etiketini eklediniz",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user}, {board} panosunun {stack} listesindeki {card} kartına {label} etiketini ekledi",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "{board} panosunun {stack} listesindeki {card} kartından {label} etiketini kaldırdınız",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user}, {board} panosunun {stack} listesindeki {card} kartından {label} etiketini kaldırdı",
    "You have assigned {assigneduser} to card {card} on board {board}" : "{board} panosundaki {card} kartını {assigneduser} kullanıcısına atadınız",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user}, {board} panosundaki {card} kartını {assigneduser} kullanıcısına atadı",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "{board} panosundaki {card} kartının atamasını {assigneduser} kullanıcısından kaldırdınız",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user}, {board} panosundaki {card} kartının atamasını {assigneduser} kullanıcısından kaldırdı",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "{card} kartını {stackBefore} listesinden {stack} listesine taşıdınız",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user}, {card} kartını {stackBefore} listesinden {stack} listesine taşıdı",
    "You have added the attachment {attachment} to card {card}" : "{card} kartına {attachment} dosyasını eklediniz",
    "{user} has added the attachment {attachment} to card {card}" : "{user}, {card} kartına {attachment} dosyasını ekledi",
    "You have updated the attachment {attachment} on card {card}" : "{card} kartındaki {attachment} dosyasını güncellediniz",
    "{user} has updated the attachment {attachment} on card {card}" : "{user}, {card} kartındaki {attachment} dosyasını güncelledi",
    "You have deleted the attachment {attachment} from card {card}" : "{card} kartındaki {attachment} dosyasını sildiniz",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user}, {card} kartındaki {attachment} dosyasını sildi",
    "You have restored the attachment {attachment} to card {card}" : "{card} kartındaki {attachment} dosyasını geri yüklediniz",
    "{user} has restored the attachment {attachment} to card {card}" : "{user}, {card} kartındaki {attachment} dosyasını geri yükledi",
    "You have commented on card {card}" : "{card} kartı ile ilgili yorum yaptınız",
    "{user} has commented on card {card}" : "{user}, {card} kartı ile ilgili yorum yaptı",
    "Deck" : "Tahta",
    "Changes in the <strong>Deck app</strong>" : "<strong>Tahta uygulamasında</strong> değişiklik yapıldığında",
    "A <strong>board, list or card</strong> was changed" : "Bir <strong>pano, liste ya da kart</strong> değiştirildi",
    "A <strong>comment</strong> was created on a card" : "Bir kart için bir <strong>yorum</strong> yapıldığında",
    "A <strong>card description</strong> has been changed" : "Bir <strong>kart açıklaması</strong> değiştirildi",
    "The file was uploaded" : "Dosya yüklendi",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Yüklenen dosya php.ini dosyasındaki yüklenebilecek dosya boyutunu belirten upload_max_filesize değişkeninin değerini aşıyor",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Yüklenen dosya HTML formundaki yüklenebilecek dosya boyutunu belirten MAX_FILE_SIZE değişkeninin değerini aşıyor",
    "The file was only partially uploaded" : "Dosyanın yalnızca bir bölümü yüklendi",
    "No file was uploaded" : "Herhangi bir dosya yüklenmedi",
    "Missing a temporary folder" : "Geçici bir klasör eksik",
    "Could not write file to disk" : "Dosya diske yazılamadı",
    "A PHP extension stopped the file upload" : "Bir PHP eklentisi dosyanın yüklenmesini engelledi",
    "No file uploaded or file size exceeds maximum of %s" : "Herhangi bir dosya yüklenmedi ya da %s olan en büyük dosya boyutu sınırı aşıldı",
    "Invalid file type. Only JSON files are allowed." : "Dosya türü geçersiz. Yalnızca JSON dosyaları kullanılabilir.",
    "Invalid JSON data" : "JSON verileri geçersiz",
    "Failed to import board" : "Pano içe aktarılamadı",
    "Cards due today" : "Bitiş tarihi bugün olan kartlar",
    "Cards due tomorrow" : "Bitiş tarihi yarın olan kartlar",
    "Upcoming cards" : "Yaklaşan kartlar",
    "Load more" : "Diğerlerini yükle",
    "Welcome to Nextcloud Deck!" : "Nextcloud Tahta uygulamasına hoş geldiniz!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "\"%s\" kartı \"%s\" zamanında %s tarafından size atandı.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user}, {deck-board} panosundaki {deck-card} kartını size atadı.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "\"%1$s\" kartının \"%2$s\" zamanında bitiş tarihi geldi.",
    "The card {deck-card} on {deck-board} has reached its due date." : "{deck-board} panosundaki {deck-card} kartının bitiş tarihi geldi.",
    "%s has mentioned you in a comment on \"%s\"." : "%s kullanıcısı, \"%s\" üzerindeki bir yorumda sizi andı",
    "{user} has mentioned you in a comment on {deck-card}." : "{user}, {deck-card} kartına yaptığı yorumda sizi andı.",
    "The board \"%s\" has been shared with you by %s." : "\"%s\" panosu %s tarafından sizinle paylaşıldı.",
    "{user} has shared {deck-board} with you." : "{user}, {deck-card} kartını sizinle paylaştı.",
    "The remote board %1$s has been shared with you by %2$s" : "%1$s uzak panosu %2$s tarafından sizinle paylaşıldı",
    "Deck board" : "Tahta panosu",
    "Owned by %1$s" : "Sahibi: %1$s",
    "Deck boards, cards and comments" : "Tahta panoları, kartlar ve yorumlar",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "%1$s tarafından, %2$s/%3$s içinde, sahibi %4$s olan",
    "Create a new deck card" : "Yeni tahta kartı oluştur",
    "Card comments" : "Kart yorumları",
    "%s on %s" : "%s, %s ile ilgili",
    "Deck boards and cards" : "Tahta panoları ve kartlar",
    "No data was provided to create an attachment." : "Bir ek dosya oluşturmak için herhangi bir veri belirtilmedi.",
    "Finished" : "Tamamlandı",
    "To review" : "İncelenecek",
    "Action needed" : "İşlem yapılması gerekiyor",
    "Later" : "Sonra",
    "copy" : "kopyala",
    "Read more inside" : "İçeriden ayrıntılı bilgi alın",
    "Custom lists - click to rename!" : "Özel listeler. Yeniden adlandırmak için tıklayın!",
    "To Do" : "Yapılacak iş",
    "In Progress" : "Sürüyor",
    "Done" : "Bitenler",
    "1. Open to learn more about boards and cards" : "1. Panolar ve kartlar ile ilgili bilgi almak için açın",
    "2. Drag cards left and right, up and down" : "2. Kartları sağa, sola, aşağı ve yukarı sürükleyin",
    "3. Apply rich formatting and link content" : "3. Zengin biçim uygulayın ve içeriği bağlayın",
    "4. Share, comment and collaborate!" : "4. Paylaşın, yorum ve iş birliği yapın! ",
    "Create your first card!" : "İlk kartınızı oluşturun!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Bu yorum %s karakterden uzun.\nKarta %s adıyla ek dosya olarak eklendi.\nŞu adresten erişilebilir: %s.",
    "Attachments" : "Ek dosyalar",
    "File" : "Dosya",
    "date" : "tarih",
    "Card not found" : "Kart bulunamadı",
    "Path is already shared with this card" : "Bu yol bu kart ile zaten paylaşılmış",
    "Invalid date, date format must be YYYY-MM-DD" : "Tarih geçersiz. Tarih biçimi YYYY-AA-GG olmalıdır",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Yığınlar, kartlar, etiketler, ödevler ve yorumlar gibi size ait tüm panolar",
    "Personal planning and team project organization" : "Kişisel planlama ve takım projesi yönetimi",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Tahta, Nextcloud ile bütünleştirilmiş takımlar için Kanban tarzında bir kişisel planlama ve proje yönetimi uygulamasıdır.\n\n\n- 📥 Görevleri kartlara ekleyerek sıralayabilirsiniz\n- 📄 Markdown ile biçimlendirilmiş notlar ekleyebilirsiniz\n- 🔖 Daha iyi düzenleme için etiketler atayabilirsiniz\n- 👥 Takımınız, arkadaşlarınız ya da aileniz ile paylaşabilirsiniz\n- 📎 Dosyalar ekleyebilir ve Markdown açıklamasına gömebilirsiniz\n- 💬 Yorumları kullanarak konular üzerinde takımınız ile yazışabilirsiniz\n- ⚡ İşlem akışı ile değişiklikleri görebilirsiniz\n- 🚀 Projelerinizi düzenli tutarak izleyebilirsiniz",
    "Add board" : "Pano ekle",
    "Card details" : "Kart ayrıntıları",
    "Select the board to link to a project" : "Bir proje ile ilişkilendirilecek pano bağlantısını seçin",
    "Search by board title" : "Pano başlığına göre ara",
    "Select board" : "Pano seçin",
    "Move/copy card" : "Kartı taşı/kopyala",
    "Select a board" : "Bir pano seçin",
    "No lists available" : "Kullanılabilecek bir liste yok",
    "Select a list" : "Bir liste seçin",
    "Move card" : "Kartı taşı",
    "Copy card" : "Kartı kopyala",
    "Select the card to link to a project" : "Bir proje ile ilişkilendirilecek kart bağlantısını seçin",
    "Link to card" : "Kart bağlantısı",
    "Select a card" : "Bir kart seçin",
    "Cancel" : "İptal",
    "This board is read only" : "Bu pano salt okunur",
    "Drop your files to upload" : "Yüklenecek dosyaları sürükleyip bırakın",
    "File already exists" : "Dosya zaten var",
    "A file with the name {filename} already exists." : "{filename} adlı bir dosya zaten var.",
    "Do you want to overwrite it?" : "Üzerine yazılmasını ister misiniz?",
    "Overwrite file" : "Üzerine yaz",
    "Keep existing file" : "Var olan dosya korunsun",
    "Add card" : "Kart ekle",
    "Archived cards" : "Arşivlenmiş kartlar",
    "Add list" : "Liste ekle",
    "List name" : "Liste adı",
    "Clear search" : "Aramayı temizle",
    "Active filters" : "Etkin süzgeçler",
    "Apply filter" : "Süzgeci uygula",
    "Filter by tag" : "Etikete göre süz",
    "Filter by assigned user" : "Atanmış kullanıcıya göre süz",
    "Unassigned" : "Atanmamış",
    "Filter by status" : "Duruma göre süz",
    "Open and completed" : "Aç ve tamamla",
    "Open" : "Aç",
    "Completed" : "Tamamlanmış",
    "Filter by due date" : "Bitiş tarihine göre süz",
    "Overdue" : "Zamanı geçmiş",
    "Next 24 hours" : "Sonraki 24 saat",
    "Next 7 days" : "Sonraki 7 gün",
    "Next 30 days" : "Sonraki 30 gün",
    "No due date" : "Bitiş tarihi yok",
    "Clear filter" : "Süzgeci temizle",
    "View Modes" : "Görünüm kipleri",
    "Toggle View Modes" : "Görünüm kiplerini değiştir",
    "Kanban view" : "Kanban görünümü",
    "Gantt view" : "Gantt görünümü",
    "Hide archived cards" : "Arşivlenmiş kartları gizle",
    "Show archived cards" : "Arşivlenmiş kartları görüntüle",
    "Toggle compact mode" : "Sıkışık kipi aç/kapat",
    "Hide card cover images" : "Kart kapak görsellerini gizle",
    "Show card cover images" : "Kart kapak görsellerini görüntüle",
    "Open details" : "Ayrıntıları aç",
    "Details" : "Ayrıntılar",
    "Deck settings" : "Tahta ayarları",
    "General" : "Genel",
    "Use bigger card view" : "Daha büyük kart görünümü kullanılsın",
    "Appearance" : "Görünüm",
    "Show card ID badge" : "Kart kimliği etiketi görüntülensin",
    "Show boards in calendar/tasks" : "Takvimler ve görevlerde panolar görüntülensin",
    "Admin settings" : "Yönetici ayarları",
    "Limit board creation to some groups" : "Pano oluşturma seçilmiş gruplarla kısıtlansın",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Bu gruplar dışında kalan kullanıcılar kendi panolarını oluşturamaz ancak kendileriyle paylaşılan panolar üzerinde çalışmayı sürdürebilir.",
    "Enable federation" : "Birlik kullanılsın",
    "Board actions" : "Pano işlemleri",
    "Scroll sideways" : "Yana doğru kaydırma",
    "Navigate between cards" : "Kartlar arasında gezinme",
    "Close card details" : "Kart ayrıntılarını kapat",
    "Search" : "Ara",
    "Show card filters" : "Kart süzgeçlerini görüntüle",
    "Clear card filters" : "Kart süzgeçlerini temizle",
    "Show those shortcuts" : "Bu kısayolları görüntüler",
    "Card actions" : "Kart işlemleri",
    "Open card details" : "Kart ayrıntılarını aç",
    "Edit the card title" : "Kart başlığını düzenle",
    "Assign yourself to the current card" : "Geçerli karta kendini ata",
    "Archive/unarchive the current card" : "Geçerli kartı arşivle ya da arşivden çıkar",
    "Mark card as completed/not completed" : "Kartı tamamlanmış ya da tamamlanmamış olarak işaretle",
    "Open card menu" : "Kart menüsünü aç",
    "Currently present people" : "Şu anda var olan kişiler",
    "Supported prefixes: {prefixes}. Wrap phrases in double quotes." : "Desteklenen ön ekler: {prefixes}. İfadeleri çift tırnak arasında yazın.",
    "Search cards" : "Kart ara",
    "Loading board" : "Pano yükleniyor",
    "Board not found" : "Pano bulunamadı",
    "Create a new list to add cards to this board" : "Bu panoya kartlar eklemek için yeni bir liste oluşturun",
    "Sharing" : "Paylaşım",
    "Tags" : "Etiketler",
    "Deleted items" : "Silinmiş ögeler",
    "Activity" : "İşlem",
    "Deleted lists" : "Silinmiş listeler",
    "Undo" : "Geri al",
    "Deleted cards" : "Silinmiş kartlar",
    "Hours" : "Saat",
    "Days" : "Gün",
    "Weeks" : "Hafta",
    "Months" : "Ay",
    "No cards yet" : "Henüz bir kart yok",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Gantt çizelgesi olarak görmek için kartlarınıza bir başlangıç tarihi ve bitiş tarihi ekleyin",
    "Cards without dates ({count})" : "Tarihleri olmayan kartlar ({count})",
    "Failed to create share with {displayName}" : "{displayName} ile paylaşım oluşturulamadı",
    "Are you sure you want to transfer the board {title} to {user}?" : "{title} panosunu {user} kullanıcısına aktarmak istediğinize emin misiniz?",
    "Transfer the board." : "Panoyu aktar.",
    "Transfer" : "Aktar",
    "The board has been transferred to {user}" : "Pano {user} kullanıcısına aktarıldı",
    "Failed to transfer the board to {user}" : "Pano {user} kullanıcısına aktarılamadı",
    "Board owner" : "Pano sahibi",
    "(Group)" : "(Grup)",
    "(Team)" : "(Takım)",
    "(remote)" : "(uzak)",
    "Can edit" : "Düzenleyebilir",
    "Can share" : "Paylaşabilir",
    "Can manage" : "Yönetebilir",
    "Owner" : "Sahibi",
    "Delete" : "Sil",
    "List deleted" : "Liste silindi",
    "Edit list title" : "Liste başlığını düzenle",
    "Archive all cards" : "Tüm kartları arşivle",
    "Unarchive all cards" : "Tüm kartları arşivden çıkar",
    "Do not set cards as \"done\"" : "Kartları \"Tamamlanmış\" olarak işaretleme",
    "Set cards as \"done\"" : "Kartları \"Tamamlanmış\" olarak işaretle",
    "Delete list" : "Listeyi sil",
    "Archive all cards in this list" : "Bu listedeki tüm kartları arşivle",
    "Unarchive all cards in this list" : "Bu listedeki tüm kartları arşivden çıkar",
    "Add a new card" : "Yeni kart ekle",
    "Card name" : "Kart adı",
    "title and color value must be provided" : "başlık ve renk değeri belirtilmelidir",
    "Edit" : "Düzenle",
    "Add a new tag" : "Yeni etiket ekle",
    "Search boards" : "Pano ara",
    "Board name" : "Pano adı",
    "Members" : "Üyeler",
    "Assign to users/groups/team" : "Kullanıcı/grup/takım ata",
    "Assign a user to this card…" : "Bu karta bir kullanıcı ata…",
    "Select a user to assign to this card…" : "Bu karta atanacak bir kullanıcı seçin…",
    "File to share" : "Paylaşılacak dosya",
    "Invalid path selected" : "Seçilen yol geçersiz",
    "Upload new files" : "Yeni dosyaları yükle",
    "Share from Files" : "Dosyalardan paylaş",
    "Pending share" : "Bekleyen paylaşım",
    "Add this attachment" : "Bu dosyayı ekle",
    "Show in Files" : "Dosyalarda görüntüle",
    "Download" : "İndir",
    "Remove attachment" : "Ek dosyayı sil",
    "Delete Attachment" : "Ek dosyasını sil",
    "Restore Attachment" : "Ek dosyasını geri yükle",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Değiştirilme: {modifiedDate} ⸱ Oluşturulma: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Değiştirilme: {modifiedDate}\nOluşturulma: {createdDate}\nOluşturan: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Değiştirilme: {modifiedDate}\nOluşturulma: {createdDate}",
    "The title cannot be empty." : "Başlık boş olamaz.",
    "Cannot close unsaved card!" : "Kaydedilmemiş bir kart kapatılamaz!",
    "Open in sidebar view" : "Kenar çubuğu görünümünde aç",
    "Open in bigger view" : "Daha büyük görünümde aç",
    "Comments" : "Yorumlar",
    "Failed to load comments" : "Yorumlar yüklenemedi",
    "No comments yet. Begin the discussion!" : "Henüz bir yorum yapılmamış. Tartışmayı başlatın!",
    "The comment cannot be empty." : "Yorum boş olamaz.",
    "The comment cannot be longer than 1000 characters." : "Yorum 1000 karakterden uzun olamaz.",
    "Submit" : "Gönder",
    "Created:" : "Oluşturulma:",
    "In reply to" : "Şuna yanıtla",
    "Cancel reply" : "Yanıtı iptal et",
    "Reply" : "Yanıtla",
    "Update" : "Güncelle",
    "Assign dependent cards" : "Bağlı kartları ata",
    "Mark as done" : "Tamamlanmış olarak işaretle",
    "Mark as not done" : "Tamamlanmamış olarak işaretle",
    "Remove dependency" : "Bağlılığı kaldır",
    "Add dependent card" : "Bağlı kart ekle",
    "Assign a dependent card…" : "Bağlı bir kart ata…",
    "Select a dependent card…" : "Bir bağlı kart seçin…",
    "Write a description …" : "Bir açıklama yazın …",
    "The description has been changed by another user." : "Açıklama başka bir kullanıcı tarafından değiştirildi.",
    "Could not save description" : "Açıklama kaydedilemedi",
    "Description" : "Açıklama",
    "(Unsaved)" : "(Kaydedilmemiş)",
    "(Saving…)" : "(Kaydediliyor…)",
    "Formatting help" : "Biçimlendirme yardımı",
    "Edit description" : "Açıklamayı düzenle",
    "View description" : "Açıklamayı görüntüle",
    "Add Attachment" : "Dosya ekle",
    "Choose attachment" : "Ek dosyayı seçin",
    "Select Date" : "Tarih Seçin",
    "Later today – {timeLocale}" : "Bugün daha sonra – {timeLocale}",
    "Set due date for later today" : "Bitiş tarihini bugün daha sonra olarak ayarla",
    "Tomorrow – {timeLocale}" : "Yarın – {timeLocale}",
    "Set due date for tomorrow" : "Bitiş tarihini yarın olarak ayarla",
    "This weekend – {timeLocale}" : "Bu hafta sonu – {timeLocale}",
    "Set due date for this weekend" : "Bitiş tarihini bu hafta sonu olarak ayarla",
    "Next week – {timeLocale}" : "Sonraki hafta – {timeLocale}",
    "Set due date for next week" : "Bitiş tarihini sonraki hafta olarak ayarla",
    "Assign a due date to this card…" : "Bu karta bir bitiş tarihi ata…",
    "Set a due date" : "Bitiş tarihi ayarla",
    "Add due date" : "Bitiş tarihi ekle",
    "Choose a date" : "Bir tarih seçin",
    "Remove due date" : "Bitiş tarihini kaldır",
    "Due at:" : "Bitiş tarihi:",
    "Not done" : "Tamamlanmamış",
    "Unarchive card" : "Kartı arşivden çıkar",
    "Archive card" : "Kartı arşivle",
    "Assign a start date to this card…" : "Bu kartın başlangıç tarihini ayarlayın…",
    "Set a start date" : "Bir başlangıç tarihi ayarlayın",
    "Add start date" : "Başlangıç tarihi ekle",
    "Remove start date" : "Başlangıç tarihini kaldır",
    "Assign a tag to this card…" : "Bu karta bir etiket ata…",
    "Select or create a tag…" : "Bir etiket seçin ya da oluşturun…",
    "Create a new tag:" : "Yeni etiket ekle:",
    "(group)" : "(grup)",
    "{count} comments, {unread} unread" : "{count} yorum, {unread} okunmamış",
    "Todo items" : "Yapılacak iş ögeleri",
    "Edit card title" : "Kart başlığını düzenle",
    "Open link" : "Bağlantıyı aç",
    "Card deleted" : "Kart silindi",
    "Change card color" : "Kart rengini değiştir",
    "Edit title" : "Başlığı düzenle",
    "Assign to me" : "Bana ata",
    "Unassign myself" : "Kendi atamamı kaldır",
    "Delete card" : "Kartı sil",
    "All boards" : "Tüm panolar",
    "Archived boards" : "Arşivlenmiş panolar",
    "Shared with you" : "Sizinle paylaşıldı",
    "Cancel edit" : "Düzenlemeyi iptal et",
    "Save board" : "Panoyu kaydet",
    "Board {0} deleted" : "{0} panosu silindi",
    "All cards" : "Tüm kartlar",
    "Only assigned cards" : "Yalnızca atanmış kartlar",
    "No reminder" : "Anımsatıcı yok",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "{title} panosunu silmek istediğinize emin misiniz? Bu işlem, bu panodaki ve arşivlenmiş kartlarındaki tüm verileri silecek.",
    "Delete the board?" : "Pano silinsin mi?",
    "Are you sure you want to leave the board {title}?" : "{title} panosundan ayrılmak istediğinize emin misiniz?",
    "Leave the board?" : "Panodan ayrılınsın mı?",
    "Leave" : "Ayrıl",
    "Failed to leave the board" : "Panodan ayrılınamadı",
    "Exporting board..." : "Pano dışa aktarılıyor...",
    "An error occurred" : "Bir sorun çıktı",
    "Board details" : "Pano ayrıntıları",
    "Edit board" : "Panoyu sil",
    "Clone board" : "Panoyu kopyala",
    "Unarchive board" : "Panoyu arşivden çıkar",
    "Archive board" : "Panoyu arşivle",
    "Export board" : "Panoyu dışa aktar",
    "Turn on due date reminders" : "Tarih anımsatıcılarını aç",
    "Turn off due date reminders" : "Tarih anımsatıcılarını kapat",
    "Remove as default board" : "Varsayılan pano olmaktan kaldır",
    "Set as default board" : "Varsayılan pano olarak ayarla",
    "Due date reminders" : "Bitiş tarihi anımsatıcıları",
    "Assigned cards" : "Atanmış kartlar",
    "No notifications" : "Bildirim yok",
    "Delete board" : "Panoyu sil",
    "Leave board" : "Panodan ayrıl",
    "Importing board..." : "Pano içe aktarılıyor...",
    "Board imported successfully" : "Pano içe aktarıldı",
    "Import board" : "Panoyu içe aktar",
    "Clone {boardTitle}" : "{boardTitle} çoğaltılsın",
    "Clone cards" : "Kartları kopyala",
    "Clone assignments" : "Atamalar çoğaltılsın",
    "Clone labels" : "Etiketler çoğaltılsın",
    "Clone due dates" : "Bitiş tarihleri çoğaltılsın",
    "Advanced options" : "Gelişmiş ayarlar",
    "Move all cards to the first list" : "Tüm kartları ilk listeye taşı",
    "Restore archived cards" : "Arşivlenmiş kartları geri yükle",
    "Clone" : "Kopyala",
    "Export {boardTitle}" : "{boardTitle} panosunu dışa aktar",
    "Export as JSON" : "JSON olarak dışa aktar",
    "Export as CSV" : "CSV olarak dışa aktar",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Not: Yeniden Tahta uygulaması içine aktarmak için yalnızca JSON biçimi desteklenir.",
    "Export" : "Dışa aktar",
    "Loading filtered view" : "Süzülmüş görünüm yükleniyor",
    "Search for {searchQuery} in other boards" : "Diğer panolarda {searchQuery} ara",
    "Search for {searchQuery} in all boards" : "Tüm panolarda {searchQuery} ara",
    "No results found" : "Herhangi bir sonuç bulunamadı",
    "Deck board {name}\n* Last modified on {lastMod}" : "{name} tahta panosu\n* Son değişiklik: {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Oluşturulma: {created}\n* Son değiştirilme: {lastMod}\n* {nbAttachments} ek dosya\n* {nbComments} yorum",
    "{nbCards} cards" : "{nbCards} kart",
    "Due on {date}" : "{date} tarihinde bitecek",
    "{stack} in {board}" : "{stack} {board} panosunda",
    "Click to expand description" : "Açıklamayı genişletmek için tıklayın",
    "Click to expand comment" : "Yorumu genişletmek için tıklayın",
    "Create card" : "Kart oluştur",
    "Create a new card" : "Yeni bir kart oluştur",
    "Card title" : "Kart başlığı",
    "Creating the new card …" : "Yeni kart ekleniyor…",
    "Card \"{card}\" was added to \"{board}\"" : "\"{card}\" kartı \"{board}\" panosuna eklendi",
    "Open card" : "Kartı aç",
    "Close" : "Kapat",
    "No upcoming cards" : "Yaklaşan bir kart yok",
    "upcoming cards today" : "bugün tamamlanması gereken kartlar",
    "upcoming cards tomorrow" : "yarın tamamlanması gereken kartlar",
    "upcoming cards" : "yaklaşan kartlar",
    "New card" : "Yeni kart",
    "Link to a board" : "Bir pano bağlantısı",
    "Link to a card" : "Bir kart bağlantısı",
    "Create a card" : "Bir kart oluştur",
    "Message from {author} in {conversationName}" : "{conversationName} görüşmesinde {author} iletisi",
    "Something went wrong" : "Bir sorun çıktı",
    "Failed to upload {name}" : "{name} yüklenemedi",
    "Maximum file size of {size} exceeded" : "{size} olan en büyük dosya boyutu sınırı aşıldı",
    "Assigned users" : "Atanmış kullanıcılar",
    "Due date" : "Bitiş tarihi",
    "Created" : "Oluşturulma",
    "Modified" : "Değiştirilme",
    "Error creating the share" : "Paylaşım oluşturulurken sorun çıktı",
    "Share with a Deck card" : "Bir tahta kartı ile paylaş",
    "Share {file} with a Deck card" : "{file} dosyasını bir tahta kartı ile paylaş",
    "Share" : "Paylaş",
    "The card \"%s\" on \"%s\" has reached its due date." : "\"%s\" kartı \"%s\" zamanında son kullanma tarihine  ulaştı.",
    "Share board with a user, group or team …" : "Panoyu kullanıcı, grup ya da takımla paylaş…",
    "Searching for users, groups and teams …" : "Kullanıcılar, gruplar ve takımlar aranıyor…",
    "No participants found" : "Herhangi bir katılımcı bulunamadı",
    "Save" : "Kaydet",
    "seconds ago" : "saniye önce",
    "Today" : "Bugün",
    "Tomorrow" : "Yarın",
    "No due" : "Bitiş yok"
},
"nplurals=2; plural=(n > 1);");
