OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "سىز يېڭى تاختا {board} قۇردىڭىز",
    "{user} has created a new board {board}" : "{user} يېڭى تاختا {board} قۇردى",
    "You have deleted the board {board}" : "تاختاي {board} ئۆچۈردىڭىز",
    "{user} has deleted the board {board}" : "{user} تاختاي {board} ئۆچۈردى",
    "You have restored the board {board}" : "تاختاي {board} ئەسلىگە كەلتۈردىڭىز",
    "{user} has restored the board {board}" : "{user} تاختاي {board} ئەسلىگە كەلتۈردى",
    "You have shared the board {board} with {acl}" : "تاختاي {board} {acl} بىلەن ئورتاقلاشتىڭىز",
    "{user} has shared the board {board} with {acl}" : "{user} تاختاي {board} {acl} بىلەن ئورتاقلاشتى",
    "You have removed {acl} from the board {board}" : "تاختاي {board} دىن {acl} نى چىقىرىۋەتتىڭىز.",
    "{user} has removed {acl} from the board {board}" : "{user} تاختاي {board}دىن {acl} نى چىقىرىۋەتتى",
    "You have renamed the board {before} to {board}" : "سىز تاختاي {before}نىڭ نامىنى {board}غا ئۆزگەرتتىڭىز.",
    "{user} has renamed the board {before} to {board}" : "{user} تاختاي {before}نىڭ نامىنى {board} غا ئۆزگەرتتى",
    "You have archived the board {board}" : "تاختاي {board} ئارخىپلاشتۇردىڭىز",
    "{user} has archived the board {before}" : "{user} تاختاي {before}نى ئارخىپلاشتۇردى",
    "You have unarchived the board {board}" : "سىز تاختاي {board} رەتسىز",
    "{user} has unarchived the board {before}" : "{user} تاختاي {before} نى ئارخىپلىمىدى",
    "You have created a new list {stack} on board {board}" : "سىز تاختا {board} يېڭى تىزىملىك {stack} قۇردىڭىز",
    "{user} has created a new list {stack} on board {board}" : "{user} board تاختا {board} يېڭى تىزىملىك {stack} قۇردى",
    "You have renamed list {before} to {stack} on board {board}" : "سىز تىزىملىكنىڭ نامىنى {before} دىن {stack} {board} ئۆزگەرتىڭ.",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user}تاختاي {board}دىكى تىزىملىك {before}نىڭ نامىنى {stack} غا ئۆزگەرتتى.",
    "You have deleted list {stack} on board {board}" : "سىز {board} تىزىملىك {stack} نى ئۆچۈردىڭىز",
    "{user} has deleted list {stack} on board {board}" : "{user} board تاختا {board} تىزىملىك {stack} نى ئۆچۈردى",
    "You have created card {card} in list {stack} on board {board}" : "سىز تىزىملىك {card} {stack} list كارتا {board} قۇردىڭىز",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} board تاختا {card} {stack} list كارتا {board} قۇردى",
    "You have deleted card {card} in list {stack} on board {board}" : "كارتا {card} list تىزىملىكتىكى {stack} board تاختا {board} نى ئۆچۈردىڭىز",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} تاختاي {board}دىكى تىزىملىك {stack} دىكى كارتا {card} نى ئۆچۈردى",
    "You have renamed the card {before} to {card}" : "سىز بۇ كارتىنىڭ نامىنى {before} بۇرۇن {card} ئۆزگەرتتىڭىز.",
    "{user} has renamed the card {before} to {card}" : "{user} كارتىنىڭ نامىنى {before} {card} ئۆزگەرتتى",
    "You have added a description to card {card} in list {stack} on board {board}" : "سىز كارتا {card} list تاختا {board} {stack} غا چۈشەندۈرۈش قوشتىڭىز.",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} board تاختا {card} {stack} list كارتا {board} غا بىر چۈشەندۈرۈش قوشتى",
    "You have updated the description of card {card} in list {stack} on board {board}" : "تاختاي {card} {stack} list كارتا {board} نىڭ چۈشەندۈرۈشىنى يېڭىلىدىڭىز",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} تاختا {card} {stack} كارتا {board} نىڭ چۈشەندۈرۈشىنى يېڭىلىدى",
    "You have archived card {card} in list {stack} on board {board}" : "سىز كارتا {card} list تىزىملىكتىكى {stack} تاختا {board} غا ئارخىپلاشتۇردىڭىز",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} تاختا {card} {stack} تىزىملىكىدە كارتا {board} arch ئارخىپلاشتۇرۇلدى",
    "You have unarchived card {card} in list {stack} on board {board}" : "سىز تاختا {board} دىكى تىزىملىك {stack} دىكى كارتا {card} نى ئارخىپلاشتۇرمىدىڭىز",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} تاختا {card} {stack} تىزىملىكتە ساقلانمىغان كارتا {board} بار",
    "You have marked the card {card} as done in list {stack} on board {board}" : "سىز كارتا {card} تاختاي {board} تىزىملىك {stack} in دا كۆرسىتىلگەندەك بەلگە قويدىڭىز",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} board تاختا {card} {stack} list دا كۆرسىتىلگەندەك كارتا {board} دەپ بەلگە قويدى",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "سىز كارتا {card} تاختا {board} {stack} تىزىملىكىدە ئەمەلدىن قالدۇرۇلغان دەپ بەلگە قويدىڭىز",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} تاختاي {board} دىكى تىزىملىك {stack} دىكى كارتا {card} نى پۈتمىگەن بەلگىسىنى قويدى",
    "You have removed the due date of card {card}" : "كارتا {card} قەرەلى توشقان ۋاقىتنى ئۆچۈردىڭىز",
    "{user} has removed the due date of card {card}" : "{user} كارتا {card} قەرەلى توشتى",
    "You have set the due date of card {card} to {after}" : "كارتا {card} نىڭ قەرەلى توشقاندىن كېيىن {after} كېيىن بېكىتىلدى",
    "{user} has set the due date of card {card} to {after}" : "{user} كارتا {card} نىڭ بەلگىلەنگەن ۋاقتىنى {after} قىلىپ بېكىتتى",
    "You have updated the due date of card {card} to {after}" : "كارتا {card} نىڭ قەرەلى توشقاندىن كېيىن {after} يېڭىلاندىڭىز",
    "{user} has updated the due date of card {card} to {after}" : "{user} كارتا {card} نىڭ قەرەلى توشقاندىن كېيىن {after} يېڭىلاندى",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "سىز تاختاي {board} دىكى تىزىملىك {stack} دىكى كارتا {card} غا خەتكۈچ {label} نى قوشتىڭىز",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} تاختا {label} {card} غا {stack} بەلگىسىنى {board} غا قوشتى.",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "سىز تاختاي {board} دىكى تىزىملىك {stack} دىكى كارتا {card} دىن خەتكۈچ {label} نى ئۆچۈردىڭىز",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} تاختا {label} {card} تىزىملىكىدىكى {stack} بەلگىسىنى {board} دىن ئۆچۈردى.",
    "You have assigned {assigneduser} to card {card} on board {board}" : "سىز تاختا {board} دىكى كارتا {card} نى {assigneduser} غا تاپشۇردىڭىز",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} تاختا {board} دىكى كارتا {card} نى {assigneduser} غا تاپشۇردى",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "سىز كارتا {assigneduser} board تاختا {card} تەقسىملەنمىگەن {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} board تاختا {assigneduser} كارتا {card} from دىن ئىمزا قويمىغان {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "سىز كارتا {card} list تىزىملىكتىن {stackBefore} دىن {stack} غا يۆتكىدىڭىز",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} كارتا {card} list تىزىملىكتىن {stackBefore} دىن {stack} غا يۆتكىدى",
    "You have added the attachment {attachment} to card {card}" : "سىز كارتا {card} غا قوشۇمچە ھۆججەت {attachment} نى قوشتىڭىز",
    "{user} has added the attachment {attachment} to card {card}" : "{user} كارتا {card} غا قوشۇمچە ھۆججەت {attachment}نى قوشتى",
    "You have updated the attachment {attachment} on card {card}" : "كارتا {attachment} on قوشۇمچە ھۆججەت {card} ھۆججەتنى يېڭىلىدىڭىز",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} كارتا {attachment} قوشۇمچە ھۆججەت {card} ھۆججەتنى يېڭىلىدى",
    "You have deleted the attachment {attachment} from card {card}" : "كارتا {attachment} قوشۇمچە ھۆججەت {card} ھۆججەتنى ئۆچۈردىڭىز",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} كارتا {card}دىن قوشۇمچە ھۆججەت {attachment}نى ئۆچۈردى",
    "You have restored the attachment {attachment} to card {card}" : "سىز كارتا {card} دىكى قوشۇمچە ھۆججەت {attachment}نى ئەسلىگە كەلتۈردىڭىز",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} كارتا {attachment} قوشۇمچە {card} ھۆججەتنى ئەسلىگە كەلتۈردى.",
    "You have commented on card {card}" : "كارتا {card} on غا باھا بەردىڭىز",
    "{user} has commented on card {card}" : "{user} card كارتا {card} on غا باھا بەردى",
    "Deck" : "پالۋان",
    "Changes in the <strong>Deck app</strong>" : "<strong> پالۋان دېتالى </strong> دىكى ئۆزگىرىشلەر",
    "A <strong>board, list or card</strong> was changed" : "<strong> تاختا ، تىزىملىك ياكى كارتا </strong> ئۆزگەرتىلدى",
    "A <strong>comment</strong> was created on a card" : "كارتا ئۈستىدە <strong> ئىنكاس </strong> قۇرۇلدى",
    "A <strong>card description</strong> has been changed" : "<strong> كارتا چۈشەندۈرۈشى </strong> ئۆزگەرتىلدى",
    "The file was uploaded" : "ھۆججەت يۈكلەندى",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "يۈكلەنگەن ھۆججەت php.ini دىكى upload_max_filesize كۆرسەتمىسىدىن ئېشىپ كەتتى",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "يۈكلەنگەن ھۆججەت HTML شەكلىدە كۆرسىتىلگەن MAX_FILE_SIZE كۆرسەتمىسىدىن ئېشىپ كەتتى",
    "The file was only partially uploaded" : "بۇ ھۆججەت پەقەت قىسمەن يوللانغان",
    "No file was uploaded" : "ھېچقانداق ھۆججەت يۈكلەنمىدى",
    "Missing a temporary folder" : "ۋاقىتلىق قىسقۇچ كەم.",
    "Could not write file to disk" : "دىسكىغا ھۆججەت يازالمىدى",
    "A PHP extension stopped the file upload" : "PHP كېڭەيتىلمىسى ھۆججەت يوللاشنى توختاتتى",
    "No file uploaded or file size exceeds maximum of %s" : "يۈكلەنگەن ياكى ھۆججەتنىڭ چوڭلۇقى ئەڭ كۆپ بولغاندا %s دىن ئېشىپ كەتمەيدۇ",
    "Invalid file type. Only JSON files are allowed." : "ئىناۋەتسىز ھۆججەت تىپى. پەقەت JSON ھۆججىتىگىلا رۇخسەت.",
    "Invalid JSON data" : "ئىناۋەتسىز JSON سانلىق-مەلۇمات",
    "Failed to import board" : "تاختاينى ئەكىرىش مەغلۇب بولدى",
    "Cards due today" : "بۈگۈن كارتا بېجىرىلىدۇ",
    "Cards due tomorrow" : "ئەتە كارتا",
    "Upcoming cards" : "كېلىۋاتقان كارتا",
    "Load more" : "تېخىمۇ كۆپ يۈكلەڭ",
    "Welcome to Nextcloud Deck!" : "Nextcloud جەدۋىلىگە خۇش كەپسىز",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "«%s» دىكى «%s» كارتىسى سىزگە %s تەرىپىدىن تەقسىم قىلىنغان.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} سىزگە {deck-card} تاختىسىدىكى كارتا {deck-board} نى تەقسىم قىلدى.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "\"%2$s\" دىكى كارتا \"%1$s\" تۇشۇش ۋاقتىغا يەتتى",
    "The card {deck-card} on {deck-board} has reached its due date." : "{deck-card} تاختىسى on دىكى كارتا {deck-board} كارتىسى قەرەلى توشقان.",
    "%s has mentioned you in a comment on \"%s\"." : "%s سىزنى «%s» دىكى باھادا تىلغا ئالدى.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} {deck-card} كارتىسى on دىكى باھادا سىزنى تىلغا ئالدى.",
    "The board \"%s\" has been shared with you by %s." : "«%s» تاختىسىنى سىز بىلەن %s ئورتاقلاشتى.",
    "{user} has shared {deck-board} with you." : "{user} سىز بىلەن {deck-board} تاختىسىنى ئورتاقلاشتى.",
    "Deck board" : "پالۋان تاختىسى",
    "Owned by %1$s" : "ئىگىسى %1$s",
    "Deck boards, cards and comments" : "پالۋان تاختىسى ، كارتا ۋە باھا",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "%1$s دىن، %3$s/%2$s دا، ئىگىسى %4$s",
    "Create a new deck card" : "يېڭى پالۋان كارتىسى قۇرۇڭ",
    "Card comments" : "كارتا باھا",
    "%s on %s" : "%s دىكى %s",
    "Deck boards and cards" : "پەلەمپەي ۋە كارتا",
    "No data was provided to create an attachment." : "قوشۇمچە ھۆججەت قۇرۇش ئۈچۈن ھېچقانداق سانلىق مەلۇمات تەمىنلەنمىدى.",
    "Finished" : "تاماملاندى",
    "To review" : "قايتا قاراپ چىقىش",
    "Action needed" : "ھەرىكەت لازىم",
    "Later" : "كېيىنچە",
    "copy" : "كۆپەيتىلگەن",
    "Read more inside" : "ئىچىدە تېخىمۇ كۆپ ئوقۇڭ",
    "Custom lists - click to rename!" : "خاس تىزىملىك - چىكىپ ئىسمىنى ئۆرگەرت!",
    "To Do" : "قىلىش",
    "In Progress" : "ئىلگىرىلەۋاتىدۇ",
    "Done" : "پۈتتى",
    "1. Open to learn more about boards and cards" : "1. تاختا ۋە كارتىلار ھەققىدە كۆپرەك مەلۇمات ئېلىش ئۈچۈن ئېچىڭ",
    "2. Drag cards left and right, up and down" : "2. كارتىلارنى سول ۋە ئوڭ، يۇقىرى ۋە تۆۋەنگە سۆرەڭ",
    "3. Apply rich formatting and link content" : "3. مول فورماتلاش ۋە ئۇلىنىش مەزمۇنىنى قوللىنىڭ",
    "4. Share, comment and collaborate!" : "4. ھەمبەھىرلەڭ، پىكىر بېرىڭ ۋە ھەمكارلىشىڭ!",
    "Create your first card!" : "تۇنجى كارتىڭىزنى قۇرۇڭ!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "بۇ باھانىڭ %s دىن كۆپ ھەرپلىرى بار.\n%s ئىسمى بار كارتىغا قوشۇمچە قىلىپ قوشۇلدى.\nURL دا زىيارەت قىلغىلى بولىدۇ: %s.",
    "Attachments" : "قوشۇمچە ھۆججەتلەر",
    "File" : "ھۆججەت",
    "date" : "چېسلا",
    "Card not found" : "كارتا تېپىلمىدى",
    "Path is already shared with this card" : "يول بۇ كارتا بىلەن ئورتاقلاشتى",
    "Invalid date, date format must be YYYY-MM-DD" : "ئىناۋەتسىز چېسلا ، چېسلا فورماتى چوقۇم YYYY-MM-DD بولۇشى كېرەك",
    "Personal planning and team project organization" : "شەخسىي پىلان ۋە گۇرۇپپا تۈر تەشكىللەش",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "پالۋان Nextcloud بىلەن بىرلەشتۈرۈلگەن گۇرۇپپىلار ئۈچۈن شەخسىي پىلانلاش ۋە تۈر تەشكىللەشنى مەقسەت قىلغان كانبان ئۇسلۇبىدىكى تەشكىللەش قورالى.\n\n\n- tasks ۋەزىپىڭىزنى كارتىغا قوشۇڭ ۋە تەرتىپكە سېلىڭ\n- Mark Markdown غا قوشۇمچە خاتىرىلەرنى يېزىڭ\n- better تېخىمۇ ياخشى تەشكىللەش ئۈچۈن بەلگە بەلگىلەڭ\n- team گۇرۇپپىڭىز ، دوستلىرىڭىز ياكى ئائىلىڭىزدىكىلەر بىلەن ئورتاقلىشىڭ\n- files ھۆججەتلەرنى باغلاپ Markdown چۈشەندۈرۈشىڭىزگە قىستۇرۇڭ\n- comments باھا ئارقىلىق گۇرۇپپىڭىزدىكىلەر بىلەن مۇنازىرە قىلىڭ\n- activity پائالىيەت ئېقىمىدىكى ئۆزگىرىشلەرنى خاتىرىلەڭ\n- project تۈرىڭىزنى تەشكىللەڭ",
    "Add board" : "تاختا قوشۇڭ",
    "Card details" : "كارتا تەپسىلاتلىرى",
    "Select the board to link to a project" : "بىر تۈرگە ئۇلىنىش ئۈچۈن تاختىنى تاللاڭ",
    "Search by board title" : "تاختاينىڭ ئىسمى بىلەن ئىزدەڭ",
    "Select board" : "تاختاينى تاللاڭ",
    "Move/copy card" : "كارتا يۆتكەش/كۆچۈرۈش",
    "Select a board" : "تاختاينى تاللاڭ",
    "No lists available" : "تىزىملىك يوق",
    "Select a list" : "تىزىملىكنى تاللاڭ",
    "Move card" : "كارتا يۆتكەش",
    "Copy card" : "كارتىنى كۆچۈر",
    "Select the card to link to a project" : "بىر تۈرگە ئۇلىنىش ئۈچۈن كارتىنى تاللاڭ",
    "Link to card" : "كارتىغا ئۇلىنىش",
    "Select a card" : "كارتا تاللاڭ",
    "Cancel" : "ۋاز كەچ",
    "This board is read only" : "بۇ تاختاي پەقەت ئوقۇلىدۇ",
    "Drop your files to upload" : "يۈكلەش ئۈچۈن ھۆججەتلىرىڭىزنى تاشلاڭ",
    "File already exists" : "ھۆججەت مەۋجۇت",
    "A file with the name {filename} already exists." : "{filename} ئىسمدىكى ھۆججەت مەۋجۇت.",
    "Do you want to overwrite it?" : "ئۇنى قاپلىماقچىمۇ؟",
    "Overwrite file" : "ھۆججەتنى قاپلىۋېتىڭ",
    "Keep existing file" : "مەۋجۇت ھۆججەتنى ساقلاڭ",
    "Add card" : "كارتا قوشۇڭ",
    "Archived cards" : "ئارخىپ كارتىلىرى",
    "Add list" : "تىزىملىك قوشۇڭ",
    "List name" : "تىزىملىك ئىسمى",
    "Clear search" : "ئىزدەشنى تازىلاش",
    "Active filters" : "ئاكتىپ سۈزگۈچ",
    "Apply filter" : "سۈزگۈچ ئىشلىتىڭ",
    "Filter by tag" : "بەلگە بويىچە سۈزۈڭ",
    "Filter by assigned user" : "تەقسىم قىلىنغان ئىشلەتكۈچى تەرىپىدىن سۈزۈڭ",
    "Unassigned" : "ئىمزاسىز",
    "Filter by status" : "ھالەت بويىچە سۈزۈڭ",
    "Open and completed" : "ئېچىش ۋە تاماملاش",
    "Open" : "ئېچىڭ",
    "Completed" : "تاماملاندى",
    "Filter by due date" : "ۋاقتى بويىچە سۈزۈڭ",
    "Overdue" : "ۋاقتى ئۆتۈپ كەتتى",
    "Next 24 hours" : "كېيىنكى 24 سائەت",
    "Next 7 days" : "كەلگۈسى 7 كۈن",
    "Next 30 days" : "كەلگۈسى 30 كۈن",
    "No due date" : "ۋاقتى توشمىغان",
    "Clear filter" : "سۈزگۈچنى تازىلاش",
    "View Modes" : "مودېللارنى كۆرۈش",
    "Toggle View Modes" : "كۆرۈش ھالىتىنى ئۆزگەرتىش",
    "Hide archived cards" : "ئارخىپ كارتىلىرىنى يوشۇرۇش",
    "Show archived cards" : "ئارخىپ كارتىلىرىنى كۆرسەت",
    "Toggle compact mode" : "ئىخچام ھالەتنى ئالماشتۇرۇڭ",
    "Hide card cover images" : "كارتا قاپلاش رەسىملىرىنى يوشۇرۇش",
    "Show card cover images" : "كارتا قاپلاش رەسىملىرىنى كۆرسىتىش",
    "Open details" : "تەپسىلاتلارنى ئېچىڭ",
    "Details" : "تەپسىلاتى",
    "Deck settings" : "بېزەك تەڭشەكلىرى",
    "General" : "ئۇمۇمىي",
    "Use bigger card view" : "چوڭراق كارتا كۆرۈنۈشىنى ئىشلىتىڭ",
    "Appearance" : "كۆرۈنۈش",
    "Show card ID badge" : "كارتا كىملىكى بەلگىسىنى كۆرسەت",
    "Show boards in calendar/tasks" : "كالېندار / ۋەزىپىلەردە تاختاينى كۆرسەت",
    "Admin settings" : "باشقۇرغۇچى تەڭشىكى",
    "Limit board creation to some groups" : "تاختاي قۇرۇشنى بەزى گۇرۇپپىلارغا چەكلەڭ",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "بۇ گۇرۇپپىلارنىڭ سىرتىدىكى ئىشلەتكۈچىلەر ئۆزلىرىنىڭ تاختىسىنى قۇرالمايدۇ ، ئەمما ئۇلار بىلەن ئورتاقلاشقان تاختايلاردا ئىشلىيەلەيدۇ.",
    "Board actions" : "مۇدىرىيەت ھەرىكىتى",
    "Scroll sideways" : "يان تەرەپكە ئۆرۈڭ",
    "Navigate between cards" : "كارتا ئارىسىدا مېڭىڭ",
    "Close card details" : "كارتا تەپسىلاتلىرىنى تاقاش",
    "Search" : "ئىزدە",
    "Show card filters" : "كارتا سۈزگۈچنى كۆرسىتىش",
    "Clear card filters" : "كارتا سۈزگۈچنى تازىلاش",
    "Show those shortcuts" : "ئۇ قىسقايولنى كۆرسەت",
    "Card actions" : "كارتا ھەرىكىتى",
    "Open card details" : "كارتا تەپسىلاتلىرىنى ئېچىڭ",
    "Edit the card title" : "كارتا نامىنى تەھرىرلەڭ",
    "Assign yourself to the current card" : "ھازىرقى كارتىغا ئۆزىڭىزنى تەقسىم قىلىڭ",
    "Archive/unarchive the current card" : "ھازىرقى كارتىنى ئارخىپلاشتۇرۇڭ",
    "Mark card as completed/not completed" : "كارتا تاماملانغان / تاماملانمىغان دەپ بەلگە قويۇڭ",
    "Open card menu" : "كارتا تىزىملىكىنى ئېچىڭ",
    "Currently present people" : "ھازىر كىشىلەر",
    "Loading board" : "يۈك تاختىسى",
    "Board not found" : "مۇدىرىيەت تېپىلمىدى",
    "Create a new list to add cards to this board" : "بۇ تاختىغا كارتا قوشۇش ئۈچۈن يېڭى تىزىملىك قۇرۇڭ",
    "Sharing" : "ھەمبەھىر",
    "Tags" : "بەلگەلەر",
    "Deleted items" : "ئۆچۈرۈلگەن تۈرلەر",
    "Activity" : "پائالىيەت",
    "Deleted lists" : "ئۆچۈرۈلگەن تىزىملىك",
    "Undo" : "ئەمەلدىن قالدۇرۇش",
    "Deleted cards" : "ئۆچۈرۈلگەن كارتا",
    "Hours" : "سائەت",
    "Days" : "كۈنلەر",
    "Weeks" : "ھەپتە",
    "Months" : "ئايلار",
    "No cards yet" : "تېخى كارتا يوق",
    "Failed to create share with {displayName}" : "{displayName} with بىلەن ئورتاقلىشىش مەغلۇب بولدى",
    "Are you sure you want to transfer the board {title} to {user}?" : "تاختا {title} نى {user} غا يۆتكىمەكچىمۇ؟",
    "Transfer the board." : "تاختىنى يۆتكەڭ.",
    "Transfer" : "يۆتكەش",
    "The board has been transferred to {user}" : "بۇ تاختا {user} غا يۆتكەلدى",
    "Failed to transfer the board to {user}" : "تاختاينى {user} گە يۆتكەش مەغلۇب بولدى",
    "Board owner" : "مۇدىرىيەت ئىگىسى",
    "(Group)" : "(گۇرۇپپا)",
    "(Team)" : "(كوماندا)",
    "(remote)" : "(يىراق)",
    "Can edit" : "تەھرىرلىيەلەيدۇ",
    "Can share" : "ئورتاقلىشالايدۇ",
    "Can manage" : "باشقۇرالايدۇ",
    "Owner" : "ئىگىسى",
    "Delete" : "ئۆچۈر",
    "List deleted" : "تىزىملىك ئۆچۈرۈلدى",
    "Edit list title" : "تىزىملىك نامىنى تەھرىرلەش",
    "Archive all cards" : "بارلىق كارتىلارنى ئارخىپلاشتۇرۇڭ",
    "Unarchive all cards" : "بارلىق كارتىلارنى تەرتىپسىز",
    "Delete list" : "تىزىملىكنى ئۆچۈرۈڭ",
    "Archive all cards in this list" : "بۇ تىزىملىكتىكى بارلىق كارتىلارنى ئارخىپلاشتۇرۇڭ",
    "Unarchive all cards in this list" : "بۇ تىزىملىكتىكى بارلىق كارتىلارنى تەرتىپسىز",
    "Add a new card" : "يېڭى كارتا قوشۇڭ",
    "Card name" : "كارتا ئىسمى",
    "title and color value must be provided" : "ماۋزۇ ۋە رەڭ قىممىتى تەمىنلىنىشى كېرەك",
    "Edit" : "تەھرىر",
    "Add a new tag" : "يېڭى بەلگە قوشۇڭ",
    "Board name" : "مۇدىرىيەت ئىسمى",
    "Members" : "ئەزالار",
    "Assign to users/groups/team" : "ئىشلەتكۈچى / گۇرۇپپا / گۇرۇپپىغا تەقسىم قىلىڭ",
    "Assign a user to this card…" : "بۇ كارتىغا ئىشلەتكۈچى تەقسىم قىلىڭ…",
    "Select a user to assign to this card…" : "بۇ كارتىغا تەقسىملەيدىغان ئىشلەتكۈچىنى تاللاڭ…",
    "File to share" : "ھەمبەھىرلىنىدىغان ھۆججەت",
    "Invalid path selected" : "ئىناۋەتسىز يول تاللانغان",
    "Upload new files" : "يېڭى ھۆججەتلەرنى يۈكلەڭ",
    "Share from Files" : "ھۆججەتلەردىن ئورتاقلىشىش",
    "Pending share" : "ساقلىنىۋاتقان ئۈلۈش",
    "Add this attachment" : "بۇ قوشۇمچە ھۆججەتنى قوشۇڭ",
    "Show in Files" : "ھۆججەتلەردە كۆرسىتىڭ",
    "Download" : "چۈشۈر",
    "Remove attachment" : "قوشۇمچە ھۆججەتنى ئۆچۈرۈڭ",
    "Delete Attachment" : "قوشۇمچە ھۆججەتنى ئۆچۈرۈڭ",
    "Restore Attachment" : "قوشۇمچە ھۆججەتنى ئەسلىگە كەلتۈرۈش",
    "The title cannot be empty." : "ماۋزۇ قۇرۇق بولالمايدۇ.",
    "Cannot close unsaved card!" : "ساقلانمىغان كارتىنى ياپالمىدى",
    "Open in sidebar view" : "يان كۆزنەكتە ئېچىڭ",
    "Open in bigger view" : "چوڭراق كۆرۈنۈشتە ئېچىڭ",
    "Comments" : "باھا",
    "Failed to load comments" : "ئىنكاسلارنى يۈكلىيەلمىدى",
    "No comments yet. Begin the discussion!" : "ھازىرچە ھېچقانداق باھا يوق. مۇنازىرىنى باشلاڭ!",
    "The comment cannot be empty." : "باھا قۇرۇق بولالمايدۇ.",
    "The comment cannot be longer than 1000 characters." : "بۇ باھا 1000 ھەرپتىن ئېشىپ كەتسە بولمايدۇ.",
    "Submit" : "يوللاڭ",
    "Created:" : "قۇرۇلغان ۋاقتى:",
    "In reply to" : "دەپ جاۋاب بەردى",
    "Cancel reply" : "جاۋابنى ئەمەلدىن قالدۇرۇڭ",
    "Reply" : "جاۋاب قايتۇر",
    "Update" : "يېڭىلا",
    "Mark as done" : "تاماملانغاندەك بەلگە",
    "Mark as not done" : "ئىشلەنمىگەن دەپ بەلگە قويۇڭ",
    "Write a description …" : "چۈشەندۈرۈش يېزىڭ…",
    "Could not save description" : "چۈشەندۈرۈشنى ساقلىيالمىدى",
    "Description" : "چۈشەندۈرۈش",
    "(Unsaved)" : "(ساقلانمىدى)",
    "(Saving…)" : "(ساقلاۋاتىدۇ…)",
    "Formatting help" : "ياردەمنى فورماتلاش",
    "Edit description" : "چۈشەندۈرۈشنى تەھرىرلەڭ",
    "View description" : "چۈشەندۈرۈشنى كۆرۈڭ",
    "Add Attachment" : "قوشۇمچە ھۆججەت قوشۇڭ",
    "Choose attachment" : "قوشۇمچە ھۆججەتنى تاللاڭ",
    "Select Date" : "چېسلانى تاللاڭ",
    "Later today – {timeLocale}" : "كېيىن بۈگۈن - {timeLocale}",
    "Set due date for later today" : "بۈگۈنگە قەدەر بەلگىلەنگەن ۋاقىتنى بەلگىلەڭ",
    "Tomorrow – {timeLocale}" : "ئەتە - {timeLocale}",
    "Set due date for tomorrow" : "ئەتە بەلگىلەنگەن ۋاقىتنى بەلگىلەڭ",
    "This weekend – {timeLocale}" : "بۇ ھەپتە ئاخىرى - {timeLocale}",
    "Set due date for this weekend" : "بۇ ھەپتە ئاخىرىدىكى ۋاقىتنى بەلگىلەڭ",
    "Next week – {timeLocale}" : "كېلەر ھەپتە - {timeLocale}",
    "Set due date for next week" : "كېلەر ھەپتە قەرەلى توشۇڭ",
    "Assign a due date to this card…" : "بۇ كارتىغا مۇۋاپىق ۋاقىت بەلگىلەڭ…",
    "Set a due date" : "بەلگىلەنگەن ۋاقىتنى بەلگىلەڭ",
    "Add due date" : "مۇۋاپىق ۋاقىت قوشۇڭ",
    "Choose a date" : "چېسلانى تاللاڭ",
    "Remove due date" : "قەرەلى توشۇش",
    "Due at:" : "ۋاقتى:",
    "Not done" : "تاماملانمىدى",
    "Unarchive card" : "كارتىنى ئارخىپلىما",
    "Archive card" : "ئارخىپ كارتىسى",
    "Assign a tag to this card…" : "بۇ كارتىغا بەلگە بەلگىلەڭ…",
    "Select or create a tag…" : "بەلگە تاللاڭ ياكى قۇر…",
    "Create a new tag:" : "يېڭى خەتكۈچ قۇر:",
    "(group)" : "(گۇرۇپپا)",
    "{count} comments, {unread} unread" : "{count} باھا ، {unread} ئوقۇمىغان",
    "Todo items" : "تودو تۈرلىرى",
    "Edit card title" : "كارتا نامىنى تەھرىرلەش",
    "Open link" : "ئۇلانمىنى ئېچىڭ",
    "Card deleted" : "كارتا ئۆچۈرۈلدى",
    "Edit title" : "ماۋزۇنى تەھرىرلەش",
    "Assign to me" : "ماڭا تاپشۇرۇڭ",
    "Unassign myself" : "ئۆزۈمنى بەلگىلىمەڭ",
    "Delete card" : "كارتىنى ئۆچۈرۈڭ",
    "All boards" : "بارلىق تاختايلار",
    "Archived boards" : "ئارخىپ تاختىسى",
    "Shared with you" : "سىز بىلەن ھەمبەھىرلىدى",
    "Cancel edit" : "تەھرىرلەشنى ئەمەلدىن قالدۇرۇڭ",
    "Save board" : "تاختاينى ساقلاڭ",
    "Board {0} deleted" : "تاختا {0} ئۆچۈرۈلدى",
    "All cards" : "بارلىق كارتىلار",
    "Only assigned cards" : "پەقەت تەقسىم قىلىنغان كارتا",
    "No reminder" : "ئەسكەرتىش يوق",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "تاختا {title} نى ئۆچۈرمەكچىمۇ؟ بۇ ئارخىپ كارتىسىنى ئۆز ئىچىگە ئالغان بۇ تاختىنىڭ بارلىق سانلىق مەلۇماتلىرىنى ئۆچۈرۈۋېتىدۇ.",
    "Delete the board?" : "تاختىنى ئۆچۈرەمسىز؟",
    "Are you sure you want to leave the board {title}?" : "تاختاي {title} دىن راستىنلا ئايرىلماقچىمۇ؟",
    "Leave the board?" : "تاختايدىن ئايرىلامسىز؟",
    "Leave" : "كەت",
    "Failed to leave the board" : "تاختايدىن ئايرىلىش مەغلۇپ بولدى",
    "Exporting board..." : "تاختىنى چىقىرۋاتىدۇ…",
    "An error occurred" : "خاتالىق كۆرۈلدى",
    "Board details" : "مۇدىرىيەت تەپسىلاتلىرى",
    "Edit board" : "تاختاينى تەھرىرلەش",
    "Clone board" : "كلون تاختىسى",
    "Unarchive board" : "تاختاينى ئارخىپلىما",
    "Archive board" : "ئارخىپ تاختىسى",
    "Export board" : "ئېكسپورت تاختىسى",
    "Turn on due date reminders" : "قەرەلىدىكى ئەسكەرتىشلەرنى ئېچىڭ",
    "Turn off due date reminders" : "قەرەلىدىكى ئەسكەرتىشلەرنى ئېتىۋېتىڭ",
    "Due date reminders" : "قەرەلىدىكى ئەسكەرتىش",
    "Assigned cards" : "تەقسىم قىلىنغان كارتا",
    "No notifications" : "ئۇقتۇرۇش يوق",
    "Delete board" : "تاختاينى ئۆچۈرۈڭ",
    "Leave board" : "تاختايدىن ئايرىل",
    "Importing board..." : "تاختىنى كىرگۈزۋاتىدۇ…",
    "Board imported successfully" : "تاختاي مۇۋاپىقىيەتلىك كىرگۈزۈلدى",
    "Import board" : "تاختاينى كىرگۈز",
    "Clone {boardTitle}" : "{boardTitle} نى كلونلا",
    "Clone cards" : "كلون كارتىلىرى",
    "Clone assignments" : "تاپشۇرۇقلارنى كىلونلا",
    "Clone labels" : "بەلگىلەرنى كىلونلا",
    "Clone due dates" : "قەرەلنى كىلونلا",
    "Advanced options" : "ئىلغار تاللاشلار",
    "Move all cards to the first list" : "بارلىق كارتىلارنى تۇنجى تىزىملىككە يۆتكە",
    "Restore archived cards" : "ئارخىپلانغان كارتىلارنى ئەسلىگە قايتۇر",
    "Clone" : "كلونلا",
    "Export {boardTitle}" : "{boardTitle} نى چىقار",
    "Export as JSON" : "JSON سۈپىتىدە چىقار",
    "Export as CSV" : "CSV قىلىپ چىقىرىش",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "ئەسكەرتىش: پەقەت JSON فورماتىلا Deck ئەپكە قايتا ئىمپورت قىلىشنى قوللايدۇ.",
    "Export" : "ئېكسپورت",
    "Loading filtered view" : "سۈزۈلگەن كۆرۈنۈشنى يۈكلەۋاتىدۇ",
    "Search for {searchQuery} in other boards" : "{searchQuery} نى باشقا تاختايلاردىن ئىزدە",
    "Search for {searchQuery} in all boards" : "بارلىق تاختايلاردا {searchQuery} نى ئىزدەڭ",
    "No results found" : "ھېچقانداق نەتىجە تېپىلمىدى",
    "Deck board {name}\n* Last modified on {lastMod}" : "پالۋان تاختىسى {name}\n* ئاخىرقى قېتىم {lastMod} on دا ئۆزگەرتىلدى",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* ئىجاد قىلىنغان {created}\n* ئاخىرقى قېتىم {lastMod} on دا ئۆزگەرتىلدى\n* {nbAttachments} قوشۇمچە ھۆججەتلەر\n* {nbComments} باھا",
    "{nbCards} cards" : "{nbCards} كارتا",
    "Due on {date}" : "ۋاقتى {date}",
    "{stack} in {board}" : "{stack} in {board}",
    "Click to expand description" : "چۈشەندۈرۈشنى كېڭەيتىش ئۈچۈن چېكىڭ",
    "Click to expand comment" : "ئىنكاسنى كېڭەيتىش ئۈچۈن چېكىڭ",
    "Create card" : "كارتا قۇرۇش",
    "Create a new card" : "يېڭى كارتا قۇرۇڭ",
    "Card title" : "كارتا ئىسمى",
    "Creating the new card …" : "يېڭى كارتا قۇرۇش…",
    "Card \"{card}\" was added to \"{board}\"" : "كارتا \"{card}\" \"{board}\" غا قوشۇلدى",
    "Open card" : "ئوچۇق كارتا",
    "Close" : "تاقاش",
    "No upcoming cards" : "كەلگۈسىدىكى كارتا يوق",
    "upcoming cards today" : "بۈگۈن كېلىدىغان كارتىلار",
    "upcoming cards tomorrow" : "ئەتە كارتىلار",
    "upcoming cards" : "كەلگۈسىدىكى كارتىلار",
    "New card" : "يېڭى كارتا",
    "Link to a board" : "تاختايغا ئۇلىنىش",
    "Link to a card" : "كارتىغا ئۇلىنىش",
    "Create a card" : "كارتا ياساڭ",
    "Message from {author} in {conversationName}" : "{conversationName} دىكى {author} دىن ئۇچۇر",
    "Something went wrong" : "چاتاق چىقتى",
    "Failed to upload {name}" : "{name} نى يۈكلىيەلمىدى",
    "Maximum file size of {size} exceeded" : "ھۆججەتنىڭ ئەڭ چوڭ چوڭلۇقى {size} دىن ئېشىپ كەتتى",
    "Assigned users" : "تاپشۇرۇلغان ئىشلەتكۈچىلەر",
    "Due date" : "قەرەلى توشقان ۋاقىت",
    "Created" : "قۇرۇلدى",
    "Modified" : "ئۆزگەرتكەن",
    "Error creating the share" : "ھەمبەھىرلەشتە خاتالىق",
    "Share with a Deck card" : "پالۋان كارتىسى بىلەن ئورتاقلىشىڭ",
    "Share {file} with a Deck card" : "{file} نى پالۋان كارتىسى بىلەن ھەمبەھىرلەڭ",
    "Share" : "ھەمبەھىر",
    "The card \"%s\" on \"%s\" has reached its due date." : "«%s» دىكى «%s» كارتىسى قەرەلى توشقان.",
    "Share board with a user, group or team …" : "ئىشلەتكۈچى ، گۇرۇپپا ياكى گۇرۇپپا بىلەن ئورتاقلىشىش تاختىسى…",
    "Searching for users, groups and teams …" : "ئىشلەتكۈچى ، گۇرۇپپا ۋە گۇرۇپپىلارنى ئىزدەش…",
    "No participants found" : "قاتناشقۇچىلار تېپىلمىدى",
    "Save" : "ساقلا",
    "seconds ago" : "سېكۇنت بۇرۇن",
    "Today" : "بۈگۈن",
    "Tomorrow" : "ئەتە",
    "No due" : "مۇۋاپىق ئەمەس"
},
"nplurals=2; plural=(n != 1);");
