<?php return array(
    'root' => array(
        'name' => '__root__',
        'pretty_version' => 'v1.18.4',
        'version' => '1.18.4.0',
        'reference' => '46faf8ddaff4972dbd073b0ef101ce149e714e6a',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        '__root__' => array(
            'pretty_version' => 'v1.18.4',
            'version' => '1.18.4.0',
            'reference' => '46faf8ddaff4972dbd073b0ef101ce149e714e6a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'amphp/amp' => array(
            'pretty_version' => 'v3.1.0',
            'version' => '3.1.0.0',
            'reference' => '7cf7fef3d667bfe4b2560bc87e67d5387a7bcde9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amphp/amp',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'amphp/byte-stream' => array(
            'pretty_version' => 'v2.1.2',
            'version' => '2.1.2.0',
            'reference' => '55a6bd071aec26fa2a3e002618c20c35e3df1b46',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amphp/byte-stream',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'amphp/cache' => array(
            'pretty_version' => 'v2.0.1',
            'version' => '2.0.1.0',
            'reference' => '46912e387e6aa94933b61ea1ead9cf7540b7797c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amphp/cache',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'amphp/dns' => array(
            'pretty_version' => 'v2.4.0',
            'version' => '2.4.0.0',
            'reference' => '78eb3db5fc69bf2fc0cb503c4fcba667bc223c71',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amphp/dns',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'amphp/parallel' => array(
            'pretty_version' => 'v2.3.1',
            'version' => '2.3.1.0',
            'reference' => '5113111de02796a782f5d90767455e7391cca190',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amphp/parallel',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'amphp/parser' => array(
            'pretty_version' => 'v1.1.1',
            'version' => '1.1.1.0',
            'reference' => '3cf1f8b32a0171d4b1bed93d25617637a77cded7',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amphp/parser',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'amphp/pipeline' => array(
            'pretty_version' => 'v1.2.3',
            'version' => '1.2.3.0',
            'reference' => '7b52598c2e9105ebcddf247fc523161581930367',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amphp/pipeline',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'amphp/process' => array(
            'pretty_version' => 'v2.0.3',
            'version' => '2.0.3.0',
            'reference' => '52e08c09dec7511d5fbc1fb00d3e4e79fc77d58d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amphp/process',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'amphp/serialization' => array(
            'pretty_version' => 'v1.0.0',
            'version' => '1.0.0.0',
            'reference' => '693e77b2fb0b266c3c7d622317f881de44ae94a1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amphp/serialization',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'amphp/socket' => array(
            'pretty_version' => 'v2.3.1',
            'version' => '2.3.1.0',
            'reference' => '58e0422221825b79681b72c50c47a930be7bf1e1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amphp/socket',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'amphp/sync' => array(
            'pretty_version' => 'v2.3.0',
            'version' => '2.3.0.0',
            'reference' => '217097b785130d77cfcc58ff583cf26cd1770bf1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amphp/sync',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'composer/pcre' => array(
            'pretty_version' => '3.3.2',
            'version' => '3.3.2.0',
            'reference' => 'b2bed4734f0cc156ee1fe9c0da2550420d99a21e',
            'type' => 'library',
            'install_path' => __DIR__ . '/./pcre',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'composer/semver' => array(
            'pretty_version' => '3.4.3',
            'version' => '3.4.3.0',
            'reference' => '4313d26ada5e0c4edfbd1dc481a92ff7bff91f12',
            'type' => 'library',
            'install_path' => __DIR__ . '/./semver',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'composer/xdebug-handler' => array(
            'pretty_version' => '3.0.5',
            'version' => '3.0.5.0',
            'reference' => '6c1925561632e83d60a44492e0b344cf48ab85ef',
            'type' => 'library',
            'install_path' => __DIR__ . '/./xdebug-handler',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'daverandom/libdns' => array(
            'pretty_version' => 'v2.1.0',
            'version' => '2.1.0.0',
            'reference' => 'b84c94e8fe6b7ee4aecfe121bfe3b6177d303c8a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../daverandom/libdns',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'dnoegel/php-xdg-base-dir' => array(
            'pretty_version' => 'v0.1.1',
            'version' => '0.1.1.0',
            'reference' => '8f8a6e48c5ecb0f991c2fdcf5f154a47d85f9ffd',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dnoegel/php-xdg-base-dir',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'doctrine/deprecations' => array(
            'pretty_version' => '1.1.5',
            'version' => '1.1.5.0',
            'reference' => '459c2f5dd3d6a4633d3b5f46ee2b1c40f57d3f38',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/deprecations',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'felixfbecker/advanced-json-rpc' => array(
            'pretty_version' => 'v3.2.1',
            'version' => '3.2.1.0',
            'reference' => 'b5f37dbff9a8ad360ca341f3240dc1c168b45447',
            'type' => 'library',
            'install_path' => __DIR__ . '/../felixfbecker/advanced-json-rpc',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'felixfbecker/language-server-protocol' => array(
            'pretty_version' => 'v1.5.3',
            'version' => '1.5.3.0',
            'reference' => 'a9e113dbc7d849e35b8776da39edaf4313b7b6c9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../felixfbecker/language-server-protocol',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'fidry/cpu-core-counter' => array(
            'pretty_version' => '1.2.0',
            'version' => '1.2.0.0',
            'reference' => '8520451a140d3f46ac33042715115e290cf5785f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../fidry/cpu-core-counter',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'kelunik/certificate' => array(
            'pretty_version' => 'v1.1.3',
            'version' => '1.1.3.0',
            'reference' => '7e00d498c264d5eb4f78c69f41c8bd6719c0199e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../kelunik/certificate',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'league/uri' => array(
            'pretty_version' => '7.5.1',
            'version' => '7.5.1.0',
            'reference' => '81fb5145d2644324614cc532b28efd0215bda430',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/uri',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'league/uri-interfaces' => array(
            'pretty_version' => '7.5.0',
            'version' => '7.5.0.0',
            'reference' => '08cfc6c4f3d811584fb09c37e2849e6a7f9b0742',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/uri-interfaces',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'netresearch/jsonmapper' => array(
            'pretty_version' => 'v4.5.0',
            'version' => '4.5.0.0',
            'reference' => '8e76efb98ee8b6afc54687045e1b8dba55ac76e5',
            'type' => 'library',
            'install_path' => __DIR__ . '/../netresearch/jsonmapper',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'nikic/php-parser' => array(
            'pretty_version' => 'v5.5.0',
            'version' => '5.5.0.0',
            'reference' => 'ae59794362fe85e051a58ad36b289443f57be7a9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nikic/php-parser',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'phpdocumentor/reflection-common' => array(
            'pretty_version' => '2.2.0',
            'version' => '2.2.0.0',
            'reference' => '1d01c49d4ed62f25aa84a747ad35d5a16924662b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpdocumentor/reflection-common',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'phpdocumentor/reflection-docblock' => array(
            'pretty_version' => '5.6.2',
            'version' => '5.6.2.0',
            'reference' => '92dde6a5919e34835c506ac8c523ef095a95ed62',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpdocumentor/reflection-docblock',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'phpdocumentor/type-resolver' => array(
            'pretty_version' => '1.10.0',
            'version' => '1.10.0.0',
            'reference' => '679e3ce485b99e84c775d28e2e96fade9a7fb50a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpdocumentor/type-resolver',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'phpstan/phpdoc-parser' => array(
            'pretty_version' => '2.1.0',
            'version' => '2.1.0.0',
            'reference' => '9b30d6fd026b2c132b3985ce6b23bec09ab3aa68',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpstan/phpdoc-parser',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'psalm/psalm' => array(
            'dev_requirement' => true,
            'provided' => array(
                0 => '6.5.0',
            ),
        ),
        'psr/container' => array(
            'pretty_version' => '2.0.2',
            'version' => '2.0.2.0',
            'reference' => 'c71ecc56dfe541dbd90c5360474fbc405f8d5963',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/container',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'psr/http-factory' => array(
            'pretty_version' => '1.1.0',
            'version' => '1.1.0.0',
            'reference' => '2b4765fddfe3b508ac62f829e852b1501d3f6e8a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-factory',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'psr/http-message' => array(
            'pretty_version' => '2.0',
            'version' => '2.0.0.0',
            'reference' => '402d35bcb92c70c026d1a6a9883f06b2ead23d71',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-message',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'psr/log' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => 'f16e1d5863e37f8d8c2a01719f5b34baa2b714d3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'psr/log-implementation' => array(
            'dev_requirement' => true,
            'provided' => array(
                0 => '1.0|2.0|3.0',
            ),
        ),
        'revolt/event-loop' => array(
            'pretty_version' => 'v1.0.7',
            'version' => '1.0.7.0',
            'reference' => '09bf1bf7f7f574453efe43044b06fafe12216eb3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../revolt/event-loop',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'sebastian/diff' => array(
            'pretty_version' => '5.1.1',
            'version' => '5.1.1.0',
            'reference' => 'c41e007b4b62af48218231d6c2275e4c9b975b2e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sebastian/diff',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'spatie/array-to-xml' => array(
            'pretty_version' => '3.4.0',
            'version' => '3.4.0.0',
            'reference' => '7dcfc67d60b0272926dabad1ec01f6b8a5fb5e67',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spatie/array-to-xml',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'symfony/console' => array(
            'pretty_version' => 'v6.4.22',
            'version' => '6.4.22.0',
            'reference' => '7d29659bc3c9d8e9a34e2c3414ef9e9e003e6cf3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/console',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'symfony/deprecation-contracts' => array(
            'pretty_version' => 'v3.6.0',
            'version' => '3.6.0.0',
            'reference' => '63afe740e99a13ba87ec199bb07bbdee937a5b62',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/deprecation-contracts',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'symfony/filesystem' => array(
            'pretty_version' => 'v6.4.13',
            'version' => '6.4.13.0',
            'reference' => '4856c9cf585d5a0313d8d35afd681a526f038dd3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/filesystem',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'symfony/polyfill-ctype' => array(
            'pretty_version' => 'v1.32.0',
            'version' => '1.32.0.0',
            'reference' => 'a3cc8b044a6ea513310cbd48ef7333b384945638',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-ctype',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'symfony/polyfill-intl-grapheme' => array(
            'pretty_version' => 'v1.32.0',
            'version' => '1.32.0.0',
            'reference' => 'b9123926e3b7bc2f98c02ad54f6a4b02b91a8abe',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-grapheme',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'symfony/polyfill-intl-normalizer' => array(
            'pretty_version' => 'v1.32.0',
            'version' => '1.32.0.0',
            'reference' => '3833d7255cc303546435cb650316bff708a1c75c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-normalizer',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'symfony/polyfill-mbstring' => array(
            'pretty_version' => 'v1.32.0',
            'version' => '1.32.0.0',
            'reference' => '6d857f4d76bd4b343eac26d6b539585d2bc56493',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-mbstring',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'symfony/service-contracts' => array(
            'pretty_version' => 'v3.6.0',
            'version' => '3.6.0.0',
            'reference' => 'f021b05a130d35510bd6b25fe9053c2a8a15d5d4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/service-contracts',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'symfony/string' => array(
            'pretty_version' => 'v6.4.21',
            'version' => '6.4.21.0',
            'reference' => '73e2c6966a5aef1d4892873ed5322245295370c6',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/string',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'vimeo/psalm' => array(
            'pretty_version' => '6.5.0',
            'version' => '6.5.0.0',
            'reference' => '38fc8444edf0cebc9205296ee6e30e906ade783b',
            'type' => 'project',
            'install_path' => __DIR__ . '/../vimeo/psalm',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'webmozart/assert' => array(
            'pretty_version' => '1.11.0',
            'version' => '1.11.0.0',
            'reference' => '11cb2199493b2f8a3b53e7f19068fc6aac760991',
            'type' => 'library',
            'install_path' => __DIR__ . '/../webmozart/assert',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
    ),
);
