OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Stworzyłeś nową tablicę {board}",
    "{user} has created a new board {board}" : "{user} utworzył nową tablicę {board}",
    "You have deleted the board {board}" : "Usunąłeś tablicę {board}",
    "{user} has deleted the board {board}" : "{user} usunął tablicę {board}",
    "You have restored the board {board}" : "Przywróciłeś tablicę {board}",
    "{user} has restored the board {board}" : "{user} przywrócił tablicę {board}",
    "You have shared the board {board} with {acl}" : "Udostępniłeś tablicę {board} z {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} udostępnił tablicę {board} z {acl}",
    "You have removed {acl} from the board {board}" : "Usunąłeś {acl} z tablicy {board}",
    "{user} has removed {acl} from the board {board}" : "{user} usunął {acl} z tablicy {board}",
    "You have renamed the board {before} to {board}" : "Zmieniłeś nazwę tablicy {before} na {board}",
    "{user} has renamed the board {before} to {board}" : "{user} zmienił nazwę tablicy {before} na {board} ",
    "You have archived the board {board}" : "Zarchiwizowałeś tablicę {board}",
    "{user} has archived the board {before}" : "{user} zarchiwizował tablicę {before}",
    "You have unarchived the board {board}" : "Przywróciłeś tablicę {board} z archiwum",
    "{user} has unarchived the board {before}" : "{user} cofnął archiwizację tablicy {before}",
    "You have created a new list {stack} on board {board}" : "Utworzyłeś nową listę {stack} na tablicy {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} utworzył nową listę {stack} na tablicy {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Zmieniłeś nazwę listy {before} na {stack} na tablicy {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} zmienił nazwę listy {before} na {stack} na tablicy {board}",
    "You have deleted list {stack} on board {board}" : "Usunąłeś listę {stack} na tablicy {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} usunął listę {stack} na tablicy {board}",
    "You have created card {card} in list {stack} on board {board}" : "Utworzyłeś kartę {card} w liście {stack} na tablicy {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} utworzył kartę {card} w liście {stack} na tablicy {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Usunąłeś kartę {card} w liście {stack} na tablicy {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} usunął kartę {card} w liście {stack} na tablicy {board}",
    "You have renamed the card {before} to {card}" : "Zmieniłeś nazwę karty {before} na {card}",
    "{user} has renamed the card {before} to {card}" : "{user} zmienił nazwę karty {before} na {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Dodałeś opis karty {card} w liście {stack} na tablicy {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} dodał opis karty {card} w liście {stack} na tablicy {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Zaktualizowałeś opis karty {card} w liście {stack} na tablicy {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} zaktualizował opis karty {card} w liście {stack} na tablicy {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Zarchiwizowałeś kartę {card} w liście {stack} na tablicy {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} zarchiwizował kartę {card} w liście {stack} na tablicy {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Masz niezarchiwizowaną kartę {card} w liście {stack} na tablicy {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} ma niezarchiwizowaną kartę {card} w liście {stack} na tablicy {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Oznaczyłeś kartę {card} jako wykonaną na liście {stack} na tablicy {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} oznaczył kartę {card} jako wykonaną na liście {stack} na tablicy {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Oznaczyłeś kartę {card} jako niewykonaną na liście {stack} na tablicy {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} oznaczył kartę {card} jako niewykonaną na liście {stack} na tablicy {board}",
    "You have removed the due date of card {card}" : "Usunąłeś datę ważności karty {card}",
    "{user} has removed the due date of card {card}" : "{user} usunął datę ważności karty {card}",
    "You have set the due date of card {card} to {after}" : "Ustawiłeś datę ważności karty {card} na {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} ustawił datę ważności karty {card} na {after}",
    "You have updated the due date of card {card} to {after}" : "Zaktualizowałeś datę ważności karty {card} na {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} zaktualizował datę ważności karty {card} na {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Dodałeś etykietę {label} do karty {card} w liście {stack} na tablicy {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} dodał etykietę {label} do karty {card} w liście {stack} na tablicy {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Usunąłeś etykietę {label} do karty {card} w liście {stack} na tablicy {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} usunął etykietę {label} do karty {card} w liście {stack} na tablicy {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Przydzieliłeś {assigneduser} do karty {card} na tablicy {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} przydzielił {assigneduser} do karty {card} na tablicy {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Usunąłeś przydzielenie {assigneduser} do karty {card} na tablicy {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} usunął przydzielenie {assigneduser} do karty {card} na tablicy {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Przeniosłeś kartę {card} z listy {stackBefore} do {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} przeniósł kartę {card} z listy {stackBefore} do {stack}",
    "You have added the attachment {attachment} to card {card}" : "Dodałeś załącznik {attachment} do karty {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} dodał załącznik {attachment} do karty {card}",
    "You have updated the attachment {attachment} on card {card}" : "Zaktualizowałeś załącznik {attachment} na karcie {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} zaktualizował załącznik {attachment} na karcie {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Usunąłeś załącznik {attachment} z karty {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} usunął załącznik {attachment} z karty {card}",
    "You have restored the attachment {attachment} to card {card}" : "Przywróciłeś załącznik {attachment} do karty {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} przywrócił załącznik {attachment} do karty {card}",
    "You have commented on card {card}" : "Skomentowałeś kartę {card}",
    "{user} has commented on card {card}" : "{user} skomentował kartę {card}",
    "Deck" : "Tablica",
    "Changes in the <strong>Deck app</strong>" : "Zmiany w <strong>aplikacji Tablica</strong>",
    "A <strong>board, list or card</strong> was changed" : "Zmieniono <strong>tablicę, listę lub kartę</strong>",
    "A <strong>comment</strong> was created on a card" : "Na karcie utworzono <strong>komentarz</strong>",
    "A <strong>card description</strong> has been changed" : "Zmieniono <strong>opis karty</strong>",
    "The file was uploaded" : "Plik został wysłany",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Wgrany plik przekracza wartość upload_max_filesize zdefiniowaną w php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Wysłany plik przekracza wielkość dyrektywy MAX_FILE_SIZE określonej w formularzu HTML",
    "The file was only partially uploaded" : "Plik został wysłany tylko częściowo.",
    "No file was uploaded" : "Nie wysłano żadnego pliku",
    "Missing a temporary folder" : "Brak katalogu tymczasowego",
    "Could not write file to disk" : "Nie można zapisać pliku na dysk",
    "A PHP extension stopped the file upload" : "Rozszerzenie PHP zatrzymało wysyłanie pliku",
    "No file uploaded or file size exceeds maximum of %s" : "Brak wysłanego pliku lub rozmiar pliku przekracza maksymalny limit %s",
    "Invalid file type. Only JSON files are allowed." : "Nieprawidłowy typ pliku. Dozwolone są tylko pliki JSON.",
    "Invalid JSON data" : "Nieprawidłowe dane JSON",
    "Failed to import board" : "Nie udało się zaimportować tablicy",
    "Cards due today" : "Karty na dzisiaj",
    "Cards due tomorrow" : "Karty na jutro",
    "Upcoming cards" : "Nadchodzące karty",
    "Load more" : "Wczytaj więcej",
    "Welcome to Nextcloud Deck!" : "Witamy w Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Karta \"%s\" na \"%s\" została przydzielona Tobie przez %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} przydzielił Tobie kartę {deck-card} na {deck-board}.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Karta \"%1$s\" na \"%2$s\" wygasła.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Karta {deck-card} na {deck-board} uzyskała swój termin.",
    "%s has mentioned you in a comment on \"%s\"." : "%s wspomniał o Tobie w komentarzu do \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} wspomniał o Tobie w komentarzu w {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Tablica \"%s\" została Tobie udostępniona przez %s.",
    "{user} has shared {deck-board} with you." : "{user} udostępnił Tobie {deck-board}.",
    "The remote board %1$s has been shared with you by %2$s" : "Zdalna tablica %1$s została Ci udostępniona przez %2$s",
    "Deck board" : "Tablica Deck",
    "Owned by %1$s" : "Właścicielem jest %1$s",
    "Deck boards, cards and comments" : "Tablice, karty i komentarze",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Od %1$s, w %2$s/%3$s, posiadane przez %4$s",
    "Create a new deck card" : "Utwórz nową kartę tablicy",
    "Card comments" : "Komentarze karty",
    "%s on %s" : "%s na %s",
    "Deck boards and cards" : "Tablice i karty",
    "No data was provided to create an attachment." : "Nie podano żadnych danych do utworzenia załącznika.",
    "Finished" : "Ukończone",
    "To review" : "Do sprawdzenia",
    "Action needed" : "Wymagana akcja",
    "Later" : "Później",
    "copy" : "kopiuj",
    "Read more inside" : "Czytaj więcej w środku",
    "Custom lists - click to rename!" : "Listy niestandardowe – kliknij, aby zmienić nazwę!",
    "To Do" : "Do zrobienia",
    "In Progress" : "W trakcie",
    "Done" : "Zrobione",
    "1. Open to learn more about boards and cards" : "1. Otwórz, aby dowiedzieć się więcej o tablicach i kartach",
    "2. Drag cards left and right, up and down" : "2. Przeciągaj karty w lewo, prawo, w górę i w dół",
    "3. Apply rich formatting and link content" : "3. Zastosuj formatowanie i dodaj linki",
    "4. Share, comment and collaborate!" : "4. Udostępniaj, komentuj i współpracuj!",
    "Create your first card!" : "Utwórz swoją pierwszą kartę!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Ten komentarz ma więcej niż %s znaków.\nDodany jako załącznik do karty o nazwie %s.\nDostępne pod adresem URL: %s.",
    "Attachments" : "Załączniki",
    "File" : "Plik",
    "date" : "data",
    "Card not found" : "Nie znaleziono karty",
    "Path is already shared with this card" : "Ścieżka jest już udostępniona dla tej karty",
    "Invalid date, date format must be YYYY-MM-DD" : "Nieprawidłowa data, format daty to RRRR-MM-DD",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Wszystkie tablice należące do Ciebie, w tym stosy, karty, etykiety, przypisania i komentarze",
    "Personal planning and team project organization" : "Indywidualne planowanie i organizacja projektu zespołowego",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Tablica to narzędzie organizacyjne w stylu kanban, którego celem jest osobiste planowanie i organizacja projektu dla zespołów zintegrowanych z Nextcloud.\n\n\n- 📥 Dodaj swoje zadania do kart i porządkuj je\n- 📄 Zapisuj dodatkowe notatki w Markdown\n- 🔖 Przypisuj etykiety dla jeszcze lepszej organizacji\n- 👥 Współdziel ze swoim zespołem, przyjaciółmi lub rodziną\n- 📎 Dołączaj pliki i umieszczaj je w opisie Markdown\n- 💬 Porozmawiaj ze swoim zespołem za pomocą komentarzy\n- ⚡ Śledź zmiany w strumieniu aktywności\n- 🚀 Organizuj swój projekt",
    "Add board" : "Dodaj tablicę",
    "Card details" : "Szczegóły karty",
    "Select the board to link to a project" : "Wybierz tablicę, aby połączyć się z projektem",
    "Search by board title" : "Szukaj według tytułu tablicy",
    "Select board" : "Wybierz tablicę",
    "Move/copy card" : "Przenieś/skopiuj kartę",
    "Select a board" : "Wybierz tablicę",
    "No lists available" : "Brak dostępnych list",
    "Select a list" : "Wybierz listę",
    "Move card" : "Przenieś kartę",
    "Copy card" : "Skopiuj kartę",
    "Select the card to link to a project" : "Wybierz kartę, aby połączyć z projektem",
    "Link to card" : "Link do karty",
    "Select a card" : "Wybierz kartę",
    "Cancel" : "Anuluj",
    "This board is read only" : "Ta tablica jest tylko do odczytu",
    "Drop your files to upload" : "Upuść swoje pliki do wysłania",
    "File already exists" : "Plik już istnieje",
    "A file with the name {filename} already exists." : "Plik o nazwie {filename} już istnieje.",
    "Do you want to overwrite it?" : "Czy chcesz go nadpisać?",
    "Overwrite file" : "Nadpisać plik",
    "Keep existing file" : "Zachowaj istniejący plik",
    "Add card" : "Dodaj kartę",
    "Archived cards" : "Zarchiwizowane karty",
    "Add list" : "Dodaj listę",
    "List name" : "Nazwa listy",
    "Clear search" : "Wyczyść wyszukiwanie",
    "Active filters" : "Aktywne filtry",
    "Apply filter" : "Zastosuj filtr",
    "Filter by tag" : "Filtruj według etykiety",
    "Filter by assigned user" : "Filtruj według przydzielonego użytkownika",
    "Unassigned" : "Nieprzydzielone",
    "Filter by status" : "Filtruj według statusu",
    "Open and completed" : "Otwarte i zakończone",
    "Open" : "Otwórz",
    "Completed" : "Zakończone",
    "Filter by due date" : "Filtruj według terminu",
    "Overdue" : "Zaległe",
    "Next 24 hours" : "Następne 24 godziny",
    "Next 7 days" : "Następne 7 dni",
    "Next 30 days" : "Następne 30 dni",
    "No due date" : "Brak daty ważności",
    "Clear filter" : "Wyczyść filtr",
    "View Modes" : "Zobacz tryby",
    "Toggle View Modes" : "Przełącz tryb widoku",
    "Kanban view" : "Widok Kanban",
    "Gantt view" : "Widok Gantta",
    "Hide archived cards" : "Ukryj zarchiwizowane karty",
    "Show archived cards" : "Pokaż zarchiwizowane karty",
    "Toggle compact mode" : "Przełącz w tryb kompaktowy",
    "Hide card cover images" : "Ukryj obrazy okładek kart",
    "Show card cover images" : "Pokaż obrazy okładek kart",
    "Open details" : "Otwórz szczegóły",
    "Details" : "Szczegóły",
    "Deck settings" : "Ustawienia Tablicy",
    "General" : "Ogólne",
    "Use bigger card view" : "Użyj większego widoku karty",
    "Appearance" : "Wygląd",
    "Show card ID badge" : "Pokaż ID karty",
    "Show boards in calendar/tasks" : "Pokaż tablice w kalendarzu/zadaniach",
    "Admin settings" : "Ustawienia administratora",
    "Limit board creation to some groups" : "Ogranicz tworzenie tablic dla niektórych grup",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Użytkownicy spoza tych grup nie będą mogli tworzyć własnych tablic, ale nadal będą mogli pracować na tablicach, które zostały im udostępnione.",
    "Enable federation" : "Włącz federację",
    "Board actions" : "Akcje tablicy",
    "Scroll sideways" : "Przewijanie w bok",
    "Navigate between cards" : "Nawigacja między kartami",
    "Close card details" : "Zamknij szczegóły karty",
    "Search" : "Szukaj",
    "Show card filters" : "Pokaż filtry kart",
    "Clear card filters" : "Wyczyść filtry kart",
    "Show those shortcuts" : "Pokaż te skróty",
    "Card actions" : "Akcje karty",
    "Open card details" : "Otwórz szczegóły karty",
    "Edit the card title" : "Edytuj tytuł karty",
    "Assign yourself to the current card" : "Przypisz siebie do tej karty",
    "Archive/unarchive the current card" : "Archiwizuj/odarchiwizuj tę kartę",
    "Mark card as completed/not completed" : "Oznacz kartę jako ukończoną/nieukończoną",
    "Open card menu" : "Otwórz menu karty",
    "Currently present people" : "Aktualnie obecne osoby",
    "Loading board" : "Wczytywanie tablicy",
    "Board not found" : "Nie znaleziono tablicy",
    "Create a new list to add cards to this board" : "Utwórz nową listę, aby dodać karty do tej tablicy",
    "Sharing" : "Udostępnianie",
    "Tags" : "Etykiety",
    "Deleted items" : "Usunięte pozycje",
    "Activity" : "Aktywność",
    "Deleted lists" : "Usunięte listy",
    "Undo" : "Cofnij",
    "Deleted cards" : "Usunięte karty",
    "Hours" : "Godzin",
    "Days" : "Dni",
    "Weeks" : "Tygodnie",
    "Months" : "Miesiące",
    "No cards yet" : "Brak kart",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Ustaw datę rozpoczęcia i termin wykonania na kartach, aby wyświetlić je na wykresie Gantta",
    "Cards without dates ({count})" : "Karty bez dat ({count})",
    "Failed to create share with {displayName}" : "Nie udało się utworzyć udostępnienia dla {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Czy na pewno chcesz przenieść tablicę {title} do {user}?",
    "Transfer the board." : "Przeniesienie tablicy.",
    "Transfer" : "Przenieś",
    "The board has been transferred to {user}" : "Tablica została przeniesiona do {user}",
    "Failed to transfer the board to {user}" : "Nie udało się przenieść tablicy do {user}",
    "Board owner" : "Właściciel tablicy",
    "(Group)" : "(Grupa)",
    "(Team)" : "(Zespół)",
    "(remote)" : "(zdalne)",
    "Can edit" : "Może edytować",
    "Can share" : "Może udostępnić",
    "Can manage" : "Może zarządzać",
    "Owner" : "Właściciel",
    "Delete" : "Usuń",
    "List deleted" : "Lista usunięta",
    "Edit list title" : "Edytuj tytuł listy",
    "Archive all cards" : "Zarchiwizuj wszystkie karty",
    "Unarchive all cards" : "Przywróć wszystkie karty z archiwum",
    "Do not set cards as \"done\"" : "Nie ustawiaj kart jako \"zrobione\"",
    "Set cards as \"done\"" : "Ustaw karty jako \"zrobione\"",
    "Delete list" : "Usuń listę",
    "Archive all cards in this list" : "Zarchiwizuj wszystkie karty na tej liście",
    "Unarchive all cards in this list" : "Cofnij archiwizację wszystkich kart z tej listy",
    "Add a new card" : "Dodaj nową kartę",
    "Card name" : "Nazwa karty",
    "title and color value must be provided" : "należy podać tytuł i kolor",
    "Edit" : "Edycja",
    "Add a new tag" : "Dodaj nową etykietę",
    "Board name" : "Nazwa tablicy",
    "Members" : "Członkowie",
    "Assign to users/groups/team" : "Przypisz do użytkowników/grup/zespołu",
    "Assign a user to this card…" : "Przypisz użytkownika do tej karty…",
    "Select a user to assign to this card…" : "Wybierz użytkownika, aby przypisać do tej karty…",
    "File to share" : "Plik do udostępnienia",
    "Invalid path selected" : "Wybrano nieprawidłową ścieżkę",
    "Upload new files" : "Wyślij nowe pliki",
    "Share from Files" : "Udostępnij z Plików",
    "Pending share" : "Oczekujące udostępnienie",
    "Add this attachment" : "Dodaj ten załącznik",
    "Show in Files" : "Pokaż w Plikach",
    "Download" : "Pobierz",
    "Remove attachment" : "Usuń załącznik",
    "Delete Attachment" : "Usuń załącznik",
    "Restore Attachment" : "Przywróć załącznik",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Zmodyfikowano:  {modifiedDate} utworzono {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Zmodyfikowano {modifiedDate}\nUtworzono: {createdDate}\nUtworzone przez: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Zmodyfikowano: {modifiedDate}\nUtworzono: {createdDate}",
    "The title cannot be empty." : "Tytuł nie może być pusty.",
    "Cannot close unsaved card!" : "Nie można zamknąć niezapisanej karty!",
    "Open in sidebar view" : "Otwórz w widoku paska bocznego",
    "Open in bigger view" : "Otwórz w dużym widoku",
    "Comments" : "Komentarze",
    "Failed to load comments" : "Nie udało się wczytać komentarzy",
    "No comments yet. Begin the discussion!" : "Brak komentarzy. Rozpocznij dyskusję!",
    "The comment cannot be empty." : "Komentarz nie może być pusty.",
    "The comment cannot be longer than 1000 characters." : "Komentarz nie może być dłuższy niż 1000 znaków.",
    "Submit" : "Wyślij",
    "Created:" : "Utworzono:",
    "In reply to" : "Odpowiedź na",
    "Cancel reply" : "Anuluj odpowiedź",
    "Reply" : "Odpowiedz",
    "Update" : "Aktualizuj",
    "Assign dependent cards" : "Przypisz karty zależne",
    "Mark as done" : "Oznacz jako wykonane",
    "Mark as not done" : "Oznacz jako niewykonane",
    "Remove dependency" : "Usuń zależność",
    "Add dependent card" : "Dodaj kartę zależną",
    "Assign a dependent card…" : "Przypisz kartę zależną...",
    "Select a dependent card…" : "Wybierz kartę zależną...",
    "Write a description …" : "Napisz opis…",
    "The description has been changed by another user." : "Opis został zmieniony przez innego użytkownika.",
    "Could not save description" : "Nie udało się zapisać opisu",
    "Description" : "Opis",
    "(Unsaved)" : "(Niezapisane)",
    "(Saving…)" : "(Zapisywanie…)",
    "Formatting help" : "Pomoc przy formatowaniu",
    "Edit description" : "Edytuj opis",
    "View description" : "Zobacz opis",
    "Add Attachment" : "Dodaj załącznik",
    "Choose attachment" : "Wybierz załącznik",
    "Select Date" : "Wybierz datę",
    "Later today – {timeLocale}" : "Później dzisiaj – {timeLocale}",
    "Set due date for later today" : "Ustaw termin na dzisiaj na późniejszy czas",
    "Tomorrow – {timeLocale}" : "Jutro – {timeLocale}",
    "Set due date for tomorrow" : "Ustaw termin na jutro",
    "This weekend – {timeLocale}" : "W ten weekend – {timeLocale}",
    "Set due date for this weekend" : "Ustaw termin na ten weekend",
    "Next week – {timeLocale}" : "Następny tydzień – {timeLocale}",
    "Set due date for next week" : "Ustaw termin na przyszły tydzień",
    "Assign a due date to this card…" : "Przypisz termin dla tej karty…",
    "Set a due date" : "Ustaw termin",
    "Add due date" : "Dodaj termin",
    "Choose a date" : "Wybierz datę",
    "Remove due date" : "Usuń datę realizacji",
    "Due at:" : "Termin:",
    "Not done" : "Nie zrobione",
    "Unarchive card" : "Nie archiwizuj karty",
    "Archive card" : "Zarchiwizuj kartę",
    "Assign a start date to this card…" : "Przypisz datę rozpoczęcia do tej karty...",
    "Set a start date" : "Ustaw datę rozpoczęcia",
    "Add start date" : "Dodaj datę rozpoczęcia",
    "Remove start date" : "Usuń datę rozpoczęcia",
    "Assign a tag to this card…" : "Przypisz etykietę do tej karty…",
    "Select or create a tag…" : "Wybierz lub utwórz tag…",
    "Create a new tag:" : "Utwórz nową etykietę:",
    "(group)" : "(grupa)",
    "{count} comments, {unread} unread" : "Komentarze: {count}, nieprzeczytane: {unread}",
    "Todo items" : "Do zrobienia",
    "Edit card title" : "Edytuj nazwę karty",
    "Open link" : "Otwórz link",
    "Card deleted" : "Karta usunięta",
    "Change card color" : "Zmień kolor karty",
    "Edit title" : "Edytuj tytuł",
    "Assign to me" : "Przydziel do mnie",
    "Unassign myself" : "Cofnij moje przypisanie",
    "Delete card" : "Usuń kartę",
    "All boards" : "Wszystkie tablice",
    "Archived boards" : "Zarchiwizowane tablice",
    "Shared with you" : "Udostępnione Tobie",
    "Cancel edit" : "Anuluj edycję",
    "Save board" : "Zapisz tablicę",
    "Board {0} deleted" : "Tablica {0} została usunięta",
    "All cards" : "Wszystkie karty",
    "Only assigned cards" : "Tylko przydzielone karty",
    "No reminder" : "Bez przypomnienia",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Czy na pewno chcesz usunąć tablicę {title}? Spowoduje to usunięcie wszystkich danych z tej tablicy łącznie z zarchiwizowanymi kartami.",
    "Delete the board?" : "Usunąć tablicę?",
    "Are you sure you want to leave the board {title}?" : "Czy na pewno chcesz opuścić tablicę {title}?",
    "Leave the board?" : "Opuścić tablicę?",
    "Leave" : "Opuść",
    "Failed to leave the board" : "Nie udało się opuścić tablicy",
    "Exporting board..." : "Eksportowanie tablicy…",
    "An error occurred" : "Wystąpił błąd",
    "Board details" : "Szczegóły tablicy",
    "Edit board" : "Edytuj tablicę",
    "Clone board" : "Klonuj tablicę",
    "Unarchive board" : "Przywróć tablicę",
    "Archive board" : "Zarchiwizuj tablicę",
    "Export board" : "Eksportuj tablicę",
    "Turn on due date reminders" : "Włącz przypomnienia o terminach",
    "Turn off due date reminders" : "Wyłącz przypomnienia o terminach",
    "Remove as default board" : "Usuń jako domyślną tablicę",
    "Set as default board" : "Ustaw jako domyślną tablicę",
    "Due date reminders" : "Przypomnienia o terminach",
    "Assigned cards" : "Przydzielone karty",
    "No notifications" : "Brak powiadomień",
    "Delete board" : "Usuń tablicę",
    "Leave board" : "Opuścić tablicę",
    "Importing board..." : "Importowanie tablicy…",
    "Board imported successfully" : "Tablica została pomyślnie zaimportowana",
    "Import board" : "Importuj tablicę",
    "Clone {boardTitle}" : "Klonuj  {boardTitle}",
    "Clone cards" : "Klonuj karty",
    "Clone assignments" : "Klonuj przypisania",
    "Clone labels" : "Klonuj etykiety",
    "Clone due dates" : "Klonuj terminy",
    "Advanced options" : "Opcje zaawansowane",
    "Move all cards to the first list" : "Przenieś wszystkie karty na pierwszą listę",
    "Restore archived cards" : "Przywróć zarchiwizowane karty",
    "Clone" : "Klonuj",
    "Export {boardTitle}" : "Eksportuj {boardTitle}",
    "Export as JSON" : "Eksportuj jako JSON",
    "Export as CSV" : "Eksportuj jako CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Uwaga: tylko format JSON jest obsługiwany przy imporcie z powrotem do aplikacji Deck.",
    "Export" : "Eksportuj",
    "Loading filtered view" : "Wczytywanie przefiltrowanego widoku",
    "Search for {searchQuery} in other boards" : "Szukaj {searchQuery} na innych tablicach",
    "Search for {searchQuery} in all boards" : "Wyszukaj dla {searchQuery} na wszystkich tablicach",
    "No results found" : "Nie znaleziono wyników",
    "Deck board {name}\n* Last modified on {lastMod}" : "Tablica {name}\n* Ostatnia modyfikacja w dniu {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Utworzono {created}\n* Ostatnia modyfikacja {lastMod}\n* Załączniki {nbAttachments}\n* Komentarze {nbComments}",
    "{nbCards} cards" : "Karty {nbCards}",
    "Due on {date}" : "Termin {date}",
    "{stack} in {board}" : "{stack} na {board}",
    "Click to expand description" : "Kliknij, aby rozwinąć opis",
    "Click to expand comment" : "Kliknij, aby rozwinąć komentarz",
    "Create card" : "Utwórz kartę",
    "Create a new card" : "Utwórz nową kartę",
    "Card title" : "Tytuł karty",
    "Creating the new card …" : "Tworzenie nowej karty…",
    "Card \"{card}\" was added to \"{board}\"" : "Karta \"{card}\" została dodana do \"{board}\"",
    "Open card" : "Otwórz kartę",
    "Close" : "Zamknij",
    "No upcoming cards" : "Brak nadchodzących kart",
    "upcoming cards today" : "nadchodzące karty dzisiaj",
    "upcoming cards tomorrow" : "nadchodzące karty jutro",
    "upcoming cards" : "nadchodzące karty",
    "New card" : "Nowa karta",
    "Link to a board" : "Link do tablicy",
    "Link to a card" : "Link do karty",
    "Create a card" : "Utwórz kartę",
    "Message from {author} in {conversationName}" : "Wiadomość od {author} w {conversationName}",
    "Something went wrong" : "Coś poszło nie tak",
    "Failed to upload {name}" : "Nie udało się wysłać {name}",
    "Maximum file size of {size} exceeded" : "Przekroczono maksymalny rozmiar pliku {size}",
    "Assigned users" : "Przypisani użytkownicy",
    "Due date" : "Data realizacji",
    "Created" : "Utworzono",
    "Modified" : "Zmodyfikowany",
    "Error creating the share" : "Błąd podczas tworzenia udostępnienia",
    "Share with a Deck card" : "Udostępnij za pomocą karty Tablicy",
    "Share {file} with a Deck card" : "Udostępnij {file} za pomocą karty Tablicy",
    "Share" : "Udostępnianie",
    "The card \"%s\" on \"%s\" has reached its due date." : "Karta \"%s\" na \"%s\" wygasła.",
    "Share board with a user, group or team …" : "Udostępnij tablicę użytkownikowi, grupie lub zespołowi…",
    "Searching for users, groups and teams …" : "Wyszukiwanie użytkowników, grup i zespołów…",
    "No participants found" : "Nie znaleziono uczestników",
    "Save" : "Zapisz",
    "seconds ago" : "przed chwilą",
    "Today" : "Dzisiaj",
    "Tomorrow" : "Jutro",
    "No due" : "Bez ważności"
},
"nplurals=4; plural=(n==1 ? 0 : (n%10>=2 && n%10<=4) && (n%100<12 || n%100>14) ? 1 : n!=1 && (n%10>=0 && n%10<=1) || (n%10>=5 && n%10<=9) || (n%100>=12 && n%100<=14) ? 2 : 3);");
