OC.L10N.register(
    "deck",
    {
    "Deck" : "Pastki qavat",
    "The file was uploaded" : "Fayl yuklangan edi",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Yuklangan fayl php.ini dagi php-dagi upload_max_filesize direktivasidan oshadi",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Yuklangan fayl HTML shaklida ko'rsatilgan MAX_FILE_SIZE direktivasidan oshadi",
    "The file was only partially uploaded" : "Fayl faqat qisman yuklandi",
    "No file was uploaded" : "Fayl yuklanmadi",
    "Missing a temporary folder" : "Vaqtinchalik papka etishmayapti",
    "Could not write file to disk" : "Faylni diskka yozib bo'lmadi",
    "A PHP extension stopped the file upload" : "PHP kengaytmasi faylni yuklashni to'xtatdi",
    "Upcoming cards" : "Kelgusi kartalar",
    "Deck board" : "Pastki taxta",
    "Later" : "Keyinchalik",
    "copy" : "copy",
    "Done" : "Done",
    "Attachments" : "Qo'shimchalar",
    "File" : "Fayl",
    "Card not found" : "Karta topilmadi",
    "Add board" : "Doska qo'shing",
    "Move card" : "Kartani ko'chirish",
    "Cancel" : "Bekor qilish",
    "Add card" : "Karta qo'shing",
    "Archived cards" : "Arxivlangan kartalar",
    "Add list" : "Roʻyxat qoʻshish",
    "Clear search" : "Qidiruvni tozalash",
    "Unassigned" : "Tayinlanmagan",
    "Open" : "Ochish",
    "Overdue" : "Muddati o'tgan",
    "Next 7 days" : "Keyingi 7 kun",
    "Next 30 days" : "Keyingi 30 kun",
    "No due date" : "Muddati yo'q",
    "Details" : "Details",
    "General" : "Umumiy",
    "Appearance" : "Tashqi ko'rinish",
    "Search" : "Qidirish",
    "Sharing" : "Ulashish",
    "Tags" : "Tags",
    "Activity" : "Faollik",
    "Undo" : "Bekor qilish",
    "Hours" : "Hours",
    "Days" : "Kunlar",
    "No cards yet" : "Hali kartalar yo'q",
    "(remote)" : "(remote)",
    "Can edit" : "Can edit",
    "Owner" : "Owner",
    "Delete" : "O'chirish",
    "Delete list" : "Roʻyxatni oʻchirish",
    "Edit" : "Tahrirlash",
    "Download" : "Yuklab olish",
    "Comments" : "Comments",
    "Submit" : "Yuborish",
    "In reply to" : "ga javoban",
    "Reply" : "Javob bering",
    "Update" : "Update",
    "Mark as done" : "Bajarildi deb belgilang",
    "Description" : "Tavsif",
    "Edit description" : "Tavsifni tahrirlash",
    "Not done" : "Bajarilmadi",
    "Archive card" : "Arxiv kartasi",
    "Edit title" : "Sarlavhani tahrirlash",
    "Delete card" : "Kartani o'chirish",
    "Archived boards" : "Arxivlangan taxtalar",
    "Shared with you" : "Shared with you",
    "Cancel edit" : "Tahrirni bekor qilish",
    "No reminder" : "Eslatma yo'q",
    "Edit board" : "Tahrirlash paneli",
    "Clone board" : "Klon taxtasi",
    "Archive board" : "Arxiv paneli",
    "No notifications" : "Hech qanday bildirishnoma yo'q",
    "Delete board" : "Panelni o'chirish",
    "Clone cards" : "Klon kartalari",
    "Clone" : "Klonlash",
    "Export" : "Eksport",
    "No results found" : "Hech qanday natija topilmadi",
    "Close" : "Yopish",
    "Due date" : "Muddati",
    "Modified" : "Modified",
    "Share" : "Ulashish",
    "Save" : "Saqlash",
    "seconds ago" : "seconds ago",
    "Today" : "Today",
    "Tomorrow" : "Ertaga"
},
"nplurals=1; plural=0;");
