OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Je creëerde een nieuw bord {board}",
    "{user} has created a new board {board}" : "{user} creëerde een nieuw bord {board}",
    "You have deleted the board {board}" : "Je verwijderde bord {board}",
    "{user} has deleted the board {board}" : "{user} verwijderde bord {board}",
    "You have restored the board {board}" : "Je herstelde bord {board}",
    "{user} has restored the board {board}" : "{user} herstelde bord {board}",
    "You have shared the board {board} with {acl}" : "Je deelde bord {board} met {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} heeft een board {board} gedeeld met {acl}",
    "You have removed {acl} from the board {board}" : "Je verwijderde {acl} van bord {board}",
    "{user} has removed {acl} from the board {board}" : "{user} verwijderde {acl} van bord {board}",
    "You have renamed the board {before} to {board}" : "Je hernoemde bord {before} naar {board}",
    "{user} has renamed the board {before} to {board}" : "{user} hernoemde bord {before} naar {board}",
    "You have archived the board {board}" : "Je hebt bord {board} gearchiveerd",
    "{user} has archived the board {before}" : "{user} archiveerde bord {before}",
    "You have unarchived the board {board}" : "Je hebt bord {board} uit het archief opgehaald",
    "{user} has unarchived the board {before}" : "{user} heeft bord {before} uit het archief opgehaald",
    "You have created a new list {stack} on board {board}" : "Je hebt een nieuwe lijst {stack} op bord {board} gemaakt",
    "{user} has created a new list {stack} on board {board}" : "{user} heeft een nieuwe lijst {stack} op bord {board} gemaakt",
    "You have renamed list {before} to {stack} on board {board}" : "Je hebt de naam van lijst {before} gewijzigd in {stack} op bord van {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} heeft de naam van lijst {before} gewijzigd in {stack} op bord van {board}",
    "You have deleted list {stack} on board {board}" : "Je hebt de lijst {stack} op bord {board} verwijderd",
    "{user} has deleted list {stack} on board {board}" : "{user} heeft lijst {stack} aan boord {board} verwijderd",
    "You have created card {card} in list {stack} on board {board}" : "Je hebt de kaart {card} gemaakt in lijst {stack} op bord {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} heeft kaart {card} gemaakt in lijst {stack} op bord {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Je hebt de kaart {card} verwijderd uit lijst {stack} op bord van {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} heeft kaart {card} verwijderd uit lijst {stack} op bord van {board}",
    "You have renamed the card {before} to {card}" : "Je hernoemde kaart {before} naar {card}",
    "{user} has renamed the card {before} to {card}" : "{user} hernoemde kaart {before} naar {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Je hebt een beschrijving toegevoegd aan kaart {card} in lijst {stack} op bord {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} heeft een beschrijving toegevoegd aan kaart {card} in lijst {stack} op bord {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Je hebt de beschrijving van kaart {card} in lijst {stack} op bord {board} bijgewerkt",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} heeft de beschrijving van de kaart {card} in lijst {stack} op bord {board} bijgewerkt",
    "You have archived card {card} in list {stack} on board {board}" : "Je hebt de kaart {card} gearchiveerd in lijst {stack} op bord {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} heeft kaart {card} gearchiveerd in lijst {stack} op bord {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Je hebt de kaart {card} in lijst {stack} op bord {board} uit het archief gehaald",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} heeft kaart {card} uit het archief verwijderd in lijst {stack} op bord {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Je hebt de kaart {card} gemarkeerd als voltooid in lijst {stack} op bord {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} heeft kaart {card} gemarkeerd als voltooid in lijst {stack} op bord {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Je hebt de kaart {card} als ongedaan gemarkeerd in lijst {stack} op bord {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} heeft de kaart {card} als ongedaan gemarkeerd in lijst {stack} op bord {board}",
    "You have removed the due date of card {card}" : "Je verwijderde de vervaldatum van kaart {card}",
    "{user} has removed the due date of card {card}" : "{user} verwijderde de vervaldatum van kaart {card}",
    "You have set the due date of card {card} to {after}" : "Je stelde de vervaldatum van kaart {card} in op {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} stelde de vervaldatum van kaart {card} in op {after}",
    "You have updated the due date of card {card} to {after}" : "Je werkte de vervaldatum van kaart {card} bij tot {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} werkte de vervaldatum van kaart {card} bij tot {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Je hebt de tag {label} toegevoegd aan kaart {card} in lijst {stack} op bord {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} heeft de tag {label} toegevoegd aan kaart {card} in lijst {stack} op bord {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Je hebt de tag {label} verwijderd van kaart {card} in lijst {stack} op bord {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} heeft de tag {label} verwijderd van kaart {card} in lijst {stack} op bord {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Je wees {assigneduser} toe aan kaart {card} op bord {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} wees {assigneduser} toe aan kaart {card} op bord {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Je verwijderd {assigneduser} van kaart {card} op bord {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} verwijderd {assigneduser} van kaart {card} op bord {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Je hebt de kaart {card} verplaatst van lijst {stackBefore} naar {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} heeft de kaart {card} verplaatst van lijst {stackBefore} naar {stack}",
    "You have added the attachment {attachment} to card {card}" : "Je voegde bijlage {attachment} toe aan kaart {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} voegde bijlage {attachment} toe aan kaart {card}",
    "You have updated the attachment {attachment} on card {card}" : "Je werkte bijlage {attachment} bij op kaart {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} werkte bijlage {attachment} bij op kaart {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Je verwijderde bijlage {attachment} van kaart {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} verwijderde bijlage {attachment} van kaart {card}",
    "You have restored the attachment {attachment} to card {card}" : "Je herstelde bijlage {attachment} bij kaart {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} herstelde bijlage {attachment} bij kaart {card}",
    "You have commented on card {card}" : "Je hebt gereageerd op kaart {card}",
    "{user} has commented on card {card}" : "{user} reageerde op kaart {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Wijzigingen in de <strong>Deck app</strong>",
    "A <strong>board, list or card</strong> was changed" : "Een <strong>board, lijst of kaart</strong> is veranderd",
    "A <strong>comment</strong> was created on a card" : "Er is <strong>reactie </strong> gegeven op een kaart",
    "A <strong>card description</strong> has been changed" : "Een <strong>kaartbeschrijving</strong> is gewijzigd.",
    "The file was uploaded" : "Het bestand werd geüpload",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Het geüploade bestand overschrijdt de upload_max_filesize richtlijn in php.ini:",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Het bestand overschrijdt de MAX_FILE_SIZE instelling die is opgegeven in het HTML formulier",
    "The file was only partially uploaded" : "Het bestand is slechts gedeeltelijk geüpload",
    "No file was uploaded" : "Er is geen bestand geüpload",
    "Missing a temporary folder" : "Mist een tijdelijke map",
    "Could not write file to disk" : "Kon bestand niet naar schijf schrijven",
    "A PHP extension stopped the file upload" : "Een PHP-extensie blokkeerde de upload.",
    "No file uploaded or file size exceeds maximum of %s" : "Geen bestand geüpload of de grootte is meer dan het maximum van %s",
    "Invalid file type. Only JSON files are allowed." : "Ongeldig bestandstype. Alleen JSON bestanden zijn ondersteund.",
    "Invalid JSON data" : "Ongeldige JSON data",
    "Failed to import board" : "Importeren van bord mislukt",
    "Cards due today" : "Kaarten die vandaag vervallen",
    "Cards due tomorrow" : "Kaarten die morgen vervallen",
    "Upcoming cards" : "Aanstaande kaarten",
    "Load more" : "Laad meer",
    "Welcome to Nextcloud Deck!" : "Welkom bij Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "De kaart \"%s\" op \"%s\" is aan jou toegewezen door %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} heeft kaart {deck-card} op {deck-board} aan jou toegewezen.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "De kaart \"%1$s\" in \"%2$s\" heeft de vervaldatum bereikt.",
    "The card {deck-card} on {deck-board} has reached its due date." : "De kaart {deck-card} in {deck-board} heeft de vervaldatum bereikt.",
    "%s has mentioned you in a comment on \"%s\"." : "%s vermeldde je in een reactie op \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} vermeldde je in een reactie op {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Het bord \"%s\" is door %s met je gedeeld.",
    "{user} has shared {deck-board} with you." : "{user} heeft het bord {deck-board} met je gedeeld.",
    "The remote board %1$s has been shared with you by %2$s" : "Het remote bord %1$s is met je gedeeld door %2$s",
    "Deck board" : "Deck board",
    "Owned by %1$s" : "Eigendom van %1$s",
    "Deck boards, cards and comments" : "Deck borden, kaarten en reacties",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Van %1$s, in %2$s/%3$s, eigendom van %4$s",
    "Create a new deck card" : "Maak een nieuwe kaart in het deck",
    "Card comments" : "Kaart opmerkingen",
    "%s on %s" : "%s op %s",
    "Deck boards and cards" : "Deck borden en kaarten",
    "No data was provided to create an attachment." : "Geen gegevens opgegeven voor het maken van een bijlage.",
    "Finished" : "Afgerond",
    "To review" : "Beoordelen",
    "Action needed" : "Actie benodigd",
    "Later" : "Later",
    "copy" : "kopiëren",
    "Read more inside" : "Lees meer binnenin",
    "Custom lists - click to rename!" : "Aangepaste lijsten - Klik om te hernoemen!",
    "To Do" : "Te doen",
    "In Progress" : "Bezig",
    "Done" : "Klaar",
    "1. Open to learn more about boards and cards" : "1. Open om meer te leren over borden en kaarten",
    "2. Drag cards left and right, up and down" : "2. Sleep kaarten links en rechts, op en neer",
    "3. Apply rich formatting and link content" : "3. Pas rijke opmaak toe en link inhoud",
    "4. Share, comment and collaborate!" : "4. Deel, reageer en werk samen!",
    "Create your first card!" : "Maak je eerste kaart!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Deze reactie heeft meer dan %s tekens.\nAls bijlage toegevoegd aan de kaart met naam %s.\nToegankelijk op URL: %s.",
    "Attachments" : "Bijlagen",
    "File" : "Bestand",
    "date" : "datum",
    "Card not found" : "Kaart niet gevonden",
    "Path is already shared with this card" : "Pad is al gedeeld met deze kaart",
    "Invalid date, date format must be YYYY-MM-DD" : "Ongeldige datum, formaat is YYYY-MM-DD",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Alle borden die jij bezit, inclusief stapels, kaarten, labels, toewijzingen en reacties",
    "Personal planning and team project organization" : "Persoonlijke planning en inrichting voor teamproject",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck is een soort kanban organisatietool gericht op persoonlijke planning en projectorganisatie voor teams geïntegreerd in Nextcloud.\n\n\n- 📥 Voeg taken toe aan kaarten en sorteer ze\n- 📄 Schrijf extra notities in Markdown\n- 🔖 Ken labels toe voor betere organisatie\n- 👥 Deel met je team, vrienden of familie\n- 📎 Voeg bestanden toe en plaats ze in je Markdown beschrijving\n- 💬 Discussieer binnen je team met reacties\n- ⚡ Volg de wijzigingen in het activiteiten kanaal\n- 🚀 Organiseer je project",
    "Add board" : "Bord toevoegen",
    "Card details" : "Kaart details",
    "Select the board to link to a project" : "Kies het bord om met een project te verbinden",
    "Search by board title" : "Zoeken op bord-titel",
    "Select board" : "Selecteer bord",
    "Move/copy card" : "Verplaats/Kopieer kaart",
    "Select a board" : "Selecteer een bord",
    "No lists available" : "Geen lijsten beschikbaar",
    "Select a list" : "Selecteer een lijst",
    "Move card" : "Kaart verplaatsen",
    "Copy card" : "Kaart kopiëren",
    "Select the card to link to a project" : "Selecteer de kaart om met een project te koppelen",
    "Link to card" : "Koppelen met kaart",
    "Select a card" : "Selecteer een kaart",
    "Cancel" : "Annuleren",
    "This board is read only" : "Dit bord is alleen-lezen",
    "Drop your files to upload" : "Zet je bestanden hier neer om te uploaden",
    "File already exists" : "Bestand bestaat al",
    "A file with the name {filename} already exists." : "Er bestaat al een gebruiker met de naam {filename}.",
    "Do you want to overwrite it?" : "Wil je het overschrijven?",
    "Overwrite file" : "Overschrijven bestand",
    "Keep existing file" : "Bewaar het al bestaande bestand",
    "Add card" : "Kaart toevoegen",
    "Archived cards" : "Gearchiveerde kaarten",
    "Add list" : "Lijst toevoegen",
    "List name" : "Lijstnaam",
    "Clear search" : "Wis zoekvak",
    "Active filters" : "Actieve filters",
    "Apply filter" : "Filter toepassen",
    "Filter by tag" : "Filteren op label",
    "Filter by assigned user" : "Filter op toegewezen gebruiker",
    "Unassigned" : "Niet-toegewezen",
    "Filter by status" : "Filter op status",
    "Open and completed" : "Open en gereed",
    "Open" : "Openen",
    "Completed" : "Voltooid",
    "Filter by due date" : "Filter op vervaldatum",
    "Overdue" : "Vervallen",
    "Next 24 hours" : "Volgende 24 uur",
    "Next 7 days" : "Komende 7 dagen",
    "Next 30 days" : "Komende 30 dagen",
    "No due date" : "Geen vervaldatum",
    "Clear filter" : "Filter leegmaken",
    "View Modes" : "Weergavemodi",
    "Toggle View Modes" : "Wissel Weergavemodi",
    "Kanban view" : "Kanban weergave",
    "Gantt view" : "Gantt weergave",
    "Hide archived cards" : "Gearchiveerde kaarten verbergen",
    "Show archived cards" : "Gearchiveerde kaarten weergeven",
    "Toggle compact mode" : "Omschakelen compacte modus",
    "Hide card cover images" : "Verberg kaartafbeeldingen",
    "Show card cover images" : "Toon afbeeldingen van kaart",
    "Open details" : "Details openen",
    "Details" : "Details",
    "Deck settings" : "Deck instellingen",
    "General" : "Algemeen",
    "Use bigger card view" : "Gebruik een grotere kaartweergave",
    "Appearance" : "Uiterlijk",
    "Show card ID badge" : "Toon je kaart IP badge",
    "Show boards in calendar/tasks" : "Toon borden in agenda/taken",
    "Admin settings" : "Beheerder instellingen",
    "Limit board creation to some groups" : "Beperk het maken van borden tot enkele groepen",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Gebruikers buiten die groepen zullen geen eigen borden kunnen maken, maar ze zullen nog steeds kunnen werken aan borden die met hen zijn gedeeld.",
    "Enable federation" : "Activeer federatie",
    "Board actions" : "Bord acties",
    "Scroll sideways" : "Scroll zijdelings",
    "Navigate between cards" : "Navigeer tussen kaarten",
    "Close card details" : "Sluit kaart details",
    "Search" : "Zoeken",
    "Show card filters" : "Toon kaart filters",
    "Clear card filters" : "Wis kaart filters",
    "Show those shortcuts" : "Toon die snelkoppelingen",
    "Card actions" : "Kaart acties",
    "Open card details" : "Open kaart details",
    "Edit the card title" : "Kaarttitel bewerken",
    "Assign yourself to the current card" : "Wijs jezelf toe aan huidige kaart",
    "Archive/unarchive the current card" : "Archiveer/Dearchiveer de huidige kaart",
    "Mark card as completed/not completed" : "Markeer kaart als afgerond/niet afgerond",
    "Open card menu" : "Open kaart menu",
    "Currently present people" : "Momenteel aanwezige personen",
    "Loading board" : "Bord laden",
    "Board not found" : "Bord niet gevonden",
    "Create a new list to add cards to this board" : "Maak een nieuwe lijst om kaarten aan dit bord toe te voegen",
    "Sharing" : "Delen",
    "Tags" : "Labels",
    "Deleted items" : "Verwijderde objecten",
    "Activity" : "Activiteit",
    "Deleted lists" : "Verwijderde lijsten",
    "Undo" : "Ongedaan maken",
    "Deleted cards" : "Verwijderde kaarten",
    "Hours" : "Uren",
    "Days" : "Dagen",
    "Weeks" : "Weken",
    "Months" : "Maanden",
    "No cards yet" : "Nog geen kaarten",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Stel een startdatum en vervaldatum in op je kaarten om ze te zien op de Gantt weergave",
    "Cards without dates ({count})" : "Kaarten zonder data ({count})",
    "Failed to create share with {displayName}" : "Delen met {displayName} mislukt",
    "Are you sure you want to transfer the board {title} to {user}?" : "Weet je zeker dat je het bord {title} wilt overdragen aan {user}?",
    "Transfer the board." : "Draag het bord over.",
    "Transfer" : "Overdracht",
    "The board has been transferred to {user}" : "Het bord is overgedragen naar {user}",
    "Failed to transfer the board to {user}" : "Niet gelukt om het bord over te dragen aan {user}",
    "Board owner" : "Bordeigenaar",
    "(Group)" : "(groep)",
    "(Team)" : "(Team)",
    "(remote)" : "(remote)",
    "Can edit" : "Kan bewerken",
    "Can share" : "Kan delen",
    "Can manage" : "Kan beheren",
    "Owner" : "Eigenaar",
    "Delete" : "Verwijderen",
    "List deleted" : "Lijst verwijderd",
    "Edit list title" : "Wijzig de titel van de lijst",
    "Archive all cards" : "Alle kaarten archiveren",
    "Unarchive all cards" : "De-archiveer alle kaarten",
    "Do not set cards as \"done\"" : "Markeer kaarten niet als \"voltooid\"",
    "Set cards as \"done\"" : "Markeer kaarten als \"voltooid\"",
    "Delete list" : "Lijst verwijderen",
    "Archive all cards in this list" : "Alle kaarten in deze lijst archiveren",
    "Unarchive all cards in this list" : "Alle kaarten in deze lijst de-archiveren",
    "Add a new card" : "Nieuwe kaart toevoegen",
    "Card name" : "Kaartnaam",
    "title and color value must be provided" : "titel- en kleurwaarde moet gegeven worden",
    "Edit" : "Bewerken",
    "Add a new tag" : "Nieuwe tag toevoegen",
    "Board name" : "Bordnaam",
    "Members" : "Leden",
    "Assign to users/groups/team" : "Wijs toe aan gebruikers/groepen/team",
    "Assign a user to this card…" : "Wijs een gebruiker toe aan deze kaart",
    "Select a user to assign to this card…" : "Selecteer een gebruiker om deze kaart aan toe te wijzen...",
    "File to share" : "Bestand om te delen",
    "Invalid path selected" : "Ongeldig pad geselecteerd",
    "Upload new files" : "Uploaden nieuwe bestanden",
    "Share from Files" : "Delen vanuit Bestanden",
    "Pending share" : "Share in afwachting",
    "Add this attachment" : "Voeg deze bijlage toe",
    "Show in Files" : "Tonen in Bestanden",
    "Download" : "Download",
    "Remove attachment" : "Bijlage verwijderen",
    "Delete Attachment" : "Bijlage verwijderen",
    "Restore Attachment" : "Bijlage herstellen",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Aangepast: {modifiedDate} ⸱ Gemaakt {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Aangepast: {modifiedDate}\nAangemaakt: {createdDate}\nAangemaakt door: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Aangepast: {modifiedDate}\nAangemaakt: {createdDate}",
    "The title cannot be empty." : "De titel kan niet leeg zijn.",
    "Cannot close unsaved card!" : "Kan niet opgeslagen kaart niet sluiten!",
    "Open in sidebar view" : "Open in zijbalk",
    "Open in bigger view" : "Open in grotere weergave",
    "Comments" : "Reacties",
    "Failed to load comments" : "Kon reacties niet laden.",
    "No comments yet. Begin the discussion!" : "Nog geen reacties. Begin de discussie!",
    "The comment cannot be empty." : "De reactie kan niet leeg zijn.",
    "The comment cannot be longer than 1000 characters." : "De reactie kan niet langer dan 1000 tekens zijn.",
    "Submit" : "Indienen",
    "Created:" : "Aangemaakt:",
    "In reply to" : "Als antwoord op",
    "Cancel reply" : "Antwoord annuleren",
    "Reply" : "Antwoord",
    "Update" : "Bijwerken",
    "Assign dependent cards" : "Wijs afhankelijke kaarten toe",
    "Mark as done" : "Markeer als voltooid",
    "Mark as not done" : "Markeren als niet gedaan",
    "Remove dependency" : "Afhankelijkheid verwijderen",
    "Add dependent card" : "Afhankelijke kaart toevoegen",
    "Assign a dependent card…" : "Wijs een afhankelijke kaart toe...",
    "Select a dependent card…" : "Selecteer een afhankelijke kaart...",
    "Write a description …" : "Voeg een beschrijving toe ...",
    "The description has been changed by another user." : "De omschrijving is gewijzigd door een andere gebruiker.",
    "Could not save description" : "Kan beschrijving niet opslaan",
    "Description" : "Beschrijving",
    "(Unsaved)" : "(Niet opgeslagen)",
    "(Saving…)" : "(Opslaan…)",
    "Formatting help" : "Formatteringshulp",
    "Edit description" : "Beschrijving bewerken",
    "View description" : "Bekijk beschrijving",
    "Add Attachment" : "Bijlage toevoegen",
    "Choose attachment" : "Kies bijlage",
    "Select Date" : "Selecteer datum",
    "Later today – {timeLocale}" : "Later vandaag – {timeLocale}",
    "Set due date for later today" : "Stel de vervaldatum in voor later vandaag",
    "Tomorrow – {timeLocale}" : "Morgen - {timeLocale}",
    "Set due date for tomorrow" : "Stel de vervaldatum in voor morgen",
    "This weekend – {timeLocale}" : "Dit weekend - {timeLocale}",
    "Set due date for this weekend" : "Stel de vervaldatum in voor dit weekend",
    "Next week – {timeLocale}" : "Volgende week - {timeLocale}",
    "Set due date for next week" : "Stel de vervaldatum in voor volgende week",
    "Assign a due date to this card…" : "Wijs een vervaldatum toe aan deze kaart...",
    "Set a due date" : "Stel een vervaldatum in",
    "Add due date" : "Vervaldatum toevoegen",
    "Choose a date" : "Kies een datum",
    "Remove due date" : "Vervaldatum verwijderen",
    "Due at:" : "Vervalt op:",
    "Not done" : "Niet gedaan",
    "Unarchive card" : "Terughalen kaart uit archief",
    "Archive card" : "Kaart archiveren",
    "Assign a start date to this card…" : "Wijs een startdatum toe aan deze kaart...",
    "Set a start date" : "Stel startdatum in",
    "Add start date" : "Startdatum toevoegen",
    "Remove start date" : "Startdatum verwijderen",
    "Assign a tag to this card…" : "Wijs een label toe aan deze kaart...",
    "Select or create a tag…" : "Selecteer of maak een tag...",
    "Create a new tag:" : "Maak een nieuwe tag aan:",
    "(group)" : "(groep)",
    "{count} comments, {unread} unread" : "{count} reacties, {unread} ongelezen",
    "Todo items" : "Te doen onderwerpen",
    "Edit card title" : "Wijzig de titel van de kaart",
    "Open link" : "Open link",
    "Card deleted" : "Kaart verwijderd",
    "Change card color" : "Verander kaartkleur",
    "Edit title" : "Titel bewerken",
    "Assign to me" : "Aan mij toewijzen",
    "Unassign myself" : "Eigen toewijzing opheffen",
    "Delete card" : "Kaart verwijderen",
    "All boards" : "Alle borden",
    "Archived boards" : "Gearchiveerde borden",
    "Shared with you" : "Deelde met jou",
    "Cancel edit" : "Bewerking annuleren",
    "Save board" : "Bord opslaan",
    "Board {0} deleted" : "Bord {0} verwijderd",
    "All cards" : "Alle kaarten",
    "Only assigned cards" : "Uitsluitend toegewezen kaarten",
    "No reminder" : "Geen herinnering",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Bent u zeker dat u het bord {title} wilt verwijderen? Dit zal alle gegevens van dit bord verwijderen, inclusief gearchiveerde kaarten.",
    "Delete the board?" : "Bord verwijderen?",
    "Are you sure you want to leave the board {title}?" : "Weet je zeker dat je bord {title} wilt verlaten?",
    "Leave the board?" : "Verlaat het bord?",
    "Leave" : "Verlaat",
    "Failed to leave the board" : "Bord verlaten mislukt",
    "Exporting board..." : "Bord wordt geëxporteerd...",
    "An error occurred" : "Er is een fout opgetreden",
    "Board details" : "Bord details",
    "Edit board" : "Bord bewerken",
    "Clone board" : "Dupliceer bord",
    "Unarchive board" : "De-archiveer bord",
    "Archive board" : "Bord archiveren",
    "Export board" : "Exporteer het bord",
    "Turn on due date reminders" : "Herinneringen voor vervaldatums aanzetten",
    "Turn off due date reminders" : "Herinneringen voor vervaldatums uitzetten",
    "Remove as default board" : "Verwijder als standaardbord",
    "Set as default board" : "Stel in als standaardbord",
    "Due date reminders" : "Herinneringen voor vervaldatums",
    "Assigned cards" : "Toegewezen kaarten",
    "No notifications" : "Geen meldingen",
    "Delete board" : "Bord verwijderen",
    "Leave board" : "Verlaat bord",
    "Importing board..." : "Bord word geïmporteerd...",
    "Board imported successfully" : "Bord succesvol geïmporteerd",
    "Import board" : "Importeer bord",
    "Clone {boardTitle}" : "Kloon {boardTitle}",
    "Clone cards" : "Kaarten klonen",
    "Clone assignments" : "Kloon toewijzingen",
    "Clone labels" : "Kloon labels",
    "Clone due dates" : "Kloon vervaldatum",
    "Advanced options" : "Geavanceerde opties",
    "Move all cards to the first list" : "Verplaats alle kaarten naar de eerste lijst",
    "Restore archived cards" : "Herstel gearchiveerde kaarten",
    "Clone" : "Klonen",
    "Export {boardTitle}" : "Export {boardTitle}",
    "Export as JSON" : "Exporteer als JSON",
    "Export as CSV" : "Exporteer als CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Let op: Alleen het JSON formaat is ondersteund om terug te importeren naar de Deck app.",
    "Export" : "Exporteren",
    "Loading filtered view" : "Gefilterde weergave wordt geladen",
    "Search for {searchQuery} in other boards" : "Zoek naar {searchQuery} op andere borden",
    "Search for {searchQuery} in all boards" : "Zoek naar {searchQuery} in alle borden",
    "No results found" : "Geen resultaten gevonden",
    "Deck board {name}\n* Last modified on {lastMod}" : "Deckplank {name}\n* Laatst gewijzigd op {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Aangemaakt op {created}\n* Laatst gewijzigd op {lastMod}\n* {nbAttachments} bijlagen\n* {nbComments} reacties",
    "{nbCards} cards" : "{nbCards} kaarten",
    "Due on {date}" : "Vervalt op {date}",
    "{stack} in {board}" : "{stack} in {board}",
    "Click to expand description" : "Klik om de beschrijving uit te vouwen",
    "Click to expand comment" : "Klik om de reactie uit te vouwen",
    "Create card" : "Kaart aanmaken",
    "Create a new card" : "Maak een nieuwe kaart aan",
    "Card title" : "Kaarttitel",
    "Creating the new card …" : "Aanmaken nieuwe kaart...",
    "Card \"{card}\" was added to \"{board}\"" : "Kaart \"{card}\" is toegevoegd aan \"{board}\"",
    "Open card" : "Open kaart",
    "Close" : "Sluiten",
    "No upcoming cards" : "Geen aanstaande kaarten",
    "upcoming cards today" : "aankomende kaarten vandaag",
    "upcoming cards tomorrow" : "aanstaande kaarten morgen",
    "upcoming cards" : "Aanstaande kaarten",
    "New card" : "Nieuwe kaart",
    "Link to a board" : "Verbind met een bord",
    "Link to a card" : "Koppel met een kaart",
    "Create a card" : "Maak een kaart aan",
    "Message from {author} in {conversationName}" : "Bericht van {author} in {conversationName}",
    "Something went wrong" : "Er ging iets verkeerd",
    "Failed to upload {name}" : "Kon {name} niet uploaden",
    "Maximum file size of {size} exceeded" : "Maximale bestandsomvang van {size} overschreden",
    "Assigned users" : "Toegewezen gebruikers",
    "Due date" : "Vervaldatum",
    "Created" : "Gemaakt",
    "Modified" : "Aangepast",
    "Error creating the share" : "Fout bij aanmaken share",
    "Share with a Deck card" : "Delen met een Deck kaart",
    "Share {file} with a Deck card" : "Delen {file} met een Deck kaart",
    "Share" : "Delen",
    "The card \"%s\" on \"%s\" has reached its due date." : "De kaart \"%s\" in \"%s\" heeft zijn vervaldatum bereikt.",
    "Share board with a user, group or team …" : "Deel bord met een gebruiker, groep of team...",
    "Searching for users, groups and teams …" : "Bezig met zoeken naar gebruikers, groepen en teams...",
    "No participants found" : "Geen deelnemers gevonden",
    "Save" : "Opslaan",
    "seconds ago" : "seconden geleden",
    "Today" : "Vandaag",
    "Tomorrow" : "Morgen",
    "No due" : "Geen vervaldatum"
},
"nplurals=2; plural=(n != 1);");
