OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "יצרת לוח חדש {board}",
    "{user} has created a new board {board}" : "נוצר לוח חדש {board} על ידי {user}",
    "You have deleted the board {board}" : "מחקת את הלוח {board}",
    "{user} has deleted the board {board}" : "הלוח {board} נמחק על ידי {user}",
    "You have restored the board {board}" : "שחזרת את הלוח {board}",
    "{user} has restored the board {board}" : "הלוח {board} שוחזר על ידי {user}",
    "You have shared the board {board} with {acl}" : "שיתפת את הלוח {board} עם {acl}",
    "{user} has shared the board {board} with {acl}" : "הלוח {board} שותף עם {acl} על ידי {user}",
    "You have removed {acl} from the board {board}" : "הסרת את {acl} מהלוח {board}",
    "{user} has removed {acl} from the board {board}" : "בוצעה הסרה של {acl} מהלוח {board} על ידי {user}",
    "You have renamed the board {before} to {board}" : "שינית את שם הלוח {before} לשם {board}",
    "{user} has renamed the board {before} to {board}" : "השם של הלוח {before} השתנה לשם {board} על ידי {user}",
    "You have archived the board {board}" : "העברת את הלוח {board} לארכיון",
    "{user} has archived the board {before}" : "הארכיון {before} הועבר לארכיון על ידי {user}",
    "You have unarchived the board {board}" : "הוצאת את הלוח {board} מהארכיון",
    "{user} has unarchived the board {before}" : "הלוח {before} הוצא מהארכיון על ידי {user}",
    "You have created a new list {stack} on board {board}" : "יצרת רשימה חדשה {stack} על הלוח {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} יצר רשימה חדשה {stack} על הלוח {board}",
    "You have renamed list {before} to {stack} on board {board}" : "שינית את שם הרשימה {לפני} ל- {stack} על הלוח {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} שינה את שם הרשימה {לפני} ל- {stack} על הלוח {board}",
    "You have deleted list {stack} on board {board}" : "מחקת את הרשימה {stack} על הלוח {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} מחק את הרשימה {stack} על הלוח {board}",
    "You have created card {card} in list {stack} on board {board}" : "יצרת כרטיס {card} ברשימה {stack} על הלוח {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} יצר כרטיס {card} ברשימה {stack} על הלוח {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "מחקת את הכרטיס {card} ברשימה {stack} על הלוח {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} מחק את הכרטיס {card} ברשימה {stack} על הלוח {board}",
    "You have renamed the card {before} to {card}" : "שינית את שם הכרטיס {before} ל- {card}",
    "{user} has renamed the card {before} to {card}" : "{user} שינה את שם הכרטיס {לפני} ל- {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "הוספת תיאור לכרטיס {card} ברשימה {stack} על הלוח {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} הוסיף תיאור לכרטיס {card} ברשימה {stack} על הלוח {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "עדכנת את תיאור הכרטיס {card} ברשימה {stack} על הלוח {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} עדכן את תיאור הכרטיס {card} ברשימה {stack} על הלוח {board}",
    "You have archived card {card} in list {stack} on board {board}" : "שמרת כרטיס {card} בארכיון ברשימה {stack} על הלוח {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} ארכיב כרטיס {card} ברשימה {stack} על הלוח {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "יש לך כרטיס {card} שלא הועבר לארכיון ברשימה {stack} על הלוח {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "ל-{user} יש כרטיס {card} שלא הועבר לארכיון ברשימה {stack} על הלוח {board}",
    "You have removed the due date of card {card}" : "הסרת את מועד היעד מהכרטיס {card}",
    "{user} has removed the due date of card {card}" : "מועד היעד של הכרטיס {card} הוסר על ידי {user}",
    "You have set the due date of card {card} to {after}" : "הגדרת את תאריך היעד של הכרטיס {card} ל- {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} הגדיר את תאריך היעד של הכרטיס {card} ל- {after}",
    "You have updated the due date of card {card} to {after}" : "עדכנת את תאריך היעד של הכרטיס {card} ל- {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} עדכן את תאריך היעד של הכרטיס {card} ל- {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "הוספת את התג {label} לכרטיס {card} ברשימה {stack} על הלוח {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} הוסיף את התג {label} לכרטיס {card} ברשימה {stack} על הלוח {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "הסרת את התג {label} מכרטיס {card} ברשימה {stack} על הלוח {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} הסיר את התג {label} מכרטיס {card} ברשימה {stack} על הלוח {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "הקצית {assigneduser} לכרטיס {card} על הלוח {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} הקצה את {assigneduser} לכרטיס {card} על הלוח {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "בטלת את הקצאת {assigneduser} מכרטיס {card} על הלוח {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} ביטל את ההקצאה של {assigneduser} מכרטיס {card} על הלוח {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "העברת את הכרטיס {card} מהרשימה {stackBefore} ל- {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} העביר את הכרטיס {card} מהרשימה {stackBefore} ל- {stack}",
    "You have added the attachment {attachment} to card {card}" : "הוספת את הקובץ המצורף {attachment} לכרטיס {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} הוסיף את הקובץ המצורף {attachment} לכרטיס {card}",
    "You have updated the attachment {attachment} on card {card}" : "עדכנת את הקובץ המצורף {attachment} בכרטיס {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} עדכן את הקובץ המצורף {attachment} בכרטיס {card}",
    "You have deleted the attachment {attachment} from card {card}" : "מחקת את הקובץ המצורף {attachment} מכרטיס {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} מחק את הקובץ המצורף {attachment} מכרטיס {card}",
    "You have restored the attachment {attachment} to card {card}" : "שחזרת את הקובץ המצורף {attachment} לכרטיס {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} שיחזר את הקובץ המצורף {attachment} לכרטיס {card}",
    "You have commented on card {card}" : "הגבת על הכרטיס {cart}",
    "{user} has commented on card {card}" : "נוספה תגובה מאת {user} על הכרטיס {card}",
    "Deck" : "חפיסה",
    "Changes in the <strong>Deck app</strong>" : "שינויים ל<strong>יישומון החבילה</strong>",
    "A <strong>board, list or card</strong> was changed" : "לוח, רשימה או כרטיס שונה",
    "A <strong>comment</strong> was created on a card" : "נוצרה <strong>הערה</strong> על כרטיס",
    "A <strong>card description</strong> has been changed" : "תיאור הכרטיס שונה",
    "The file was uploaded" : "הקובץ הועלה",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "הקובץ שהועלה חורג מההנחיה upload_max_filesize ב־php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "הקובץ שהועלה חורג מההנחיה MAX_FILE_SIZE שצוינה בטופס ה־HTML",
    "The file was only partially uploaded" : "הקובץ הועלה באופן חלקי בלבד",
    "No file was uploaded" : "לא הועלה קובץ",
    "Missing a temporary folder" : "חסרה תיקייה זמנית",
    "Could not write file to disk" : "לא ניתן לכתוב לכונן",
    "A PHP extension stopped the file upload" : "הרחבת PHP עצרה את העלאת הקובץ",
    "No file uploaded or file size exceeds maximum of %s" : "לא הועלה אף קובץ או שגודל הקובץ חרג מהסף המרבי של %s",
    "Upcoming cards" : "כרטיסים עתידיים",
    "Load more" : "לטעון עוד",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "הכרטיס \"%s\" שב־„%s” הוקצה אליך על ידי %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} has assigned the card {deck-card} on {deck-board} to you.",
    "%s has mentioned you in a comment on \"%s\"." : "אוזכרת על ידי %s בתגובה על „%s”.",
    "The board \"%s\" has been shared with you by %s." : "הלוח „%s” שותף אתך על ידי %s.",
    "Deck board" : "לוח חסיפה",
    "No data was provided to create an attachment." : "לא סופקו נתונים ליצירת קובץ מצורף",
    "Finished" : "הסתיים",
    "To review" : "לסקירה",
    "Action needed" : "נדרשת פעולה",
    "Later" : "מאוחר יותר",
    "copy" : "עותק",
    "To Do" : "מטלות",
    "Done" : "הסתיים",
    "Attachments" : "קבצים מצורפים",
    "File" : "קובץ",
    "Card not found" : "כרטיס לא נמצא",
    "Path is already shared with this card" : "הנתיב כבר משותף עם הכרטיס הזה",
    "Invalid date, date format must be YYYY-MM-DD" : "תאריך שגוי, תבנית התאריך חייבת להיות YYYY-MM-DD",
    "Personal planning and team project organization" : "ארגון אישי וקבוצתי של מיזמים",
    "Add board" : "הוספת לוח",
    "Card details" : "פרטי הכרטיס",
    "Select the board to link to a project" : "נא לבחור את הלוח לקישור למיזם",
    "Search by board title" : "חיפוש לפי כותרת לו",
    "Select board" : "בחירת לוח",
    "Select a board" : "נא לבחור לוח",
    "No lists available" : "אין רשימות זמינות",
    "Select a list" : "בחר רשימה",
    "Move card" : "העברת כרטיס",
    "Select the card to link to a project" : "נא לבחור את הכרטיס לקישור למיזם",
    "Link to card" : "קישור לכרטיס",
    "Select a card" : "נא לבחור כרטיס",
    "Cancel" : "ביטול",
    "This board is read only" : "הלוח הזה הוא לקריאה בלבד",
    "Drop your files to upload" : "יש לגרור לכאן קבצים כדי להעלות",
    "File already exists" : "הקובץ כבר קיים",
    "A file with the name {filename} already exists." : "כבר קיים קובץ עם השם {filename}.",
    "Do you want to overwrite it?" : "לשכתב עליו?",
    "Overwrite file" : "דריסת קובץ",
    "Keep existing file" : "לשמור על הקובץ הקיים",
    "Add card" : "הוספת כרטיס",
    "Archived cards" : "כרטיסים בארכיון",
    "Add list" : "הוספת רשימה",
    "List name" : "שם הרשימה",
    "Apply filter" : "החלת מסנן",
    "Filter by tag" : "סינון לפי תגית",
    "Filter by assigned user" : "סינון לפי משתמש מוקצה",
    "Unassigned" : "לא הוקצה",
    "Open" : "פתוח",
    "Completed" : "הושלם",
    "Filter by due date" : "סינון לפי תאריך יעד",
    "Overdue" : "באיחור",
    "Next 24 hours" : "ב־24 השעות הבאות",
    "Next 7 days" : "ב־7 הימים הבאים",
    "Next 30 days" : "ב־30 הימים הבאים",
    "No due date" : "ללא תאריך יעד",
    "Clear filter" : "מחיקת מסנן",
    "Hide archived cards" : "הסתרת כרטיסים בארכיון",
    "Show archived cards" : "הצגת כרטיסים בארכיון",
    "Toggle compact mode" : "החלפת מצב חסכוני",
    "Open details" : "פתיחת פרטים",
    "Details" : "פרטים",
    "General" : "כללי",
    "Use bigger card view" : "להשתמש בתצוגת כרטיסים גדולה יותר",
    "Appearance" : "מראה",
    "Show boards in calendar/tasks" : "הצג הלוחות בלוח השנה/המשימות",
    "Search" : "חיפוש",
    "Loading board" : "הלוח נטען",
    "Board not found" : "הלוח לא נמצא",
    "Create a new list to add cards to this board" : "צור רשימה חדשה כדי להוסיף כרטיסים ללוח זה",
    "Sharing" : "שיתוף",
    "Tags" : "תגיות",
    "Deleted items" : "פריטים שנמחקו",
    "Activity" : "פעילות",
    "Deleted lists" : "רשימות שנמחקו",
    "Undo" : "ביטול",
    "Deleted cards" : "כרטיסים שנמחקו",
    "Hours" : "שעות",
    "Days" : "ימים",
    "Weeks" : "שבועות",
    "Months" : "חודשים",
    "No cards yet" : "אין כרטיסים עדיין",
    "Failed to create share with {displayName}" : "יצירת השיתוף עם {displayName} נכשלה",
    "Transfer" : "העברה",
    "Board owner" : "בעלות על הלוח",
    "(Group)" : "(קבוצה)",
    "Can edit" : "ניתן לערוך",
    "Can share" : "Can share",
    "Can manage" : "הרשאת ניהול",
    "Owner" : "בעלות",
    "Delete" : "מחיקה",
    "List deleted" : "הרשימה נמחקה",
    "Archive all cards" : "ארכיב את כל הכרטיסים",
    "Delete list" : "מחיקת רשימה",
    "Archive all cards in this list" : "ארכיב את כל הכרטיסים ברשימה זו",
    "Add a new card" : "הוספת כרטיס חדש",
    "Card name" : "שם כרטיס",
    "title and color value must be provided" : "יש לספק כותרת וערך צבע",
    "Edit" : "עריכה",
    "Add a new tag" : "הוספת תגית חדשה",
    "Board name" : "שם הלוח",
    "Members" : "חברים",
    "Assign a user to this card…" : "הקצאת משתמש לכרטיס זה…",
    "File to share" : "קובץ לשיתוף",
    "Invalid path selected" : "הנתיב שנבחר שגוי",
    "Upload new files" : "העלאת קבצים חדשים",
    "Share from Files" : "שיתוף מקבצים",
    "Add this attachment" : "הוספת קובץ מצורף זה",
    "Show in Files" : "הצגה בקבצים",
    "Download" : "הורדה",
    "Delete Attachment" : "מחיקת קובץ מצורף",
    "Restore Attachment" : "שחזור קובץ מצורף",
    "The title cannot be empty." : "הכותרת לא יכולה להיות ריקה.",
    "Open in sidebar view" : "פתח בתצוגת סרגל הצד",
    "Open in bigger view" : "פתח בתצוגה גדולה יותר",
    "Comments" : "תגובות",
    "No comments yet. Begin the discussion!" : "אין עדיין הערות. אפשר להתחיל לדון!",
    "The comment cannot be empty." : "ההערה לא יכולה להיות ריקה.",
    "The comment cannot be longer than 1000 characters." : "אורך ההערה לא יכול לחצות את רף 1000 התווים.",
    "Submit" : "שליחה",
    "Created:" : "מועד היצירה:",
    "In reply to" : "בתגובה אל",
    "Reply" : "תגובה",
    "Update" : "עדכון",
    "Write a description …" : "כתוב תיאור ...",
    "Description" : "תיאור",
    "(Unsaved)" : "(לא נשמר)",
    "(Saving…)" : "(מתבצעת שמירה…)",
    "Formatting help" : "עזרה בסידור בתבנית",
    "Edit description" : "עריכת תיאור",
    "View description" : "הצגת תיאור",
    "Add Attachment" : "הוספת קובץ מצורף",
    "Choose attachment" : "בחירת קובץ מצורף",
    "Select Date" : "בחירת תאריך",
    "Set a due date" : "הגדרת תאריך יעד",
    "Remove due date" : "הסרת מועד התפוגה",
    "Unarchive card" : "הוצאת הכרטיס מהארכיון",
    "Archive card" : "העברת כרטיס לארכיון",
    "Assign a tag to this card…" : "הקצאת תגית לכרטיס זה…",
    "(group)" : "(קבוצה)",
    "Card deleted" : "הכרטיס נמחק",
    "Edit title" : "עריכת כותרת",
    "Assign to me" : "הקצאה אלי",
    "Unassign myself" : "לבטל את הקצאת עצמי",
    "Delete card" : "מחיקת כרטיס לארכיון",
    "All boards" : "כל הלוחות",
    "Archived boards" : "לוחות שנשמרו בארכיון",
    "Shared with you" : "משותף אתך",
    "Cancel edit" : "בטל עריכה",
    "Board {0} deleted" : "הלוח {0} נמחק",
    "All cards" : "כל הכרטיסים",
    "Only assigned cards" : "רק כרטיסים שהוקצו",
    "No reminder" : "אין תזכורת",
    "Delete the board?" : "למחוק את הלוח הזה?",
    "An error occurred" : "אירעה שגיאה",
    "Board details" : "פרטי לוח",
    "Edit board" : "עריכת לוח",
    "Clone board" : "שכפל את הלוח",
    "Unarchive board" : "בטל ארכיון של הלוח",
    "Archive board" : "העבר את הלוח לארכיון",
    "Turn on due date reminders" : "הפעל תזכורות לתאריך היעד",
    "Turn off due date reminders" : "השבת תזכורות לתאריך היעד",
    "Due date reminders" : "תזכורות לתאריך יעד",
    "Assigned cards" : "כרטיסים שהוקצו",
    "No notifications" : "אין התראות",
    "Delete board" : "מחק לוח",
    "Advanced options" : "אפשרויות מתקדמות",
    "Export" : "ייצוא",
    "Loading filtered view" : "טוען תצוגה מסוננת",
    "No results found" : "לא נמצאו תוצאות",
    "Create card" : "יצירת כרטיס",
    "Close" : "סגירה",
    "No upcoming cards" : "אין כרטיסים עתידיים",
    "upcoming cards" : "כרטיסים עתידיים",
    "Link to a board" : "קישור ללוח",
    "Link to a card" : "קישור לכרטיס",
    "Something went wrong" : "משהו השתבש",
    "Failed to upload {name}" : "העלאת {name} נכשלה",
    "Maximum file size of {size} exceeded" : "גודל הקבצים המרבי {size} הושג",
    "Due date" : "מועד יעד",
    "Created" : "מועד היצירה",
    "Modified" : "מועד שינוי",
    "Error creating the share" : "שגיאה ביצירת השיתוף",
    "Share with a Deck card" : "שיתוף עם כרטיס חפיסה",
    "Share {file} with a Deck card" : "שיתוף {file} עם כרטיס חפיסה",
    "Share" : "שיתוף",
    "The card \"%s\" on \"%s\" has reached its due date." : "הכרטיס „%s” שתחת „%s” הגיע למועד התפוגה שלו.",
    "No participants found" : "לא נמצאו משתתפים",
    "Save" : "שמור",
    "seconds ago" : "לפני מספר שניות",
    "Today" : "היום",
    "Tomorrow" : "מחר",
    "No due" : "אין תאריך יעד"
},
"nplurals=3; plural=(n == 1 && n % 1 == 0) ? 0 : (n == 2 && n % 1 == 0) ? 1: 2;");
