<?php // phpcs:ignoreFile

return array (
  'added' => 
  array (
    'argumentcounterror::__clone' => 
    array (
      0 => 'void',
    ),
    'argumentcounterror::__construct' => 
    array (
      0 => 'void',
      'message=' => 'string',
      'code=' => 'int',
      'previous=' => 'Throwable|null',
    ),
    'argumentcounterror::__tostring' => 
    array (
      0 => 'string',
    ),
    'argumentcounterror::__wakeup' => 
    array (
      0 => 'void',
    ),
    'argumentcounterror::getcode' => 
    array (
      0 => 'int',
    ),
    'argumentcounterror::getfile' => 
    array (
      0 => 'string',
    ),
    'argumentcounterror::getline' => 
    array (
      0 => 'int',
    ),
    'argumentcounterror::getmessage' => 
    array (
      0 => 'string',
    ),
    'argumentcounterror::getprevious' => 
    array (
      0 => 'Throwable|null',
    ),
    'argumentcounterror::gettrace' => 
    array (
      0 => 'list<array{args?: array<array-key, mixed>, class?: class-string, file?: string, function: string, line?: int, type?: \'->\'|\'::\'}>',
    ),
    'argumentcounterror::gettraceasstring' => 
    array (
      0 => 'string',
    ),
    'chroot' => 
    array (
      0 => 'bool',
      'directory' => 'string',
    ),
    'closure::fromcallable' => 
    array (
      0 => 'Closure',
      'callable' => 'callable',
    ),
    'curl_multi_errno' => 
    array (
      0 => 'false|int',
      'mh' => 'resource',
    ),
    'curl_share_errno' => 
    array (
      0 => 'false|int',
      'sh' => 'resource',
    ),
    'curl_share_strerror' => 
    array (
      0 => 'null|string',
      'errornum' => 'int',
    ),
    'getenv\'1' => 
    array (
      0 => 'array<string, string>',
    ),
    'hash_hkdf' => 
    array (
      0 => 'false|non-empty-string',
      'ikm' => 'string',
      'algo' => 'string',
      'length=' => 'int',
      'string=' => 'string',
      'salt=' => 'string',
    ),
    'intltimezone::getidforwindowsid' => 
    array (
      0 => 'false|string',
      'timezone' => 'string',
      'region=' => 'string',
    ),
    'intltimezone::getwindowsid' => 
    array (
      0 => 'false|string',
      'timezone' => 'string',
    ),
    'is_iterable' => 
    array (
      0 => 'bool',
      'var' => 'mixed',
    ),
    'mongodb\\driver\\manager::addsubscriber' => 
    array (
      0 => 'void',
      'subscriber' => 'MongoDB\\Driver\\Monitoring\\Subscriber',
    ),
    'mongodb\\driver\\manager::removesubscriber' => 
    array (
      0 => 'void',
      'subscriber' => 'MongoDB\\Driver\\Monitoring\\Subscriber',
    ),
    'mongodb\\driver\\monitoring\\commandfailedevent::getserviceid' => 
    array (
      0 => 'MongoDB\\BSON\\ObjectId|null',
    ),
    'mongodb\\driver\\monitoring\\commandstartedevent::getserviceid' => 
    array (
      0 => 'MongoDB\\BSON\\ObjectId|null',
    ),
    'mongodb\\driver\\monitoring\\commandsucceededevent::getserviceid' => 
    array (
      0 => 'MongoDB\\BSON\\ObjectId|null',
    ),
    'mongodb\\driver\\serverapi::bsonserialize' => 
    array (
      0 => 'stdClass',
    ),
    'mongodb\\driver\\serverapi::serialize' => 
    array (
      0 => 'string',
    ),
    'mongodb\\driver\\serverapi::unserialize' => 
    array (
      0 => 'void',
      'serialized' => 'string',
    ),
    'openssl_get_curve_names' => 
    array (
      0 => 'list<string>',
    ),
    'pcntl_async_signals' => 
    array (
      0 => 'bool',
      'on' => 'bool',
    ),
    'pcntl_signal_get_handler' => 
    array (
      0 => 'int|string',
      'signo' => 'int',
    ),
    'reflectionclass::getreflectionconstant' => 
    array (
      0 => 'ReflectionClassConstant|false',
      'name' => 'string',
    ),
    'reflectionclass::getreflectionconstants' => 
    array (
      0 => 'list<ReflectionClassConstant>',
    ),
    'reflectionclassconstant::__construct' => 
    array (
      0 => 'void',
      'class' => 'class-string|object',
      'name' => 'string',
    ),
    'reflectionclassconstant::__tostring' => 
    array (
      0 => 'string',
    ),
    'reflectionclassconstant::export' => 
    array (
      0 => 'string',
      'class' => 'mixed',
      'name' => 'string',
      'return=' => 'bool',
    ),
    'reflectionclassconstant::getdeclaringclass' => 
    array (
      0 => 'ReflectionClass',
    ),
    'reflectionclassconstant::getdoccomment' => 
    array (
      0 => 'false|string',
    ),
    'reflectionclassconstant::getmodifiers' => 
    array (
      0 => 'int',
    ),
    'reflectionclassconstant::getname' => 
    array (
      0 => 'string',
    ),
    'reflectionclassconstant::getvalue' => 
    array (
      0 => 'array<array-key, scalar>|null|scalar',
    ),
    'reflectionclassconstant::isprivate' => 
    array (
      0 => 'bool',
    ),
    'reflectionclassconstant::isprotected' => 
    array (
      0 => 'bool',
    ),
    'reflectionclassconstant::ispublic' => 
    array (
      0 => 'bool',
    ),
    'reflectionnamedtype::__clone' => 
    array (
      0 => 'void',
    ),
    'reflectionnamedtype::__tostring' => 
    array (
      0 => 'string',
    ),
    'reflectionnamedtype::allowsnull' => 
    array (
      0 => 'bool',
    ),
    'reflectionnamedtype::getname' => 
    array (
      0 => 'string',
    ),
    'reflectionnamedtype::isbuiltin' => 
    array (
      0 => 'bool',
    ),
    'reflectionobject::getreflectionconstant' => 
    array (
      0 => 'ReflectionClassConstant',
      'name' => 'string',
    ),
    'reflectionobject::getreflectionconstants' => 
    array (
      0 => 'list<ReflectionClassConstant>',
    ),
    'sapi_windows_cp_conv' => 
    array (
      0 => 'null|string',
      'in_codepage' => 'int|string',
      'out_codepage' => 'int|string',
      'subject' => 'string',
    ),
    'sapi_windows_cp_get' => 
    array (
      0 => 'int',
      'kind=' => 'string',
    ),
    'sapi_windows_cp_is_utf8' => 
    array (
      0 => 'bool',
    ),
    'sapi_windows_cp_set' => 
    array (
      0 => 'bool',
      'codepage' => 'int',
    ),
    'session_create_id' => 
    array (
      0 => 'false|string',
      'prefix=' => 'string',
    ),
    'session_gc' => 
    array (
      0 => 'false|int',
    ),
  ),
  'changed' => 
  array (
    'array_merge' => 
    array (
      'old' => 
      array (
        0 => 'array<array-key, mixed>',
        'arr1' => 'array<array-key, mixed>',
        '...arrays' => 'array<array-key, mixed>',
      ),
      'new' => 
      array (
        0 => 'array<array-key, mixed>',
        'arr1' => 'array<array-key, mixed>',
        '...arrays=' => 'array<array-key, mixed>',
      ),
    ),
    'array_merge_recursive' => 
    array (
      'old' => 
      array (
        0 => 'array<array-key, mixed>',
        'arr1' => 'array<array-key, mixed>',
        '...arrays' => 'array<array-key, mixed>',
      ),
      'new' => 
      array (
        0 => 'array<array-key, mixed>',
        'arr1' => 'array<array-key, mixed>',
        '...arrays=' => 'array<array-key, mixed>',
      ),
    ),
    'array_replace' => 
    array (
      'old' => 
      array (
        0 => 'array<array-key, mixed>',
        'arr1' => 'array<array-key, mixed>',
        '...arrays' => 'array<array-key, mixed>',
      ),
      'new' => 
      array (
        0 => 'array<array-key, mixed>',
        'arr1' => 'array<array-key, mixed>',
        '...arrays=' => 'array<array-key, mixed>',
      ),
    ),
    'array_replace_recursive' => 
    array (
      'old' => 
      array (
        0 => 'array<array-key, mixed>',
        'arr1' => 'array<array-key, mixed>',
        '...arrays' => 'array<array-key, mixed>',
      ),
      'new' => 
      array (
        0 => 'array<array-key, mixed>',
        'arr1' => 'array<array-key, mixed>',
        '...arrays=' => 'array<array-key, mixed>',
      ),
    ),
    'arrayiterator::__construct' => 
    array (
      'old' => 
      array (
        0 => 'void',
        'array=' => 'array<array-key, mixed>|object',
        'ar_flags=' => 'int',
        'iterator_class=' => 'mixed',
      ),
      'new' => 
      array (
        0 => 'void',
        'input=' => 'array<array-key, mixed>|object',
        'flags=' => 'int',
        'iterator_class=' => 'mixed',
      ),
    ),
    'arrayobject::__construct' => 
    array (
      'old' => 
      array (
        0 => 'void',
        'array=' => 'array<array-key, mixed>|object',
        'ar_flags=' => 'int',
        'iterator_class=' => 'class-string',
      ),
      'new' => 
      array (
        0 => 'void',
        'input=' => 'array<array-key, mixed>|object',
        'flags=' => 'int',
        'iterator_class=' => 'class-string',
      ),
    ),
    'date_create' => 
    array (
      'old' => 
      array (
        0 => 'DateTime|false',
        'time=' => 'string',
        'object=' => 'DateTimeZone|null',
      ),
      'new' => 
      array (
        0 => 'DateTime|false',
        'time=' => 'string',
        'timezone=' => 'DateTimeZone|null',
      ),
    ),
    'date_create_immutable' => 
    array (
      'old' => 
      array (
        0 => 'DateTimeImmutable|false',
        'time=' => 'string',
        'object=' => 'DateTimeZone|null',
      ),
      'new' => 
      array (
        0 => 'DateTimeImmutable|false',
        'time=' => 'string',
        'timezone=' => 'DateTimeZone|null',
      ),
    ),
    'datetime::__construct' => 
    array (
      'old' => 
      array (
        0 => 'void',
        'time=' => 'string',
        'object=' => 'mixed',
      ),
      'new' => 
      array (
        0 => 'void',
        'time=' => 'string',
        'timezone=' => 'mixed',
      ),
    ),
    'datetime::settime' => 
    array (
      'old' => 
      array (
        0 => 'static',
        'hour' => 'int',
        'minute' => 'int',
        'second=' => 'int',
      ),
      'new' => 
      array (
        0 => 'static',
        'hour' => 'int',
        'minute' => 'int',
        'second=' => 'int',
        'microseconds=' => 'int',
      ),
    ),
    'datetimezone::listidentifiers' => 
    array (
      'old' => 
      array (
        0 => 'false|list<string>',
        'what=' => 'int',
        'country=' => 'string',
      ),
      'new' => 
      array (
        0 => 'false|list<string>',
        'what=' => 'int',
        'country=' => 'null|string',
      ),
    ),
    'fsockopen' => 
    array (
      'old' => 
      array (
        0 => 'false|resource',
        'hostname' => 'string',
        'port' => 'int',
        '&w_errno=' => 'int',
        '&w_errstr=' => 'string',
        'timeout=' => 'float',
      ),
      'new' => 
      array (
        0 => 'false|resource',
        'hostname' => 'string',
        'port=' => 'int',
        '&w_errno=' => 'int',
        '&w_errstr=' => 'string',
        'timeout=' => 'float',
      ),
    ),
    'get_defined_functions' => 
    array (
      'old' => 
      array (
        0 => 'array{internal: list<callable-string>, user: list<callable-string>}',
      ),
      'new' => 
      array (
        0 => 'array{internal: list<callable-string>, user: list<callable-string>}',
        'exclude_disabled=' => 'bool',
      ),
    ),
    'get_headers' => 
    array (
      'old' => 
      array (
        0 => 'array<array-key, mixed>|false',
        'url' => 'string',
        'format=' => 'int',
      ),
      'new' => 
      array (
        0 => 'array<array-key, mixed>|false',
        'url' => 'string',
        'format=' => 'int',
        'context=' => 'null|resource',
      ),
    ),
    'getenv' => 
    array (
      'old' => 
      array (
        0 => 'false|string',
        'varname' => 'string',
        'local_only=' => 'bool',
      ),
      'new' => 
      array (
        0 => 'false|string',
        'varname=' => 'string',
        'local_only=' => 'bool',
      ),
    ),
    'getopt' => 
    array (
      'old' => 
      array (
        0 => 'array<string, false|list<false|string>|string>|false',
        'options' => 'string',
        'opts=' => 'array<array-key, mixed>',
      ),
      'new' => 
      array (
        0 => 'array<string, false|list<false|string>|string>|false',
        'options' => 'string',
        'opts=' => 'array<array-key, mixed>',
        '&w_optind=' => 'int',
      ),
    ),
    'hash_update_file' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'context=' => 'resource',
        'filename' => 'string',
      ),
      'new' => 
      array (
        0 => 'bool',
        'context' => 'resource',
        'filename' => 'string',
        'stream_context=' => 'resource',
      ),
    ),
    'imagegd' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'filename=' => 'null|resource|string',
      ),
      'new' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'to=' => 'null|resource|string',
      ),
    ),
    'imagegd2' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'filename=' => 'null|resource|string',
        'chunk_size=' => 'int',
        'type=' => 'int',
      ),
      'new' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'to=' => 'null|resource|string',
        'chunk_size=' => 'int',
        'type=' => 'int',
      ),
    ),
    'imagegif' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'filename=' => 'null|resource|string',
      ),
      'new' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'to=' => 'null|resource|string',
      ),
    ),
    'imagejpeg' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'filename=' => 'null|resource|string',
        'quality=' => 'int',
      ),
      'new' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'to=' => 'null|resource|string',
        'quality=' => 'int',
      ),
    ),
    'imagepng' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'filename=' => 'null|resource|string',
        'quality=' => 'int',
        'filters=' => 'int',
      ),
      'new' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'to=' => 'null|resource|string',
        'quality=' => 'int',
        'filters=' => 'int',
      ),
    ),
    'imagesetinterpolation' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'method' => 'int',
      ),
      'new' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'method=' => 'int',
      ),
    ),
    'imagewbmp' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'filename=' => 'null|resource|string',
        'foreground=' => 'int',
      ),
      'new' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'to=' => 'null|resource|string',
        'foreground=' => 'int',
      ),
    ),
    'imagewebp' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'filename=' => 'null|resource|string',
        'quality=' => 'int',
      ),
      'new' => 
      array (
        0 => 'bool',
        'im' => 'resource',
        'to=' => 'null|resource|string',
        'quality=' => 'int',
      ),
    ),
    'intldateformatter::format' => 
    array (
      'old' => 
      array (
        0 => 'false|string',
        'args=' => 'DateTime|IntlCalendar|array{0?: int, 1?: int, 2?: int, 3?: int, 4?: int, 5?: int, 6?: int, 7?: int, 8?: int, tm_hour?: int, tm_isdst?: int, tm_mday?: int, tm_min?: int, tm_mon?: int, tm_sec?: int, tm_wday?: int, tm_yday?: int, tm_year?: int}|float|int|string',
        'array=' => 'mixed',
      ),
      'new' => 
      array (
        0 => 'false|string',
        'args=' => 'DateTimeInterface|IntlCalendar|array{0?: int, 1?: int, 2?: int, 3?: int, 4?: int, 5?: int, 6?: int, 7?: int, 8?: int, tm_hour?: int, tm_isdst?: int, tm_mday?: int, tm_min?: int, tm_mon?: int, tm_sec?: int, tm_wday?: int, tm_yday?: int, tm_year?: int}|float|int|string',
        'array=' => 'mixed',
      ),
    ),
    'mongodb\\driver\\manager::selectserver' => 
    array (
      'old' => 
      array (
        0 => 'MongoDB\\Driver\\Server',
        'readPreference' => 'MongoDB\\Driver\\ReadPreference',
      ),
      'new' => 
      array (
        0 => 'MongoDB\\Driver\\Server',
        'readPreference=' => 'MongoDB\\Driver\\ReadPreference|null',
      ),
    ),
    'mt_srand' => 
    array (
      'old' => 
      array (
        0 => 'void',
        'seed=' => 'int',
      ),
      'new' => 
      array (
        0 => 'void',
        'seed=' => 'int',
        'mode=' => 'int',
      ),
    ),
    'openssl_csr_get_public_key' => 
    array (
      'old' => 
      array (
        0 => 'false|resource',
        'csr' => 'resource|string',
      ),
      'new' => 
      array (
        0 => 'false|resource',
        'csr' => 'resource|string',
        'use_shortnames=' => 'bool',
      ),
    ),
    'openssl_csr_get_subject' => 
    array (
      'old' => 
      array (
        0 => 'array<array-key, mixed>|false',
        'csr' => 'resource|string',
      ),
      'new' => 
      array (
        0 => 'array<array-key, mixed>|false',
        'csr' => 'resource|string',
        'use_shortnames=' => 'bool',
      ),
    ),
    'openssl_decrypt' => 
    array (
      'old' => 
      array (
        0 => 'false|string',
        'data' => 'string',
        'method' => 'string',
        'password' => 'string',
        'options=' => 'int',
        'iv=' => 'string',
      ),
      'new' => 
      array (
        0 => 'false|string',
        'data' => 'string',
        'method' => 'string',
        'password' => 'string',
        'options=' => 'int',
        'iv=' => 'string',
        'tag=' => 'string',
        'aad=' => 'string',
      ),
    ),
    'openssl_encrypt' => 
    array (
      'old' => 
      array (
        0 => 'false|string',
        'data' => 'string',
        'method' => 'string',
        'password' => 'string',
        'options=' => 'int',
        'iv=' => 'string',
      ),
      'new' => 
      array (
        0 => 'false|string',
        'data' => 'string',
        'method' => 'string',
        'password' => 'string',
        'options=' => 'int',
        'iv=' => 'string',
        '&w_tag=' => 'string',
        'aad=' => 'string',
        'tag_length=' => 'int',
      ),
    ),
    'openssl_pkcs12_export' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'x509' => 'resource|string',
        '&w_out' => 'string',
        'priv_key' => 'array<array-key, mixed>|resource|string',
        'pass' => 'string',
        'args' => 'array<array-key, mixed>',
      ),
      'new' => 
      array (
        0 => 'bool',
        'x509' => 'resource|string',
        '&w_out' => 'string',
        'priv_key' => 'array<array-key, mixed>|resource|string',
        'pass' => 'string',
        'args=' => 'array<array-key, mixed>',
      ),
    ),
    'openssl_x509_checkpurpose' => 
    array (
      'old' => 
      array (
        0 => 'bool|int',
        'x509cert' => 'resource|string',
        'purpose' => 'int',
        'cainfo' => 'array<array-key, mixed>',
        'untrustedfile=' => 'string',
      ),
      'new' => 
      array (
        0 => 'bool|int',
        'x509cert' => 'resource|string',
        'purpose' => 'int',
        'cainfo=' => 'array<array-key, mixed>',
        'untrustedfile=' => 'string',
      ),
    ),
    'openssl_x509_parse' => 
    array (
      'old' => 
      array (
        0 => 'array<array-key, mixed>|false',
        'x509' => 'resource|string',
        'shortname' => 'bool',
      ),
      'new' => 
      array (
        0 => 'array<array-key, mixed>|false',
        'x509' => 'resource|string',
        'shortname=' => 'bool',
      ),
    ),
    'pfsockopen' => 
    array (
      'old' => 
      array (
        0 => 'false|resource',
        'hostname' => 'string',
        'port' => 'int',
        '&w_errno=' => 'int',
        '&w_errstr=' => 'string',
        'timeout=' => 'float',
      ),
      'new' => 
      array (
        0 => 'false|resource',
        'hostname' => 'string',
        'port=' => 'int',
        '&w_errno=' => 'int',
        '&w_errstr=' => 'string',
        'timeout=' => 'float',
      ),
    ),
    'pg_fetch_all' => 
    array (
      'old' => 
      array (
        0 => 'array<array-key, array<array-key, mixed>>',
        'result' => 'resource',
      ),
      'new' => 
      array (
        0 => 'array<array-key, array<array-key, mixed>>',
        'result' => 'resource',
        'result_type=' => 'int',
      ),
    ),
    'pg_last_notice' => 
    array (
      'old' => 
      array (
        0 => 'array<array-key, mixed>|bool|string',
        'connection' => 'resource',
      ),
      'new' => 
      array (
        0 => 'array<array-key, mixed>|bool|string',
        'connection' => 'resource',
        'option=' => 'int',
      ),
    ),
    'pg_select' => 
    array (
      'old' => 
      array (
        0 => 'array<array-key, mixed>|false|string',
        'db' => 'resource',
        'table' => 'string',
        'ids' => 'array<array-key, mixed>',
        'options=' => 'int',
      ),
      'new' => 
      array (
        0 => 'array<array-key, mixed>|false|string',
        'db' => 'resource',
        'table' => 'string',
        'ids' => 'array<array-key, mixed>',
        'options=' => 'int',
        'result_type=' => 'int',
      ),
    ),
    'recursivearrayiterator::__construct' => 
    array (
      'old' => 
      array (
        0 => 'void',
        'array=' => 'array<array-key, mixed>|object',
        'ar_flags=' => 'int',
        'iterator_class=' => 'mixed',
      ),
      'new' => 
      array (
        0 => 'void',
        'input=' => 'array<array-key, mixed>|object',
        'flags=' => 'int',
        'iterator_class=' => 'mixed',
      ),
    ),
    'sessionhandler::gc' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'maxlifetime' => 'int',
      ),
      'new' => 
      array (
        0 => 'false|int',
        'maxlifetime' => 'int',
      ),
    ),
    'sqlite3::createfunction' => 
    array (
      'old' => 
      array (
        0 => 'bool',
        'name' => 'string',
        'callback' => 'callable',
        'argument_count=' => 'int',
      ),
      'new' => 
      array (
        0 => 'bool',
        'name' => 'string',
        'callback' => 'callable',
        'argument_count=' => 'int',
        'flags=' => 'int',
      ),
    ),
    'srand' => 
    array (
      'old' => 
      array (
        0 => 'void',
        'seed=' => 'int',
      ),
      'new' => 
      array (
        0 => 'void',
        'seed=' => 'int',
        'mode=' => 'int',
      ),
    ),
    'strpbrk' => 
    array (
      'old' => 
      array (
        0 => 'false|string',
        'haystack' => 'string',
        'char_list=' => 'string',
      ),
      'new' => 
      array (
        0 => 'false|string',
        'haystack' => 'string',
        'char_list' => 'string',
      ),
    ),
    'strtok' => 
    array (
      'old' => 
      array (
        0 => 'false|non-empty-string',
        'str' => 'string',
        'token' => 'string',
      ),
      'new' => 
      array (
        0 => 'false|non-empty-string',
        'str' => 'string',
        'token=' => 'string',
      ),
    ),
    'swoole\\coroutine\\mysql::connect' => 
    array (
      'old' => 
      array (
        0 => 'ReturnType',
        'server_config' => 'array<array-key, mixed>',
      ),
      'new' => 
      array (
        0 => 'ReturnType',
        'server_config=' => 'array<array-key, mixed>',
      ),
    ),
    'swoole\\http\\response::cookie' => 
    array (
      'old' => 
      array (
        0 => 'string',
        'name' => 'string',
        'value=' => 'string',
        'expires=' => 'string',
        'path=' => 'string',
        'domain=' => 'string',
        'secure=' => 'string',
        'httponly=' => 'string',
      ),
      'new' => 
      array (
        0 => 'string',
        'name' => 'string',
        'value=' => 'string',
        'expires=' => 'string',
        'path=' => 'string',
        'domain=' => 'string',
        'secure=' => 'string',
        'httponly=' => 'string',
        'samesite=' => 'mixed',
        'priority=' => 'mixed',
      ),
    ),
    'swoole\\http\\response::rawcookie' => 
    array (
      'old' => 
      array (
        0 => 'ReturnType',
        'name' => 'string',
        'value=' => 'string',
        'expires=' => 'string',
        'path=' => 'string',
        'domain=' => 'string',
        'secure=' => 'string',
        'httponly=' => 'string',
      ),
      'new' => 
      array (
        0 => 'ReturnType',
        'name' => 'string',
        'value=' => 'string',
        'expires=' => 'string',
        'path=' => 'string',
        'domain=' => 'string',
        'secure=' => 'string',
        'httponly=' => 'string',
        'samesite=' => 'mixed',
        'priority=' => 'mixed',
      ),
    ),
    'swoole\\process::daemon' => 
    array (
      'old' => 
      array (
        0 => 'void',
        'nochdir=' => 'bool',
        'noclose=' => 'bool',
      ),
      'new' => 
      array (
        0 => 'void',
        'nochdir=' => 'bool',
        'noclose=' => 'bool',
        'pipes=' => 'mixed',
      ),
    ),
    'swoole\\server::after' => 
    array (
      'old' => 
      array (
        0 => 'ReturnType',
        'ms' => 'int',
        'callback' => 'callable',
        'param=' => 'string',
      ),
      'new' => 
      array (
        0 => 'ReturnType',
        'ms' => 'int',
        'callback' => 'callable',
      ),
    ),
    'swoole\\websocket\\server::pack' => 
    array (
      'old' => 
      array (
        0 => 'binary',
        'data' => 'string',
        'opcode=' => 'string',
        'finish=' => 'string',
        'mask=' => 'string',
      ),
      'new' => 
      array (
        0 => 'binary',
        'data' => 'string',
        'opcode=' => 'string',
        'flags=' => 'string',
      ),
    ),
    'swoole\\websocket\\server::push' => 
    array (
      'old' => 
      array (
        0 => 'void',
        'fd' => 'string',
        'data' => 'string',
        'opcode=' => 'string',
        'finish=' => 'string',
      ),
      'new' => 
      array (
        0 => 'void',
        'fd' => 'string',
        'data' => 'string',
        'opcode=' => 'string',
        'flags=' => 'string',
      ),
    ),
    'swoole_timer_after' => 
    array (
      'old' => 
      array (
        0 => 'int',
        'ms' => 'int',
        'callback' => 'callable',
        '...params=' => 'mixed',
      ),
      'new' => 
      array (
        0 => 'int',
        'ms' => 'int',
        'callback' => 'callable',
      ),
    ),
    'swoole_timer_tick' => 
    array (
      'old' => 
      array (
        0 => 'int',
        'ms' => 'int',
        'callback' => 'callable',
        '...params=' => 'mixed',
      ),
      'new' => 
      array (
        0 => 'int',
        'ms' => 'int',
        'callback' => 'callable',
      ),
    ),
    'timezone_identifiers_list' => 
    array (
      'old' => 
      array (
        0 => 'false|list<string>',
        'what=' => 'int',
        'country=' => 'string',
      ),
      'new' => 
      array (
        0 => 'false|list<string>',
        'what=' => 'int',
        'country=' => 'null|string',
      ),
    ),
    'unpack' => 
    array (
      'old' => 
      array (
        0 => 'array<array-key, mixed>',
        'format' => 'string',
        'input' => 'string',
      ),
      'new' => 
      array (
        0 => 'array<array-key, mixed>|false',
        'format' => 'string',
        'input' => 'string',
        'offset=' => 'int',
      ),
    ),
    'xml_set_object' => 
    array (
      'old' => 
      array (
        0 => 'true',
        'parser' => 'resource',
        '&obj' => 'object',
      ),
      'new' => 
      array (
        0 => 'true',
        'parser' => 'resource',
        'obj' => 'object',
      ),
    ),
  ),
  'removed' => 
  array (
    'swoole\\buffer::__destruct' => 
    array (
      0 => 'void',
    ),
    'swoole\\buffer::__tostring' => 
    array (
      0 => 'string',
    ),
    'swoole\\buffer::append' => 
    array (
      0 => 'int',
      'data' => 'string',
    ),
    'swoole\\buffer::clear' => 
    array (
      0 => 'void',
    ),
    'swoole\\buffer::expand' => 
    array (
      0 => 'int',
      'size' => 'int',
    ),
    'swoole\\buffer::read' => 
    array (
      0 => 'string',
      'offset' => 'int',
      'length' => 'int',
    ),
    'swoole\\buffer::recycle' => 
    array (
      0 => 'void',
    ),
    'swoole\\buffer::substr' => 
    array (
      0 => 'string',
      'offset' => 'int',
      'length=' => 'int',
      'remove=' => 'bool',
    ),
    'swoole\\buffer::write' => 
    array (
      0 => 'void',
      'offset' => 'int',
      'data' => 'string',
    ),
    'swoole\\client::on' => 
    array (
      0 => 'void',
      'event_name' => 'string',
      'callback' => 'callable',
    ),
    'swoole\\client::pause' => 
    array (
      0 => 'void',
    ),
    'swoole\\client::pipe' => 
    array (
      0 => 'void',
      'dst_socket' => 'string',
    ),
    'swoole\\client::resume' => 
    array (
      0 => 'void',
    ),
    'swoole\\client::sleep' => 
    array (
      0 => 'void',
    ),
    'swoole\\client::wakeup' => 
    array (
      0 => 'void',
    ),
    'swoole\\serialize::pack' => 
    array (
      0 => 'ReturnType',
      'data' => 'string',
      'flag=' => 'int',
    ),
    'swoole\\serialize::unpack' => 
    array (
      0 => 'ReturnType',
      'string' => 'string',
      'flag=' => 'string',
    ),
  ),
);