OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Δημιουργήσατε νέο πίνακα {board}",
    "{user} has created a new board {board}" : "Ο/η {user} δημιούργησε νέο πίνακα {board}",
    "You have deleted the board {board}" : "Έχετε διαγράψει τον πίνακα {board}",
    "{user} has deleted the board {board}" : "Ο/η {user} διέγραψε τον πίνακα {board}",
    "You have restored the board {board}" : "Εχετε επαναφέρει τον πίνακα {board}",
    "{user} has restored the board {board}" : "Ο/η {user} επανέφερε τον πίνακα {board}",
    "You have shared the board {board} with {acl}" : "Εχετε διαμοιράσει τον πίνακα {board} με {acl}",
    "{user} has shared the board {board} with {acl}" : "Ο/η {user} διαμοίρασε τον πίνακα {board} με {acl}",
    "You have removed {acl} from the board {board}" : "Αφαιρέθηκε η {acl} από τον πίνακα {board}",
    "{user} has removed {acl} from the board {board}" : "Ο/η {user} αφαίρεσε την {acl} από τον πίνακα {board}",
    "You have renamed the board {before} to {board}" : "Μετονομάσατε τον πίνακα {before} σε {board}",
    "{user} has renamed the board {before} to {board}" : "Ο/η {user} μετονόμασε τον πίνακα {before} σε {board}",
    "You have archived the board {board}" : "Αρχειοθετήσατε τον πίνακα {board}",
    "{user} has archived the board {before}" : "Ο/η {user} αρχειοθέτησε τον πίνακα {before}",
    "You have unarchived the board {board}" : "Επαναφέρατε τον πίνακα {board} από το αρχείο",
    "{user} has unarchived the board {before}" : "Ο/η {user} επανέφερε τον πίνακα {before} από αρχείο",
    "You have created a new list {stack} on board {board}" : "Έχετε δημιουργήσει μια νέα λίστα {stack} στον πίνακα {board}",
    "{user} has created a new list {stack} on board {board}" : "Ο/η {user} δημιούργησε μια νέα λίστα {stack} στον πίνακα {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Μετονομάσατε την λίστα {before} σε {stack} στον πίνακα {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "Ο/η {user} μετονόμασε την λίστα {before} σε {stack} στον πίνακα {board}",
    "You have deleted list {stack} on board {board}" : "Διαγράψατε την λίστα {stack} στον πίνακα {board}",
    "{user} has deleted list {stack} on board {board}" : "Ο/η {user} διέγραψε την λίστα {stack} στον πίνακα {board}",
    "You have created card {card} in list {stack} on board {board}" : "Δημιουργήσατε την καρτέλα {card} στην λίστα {stack} του πίνακα {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "Ο/η {user} δημιούργησε την καρτέλα {card} στην λίστα {stack} του πίνακα {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Διαγράψατε την καρτέλα {card} στην λίστα {stack} του πίνακα {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "Ο/η {user} διέγραψε την καρτέλα {card} στην λίστα {stack} του πίνακα {board}",
    "You have renamed the card {before} to {card}" : "Μετονομάσατε την καρτέλα {before} σε {card}",
    "{user} has renamed the card {before} to {card}" : "Ο/η {user} μετονόμασε την καρτέλα {before} σε {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Προσθέσατε μια περιγραφή στην καρτέλα {card} της λίστας {stack} του πίνακα {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "Ο/η {user} πρόσθεσε μια περιγραφή στην καρτέλα {card} της λίστας {stack} του πίνακα {board} ",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Ενημερώσατε την περιγραφή στην καρτέλα {card} της λίστας {stack} του πίνακα {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "Ο/η {user} ενημέρωσε την περιγραφή της καρτέλας {card} στη λίστα {stack} του πίνακα {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Αρχειοθετήσατε την κάρτα {card} στην λίστα {stack} του πίνακα {board} ",
    "{user} has archived card {card} in list {stack} on board {board}" : "Ο/η {user} αρχειοθέτησε την κάρτα {card} στην λίστα {stack} του πίνακα {board} ",
    "You have unarchived card {card} in list {stack} on board {board}" : "Επαναφέρατε από το αρχείο την καρτέλα {card} της λίστας {stack} του πίνακα {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "Ο/η {user} επανέφερε από το αρχείο την κάρτα {card} της λίστας {stack} του πίνακα {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Έχετε επισημάνει την κάρτα {card} ως ολοκληρωμένη στη λίστα {stack} του πίνακα {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} έχει επισημάνει την κάρτα {card} ως ολοκληρωμένη στη λίστα {stack} του πίνακα {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Έχετε επισημάνει την κάρτα {card} ως μη ολοκληρωμένη στη λίστα {stack} του πίνακα {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} έχει επισημάνει την κάρτα {card} ως μη ολοκληρωμένη στη λίστα {stack} του πίνακα {board}",
    "You have removed the due date of card {card}" : "Καταργήσατε την ημερομηνία λήξης της καρτέλας {card}",
    "{user} has removed the due date of card {card}" : "Ο/η {user} κατάργησε την ημερομηνία λήξης της καρτέλας {card}",
    "You have set the due date of card {card} to {after}" : "Ορίσατε την ημερομηνία λήξης της καρτέλας {card} σε {after}",
    "{user} has set the due date of card {card} to {after}" : "Ο/η {user} όρισε την ημερομηνία λήξης της καρτέλας {card} σε {after} ",
    "You have updated the due date of card {card} to {after}" : "Ενημερώσατε την ημερομηνία λήξης της καρτέλας {card} σε {after}",
    "{user} has updated the due date of card {card} to {after}" : "Ο/η {user} ενημέρωσε την ημερομηνία λήξης της καρτέλας {card} σε {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Προσθέσατε ετικέτα {label}  στην καρτέλα {card} της λίστας {stack} του πίνακα {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "Ο/η {user} πρόσθεσε ετικέτα {label} στην καρτέλα {card} της λίστας {stack} του πίνακα {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Αφαιρέσατε την ετικέτα {label} από την καρτέλα {card} της λίστας {stack} του πίνακα {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "Ο/η {user} αφαίρεσε την ετικέτα {label}  της καρτέλας {card} της λίστας {stack} του πίνακα {board} ",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Έχετε ορίσει τον {assigneduser} στην καρτέλα {card} στον πίνακα {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "Ο/η {user} έχει ορισθεί {assigneduser} στην καρτέλα {card} του πίνακα {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Έχετε αφαιρέσει {assigneduser} από την καρτέλα {card}  του πίνακα {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "Ο/η {user} έχει αφαιρεθεί {assigneduser} από την καρτέλα {card} του  πίνακα {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Μετακινήσατε την καρτέλα {card} από την λίστα {stackBefore} στη {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "Ο/η {user} μετακίνησε την καρτέλα {card} από την λίστα {stackBefore} στην {stack}",
    "You have added the attachment {attachment} to card {card}" : "Προσθέσατε το συνημμένο {attachment} στην καρτέλα {card}",
    "{user} has added the attachment {attachment} to card {card}" : "Ο/η {user} πρόσθεσε το συνημμένο {attachment} στην καρτέλα {card}",
    "You have updated the attachment {attachment} on card {card}" : "Ενημερώσατε το συνημμένο {attachment} της καρτέλας {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "Ο/η {user} ενημέρωσε το συνημμένο {attachment} της καρτέλας {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Διαγράψατε το συνημμένο {attachment} της καρτέλας {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "Ο/η {user} διέγραψε το συνημμένο {attachment} της καρτέλας {card}",
    "You have restored the attachment {attachment} to card {card}" : "Επαναφέρατε το συνημμένο {attachment} στην καρτέλα {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "Ο/η {user} επανέφερε το συνημμένο {attachment} στην καρτέλα {card}",
    "You have commented on card {card}" : "Σχολιάσατε την καρτέλα {card}",
    "{user} has commented on card {card}" : "Ο/η {user} σχολίασε την καρτέλα {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Αλλαγές στην <strong>εφαρμογή Deck</strong>",
    "A <strong>board, list or card</strong> was changed" : "Ένας <strong>πίνακας, λίστα ή κάρτα</strong> άλλαξε",
    "A <strong>comment</strong> was created on a card" : "Ένα <strong>σχόλιο</strong> δημιουργήθηκε σε μια καρτέλα",
    "A <strong>card description</strong> has been changed" : "Μια <strong>περιγραφή κάρτας</strong> έχει αλλάξει",
    "The file was uploaded" : "Το αρχείο μεταφορτώθηκε",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Το αρχείο που εστάλη υπερβαίνει την οδηγία μέγιστου επιτρεπτού μεγέθους \"upload_max_filesize\" του php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Το ανεβασμένο αρχείο υπερβαίνει το MAX_FILE_SIZE που ορίζεται στην  HTML φόρμα",
    "The file was only partially uploaded" : "Το αρχείο μεταφορτώθηκε εν μέρει",
    "No file was uploaded" : "Δεν μεταφορτώθηκε κάποιο αρχείο",
    "Missing a temporary folder" : "Λείπει κάποιος προσωρινός φάκελος",
    "Could not write file to disk" : "Αδυναμία εγγραφής αρχείου στον δίσκο",
    "A PHP extension stopped the file upload" : "Ένα πρόσθετο PHP διέκοψε την μεταφόρτωση του αρχείου",
    "No file uploaded or file size exceeds maximum of %s" : "Δεν μεταφορτώθηκε αρχείο ή το μέγεθος αρχείου υπερβαίνει το μέγιστο %s",
    "Invalid file type. Only JSON files are allowed." : "Μη έγκυρος τύπος αρχείου. Επιτρέπονται μόνο αρχεία JSON.",
    "Invalid JSON data" : "Μη έγκυρα δεδομένα JSON",
    "Failed to import board" : "Αποτυχία εισαγωγής πίνακα",
    "Cards due today" : "Κάρτες που λήγουν σήμερα",
    "Cards due tomorrow" : "Κάρτες που λήγουν αύριο",
    "Upcoming cards" : "Επερχόμενες καρτέλες",
    "Load more" : "Φόρτωση περισσότερων",
    "Welcome to Nextcloud Deck!" : "Καλώς ήρθατε στο Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Η καρτέλα \"%s\" του \"%s\" ανατέθηκε σε εσάς από τον %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "Ο/Η {user} έχει αναθέσει την καρτέλα {deck-card} του πίνακα {deck-board} σε εσάς.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Η καρτέλα {deck-card} στο {deck-board} έχει λήξει.",
    "%s has mentioned you in a comment on \"%s\"." : "Ο/η%s σας ανέφερε σε σχόλιο στο \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "Ο/Η {user} σας ανέφερε σε ένα σχόλιο στο {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Ο πίνακας \"%s\" είναι σε κοινή χρήση μαζί σας από %s.",
    "{user} has shared {deck-board} with you." : "Ο/Η διαμοιράστηκε μαζί σας το {deck-board}",
    "Deck board" : "Πίνακας του Deck",
    "Owned by %1$s" : "Ανήκει στον/στην %1$s",
    "Deck boards, cards and comments" : "Πίνακες, κάρτες και σχόλια Deck",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Από %1$s, στον %2$s/%3$s, που ανήκει στον %4$s",
    "Create a new deck card" : "Δημιουργήστε μια νέα κάρτα",
    "Card comments" : "Σχόλια καρτέλας",
    "%s on %s" : "%s στο %s",
    "Deck boards and cards" : "Πίνακες και κάρτες Deck",
    "No data was provided to create an attachment." : "Δεν δόθηκαν στοιχεία για δημιουργία συνημμένου.",
    "Finished" : "Ολοκληρώθηκε",
    "To review" : "Προς επισκόπηση",
    "Action needed" : "Απαιτείται ενέργεια",
    "Later" : "Αργότερα",
    "copy" : "Αντιγραφή",
    "Read more inside" : "Διαβάστε περισσότερα εντός",
    "Custom lists - click to rename!" : "Προσαρμοσμένες λίστες - κάντε κλικ για μετονομασία!",
    "To Do" : "Προς Ενέργεια",
    "In Progress" : "Σε Εξέλιξη",
    "Done" : "Ολοκληρώθηκε",
    "1. Open to learn more about boards and cards" : "1. Ανοίξτε για να μάθετε περισσότερα για τους πίνακες και τις κάρτες",
    "2. Drag cards left and right, up and down" : "2. Σύρετε κάρτες αριστερά και δεξιά, πάνω και κάτω",
    "3. Apply rich formatting and link content" : "3. Εφαρμόστε πλούσια μορφοποίηση και συνδέστε περιεχόμενο",
    "4. Share, comment and collaborate!" : "4. Μοιραστείτε, σχολιάστε και συνεργαστείτε!",
    "Create your first card!" : "Δημιουργήστε την πρώτη σας κάρτα!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Αυτό το σχόλιο έχει περισσότερους από %s χαρακτήρες.\nΠροστέθηκε ως συνημμένο στην καρτέλα με όνομα %s .\nΠροσβάσιμο στη διεύθυνση URL: %s.",
    "Attachments" : "Συνημμένα",
    "File" : "Αρχείο",
    "date" : "ημερομηνία",
    "Card not found" : "Η καρτέλα δεν βρέθηκε",
    "Path is already shared with this card" : "Η διαδρομή κοινοποιείται ήδη σε αυτήν την καρτέλα",
    "Invalid date, date format must be YYYY-MM-DD" : "Μη έγκυρη ημερομηνία, η μορφή ημερομηνίας πρέπει να είναι ΕΕΕΕ-ΜΜ-ΗΗ",
    "Personal planning and team project organization" : "Προσωπικός σχεδιασμός και οργάνωση ομαδικών έργων",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Το Deck είναι ένα εργαλείο οργάνωσης τύπου kanban με στόχο τον προσωπικό προγραμματισμό και την  οργάνωση έργων για ομάδες που έχουν ενσωματωθεί στο Nextcloud.\n\n\n- 📥 Προσθέστε τις εργασίες σας στις καρτέλες και βάλτε τες στη σειρά\n- 📄 Γράψτε τις πρόσθετες σημειώσεις\n- 🔖 Αντιστοιχίστε τις ετικέτες για ακόμη καλύτερη οργάνωση\n- 👥 Μοιραστείτε με την ομάδα, φίλους ή την οικογένειά σας\n- 📎 Επισυνάψτε αρχεία και ενσωματώστε τα στην περιγραφή\n- 💬 Συζητήστε με την ομάδα σας χρησιμοποιώντας σχόλια\n- ⚡ Παρακολουθήστε τις αλλαγές στη ροή δραστηριοτήτων\n- 🚀 Έχετε τα όλα οργανωμένα",
    "Add board" : "Προσθήκη πίνακα",
    "Card details" : "Λεπτομέρειες καρτέλας",
    "Select the board to link to a project" : "Επιλέξτε πίνακα και συνδέστε τον σε ένα έργο",
    "Search by board title" : "Αναζήτηση με το όνομα πίνακα",
    "Select board" : "Επιλογή πίνακα",
    "Move/copy card" : "Μετακίνηση/αντιγραφή κάρτας",
    "Select a board" : "Επιλογή ενός πίνακα",
    "No lists available" : "Δεν υπάρχουν διαθέσιμες λίστες",
    "Select a list" : "Επιλέξτε μια λίστα",
    "Move card" : "Μετακίνηση καρτέλας",
    "Copy card" : "Αντίγραφο κάρτας",
    "Select the card to link to a project" : "Επιλογή καρτέλας για σύνδεση στο έργο",
    "Link to card" : "Σύνδεσμος σε καρτέλα",
    "Select a card" : "Επιλογή μιας καρτέλας",
    "Cancel" : "Aκύρωση",
    "This board is read only" : "Ο πίνακας είναι μόνο για ανάγνωση",
    "Drop your files to upload" : "Αποθέστε τα αρχεία σας για ανέβασμα",
    "File already exists" : "Το αρχείο υπάρχει ήδη",
    "A file with the name {filename} already exists." : "Το αρχείο με όνομα {filename} υπάρχει ήδη.",
    "Do you want to overwrite it?" : "Επιθυμείτε να γίνει αντικατάσταση του?",
    "Overwrite file" : "Αντικατάσταση αρχείου",
    "Keep existing file" : "Διατήρηση υπάρχοντος αρχείου",
    "Add card" : "Προσθήκη καρτέλας",
    "Archived cards" : "Αρχειοθετημένες καρτέλες",
    "Add list" : "Προσθήκη λίστας",
    "List name" : "Όνομα λίστας",
    "Clear search" : "Εκκαθάριση αναζήτησης",
    "Active filters" : "Ενεργά φίλτρα",
    "Apply filter" : "Εφαρμογή φίλτρου",
    "Filter by tag" : "Φίλτρο ανά ετικέτα",
    "Filter by assigned user" : "Φίλτρο ανά χρήστη",
    "Unassigned" : "Χωρίς ανάθεση",
    "Filter by status" : "Φίλτρο με βάση την κατάσταση",
    "Open and completed" : "Ανοιχτά και ολοκληρωμένα",
    "Open" : "Ανοιχτός",
    "Completed" : "Ολοκληρωμένες",
    "Filter by due date" : "Φίλτρο ανά ημερομηνία λήξης",
    "Overdue" : "Εκπρόθεσμες",
    "Next 24 hours" : "Επόμενες 24 ώρες",
    "Next 7 days" : "Επόμενες 7 ημέρες",
    "Next 30 days" : "Επόμενες 30 ημέρες",
    "No due date" : "Χωρίς ημερομηνία λήξης",
    "Clear filter" : "Εκκαθάριση φίλτρου",
    "View Modes" : "Τρόποι προβολής",
    "Toggle View Modes" : "Εναλλαγή τρόπων προβολής",
    "Hide archived cards" : "Απόκρυψη καρτελών που αρχειοθετήθηκαν",
    "Show archived cards" : "Εμφάνιση καρτελών που αρχειοθετήθηκαν",
    "Toggle compact mode" : "Εναλλαγή λειτουργίας μικρού μεγέθους",
    "Hide card cover images" : "Απόκρυψη εικόνων εξωφύλλου καρτών",
    "Show card cover images" : "Εμφάνιση εικόνων εξωφύλλου καρτών",
    "Open details" : "Άνοιγμα λεπτομερειών",
    "Details" : "Λεπτομέρειες",
    "Deck settings" : "Ρυθμίσεις Deck",
    "General" : "Γενικά",
    "Use bigger card view" : "Χρησιμοποιήστε μεγαλύτερη προβολή καρτέλας",
    "Appearance" : "Εμφάνιση",
    "Show card ID badge" : "Εμφάνιση ID κάρτας",
    "Show boards in calendar/tasks" : "Εμφάνιση πινάκων στο ημερολόγιο / εργασίες",
    "Limit board creation to some groups" : "Περιορισμός της δημιουργίας πινάκων σε ορισμένες ομάδες",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Οι χρήστες εκτός αυτών των ομάδων δεν θα μπορούν να δημιουργούν τους δικούς τους πίνακες, αλλά θα μπορούν να εργάζονται σε πίνακες που τους έχουν διαμοιραστεί.",
    "Enable federation" : "Ενεργοποίηση ομοσπονδίας",
    "Board actions" : "Ενέργειες πίνακα",
    "Scroll sideways" : "Κύλιση οριζόντια",
    "Navigate between cards" : "Πλοήγηση μεταξύ καρτών",
    "Close card details" : "Κλείσιμο λεπτομερειών κάρτας",
    "Search" : "Αναζήτηση",
    "Show card filters" : "Εμφάνιση φίλτρων κάρτας",
    "Clear card filters" : "Εκκαθάριση φίλτρων κάρτας",
    "Show those shortcuts" : "Εμφάνιση αυτών των συντομεύσεων",
    "Card actions" : "Ενέργειες κάρτας",
    "Open card details" : "Άνοιγμα λεπτομερειών κάρτας",
    "Edit the card title" : "Επεξεργασία τίτλου κάρτας",
    "Assign yourself to the current card" : "Αναθέστε τον εαυτό σας στην τρέχουσα κάρτα",
    "Archive/unarchive the current card" : "Αρχειοθέτηση/αποαρχειοθέτηση της τρέχουσας κάρτας",
    "Mark card as completed/not completed" : "Σημειώστε την κάρτα ως ολοκληρωμένη/μη ολοκληρωμένη",
    "Open card menu" : "Άνοιγμα μενού κάρτας",
    "Currently present people" : "Παρόντες αυτή τη στιγμή",
    "Loading board" : "Φόρτωση πίνακα",
    "Board not found" : "Ο πίνακας δεν βρέθηκε",
    "Create a new list to add cards to this board" : "Δημιουργήστε νέα λίστα για να προσθέσετε καρτέλες σε αυτό τον πίνακα.",
    "Sharing" : "Διαμοιρασμός",
    "Tags" : "Ετικέτες",
    "Deleted items" : "Διαγραμμένα αντικείμενα",
    "Activity" : "Δραστηριότητα",
    "Deleted lists" : "Διαγραμμένες λίστες",
    "Undo" : "Αναίρεση",
    "Deleted cards" : "Διαγραμμένες καρτέλες",
    "Hours" : "Ώρες",
    "Days" : "Ημέρες",
    "Weeks" : "Εβδομάδες",
    "Months" : "Μήνες",
    "No cards yet" : "Καμία κάρτα ακόμη",
    "Failed to create share with {displayName}" : "Αποτυχία δημιουργίας κοινής χρήσης με το {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Είστε σίγουροι ότι θέλετε να μεταφέρετε τον πίνακα {title} στον {user}?  ",
    "Transfer the board." : "Μεταφορά του πίνακα.",
    "Transfer" : "Μεταφορά",
    "The board has been transferred to {user}" : "Ο πίνακας έχει μεταφερθεί στον/στην {user}",
    "Failed to transfer the board to {user}" : "Απέτυχε η μεταφορά του πίνακα στον χρήστη {user}",
    "Board owner" : "Κάτοχος πίνακα",
    "(Group)" : "(Ομάδα)",
    "(Team)" : "(Ομάδα)",
    "(remote)" : "(εξ αποστάσεως)",
    "Can edit" : "Μπορεί να επεξεργαστεί",
    "Can share" : "Μπορεί να διαμοιράσει",
    "Can manage" : "Μπορεί να διαχειριστεί",
    "Owner" : "Κάτοχος",
    "Delete" : "Διαγραφή",
    "List deleted" : "Η λίστα διαγράφηκε",
    "Edit list title" : "Επεξεργασία τίτλου λίστας",
    "Archive all cards" : "Αρχειοθέτηση όλων των καρτελών.",
    "Unarchive all cards" : "Κατάργηση αρχειοθέτησης όλων των καρτών",
    "Delete list" : "Διαγραφή λίστας",
    "Archive all cards in this list" : "Αρχειοθέτηση όλων των καρτελών σε αυτή τη λίστα.",
    "Unarchive all cards in this list" : "Κατάργηση αρχειοθέτησης όλων των καρτών σε αυτή τη λίστα",
    "Add a new card" : "Προσθήκη νέας καρτέλας",
    "Card name" : "Όνομα καρτέλας",
    "title and color value must be provided" : "ο τίτλος και η τιμή χρώματος πρέπει να παρέχονται",
    "Edit" : "Επεξεργασία",
    "Add a new tag" : "Προσθήκη νέας ετικέτας",
    "Board name" : "Όνομα πίνακα",
    "Members" : "Μέλη",
    "Assign to users/groups/team" : "Ανάθεση σε χρήστες/ομάδες/teams",
    "Assign a user to this card…" : "Ανάθεση χρήστη στην καρτέλα...",
    "Select a user to assign to this card…" : "Επιλέξτε έναν χρήστη για να του αναθέσετε αυτή την κάρτα...",
    "File to share" : "Αρχείο για κοινή χρήση",
    "Invalid path selected" : "Επιλέχθηκε μη έγκυρη διαδρομή",
    "Upload new files" : "Μεταφορτώστε νέα αρχεία",
    "Share from Files" : "Κοινή χρήση από Αρχεία",
    "Pending share" : "Κοινή χρήση σε εκκρεμότητα",
    "Add this attachment" : "Προσθήκη αυτού του συνημμένου",
    "Show in Files" : "Εμφάνιση σε Αρχεία",
    "Download" : "Λήψη",
    "Remove attachment" : "Αφαίρεση συνημμένου",
    "Delete Attachment" : "Διαγραφή συνημμένου",
    "Restore Attachment" : "Επαναφορά συνημμένου",
    "The title cannot be empty." : "Ο τίτλος δεν μπορεί να είναι κενός.",
    "Cannot close unsaved card!" : "Αδυναμία κλεισίματος της κάρτας που δεν έχει αποθηκευτεί!",
    "Open in sidebar view" : "Άνοιγμα σε προβολή πλευρικής στήλης",
    "Open in bigger view" : "Άνοιγμα σε μεγαλύτερη προβολή",
    "Comments" : "Σχόλια",
    "Failed to load comments" : "Αποτυχία φόρτωσης σχολίων",
    "No comments yet. Begin the discussion!" : "Χωρίς σχόλια ακόμη. Ξεκινήστε την συζήτηση!",
    "The comment cannot be empty." : "Το σχόλιο δεν μπορεί να είναι κενό.",
    "The comment cannot be longer than 1000 characters." : "Το σχόλιο δεν μπορεί να έχι περισσότερους από 1000 χαρακτήρες.",
    "Submit" : "Υποβολή",
    "Created:" : "Δημιουργήθηκε:",
    "In reply to" : "Σε απάντηση σε",
    "Cancel reply" : "Ακύρωση απάντησης",
    "Reply" : "Απάντηση",
    "Update" : "Ενημέρωση",
    "Mark as done" : "Σήμανση ως ολοκληρωμένου",
    "Mark as not done" : "Σήμανση ως ολοκληρωμένου",
    "Write a description …" : "Γράψτε μια περιγραφή…",
    "Could not save description" : "Αδυναμία αποθήκευσης της περιγραφής",
    "Description" : "Περιγραφή",
    "(Unsaved)" : "(Δεν αποθηκεύτηκε)",
    "(Saving…)" : "(Αποθήκευση...)",
    "Formatting help" : "Βοήθεια μορφοποίησης",
    "Edit description" : "Επεξεργασία περιγραφής",
    "View description" : "Εμφάνιση περιγραφής",
    "Add Attachment" : "Προσθήκη συνημμένου",
    "Choose attachment" : "Επιλογή συνημμένου",
    "Select Date" : "Επιλέξτε ημερομηνία",
    "Later today – {timeLocale}" : "Αργότερα σήμερα – {timeLocale}",
    "Set due date for later today" : "Ορίστε ημερομηνία λήξης για αργότερα σήμερα",
    "Tomorrow – {timeLocale}" : "Αύριο – {timeLocale}",
    "Set due date for tomorrow" : "Ορίστε ημερομηνία λήξης για αύριο",
    "This weekend – {timeLocale}" : "Αυτή την εβδομάδα – {timeLocale}",
    "Set due date for this weekend" : "Ορίστε ημερομηνία λήξης για αυτό το Σαββατοκύριακο",
    "Next week – {timeLocale}" : "Επόμενη εβδομάδα – {timeLocale}",
    "Set due date for next week" : "Ορίστε ημερομηνία λήξης για την επόμενη εβδομάδα",
    "Assign a due date to this card…" : "Ορίστε μια ημερομηνία λήξης σε αυτή την κάρτα...",
    "Set a due date" : "Καθορίστε ημερομηνίας λήξης",
    "Add due date" : "Προσθήκη ημερομηνίας λήξης",
    "Choose a date" : "Επιλέξτε ημερομηνίας",
    "Remove due date" : "Αφαίρεση ημερομηνίας λήξης",
    "Due at:" : "Λήγει στις:",
    "Not done" : "Μη ολοκληρωμένο",
    "Unarchive card" : "Αναίρεση αρχειοθέτησης καρτέλας",
    "Archive card" : "Αρχειοθέτηση καρτέλας",
    "Assign a tag to this card…" : "Ορίστε μια ετικέτα σε αυτήν την καρτέλα...",
    "Select or create a tag…" : "Επιλέξτε ή δημιουργήστε μια ετικέτα...",
    "Create a new tag:" : "Δημιουργήστε νέα ετικέτα",
    "(group)" : "(ομάδα)",
    "{count} comments, {unread} unread" : "{count} σχόλια, {unread} μη αναγνωσμένα",
    "Todo items" : "Στοιχεία todo",
    "Edit card title" : "Επεξεργασία τίτλου κάρτας",
    "Open link" : "Άνοιγμα συνδέσμου",
    "Card deleted" : "Η καρτέλα διαγράφηκε",
    "Edit title" : "Επεξεργασία τίτλου",
    "Assign to me" : "Ανάθεση σε εμένα",
    "Unassign myself" : "Αποδέσμευσή μου",
    "Delete card" : "Διαγραφή καρτέλας",
    "All boards" : "Όλοι οι πίνακες",
    "Archived boards" : "Αρχειοθέτηση πινάκων ",
    "Shared with you" : "Διαμοιρασμένα μαζί σας",
    "Cancel edit" : "Ακύρωση επεξεργασίας",
    "Save board" : "Αποθήκευση πίνακα",
    "Board {0} deleted" : "Διαγράφηκε {0} πίνακας ",
    "All cards" : "Όλες οι καρτέλες",
    "Only assigned cards" : "Μόνο καρτέλες που έχουν ανατεθεί",
    "No reminder" : "Δεν υπάρχει υπενθύμιση",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Είστε βέβαιοι ότι θέλετε να διαγράψετε τον πίνακα {title}; Αυτό θα διαγράψει όλα τα δεδομένα του πίνακα συμπεριλαμβανομένων και των αρχειοθετημένων καρτών.",
    "Delete the board?" : "Διαγραφή του πίνακα;",
    "Leave" : "Αποχώρηση",
    "Exporting board..." : "Εξαγωγή πίνακα...",
    "An error occurred" : "Παρουσιάστηκε σφάλμα",
    "Board details" : "Λεπτομέριες πίνακα",
    "Edit board" : "Επεξεργασία πίνακα",
    "Clone board" : "Κλώνος πίνακα",
    "Unarchive board" : "Κατάργηση αρχειοθέτησης πίνακα",
    "Archive board" : "Αρχειοθέτηση πίνακα",
    "Export board" : "Εξαγωγή πίνακα",
    "Turn on due date reminders" : "Ενεργοποιήστε τις υπενθυμίσεις ημερομηνίας προθεσμίας",
    "Turn off due date reminders" : "Απενεργοποιήστε τις υπενθυμίσεις ημερομηνίας προθεσμίας",
    "Due date reminders" : "Υπενθυμίσεις ημερομηνίας προθεσμίας",
    "Assigned cards" : "Ανατεθειμένες καρτέλες",
    "No notifications" : "Δεν υπάρχουν ειδοποιήσεις",
    "Delete board" : "Διαγραφή πίνακα",
    "Importing board..." : "Εισαγωγή πίνακα...",
    "Board imported successfully" : "Ο πίνακας εισήχθη επιτυχώς",
    "Import board" : "Εισαγωγή πίνακα",
    "Clone {boardTitle}" : "Κλωνοποίηση {boardTitle}",
    "Clone cards" : "Κλωνοποίηση καρτών",
    "Clone assignments" : "Κλωνοποίηση αναθέσεων",
    "Clone labels" : "Κλωνοποίηση ετικετών",
    "Clone due dates" : "Κλωνοποίηση προθεσμιών",
    "Advanced options" : "Προχωρημένες επιλογές",
    "Move all cards to the first list" : "Μετακίνηση όλων των καρτών στην πρώτη λίστα",
    "Restore archived cards" : "Επαναφορά αρχειοθετημένων καρτών",
    "Clone" : "Κλωνοποίηση",
    "Export {boardTitle}" : "Εξαγωγή {boardTitle}",
    "Export as JSON" : "Εξαγωγή ως JSON",
    "Export as CSV" : "Εξαγωγή ως CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Σημείωση: Μόνο η μορφή JSON υποστηρίζεται για εισαγωγή πίσω στην εφαρμογή Deck.",
    "Export" : "Εξαγωγή",
    "Loading filtered view" : "Φόρτωση εμφάνισης με βάση το φίλτρο",
    "Search for {searchQuery} in other boards" : "Αναζήτηση για {searchQuery} σε άλλους πίνακες",
    "Search for {searchQuery} in all boards" : "Αναζήτηση για {searchQuery} σε όλους τους πίνακες",
    "No results found" : "Δεν βρέθηκαν αποτελέσματα",
    "Deck board {name}\n* Last modified on {lastMod}" : "Πίνακας Deck {name}\n* Τελευταία τροποποίηση στις {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Δημιουργήθηκε στις {created}\n* Τροποποιήθηκε στις {lastMod}\n* {nbAttachments} συνημμένα\n* {nbComments} σχόλια",
    "{nbCards} cards" : "{nbCards} κάρτες",
    "Due on {date}" : "Προθεσμία στις {date}",
    "{stack} in {board}" : "{stack} στο {board}",
    "Click to expand description" : "Κλικ για επέκταση περιγραφής",
    "Click to expand comment" : "Κλικ για επέκταση σχολίου",
    "Create card" : "Δημιουργία καρτέλας",
    "Create a new card" : "Δημιουργία νέας καρτέλας",
    "Card title" : "Τίτλος καρτέλας",
    "Creating the new card …" : "Γίνεται δημιουργία της νέας καρτέλας...",
    "Card \"{card}\" was added to \"{board}\"" : "Η καρτέλα \"{card}\" προστέθηκε στο \"{board}\"",
    "Open card" : "Άνοιγμα καρτέλας",
    "Close" : "Κλείσιμο",
    "No upcoming cards" : "Δεν υπάρχουν επερχόμενες καρτέλες",
    "upcoming cards today" : "επερχόμενες κάρτες σήμερα",
    "upcoming cards tomorrow" : "επερχόμενες κάρτες αύριο",
    "upcoming cards" : "επερχόμενες καρτέλες",
    "New card" : "Νέα κάρτα",
    "Link to a board" : "Σύνδεσμος στον πίνακα",
    "Link to a card" : "Σύνδεσμος σε καρτέλα",
    "Create a card" : "Δημιουργία καρτέλας",
    "Message from {author} in {conversationName}" : "Μήνυμα από {author} σε {conversationName}",
    "Something went wrong" : "Κάτι πήγε στραβά",
    "Failed to upload {name}" : "Αποτυχία μεταφόρτωσης {name}",
    "Maximum file size of {size} exceeded" : "Υπέρβαση επιτρεπόμενου μεγέθους αρχείου {size}",
    "Assigned users" : "Ανατεθειμένοι χρήστες",
    "Due date" : "Προθεσμία",
    "Created" : "Δημιουργήθηκε",
    "Modified" : "Τροποποιήθηκε",
    "Error creating the share" : "Σφάλμα κατά τη δημιουργία της κοινοποίησης",
    "Share with a Deck card" : "Μοιραστείτε με μια καρτέλα Deck",
    "Share {file} with a Deck card" : "Μοιραστείτε το {file} με μια καρτέλα Deck",
    "Share" : "Μοιραστείτε",
    "The card \"%s\" on \"%s\" has reached its due date." : "Η καρτέλα \"%s\" στο \"%s\" έχει λήξει.",
    "Share board with a user, group or team …" : "Κοινή χρήση πίνακα με έναν χρήστη, μια ομάδα ή team ...",
    "Searching for users, groups and teams …" : "Αναζήτηση χρηστών, ομάδων και teams …",
    "No participants found" : "Δε βρέθηκαν συμμετέχοντες",
    "Save" : "Αποθήκευση",
    "seconds ago" : " δευτερόλεπτα πριν ",
    "Today" : "Σήμερα",
    "Tomorrow" : "Αύριο",
    "No due" : "Χωρίς λήξη"
},
"nplurals=2; plural=(n != 1);");
