OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "لقد قمت بانشاء لوح جديد {board}",
    "{user} has created a new board {board}" : "{user} قام بانشاء لوح {board}",
    "You have deleted the board {board}" : "قمت بمسح اللوح {board}",
    "{user} has deleted the board {board}" : "{user} مَسحْ اللوح {board}",
    "You have restored the board {board}" : "لقد استعدت اللوح {board}",
    "{user} has restored the board {board}" : "{user} قام باسترجاع اللوح {board}",
    "You have shared the board {board} with {acl}" : "قمت بمشاركة اللوح {board} مع {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} قام بمشاركة اللوح {board} مع {acl}",
    "You have removed {acl} from the board {board}" : "قمت بإزالة {acl} من اللوح {board}",
    "{user} has removed {acl} from the board {board}" : "{user} قام بازالة {acl} من اللوح {board}",
    "You have renamed the board {before} to {board}" : "قمت بإعادة تسمية اللوح من {before} الى {board}",
    "{user} has renamed the board {before} to {board}" : "{user} قام بإعادة تسمية اللوح من {before} الى {board}",
    "You have archived the board {board}" : "لقد ارشفت اللوح {board}",
    "{user} has archived the board {before}" : "{user} قام بأرشفة اللوح {before}",
    "You have unarchived the board {board}" : "قمت بالغاء ارشفة اللوح {board}",
    "{user} has unarchived the board {before}" : "{user} قام بالغاء ارشفة اللوح {before}",
    "You have created a new list {stack} on board {board}" : "لقد قمت بانشاء قائمة جديدة {stack} على اللوح {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} قام بانشاء قائمة جديدة {stack} على اللوح {board}",
    "You have renamed list {before} to {stack} on board {board}" : "قمت باعادة تسمية القائمة من {before} إلى {stack} على اللوح {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} قام باعادة تسمية القائمة {before} إلى {stack} على اللوح {board}",
    "You have deleted list {stack} on board {board}" : "قمت بمسح القائمة  {stack} على اللوح {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} قام بمسح القائمة {stack} على اللوح {board}",
    "You have created card {card} in list {stack} on board {board}" : "لقد قمت بانشاء بطاقة {card} في القائمة {stack} على اللوح {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} قام بانشاء بطاقة {card} في القائمة {stack} على اللوح {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "قمت بمسح بطاقة {card} في القائمة {stack} على اللوح {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} قام بحذف بطاقة {card} في القائمة {stack} على اللوح {board}",
    "You have renamed the card {before} to {card}" : "قمت بإعادة تسمية اللوح من {before} إلى {card}",
    "{user} has renamed the card {before} to {card}" : "{user} قام بإعادة تسمية اللوح من {before} إلى {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "قمت بإضافة وصف إلى البطاقة {card} في القائمة {stack} على اللوح {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} قام بإضافة وصف إلى البطاقة {card} في القائمة {stack} على اللوح {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "قمت بتحديث الوصف إلى البطاقة {card} في القائمة {stack} على اللوح {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} قام بتحديث الوصف إلى البطاقة {card} في القائمة {stack} على اللوح {board}",
    "You have archived card {card} in list {stack} on board {board}" : "لقد قمت بأرشفة بطاقة {card} في القائمة {stack} على اللوح {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} قام بأرشفة بطاقة {card} في القائمة {stack} على اللوح {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "لديك بطاقة غير مؤرشفة {card} في القائمة {stack} على اللوح {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} لديه بطاقة غير مؤرشفة {card} في القائمة {stack} على اللوح {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "لقد قُمت بتمييز البطاقة {card} باعتبارها \"منتهية\" في القائمة {stack} على اللوحة {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "قام المستخدِم {user} بتمييز البطاقة {card} باعتبارها \"منتهية\" في القائمة {stack} على اللوحة{board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "لقد قُمتَ أنت بتمييز البطاقة {card} باعتبارها \"غير منتهية\" في القائمة {stack} على اللوحة {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "قام المستخدِم {user} بتمييز البطاقة {card} باعتبارها \"غير منتهية\" في القائمة {stack} على اللوحة {board}",
    "You have removed the due date of card {card}" : "قمت بإزالة التاريخ المحدد من البطاقة {card}",
    "{user} has removed the due date of card {card}" : "{user} قام بإزالة التاريخ المحدد من البطاقة {card}",
    "You have set the due date of card {card} to {after}" : "قمت بتعيين التاريخ المحدد للبطاقة {card} إلى {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} قام بتعيين التاريخ المحدد للبطاقة {card} إلى {after}",
    "You have updated the due date of card {card} to {after}" : "قمت بتحديث التاريخ المحدد للبطاقة {card} إلى {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} قام بتحديث التاريخ المحدد للبطاقة {card} إلى {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "قمت بإضافة وسم {label} إلى البطاقة {card} في القائمة {stack} على اللوح {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} قام بإضافة وسم {label} إلى البطاقة {card} في القائمة {stack} على اللوح {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "قمت بإزالة وسم {label} من البطاقة {card} في القائمة {stack} على اللوح {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} قام بإزالة وسم {label} من البطاقة {card} في القائمة {stack} على اللوح {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "قمت بتكليف {assigneduser} في بطاقة {card} على اللوح {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} قام بتكليف {assigneduser} في بطاقة {card} على اللوح {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "لقد إلغيت تكليف {assigneduser} من البطاقة {card} على اللوح {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} الغى تكليف {assigneduser} من البطاقة {card} على اللوح {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : " قمت بنقل البطاقة {card} من القائمة {stackBefore} إلى {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} قام بنقل البطاقة {card} من القائمة {stackBefore} إلى {stack}",
    "You have added the attachment {attachment} to card {card}" : "قمت بإضافة المرفق {attachment} إلى البطاقة {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} قام بإضافة المرفق {attachment} إلى البطاقة {card}",
    "You have updated the attachment {attachment} on card {card}" : "قمت بتحديث المرفق {attachment} على البطاقة {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} قام بتحديث المرفق {attachment} على البطاقة {card}",
    "You have deleted the attachment {attachment} from card {card}" : "قمت بحذف المرفق {attachment} من البطاقة {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} قام بحذف المرفق {attachment} من البطاقة {card}",
    "You have restored the attachment {attachment} to card {card}" : "قمت باسترجاع المرفق {attachment} إلى البطاقة {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} قام باسترجاع المرفق {attachment} إلى البطاقة {card}",
    "You have commented on card {card}" : "لقد علقت على البطاقة {بطاقة}",
    "{user} has commented on card {card}" : "{مستخدم} علق على بطاقة {بطاقة}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "تغييرات في <strong>تطبيق Deck</strong> ",
    "A <strong>board, list or card</strong> was changed" : "تمّ تغيير <strong>لوحة أو قائمة أو بطاقة </strong>.",
    "A <strong>comment</strong> was created on a card" : "<strong>تعليق</strong> تم إنشاؤه على بطاقة",
    "A <strong>card description</strong> has been changed" : "تمّ تغيير <strong>وصف بطاقة</strong>.",
    "The file was uploaded" : "تم رفع الملف",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "حجم الملف المرفوع قد تجاوز  upload_max_filesize الموجودة في ملف php.ini ",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "حجم الملف الذي تريد رفعه أعلى من الحد المسموح به في واجهة ال HTML.",
    "The file was only partially uploaded" : "الملف قد رُفع جزءا منه فقط",
    "No file was uploaded" : "لم يتم رفع أي ملف",
    "Missing a temporary folder" : "المجلد المؤقت غير موجود",
    "Could not write file to disk" : "لم يستطع كتابة ملف للقرص",
    "A PHP extension stopped the file upload" : "إضافة البي اچ بي PHP أوقفت رفع الملف",
    "No file uploaded or file size exceeds maximum of %s" : "لم يتم تحميل أي ملف أو أن يتجاوز حجم الملف الحد الأقصى %s",
    "Cards due today" : "البطاقات المستحقة اليوم",
    "Cards due tomorrow" : "البطاقات المستحقة غداً",
    "Upcoming cards" : "البطاقات القادمة",
    "Load more" : "حمّل أكثر",
    "Welcome to Nextcloud Deck!" : "أهلا بكم في تطبيق \"الكدسات\" Deck من نكست كلاود! ",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "البطاقة \"%s\" على \"%s\" كُلفت بها من قبل %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} قام بتكليف البطاقة {deck-board} على {deck-board} لك",
    "The card {deck-card} on {deck-board} has reached its due date." : "تجاوزت البطاقة {deck-card} على {deck-board} التاريخ المحدد.",
    "%s has mentioned you in a comment on \"%s\"." : "%s أشار إليك في تعليق على \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} أشار إليك في تعليق على {deck-card}",
    "The board \"%s\" has been shared with you by %s." : "تمت مشاركة اللوح \"%s\" معك من قبل %s.",
    "{user} has shared {deck-board} with you." : "{user} قام بمشاركة {deck-board} معك.",
    "Deck board" : "لوحة البطاقات",
    "Owned by %1$s" : "يملكها %1$s",
    "Deck boards, cards and comments" : "اللوحات و البطاقات و الملاحظات",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : " من %1$s، في %2$s/%3$s، يملكها %4$s",
    "Create a new deck card" : "إنشاء كدسة جديدة للبطاقات",
    "Card comments" : "تعليقات البطاقة ",
    "%s on %s" : "%s على %s",
    "Deck boards and cards" : "اللوحات و البطاقات",
    "No data was provided to create an attachment." : "لا بيانات تم تقديمها لانشاء مرفق",
    "Finished" : "اكتمل",
    "To review" : "لإعادة المراجعة",
    "Action needed" : "يحتاج إلى إجراء",
    "Later" : "لاحقا",
    "copy" : "نسخ",
    "Read more inside" : "إقرإ المزيد بالداخل",
    "Custom lists - click to rename!" : "قوائم مخصصة - أنقُر لتغيير التسمية!",
    "To Do" : "مَهَامٌّ",
    "In Progress" : "قيد التجهيز",
    "Done" : "أُنجز",
    "1. Open to learn more about boards and cards" : "1. إفتَح لتعرِف المزيد عن اللوحات والبطاقات",
    "2. Drag cards left and right, up and down" : "2. سحب البطاقات يميناً ويساراً، لأعلي وأسفل",
    "3. Apply rich formatting and link content" : "3. تطبيق التنسيق و ربط المحتوى",
    "4. Share, comment and collaborate!" : "4. مشاركات، و ملاحظات، وتعاون!",
    "Create your first card!" : "أنشِئ بطاقتك الأولى!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : " هذه الملاحظات فيها أكثر من %s حرفاً.\n تمّت إضافتها كمُرفق إلى البطاقة التي اسمها%s.\n الموجودة على الرابط %s.",
    "Attachments" : "المرفقات",
    "File" : "File",
    "Card not found" : "لم يتم العثور على البطاقة ",
    "Path is already shared with this card" : "تم مشاركة المسار بالفعل مع هذه البطاقة ",
    "Invalid date, date format must be YYYY-MM-DD" : "تاريخ غير صحيح, يجب أن يكون تنسيق التاريخ YYYY-MM-DD",
    "Personal planning and team project organization" : "التخطيط الشخصي و تنظيم مشروع الفريق",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "التطبيق \"كَدسَات البطاقات\" Deck هو أداة لتنظيم المهام الشخصية و الجماعية في نكست كلاود على نمط \"خطوط التصنيع أو التجميع في المصانع\" و المعروف بـ kanban.\n\n\n- 📥 أكتب مهامّك او مهامّ فريقك على بطاقة و ضع البطاقة في مكانها المناسب\n- 📄 دوّن ملاحظاتك الإضافية على البطاقة بصيغة ماركداون markdown\n- 🔖 و يمكنك أيضاً وضع لواصق labels على البطاقة لتصنيفها بشكل أفضل\n- 👥 شارك البطاقة مع من ترغب من أعضاء فريقك أو عائلتك أو أصدقائك\n- 📎 إرفاق ملفات و تضمينها بصيغة ماركداون في البطاقة\n- 💬ناقش البطاقة مع زملائك في خانة الملاحظات\n- ⚡ تابع ما يُستجد من عمليات و تطوّرات على البطاقة أو المهمة\n- 🚀 حافظ على مشروعك مُنسّقاً باستمرار.",
    "Add board" : "إضافة لوح",
    "Card details" : "تفاصيل البطاقة",
    "Select the board to link to a project" : "تحديد اللوح لربطه بمشروع",
    "Search by board title" : "بحث بواسطة عنوان اللوح",
    "Select board" : "حدد لوح",
    "Move/copy card" : "نسخ أو نقل بطاقة",
    "Select a board" : "حدد لوح",
    "No lists available" : "لا يوجد قائمة متاحه ",
    "Select a list" : "اختر قائمة ",
    "Move card" : "حرك البطاقة",
    "Copy card" : "نسخ بطاقة",
    "Select the card to link to a project" : "حدد البطاقة لربطها بمشروع",
    "Link to card" : "اربط بطاقة",
    "Select a card" : "حدد بطاقة",
    "Cancel" : "إلغاء",
    "This board is read only" : "هذا اللوح بوضع القراءة فقط",
    "Drop your files to upload" : "افلت الملفات لرفعها",
    "File already exists" : "الملف موجود مسبقاً",
    "A file with the name {filename} already exists." : "ملف باسم {اسم الملف} موجود مسبقًا ",
    "Do you want to overwrite it?" : "هل تريد تجاوزه؟",
    "Overwrite file" : "تجاوز ملف",
    "Keep existing file" : "ابقي الملف الموجود",
    "Add card" : "إضافة بطاقة",
    "Archived cards" : "البطاقات المؤرشفة",
    "Add list" : "إضافة قائمة",
    "List name" : "قامة اسماء",
    "Clear search" : "مَحوُ البحث",
    "Active filters" : "فلاتر نشطة",
    "Apply filter" : "تطبيق التصفية",
    "Filter by tag" : "تصفية بواسطة الوسم",
    "Filter by assigned user" : "تصفية بواسطة المستخدم الموكل اليه",
    "Unassigned" : "غير معيين ",
    "Filter by status" : "تصفية بحسب الحالة",
    "Open and completed" : "مفتوحة و منجزة",
    "Open" : "فتح",
    "Completed" : "مُكتمل",
    "Filter by due date" : "تصفية بواسطة تاريخ الإنجاز",
    "Overdue" : "تأخر",
    "Next 24 hours" : "ال 24 ساعة القادمة",
    "Next 7 days" : "ال 7 أيام القادمة",
    "Next 30 days" : "ال 30 يوم القادمة",
    "No due date" : "لا تاريخ إنجاز",
    "Clear filter" : "ازل التصفية",
    "View Modes" : "وضعيات العرض",
    "Toggle View Modes" : "التبديل بين وضعيات العرض",
    "Hide archived cards" : "إخفاء البطاقات المؤرشفة ",
    "Show archived cards" : "اظهر البطاقات المؤرشفة",
    "Toggle compact mode" : "تبديل النمط المضغوط",
    "Hide card cover images" : "إخفِ صور غلاف البطاقات",
    "Show card cover images" : "أظهِر صور غلاف البطاقات",
    "Open details" : "افتح التفاصيل",
    "Details" : "التفاصيل",
    "Deck settings" : "إعدادات \"كَدسَات البطاقات\"",
    "General" : "عامٌّ",
    "Use bigger card view" : "استخدام بطاقة عرض أكبر ",
    "Appearance" : "المظهر",
    "Show card ID badge" : "أعرض شارة رقم البطاقة",
    "Show boards in calendar/tasks" : "إظهار اللوح في التقويم/المهام",
    "Admin settings" : "إعدادات المُشرِف",
    "Limit board creation to some groups" : "قَصْرُ حَقِّ إنشاء اللوحات على بعض المجموعات ",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "المستخدِمون من خارج تلك المجموعات لن يمكنكم إنشاء لوحاتهم الخاصة؛ لكن يمكنهم العمل على اللوحات التي تتمّ مشاركتها معهم.",
    "Board actions" : "إجراءات اللوحة",
    "Scroll sideways" : "تحريك للجنب",
    "Navigate between cards" : "التنقُّل بين البطاقات",
    "Close card details" : "أغلِق تفاصيل البطاقة",
    "Search" : "بحث",
    "Show card filters" : "أظهِر فلاتر البطاقة",
    "Clear card filters" : "إمحُ فلاتر البطاقة",
    "Show those shortcuts" : "أعرُض تلك الاختصارات",
    "Card actions" : "إجراءات البطاقة",
    "Open card details" : "إفتَح تفاصيل البطاقة",
    "Edit the card title" : "عدِّل عنوان البطاقة",
    "Assign yourself to the current card" : "أسنِد هذه البطاقة إلى نفسِك",
    "Archive/unarchive the current card" : "أرشِف أو ألغِ ترشيف البطاقة الحالية",
    "Mark card as completed/not completed" : "علِّم بطاقة كـ\"منجزة\" أو \"غير منجزة\"",
    "Open card menu" : "إفتَح قائمة البطاقات",
    "Currently present people" : "الأشخاص الحاضرون الآن",
    "Loading board" : "جارِ تحميل اللوح..",
    "Board not found" : "اللوح غير موجود",
    "Create a new list to add cards to this board" : "إنشاء قائمة جديدة لإضافة البطاقات على هذا اللوح",
    "Sharing" : "المشاركة",
    "Tags" : "الوسوم",
    "Deleted items" : "العناصر الممسوحة",
    "Activity" : "النشاط",
    "Deleted lists" : "القوائم المحذوفة",
    "Undo" : "تراجع",
    "Deleted cards" : "البطاقات المحذوفة",
    "Hours" : "ساعات",
    "Days" : "أيام",
    "Weeks" : "أسابيع",
    "Months" : "شهور",
    "No cards yet" : "لا توجد بطاقات حتى الآن",
    "Failed to create share with {displayName}" : "فشل في إنشاء المشاركة مع {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "هل أنت متأكد أنك تريد نقل اللوحة {title} إلى {user} ؟",
    "Transfer the board." : "نقل اللوحة.",
    "Transfer" : "نقل",
    "The board has been transferred to {user}" : "تمّ نقل اللوحة إلى {user}",
    "Failed to transfer the board to {user}" : "تعذّر نقل اللوحة إلى {user}",
    "Board owner" : "منشئ اللوح",
    "(Group)" : "(مجموعة)",
    "(Team)" : "(الفريق)",
    "(remote)" : "(عن بعد)",
    "Can edit" : "يمكن تعديله",
    "Can share" : "يمكن مشاركته",
    "Can manage" : "يمكن إدارته",
    "Owner" : "المالك",
    "Delete" : "حذف ",
    "List deleted" : "القائمة حٌذفت ",
    "Edit list title" : "تعديل اسم القائمة",
    "Archive all cards" : "أرشفة جميع البطاقات ",
    "Unarchive all cards" : "تراجع عن أرشفة كل البطاقات",
    "Delete list" : "حذف القائمة",
    "Archive all cards in this list" : "أرشفة جميع البطاقات في القائمة ",
    "Unarchive all cards in this list" : "تراجع عن أرشفة كل البطاقات في هذه القائمة",
    "Add a new card" : "إضافة بطاقة جديدة",
    "Card name" : "اسم البطاقة ",
    "title and color value must be provided" : "العنوان و قيمة اللون يجب تقديمها ",
    "Edit" : "تعديل",
    "Add a new tag" : "إضافة وسم جديد",
    "Board name" : "اسم اللوح",
    "Members" : "الاعضاء",
    "Assign to users/groups/team" : "إسناد إلى مستخدمين/مجموعات/فرق",
    "Assign a user to this card…" : "انسب مستخدم الى هذه البطاقة..",
    "Select a user to assign to this card…" : "حدِّد مستخدِماً ليتم إسناد هذه البطاقة له...",
    "File to share" : "ملف للمشاركة",
    "Invalid path selected" : "تم تحديد مسار غير صحيح",
    "Upload new files" : "رفع ملفات جديدة",
    "Share from Files" : "مشاركة من الملفات",
    "Pending share" : "مشاركة مُعلّقة",
    "Add this attachment" : "إضافة هذا المرفق",
    "Show in Files" : "عرض في الملفات ",
    "Download" : "تنزيل",
    "Remove attachment" : "حذف المُرفق",
    "Delete Attachment" : "مسح المرفق",
    "Restore Attachment" : "إستعادة المرفق",
    "The title cannot be empty." : "لا يمكن ترك العنوان فارغًا.",
    "Open in sidebar view" : "فتح في الشريط الجانبي ",
    "Open in bigger view" : "فتح في نافذة أكبر ",
    "Comments" : "تعليقات",
    "Failed to load comments" : "فشل في تحميل التعليقات ",
    "No comments yet. Begin the discussion!" : "لا يوجد تعليقات بعد, ابدأ النقاش الآن!",
    "The comment cannot be empty." : "التعليق لايمكن ان يكون فارغا.",
    "The comment cannot be longer than 1000 characters." : "التعليق لا يمكن ان يكون اطول من 1000 حرف.",
    "Submit" : "إرسال ",
    "Created:" : "تاريخ الإنشاء :",
    "In reply to" : "يقوم بالرد على",
    "Cancel reply" : "إلغاء الرد",
    "Reply" : "رد",
    "Update" : "تحديث",
    "Mark as done" : "تمييز كمُنجَز",
    "Mark as not done" : "تمييز كـ \"غير منتهٍ\"",
    "Write a description …" : "كتابة وصف...",
    "Description" : "الوصف",
    "(Unsaved)" : "(غير محفوظ)",
    "(Saving…)" : "(يُحفظ..)",
    "Formatting help" : "مساعدة التنسيق",
    "Edit description" : "تعديل الوصف",
    "View description" : "إظهار الوصف",
    "Add Attachment" : "أضف ملحق",
    "Choose attachment" : "اختيار مرفق",
    "Select Date" : "اختر التاريخ ",
    "Later today – {timeLocale}" : "في وقت لاحقٍ اليوم – {timeLocale}",
    "Set due date for later today" : "تعيين آخِرَ هذا اليوم كموعدٍ لاستحقاق البطاقة",
    "Tomorrow – {timeLocale}" : "غداً – {timeLocale}",
    "Set due date for tomorrow" : "تعيين يوم الغد كموعدٍ لاستحقاق البطاقة",
    "This weekend – {timeLocale}" : "نهاية هذا الأسبوع – {timeLocale}",
    "Set due date for this weekend" : "تعيين نهاية هذا الأسبوع كموعدٍ لاستحقاق البطاقة",
    "Next week – {timeLocale}" : "الأسبوع القادم – {timeLocale}",
    "Set due date for next week" : "تعيين الأسبوع المُقبل كموعدٍ لاستحقاق البطاقة",
    "Assign a due date to this card…" : "تعيين تاريخ استحقاق لهذه البطاقة...",
    "Set a due date" : "تعيين تاريخ الانجاز",
    "Add due date" : "إضافة تاريخ الاستحقاق",
    "Choose a date" : "إختَر تاريخاً",
    "Remove due date" : "ازالة تاريخ الانجاز",
    "Due at:" : "تاريخ الاستحقاق:",
    "Not done" : "غير مُنجَزة",
    "Unarchive card" : "إلغاء أرشفة البطاقة ",
    "Archive card" : "أرشفة البطاقة",
    "Assign a tag to this card…" : "انسب وسم الى هذه البطاقة..",
    "Select or create a tag…" : "اختيار أو إنشاء وسم جديد...",
    "Create a new tag:" : "إنشاء وسم جديد:",
    "(group)" : "(مجموعة)",
    "{count} comments, {unread} unread" : "{عدد} التعليقات، {غير مقروءة} غير مقروءة",
    "Todo items" : "قائمة المهام",
    "Edit card title" : "تعديل اسم القائمة",
    "Open link" : "إفتَح الرابط",
    "Card deleted" : "البطاقة حٌذفت ",
    "Edit title" : "تحرير عنوان",
    "Assign to me" : "ينسب لي",
    "Unassign myself" : "إلغاء تعييني ",
    "Delete card" : "حذف البطاقة",
    "All boards" : "جميع الالواح",
    "Archived boards" : "الالواح المؤرشفة",
    "Shared with you" : "تم مشاركتها معك",
    "Cancel edit" : "إلغاء التعديل",
    "Save board" : "حفظ اللوحة",
    "Board {0} deleted" : "اللوح {0} تم مسحه",
    "All cards" : "جميع البطاقات ",
    "Only assigned cards" : "البطاقات المخصصة فقط",
    "No reminder" : "لا يوجد تذكير ",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "هل أنت متأكد من حذف اللوحة {title}؟ هذا سيؤدي إلى حذف كل بيانات هذه اللوحة بما في ذلك البطاقات المؤرشفة.",
    "Delete the board?" : "مسح اللوح؟",
    "Leave" : "إجازة",
    "An error occurred" : "حدث خطأ",
    "Board details" : "تفاصيل لوح",
    "Edit board" : "تعديل اللوح",
    "Clone board" : "استنسخ اللوح",
    "Unarchive board" : "إلغاء أرشفة اللوح",
    "Archive board" : "أرشفة اللوح ",
    "Export board" : "تصدير لوحة",
    "Turn on due date reminders" : "تفعيل التذكيرات للموعد المحدد ",
    "Turn off due date reminders" : "إيقاف تشغيل التذكيرات للموعد المحددد",
    "Due date reminders" : "تذكيرات الموعد المحدد ",
    "Assigned cards" : "البطاقات المخصصة",
    "No notifications" : "لا توجد إشعارات",
    "Delete board" : "مسح اللوح",
    "Clone {boardTitle}" : "استنساخ {boardTitle}",
    "Clone cards" : "استنسخ البطاقة ",
    "Clone assignments" : "استنساخ مهام",
    "Clone labels" : "استنساخ عناوين",
    "Clone due dates" : "استنساخ تواريخ استحقاق",
    "Advanced options" : "الخيارات المتقدمة",
    "Move all cards to the first list" : "نقل كل البطاقات للقائمة الأولى",
    "Restore archived cards" : "استرجاع البطاقات المؤرشفة",
    "Clone" : "استنسخ",
    "Export as CSV" : "صدِّر كـ CSV",
    "Export" : "تصدير",
    "Loading filtered view" : "جارِ تحميل التصفية ",
    "Search for {searchQuery} in all boards" : "البحث عن {searchQuery} في جميع اللوح",
    "No results found" : "لا يتم العثور على أي نتائج ",
    "Deck board {name}\n* Last modified on {lastMod}" : "اسم اللوحة {name} \n* آخر تعديل لها في {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* أُنشأت في {created}\n* آخر تعديل لها في {lastMod}\n* {nbAttachments} مُرفقات\n* {nbComments} مُلاحظات",
    "{nbCards} cards" : "{nbCards} بطاقات",
    "Due on {date}" : "مطلوب في {date}",
    "{stack} in {board}" : "{stack} في  {board}",
    "Click to expand description" : "إضعط للتوسّع في الوصف",
    "Click to expand comment" : "إضغط للتوسع في الملاحظات",
    "Create card" : "أنشى بطاقة ",
    "Create a new card" : "أنشى بطاقة جديدة ",
    "Card title" : "عنوان البطاقة ",
    "Creating the new card …" : "أنشى البطاقة الجديدة ",
    "Card \"{card}\" was added to \"{board}\"" : "\"{بطاقة}\" تمت إضافتها في \"{اللوح}\"",
    "Open card" : "فتح البطاقة ",
    "Close" : "إغلاق",
    "No upcoming cards" : "لا يوجد بطاقات قادمة ",
    "upcoming cards today" : "البطاقات القادمة اليومَ",
    "upcoming cards tomorrow" : "البطاقات القادمة غداً",
    "upcoming cards" : "البطاقات القادمة",
    "New card" : "بطاقة جديدة",
    "Link to a board" : "ربط بلوح",
    "Link to a card" : "ربط ببطاقة",
    "Create a card" : "أنشى بطاقة ",
    "Message from {author} in {conversationName}" : "رسالة من {author} في {conversationName}",
    "Something went wrong" : "حدث خطأ ما",
    "Failed to upload {name}" : "فشل في رفع {اسم}",
    "Maximum file size of {size} exceeded" : " تجاوز الملف الحد الاقصى {size} بالحجم المسموح ",
    "Due date" : "تاريخ الاكتمال",
    "Created" : "أُنشئ",
    "Modified" : "عُدل",
    "Error creating the share" : "خطأ في إنشاء المشاركة",
    "Share with a Deck card" : "مشاركة مع بطاقة Deck",
    "Share {file} with a Deck card" : "مشاركة الملف {file} مع بطاقة Deck",
    "Share" : "مشاركة ",
    "The card \"%s\" on \"%s\" has reached its due date." : "وصلت بطاقة \"%s\" على %s\" الموعد المحدد ",
    "Share board with a user, group or team …" : "مشاركة اللوحة مع مستخدِم او مجموعة أو فريق ...",
    "Searching for users, groups and teams …" : "البحث عن مستخدِمين، أو مجموعات، أو فرق",
    "No participants found" : "لم يتم العثور على مشاركين",
    "Save" : "حفظ",
    "seconds ago" : "ثوانٍ مضت",
    "Today" : "اليوم",
    "Tomorrow" : "غدا",
    "No due" : "غير محدد"
},
"nplurals=6; plural=n==0 ? 0 : n==1 ? 1 : n==2 ? 2 : n%100>=3 && n%100<=10 ? 3 : n%100>=11 && n%100<=99 ? 4 : 5;");
