OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Você criou um novo painel {board}",
    "{user} has created a new board {board}" : "{user} criou um novo painel {board}",
    "You have deleted the board {board}" : "Você excluiu o painel {board}",
    "{user} has deleted the board {board}" : "{user} excluiu o painel {board}",
    "You have restored the board {board}" : "Você restaurou o painel {board}",
    "{user} has restored the board {board}" : "{user} restaurou o painel {board}",
    "You have shared the board {board} with {acl}" : "Você compartilhou o painel {board} com {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} compartilhou o painel {board} com {acl}",
    "You have removed {acl} from the board {board}" : "Você excluiu {acl} do painel {board}",
    "{user} has removed {acl} from the board {board}" : "{user} excluiu {acl} do painel {board}",
    "You have renamed the board {before} to {board}" : "Você renomeou o painel {before} para {board}",
    "{user} has renamed the board {before} to {board}" : "{user} renomeou o painel {before} para {board}",
    "You have archived the board {board}" : "Você arquivou o painel {board}",
    "{user} has archived the board {before}" : "{user} arquivou o painel  {before}",
    "You have unarchived the board {board}" : "Você desarquivou o painel  {board}",
    "{user} has unarchived the board {before}" : "{user} desarquivou o painel {board}",
    "You have created a new list {stack} on board {board}" : "Você criou uma nova lista {stack} no painel {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} criou uma nova lista {stack} no painel {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Você renomeou a lista {before} para {stack} no painel {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} renomeou a lista {before} para {stack} no painel {board}",
    "You have deleted list {stack} on board {board}" : "Você excluiu a lista {stack} do painel {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} excluiu a lista {stack} do painel {board}",
    "You have created card {card} in list {stack} on board {board}" : "Você criou o cartão {card} na lista {stack} do painel {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} criou o cartão {card} na lista {stack} do painel {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Você excluiu o cartão {card} na lista {stack} do painel {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} excluiu o cartão {card} na lista {stack} do painel {board}",
    "You have renamed the card {before} to {card}" : "Você renomeou o cartão {before} para {card}",
    "{user} has renamed the card {before} to {card}" : "{user} renomeou o cartão {before} para {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Você adicionou uma descrição ao cartão {card} na lista {stack} do painel {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} adicionou uma descrição ao cartão {card} na lista {stack} do painel {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Você atualizou a descrição do cartão {card} na lista {stack} do painel {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} atualizou a descrição do cartão {card} na lista {stack} do painel {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Você arquivou o cartão {card} na lista {stack} do painel {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} arquivou o cartão {card} na lista {stack} do painel {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Você desarquivou o cartão {card} na lista {stack} do painel {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} desarquivou o cartão {card} na lista {stack} do painel {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Você marcou o cartão {card} como concluído na lista {stack} no painel {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} marcou o cartão {card} como concluído na lista {stack} no painel {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Você marcou o cartão {card} como não concluído na lista {stack} no painel {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} marcou o cartão {card} como não concluído na lista {stack} no painel {board}",
    "You have removed the due date of card {card}" : "Você excluiu a data de vencimento do cartão {card}",
    "{user} has removed the due date of card {card}" : "{user} excluiu a data de vencimento do cartão {card}",
    "You have set the due date of card {card} to {after}" : "Você definiu a data de vencimento do cartão {card} para {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} definiu a data de vencimento do cartão {card} para {after}",
    "You have updated the due date of card {card} to {after}" : "Você atualizou a data de vencimento do cartão {card} para {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} atualizou a data de vencimento do cartão {card} para {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Você adicionou a etiqueta {label} ao cartão {card} na lista {stack} do painel {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} adicionou a etiqueta {label} ao cartão {card} na lista {stack} do painel {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Você removeu a etiqueta {label} do cartão {card} na lista {stack} do painel {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} removeu a etiqueta {label} do cartão {card} na lista {stack} do painel {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Você vinculou {assigneduser} ao cartão {card} do painel {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} vinculou {assigneduser} ao cartão {card} do painel {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Você desvinculou {assigneduser} do cartão {card} no painel {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} desvinculou {assigneduser} do cartão {card} no painel {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Você moveu o cartão {card} da lista {stackBefore} para {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} moveu o cartão {card} da lista {stackBefore} para {stack}",
    "You have added the attachment {attachment} to card {card}" : "Você adicionou o anexo {attachment} no cartão {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} adicionou um anexo {attachment} ao cartão {card}",
    "You have updated the attachment {attachment} on card {card}" : "Você atualizou o anexo {attachment} do cartão {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} atualizou o anexo {attachment} no cartão {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Você excluiu o anexo {attachment} do cartão {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} excluiu o anexo {attachment} do cartão {card}",
    "You have restored the attachment {attachment} to card {card}" : "Você restaurou o anexo {attachment} do cartão {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} restaurou o anexo {attachment} do cartão {card}",
    "You have commented on card {card}" : "Você comentou no cartão {card}",
    "{user} has commented on card {card}" : "{user} comentou no cartão {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Mudanças no <strong>aplicativo Deck</strong>",
    "A <strong>board, list or card</strong> was changed" : "Um <strong>painel, lista ou cartão</strong> foi alterado",
    "A <strong>comment</strong> was created on a card" : "Um <strong>comentário</strong> foi criado no cartão",
    "A <strong>card description</strong> has been changed" : "Uma <strong>descrição do cartão</strong> foi alterada",
    "The file was uploaded" : "O arquivo foi enviado",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "O arquivo enviado excede a diretiva upload_max_filesize do php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "O arquivo enviado excede a diretiva MAX_FILE_SIZE especificada no formulário HTML",
    "The file was only partially uploaded" : "O arquivo foi parcialmente enviado",
    "No file was uploaded" : "Nenhum arquivo foi enviado",
    "Missing a temporary folder" : "Falta uma pasta temporária",
    "Could not write file to disk" : "Não foi possível escrever no disco",
    "A PHP extension stopped the file upload" : "Uma extensão PHP parou o upload do arquivo",
    "No file uploaded or file size exceeds maximum of %s" : "Nenhum arquivo enviado ou o tamanho excede o máximo de %s",
    "Invalid file type. Only JSON files are allowed." : "Tipo de arquivo inválido. Somente arquivos JSON são permitidos.",
    "Invalid JSON data" : "Dados JSON inválidos",
    "Failed to import board" : "Falha ao importar painel",
    "Cards due today" : "Cartões com vencimento hoje",
    "Cards due tomorrow" : "Cartões com vencimento amanhã",
    "Upcoming cards" : "Próximos cartões",
    "Load more" : "Carregar mais",
    "Welcome to Nextcloud Deck!" : "Bem-vindo ao Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "O cartão \"%s\" em \"%s\" foi vinculado com você por %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} atribuiu o cartão {deck-card} no {deck-board} a você.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "O cartão \"%1$s\" em \"%2$s\" atingiu a sua data limite.",
    "The card {deck-card} on {deck-board} has reached its due date." : "O cartão {deck-card} em {deck-board} atingiu sua data de vencimento.",
    "%s has mentioned you in a comment on \"%s\"." : "%s citou você num comentário em \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} mencionou você em um comentário em {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "O painel \"%s\" foi compartilhado com você por %s.",
    "{user} has shared {deck-board} with you." : "{user} compartilhou o {deck-board} com você.",
    "The remote board %1$s has been shared with you by %2$s" : "O painel remoto %1$s foi compartilhado com você por %2$s",
    "Deck board" : "Painel de Deck",
    "Owned by %1$s" : "Propriedade de %1$s",
    "Deck boards, cards and comments" : "Paineis, cartões e comentários de Deck",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "De %1$s, em %2$s/%3$s, propriedade de %4$s",
    "Create a new deck card" : "Criar um novo cartão de Deck",
    "Card comments" : "Comentários do cartão",
    "%s on %s" : "%s em %s",
    "Deck boards and cards" : "Paineis e cartões de Deck",
    "No data was provided to create an attachment." : "Nenhum dado foi fornecido para criar um anexo.",
    "Finished" : "Terminado",
    "To review" : "Para revisão",
    "Action needed" : "Ação necessária",
    "Later" : "Depois",
    "copy" : "copiar",
    "Read more inside" : "Leia mais no interior",
    "Custom lists - click to rename!" : "Listas personalizadas - clique para renomear!",
    "To Do" : "A Fazer",
    "In Progress" : "Em Andamento",
    "Done" : "Concluído",
    "1. Open to learn more about boards and cards" : "1. Abra para saber mais sobre painéis e cartões",
    "2. Drag cards left and right, up and down" : "2. Arraste os cartões para a esquerda e para a direita, para cima e para baixo",
    "3. Apply rich formatting and link content" : "3. Aplique formatação avançada e conteúdo de links",
    "4. Share, comment and collaborate!" : "4. Compartilhe, comente e colabore!",
    "Create your first card!" : "Crie seu primeiro cartão!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Este comentário tem mais de %s caracteres.\nEle foi adicionado como um anexo ao cartão de nome %s.\nAcessível na URL: %s.",
    "Attachments" : "Anexos",
    "File" : "Arquivo",
    "date" : "data",
    "Card not found" : "Cartão não encontrado",
    "Path is already shared with this card" : "O caminho já é compartilhado com este cartão",
    "Invalid date, date format must be YYYY-MM-DD" : "Data inválida, o formato da data deve ser AAAA-MM-DD",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Todos os painéis de que você é proprietário, incluindo pilhas, cartões, etiquetas, tarefas e comentários",
    "Personal planning and team project organization" : "Planejamento pessoal e organização de projetos em equipe",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "O Deck é uma ferramenta de organização ao estilo kanban voltada para o planejamento pessoal e para a organização de projetos para equipes, integrada ao Nextcloud.\n\n\n- 📥Adicione suas tarefas aos cartões e coloque-os em ordem\n- 📄 Escreva notas adicionais formatadas em Markdown \n- 🔖 Atribua rótulos para uma organização ainda melhor\n- 👥 Compartilhe com sua equipe, seus amigos ou sua família\n- 📎 Anexe arquivos e incorpore-os à sua descrição em Markdown\n- 💬 Discuta com sua equipe usando comentários\n- ⚡ Acompanhe as alterações no fluxo de atividades \n- 🚀 Organize seu projeto ",
    "Add board" : "Adicionar painel",
    "Card details" : "Detalhes do cartão",
    "Select the board to link to a project" : "Selecione o painel para vincular a um projeto",
    "Search by board title" : "Pesquisar pelo título do painel",
    "Select board" : "Selecionar painel",
    "Move/copy card" : "Mover/copiar cartão",
    "Select a board" : "Selecionar um painel",
    "No lists available" : "Nenhuma lista disponível",
    "Select a list" : "Selecione uma lista",
    "Move card" : "Mover cartão",
    "Copy card" : "Copiar cartão",
    "Select the card to link to a project" : "Selecione o cartão para vincular a um projeto",
    "Link to card" : "Vincular ao cartão",
    "Select a card" : "Selecionar um cartão",
    "Cancel" : "Cancelar",
    "This board is read only" : "Este painel é somente leitura",
    "Drop your files to upload" : "Arraste seus arquivos para enviar",
    "File already exists" : "Arquivo já existe",
    "A file with the name {filename} already exists." : "Um arquivo com o nome {filename} já existe.",
    "Do you want to overwrite it?" : "Quer sobrescrever?",
    "Overwrite file" : "Sobrescrever arquivo",
    "Keep existing file" : "Manter o arquivo existente",
    "Add card" : "Adicionar cartão",
    "Archived cards" : "Cartões arquivados",
    "Add list" : "Adicionar lista",
    "List name" : "Nome da lista",
    "Clear search" : "Limpar pesquisa",
    "Active filters" : "Filtros ativos",
    "Apply filter" : "Aplicar filtro",
    "Filter by tag" : "Filtrar por etiqueta",
    "Filter by assigned user" : "Filtrar por usuário atribuído",
    "Unassigned" : "Não atribuído",
    "Filter by status" : "Filtrar por status",
    "Open and completed" : "Aberto e concluído",
    "Open" : "Aberto",
    "Completed" : "Concluído",
    "Filter by due date" : "Filtrar por data de vencimento",
    "Overdue" : "Vencidos",
    "Next 24 hours" : "Próximas 24 horas",
    "Next 7 days" : "Próximos 7 dias",
    "Next 30 days" : "Próximos 30 dias",
    "No due date" : "Sem data de vencimento",
    "Clear filter" : "Limpar filtro",
    "View Modes" : "Modos de Visualização",
    "Toggle View Modes" : "Alternar Modos de Visualização",
    "Kanban view" : "Visualização Kanban",
    "Gantt view" : "Visualização Gantt",
    "Hide archived cards" : "Ocultar cartões arquivados",
    "Show archived cards" : "Exibir cartões arquivados",
    "Toggle compact mode" : "Alternar modo compacto",
    "Hide card cover images" : "Ocultar imagens de capa de cartão",
    "Show card cover images" : "Mostrar imagens de capa de cartão",
    "Open details" : "Abrir detalhes",
    "Details" : "Detalhes",
    "Deck settings" : "Configurações do Deck",
    "General" : "Geral",
    "Use bigger card view" : "Use uma exibição de cartão maior",
    "Appearance" : "Aparência",
    "Show card ID badge" : "Exibir o distintivo de identificação do cartão",
    "Show boards in calendar/tasks" : "Exibir os painéis em calendários/tarefas",
    "Admin settings" : "Configurações administrativas",
    "Limit board creation to some groups" : "Limitar a criação de painéis a alguns grupos",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Os usuários fora desses grupos não poderão criar seus próprios painéis, mas ainda poderão trabalhar nos painéis que foram compartilhados com eles.",
    "Enable federation" : "Ativar federação",
    "Board actions" : "Ações de painel",
    "Scroll sideways" : "Rolar para o lado",
    "Navigate between cards" : "Navegar entre os cartões",
    "Close card details" : "Fechar detalhes do cartão",
    "Search" : "Pesquisar",
    "Show card filters" : "Mostrar filtros de cartão",
    "Clear card filters" : "Limpar filtros de cartão",
    "Show those shortcuts" : "Mostrar esses atalhos",
    "Card actions" : "Ações de cartão",
    "Open card details" : "Abrir detalhes do cartão",
    "Edit the card title" : "Editar o título do cartão",
    "Assign yourself to the current card" : "Atribuir-se ao cartão atual",
    "Archive/unarchive the current card" : "Arquivar/desarquivar o cartão atual",
    "Mark card as completed/not completed" : "Marcar cartão como concluído/não concluído",
    "Open card menu" : "Abrir menu do cartão",
    "Currently present people" : "Pessoas atualmente presentes",
    "Supported prefixes: {prefixes}. Wrap phrases in double quotes." : "Prefixos suportados: {prefixes}. Coloque as frases entre aspas duplas.",
    "Search cards" : "Pesquisar cartões",
    "Loading board" : "Carregando painel",
    "Board not found" : "Painel não encontrado",
    "Create a new list to add cards to this board" : "Criar uma nova lista para adicionar cartões a este painel",
    "Sharing" : "Compartilhando",
    "Tags" : "Etiquetas",
    "Deleted items" : "Itens excluídos",
    "Activity" : "Atividade",
    "Deleted lists" : "Listas excluídas",
    "Undo" : "Desfazer",
    "Deleted cards" : "Cartões excluídos",
    "Hours" : "Horas",
    "Days" : "Dias",
    "Weeks" : "Semanas",
    "Months" : "Meses",
    "No cards yet" : "Ainda sem cartões",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Defina uma data de início e um prazo nos seus cartões para visualizá-los no gráfico de Gantt",
    "Cards without dates ({count})" : "Cartões sem datas ({count})",
    "Failed to create share with {displayName}" : "Falha ao criar compartilhamento com {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Deseja realmente transferir o painel {title} para {user}?",
    "Transfer the board." : "Transferir o painel.",
    "Transfer" : "Transferir",
    "The board has been transferred to {user}" : "A painel foi transferida para {user}",
    "Failed to transfer the board to {user}" : "Não foi possível transferir o painel para {user}",
    "Board owner" : "Proprietário do painel",
    "(Group)" : "(Grupo)",
    "(Team)" : "(Equipe)",
    "(remote)" : "(remoto)",
    "Can edit" : "Pode editar",
    "Can share" : "Pode compartilhar",
    "Can manage" : "Pode gerenciar",
    "Owner" : "Proprietário",
    "Delete" : "Excluir",
    "List deleted" : "Lista excluída",
    "Edit list title" : "Editar título da lista",
    "Archive all cards" : "Arquivar todos os cartões",
    "Unarchive all cards" : "Desarquivar todos os cartões",
    "Do not set cards as \"done\"" : "Não definir cartões como \"concluído\"",
    "Set cards as \"done\"" : "Definir cartões como \"concluído\"",
    "Delete list" : "Excluir lista",
    "Archive all cards in this list" : "Arquivar todos os cartões desta lista",
    "Unarchive all cards in this list" : "Desarquivar todos os cartões desta lista",
    "Add a new card" : "Adicionar um novo cartão",
    "Card name" : "Nome do cartão",
    "title and color value must be provided" : "o título e o valor da cor devem ser fornecidos",
    "Edit" : "Editar",
    "Add a new tag" : "Adicionar uma nova etiqueta",
    "Search boards" : "Pesquisar painéis",
    "Board name" : "Nome do painel",
    "Members" : "Membros",
    "Assign to users/groups/team" : "Atribuir a usuários/grupos/equipe",
    "Assign a user to this card…" : "Atribuir um usuário a este cartão...",
    "Select a user to assign to this card…" : "Selecione um usuário para atribuir a este cartão…",
    "File to share" : "Arquivo a compartilhar",
    "Invalid path selected" : "Caminho inválido selecionado",
    "Upload new files" : "Enviar novos arquivos",
    "Share from Files" : "Compartilhar de Arquivos",
    "Pending share" : "Compartilhamento pendente",
    "Add this attachment" : "Adicionar este anexo",
    "Show in Files" : "Exibir em Arquivos",
    "Download" : "Baixar",
    "Remove attachment" : "Remover anexo",
    "Delete Attachment" : "Excluir Anexo",
    "Restore Attachment" : "Restaurar Anexo",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Modificado: ⸱{modifiedDate} ⸱ Criado: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Modificado: {modifiedDate}\nCriado: {createdDate}\nCriado por: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Modificado: {modifiedDate}\nCriado: {createdDate}",
    "The title cannot be empty." : "O título não pode ficar em branco.",
    "Cannot close unsaved card!" : "Não é possível fechar um cartão não salvo!",
    "Open in sidebar view" : "Abrir em visualização da barra lateral",
    "Open in bigger view" : "Abrir em  visualização ampliada",
    "Comments" : "Comentários",
    "Failed to load comments" : "Não foi possível carregar os comentários",
    "No comments yet. Begin the discussion!" : "Nenhum comentário ainda. Inicie a conversa!",
    "The comment cannot be empty." : "O comentário não pode zer vazio.",
    "The comment cannot be longer than 1000 characters." : "O comentário não pode ter mais que 1000 caracteres.",
    "Submit" : "Enviar",
    "Created:" : "Criado:",
    "In reply to" : "Em resposta para",
    "Cancel reply" : "Cancelar resposta",
    "Reply" : "Responder",
    "Update" : "Atualizar",
    "Assign dependent cards" : "Atribuir cartões dependentes",
    "Mark as done" : "Marcar como concluído",
    "Mark as not done" : "Marcar como não concluído",
    "Remove dependency" : "Remover dependência",
    "Add dependent card" : "Adicionar cartão dependente",
    "Assign a dependent card…" : "Atribuir um cartão dependente…",
    "Select a dependent card…" : "Selecione um cartão dependente…",
    "Write a description …" : "Escreva uma descrição...",
    "The description has been changed by another user." : "A descrição foi alterada por outro usuário.",
    "Could not save description" : "Não foi possível salvar a descrição",
    "Description" : "Descrição",
    "(Unsaved)" : "(Não salvo)",
    "(Saving…)" : "(Salvando...)",
    "Formatting help" : "Formatando ajuda",
    "Edit description" : "Editar descrição",
    "View description" : "Exibir descrição",
    "Add Attachment" : "Adicionar anexo",
    "Choose attachment" : "Escolher anexo",
    "Select Date" : "Selecionar Data",
    "Later today – {timeLocale}" : "Hoje mais tarde – {timeLocale}",
    "Set due date for later today" : "Definir a data de vencimento para hoje mais tarde",
    "Tomorrow – {timeLocale}" : "Amanhã – {timeLocale}",
    "Set due date for tomorrow" : "Defina a data de vencimento para amanhã",
    "This weekend – {timeLocale}" : "Este fim de semana – {timeLocale}",
    "Set due date for this weekend" : "Defina a data de vencimento para este fim de semana",
    "Next week – {timeLocale}" : "Semana que vem – {timeLocale}",
    "Set due date for next week" : "Defina a data de vencimento para a próxima semana",
    "Assign a due date to this card…" : "Atribuir uma data de vencimento a este cartão…",
    "Set a due date" : "Definir uma data de vencimento",
    "Add due date" : "Adicionar data de vencimento",
    "Choose a date" : "Escolha uma data",
    "Remove due date" : "Remover data de vencimento",
    "Due at:" : "Vencimento em:",
    "Not done" : "Não concluído",
    "Unarchive card" : "Desarquivar cartão",
    "Archive card" : "Arquivar cartão",
    "Assign a start date to this card…" : "Defina uma data de início para este cartão…",
    "Set a start date" : "Definir uma data de início",
    "Add start date" : "Adicionar data de início",
    "Remove start date" : "Remover data de início",
    "Assign a tag to this card…" : "Atribuir uma etiqueta a este cartão...",
    "Select or create a tag…" : "Selecionar ou criar uma etiqueta…",
    "Create a new tag:" : "Criar uma nova etiqueta:",
    "(group)" : "(grupo)",
    "{count} comments, {unread} unread" : "{count} comentários, {unread} não lidos",
    "Todo items" : "Itens a fazer",
    "Edit card title" : "Editar título do cartão",
    "Open link" : "Abrir link",
    "Card deleted" : "Cartão excluído",
    "Change card color" : "Alterar cor do cartão",
    "Edit title" : "Editar título",
    "Assign to me" : "Atribuir a mim",
    "Unassign myself" : "Desatribuir-me",
    "Delete card" : "Excluir cartão",
    "All boards" : "Todos os painéis",
    "Archived boards" : "Painéis arquivados",
    "Shared with you" : "Compartilhado com você",
    "Cancel edit" : "Cancelar edição",
    "Save board" : "Salvar painel",
    "Board {0} deleted" : "Painel {0} excluído",
    "All cards" : "Todos os cartões",
    "Only assigned cards" : "Apenas cartões atribuídos",
    "No reminder" : "Nenhum lembrete",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Deseja realmente excluir o painel {title}? Isso excluirá todos os dados deste painel, inclusive os cartões arquivados.",
    "Delete the board?" : "Excluir o painel?",
    "Are you sure you want to leave the board {title}?" : "Tem certeza de que deseja sair do painel {title}?",
    "Leave the board?" : "Sair do painel?",
    "Leave" : "Sair",
    "Failed to leave the board" : "Falha ao sair do painel",
    "Exporting board..." : "Exportando painel...",
    "An error occurred" : "Ocorreu um erro",
    "Board details" : "Detalhes do painel",
    "Edit board" : "Editar painel",
    "Clone board" : "Clonar painel",
    "Unarchive board" : "Desarquivar painel",
    "Archive board" : "Arquivar painel",
    "Export board" : "Exportar painel",
    "Turn on due date reminders" : "Ativar lembretes de vencimento",
    "Turn off due date reminders" : "Desativar lembretes de vencimento",
    "Remove as default board" : "Remover como painel padrão",
    "Set as default board" : "Definir como painel padrão",
    "Due date reminders" : "Lembretes de vencimento",
    "Assigned cards" : "Cartões atribuídos",
    "No notifications" : "Sem notificações",
    "Delete board" : "Excluir painel",
    "Leave board" : "Sair do painel",
    "Importing board..." : "Importando painel...",
    "Board imported successfully" : "Painel importado com sucesso",
    "Import board" : "Importar painel",
    "Clone {boardTitle}" : "Clonar {boardTitle}",
    "Clone cards" : "Clonar cartões",
    "Clone assignments" : "Clonar atribuições",
    "Clone labels" : "Clonar rótulos",
    "Clone due dates" : "Clonar datas de vencimento",
    "Advanced options" : "Opções avançadas",
    "Move all cards to the first list" : "Mover todos os cartões para a primeira lista",
    "Restore archived cards" : "Restaurar cartões arquivados",
    "Clone" : "Clonar",
    "Export {boardTitle}" : "Exportar {boardTitle}",
    "Export as JSON" : "Exportar como JSON",
    "Export as CSV" : "Exportar como CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Observação: somente o formato JSON é compatível com a importação de volta para o aplicativo Deck.",
    "Export" : "Exportar",
    "Loading filtered view" : "Carregando exibição filtrada",
    "Search for {searchQuery} in other boards" : "Pesquisar {searchQuery} em outros painéis.",
    "Search for {searchQuery} in all boards" : "Pesquisar por {searchQuery} em todos os painéis",
    "No results found" : "Nenhum resultado encontrado",
    "Deck board {name}\n* Last modified on {lastMod}" : "Painel de Deck {name}\n* Última modificação em {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Criado em {created}\n* Última modificação em {lastMod}\n* {nbAttachments} anexos\n* {nbComments} comentários",
    "{nbCards} cards" : "{nbCards} cartões",
    "Due on {date}" : "Vencimento em {date}",
    "{stack} in {board}" : "{stack} em {board}",
    "Click to expand description" : "Clique para expandir a descrição",
    "Click to expand comment" : "Clique para expandir o comentário",
    "Create card" : "Criar cartão",
    "Create a new card" : "Criar um novo cartão",
    "Card title" : "Título do cartão",
    "Creating the new card …" : "Criando o novo cartão …",
    "Card \"{card}\" was added to \"{board}\"" : "O cartão \"{card}\" foi adicionado a \"{board}\" ",
    "Open card" : "Abrir o cartão",
    "Close" : "Fechar",
    "No upcoming cards" : "Não há próximos cartões",
    "upcoming cards today" : "próximos cartões hoje",
    "upcoming cards tomorrow" : "próximos cartões amanhã",
    "upcoming cards" : "próximos cartões",
    "New card" : "Novo cartão",
    "Link to a board" : "Vincular a um painel",
    "Link to a card" : "Vincular a um cartão",
    "Create a card" : "Criar um cartão",
    "Message from {author} in {conversationName}" : "Mensagem de {author} em {conversationName}",
    "Something went wrong" : "Algo deu errado",
    "Failed to upload {name}" : "Falha ao enviar {name}",
    "Maximum file size of {size} exceeded" : "Excedeu o tamanho máximo de arquivo de {size}",
    "Assigned users" : "Usuários atribuídos",
    "Due date" : "Data de vencimento",
    "Created" : "Criado",
    "Modified" : "Modificado",
    "Error creating the share" : "Erro ao criar o compartilhamento",
    "Share with a Deck card" : "Compartilhar com um cartão Deck",
    "Share {file} with a Deck card" : "Compartilhar {file} com um cartão Deck",
    "Share" : "Compartilhar",
    "The card \"%s\" on \"%s\" has reached its due date." : "O cartão \"%s\" em \"%s\" atingiu sua data de vencimento.",
    "Share board with a user, group or team …" : "Compartilhar painel com um usuário, grupo ou equipe…",
    "Searching for users, groups and teams …" : "Pesquisando usuários, grupos e equipes …",
    "No participants found" : "Nenhum participante encontrado",
    "Save" : "Salvar",
    "seconds ago" : "segundos atrás",
    "Today" : "Hoje",
    "Tomorrow" : "Amanhã",
    "No due" : "Sem vencimento"
},
"nplurals=3; plural=(n == 0 || n == 1) ? 0 : n != 0 && n % 1000000 == 0 ? 1 : 2;");
