OC.L10N.register(
    "deck",
    {
    "The file was uploaded" : "ไฟล์อัปโหลดแล้ว",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "ขนาดไฟล์ที่อัปโหลดมีขนาดเกินค่า upload_max_filesize ที่ระบุไว้ใน php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "ไฟล์ที่อัปโหลดมีขนาดใหญ่เกินค่า MAX_FILE_SIZE ที่ระบุไว้ในรูปแบบของ HTML",
    "The file was only partially uploaded" : "ไฟล์ถูกอัปโหลดเพียงบางส่วน",
    "No file was uploaded" : "ไม่มีไฟล์ที่ถูกอัปโหลด",
    "Missing a temporary folder" : "โฟลเดอร์ชั่วคราวขาดหาย",
    "Could not write file to disk" : "ไม่สามารถเขียนไฟล์ลงดิสก์",
    "A PHP extension stopped the file upload" : "ส่วนขยาย PHP ได้หยุดการอัปโหลดไฟล์",
    "Finished" : "เสร็จสิ้น",
    "copy" : "คัดลอก",
    "In Progress" : "อยู่ระหว่างการดำเนินการ",
    "Done" : "เสร็จสิ้น",
    "Attachments" : "ไฟล์แนบ",
    "File" : "ไฟล์",
    "Card not found" : "ไม่พบการ์ด",
    "Invalid date, date format must be YYYY-MM-DD" : "วันที่ไม่ถูกต้อง รูปแบบวันที่จะต้องเป็น ปปปป-ดด-วว",
    "Cancel" : "ยกเลิก",
    "File already exists" : "ไฟล์นี้มีอยู่แล้ว",
    "Add list" : "เพิ่มรายการ",
    "Clear search" : "ล้างการค้นหา",
    "Open" : "เปิด",
    "Completed" : "เสร็จแล้ว",
    "Overdue" : "เกินกำหนด",
    "Details" : "รายละเอียด",
    "General" : "ทั่วไป",
    "Appearance" : "ลักษณะที่ปรากฏ",
    "Search" : "ค้นหา",
    "Sharing" : "การแชร์",
    "Tags" : "แท็ก",
    "Activity" : "กิจกรรม",
    "Undo" : "เลิกทำ",
    "Hours" : "ชั่วโมง",
    "Transfer" : "โอนย้าย",
    "Can edit" : "สามารถแก้ไข",
    "Can share" : "สามารถแชร์",
    "Owner" : "เจ้าของ",
    "Delete" : "ลบ",
    "Delete list" : "ลบรายการ",
    "Edit" : "แก้ไข",
    "Invalid path selected" : "เลือกเส้นทางไม่ถูกต้อง",
    "Download" : "ดาวน์โหลด",
    "Comments" : "ความคิดเห็น",
    "Submit" : "ส่ง",
    "Created:" : "สร้างเมื่อ:",
    "Reply" : "ตอบกลับ",
    "Update" : "อัปเดต",
    "Description" : "รายละเอียด",
    "(group)" : "(กลุ่ม)",
    "Shared with you" : "แชร์กับคุณ",
    "An error occurred" : "เกิดข้อผิดพลาด",
    "Edit board" : "แก้ไขกระดาน",
    "No notifications" : "ไม่มีการแจ้งเตือน",
    "Delete board" : "ลบกระดาน",
    "Advanced options" : "ตัวเลือกขั้นสูง",
    "Export" : "ส่งออก",
    "Close" : "ปิด",
    "Due date" : "วันที่ครบกำหนด",
    "Modified" : "แก้ไขเมื่อ",
    "Share" : "แชร์",
    "Save" : "บันทึก",
    "seconds ago" : "วินาทีที่ผ่านมา",
    "Today" : "วันนี้",
    "Tomorrow" : "พรุ่งนี้"
},
"nplurals=1; plural=0;");
