OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Vi kreis novan tabulon {board}",
    "{user} has created a new board {board}" : "{user} kreis novan tabulon {board}",
    "You have deleted the board {board}" : "Vi forigis tabulon {board}",
    "{user} has deleted the board {board}" : "{user} forigis tabulon {board}",
    "You have restored the board {board}" : "Vi restaŭris tabulon {board}",
    "{user} has restored the board {board}" : "{user} restaŭris tabulon {board}",
    "You have shared the board {board} with {acl}" : "Vi kunhavigis tabulon {board} kun {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} kunhavigis tabulon {board} kun {acl}",
    "You have removed {acl} from the board {board}" : "Vi forigis {acl} el la tabulo {board}",
    "{user} has removed {acl} from the board {board}" : "{user} forigis {acl} el la tabulo {board}",
    "You have renamed the board {before} to {board}" : "Vi alinomis la tabulon {before} al {board}",
    "{user} has renamed the board {before} to {board}" : "{user} alinomis la tabulon {before} al {board}",
    "You have archived the board {board}" : "Vi enarĥivigis la tabulon {board}",
    "{user} has archived the board {before}" : "{user} enarĥivigis la tabulon {before}",
    "You have unarchived the board {board}" : "Vi elarĥivigis la tabulon {board}",
    "{user} has unarchived the board {before}" : "{user} elarĥivigis la tabulon {before}",
    "You have renamed the card {before} to {card}" : "Vi alinomis la karton {before} al {card}",
    "{user} has renamed the card {before} to {card}" : "{user} alinomis la karton {before} al {card}",
    "You have removed the due date of card {card}" : "Vi forigis la limdaton el karto {card}",
    "{user} has removed the due date of card {card}" : "{user} forigis la limdaton el karto {card}",
    "You have set the due date of card {card} to {after}" : "Vi agordis la limdaton de karto {card} je {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} agordis la limdaton de karto {card} je {after}",
    "You have updated the due date of card {card} to {after}" : "Vi ĝisdatigis la limdaton de karto {card} al {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} ĝisdatigis la limdaton de karto {card} al {after}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Vi atribuis al {assigneduser} la karton {card} de tabulo {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} atribuis al {assigneduser} la karton {card} de tabulo {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Vi malatribuis al {assigneduser} la karton {card} de tabulo {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} malatribuis al {assigneduser} la karton {card} de tabulo {board}",
    "You have added the attachment {attachment} to card {card}" : "Vi aldonis dosieron {attachment} al karto {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} aldonis dosieron {attachment} al karto {card}",
    "You have updated the attachment {attachment} on card {card}" : "Vi ĝisdatigis la dosieron {attachment} de karto {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} ĝisdatigis la dosieron {attachment} de karto {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Vi forigis la dosieron {attachment} el karto {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} forigis la dosieron {attachment} el karto {card}",
    "You have restored the attachment {attachment} to card {card}" : "Vi restaŭris la dosieron {attachment} al karto {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} restaŭris la dosieron {attachment} al karto {card}",
    "You have commented on card {card}" : "Vi komentis la karton {card}",
    "{user} has commented on card {card}" : "{user} komentis la karton {card}",
    "Deck" : "Kartaro",
    "Changes in the <strong>Deck app</strong>" : "Ŝanĝoj en la <strong>aplikaĵo Kartaro</strong>",
    "A <strong>comment</strong> was created on a card" : "<strong>Komento</strong> kreiĝis sur karton",
    "The file was uploaded" : "La dosiero alŝutiĝis",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "La dosiero alŝutita superas la regulon „upload_max_filesize“ el „php.ini“",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "La dosiero alŝutita superas la regulon „MAX_FILE_SIZE“ specifita en la HTML-formularo",
    "The file was only partially uploaded" : "La dosiero alŝutiĝis nur parte",
    "No file was uploaded" : "Neniu dosiero alŝutiĝis",
    "Missing a temporary folder" : "Mankas provizora dosierujo",
    "Could not write file to disk" : "Ne eblis skribi dosieron sur diskon",
    "A PHP extension stopped the file upload" : "PHP-modulo haltigis la dosieralŝuton",
    "No file uploaded or file size exceeds maximum of %s" : "Neniu dosiero alŝutita, aŭ dosiergrando transpasas la maksimumon %s",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "La karto „%s“ sur „%s“ estis atribuita al vi de %s.",
    "%s has mentioned you in a comment on \"%s\"." : "%s menciis vin en komento ĉe „%s“.",
    "The board \"%s\" has been shared with you by %s." : "La tabulo „%s“ estis kunhavigita kun vi de %s.",
    "%s on %s" : "%s el %s",
    "No data was provided to create an attachment." : "Neniu datumo troviĝis por krei aldonaĵon.",
    "Finished" : "Finita",
    "To review" : "Reviziota",
    "Action needed" : "Ago bezonata",
    "Later" : "Poste",
    "copy" : "kopio",
    "Done" : "Farita",
    "Attachments" : "Dosieraj aldonaĵoj",
    "File" : "Dosiero",
    "Invalid date, date format must be YYYY-MM-DD" : "Nevalida dato; datoformo estu JJJJ-MM-TT",
    "Select the board to link to a project" : "Elekti la tabulon ligotan al projekto",
    "Select board" : "Elekti tabulon",
    "Cancel" : "Nuligi",
    "File already exists" : "La dosiero jam ekzistas",
    "Add card" : "Aldoni karton",
    "Archived cards" : "Arĥivigitaj kartoj",
    "Open" : "Malfermi",
    "Completed" : "Plenumita",
    "Hide archived cards" : "Kaŝi arĥivigitajn kartojn",
    "Show archived cards" : "Montri arĥivigitajn kartojn",
    "Toggle compact mode" : "Baskuligi densigan vidon",
    "Open details" : "Montri detalojn",
    "Details" : "Detaloj",
    "General" : "Ĝenerala",
    "Search" : "Serĉi",
    "Sharing" : "Kunhavigo",
    "Tags" : "Etikedoj",
    "Deleted items" : "Forigitaj elementoj",
    "Activity" : "Aktivaĵoj",
    "Undo" : "Malfari",
    "Deleted cards" : "Forigitaj kartoj",
    "Hours" : "Horoj",
    "(Group)" : "(grupo)",
    "Can edit" : "Povas redakti",
    "Can share" : "Can share",
    "Owner" : "Posedanto",
    "Delete" : "Forigi",
    "Edit" : "Redakti",
    "Members" : "Membroj",
    "Download" : "Elŝuti",
    "Comments" : "Komentoj",
    "Submit" : "Sendi",
    "Created:" : "Kreita:",
    "Reply" : "Respondi",
    "Update" : "Ĝisdatigi",
    "Description" : "Priskribo",
    "Formatting help" : "Helpo pri tekstaranĝo",
    "Select Date" : "Elekti daton",
    "Remove due date" : "Viŝi limdaton",
    "Unarchive card" : "Malarĥivigi karton",
    "Archive card" : "Arĥivigi karton",
    "(group)" : "(grupo)",
    "Delete card" : "Forigi karton",
    "Archived boards" : "Arĥivigitaj tabuloj",
    "Shared with you" : "Kunhavata kun vi",
    "An error occurred" : "Eraro okazis",
    "Edit board" : "Modifi tabulon",
    "Unarchive board" : "Elarĥivigi tabulon",
    "Archive board" : "Enarĥivigi tabulon",
    "No notifications" : "Neniu sciigo",
    "Delete board" : "Forigi tabulon",
    "Advanced options" : "Detalaj agordoj",
    "Export" : "Eksporti",
    "Close" : "Malfermi",
    "Link to a board" : "Ligilo al tabulo",
    "Maximum file size of {size} exceeded" : "Maksimuma dosiergrando {size} transpasita",
    "Due date" : "Limdato",
    "Created" : "Kreita",
    "Modified" : "Modifita",
    "Error creating the share" : "Eraro dum kreo de la kunhavigo",
    "Share" : "Kunhavigi",
    "The card \"%s\" on \"%s\" has reached its due date." : "La karto „%s“ sur „%s“ atingis sian limdaton.",
    "Save" : "Konservi",
    "seconds ago" : "antaŭ kelkaj sekundoj",
    "Today" : "Hodiaŭ",
    "Tomorrow" : "Morgaŭ"
},
"nplurals=2; plural=(n != 1);");
