<?php
/**
 * This file was automatically generated by autoload_generator.php
 *
 * Do not edit this file directly
 */

spl_autoload_register(function($className) {
    static $classMap;
    if (!isset($classMap)) {
        $classMap = [
            'libdns\decoder\decoder'                                      => __DIR__ . '/../src/Decoder/Decoder.php',
            'libdns\decoder\decoderfactory'                               => __DIR__ . '/../src/Decoder/DecoderFactory.php',
            'libdns\decoder\decodingcontext'                              => __DIR__ . '/../src/Decoder/DecodingContext.php',
            'libdns\decoder\decodingcontextfactory'                       => __DIR__ . '/../src/Decoder/DecodingContextFactory.php',
            'libdns\encoder\encoder'                                      => __DIR__ . '/../src/Encoder/Encoder.php',
            'libdns\encoder\encoderfactory'                               => __DIR__ . '/../src/Encoder/EncoderFactory.php',
            'libdns\encoder\encodingcontext'                              => __DIR__ . '/../src/Encoder/EncodingContext.php',
            'libdns\encoder\encodingcontextfactory'                       => __DIR__ . '/../src/Encoder/EncodingContextFactory.php',
            'libdns\enumeration'                                          => __DIR__ . '/../src/Enumeration.php',
            'libdns\messages\message'                                     => __DIR__ . '/../src/Messages/Message.php',
            'libdns\messages\messagefactory'                              => __DIR__ . '/../src/Messages/MessageFactory.php',
            'libdns\messages\messageopcodes'                              => __DIR__ . '/../src/Messages/MessageOpCodes.php',
            'libdns\messages\messageresponsecodes'                        => __DIR__ . '/../src/Messages/MessageResponseCodes.php',
            'libdns\messages\messagetypes'                                => __DIR__ . '/../src/Messages/MessageTypes.php',
            'libdns\packets\labelregistry'                                => __DIR__ . '/../src/Packets/LabelRegistry.php',
            'libdns\packets\packet'                                       => __DIR__ . '/../src/Packets/Packet.php',
            'libdns\packets\packetfactory'                                => __DIR__ . '/../src/Packets/PacketFactory.php',
            'libdns\records\question'                                     => __DIR__ . '/../src/Records/Question.php',
            'libdns\records\questionfactory'                              => __DIR__ . '/../src/Records/QuestionFactory.php',
            'libdns\records\rdata'                                        => __DIR__ . '/../src/Records/RData.php',
            'libdns\records\rdatabuilder'                                 => __DIR__ . '/../src/Records/RDataBuilder.php',
            'libdns\records\rdatafactory'                                 => __DIR__ . '/../src/Records/RDataFactory.php',
            'libdns\records\record'                                       => __DIR__ . '/../src/Records/Record.php',
            'libdns\records\recordcollection'                             => __DIR__ . '/../src/Records/RecordCollection.php',
            'libdns\records\recordcollectionfactory'                      => __DIR__ . '/../src/Records/RecordCollectionFactory.php',
            'libdns\records\recordtypes'                                  => __DIR__ . '/../src/Records/RecordTypes.php',
            'libdns\records\resource'                                     => __DIR__ . '/../src/Records/Resource.php',
            'libdns\records\resourcebuilder'                              => __DIR__ . '/../src/Records/ResourceBuilder.php',
            'libdns\records\resourcebuilderfactory'                       => __DIR__ . '/../src/Records/ResourceBuilderFactory.php',
            'libdns\records\resourceclasses'                              => __DIR__ . '/../src/Records/ResourceClasses.php',
            'libdns\records\resourcefactory'                              => __DIR__ . '/../src/Records/ResourceFactory.php',
            'libdns\records\resourceqclasses'                             => __DIR__ . '/../src/Records/ResourceQClasses.php',
            'libdns\records\resourceqtypes'                               => __DIR__ . '/../src/Records/ResourceQTypes.php',
            'libdns\records\resourcetypes'                                => __DIR__ . '/../src/Records/ResourceTypes.php',
            'libdns\records\typedefinitions\fielddefinition'              => __DIR__ . '/../src/Records/TypeDefinitions/FieldDefinition.php',
            'libdns\records\typedefinitions\fielddefinitionfactory'       => __DIR__ . '/../src/Records/TypeDefinitions/FieldDefinitionFactory.php',
            'libdns\records\typedefinitions\typedefinition'               => __DIR__ . '/../src/Records/TypeDefinitions/TypeDefinition.php',
            'libdns\records\typedefinitions\typedefinitionfactory'        => __DIR__ . '/../src/Records/TypeDefinitions/TypeDefinitionFactory.php',
            'libdns\records\typedefinitions\typedefinitionmanager'        => __DIR__ . '/../src/Records/TypeDefinitions/TypeDefinitionManager.php',
            'libdns\records\typedefinitions\typedefinitionmanagerfactory' => __DIR__ . '/../src/Records/TypeDefinitions/TypeDefinitionManagerFactory.php',
            'libdns\records\types\anything'                               => __DIR__ . '/../src/Records/Types/Anything.php',
            'libdns\records\types\bitmap'                                 => __DIR__ . '/../src/Records/Types/BitMap.php',
            'libdns\records\types\char'                                   => __DIR__ . '/../src/Records/Types/Char.php',
            'libdns\records\types\characterstring'                        => __DIR__ . '/../src/Records/Types/CharacterString.php',
            'libdns\records\types\domainname'                             => __DIR__ . '/../src/Records/Types/DomainName.php',
            'libdns\records\types\ipv4address'                            => __DIR__ . '/../src/Records/Types/IPv4Address.php',
            'libdns\records\types\ipv6address'                            => __DIR__ . '/../src/Records/Types/IPv6Address.php',
            'libdns\records\types\long'                                   => __DIR__ . '/../src/Records/Types/Long.php',
            'libdns\records\types\short'                                  => __DIR__ . '/../src/Records/Types/Short.php',
            'libdns\records\types\type'                                   => __DIR__ . '/../src/Records/Types/Type.php',
            'libdns\records\types\typebuilder'                            => __DIR__ . '/../src/Records/Types/TypeBuilder.php',
            'libdns\records\types\typefactory'                            => __DIR__ . '/../src/Records/Types/TypeFactory.php',
            'libdns\records\types\types'                                  => __DIR__ . '/../src/Records/Types/Types.php',
        ];
    }

    $className = strtolower($className);
    if (isset($classMap[$className])) {
        /** @noinspection PhpIncludeInspection */
        require $classMap[$className];
    }
});
