OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Anda telah membuat papan {board} baru",
    "{user} has created a new board {board}" : "{user} telah membuat papan {board} baru",
    "You have deleted the board {board}" : "Anda telah menghapus papan {board}",
    "{user} has deleted the board {board}" : "{user} telah menghapus papan {board}",
    "You have restored the board {board}" : "Anda telah memulihkan papan {board}",
    "{user} has restored the board {board}" : "{user} telah memulihkan papan {board}",
    "You have shared the board {board} with {acl}" : "Anda telah membagikan papan {board} dengan akses {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} telah membagikan papan {board} dengan akses {acl}",
    "You have removed {acl} from the board {board}" : "Anda telah mencabut akses {acl} pada papan {board}",
    "{user} has removed {acl} from the board {board}" : "{user} telah mencabut akses {acl} pada papan {board}",
    "You have renamed the board {before} to {board}" : "Anda telah mengubah nama papan {before} menjadi {board}",
    "{user} has renamed the board {before} to {board}" : "{user} telah mengubah nama papan {before} menjadi {board}",
    "You have archived the board {board}" : "Anda telah mengarsipkan papan {board}",
    "{user} has archived the board {before}" : "{user} telah mengarsipkan papan {board}",
    "You have unarchived the board {board}" : "Anda telah memulihkan papan {board}",
    "{user} has unarchived the board {before}" : "{user} telah memulihkan papan {board}",
    "You have created a new list {stack} on board {board}" : "Anda telah membuat daftar baru {stack} pada papan {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} telah membuat daftar baru {stack} pada papan {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Anda telah mengubah nama daftar {before} menjadi {stack} pada papan {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} telah mengubah nama daftar {before} menjadi {stack} pada papan {board}",
    "You have deleted list {stack} on board {board}" : "Anda telah menghapus daftar {stack} pada papan {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} telah menghapus daftar {stack} pada papan {board}",
    "You have created card {card} in list {stack} on board {board}" : "Anda telah membuat kartu {card} didalam daftar {stack} pada papan {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} telah membuat kartu {card} didalam daftar {stack} pada papan {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Anda telah menghapus kartu {card} didalam daftar {stack} pada papan {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} telah menghapus kartu {card} didalam daftar {stack} pada papan {board}",
    "You have renamed the card {before} to {card}" : "Anda telah mengubah nama kartu {before} menjadi {card}",
    "{user} has renamed the card {before} to {card}" : "{user} telah mengubah nama kartu {before} menjadi {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Anda telah menambahkan deskripsi kartu {card} didalam daftar {stack} pada papan {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} telah menambahkan deskripsi kartu {card} didalam daftar {stack} pada papan {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Anda telah memperbarui deskripsi kartu {card} didalam daftar {stack} pada papan {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} telah memperbarui deskripsi kartu {card} didalam daftar {stack} pada papan {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Anda telah mengarsipkan kartu {card} didalam daftar {stack} pada papan {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} telah mengarsipkan kartu {card} didalam daftar {stack} pada papan {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Anda telah memulihkan arsip kartu {card} didalam daftar {stack} pada papan {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} telah memulihkan kartu {card} didalam daftar {stack} pada papan {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Anda telah menandai kartu {card} sebagai selesai dalam daftar {stack} di papan {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} telah menandai kartu {card} sebagai selesai dalam daftar {stack} di papan {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Anda telah menandai kartu {card} sebagai belum selesai dalam daftar {stack} di papan {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} telah menandai kartu {card} sebagai belum selesai dalam daftar {stack} di papan {board}",
    "You have removed the due date of card {card}" : "Anda telah menghapus tenggat kartu {card}",
    "{user} has removed the due date of card {card}" : "{user} telah menghapus tenggat kartu {card}",
    "You have set the due date of card {card} to {after}" : "Anda telah mengatur tenggat kartu {card} ke {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} telah mengatur tenggat kartu {card} ke {after}",
    "You have updated the due date of card {card} to {after}" : "Anda telah memperbarui tenggat kartu {card} ke {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} telah memperbarui tenggat kartu {card} ke {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Anda telah menambahkan label {label} ke kartu {card} didalam daftar {stack} pada papan {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} telah menambahkan label {label} ke kartu {card} didalam daftar {stack} pada papan {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Anda telah menghapus label {label} dari kartu {card} didalam daftar {stack} pada papan {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} telah menghapus label {label} dari kartu {card} didalam daftar {stack} pada papan {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Anda telah menugaskan {assigneduser} ke kartu {card} pada papan {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} telah menugaskan {assigneduser} ke kartu {card} pada papan {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Anda telah tidak menugaskan {assigneduser} ke kartu {card} pada papan {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} telah tidak menugaskan {assigneduser} ke kartu {card} pada papan {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Anda telah memindahkan kartu {card} dari daftar {stackBefore} ke {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} telah memindahkan kartu {card} dari daftar {stackBefore} ke {stack}",
    "You have added the attachment {attachment} to card {card}" : "Anda telah menyematkan lampiran {attachment} ke kartu {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} telah menyematkan lampiran {attachment} ke kartu {card}",
    "You have updated the attachment {attachment} on card {card}" : "Anda telah memperbarui lampiran {attachment} pada kartu {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} telah memperbarui lampiran {attachment} pada kartu {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Anda telah menghapus lampiran {attachment} dari kartu {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} telah menghapus lampiran {attachment} dari kartu {card}",
    "You have restored the attachment {attachment} to card {card}" : "Anda telah memulihkan lampiran {attachment} ke kartu {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} telah memulihkan lampiran {attachment} ke kartu {card}",
    "You have commented on card {card}" : "Anda telah berkomentar pada kartu {card}",
    "{user} has commented on card {card}" : "{user} telah berkomentar pada kartu {card}",
    "Deck" : "Longgok",
    "Changes in the <strong>Deck app</strong>" : "Perubahan pada <strong>aplikasi Longgok</strong>",
    "A <strong>board, list or card</strong> was changed" : "Sebuah <strong>papan, daftar atau kartu</strong> telah diubah",
    "A <strong>comment</strong> was created on a card" : "<strong>Komentar</strong> telah dibuat pada suatu kartu",
    "A <strong>card description</strong> has been changed" : "Sebuah <strong>deskripsi kartu</strong> telah diubah",
    "The file was uploaded" : "Berkas terunggah",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Berkas terunggah melampau parameter upload_max_filesize pada php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Berkas yang diunggah melampaui direktif MAX_FILE_SIZE yang ditentukan dalam formulir HTML.",
    "The file was only partially uploaded" : "Berkas hanya sebagian terunggah",
    "No file was uploaded" : "Tidak ada berkas yang diunggah",
    "Missing a temporary folder" : "Folder sementara tidak ada",
    "Could not write file to disk" : "Tidak dapat menulis berkas ke diska",
    "A PHP extension stopped the file upload" : "Ekstensi PHP menghentikan proses unggah berkas",
    "No file uploaded or file size exceeds maximum of %s" : "Gagal unggah berkas atau ukuran melampaui batas maksimum %s",
    "Invalid file type. Only JSON files are allowed." : "Jenis berkas tidak valid. Hanya berkas JSON yang diizinkan.",
    "Invalid JSON data" : "Data JSON tidak valid",
    "Failed to import board" : "Gagal mengimpor papan",
    "Cards due today" : "Kartu jatuh tempo hari ini",
    "Cards due tomorrow" : "Kartu jatuh tempo besok",
    "Upcoming cards" : "Kartu berikut",
    "Load more" : "Muat lebih banyak",
    "Welcome to Nextcloud Deck!" : "Selamat datang di Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Kartu \"%s\" pada \"%s\" telah ditugaskan kepada Anda oleh %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} telah menetapkan kartu {deck-card} di {deck-board} kepada Anda.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Kartu \"%1$s\" di \"%2$s\" telah mencapai tanggal jatuh temponya.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Kartu {deck-card} di {deck-board} telah mencapai tanggal jatuh temponya.",
    "%s has mentioned you in a comment on \"%s\"." : "%s telah menyebut Anda didalam suatu komentar pada \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} telah menyebut Anda dalam komentar di {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Papan \"%s\" telah dibagikan kepada Anda oleh %s.",
    "{user} has shared {deck-board} with you." : "{user} telah membagikan {deck-board} dengan Anda.",
    "Deck board" : "Papan Longgok",
    "Owned by %1$s" : "Dimiliki oleh %1$s",
    "Deck boards, cards and comments" : "Papan, kartu, dan komentar Deck",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Dari %1$s, di %2$s/%3$s, dimiliki oleh %4$s",
    "Create a new deck card" : "Buat kartu deck baru",
    "Card comments" : "Komentar kartu",
    "%s on %s" : "%s pada %s",
    "Deck boards and cards" : "Papan dan kartu Deck",
    "No data was provided to create an attachment." : "Tidak ada data untuk membuat lampiran.",
    "Finished" : "Selesai",
    "To review" : "Perlu ditinjau",
    "Action needed" : "Butuh tindakan",
    "Later" : "Nanti",
    "copy" : "salin",
    "Read more inside" : "Baca lebih lanjut di dalam",
    "Custom lists - click to rename!" : "Daftar khusus - klik untuk mengganti nama!",
    "To Do" : "Yang Harus Dilakukan",
    "In Progress" : "Sedang Berlangsung",
    "Done" : "Selesai",
    "1. Open to learn more about boards and cards" : "1. Buka untuk mempelajari lebih lanjut tentang papan dan kartu",
    "2. Drag cards left and right, up and down" : "2. Seret kartu ke kiri dan kanan, atas dan bawah",
    "3. Apply rich formatting and link content" : "3. Terapkan pemformatan kaya dan tautkan konten",
    "4. Share, comment and collaborate!" : "4. Bagikan, komentari, dan berkolaborasi!",
    "Create your first card!" : "Buat kartu pertama Anda!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Komentar ini memiliki lebih dari %s karakter.\nDitambahkan sebagai lampiran ke kartu dengan nama %s.\nDapat diakses di URL: %s.",
    "Attachments" : "Lampiran",
    "File" : "File",
    "date" : "tanggal",
    "Card not found" : "Kartu tidak ditemukan",
    "Path is already shared with this card" : "Jalur sudah dibagikan dengan kartu ini",
    "Invalid date, date format must be YYYY-MM-DD" : "Tanggal salah, format tanggal harus TTTT-BB-HH",
    "Personal planning and team project organization" : "Perencanaan pribadi dan pengelolaan proyek tim",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck adalah alat organisasi gaya kanban yang ditujukan untuk perencanaan pribadi dan organisasi proyek untuk tim yang terintegrasi dengan Nextcloud.\n\n\n- 📥 Tambahkan tugas Anda ke kartu dan susun urutannya\n- 📄 Tulis catatan tambahan dalam Markdown\n- 🔖 Tetapkan label untuk organisasi yang lebih baik\n- 👥 Bagikan dengan tim, teman, atau keluarga Anda\n- 📎 Lampirkan berkas dan sematkan dalam deskripsi Markdown Anda\n- 💬 Diskusikan dengan tim Anda menggunakan komentar\n- ⚡ Lacak perubahan di aliran aktivitas\n- 🚀 Atur proyek Anda",
    "Add board" : "Tambah papan",
    "Card details" : "Detail kartu",
    "Select the board to link to a project" : "Pilih papan untuk ditautkan ke proyek",
    "Search by board title" : "Cari berdasarkan judul papan",
    "Select board" : "Pilih papan",
    "Move/copy card" : "Pindahkan/salin kartu",
    "Select a board" : "Pilih papan",
    "No lists available" : "Tidak ada daftar tersedia",
    "Select a list" : "Pilih daftar",
    "Move card" : "Pindahkan kartu",
    "Copy card" : "Salin kartu",
    "Select the card to link to a project" : "Pilih kartu untuk ditautkan ke proyek",
    "Link to card" : "Tautan ke kartu",
    "Select a card" : "Pilih kartu",
    "Cancel" : "Membatalkan",
    "This board is read only" : "Papan hanya dapat dilihat",
    "Drop your files to upload" : "Lepas berkas Anda untuk mengunggah",
    "File already exists" : "Berkas sudah ada",
    "A file with the name {filename} already exists." : "Berkas dengan nama {filename} sudah ada.",
    "Do you want to overwrite it?" : "Apakah Anda ingin menimpanya?",
    "Overwrite file" : "Menimpa berkas",
    "Keep existing file" : "Pertahankan berkas yang ada",
    "Add card" : "Tambah kartu",
    "Archived cards" : "Arsip kartu",
    "Add list" : "Tambahkan daftar",
    "List name" : "Nama daftar",
    "Clear search" : "Hapus pencarian",
    "Active filters" : "Filter aktif",
    "Apply filter" : "Terapkan filter",
    "Filter by tag" : "Filter dengan tag",
    "Filter by assigned user" : "Filter dengan pengguna",
    "Unassigned" : "Belum ditugaskan",
    "Filter by status" : "Filter berdasarkan status",
    "Open and completed" : "Terbuka dan selesai",
    "Open" : "Buka",
    "Completed" : "Selesai",
    "Filter by due date" : "Filter dengan tenggat",
    "Overdue" : "Lewat tenggat",
    "Next 24 hours" : "Jangka 24 jam",
    "Next 7 days" : "Jangka 7 hari",
    "Next 30 days" : "Jangka 30 hari",
    "No due date" : "Tanpa tenggat",
    "Clear filter" : "Bersihkan filter",
    "View Modes" : "Mode Tampilan",
    "Toggle View Modes" : "Alihkan Mode Tampilan",
    "Hide archived cards" : "Sembunyikan kartu yang diarsipkan",
    "Show archived cards" : "Tampilan arsip kartu",
    "Toggle compact mode" : "Beralih mode ringkas",
    "Hide card cover images" : "Sembunyikan gambar sampul kartu",
    "Show card cover images" : "Tampilkan gambar sampul kartu",
    "Open details" : "Buka detail",
    "Details" : "Detail",
    "Deck settings" : "Pengaturan Deck",
    "General" : "Umum",
    "Use bigger card view" : "Gunakan tampilan kartu yang lebih besar",
    "Appearance" : "Tampilan",
    "Show card ID badge" : "Tampilkan lencana ID kartu",
    "Show boards in calendar/tasks" : "Tampilkan papan di kalender/tugas",
    "Admin settings" : "Pengaturan admin",
    "Limit board creation to some groups" : "Batasi pembuatan papan untuk beberapa grup",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Pengguna di luar grup tersebut tidak akan dapat membuat papan mereka sendiri, tetapi masih dapat bekerja di papan yang telah dibagikan dengan mereka.",
    "Board actions" : "Aksi papan",
    "Scroll sideways" : "Gulir ke samping",
    "Navigate between cards" : "Navigasi antar kartu",
    "Close card details" : "Tutup detail kartu",
    "Search" : "Cari",
    "Show card filters" : "Tampilkan filter kartu",
    "Clear card filters" : "Hapus filter kartu",
    "Show those shortcuts" : "Tampilkan pintasan tersebut",
    "Card actions" : "Aksi kartu",
    "Open card details" : "Buka detail kartu",
    "Edit the card title" : "Edit judul kartu",
    "Assign yourself to the current card" : "Tetapkan diri Anda ke kartu saat ini",
    "Archive/unarchive the current card" : "Arsipkan/batalkan pengarsipan kartu saat ini",
    "Mark card as completed/not completed" : "Tandai kartu sebagai selesai/belum selesai",
    "Open card menu" : "Buka menu kartu",
    "Currently present people" : "Orang yang saat ini hadir",
    "Loading board" : "Memuat papan",
    "Board not found" : "Papan tidak ditemukan",
    "Create a new list to add cards to this board" : "Buat daftar baru untuk menambahkan kartu ke papan ini",
    "Sharing" : "Berbagi",
    "Tags" : "Tag",
    "Deleted items" : "Entri terhapus",
    "Activity" : "Aktivitas",
    "Deleted lists" : "Daftar terhapus",
    "Undo" : "Undo",
    "Deleted cards" : "Kartu terhapus",
    "Hours" : "Jam",
    "Days" : "Hari",
    "Weeks" : "Pekan",
    "Months" : "Bulan",
    "No cards yet" : "Belum ada kartu",
    "Failed to create share with {displayName}" : "Gagal membuat berbagi dengan {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Apakah Anda yakin ingin mentransfer papan {title} ke {user}?",
    "Transfer the board." : "Transfer papan.",
    "Transfer" : "Transfer",
    "The board has been transferred to {user}" : "Papan telah ditransfer ke {user}",
    "Failed to transfer the board to {user}" : "Gagal mentransfer papan ke {user}",
    "Board owner" : "Pemilik papan",
    "(Group)" : "(Grup)",
    "(Team)" : "(Tim)",
    "(remote)" : "remote",
    "Can edit" : "Can edit",
    "Can share" : "Can share",
    "Can manage" : "Dapat mengelola",
    "Owner" : "Owner",
    "Delete" : "Hapus",
    "List deleted" : "Daftar dihapus",
    "Edit list title" : "Edit judul daftar",
    "Archive all cards" : "Arsipkan semua kartu",
    "Unarchive all cards" : "Batalkan pengarsipan semua kartu",
    "Delete list" : "Hapus daftar",
    "Archive all cards in this list" : "Arsipkan semua kartu dalam daftar ini",
    "Unarchive all cards in this list" : "Batalkan pengarsipan semua kartu dalam daftar ini",
    "Add a new card" : "Tambah kartu baru",
    "Card name" : "Nama kartu",
    "title and color value must be provided" : "judul dan nilai warna harus ditentukan",
    "Edit" : "Sunting",
    "Add a new tag" : "Tambah tag baru",
    "Board name" : "Nama papan",
    "Members" : "Anggota",
    "Assign to users/groups/team" : "Tetapkan ke pengguna/grup/tim",
    "Assign a user to this card…" : "Menugaskan pengguna kepada kartu ini..",
    "Select a user to assign to this card…" : "Pilih pengguna untuk ditetapkan ke kartu ini…",
    "File to share" : "Berkas untuk dibagikan",
    "Invalid path selected" : "Jalur terpilih invalid",
    "Upload new files" : "Unggah berkas baru",
    "Share from Files" : "Bagikan dari Berkas",
    "Pending share" : "Berbagi tertunda",
    "Add this attachment" : "Tambah lampiran ini",
    "Show in Files" : "Tampilkan di Berkas",
    "Download" : "Unduh",
    "Remove attachment" : "Hapus lampiran",
    "Delete Attachment" : "Hapus Lampiran",
    "Restore Attachment" : "Pulihkan Lampiran",
    "The title cannot be empty." : "Judul tidak boleh kosong.",
    "Cannot close unsaved card!" : "Tidak dapat menutup kartu yang belum disimpan!",
    "Open in sidebar view" : "Buka dalam tampilan bilah samping",
    "Open in bigger view" : "Buka dalam tampilan lebih besar",
    "Comments" : "Komentar",
    "Failed to load comments" : "Gagal memuat komentar",
    "No comments yet. Begin the discussion!" : "Belum ada komentar. Mulai berdiskusi!",
    "The comment cannot be empty." : "Komentar tidak dapat kosong.",
    "The comment cannot be longer than 1000 characters." : "Komentar tidak dapat melebihi 1000 karakter.",
    "Submit" : "Kirim",
    "Created:" : "Dibuat:",
    "In reply to" : "Membalas kepada",
    "Cancel reply" : "Batalkan balasan",
    "Reply" : "Balas",
    "Update" : "Perbarui",
    "Mark as done" : "Tandai sebagai selesai",
    "Mark as not done" : "Tandai sebagai belum selesai",
    "Write a description …" : "Tulis deskripsi …",
    "Could not save description" : "Tidak dapat menyimpan deskripsi",
    "Description" : "Deskrisi",
    "(Unsaved)" : "(Tidak tersimpan)",
    "(Saving…)" : "(Menyimpan…)",
    "Formatting help" : "Bantuan pemformatan",
    "Edit description" : "Edit deskripsi.",
    "View description" : "Tampilkan deskripsi",
    "Add Attachment" : "Tambah Lampiran",
    "Choose attachment" : "Pilih lampiran",
    "Select Date" : "Pilih Tanggal",
    "Later today – {timeLocale}" : "Nanti hari ini – {timeLocale}",
    "Set due date for later today" : "Tetapkan tanggal jatuh tempo untuk nanti hari ini",
    "Tomorrow – {timeLocale}" : "Besok – {timeLocale}",
    "Set due date for tomorrow" : "Tetapkan tanggal jatuh tempo untuk besok",
    "This weekend – {timeLocale}" : "Akhir pekan ini – {timeLocale}",
    "Set due date for this weekend" : "Tetapkan tanggal jatuh tempo untuk akhir pekan ini",
    "Next week – {timeLocale}" : "Minggu depan – {timeLocale}",
    "Set due date for next week" : "Tetapkan tanggal jatuh tempo untuk minggu depan",
    "Assign a due date to this card…" : "Tetapkan tanggal jatuh tempo ke kartu ini…",
    "Set a due date" : "Tentukan tenggat",
    "Add due date" : "Tambahkan tanggal jatuh tempo",
    "Choose a date" : "Pilih tanggal",
    "Remove due date" : "Hapus tenggat",
    "Due at:" : "Jatuh tempo pada:",
    "Not done" : "Belum selesai",
    "Unarchive card" : "Memulihkan kartu",
    "Archive card" : "Mengarsipkan kartu",
    "Assign a tag to this card…" : "Berikan tag pada kartu ini...",
    "Select or create a tag…" : "Pilih atau buat tag…",
    "Create a new tag:" : "Buat tag baru:",
    "(group)" : "(grup)",
    "{count} comments, {unread} unread" : "{count} komentar, {unread} belum dibaca",
    "Todo items" : "Item yang harus dilakukan",
    "Edit card title" : "Edit judul kartu",
    "Open link" : "Buka tautan",
    "Card deleted" : "Kartu dihapus",
    "Edit title" : "Edit judul",
    "Assign to me" : "Tugaskan saya",
    "Unassign myself" : "Batalkan penetapan diri saya",
    "Delete card" : "Hapus kartu",
    "All boards" : "Semua papan",
    "Archived boards" : "Papan terarsip",
    "Shared with you" : "Shared with you",
    "Cancel edit" : "Batal sunting",
    "Save board" : "Simpan papan",
    "Board {0} deleted" : "{0} papan terhapus",
    "All cards" : "Semua kartu",
    "Only assigned cards" : "Hanya kartu yang ditetapkan",
    "No reminder" : "Tanpa pengingat",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Apakah Anda yakin ingin menghapus papan {title}? Ini akan menghapus semua data papan ini termasuk kartu yang diarsipkan.",
    "Delete the board?" : "Hapus papan?",
    "Are you sure you want to leave the board {title}?" : "Apakah Anda yakin ingin meninggalkan papan {title}?",
    "Leave the board?" : "Tinggalkan papan?",
    "Leave" : "Tinggalkan",
    "Failed to leave the board" : "Gagal meninggalkan papan",
    "Exporting board..." : "Mengekspor papan...",
    "An error occurred" : "Terjadi kesalahan",
    "Board details" : "Detail papan",
    "Edit board" : "Edit papan",
    "Clone board" : "Pengklonaan papan",
    "Unarchive board" : "Memulihkan papan",
    "Archive board" : "Mengarsipkan papan",
    "Export board" : "Ekspor papan",
    "Turn on due date reminders" : "Aktifkan pengingat tanggal jatuh tempo",
    "Turn off due date reminders" : "Nonaktifkan pengingat tanggal jatuh tempo",
    "Due date reminders" : "Pengingat tanggal jatuh tempo",
    "Assigned cards" : "Kartu yang ditetapkan",
    "No notifications" : "Tidak ada notifikasi",
    "Delete board" : "Hapus papan",
    "Leave board" : "Tinggalkan papan",
    "Importing board..." : "Mengimpor papan...",
    "Board imported successfully" : "Papan berhasil diimpor",
    "Import board" : "Impor papan",
    "Clone {boardTitle}" : "Gandakan {boardTitle}",
    "Clone cards" : "Klona kartu",
    "Clone assignments" : "Gandakan penetapan",
    "Clone labels" : "Gandakan label",
    "Clone due dates" : "Gandakan tanggal jatuh tempo",
    "Advanced options" : "Opsi lanjutan",
    "Move all cards to the first list" : "Pindahkan semua kartu ke daftar pertama",
    "Restore archived cards" : "Pulihkan kartu yang diarsipkan",
    "Clone" : "Klona",
    "Export {boardTitle}" : "Ekspor {boardTitle}",
    "Export as JSON" : "Ekspor sebagai JSON",
    "Export as CSV" : "Ekspor sebagai CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Catatan: Hanya format JSON yang didukung untuk mengimpor kembali ke aplikasi Deck.",
    "Export" : "Ekspor",
    "Loading filtered view" : "Memuat tampilan terfilter",
    "Search for {searchQuery} in other boards" : "Cari {searchQuery} di papan lain",
    "Search for {searchQuery} in all boards" : "Cari {searchQuery} di semua papan",
    "No results found" : "Tidak ada hasil ditemukan",
    "Deck board {name}\n* Last modified on {lastMod}" : "Papan Deck {name}\n* Terakhir diubah pada {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Dibuat pada {created}\n* Terakhir diubah pada {lastMod}\n* {nbAttachments} lampiran\n* {nbComments} komentar",
    "{nbCards} cards" : "{nbCards} kartu",
    "Due on {date}" : "Jatuh tempo pada {date}",
    "{stack} in {board}" : "{stack} di {board}",
    "Click to expand description" : "Klik untuk membuka deskripsi",
    "Click to expand comment" : "Klik untuk membuka komentar",
    "Create card" : "Buat kartu",
    "Create a new card" : "Buat kartu baru",
    "Card title" : "Judul kartu",
    "Creating the new card …" : "Membuat kartu baru …",
    "Card \"{card}\" was added to \"{board}\"" : "Kartu \"{card}\" ditambahkan ke \"{board}\"",
    "Open card" : "Buka kartu",
    "Close" : "Tutup",
    "No upcoming cards" : "Tidak ada kartu berikut",
    "upcoming cards today" : "kartu mendatang hari ini",
    "upcoming cards tomorrow" : "kartu mendatang besok",
    "upcoming cards" : "kartu berikut",
    "New card" : "Kartu baru",
    "Link to a board" : "Tautan ke papan",
    "Link to a card" : "Tautan ke kartu",
    "Create a card" : "Buat kartu",
    "Message from {author} in {conversationName}" : "Pesan dari {author} di {conversationName}",
    "Something went wrong" : "Ada yang salah",
    "Failed to upload {name}" : "Gagal mengunggah {name}",
    "Maximum file size of {size} exceeded" : "Melampaui batas ukuran maksimal {size}",
    "Assigned users" : "Pengguna yang ditetapkan",
    "Due date" : "Tenggat",
    "Created" : "Dibuat",
    "Modified" : "Dimodifikasi",
    "Error creating the share" : "Kesalahan saat membuat berbagi",
    "Share with a Deck card" : "Bagikan dengan kartu Deck",
    "Share {file} with a Deck card" : "Bagikan {file} dengan kartu Deck",
    "Share" : "Bagikan",
    "The card \"%s\" on \"%s\" has reached its due date." : "Kartu \"%s\" pada \"%s\" telah melampaui tenggat.",
    "Share board with a user, group or team …" : "Bagikan papan dengan pengguna, grup, atau tim …",
    "Searching for users, groups and teams …" : "Mencari pengguna, grup, dan tim …",
    "No participants found" : "Tidak ada peserta ditemukan",
    "Save" : "Simpan",
    "seconds ago" : "beberapa detik yang lalu",
    "Today" : "Hari ini",
    "Tomorrow" : "Besok",
    "No due" : "Tanpa jatuh tempo"
},
"nplurals=1; plural=0;");
