OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Du har skapat en ny tavla {board}",
    "{user} has created a new board {board}" : "{user} har skapat en ny tavla {board}",
    "You have deleted the board {board}" : "Du har tagit bort tavlan {board}",
    "{user} has deleted the board {board}" : "{user} har tagit bort tavlan {board}",
    "You have restored the board {board}" : "Du har återställt tavlan {board}",
    "{user} has restored the board {board}" : "{user} har återställt tavlan {board}",
    "You have shared the board {board} with {acl}" : "Du har delat tavlan {board} med {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} har delat tavlan {board} med {acl}",
    "You have removed {acl} from the board {board}" : "Du har tagit bort {acl} från tavlan {board}",
    "{user} has removed {acl} from the board {board}" : "{user} har tagit bort {acl} från tavlan {board}",
    "You have renamed the board {before} to {board}" : "Du har bytt namn på tavlan {before} till {board}",
    "{user} has renamed the board {before} to {board}" : "{user} har bytt namn på tavlan {before} till {board}",
    "You have archived the board {board}" : "Du har arkiverat tavlan {board}",
    "{user} has archived the board {before}" : "{user} har arkiverat tavlan {before}",
    "You have unarchived the board {board}" : "Du har avarkiverat tavlan {board}",
    "{user} has unarchived the board {before}" : "{user} har avarkiverat tavlan {before}",
    "You have created a new list {stack} on board {board}" : "Du har skapat en ny lista {stack} på tavlan {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} har skapat en ny lista {stack} på tavlan {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Du har bytt namn på listan {before} till {stack} på tavlan {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} har bytt namn på listan {before} till {stack} på tavlan {board}",
    "You have deleted list {stack} on board {board}" : "Du har tagit bort listan {stack} på tavlan {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} har tagit bort listan {stack} på tavlan {board}",
    "You have created card {card} in list {stack} on board {board}" : "Du har skapat kortet {card} i listan {stack} på tavlan {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} har skapat kortet {card} i listan {stack} på tavlan {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Du har tagit bort kortet {card} i listan {stack} på tavlan {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} har tagit bort kortet {card} i listan {stack} på tavlan {board}",
    "You have renamed the card {before} to {card}" : "Du har bytt namn på kortet {before} till {card}",
    "{user} has renamed the card {before} to {card}" : "{user} har bytt namn på kortet {before} till {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Du har lagt till en beskrivning av kortet {card} i listan {stack} på tavlan {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} har lagt till en beskrivning av kortet {card} i listan {stack} på tavlan {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Du har uppdaterat beskrivningen för kortet {card} i listan {stack} på tavlan {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} har uppdaterat beskrivningen för kortet {card} i listan {stack} på tavlan {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Du har arkiverat kortet {card} i listan {stack} på tavlan {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} har arkiverat kortet {card} i listan {stack} på tavlan {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Du har avarkiverat kortet {card} i listan {stack} på tavlan {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} har avarkiverat kortet {card} i listan {stack} på tavlan {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Du har markerat kortet {card} som klart i listan {stack} på tavlan {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} har markerat kortet {card} som klart i listan {stack} på tavlan {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Du har markerat kortet {card} som inte slutfört i listan {stack} på tavlan {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} har markerat kortet {card} som inte slutfört i listan {stack} på tavlan {board}",
    "You have removed the due date of card {card}" : "Du har tagit bort förfallodatum för kortet {card}",
    "{user} has removed the due date of card {card}" : "{user} har tagit bort förfallodatumet för kortet {card}",
    "You have set the due date of card {card} to {after}" : "Du har satt förfallodatum för kortet {card} till {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} har satt förfallodatum för kortet {card} till {after}",
    "You have updated the due date of card {card} to {after}" : "Du har uppdaterat förfallodatum för kortet {card} till {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} har uppdaterat förfallodatum för kortet {card} till {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Du har lagt till taggen {label} på kortet {card} i listan {stack} på tavlan {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} har lagt till taggen {label} på kortet {card} i listan {stack} på tavlan {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Du har tagit bort taggen {label} från kortet {card} i listan {stack} på tavlan {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} har tagit bort taggen {label} från kortet {card} i listan {stack} på tavlan {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Du har tilldelat kortet {card} på tavlan {board} till {assigneduser}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} har tilldelat kortet {card} på tavlan {board} till {assigneduser}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Du har tagit bort tilldelningen för {assigneduser} från kortet {card} på tavlan {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} har tagit bort tilldelningen för {assigneduser} från kortet {card} på tavlan {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Du har flyttat kortet {card} från listan {stackBefore} till {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} har flyttat kortet {card} från listan {stackBefore} till {stack}",
    "You have added the attachment {attachment} to card {card}" : "Du har lagt till bilagan {attachment} på kortet {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} har lagt till bilagan {attachment} på kortet {card}",
    "You have updated the attachment {attachment} on card {card}" : "Du har uppdaterat bilagan {attachment} på kortet {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} har uppdaterat bilagan {attachment} på kortet {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Du har tagit bort bilagan {attachment} från kortet {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} har tagit bort bilagan {attachment} från kortet {card}",
    "You have restored the attachment {attachment} to card {card}" : "Du har återställt bilagan {attachment} till kortet {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} har återställt bilagan {attachment} till kortet {card}",
    "You have commented on card {card}" : "Du har kommenterat kortet {card}",
    "{user} has commented on card {card}" : "{user} har kommenterat kortet {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Ändringar i <strong>Deck-appen</strong>",
    "A <strong>board, list or card</strong> was changed" : "En <strong>tavla, lista eller kort</strong> ändrades",
    "A <strong>comment</strong> was created on a card" : "En <strong>kommentar</strong> skapades på ett kort",
    "A <strong>card description</strong> has been changed" : "En <strong>kortbeskrivning</strong> har ändrats",
    "The file was uploaded" : "Filen laddades upp",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Den uppladdade filen överstiger direktivet upload_max_filesize i php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Den uppladdade filen överstiger MAX_FILE_SIZE-direktivet som specificerades i HTML-formuläret",
    "The file was only partially uploaded" : "Filen var endast delvis uppladdad",
    "No file was uploaded" : "Ingen fil laddades upp",
    "Missing a temporary folder" : "Saknas en tillfällig mapp",
    "Could not write file to disk" : "Det gick inte att skriva filen till disken",
    "A PHP extension stopped the file upload" : "Ett PHP-tillägg stoppade filuppladdningen",
    "No file uploaded or file size exceeds maximum of %s" : "Ingen fil uppladdad eller filstorleken överstiger max %s",
    "Invalid file type. Only JSON files are allowed." : "Ogiltig filtyp. Endast JSON-filer är tillåtna.",
    "Invalid JSON data" : "Ogiltig JSON-data",
    "Failed to import board" : "Det gick inte att importera tavlan",
    "Cards due today" : "Kort som förfaller idag",
    "Cards due tomorrow" : "Kort som förfaller i morgon",
    "Upcoming cards" : "Kommande kort",
    "Load more" : "Ladda mer",
    "Welcome to Nextcloud Deck!" : "Välkommen till Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Kortet \"%s\" på \"%s\" har tilldelats dig av %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} har tilldelat kortet {deck-card} från {deck-board} till dig.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Kortet \"%1$s\" på \"%2$s\" har nått sitt förfallodatum.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Kortet {deck-card} på {deck-board} har nått sitt förfallodatum.",
    "%s has mentioned you in a comment on \"%s\"." : "%s har nämnt dig i en kommentar på \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} har nämnt dig i en kommentar på {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Tavlan \"%s\" har delats med dig av %s.",
    "{user} has shared {deck-board} with you." : "{user} har delat {deck-board} med dig.",
    "The remote board %1$s has been shared with you by %2$s" : "Den externa tavlan %1$s har delats med dig av %2$s",
    "Deck board" : "Deck-tavla",
    "Owned by %1$s" : "Ägd av %1$s",
    "Deck boards, cards and comments" : "Deck-tavlor, kort och kommentarer",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Från %1$s, i %2$s/%3$s, ägd av %4$s",
    "Create a new deck card" : "Skapa ett nytt Deck-kort",
    "Card comments" : "Kortkommentarer",
    "%s on %s" : "%s på %s",
    "Deck boards and cards" : "Deck-tavlor och kort",
    "No data was provided to create an attachment." : "Inga data lämnades för att skapa en bilaga.",
    "Finished" : "Slutförd",
    "To review" : "Att granska",
    "Action needed" : "Åtgärd behövs",
    "Later" : "Senare",
    "copy" : "kopia",
    "Read more inside" : "Läs mer inuti",
    "Custom lists - click to rename!" : "Anpassade listor – klicka för att byta namn!",
    "To Do" : "Att göra",
    "In Progress" : "Pågående",
    "Done" : "Klart",
    "1. Open to learn more about boards and cards" : "1. Öppna för att lära dig mer om tavlor och kort",
    "2. Drag cards left and right, up and down" : "2. Dra kort åt vänster och höger, upp och ner",
    "3. Apply rich formatting and link content" : "3. Använd rik formatering och länka innehåll",
    "4. Share, comment and collaborate!" : "4. Dela, kommentera och samarbeta!",
    "Create your first card!" : "Skapa ditt första kort!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Kommentaren innehåller fler än %s tecken.\nDen lades till som en bilaga på kortet med namnet %s.\nDen är tillgänglig på webbadressen %s.",
    "Attachments" : "Bilagor",
    "File" : "Fil",
    "date" : "datum",
    "Card not found" : "Kortet hittades inte",
    "Path is already shared with this card" : "Sökvägen är redan delad med detta kort",
    "Invalid date, date format must be YYYY-MM-DD" : "Ogiltigt datum, måste anges: ÅÅÅÅ-MM-DD",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Alla tavlor som ägs av dig inklusive listor, kort, etiketter, tilldelningar och kommentarer",
    "Personal planning and team project organization" : "Personlig planering och projektorganisation för team",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck är ett organisationsverktyg i kanbanstil för personlig planering och projektarbete i team, integrerat med Nextcloud.\n\n\n- 📥 Lägg till uppgifter på kort och ordna dem\n- 📄 Skriv ytterligare anteckningar i Markdown\n- 🔖 Tilldela etiketter för bättre organisation\n- 👥 Dela med teamet, vänner eller familj\n- 📎 Bifoga filer och bädda in dem i Markdown-beskrivningen\n- 💬 Diskutera med teamet genom kommentarer\n- ⚡ Följ ändringar i aktivitetsflödet\n- 🚀 Organisera projektet",
    "Add board" : "Lägg till tavla",
    "Card details" : "Kortdetaljer",
    "Select the board to link to a project" : "Välj tavla att länka till ett projekt",
    "Search by board title" : "Sök efter namn på tavla",
    "Select board" : "Välj tavla",
    "Move/copy card" : "Flytta/kopiera kort",
    "Select a board" : "Välj en tavla",
    "No lists available" : "Inga listor tillgängliga",
    "Select a list" : "Välj en lista",
    "Move card" : "Flytta kort",
    "Copy card" : "Kopiera kort",
    "Select the card to link to a project" : "Välj kort att länka till ett projekt",
    "Link to card" : "Länka till kort",
    "Select a card" : "Välj ett kort",
    "Cancel" : "Avbryt",
    "This board is read only" : "Denna tavla är skrivskyddad",
    "Drop your files to upload" : "Släpp dina filer för att ladda upp",
    "File already exists" : "Filen finns redan",
    "A file with the name {filename} already exists." : "En fil med namnet {filename} finns redan.",
    "Do you want to overwrite it?" : "Vill du skriva över den?",
    "Overwrite file" : "Skriv över filen",
    "Keep existing file" : "Behåll nuvarande fil",
    "Add card" : "Lägg till kort",
    "Archived cards" : "Arkiverade kort",
    "Add list" : "Lägg till lista",
    "List name" : "Namn på lista",
    "Clear search" : "Rensa sökning",
    "Active filters" : "Aktiva filter",
    "Apply filter" : "Tillämpa filter",
    "Filter by tag" : "Filtrera efter tagg",
    "Filter by assigned user" : "Filtrera efter tilldelad användare",
    "Unassigned" : "Ej tilldelad",
    "Filter by status" : "Filtrera efter status",
    "Open and completed" : "Öppen och klar",
    "Open" : "Öppen",
    "Completed" : "Slutförd",
    "Filter by due date" : "Filtrera efter förfallodatum",
    "Overdue" : "Förfallen",
    "Next 24 hours" : "Nästa 24 timmar",
    "Next 7 days" : "Nästa 7 dagar",
    "Next 30 days" : "Nästa 30 dagar",
    "No due date" : "Inget förfallodatum",
    "Clear filter" : "Rensa filter",
    "View Modes" : "Visningslägen",
    "Toggle View Modes" : "Växla visningsläge",
    "Kanban view" : "Kanban-vy",
    "Gantt view" : "Gantt-vy",
    "Hide archived cards" : "Göm arkiverade kort",
    "Show archived cards" : "Visa arkiverade kort",
    "Toggle compact mode" : "Växla kompakt läge",
    "Hide card cover images" : "Dölj omslagsbilder för kort",
    "Show card cover images" : "Visa omslagsbilder för kort",
    "Open details" : "Öppna detaljer",
    "Details" : "Detaljer",
    "Deck settings" : "Deck-inställningar",
    "General" : "Allmänt",
    "Use bigger card view" : "Visa större kort",
    "Appearance" : "Utseende",
    "Show card ID badge" : "Visa kortets ID-märke",
    "Show boards in calendar/tasks" : "Visa tavlor i kalender / uppgifter",
    "Admin settings" : "Administratörsinställningar",
    "Limit board creation to some groups" : "Begränsa skapandet av tavlor till vissa grupper",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Användare utanför dessa grupper kommer inte att kunna skapa sina egna tavlor, men kommer fortfarande att kunna arbeta på tavlor som har delats med dem.",
    "Enable federation" : "Aktivera federering",
    "Board actions" : "Tavlåtgärder",
    "Scroll sideways" : "Rulla i sidled",
    "Navigate between cards" : "Navigera mellan korten",
    "Close card details" : "Stäng kortdetaljer",
    "Search" : "Sök",
    "Show card filters" : "Visa kortfilter",
    "Clear card filters" : "Rensa kortfilter",
    "Show those shortcuts" : "Visa dessa genvägar",
    "Card actions" : "Kortåtgärder",
    "Open card details" : "Öppna kortdetaljer",
    "Edit the card title" : "Ändra kortets titel",
    "Assign yourself to the current card" : "Tilldela dig själv till det aktuella kortet",
    "Archive/unarchive the current card" : "Arkivera/avarkivera det aktuella kortet",
    "Mark card as completed/not completed" : "Markera kortet som slutfört/inte slutfört",
    "Open card menu" : "Öppna kortmenyn",
    "Currently present people" : "Närvarande personer",
    "Loading board" : "Läser in tavla",
    "Board not found" : "Tavlan hittades inte",
    "Create a new list to add cards to this board" : "Skapa en ny lista för att lägga till kort på denna tavla",
    "Sharing" : "Delning",
    "Tags" : "Taggar",
    "Deleted items" : "Borttagna poster",
    "Activity" : "Aktivitet",
    "Deleted lists" : "Borttagna listor",
    "Undo" : "Ångra",
    "Deleted cards" : "Borttagna kort",
    "Hours" : "Timmar",
    "Days" : "Dagar",
    "Weeks" : "Veckor",
    "Months" : "Månader",
    "No cards yet" : "Inga kort än",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Ange ett startdatum och ett förfallodatum på dina kort för att se dem i Gantt-diagrammet",
    "Cards without dates ({count})" : "Kort utan datum ({count})",
    "Failed to create share with {displayName}" : "Det gick inte att skapa delning med {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Är du säker på att du vill överföra tavlan {title} till {user}?",
    "Transfer the board." : "Överför tavlan.",
    "Transfer" : "Överför",
    "The board has been transferred to {user}" : "Tavlan har överförts till {user}",
    "Failed to transfer the board to {user}" : "Det gick inte att överföra tavlan till {user}",
    "Board owner" : "Tavlans ägare",
    "(Group)" : "(Grupp)",
    "(Team)" : "(Team)",
    "(remote)" : "(fjärr)",
    "Can edit" : "Kan redigera",
    "Can share" : "Kan dela",
    "Can manage" : "Kan hantera",
    "Owner" : "Ägare",
    "Delete" : "Ta bort",
    "List deleted" : "Listan har tagits bort",
    "Edit list title" : "Redigera listans titel",
    "Archive all cards" : "Arkivera alla kort",
    "Unarchive all cards" : "Ta ut alla kort ur arkivet",
    "Do not set cards as \"done\"" : "Markera inte kort som “klara”",
    "Set cards as \"done\"" : "Markera kort som “klara”",
    "Delete list" : "Ta bort lista",
    "Archive all cards in this list" : "Arkivera alla kort i denna lista",
    "Unarchive all cards in this list" : "Ta ut alla kort i denna lista ur arkivet",
    "Add a new card" : "Lägg till ett nytt kort",
    "Card name" : "Kortets namn",
    "title and color value must be provided" : "titel och färg måste anges",
    "Edit" : "Redigera",
    "Add a new tag" : "Lägg till en ny tagg",
    "Board name" : "Tavlans namn",
    "Members" : "Medlemmar",
    "Assign to users/groups/team" : "Tilldela användare, grupper eller team",
    "Assign a user to this card…" : "Tilldela en användare till kortet …",
    "Select a user to assign to this card…" : "Välj en användare att tilldela kortet …",
    "File to share" : "Fil att dela",
    "Invalid path selected" : "Ogiltig sökväg vald",
    "Upload new files" : "Ladda upp nya filer",
    "Share from Files" : "Dela från Filer",
    "Pending share" : "Väntande delning",
    "Add this attachment" : "Lägg till denna bilaga",
    "Show in Files" : "Visa i Filer",
    "Download" : "Ladda ner",
    "Remove attachment" : "Ta bort bilaga",
    "Delete Attachment" : "Ta bort bilaga",
    "Restore Attachment" : "Återställ bilaga",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Ändrad: {modifiedDate} ⸱ Skapad: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Ändrad: {modifiedDate}\nSkapad: {createdDate}\nSkapad av: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Ändrad: {modifiedDate}\nSkapad: {createdDate}",
    "The title cannot be empty." : "Titeln får inte lämnas tom.",
    "Cannot close unsaved card!" : "Det går inte att stänga osparat kort!",
    "Open in sidebar view" : "Öppna i sidofältet",
    "Open in bigger view" : "Öppna i större vy",
    "Comments" : "Kommentarer",
    "Failed to load comments" : "Det gick inte att läsa in kommentarerna",
    "No comments yet. Begin the discussion!" : "Inga kommentarer än. Börja diskussionen!",
    "The comment cannot be empty." : "Kommentaren kan inte vara tom.",
    "The comment cannot be longer than 1000 characters." : "Kommentaren kan inte vara längre än 1000 tecken.",
    "Submit" : "Skicka",
    "Created:" : "Skapad:",
    "In reply to" : "Som svar på",
    "Cancel reply" : "Avbryt svar",
    "Reply" : "Svara",
    "Update" : "Uppdatera",
    "Assign dependent cards" : "Tilldela beroende kort",
    "Mark as done" : "Markera som klar",
    "Mark as not done" : "Markera som ej klar",
    "Remove dependency" : "Ta bort beroende",
    "Add dependent card" : "Lägg till beroende kort",
    "Assign a dependent card…" : "Tilldela ett beroende kort…",
    "Select a dependent card…" : "Välj ett beroende kort…",
    "Write a description …" : "Skriv en beskrivning …",
    "The description has been changed by another user." : "Beskrivningen har ändrats av en annan användare",
    "Could not save description" : "Det gick inte att spara beskrivningen",
    "Description" : "Beskrivning",
    "(Unsaved)" : "(Inte sparad)",
    "(Saving…)" : "(Sparar…)",
    "Formatting help" : "Formateringshjälp",
    "Edit description" : "Ändra beskrivning",
    "View description" : "Visa beskrivning",
    "Add Attachment" : "Lägg till bilaga",
    "Choose attachment" : "Välj bilaga",
    "Select Date" : "Välj datum",
    "Later today – {timeLocale}" : "Senare idag – {timeLocale}",
    "Set due date for later today" : "Ställ in förfallodatum för senare idag",
    "Tomorrow – {timeLocale}" : "Imorgon – {timeLocale}",
    "Set due date for tomorrow" : "Ställ in förfallodatum för imorgon",
    "This weekend – {timeLocale}" : "Den här helgen – {timeLocale}",
    "Set due date for this weekend" : "Ställ in förfallodatum för denna helg",
    "Next week – {timeLocale}" : "Nästa vecka – {timeLocale}",
    "Set due date for next week" : "Ställ in förfallodatum för nästa vecka",
    "Assign a due date to this card…" : "Tilldela kortet ett förfallodatum …",
    "Set a due date" : "Ange ett förfallodatum",
    "Add due date" : "Lägg till förfallodatum",
    "Choose a date" : "Välj ett datum",
    "Remove due date" : "Ta bort förfallodatum",
    "Due at:" : "Förfaller:",
    "Not done" : "Inte klar",
    "Unarchive card" : "Avarkivera kort",
    "Archive card" : "Arkivera kort",
    "Assign a start date to this card…" : "Tilldela ett startdatum till detta kort…",
    "Set a start date" : "Ange startdatum",
    "Add start date" : "Lägg till startdatum",
    "Remove start date" : "Ta bort startdatum",
    "Assign a tag to this card…" : "Tilldela kortet en tagg …",
    "Select or create a tag…" : "Välj eller skapa en tagg …",
    "Create a new tag:" : "Skapa en ny tagg:",
    "(group)" : " (grupp)",
    "{count} comments, {unread} unread" : "{count} kommentarer, {unread} olästa",
    "Todo items" : "Att göra-poster",
    "Edit card title" : "Ändra korttitel",
    "Open link" : "Öppna länken",
    "Card deleted" : "Kortet har raderats",
    "Change card color" : "Ändra kortfärg",
    "Edit title" : "Redigera titel",
    "Assign to me" : "Tilldela till mig",
    "Unassign myself" : "Ta bort från mig själv",
    "Delete card" : "Ta bort kort",
    "All boards" : "Alla tavlor",
    "Archived boards" : "Arkiverade tavlor",
    "Shared with you" : "Delad med dig",
    "Cancel edit" : "Avbryt redigering",
    "Save board" : "Spara tavla",
    "Board {0} deleted" : "Tavlan {0} har tagits bort",
    "All cards" : "Alla kort",
    "Only assigned cards" : "Bara tilldelade kort",
    "No reminder" : "Ingen påminnelse",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Är du säker på att du vill ta bort tavlan {title}? Alla tavlans uppgifter, inklusive arkiverade kort, tas bort.",
    "Delete the board?" : "Ta bort tavlan?",
    "Are you sure you want to leave the board {title}?" : "Är du säker på att du vill lämna tavlan {title}?",
    "Leave the board?" : "Lämna tavlan?",
    "Leave" : "Lämna",
    "Failed to leave the board" : "Det gick inte att lämna tavlan",
    "Exporting board..." : "Exporterar tavlan …",
    "An error occurred" : "Ett fel uppstod",
    "Board details" : "Taveldetaljer",
    "Edit board" : "Ändra tavla",
    "Clone board" : "Kopiera tavla",
    "Unarchive board" : "Ta bort tavlan ur arkivet",
    "Archive board" : "Arkivera tavla",
    "Export board" : "Exportera tavla",
    "Turn on due date reminders" : "Aktivera påminnelser om förfallodatum",
    "Turn off due date reminders" : "Inaktivera påminnelser om förfallodatum",
    "Remove as default board" : "Ta bort som standardtavla",
    "Set as default board" : "Ange som standardtavla",
    "Due date reminders" : "Påminnelser om förfallodatum",
    "Assigned cards" : "Tilldelade kort",
    "No notifications" : "Inga aviseringar",
    "Delete board" : "Ta bort tavla",
    "Leave board" : "Lämna tavla",
    "Importing board..." : "Importerar tavlan …",
    "Board imported successfully" : "Tavlan har importerats",
    "Import board" : "Importera tavla",
    "Clone {boardTitle}" : "Klona {boardTitle}",
    "Clone cards" : "Klona kort",
    "Clone assignments" : "Klona tilldelningar",
    "Clone labels" : "Klona etiketter",
    "Clone due dates" : "Kopiera förfallodatum",
    "Advanced options" : "Avancerade inställningar",
    "Move all cards to the first list" : "Flytta alla kort till den första listan",
    "Restore archived cards" : "Återställ arkiverade kort",
    "Clone" : "Klona",
    "Export {boardTitle}" : "Exportera {boardTitle}",
    "Export as JSON" : "Exportera som JSON",
    "Export as CSV" : "Exportera som CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Observera: Endast JSON-formatet stöds för import tillbaka till Deck-appen.",
    "Export" : "Exportera",
    "Loading filtered view" : "Laddar filtrerad vy",
    "Search for {searchQuery} in other boards" : "Sök efter {searchQuery} i andra tavlor",
    "Search for {searchQuery} in all boards" : "Sök efter {searchQuery} i alla tavlor",
    "No results found" : "Inga resultat hittades",
    "Deck board {name}\n* Last modified on {lastMod}" : "Deck-tavlan {name}\n* Senast ändrad {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Skapades {created}\n* Ändrades senast {lastMod}\n* {nbAttachments} bilagor\n* {nbComments} kommentarer",
    "{nbCards} cards" : "{nbCards} kort",
    "Due on {date}" : "Förfaller {date}",
    "{stack} in {board}" : "{stack} i {board}",
    "Click to expand description" : "Klicka för att utöka beskrivningen",
    "Click to expand comment" : "Klicka för att utöka kommentaren",
    "Create card" : "Skapa kort",
    "Create a new card" : "Skapa ett nytt kort",
    "Card title" : "Korttitel",
    "Creating the new card …" : "Skapar det nya kortet …",
    "Card \"{card}\" was added to \"{board}\"" : "Kortet \"{card}\" lades till på \"{board}\"",
    "Open card" : "Öppna kort",
    "Close" : "Stäng",
    "No upcoming cards" : "Inga kommande kort",
    "upcoming cards today" : "kommande kort idag",
    "upcoming cards tomorrow" : "kommande kort imorgon",
    "upcoming cards" : "kommande kort",
    "New card" : "Nytt kort",
    "Link to a board" : "Länka till en tavla",
    "Link to a card" : "Länka till ett kort",
    "Create a card" : "Skapa ett kort",
    "Message from {author} in {conversationName}" : "Meddelande från {author} i {conversationName}",
    "Something went wrong" : "Något gick fel",
    "Failed to upload {name}" : "Det gick inte att ladda upp {name}",
    "Maximum file size of {size} exceeded" : "Den maximala filstorleken på {size} har överskridits",
    "Assigned users" : "Tilldelade användare",
    "Due date" : "Förfallodatum",
    "Created" : "Skapat",
    "Modified" : "Ändrad",
    "Error creating the share" : "Kunde inte skapa delning",
    "Share with a Deck card" : "Dela med ett Deck-kort",
    "Share {file} with a Deck card" : "Dela {file} med ett Deck-kort",
    "Share" : "Dela",
    "The card \"%s\" on \"%s\" has reached its due date." : "Kortet \"%s\" på \"%s\" har nått sitt förfallodatum.",
    "Share board with a user, group or team …" : "Dela tavlan med en användare, grupp eller team …",
    "Searching for users, groups and teams …" : "Söker efter användare, grupper och team …",
    "No participants found" : "Inga deltagare hittades",
    "Save" : "Spara",
    "seconds ago" : "sekunder sedan",
    "Today" : "Idag",
    "Tomorrow" : "Imorgon",
    "No due" : "Inget förfallodatum"
},
"nplurals=2; plural=(n != 1);");
