<?php

declare(strict_types=1);

namespace Psalm\Node\Stmt\TraitUseAdaptation;

use PhpParser\Node\Stmt\TraitUseAdaptation\Precedence;
use Psalm\Node\VirtualNode;

final class VirtualPrecedence extends Precedence implements VirtualNode
{

}
