OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "{board} taula berria sortu duzu",
    "{user} has created a new board {board}" : "{user} erabiltzaileak {board} taula berria sortu du",
    "You have deleted the board {board}" : "{board} taula ezabatu duzu",
    "{user} has deleted the board {board}" : "{user} erabiltzaileak {board} taula ezabatu du",
    "You have restored the board {board}" : "{board} taula leheneratu duzu",
    "{user} has restored the board {board}" : "{user} erabiltzaileak {board} taula leheneratu du",
    "You have shared the board {board} with {acl}" : "{board} taula {acl} erabiltzailearekin partekatu duzu",
    "{user} has shared the board {board} with {acl}" : "{user} erabiltzaileak {board} taula {acl} erabiltzailearekin partekatu du",
    "You have removed {acl} from the board {board}" : "{acl} erabiltzailea kendu duzu {board} taulatik",
    "{user} has removed {acl} from the board {board}" : "{user} erabiltzaileak {acl} erabiltzailea kendu du {board} taulatik",
    "You have renamed the board {before} to {board}" : "{before} taula zena {board} bezala berrizendatu duzu",
    "{user} has renamed the board {before} to {board}" : "{user} erabiltzaileak {before} taula zena {board} bezala berrizendatu du",
    "You have archived the board {board}" : "{board} taula artxibatu duzu",
    "{user} has archived the board {before}" : "{user} erabiltzaileak {before} taula artxibatu du",
    "You have unarchived the board {board}" : "{board} taula artxibotik berreskuratu duzu",
    "{user} has unarchived the board {before}" : "{user} erabiltzaileak {before} taula artxibotik kendu du",
    "You have created a new list {stack} on board {board}" : "{stack} zerrenda berria sortu duzu {board} taulan",
    "{user} has created a new list {stack} on board {board}" : "{user} erabiltzaileak {stack} zerrenda berria sortu du {board} taulan",
    "You have renamed list {before} to {stack} on board {board}" : "{before} zerrenda zena {stack} bezala berrizendatu duzu {board} taulan",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} erabiltzaileak {before} zerrenda zena {stack} bezala berrizendatu du {board} taulan",
    "You have deleted list {stack} on board {board}" : "{stack} zerrenda ezabatu duzu {board} taulan",
    "{user} has deleted list {stack} on board {board}" : "{user} erabiltzaileak {stack} zerrenda ezabatu du {board} taulan",
    "You have created card {card} in list {stack} on board {board}" : "{card} txartela sortu duzu {board} taulako {stack} zerrendan",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} erabiltzaileak {card} txartela sortu du {board} taulako {stack} zerrendan",
    "You have deleted card {card} in list {stack} on board {board}" : "{card} txartela ezabatu duzu {board} taulako {stack} zerrendan",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} erabiltzaileak {card} txartela ezabatu du {board} taulako {stack} zerrendan",
    "You have renamed the card {before} to {card}" : "{before} taula zena {card} bezala berrizendatu duzu",
    "{user} has renamed the card {before} to {card}" : "{user} erabiltzaileak {before} taula zena {card} bezala berrizendatu du",
    "You have added a description to card {card} in list {stack} on board {board}" : "{board} taulako {stack} zerrendako {card} txartelari deskripzioa gehitu diozu",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} erabiltzaileak {board} taulako {stack} zerrendako {card} txartelari deskripzioa gehitu dio",
    "You have updated the description of card {card} in list {stack} on board {board}" : "{board} taulako {stack} zerrendako {card} txartelaren deskripzioa eguneratu duzu",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} erabiltzaileak {board} taulako {stack} zerrendako {card} txartelaren deskripzioa eguneratu du",
    "You have archived card {card} in list {stack} on board {board}" : "{board} taulako {stack} zerrendako {card} txartela artxibatu duzu",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} erabiltzaileak {board} taulako {stack} zerrendako {card} txartela artxibatu du",
    "You have unarchived card {card} in list {stack} on board {board}" : "{board} taulako {stack} zerrendako {card} txartela berreskuratu duzu artxibotik",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} erabiltzaileak {board} taulako {stack} zerrendako {card} txartela berreskuratu du artxibotik",
    "You have marked the card {card} as done in list {stack} on board {board}" : " {card} txartela egindako gisa markatu duzu {board} taulako {stack}  zerrendan",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} erabiltzaileak {card} txartela egindako gisa markatu du {board} taulako {stack} zerrendan",
    "You have marked the card {card} as undone in list {stack} on board {board}" : " {card} txartela egin gabeko gisa markatu duzu {board} taulako {stack} zerrendan",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} erabiltzaileak {card} txartela egin gabeko gisa markatu du {board} taulako {stack} zerrendan",
    "You have removed the due date of card {card}" : "{card} txartelari epemuga kendu diozu",
    "{user} has removed the due date of card {card}" : "{user} erabiltzaileak {card} txartelari epemuga kendu dio",
    "You have set the due date of card {card} to {after}" : "{card} txartelari {after} epemuga ezarri diozu",
    "{user} has set the due date of card {card} to {after}" : "{user} erabiltzaileak {card} txartelari {after} epemuga ezarri dio",
    "You have updated the due date of card {card} to {after}" : "{card} txartelari epemuga eguneratu diozu: {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} erabiltzaileak {card} txartelari epemuga eguneratu dio: {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "{board} taulako {stack} zerrendako {card} txartelari {label} etiketa gehitu diozu",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} erabiltzaileak {board} taulako {stack} zerrendako {card} txartelari {label} etiketa gehitu dio",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "{board} taulako {stack} zerrendako {card} txartelari {label} etiketa kendu diozu",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} erabiltzaileak {board} taulako {stack} zerrendako {card} txartelari {label} etiketa kendu dio",
    "You have assigned {assigneduser} to card {card} on board {board}" : "{board} taulako {card} txartela {assigneduser} erabiltzaileari esleitu diozu",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} erabiltzaileak {board} taulako {card} txartela {assigneduser} erabiltzaileari esleitu dio",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "{board} taulako {card} txartela {assigneduser} erabiltzaileari esleitzea ezeztatu duzu",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} erabiltzaileak {board} taulako {card} txartela {assigneduser} erabiltzaileari esleitzea ezeztatu du",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "{card} txartela {stackBefore} zerrendatik {stack} zerrendara aldatu duzu",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} erabiltzaileak {card} txartela {stackBefore} zerrendatik {stack} zerrendara aldatu du",
    "You have added the attachment {attachment} to card {card}" : "{attachment} eranskina gehitu diozu {card} txartelari",
    "{user} has added the attachment {attachment} to card {card}" : "{user} erabiltzaileak {attachment} eranskina gehitu dio {card} txartelari",
    "You have updated the attachment {attachment} on card {card}" : "{card} txartelaren {attachment} eranskina eguneratu duzu",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} erabiltzaileak {card} txartelaren {attachment} eranskina eguneratu du",
    "You have deleted the attachment {attachment} from card {card}" : "{card} txartelaren {attachment} eranskina ezabatu duzu",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} erabiltzaileak {card} txartelaren {attachment} eranskina ezabatu du",
    "You have restored the attachment {attachment} to card {card}" : "{card} txartelaren {attachment} eranskina leheneratu duzu",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} erabiltzaileak {card} txartelaren {attachment} eranskina leheneratu du",
    "You have commented on card {card}" : "{card} txartela iruzkindu duzu",
    "{user} has commented on card {card}" : "{user} erabiltzaileak {card} txartela iruzkindu du",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "<strong>Deck</strong> aplikazioan aldaketak",
    "A <strong>board, list or card</strong> was changed" : "<strong>Taula, zerrenda edo txartel</strong> bat aldatu da",
    "A <strong>comment</strong> was created on a card" : "<strong>Iruzkin bat</strong> sortu da txartel batean",
    "A <strong>card description</strong> has been changed" : "<strong>Txartel baten deskribapena</strong> aldatu da",
    "The file was uploaded" : "Fitxategia igo da",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Igotako fitxategiak php.ini fitxategiko upload_max_filesize direktiban zehazturikoa gainditzen du",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Igotako fitxategiak HTML formularioan ezarritako MAX_FILE_SIZE direktiban zehazturikoa gainditzen du",
    "The file was only partially uploaded" : "Fitxategia erdizka igo da",
    "No file was uploaded" : "Ez da fitxategirik igo",
    "Missing a temporary folder" : "Aldi baterako karpeta bat falta da",
    "Could not write file to disk" : "Ezin izan da fitxategia diskoan idatzi",
    "A PHP extension stopped the file upload" : "PHP hedapen batek fitxategiaren igoera gelditu du",
    "No file uploaded or file size exceeds maximum of %s" : "Ez da fitxategirik igo edo fitxategi-tamainak gehienezko %s muga gainditzen du",
    "Cards due today" : "Gaurko txartelak",
    "Cards due tomorrow" : "Biharko txartelak",
    "Upcoming cards" : "Hurrengo txartelak",
    "Load more" : "Kargatu gehiago",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "\"%s\" txartela \"%s\"-n %sk esleitu zaizu.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user}-k {deck-board}-ko  {deck-card} txartela esleitu dizu.",
    "The card {deck-card} on {deck-board} has reached its due date." : " {deck-board}-ko {deck-card} txartela epe-mugara ailegatu da.",
    "%s has mentioned you in a comment on \"%s\"." : "%sk aipatu dizu \"%s\"-ri buruzko iruzkin batean.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user}-k zure izena aipatu du {deck-card}-ko iruzkin batean.",
    "The board \"%s\" has been shared with you by %s." : "\"%s\" taula zurekin partekatu du %s(e)k",
    "{user} has shared {deck-board} with you." : "{user} erabiltzaileak {deck-board} partekatu du zurekin.",
    "Deck board" : "Deck taula",
    "Owned by %1$s" : "Jabea: %1$s",
    "Deck boards, cards and comments" : "Deck taulak, txartelak eta iruzkinak",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "%1$s-(e)tik,%2$s/%3$s-n, jabea:%4$s",
    "Create a new deck card" : "Sortu deck txartel berri bat",
    "Card comments" : "Txartelaren iruzkinak",
    "%s on %s" : "%s hemen: %s",
    "Deck boards and cards" : "Deck taulak eta txartelak",
    "No data was provided to create an attachment." : "Ez da daturik eman eranskina sortzeko.",
    "Finished" : "Bukatuta",
    "To review" : "Errebisatzeko",
    "Action needed" : "Ekintza beharrezkoa",
    "Later" : "Beranduago",
    "copy" : "kopiatu",
    "To Do" : "Egiteke",
    "In Progress" : "Abian",
    "Done" : "Egina",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Iruzkin honek %s karaktere baino gehiago ditu.\n%s izena duen txartelari eranskin gisa gehitu da.\nURLan eskura daiteke: %s.",
    "Attachments" : "Eranskinak",
    "File" : "Fitxategia",
    "Card not found" : "Txartela ez da aurkitu",
    "Path is already shared with this card" : "Bidea dagoeneko partekatu da txartel honekin",
    "Invalid date, date format must be YYYY-MM-DD" : "Data baliogabea, dataren formatuak UUUU-HH-EE izan behar du",
    "Personal planning and team project organization" : "Plangintza pertsonala eta talde proiektuen kudeaketa",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck aplikazioa plangintza pertsonalera eta proiektuen antolaketara zuzenduta dagoen Nextcloudekin integratutako kanban moduko tresna bat da.\n\n\n-📥 Gehitu atazak txarteletan eta ordenatu\n- 📄 Idatzi ohar gehigarriak Markdown erabiliz\n- 🔖 Esleitu etiketak antolaketa are gehiago hobetzeko\n- 👥 Partekatu zure talde, lagun edo familiarekin\n- 📎 Erantsi fitxategiak eta kapsulatu zure Markdown deskribapenean\n- 💬 Eztabaidatu zure taldearekin iruzkinak erabiliz\n- ⚡ Egin aldaketen jarraipena jarduera jarioa erabiliz\n- 🚀 Antolatu zure proiektua",
    "Add board" : "Gehitu taula",
    "Card details" : "Txartelaren xehetasunak",
    "Select the board to link to a project" : "Hautatu taula proiektu bati estekatzeko",
    "Search by board title" : "Bilatu taula izenez",
    "Select board" : "Hautatu taula",
    "Select a board" : "Hautatu taula bat",
    "No lists available" : "Ez dago zerrendarik eskuragarri",
    "Select a list" : "Hautatu zerrenda bat",
    "Move card" : "Mugitu txartela",
    "Select the card to link to a project" : "Hautatu proiektu bati estekatzeko txartela",
    "Link to card" : "Lotu txartelera",
    "Select a card" : "Hautatu txartel bat",
    "Cancel" : "Utzi",
    "This board is read only" : "Taula hau irakurtzeko soilik da",
    "Drop your files to upload" : "Jaregin zure fitxategiak igotzeko",
    "File already exists" : "Badago izen bereko fitxategi bat",
    "A file with the name {filename} already exists." : "{filename} izeneko fitxategia existitzen da dagoeneko.",
    "Do you want to overwrite it?" : "Gainidatzi nahi duzu?",
    "Overwrite file" : "Gainidatzi fitxategia",
    "Keep existing file" : "Mantendu dagoen fitxategia",
    "Add card" : "Gehitu txartela",
    "Archived cards" : "Artxibatutako txartelak",
    "Add list" : "Gehitu zerrenda",
    "List name" : "Zerrendaren izena",
    "Clear search" : "Garbitu bilaketa",
    "Active filters" : "Iragazki aktiboak",
    "Apply filter" : "Ezarri iragazkia",
    "Filter by tag" : "Iragazi etiketaz",
    "Filter by assigned user" : "Iragazi esleitutako erabiltzailez",
    "Unassigned" : "Esleitu gabea",
    "Filter by status" : "Iragazi egoeraren arabera",
    "Open and completed" : "Ireki eta osatutakoak",
    "Open" : "Ireki",
    "Completed" : "Osatua",
    "Filter by due date" : "Iragazi epe-mugaz",
    "Overdue" : "Atzeratuta",
    "Next 24 hours" : "Hurrengo 24 orduak",
    "Next 7 days" : "Hurrengo 7 egunetan",
    "Next 30 days" : "Hurrengo 30 egunetan",
    "No due date" : "Epe-mugarik gabe",
    "Clear filter" : "Garbitu iragazia",
    "View Modes" : "Bistaratze moduak",
    "Toggle View Modes" : "Txandakatu bisataratze moduak",
    "Hide archived cards" : "Ezkutatu artxibatutako txartelak",
    "Show archived cards" : "Erakutsi artxibatutako txartelak",
    "Toggle compact mode" : "Txandakatu modu trinkoa",
    "Hide card cover images" : "Ezkutatu txartelen azaleko irudiak",
    "Show card cover images" : "Erakutsi txartelen azaleko irudiak",
    "Open details" : "Ireki xehetasunak",
    "Details" : "Xehetasunak",
    "Deck settings" : "Deck ezarpenak",
    "General" : "Orokorra",
    "Use bigger card view" : "Erabili txartel ikuspegi handiagoa",
    "Appearance" : "Itxura",
    "Show card ID badge" : "Erakutsi txartelaren ID plaka",
    "Show boards in calendar/tasks" : "Erakutsi taulak egutegi/zereginetan",
    "Limit board creation to some groups" : "Mugatu taulak sortzea talde batzuei",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Talde horietatik kanpo dauden erabiltzaileek ezingo dituzte haien taulak sortu, baina partekatu zaizkien tauletan lan egiten jarraitu ahal izango dute.",
    "Board actions" : "Taularen ekintzak",
    "Scroll sideways" : "Korritu alboetara",
    "Navigate between cards" : "Nabigatu txartelen artean",
    "Close card details" : "Itxi txartelaren xehetasunak",
    "Search" : "Bilatu",
    "Show card filters" : "Erakutsi txartelaren iragazkiak",
    "Clear card filters" : "Garbitu txartelaren iragazkiak",
    "Show those shortcuts" : "Erakutsi lasterbide horiek",
    "Card actions" : "Txartelaren ekintzak",
    "Open card details" : "Ireki txartelaren xehetasunak",
    "Edit the card title" : "Editatu txartelaren izenburua",
    "Assign yourself to the current card" : "Esleitu zeure burua uneko txartelari",
    "Archive/unarchive the current card" : "Artxibatu/desartxibatu uneko txartela",
    "Mark card as completed/not completed" : "Markatu txartela osatuta/ez osatuta bezala",
    "Open card menu" : "Ireki txartelaren menua",
    "Currently present people" : "Unean dauden pertsonak",
    "Loading board" : "Taula kargatzen",
    "Board not found" : "Ez da taula aurkitu",
    "Create a new list to add cards to this board" : "Sortu zerrenda berria, taula honetan txartelak gehitzeko",
    "Sharing" : "Partekatzea",
    "Tags" : "Etiketak",
    "Deleted items" : "Ezabatutako elementuak",
    "Activity" : "Jarduerak",
    "Deleted lists" : "Ezabatutako zerrendak",
    "Undo" : "Desegin",
    "Deleted cards" : "Ezabatutako txartelak",
    "Hours" : "Orduak",
    "Days" : "Egunak",
    "Weeks" : "Asteak",
    "Months" : "Hilabeteak",
    "No cards yet" : "Ez dago txartelik oraindik",
    "Failed to create share with {displayName}" : "Ezin izan da {displayName}-(r)ekin partekatzea sortu",
    "Are you sure you want to transfer the board {title} to {user}?" : "Ziur {title} taula {user}-(r)i transferitu nahi diozula?",
    "Transfer the board." : "Transferitu taula.",
    "Transfer" : "Transferitu",
    "The board has been transferred to {user}" : "Taula {user} erabiltzaileari transferitu zaio",
    "Failed to transfer the board to {user}" : "Ezin izan zaio taula transferitu {user} erabiltzaileari",
    "Board owner" : "Taularen jabea",
    "(Group)" : "(Taldea)",
    "(Team)" : "(Lantaldea)",
    "(remote)" : "(urrunekoa)",
    "Can edit" : "Editatu dezake",
    "Can share" : "Partekatu dezake",
    "Can manage" : "Kudeatu dezake",
    "Owner" : "Jabea",
    "Delete" : "Ezabatu",
    "List deleted" : "Zerrenda ezabatua",
    "Edit list title" : "Editatu zerrendaren izenburua",
    "Archive all cards" : "Artxibatu txartel guztiak",
    "Unarchive all cards" : "Berreskuratu txartel guztiak artxibotik",
    "Delete list" : "Zerrenda ezabatu",
    "Archive all cards in this list" : "Artxibatu zerrenda honetako txartel guztiak",
    "Unarchive all cards in this list" : "Berreskuratu artxibotik zerrenda honetako txartel guztiak",
    "Add a new card" : "Gehitu txartel berri bat",
    "Card name" : "Txartel izena",
    "title and color value must be provided" : "izenburu eta kolore balioak hornitu behar dira",
    "Edit" : "Editatu",
    "Add a new tag" : "Gehitu etiketa berri bat",
    "Board name" : "Taularen izena",
    "Members" : "Partaideak",
    "Assign to users/groups/team" : "Esleitu erabiltzaile/talde/lantaldeei",
    "Assign a user to this card…" : "Esleitu erabiltzaile bat txartel honi...",
    "Select a user to assign to this card…" : "Hautatu erabiltzaile bat txartel honetara esleitzeko...",
    "File to share" : "Partekatzeko fitxategia",
    "Invalid path selected" : "Bide baliogabea hautatuta",
    "Upload new files" : "Igo fitxategi berriak",
    "Share from Files" : "Partekatu Fitxategiak-etik",
    "Pending share" : "Partekatzeko zain",
    "Add this attachment" : "Gehitu eranskin hau",
    "Show in Files" : "Erakutsi Fitxategiak aplikazioan",
    "Download" : "Deskargatu",
    "Remove attachment" : "Kendu eranskina",
    "Delete Attachment" : "Ezabatu eranskina",
    "Restore Attachment" : "Berezarri eranskina",
    "The title cannot be empty." : "Izenburua ezin da hutsik egon.",
    "Open in sidebar view" : "Ireki alboko barraren ikuspegian",
    "Open in bigger view" : "Ireki ikuspegi handiago batean",
    "Comments" : "Iruzkinak",
    "Failed to load comments" : "Iruzkinen kargatzeak huts egin du.",
    "No comments yet. Begin the discussion!" : "Ez dago iruzkinik oraindik. Hasi eztabaidatzen!",
    "The comment cannot be empty." : "Iruzkina ezin da hutsik egon.",
    "The comment cannot be longer than 1000 characters." : "Iruzkina ezin da 1000 karaktere baino luzeagoa izan.",
    "Submit" : "Bidali",
    "Created:" : "Sortua:",
    "In reply to" : "Honi erantzunez",
    "Cancel reply" : "Utzi erantzuna bertan behera",
    "Reply" : "Erantzun",
    "Update" : "Eguneratu",
    "Mark as done" : "Markatu egindako gisa",
    "Mark as not done" : "Markatu ez egindako gisa",
    "Write a description …" : "Idatzi deskribapen bat ...",
    "Description" : "Deskribapena",
    "(Unsaved)" : "(Gorde gabe)",
    "(Saving…)" : "(Gordetzen...)",
    "Formatting help" : "Formatua emateko laguntza",
    "Edit description" : "Editatu azalpena",
    "View description" : "Ikusi deskribapena",
    "Add Attachment" : "Gehitu eranskina",
    "Choose attachment" : "Aukeratu eranskina",
    "Select Date" : "Hautatu data",
    "Later today – {timeLocale}" : "Beranduago gaur – {timeLocale}",
    "Set due date for later today" : "Ezarri epemuga gaurgero",
    "Tomorrow – {timeLocale}" : "Bihar – {timeLocale}",
    "Set due date for tomorrow" : "Ezarri epemuga biharko",
    "This weekend – {timeLocale}" : "Hurrengo asteburua – {timeLocale}",
    "Set due date for this weekend" : "Ezarri epemuga asteburu honetarako",
    "Next week – {timeLocale}" : "Hurrengo astea – {timeLocale}",
    "Set due date for next week" : "Ezarri epemuga datorren asterako",
    "Assign a due date to this card…" : "Esleitu iraungitze data bat txartel honi...",
    "Set a due date" : "Ezarri epe-muga",
    "Add due date" : "Gehitu epemuga",
    "Choose a date" : "Aukeratu data",
    "Remove due date" : "Ezabatu epe-muga",
    "Due at:" : "Epemuga:",
    "Not done" : "Egin gabe",
    "Unarchive card" : "Berreskuratu txartela artxibotik",
    "Archive card" : "Artxibatu txartela",
    "Assign a tag to this card…" : "Esleitu etiketa bat txartel honi...",
    "Select or create a tag…" : "Hautatu edo sortu etiketa bat...",
    "Create a new tag:" : "Sortu etiketa berria:",
    "(group)" : "(taldea)",
    "{count} comments, {unread} unread" : "{count} iruzkin, {unread} irakurri gabe",
    "Todo items" : "Elementuak egiteke",
    "Edit card title" : "Editatu txartelaren izenburua",
    "Open link" : "Ireki esteka",
    "Card deleted" : "Txartela ezabatuta",
    "Edit title" : "Editatu izenburua",
    "Assign to me" : "Esleitu niri",
    "Unassign myself" : "Niri esleitzeari utzi",
    "Delete card" : "Ezabatu txartela",
    "All boards" : "Taula guztiak",
    "Archived boards" : "Artxibatutako taulak",
    "Shared with you" : "Zurekin partekatua",
    "Cancel edit" : "Utzi editatzeari",
    "Save board" : "Gorde mahaia",
    "Board {0} deleted" : "{0} taula ezabatu da",
    "All cards" : "Txartel guztiak",
    "Only assigned cards" : "Bakarrik esleitutako txartelak",
    "No reminder" : "Gogorarazpenik ez",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Ziur zaude {title} taula ezabatu nahi duzula? Honek taula honen datu guztiak ezabatuko ditu, artxibatutako txartelak barne.",
    "Delete the board?" : "Taula ezabatu?",
    "Leave" : "Irten",
    "An error occurred" : "Errore bat gertatu da",
    "Board details" : "Taularen xehetasunak",
    "Edit board" : "Editatu taula",
    "Clone board" : "Klonatu taula",
    "Unarchive board" : "Atera taula artxibotik",
    "Archive board" : "Artxibatu taula",
    "Export board" : "Esportatu taula",
    "Turn on due date reminders" : "Aktibatu epemuga gogorarazpenak",
    "Turn off due date reminders" : "Desaktibatu epemuga gogorarazpenak",
    "Due date reminders" : "Epemuga gogorarazpenak",
    "Assigned cards" : "Esleitutako txartelak",
    "No notifications" : "Jakinarazpenik ez",
    "Delete board" : "Ezabatu taula",
    "Clone cards" : "Klonatu txartelak",
    "Advanced options" : "Ezarpen aurreratuak",
    "Clone" : "Klonatu",
    "Export as CSV" : "Esportatu CSV gisa",
    "Export" : "Esportatu",
    "Loading filtered view" : "Kargatzen iragazitako ikuspegia",
    "Search for {searchQuery} in all boards" : "Bilatu {searchQuery} taula guztietan",
    "No results found" : "Ez da emaitzarik aurkitu",
    "Deck board {name}\n* Last modified on {lastMod}" : " Deck-eko {izena} taula\n* Azken aldaketa {lastMod}-n",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Sorrera-data {created}\n* Azken aldaketa data {lastMod}\n* {nbAttachments} eranskin\n* {nbComments} iruzkin",
    "{nbCards} cards" : "{nbCards} karta",
    "Due on {date}" : "Iraungitze-data {date}",
    "{stack} in {board}" : "{board}-(e)ko {stack}",
    "Click to expand description" : "Egin klik deskribapena handitzeko",
    "Click to expand comment" : "Egin klik iruzkina handitzeko",
    "Create card" : "Sortu txartela",
    "Create a new card" : "Sortu txartel berri bat",
    "Card title" : "Txartelaren izenburua",
    "Creating the new card …" : "Txartel berria sortzen...",
    "Card \"{card}\" was added to \"{board}\"" : "\"{card}\" txartela gehitu da \"{board}\" taulara",
    "Open card" : "Ireki txartela",
    "Close" : "Itxi",
    "No upcoming cards" : "Ez dago hurrengo txartelik",
    "upcoming cards today" : "hurrengo txartelak gaur",
    "upcoming cards tomorrow" : "hurrengo txartelak bihar",
    "upcoming cards" : "hurrengo txartelak",
    "New card" : "Txartel berria",
    "Link to a board" : "Estekatu taula batera",
    "Link to a card" : "Estekatu txartel batera",
    "Create a card" : "Sortu txartela",
    "Message from {author} in {conversationName}" : "{author} erabiltzailearen mezua {conversationName}-an",
    "Something went wrong" : "Zerbait gaizki joan da",
    "Failed to upload {name}" : "{name} igotzeak huts egin du",
    "Maximum file size of {size} exceeded" : "Fitxategiaren gehienezko {size} tamaina gainditu da",
    "Due date" : "Epe-muga",
    "Created" : "Sortua",
    "Modified" : "Aldatua",
    "Error creating the share" : "Errorea partekatzea sortzean",
    "Share with a Deck card" : "Partekatu Deck txartel baten bidez",
    "Share {file} with a Deck card" : "Partekatu {file} Deck txartel baten bidez",
    "Share" : "Partekatu",
    "The card \"%s\" on \"%s\" has reached its due date." : " \"%s\" txartela,  \"%s\" -n dagoena, epe-mugara ailegatu da.",
    "Share board with a user, group or team …" : "Partekatu taula erabiltzaile, talde edo lantalde batekin…",
    "Searching for users, groups and teams …" : "Erabiltzaileak, taldeak, lantaldeak... bilatzen",
    "No participants found" : "Ez da parte-hartzailerik aurkitu",
    "Save" : "Gorde",
    "seconds ago" : "segundo lehenago",
    "Today" : "Gaur",
    "Tomorrow" : "Bihar",
    "No due" : "Epemugarik ez"
},
"nplurals=2; plural=(n != 1);");
