OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "새로운 보드{board}를 만들었습니다.",
    "{user} has created a new board {board}" : "{user}가 새 보드 {board}을(를) 만들었습니다.",
    "You have deleted the board {board}" : "{board} 보드를 삭제했습니다.",
    "{user} has deleted the board {board}" : "{user}가 {board} 보드를 삭제했습니다.",
    "You have restored the board {board}" : "{board} 보드를 복원했습니다.",
    "{user} has restored the board {board}" : "{user}가 {board} 보드를 복원했습니다.",
    "You have shared the board {board} with {acl}" : "{acl}과 보드 {board}을(를) 공유했습니다.",
    "{user} has shared the board {board} with {acl}" : "{user}가 {acl}과 보드 {board}을(를) 공유했습니다.",
    "You have removed {acl} from the board {board}" : "{board} 보드에서 {acl}을(를) 제거했습니다.",
    "{user} has removed {acl} from the board {board}" : "{user}가 {board} 보드에서 {acl}을(를) 제거했습니다.",
    "You have renamed the board {before} to {board}" : "보드 이름을 {before}에서 {board}로 변경했습니다.",
    "{user} has renamed the board {before} to {board}" : "{user}가 {before} 보드 이름을 {board}(으)로 변경했습니다.",
    "You have archived the board {board}" : "{board} 보드를 보관했습니다.",
    "{user} has archived the board {before}" : "{user}가 {before} 보드를 보관했습니다.",
    "You have unarchived the board {board}" : "{board} 보드를 보관 취소했습니다.",
    "{user} has unarchived the board {before}" : "{user}가 {before}에 보드 보관을 취소했습니다.",
    "You have created a new list {stack} on board {board}" : "{board} 보드에 새 목록 {stack}을(를) 생성했습니다.",
    "{user} has created a new list {stack} on board {board}" : "{user}가 {board} 보드에 새 목록 {stack}을(를) 만들었습니다.",
    "You have renamed list {before} to {stack} on board {board}" : "{board} 보드에서 {before} 목록의 이름을 {stack}으로 변경했습니다.",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user}가 {board} 보드에서 {before} 목록의 이름을 {stack}으로 변경했습니다.",
    "You have deleted list {stack} on board {board}" : "{board} 보드에서 {stack} 목록을 삭제했습니다.",
    "{user} has deleted list {stack} on board {board}" : "{user}가 {board} 보드에서 {stack} 목록을 삭제했습니다.",
    "You have created card {card} in list {stack} on board {board}" : "{board} 보드의 {stack} 목록에 {card} 카드를 생성했습니다.",
    "{user} has created card {card} in list {stack} on board {board}" : "{user}가 {board} 보드의 {stack} 목록에 {card} 카드를 생성했습니다.",
    "You have deleted card {card} in list {stack} on board {board}" : "{board} 보드의 {stack} 목록에서 {card} 카드를 삭제했습니다.",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user}가 {board} 보드의 {stack} 목록에서 {card} 카드를 삭제했습니다.",
    "You have renamed the card {before} to {card}" : "{before} 카드 이름을 {card}(으)로 변경했습니다.",
    "{user} has renamed the card {before} to {card}" : "{user}가 {before} 카드 이름을 {card}(으)로 변경했습니다.",
    "You have added a description to card {card} in list {stack} on board {board}" : "보드 {board}의 목록 {stack}에 있는 카드 {card}에 설명을 추가했습니다.",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user}가 {board} 보드의 {stack} 목록에 있는 {card} 카드에 설명을 추가했습니다.",
    "You have updated the description of card {card} in list {stack} on board {board}" : "보드 {board}의 목록 {stack}에 있는 카드 {card}에 대한 설명을 업데이트했습니다.",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user}가 {board} 보드의 {stack} 목록에 있는 {card} 카드의 설명을 업데이트했습니다.",
    "You have archived card {card} in list {stack} on board {board}" : "{board} 보드의 {stack} 목록에 {card} 카드를 보관했습니다.",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user}가 {board} 보드의 {stack} 목록에 {card} 카드를 보관했습니다.",
    "You have unarchived card {card} in list {stack} on board {board}" : "{board} 보드의 {stack} 목록에 보관되지 않은 카드 {card}을(를) 가지고 있습니다.",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user}가 {board} 보드의 {stack} 목록에 보관되지 않은 카드 {card}을(를) 가지고 있습니다.",
    "You have marked the card {card} as done in list {stack} on board {board}" : "보드 {board}의 목록 {stack}에서 카드 {card}을(를) 완료로 표시했습니다.",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user}가 {board} 보드의 {stack} 목록에서 {card} 카드를 완료로 표시했습니다.",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "보드 {board}의 목록 {stack}에서 카드 {card}을 실행 취소된 것으로 표시했습니다.",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user}가 {board} 보드의 {stack} 목록에서 {card} 카드를 실행 취소된 것으로 표시했습니다.",
    "You have removed the due date of card {card}" : "{card} 카드의 만기일을 삭제했습니다.",
    "{user} has removed the due date of card {card}" : "{user}가 {card} 카드의 만기일을 삭제했습니다.",
    "You have set the due date of card {card} to {after}" : "{card} 카드의 만기일을 {after}로 설정했습니다.",
    "{user} has set the due date of card {card} to {after}" : "{user}가 {card} 카드의 만기일을 {after}로 설정했습니다.",
    "You have updated the due date of card {card} to {after}" : "{card} 카드의 만기일을 {after}로 업데이트했습니다.",
    "{user} has updated the due date of card {card} to {after}" : "{user}가 {card} 카드의 만기일을 {after}로 업데이트했습니다.",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "{board} 보드의 {stack} 목록에 있는 {card} 카드에 {label} 태그를 추가했습니다.",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user}가 {board} 보드의 {stack} 목록에 있는 {card} 카드에 {label} 태그를 추가했습니다.",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "{board} 보드의 {stack} 목록에 있는 {card} 카드에서 {label} 태그를 제거했습니다.",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user}가 {board} 보드의 {stack} 목록에 있는 {card} 카드에서 {label} 태그를 제거했습니다.",
    "You have assigned {assigneduser} to card {card} on board {board}" : "{board} 보드의 {card} 카드에 {assigneduser}을(를) 할당했습니다.",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user}가 {board} 보드의 {card} 카드에 {assigneduser}을(를) 할당했습니다.",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "{board} 보드의 {card} 카드에서 {assigneduser}을(를) 할당 해제했습니다.",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user}가 {board} 보드의 {card} 카드에서 {assigneduser}을(를) 할당 해제했습니다.",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "{card} 카드를 {stackBefore} 목록에서 {stack}(으)로 이동했습니다.",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user}가 {card} 카드를 {stackBefore} 목록에서 {stack}(으)로 이동했습니다.",
    "You have added the attachment {attachment} to card {card}" : "{card} 카드에 첨부파일 {attachment}을(를) 추가했습니다.",
    "{user} has added the attachment {attachment} to card {card}" : "{user}가 {card} 카드에 첨부파일 {attachment}을(를) 추가했습니다.",
    "You have updated the attachment {attachment} on card {card}" : "{card} 카드의 첨부 파일 {attachment}을(를) 업데이트했습니다.",
    "{user} has updated the attachment {attachment} on card {card}" : "{user}가 {card} 카드의 첨부 파일 {attachment}을(를) 업데이트했습니다.",
    "You have deleted the attachment {attachment} from card {card}" : "{card} 카드에서 첨부 파일 {attachment}을(를) 삭제했습니다.",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user}가 {card} 카드에서 첨부 파일 {attachment}을(를) 삭제했습니다.",
    "You have restored the attachment {attachment} to card {card}" : "첨부 파일 {attachment}을(를) 카드 {card}에 복원했습니다.",
    "{user} has restored the attachment {attachment} to card {card}" : "{user}가 첨부 파일 {attachment}을(를) 카드 {card}에 복원했습니다.",
    "You have commented on card {card}" : "{card} 카드에 댓글을 작성하셨습니다.",
    "{user} has commented on card {card}" : "{user}가 {card} 카드에 댓글을 작성하였습니다.",
    "Deck" : "덱",
    "Changes in the <strong>Deck app</strong>" : "<strong>Deck 앱</strong>의 변경 사항",
    "A <strong>board, list or card</strong> was changed" : "<strong>보드, 목록 또는 카드</strong>가 변경되었습니다.",
    "A <strong>comment</strong> was created on a card" : "카드에 <strong>댓글</strong>이 작성되었습니다.",
    "A <strong>card description</strong> has been changed" : "<strong>카드 설명</strong>이 변경되었습니다.",
    "The file was uploaded" : "파일을 업로드함",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "업로드한 파일의 크기가 php.ini의 upload_max_filesize를 초과함",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "업로드한 파일의 크기가 HTML 폼에 지정한 MAX_FILE_SIZE를 초과함",
    "The file was only partially uploaded" : "파일이 일부분만 업로드됨",
    "No file was uploaded" : "업로드한 파일 없음",
    "Missing a temporary folder" : "임시 폴더 없음",
    "Could not write file to disk" : "디스크에 파일을 쓸 수 없음",
    "A PHP extension stopped the file upload" : "PHP 확장 기능에서 파일 업로드를 차단함",
    "No file uploaded or file size exceeds maximum of %s" : "파일이 업로드 되지 않았거나 크기가 %s(을)를 초과함",
    "Cards due today" : "오늘 만료인 카드",
    "Cards due tomorrow" : "명일 만료인 카드",
    "Upcoming cards" : "다음 카드들",
    "Load more" : "더 불러오기",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "\"%s\"의 \"%s\" 카드가 %s에 의해 당신에게 할당되었습니다.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user}님이 {deck-board}의 {deck-card} 카드를 당신에게 할당했습니다.",
    "The card {deck-card} on {deck-board} has reached its due date." : "{deck-board}의 {deck-card} 카드가 만기일에 도달했습니다.",
    "%s has mentioned you in a comment on \"%s\"." : "%s 님이 \"%s\"에 대한 댓글에서 당신을 언급했습니다.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user}님이 {deck-card}의 댓글에서 당신을 언급했습니다.",
    "The board \"%s\" has been shared with you by %s." : "\"%s\" 게시판을 %s 님이 공유했습니다.",
    "{user} has shared {deck-board} with you." : "{user}님이 귀하와 {deck-board}를 공유했습니다.",
    "Deck board" : "Deck 보드",
    "Owned by %1$s" : "%1$s이 소유함",
    "Deck boards, cards and comments" : "Deck 보드, 카드 및 댓글",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "%1$s에서, %2$s/%3$s, %4$s 소유",
    "Create a new deck card" : "새로운 덱 카드 만들기",
    "Card comments" : "카드 코멘트",
    "%s on %s" : "%s(%s의)",
    "Deck boards and cards" : "덱 보드 및 카드",
    "No data was provided to create an attachment." : "첨부할 데이터가 제공되지 않았습니다.",
    "Finished" : "완료됨",
    "To review" : "리뷰할 항목",
    "Action needed" : "동작 필요",
    "Later" : "나중에",
    "copy" : "복사",
    "To Do" : "할 일",
    "In Progress" : "진행 중",
    "Done" : "완료",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "이 댓글에는 %s자 이상이 있습니다.\n이름이 %s인 카드에 첨부 파일로 추가되었습니다.\nURL에서 액세스 가능: %s.",
    "Attachments" : "첨부파일",
    "File" : "파일",
    "Card not found" : "카드 없음",
    "Path is already shared with this card" : "경로가 이미 이 카드와 공유되어 있습니다.",
    "Invalid date, date format must be YYYY-MM-DD" : "잘못된 날짜, YYYY-MM-DD 형식이어야 합니다.",
    "Personal planning and team project organization" : "개인 기획 및 팀 프로젝트 구성",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck은 Nextcloud와 통합된 팀의 개인 계획 및 프로젝트 구성을 목표로 하는 칸반 스타일 구성 도구입니다.\n\n\n- 📥 카드에 작업을 추가하고 순서대로 정리하세요\n- 📄 Markdown에 추가 메모를 작성하세요.\n- 🔖 더 나은 정리를 위해 라벨을 할당하세요\n- 👥 팀, 친구, 가족과 공유하세요\n- 📎 파일을 첨부하고 Markdown 설명에 포함하세요.\n- 💬 댓글을 사용하여 팀과 토론하세요\n- ⚡ 활동 스트림의 변경 사항을 추적하세요.\n- 🚀 프로젝트를 체계적으로 정리하세요",
    "Add board" : "보드 추가",
    "Card details" : "카드 세부사항",
    "Select the board to link to a project" : "프로젝트에 연결할 보드를 선택하세요.",
    "Search by board title" : "보드 제목으로 검색",
    "Select board" : "보드 선택",
    "Select a board" : "보드 선택",
    "No lists available" : "사용 가능한 목록이 없음",
    "Select a list" : "목록 선택",
    "Move card" : "카드 이동",
    "Select the card to link to a project" : "카드를 선택해 프로젝트에 연결",
    "Link to card" : "카드 링크",
    "Select a card" : "카드 선택",
    "Cancel" : "취소",
    "This board is read only" : "읽기 전용 보드",
    "Drop your files to upload" : "업로드를 하려면 파일을 올리시오.",
    "File already exists" : "파일이 이미 존재함",
    "A file with the name {filename} already exists." : "이미 존재하는  파일이름 {filename} ",
    "Do you want to overwrite it?" : "덮어쓰시겠습니까?",
    "Overwrite file" : "덮어쓰기",
    "Keep existing file" : "기존 파일 보존",
    "Add card" : "카드 추가",
    "Archived cards" : "보관된 카드들",
    "Add list" : "목록 추가",
    "List name" : "리스트 추가",
    "Clear search" : "찾기 초기화",
    "Active filters" : "활성 필터",
    "Apply filter" : "필터링 적용",
    "Filter by tag" : "태그로 필터링",
    "Filter by assigned user" : "지정된 사용자로 필터링",
    "Unassigned" : "할당되지 않음",
    "Filter by status" : "상태로 필터링",
    "Open and completed" : "개봉 완료됨",
    "Open" : "열기",
    "Completed" : "완료됨",
    "Filter by due date" : "기한으로 필터링",
    "Overdue" : "기한 넘김",
    "Next 24 hours" : "다음 24시간",
    "Next 7 days" : "다음 7일 동안",
    "Next 30 days" : "다음 30일 동안",
    "No due date" : "기한 없음",
    "Clear filter" : "필터 지우기",
    "View Modes" : "보기 모드",
    "Toggle View Modes" : "보기 모드 전환",
    "Hide archived cards" : "보관된 카드 숨기기",
    "Show archived cards" : "보관된 카드 보기",
    "Toggle compact mode" : "컴팩트 모드 전환",
    "Hide card cover images" : "카드 표지 이미지 숨기기",
    "Show card cover images" : "카드 표지 이미지 표시",
    "Open details" : "자세한 정보 열기",
    "Details" : "세부사항",
    "Deck settings" : "Deck 설정",
    "General" : "일반",
    "Use bigger card view" : "카드 크게 보기 사용",
    "Appearance" : "외형",
    "Show card ID badge" : "카드 ID 배지 표시",
    "Show boards in calendar/tasks" : "캘린더/작업에 보드 표시",
    "Admin settings" : "관리자 설정",
    "Limit board creation to some groups" : "보드 생성을 일부 그룹으로 제한",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "해당 그룹 외부의 사용자는 자신의 보드를 만들 수 없지만 공유된 보드에서는 계속 작업할 수 있습니다.",
    "Board actions" : "보드 동작",
    "Scroll sideways" : "옆으로 스크롤",
    "Navigate between cards" : "카드 간 이동",
    "Close card details" : "카드 세부정보 닫기",
    "Search" : "검색",
    "Show card filters" : "카드 필터 표시",
    "Clear card filters" : "카드 필터 초기화",
    "Show those shortcuts" : "다음 단축키 표시",
    "Card actions" : "카드 동작",
    "Open card details" : "카드 세부정보 열기",
    "Edit the card title" : "카드 제목 수정",
    "Assign yourself to the current card" : "현재 카드에 자신을 할당",
    "Archive/unarchive the current card" : "현재 카드 보관/보관 취소",
    "Mark card as completed/not completed" : "카드를 완료됨/완료되지 않음으로 표시",
    "Open card menu" : "카드 메뉴 열기",
    "Currently present people" : "현재 존재하는 사람들",
    "Loading board" : "보드 불러오는 중",
    "Board not found" : "보드를 찾을 수 없음",
    "Create a new list to add cards to this board" : "이 보드에 카드를 추가하려면 새 목록을 만드세요.",
    "Sharing" : "공유",
    "Tags" : "태그",
    "Deleted items" : "삭제된 항목",
    "Activity" : "활동",
    "Deleted lists" : "삭제된 목록",
    "Undo" : "실행 취소",
    "Deleted cards" : "삭제된 카드",
    "Hours" : "시간",
    "Days" : "일",
    "Weeks" : "주",
    "Months" : "개월",
    "No cards yet" : "아직 카드가 없음",
    "Failed to create share with {displayName}" : "displayName}과(와) 공유를 생성하지 못했습니다.",
    "Are you sure you want to transfer the board {title} to {user}?" : "{title} 보드를 {user}에게 양도하시겠습니까?",
    "Transfer the board." : "보드를 옮깁니다.",
    "Transfer" : "전송",
    "The board has been transferred to {user}" : "보드가 {user}에게 이전되었습니다",
    "Failed to transfer the board to {user}" : "보드를 {user}에게 전송하지 못했습니다.",
    "Board owner" : "보드 소유자",
    "(Group)" : "(그룹)",
    "(Team)" : "(팀)",
    "(remote)" : "(원격)",
    "Can edit" : "편집 허용",
    "Can share" : "공유할 수 있음",
    "Can manage" : "관리 가능",
    "Owner" : "소유자",
    "Delete" : "삭제",
    "List deleted" : "목록 삭제됨",
    "Edit list title" : "목록 제목 수정",
    "Archive all cards" : "모든 카드 보관",
    "Unarchive all cards" : "모든 카드 보관 취소",
    "Delete list" : "목록 지우기",
    "Archive all cards in this list" : "이 목록의 모든 카드를 보관",
    "Unarchive all cards in this list" : "이 목록의 모든 카드를 보관 취소",
    "Add a new card" : "새로운 카드 추가",
    "Card name" : "카드 이름",
    "title and color value must be provided" : "제목과 색상 값을 제공해야 함",
    "Edit" : "편집",
    "Add a new tag" : "새로운 태그 추가",
    "Board name" : "보드 이름",
    "Members" : "구성원",
    "Assign to users/groups/team" : "사용자/그룹/팀에 할당",
    "Assign a user to this card…" : "이 카드에 사용자를 할당합니다...",
    "Select a user to assign to this card…" : "이 카드에 할당할 사용자를 선택하세요...",
    "File to share" : "공유할 파일",
    "Invalid path selected" : "잘못된 경로가 선택됨",
    "Upload new files" : "새로운 파일 업로드",
    "Share from Files" : "파일에서 공유",
    "Pending share" : "보류 중인 공유",
    "Add this attachment" : "해당 첨부 파일 추가",
    "Show in Files" : "파일로 표시",
    "Download" : "다운로드",
    "Remove attachment" : "첨부파일 제거",
    "Delete Attachment" : "첨부파일 삭제",
    "Restore Attachment" : "첨부파일 복원",
    "The title cannot be empty." : "제목은 공백일 수 없습니다.",
    "Open in sidebar view" : "사이드바 보기에서 열기",
    "Open in bigger view" : "더 크게 보기",
    "Comments" : "댓글",
    "Failed to load comments" : "댓글을 불러오지 못했습니다.",
    "No comments yet. Begin the discussion!" : "아직 댓글이 없습니다. 토론을 시작하세요!",
    "The comment cannot be empty." : "댓글은 공백일 수 없습니다.",
    "The comment cannot be longer than 1000 characters." : "설명은 1000자를 초과할 수 없습니다.",
    "Submit" : "제출",
    "Created:" : "생성한 날짜:",
    "In reply to" : "답장",
    "Cancel reply" : "응답 취소",
    "Reply" : "답장",
    "Update" : "업데이트",
    "Mark as done" : "완료된 것으로 표시",
    "Mark as not done" : "완료되지 않은 것으로 표시",
    "Write a description …" : "설명 작성",
    "Description" : "설명",
    "(Unsaved)" : "(저장되지 않음)",
    "(Saving…)" : "(저장 중…)",
    "Formatting help" : "서식 도움말",
    "Edit description" : "설명 편집",
    "View description" : "설명 보기",
    "Add Attachment" : "첨부파일 추가",
    "Choose attachment" : "첨부파일 선택",
    "Select Date" : "날짜 선택",
    "Later today – {timeLocale}" : "오늘 나중 – {timeLocale}",
    "Set due date for later today" : "마감일을 오늘 이후로 설정",
    "Tomorrow – {timeLocale}" : "내일 – {timeLocale}",
    "Set due date for tomorrow" : "마감일을 내일로 설정",
    "This weekend – {timeLocale}" : "이번 주말 – {timeLocale}",
    "Set due date for this weekend" : "마감일을 이번 주말로 설정",
    "Next week – {timeLocale}" : "다음주 – {timeLocale}",
    "Set due date for next week" : "마감일을 다음 주로 설정",
    "Assign a due date to this card…" : "이 카드에 만료일을 할당합니다...",
    "Set a due date" : "만료일 설정",
    "Add due date" : "만료일 추가",
    "Choose a date" : "날짜 선택",
    "Remove due date" : "만료 날짜 삭제",
    "Due at:" : "마감일:",
    "Not done" : "완료되지 않음",
    "Unarchive card" : "카드 보관 취소",
    "Archive card" : "보관 카드",
    "Assign a tag to this card…" : "이 카드에 태그를 할당합니다...",
    "Select or create a tag…" : "태그를 선택하거나 생성하기...",
    "Create a new tag:" : "새로운 태그 생성:",
    "(group)" : "(그룹)",
    "{count} comments, {unread} unread" : "댓글 {count}개, 읽지 않음 {unread}개",
    "Todo items" : "작업 항목",
    "Edit card title" : "카드 제목 수정",
    "Card deleted" : "카드가 삭제됨",
    "Edit title" : "제목 수정",
    "Assign to me" : "나에게 할당",
    "Unassign myself" : "스스로를 할당 해제",
    "Delete card" : "카드 삭제",
    "All boards" : "모든 보드",
    "Archived boards" : "보관된 게시판",
    "Shared with you" : "나와 공유됨",
    "Cancel edit" : "편집 취소",
    "Board {0} deleted" : "보드 {0}개가 삭제되었습니다.",
    "All cards" : "모든 카드",
    "Only assigned cards" : "할당된 카드만",
    "No reminder" : "알림 없음",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "{title} 보드를 정말로 삭제하시겠습니까? 보관된 카드를 포함하여 이 보드의 모든 데이터가 삭제됩니다.",
    "Delete the board?" : "보드를 삭제합니까?",
    "An error occurred" : "오류가 발생함",
    "Board details" : "게시판 정보",
    "Edit board" : "게시판 편집",
    "Clone board" : "게시판 복제",
    "Unarchive board" : "게시판 보관 해제",
    "Archive board" : "게시판 보관",
    "Export board" : "보드 내보내기",
    "Turn on due date reminders" : "마감일 알림 켜기",
    "Turn off due date reminders" : "마감일 알림 끄기",
    "Due date reminders" : "마감일 알림",
    "Assigned cards" : "할당된 카드",
    "No notifications" : "알림 없음",
    "Delete board" : "게시판 삭제",
    "Clone cards" : "복제 카드",
    "Advanced options" : "고급 설정",
    "Clone" : "복제",
    "Export as CSV" : "CSV로 내보내기",
    "Export" : "내보내기",
    "Loading filtered view" : "필터링된 보기 불러오는 중",
    "Search for {searchQuery} in all boards" : "모든 게시판에서 {searchQuery}를 검색하세요",
    "No results found" : "결과 없음",
    "Deck board {name}\n* Last modified on {lastMod}" : "데크 보드 {name}\n* 마지막 수정일 {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* 생성일: {created}\n* 마지막 수정일: {lastMod}\n* 첨부 파일 {nbAttachments}개\n* {nbComments} 댓글",
    "{nbCards} cards" : "{nbCards} 카드",
    "Due on {date}" : "마감일: {date}",
    "{stack} in {board}" : "{board}의 {stack}",
    "Click to expand description" : "설명을 더 보려면 클릭하세요.",
    "Click to expand comment" : "댓글을 더 보려면 클릭하세요.",
    "Create card" : "크레딧 카드",
    "Create a new card" : "새로운 카드 생성",
    "Card title" : "카드 제목",
    "Creating the new card …" : "새로운 카드 생성 중...",
    "Card \"{card}\" was added to \"{board}\"" : "\"{card}\" 카드가 \"{board}\"에 추가되었습니다.",
    "Open card" : "카드 열기",
    "Close" : "닫기",
    "No upcoming cards" : "예정된 카드 없음",
    "upcoming cards today" : "오늘 예정인 카드",
    "upcoming cards tomorrow" : "내일 예정인 카드",
    "upcoming cards" : "곧 있을 카드",
    "New card" : "새로운 카드",
    "Link to a board" : "보드에 연결",
    "Link to a card" : "카드에 연결",
    "Create a card" : "카드 생성",
    "Message from {author} in {conversationName}" : "{conversationName}에서 {author}가 보낸 메시지",
    "Something went wrong" : "잘못된 접근",
    "Failed to upload {name}" : "{name}을(를) 업로드하지 못했습니다.",
    "Maximum file size of {size} exceeded" : "최대 파일 크기인 {size}을(를) 초과했습니다.",
    "Due date" : "만료 날짜",
    "Created" : "생성됨",
    "Modified" : "수정한 날짜",
    "Error creating the share" : "공유를 만드는 중 오류 발생",
    "Share with a Deck card" : "Deck 카드와 공유",
    "Share {file} with a Deck card" : "Deck 카드와 함께 {file} 공유",
    "Share" : "공유",
    "The card \"%s\" on \"%s\" has reached its due date." : "카드 \"%s\"(\"%s\"에 있음)의 만료 날짜가 다가왔습니다.",
    "Share board with a user, group or team …" : "사용자, 그룹 또는 팀과 보드 공유…",
    "Searching for users, groups and teams …" : "사용자, 그룹, 팀 검색 중…",
    "No participants found" : "참가자를 찾을 수 없음",
    "Save" : "저장",
    "seconds ago" : "초 전",
    "Today" : "오늘",
    "Tomorrow" : "내일",
    "No due" : "기한 없음"
},
"nplurals=1; plural=0;");
