OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Has creado un nuevo tablero {board}",
    "{user} has created a new board {board}" : "{user} ha creado un nuevo tablero {board}",
    "You have deleted the board {board}" : "Has eliminado el tablero {board}",
    "{user} has deleted the board {board}" : "{user} ha eliminado el tablero {board}",
    "You have restored the board {board}" : "Has restaurado el tablero {board}",
    "{user} has restored the board {board}" : "{user} ha restaurado el tablero {board}",
    "You have shared the board {board} with {acl}" : "Has compartido el tablero {board} con {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} ha compartido el tablero {board} con {acl}",
    "You have removed {acl} from the board {board}" : "Has eliminado {acl} del tablero {board}",
    "{user} has removed {acl} from the board {board}" : "{user} ha eliminado {acl} del tablero {board}",
    "You have renamed the board {before} to {board}" : "Has cambiado el nombre del tablero {before} a {board}",
    "{user} has renamed the board {before} to {board}" : "{user} ha cambiado el nombre del tablero {before} a {board}",
    "You have archived the board {board}" : "Has archivado el tablero {board}",
    "{user} has archived the board {before}" : "{user} ha archivado el tablero {before}",
    "You have unarchived the board {board}" : "Has desarchivado el tablero {board}",
    "{user} has unarchived the board {before}" : "{user} ha desarchivado el tablero {before}",
    "You have created a new list {stack} on board {board}" : "Has creado una nueva lista {stack} en el tablero {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} ha creado una nueva lista {stack} en el tablero {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Has cambiado el nombre de la lista {before} a {stack} en el tablero {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} ha cambiado el nombre de la lista {before} a {stack} en el tablero {board}",
    "You have deleted list {stack} on board {board}" : "Has eliminado la lista {stack} en el tablero {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} ha eliminado la lista {stack} en el tablero {board}",
    "You have created card {card} in list {stack} on board {board}" : "Has creado la tarjeta {card} en la lista {stack} en el tablero {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} ha creado la tarjeta {card} en la lista {stack} en el tablero {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Has eliminado la tarjeta {card} en la lista {stack} en el tablero {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} ha eliminado la tarjeta {card} en la lista {stack} en el tablero {board}",
    "You have renamed the card {before} to {card}" : "Has cambiado el nombre de la tarjeta {before} a {card}",
    "{user} has renamed the card {before} to {card}" : "{user} ha cambiado el nombre de la tarjeta {before} a {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Has añadido una descripción a la tarjeta {card} en la lista {stack} en el tablero {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} ha añadido una descripción a la tarjeta {card} en la lista {stack} en el tablero {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Has actualizado la descripción de la tarjeta {card} en la lista {stack} en el tablero {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} ha actualizado la descripción de la tarjeta {card} en la lista {stack} en el tablero {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Has archivado la tarjeta {card} en la lista {stack} en el tablero {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} ha archivado la tarjeta {card} en la lista {stack} en el tablero {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Has desarchivado la tarjeta {card} en la lista {stack} en el tablero {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} ha desarchivado la tarjeta {card} en la lista {stack} en el tablero {board}",
    "You have removed the due date of card {card}" : "Has eliminado la fecha de vencimiento de la tarjeta {card}",
    "{user} has removed the due date of card {card}" : "{user} ha eliminado la fecha de vencimiento de la tarjeta {card}",
    "You have set the due date of card {card} to {after}" : "Has establecido la fecha de vencimiento de la tarjeta {card} a {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} ha establecido la fecha de vencimiento de la tarjeta {card} a {after}",
    "You have updated the due date of card {card} to {after}" : "Has actualizado la fecha de vencimiento de la tarjeta {card} a {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} ha actualizado la fecha de vencimiento de la tarjeta {card} a {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Has añadido la etiqueta {label} a la tarjeta {card} en la lista {stack} en el tablero {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} ha añadido la etiqueta {label} a la tarjeta {card} en la lista {stack} en el tablero {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Has eliminado la etiqueta {label} de la tarjeta {card} en la lista {stack} en el tablero {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} ha eliminado la etiqueta {label} de la tarjeta {card} en la lista {stack} en el tablero {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Has asignado a {assigneduser} la tarjeta {card} en el tablero {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} ha asignado a {assigneduser} la tarjeta {card} en el tablero {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Has quitado la asignación a {assigneduser} de la tarjeta {card} en el tablero {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} ha quitado la asignación a {assigneduser} de la tarjeta {card} en el tablero {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Has movido la tarjeta {card} de la lista {stackBefore} a {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} ha movido la tarjeta {card} de la lista {stackBefore} a {stack}",
    "You have added the attachment {attachment} to card {card}" : "Has añadido el archivo {attachment} a la tarjeta {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} ha añadido el archivo {attachment} a la tarjeta {card}",
    "You have updated the attachment {attachment} on card {card}" : "Has actualizado el archivo {attachment} en la tarjeta {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} ha actualizado el archivo {attachment} en la tarjeta {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Has eliminado el archivo {attachment} de la tarjeta {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} ha eliminado el archivo {attachment} de la tarjeta {card}",
    "You have restored the attachment {attachment} to card {card}" : "Has restaurado el archivo {attachment} a la tarjeta {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} ha restaurado el archivo {attachment} a la tarjeta {card}",
    "You have commented on card {card}" : "Has comentado en la tarjeta {card}",
    "{user} has commented on card {card}" : "{user} ha comentado en la tarjeta {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Cambios en la aplicación <strong>Deck</strong>",
    "A <strong>comment</strong> was created on a card" : "Se ha creado un <strong>comentario</strong> en una tarjeta",
    "The file was uploaded" : "El archivo fue cargado",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "El archivo cargado excede el valor establecido en la directiva upload_max_filesize en el archivo php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "El archivo cargado excede el valor especificado de la directiva MAX_FILE_SIZE en la forma de HTML",
    "The file was only partially uploaded" : "El archivo sólo fue cargado parcialmente",
    "No file was uploaded" : "No se cargó el archivo",
    "Missing a temporary folder" : "Falta una carpeta temporal",
    "Could not write file to disk" : "No fue posible escribir a disco",
    "A PHP extension stopped the file upload" : "Una extensión de PHP detuvo la carga del archivo",
    "No file uploaded or file size exceeds maximum of %s" : "No se ha subido ningún archivo o el tamaño del archivo supera el máximo de %s",
    "Upcoming cards" : "Tarjetas próximas",
    "Load more" : "Cargar más",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "La tarjeta \"%s\" en \"%s\" te ha sido asignada por %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} te ha asignado la tarjeta {deck-card} en {deck-board}.",
    "The card {deck-card} on {deck-board} has reached its due date." : "La tarjeta {deck-card} en {deck-board} ha alcanzado su fecha de vencimiento.",
    "%s has mentioned you in a comment on \"%s\"." : "%s te ha mencionado en un comentario en \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} te ha mencionado en un comentario en {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "El tablero \"%s\" ha sido compartido contigo por %s.",
    "{user} has shared {deck-board} with you." : "{user} ha compartido contigo {deck-board}.",
    "Deck board" : "Tablero de Deck",
    "Owned by %1$s" : "Perteneciente a %1$s",
    "Deck boards, cards and comments" : "Tableros de Deck, tarjetas y comentarios",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "De %1$s, en %2$s/%3$s, perteneciente a %4$s",
    "Card comments" : "Comentarios de tarjetas",
    "%s on %s" : "%s en %s",
    "Deck boards and cards" : "Tableros y tarjetas de Deck",
    "No data was provided to create an attachment." : "No se proporcionaron datos para crear un archivo adjunto.",
    "Finished" : "Terminado",
    "To review" : "Para revisar",
    "Action needed" : "Acción requerida",
    "Later" : "Después",
    "copy" : "copia",
    "To Do" : "Por hacer",
    "Done" : "Terminado",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Este comentario tiene más de %s caracteres.\n      Añadido como un archivo adjunto a la tarjeta con el nombre %s.\n      Accesible en la URL: %s.",
    "Attachments" : "Adjuntos",
    "File" : "Archivo",
    "Card not found" : "No se encontró la tarjeta",
    "Path is already shared with this card" : "La ruta ya está compartida con esta tarjeta",
    "Invalid date, date format must be YYYY-MM-DD" : "La fecha es inválida, por favor sigue el formato AAAA-MM-DD",
    "Personal planning and team project organization" : "Planificación personal y organización de proyectos en equipo",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck es una herramienta de organización estilo Kanban destinada a la planificación personal y la organización de proyectos en equipo integrada con Nextcloud.\n      \n      \n      - 📥 Añade tus tareas a tarjetas y ordénalas\n      - 📄 Toma notas adicionales en Markdown\n      - 🔖 Asigna etiquetas para una mejor organización\n      - 👥 Comparte con tu equipo, amigos o familia\n      - 📎 Adjunta archivos e incrustarlos en tu descripción en Markdown\n      - 💬 Discute con tu equipo usando comentarios\n      - ⚡ Sigue los cambios en el flujo de actividad\n      - 🚀 Organiza tu proyecto",
    "Add board" : "Agregar tablero",
    "Card details" : "Detalles de la tarjeta",
    "Select the board to link to a project" : "Selecciona el tablero para vincularlo a un proyecto",
    "Search by board title" : "Buscar por título del tablero",
    "Select board" : "Seleccionar tablero",
    "Select a board" : "Seleccionar un tablero",
    "No lists available" : "No hay listas disponibles",
    "Select a list" : "Seleccionar una lista",
    "Move card" : "Mover tarjeta",
    "Select the card to link to a project" : "Selecciona la tarjeta para vincularla a un proyecto",
    "Link to card" : "Vincular con tarjeta",
    "Select a card" : "Seleccionar una tarjeta",
    "Cancel" : "Cancelar",
    "This board is read only" : "Este tablero es de solo lectura",
    "Drop your files to upload" : "Suelta tus archivos para subirlos",
    "File already exists" : "El archivo ya existe",
    "A file with the name {filename} already exists." : "Ya existe un archivo con el nombre {filename}.",
    "Do you want to overwrite it?" : "¿Deseas sobre escribirlo?",
    "Overwrite file" : "Sobrescribir archivo",
    "Keep existing file" : "Mantener el archivo existente",
    "Add card" : "Agregar tarjeta",
    "Archived cards" : "Tarjetas archivadas",
    "Add list" : "Agregar lista",
    "List name" : "Nombre de la lista",
    "Active filters" : "Filtros activos",
    "Apply filter" : "Aplicar filtro",
    "Filter by tag" : "Filtrar por etiqueta",
    "Filter by assigned user" : "Filtrar por usuario asignado",
    "Unassigned" : "No asignado",
    "Open" : "Abrir",
    "Completed" : "Completado",
    "Filter by due date" : "Filtrar por fecha de vencimiento",
    "Overdue" : "Atrasado",
    "Next 24 hours" : "Próximas 24 horas",
    "Next 7 days" : "Próximos 7 días",
    "Next 30 days" : "Próximos 30 días",
    "No due date" : "Sin fecha de vencimiento",
    "Clear filter" : "Borrar filtro",
    "Hide archived cards" : "Ocultar tarjetas archivadas",
    "Show archived cards" : "Mostrar tarjetas archivadas",
    "Toggle compact mode" : "Activar modo compacto",
    "Open details" : "Abrir detalles.",
    "Details" : "Detalles",
    "Deck settings" : "Configuración de Deck",
    "General" : "General",
    "Use bigger card view" : "Usar vista de tarjeta más grande",
    "Appearance" : "Apariencia",
    "Show card ID badge" : "Mostrar identificación de la tarjeta",
    "Show boards in calendar/tasks" : "Mostrar tableros en calendario/tareas",
    "Search" : "Buscar",
    "Currently present people" : "Personas actualmente presentes",
    "Loading board" : "Cargando tablero",
    "Board not found" : "No se encontró el tablero",
    "Create a new list to add cards to this board" : "Crea una nueva lista para añadir tarjetas a este tablero",
    "Sharing" : "Compartiendo",
    "Tags" : "Etiquetas",
    "Deleted items" : "Elementos eliminados",
    "Activity" : "Actividad",
    "Deleted lists" : "Listas eliminadas",
    "Undo" : "Deshacer",
    "Deleted cards" : "Tarjetas eliminadas",
    "Hours" : "Horas",
    "Days" : "Días",
    "Weeks" : "Semanas",
    "Months" : "Meses",
    "No cards yet" : "Aún no hay tarjetas",
    "Failed to create share with {displayName}" : "Error al crear el uso compartido con {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "¿Estás seguro de que quieres transferir el tablero {title} a {user}?",
    "Transfer the board." : "Transferir el tablero.",
    "Transfer" : "Transferencia",
    "The board has been transferred to {user}" : "El tablero se ha transferido a {user}",
    "Failed to transfer the board to {user}" : "Error al transferir el tablero a {user}",
    "Board owner" : "Propietario del tablero",
    "(Group)" : "(Grupo)",
    "(remote)" : "(remoto)",
    "Can edit" : "Puede editar",
    "Can share" : "Puede compartir",
    "Can manage" : "Puede gestionar",
    "Owner" : "Dueño",
    "Delete" : "Borrar",
    "List deleted" : "Lista eliminada",
    "Edit list title" : "Editar título de la lista",
    "Archive all cards" : "Archivar todas las tarjetas",
    "Unarchive all cards" : "Desarchivar todas las tarjetas",
    "Delete list" : "Eliminar lista",
    "Archive all cards in this list" : "Archivar todas las tarjetas en esta lista",
    "Unarchive all cards in this list" : "Desarchivar todas las tarjetas en esta lista",
    "Add a new card" : "Añadir una nueva tarjeta",
    "Card name" : "Nombre de la tarjeta",
    "title and color value must be provided" : "Debe proporcionarse el título y el valor del color",
    "Edit" : "Editar",
    "Add a new tag" : "Añadir una nueva etiqueta",
    "Board name" : "Nombre del tablero",
    "Members" : "Miembros",
    "Assign a user to this card…" : "Asignar un usuario a esta tarjeta…",
    "File to share" : "Archivo para compartir",
    "Invalid path selected" : "Ruta seleccionada no válida.",
    "Upload new files" : "Subir nuevos archivos",
    "Share from Files" : "Compartir desde Archivos",
    "Pending share" : "Uso compartido pendiente",
    "Add this attachment" : "Añadir este archivo adjunto",
    "Show in Files" : "Mostrar en Archivos",
    "Download" : "Descargar",
    "Remove attachment" : "Eliminar archivo adjunto",
    "Delete Attachment" : "Eliminar adjunto",
    "Restore Attachment" : "Restaurar adjunto",
    "The title cannot be empty." : "El título no puede estar vacío.",
    "Open in sidebar view" : "Abrir en vista de barra lateral",
    "Open in bigger view" : "Abrir en vista más grande",
    "Comments" : "Comentarios",
    "Failed to load comments" : "Error al cargar los comentarios",
    "No comments yet. Begin the discussion!" : "Aún no hay comentarios. ¡Inicia la discusión!",
    "The comment cannot be empty." : "El comentario no puede estar vacío.",
    "The comment cannot be longer than 1000 characters." : "El comentario no puede tener más de 1000 caracteres.",
    "Submit" : "Enviar",
    "Created:" : "Creado:",
    "In reply to" : "En respuesta a",
    "Cancel reply" : "Cancelar respuesta",
    "Reply" : "Responder",
    "Update" : "Actualizar",
    "Mark as done" : "Marcar como hecha",
    "Write a description …" : "Escribir una descripción …",
    "Description" : "Descripción",
    "(Unsaved)" : "(No guardado)",
    "(Saving…)" : "(Guardando…)",
    "Formatting help" : "Ayuda de formato",
    "Edit description" : "Editar descripción",
    "View description" : "Ver descripción",
    "Add Attachment" : "Añadir adjunto",
    "Choose attachment" : "Elegir adjunto",
    "Select Date" : "Seleccionar fecha",
    "Set a due date" : "Establecer una fecha de vencimiento",
    "Remove due date" : "Eliminar fecha de expiración",
    "Unarchive card" : "Desarchivar tarjeta",
    "Archive card" : "Archivar tarjeta",
    "Assign a tag to this card…" : "Asignar una etiqueta a esta tarjeta…",
    "(group)" : "(grupo)",
    "{count} comments, {unread} unread" : "{count} comentarios, {unread} sin leer",
    "Todo items" : "Elementos pendientes",
    "Edit card title" : "Editar título de la tarjeta",
    "Card deleted" : "Tarjeta eliminada",
    "Edit title" : "Editar título",
    "Assign to me" : "Asignarme la tarjeta",
    "Unassign myself" : "Desasignarme la tarjeta",
    "Delete card" : "Eliminar tarjeta",
    "All boards" : "Todos los tableros",
    "Archived boards" : "Tableros archivados",
    "Shared with you" : "Compartido con usted",
    "Cancel edit" : "Cancelar edición",
    "Board {0} deleted" : "Tablero {0} eliminado",
    "All cards" : "Todas las tarjetas",
    "Only assigned cards" : "Sólo tarjetas asignadas",
    "No reminder" : "Sin recordatorio",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "¿Estás seguro de que quieres eliminar el tablero {title}? Esto eliminará todos los datos de este tablero, incluidas las tarjetas archivadas.",
    "Delete the board?" : "¿Eliminar el tablero?",
    "An error occurred" : "Se produjo un error",
    "Board details" : "Detalles del tablero",
    "Edit board" : "Editar el tablero",
    "Clone board" : "Clonar tablero",
    "Unarchive board" : "Desarchivar tablero",
    "Archive board" : "Archivar tablero",
    "Export board" : "Exportar tablero",
    "Turn on due date reminders" : "Activar recordatorios de fecha de vencimiento",
    "Turn off due date reminders" : "Desactivar recordatorios de fecha de vencimiento",
    "Due date reminders" : "Recordatorios de fecha de vencimiento",
    "Assigned cards" : "Tarjetas asignadas",
    "No notifications" : "No hay notificaciones",
    "Delete board" : "Borrar tableros",
    "Clone cards" : "Clonar tarjetas",
    "Advanced options" : "Opciones avanzadas",
    "Clone" : "Clonar",
    "Export as CSV" : "Exportar como CSV",
    "Export" : "Exportar",
    "Loading filtered view" : "Cargando vista filtrada",
    "Search for {searchQuery} in all boards" : "Buscar {searchQuery} en todos los tableros",
    "No results found" : "No se encontraron resultados",
    "Deck board {name}\n* Last modified on {lastMod}" : "Tablero de Deck {name}\n      * Última modificación el {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Creado el {created}\n      * Última modificación el {lastMod}\n      * {nbAttachments} adjuntos\n      * {nbComments} comentarios",
    "{nbCards} cards" : "{nbCards} tarjetas",
    "Due on {date}" : "Vence el {date}",
    "{stack} in {board}" : "{stack} en {board}",
    "Click to expand description" : "Haz clic para expandir la descripción",
    "Click to expand comment" : "Haz clic para expandir el comentario",
    "Create card" : "Crear tarjeta",
    "Create a new card" : "Crear una nueva tarjeta",
    "Card title" : "Título de la tarjeta",
    "Creating the new card …" : "Creando la nueva tarjeta …",
    "Card \"{card}\" was added to \"{board}\"" : "La tarjeta \"{card}\" se ha añadido a \"{board}\"",
    "Open card" : "Abrir tarjeta",
    "Close" : "Cerrar",
    "No upcoming cards" : "No hay tarjetas próximas",
    "upcoming cards" : "tarjetas próximas",
    "New card" : "Nueva tarjeta",
    "Link to a board" : "Vincular a un tablero",
    "Link to a card" : "Vincular a una tarjeta",
    "Create a card" : "Crear una tarjeta",
    "Message from {author} in {conversationName}" : "Mensaje de {author} en {conversationName}",
    "Something went wrong" : "Algo salió mal",
    "Failed to upload {name}" : "Error al subir {name}",
    "Maximum file size of {size} exceeded" : "Tamaño máximo de archivo de {size} excedido",
    "Due date" : "Fecha de vencimiento",
    "Created" : "Creado",
    "Modified" : "Modificado",
    "Error creating the share" : "Error al crear la compartición",
    "Share with a Deck card" : "Compartir con una tarjeta de Deck",
    "Share {file} with a Deck card" : "Compartir {file} con una tarjeta de Deck",
    "Share" : "Compartir",
    "The card \"%s\" on \"%s\" has reached its due date." : "La tarjeta \"%s\" en \"%s\" ha alacanzado su fecha de entrega",
    "No participants found" : "No se encontraron participantes",
    "Save" : "Guardar",
    "seconds ago" : "hace segundos",
    "Today" : "Hoy",
    "Tomorrow" : "Mañana",
    "No due" : "Sin vencimiento"
},
"nplurals=3; plural=n == 1 ? 0 : n != 0 && n % 1000000 == 0 ? 1 : 2;");
