OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Tá bord {board} nua cruthaithe agat",
    "{user} has created a new board {board}" : "Tá bord {board} nua cruthaithe ag {user}",
    "You have deleted the board {board}" : "Tá an bord {board} scriosta agat",
    "{user} has deleted the board {board}" : "Tá an bord {board} scriosta ag {user}",
    "You have restored the board {board}" : "Tá an bord {board} aischurtha agat",
    "{user} has restored the board {board}" : "Tá an bord {board} athchóirithe ag {user}",
    "You have shared the board {board} with {acl}" : "Tá an bord {board} roinnte agat le {acl}",
    "{user} has shared the board {board} with {acl}" : "Roinn {user} an bord {board} le {acl}",
    "You have removed {acl} from the board {board}" : "Bhain tú {acl} den chlár {board}",
    "{user} has removed {acl} from the board {board}" : "Bhain {user} {acl} den chlár {board}",
    "You have renamed the board {before} to {board}" : "D'athainmnigh tú an bord {before} go {board}",
    "{user} has renamed the board {before} to {board}" : "D'athainmnigh {user} an bord {before} go {board}",
    "You have archived the board {board}" : "Tá an bord {board} curtha i gcartlann agat",
    "{user} has archived the board {before}" : "Chuir {user} an clár i gcartlann {roimh}",
    "You have unarchived the board {board}" : "Tá an bord {board} díchartlannaithe agat",
    "{user} has unarchived the board {before}" : "Rinne {user} an clár a dhíchartlannú {before}",
    "You have created a new list {stack} on board {board}" : "Tá liosta {stack} nua cruthaithe agat ar bord {board}",
    "{user} has created a new list {stack} on board {board}" : "Chruthaigh {user} liosta {stack} nua ar bord {board}",
    "You have renamed list {before} to {stack} on board {board}" : "D'athainmnigh tú an liosta {before} mar {stack} ar bord {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "D'athainmnigh {user} an liosta {before} go {stack} ar bord {board}",
    "You have deleted list {stack} on board {board}" : "Tá an liosta {stack} scriosta agat ar bord {board}",
    "{user} has deleted list {stack} on board {board}" : "Scrios {user} an liosta {stack} ar bord {board}",
    "You have created card {card} in list {stack} on board {board}" : "Chruthaigh tú cárta {card} sa liosta {stack} ar bord {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "Chruthaigh {user} cárta {card} sa liosta {stack} ar bord {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Tá cárta {card} scriosta agat sa liosta {stack} ar bord {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "scrios {user} cárta {card} sa liosta {stack} ar bord {board}",
    "You have renamed the card {before} to {card}" : "D'athainmnigh tú an cárta {before} go {card}",
    "{user} has renamed the card {before} to {card}" : "D'athainmnigh {user} an cárta {before} go {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Chuir tú cur síos le cárta {card} sa liosta {stack} ar bord {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "Chuir {user} cur síos le cárta {card} sa liosta {stack} ar bord {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Tá an cur síos ar chárta {card} sa liosta {stack} ar bord {board} nuashonraithe agat",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "Tá an cur síos ar an gcárta {card} sa liosta {stack} ar bord {board} nuashonraithe ag {user}",
    "You have archived card {card} in list {stack} on board {board}" : "Tá cárta {card} sa chartlann agat sa liosta {stack} ar bord {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "Chuir {user} cárta {card} sa liosta {stack} ar bord {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Tá cárta {card} dí-cartlannaithe agat sa liosta {stack} ar bord {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "Tá {card} neamhchartlannaithe ag {user} sa liosta {stack} ar bord {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Tá an cárta {card} marcáilte agat mar a rinneadh sa liosta {stack} ar bord {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "Mharcáil {user} cárta {card} mar a rinneadh sa liosta {stack} ar bord {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Tá tú tar éis an cárta {card} a mharcáil mar rud cealaithe sa liosta {stack} ar bord {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "Mharcáil {user} an cárta {card} mar rud cealaithe sa liosta {stack} ar bord {board}",
    "You have removed the due date of card {card}" : "Bhain tú dáta dlite an chárta {card}",
    "{user} has removed the due date of card {card}" : "Bhain {user} dáta dlite an chárta {card}",
    "You have set the due date of card {card} to {after}" : "Tá dáta dlite an chárta {card} socraithe agat go dtí {after}",
    "{user} has set the due date of card {card} to {after}" : "Tá dáta dlite an chárta {card} socraithe ag {user} go dtí {after}",
    "You have updated the due date of card {card} to {after}" : "Tá dáta dlite cárta {card} nuashonraithe agat go dtí {after}",
    "{user} has updated the due date of card {card} to {after}" : "Tá dáta dlite an chárta {card} nuashonraithe ag {user} go dtí {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Chuir tú an chlib {label} le cárta {card} sa liosta {stack} ar bord {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "Chuir {user} an chlib {label} le cárta {card} sa liosta {stack} ar bord {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Bhain tú an chlib {label} de chárta {card} sa liosta {stack} ar bord {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "Bhain {user} an chlib {label} de chárta {card} sa liosta {stack} ar bord {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Shann tú {assigneduser} do chárta {card} ar bord {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "Tá {user} sannta {assigneduser} do chárta {card} ar bord {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Tá {assigneduser} díshannta agat ó chárta {card} ar bord {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "Tá {user} díshannaithe {user} ó chárta {card} ar bord {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Bhog tú an cárta {card} ón liosta {stackBefore} go {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "Bhog {user} an cárta {card} ón liosta {stackBefore} go {stack}",
    "You have added the attachment {attachment} to card {card}" : "Chuir tú an ceangaltán {attachment} le cárta {card}",
    "{user} has added the attachment {attachment} to card {card}" : "Chuir {user} an ceangaltán {attachment} le cárta {card}",
    "You have updated the attachment {attachment} on card {card}" : "Tá an ceangaltán {attachment} ar chárta {card} nuashonraithe agat",
    "{user} has updated the attachment {attachment} on card {card}" : "Tá an ceangaltán {attachment} ar chárta {card} nuashonraithe ag {user}",
    "You have deleted the attachment {attachment} from card {card}" : "Scrios tú an ceangaltán {attachment} de chárta {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "Scrios {user} an ceangaltán {attachment} ó chárta {card}",
    "You have restored the attachment {attachment} to card {card}" : "Tá an ceangaltán {attachment} aischurtha go cárta {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "Tá {user} tar éis an ceangaltán {attachment} a aischur chuig cárta {card}",
    "You have commented on card {card}" : "Rinne tú nóta tráchta ar chárta {card}",
    "{user} has commented on card {card}" : "Rinne {user} nóta tráchta ar chárta {card}",
    "Deck" : "Deic",
    "Changes in the <strong>Deck app</strong>" : "Athruithe san aip 1Deck1",
    "A <strong>board, list or card</strong> was changed" : "Athraíodh <strong>chlár, liosta nó cárta</strong> ",
    "A <strong>comment</strong> was created on a card" : "Cruthaíodh <strong>trácht</strong> ar chárta",
    "A <strong>card description</strong> has been changed" : "Athraíodh <strong>cur síos chárta</strong> ",
    "The file was uploaded" : "Uaslódáladh an comhad",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Sáraíonn an comhad uaslódáilte an treoir upload_max_filesize i php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Sáraíonn an comhad uaslódáilte an treoir MAX_FILE_SIZE a sonraíodh san fhoirm HTML",
    "The file was only partially uploaded" : "Níor uaslódáladh an comhad ach go páirteach",
    "No file was uploaded" : "Níor uaslódáladh aon chomhad",
    "Missing a temporary folder" : "Fillteán sealadach ar iarraidh",
    "Could not write file to disk" : "Níorbh fhéidir an comhad a scríobh ar an diosca",
    "A PHP extension stopped the file upload" : "Chuir síneadh PHP stop le huaslódáil an chomhaid",
    "No file uploaded or file size exceeds maximum of %s" : "Níl aon chomhad uaslódáilte nó méid comhaid níos mó ná uasmhéid%s",
    "Invalid file type. Only JSON files are allowed." : "Cineál comhaid neamhbhailí. Ní cheadaítear ach comhaid JSON.",
    "Invalid JSON data" : "Sonraí JSON neamhbhailí",
    "Failed to import board" : "Theip ar an mbord a allmhairiú",
    "Cards due today" : "Cártaí dlite inniu",
    "Cards due tomorrow" : "Cártaí dlite amárach",
    "Upcoming cards" : "Cártaí atá le teacht",
    "Load more" : "Luchtaigh níos mó",
    "Welcome to Nextcloud Deck!" : "Fáilte go dtí Deic Nextcloud!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Tá an cárta \"%s\" ar \"%s\" tugtha duit ag %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "Shann {user} an cárta {deck-card} ar {deck-board} duit.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Tá an cárta \"%1$s\" ar \"%2$s\" sroichte a dháta dlite.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Tá an dáta dlite sroichte ag an gcárta {deck-card} ar {deck-board}.",
    "%s has mentioned you in a comment on \"%s\"." : "Luaigh %s tú i nóta tráchta ar \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "Luaigh {user} thú i nóta tráchta ar {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Tá an clár \"%s\" roinnte leat ag %s.",
    "{user} has shared {deck-board} with you." : "Roinn {user} {deck-board} leat.",
    "The remote board %1$s has been shared with you by %2$s" : "Tá an bord cianda %1$s roinnte leat ag %2$s",
    "Deck board" : "Bord deic",
    "Owned by %1$s" : "Is le %1$s í",
    "Deck boards, cards and comments" : "Cláir deic, cártaí agus tuairimí",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Ó %1$s, i %2$s/%3$s, ar le %4$s é",
    "Create a new deck card" : "Cruthaigh cárta deic nua",
    "Card comments" : "Tuairimí cártaí",
    "%s on %s" : "%s ar %s",
    "Deck boards and cards" : "Cláir deic agus cártaí",
    "No data was provided to create an attachment." : "Níor soláthraíodh aon sonraí chun ceangaltán a chruthú.",
    "Finished" : "Críochnaithe",
    "To review" : "Chun athbhreithniú a dhéanamh",
    "Action needed" : "Gníomh de dhíth",
    "Later" : "Níos déanaí",
    "copy" : "cóip",
    "Read more inside" : "Léigh tuilleadh taobh istigh",
    "Custom lists - click to rename!" : "Liostaí saincheaptha - cliceáil chun athainmniú!",
    "To Do" : "A dhéanamh",
    "In Progress" : "Ar Siúl",
    "Done" : "Déanta",
    "1. Open to learn more about boards and cards" : "1. Oscail chun níos mó a fhoghlaim faoi bhoird agus cártaí",
    "2. Drag cards left and right, up and down" : "2. Tarraing cártaí ar chlé agus ar dheis, suas agus síos",
    "3. Apply rich formatting and link content" : "3. Cuir formáidiú saibhir agus ábhar nasc i bhfeidhm",
    "4. Share, comment and collaborate!" : "4. Roinn, déan trácht agus comhoibrigh!",
    "Create your first card!" : "Cruthaigh do chéad chárta!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Tá níos mó ná %s carachtar sa nóta tráchta seo.\nCurtha leis mar cheangaltán leis an gcárta ar a bhfuil an t-ainm %s.\nInrochtana ar URL: %s.",
    "Attachments" : "Ceangaltáin",
    "File" : "Comhad",
    "date" : "dáta",
    "Card not found" : "Cárta gan aimsiú",
    "Path is already shared with this card" : "Tá an chonair roinnte leis an gcárta seo cheana féin",
    "Invalid date, date format must be YYYY-MM-DD" : "Dáta neamhbhailí, caithfidh formáid an dáta a bheith BBBB-MM-DD",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Gach bord atá i do sheilbh, lena n-áirítear cruacha, cártaí, lipéid, tascanna agus tuairimí",
    "Personal planning and team project organization" : "Pleanáil phearsanta agus eagrú tionscadail foirne",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Is uirlis eagraíochta stíl kanban é Deck atá dírithe ar phleanáil phearsanta agus eagrú tionscadail d'fhoirne atá comhtháite le Nextcloud.\n\n\n- 📥 Cuir do thascanna le cártaí agus cuir in ord iad\n- 📄 Scríobh síos nótaí breise i Markdown\n- 🔖 Sann lipéid le haghaidh eagrú níos fearr fós\n- 👥 Roinn le d'fhoireann, do chairde nó do theaghlach\n- 📎 Ceangail comhaid agus leabaigh iad i do chur síos Markdown\n- 💬 Déan plé le d’fhoireann ag úsáid tuairimí\n- ⚡ Coinnigh súil ar athruithe sa sruth gníomhaíochta\n- 🚀 Eagraigh do thionscadal",
    "Add board" : "Cuir bord leis",
    "Card details" : "Sonraí cártaí",
    "Select the board to link to a project" : "Roghnaigh an clár chun nasc a dhéanamh le tionscadal",
    "Search by board title" : "Cuardaigh de réir teideal an bhoird",
    "Select board" : "Roghnaigh bord",
    "Move/copy card" : "Bog/cóipeáil an cárta",
    "Select a board" : "Roghnaigh bord",
    "No lists available" : "Níl aon liostaí ar fáil",
    "Select a list" : "Roghnaigh liosta",
    "Move card" : "Bog cárta",
    "Copy card" : "Cóip cárta",
    "Select the card to link to a project" : "Roghnaigh an cárta chun nascadh le tionscadal",
    "Link to card" : "Nasc le cárta",
    "Select a card" : "Roghnaigh cárta",
    "Cancel" : "Cealaigh",
    "This board is read only" : "Tá an bord seo léite amháin",
    "Drop your files to upload" : "Scaoil do chuid comhad le huaslódáil",
    "File already exists" : "Tá an comhad ann cheana féin",
    "A file with the name {filename} already exists." : "Tá comhad leis an ainm {filename} ann cheana féin.",
    "Do you want to overwrite it?" : "Ar mhaith leat é a fhorscríobh?",
    "Overwrite file" : "Forscríobh comhad",
    "Keep existing file" : "Coinnigh an comhad atá ann cheana féin",
    "Add card" : "Cuir cárta leis",
    "Archived cards" : "Cártaí cartlainne",
    "Add list" : "Cuir liosta leis",
    "List name" : "Ainm an liosta",
    "Clear search" : "Glan cuardach",
    "Active filters" : "Scagairí gníomhacha",
    "Apply filter" : "Cuir scagaire i bhfeidhm",
    "Filter by tag" : "Scag le clib",
    "Filter by assigned user" : "Scag de réir úsáideora sannta",
    "Unassigned" : "Neamhshannta",
    "Filter by status" : "Scag de réir stádais",
    "Open and completed" : "Oscailte agus críochnaithe",
    "Open" : "Oscail",
    "Completed" : "Críochnaithe",
    "Filter by due date" : "Scag faoin dáta dlite",
    "Overdue" : "Thar téarma",
    "Next 24 hours" : "24 uair an chloig eile",
    "Next 7 days" : "7 lá ina dhiaidh sin",
    "Next 30 days" : "30 lá eile",
    "No due date" : "Gan dáta dlite",
    "Clear filter" : "Glan an scagaire",
    "View Modes" : "Modhanna Amharc",
    "Toggle View Modes" : "Scoránaigh Modhanna Amharc",
    "Kanban view" : "Radharc Kanban",
    "Gantt view" : "Radharc Gantt",
    "Hide archived cards" : "Folaigh cártaí cartlainne",
    "Show archived cards" : "Taispeáin cártaí cartlainne",
    "Toggle compact mode" : "Scoránaigh mód dlúth",
    "Hide card cover images" : "Folaigh íomhánna clúdaigh cártaí",
    "Show card cover images" : "Taispeáin íomhánna clúdaigh cártaí",
    "Open details" : "Sonraí oscailte",
    "Details" : "Sonraí",
    "Deck settings" : "Socruithe deic",
    "General" : "Ginearálta",
    "Use bigger card view" : "Úsáid radharc cárta níos mó",
    "Appearance" : "Dealramh",
    "Show card ID badge" : "Taispeáin suaitheantas aitheantais an chárta",
    "Show boards in calendar/tasks" : "Taispeáin cláir san fhéilire/tascanna",
    "Admin settings" : "Socruithe admin",
    "Limit board creation to some groups" : "Teorainn a chur le cruthú cláir do roinnt grúpaí",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Ní bheidh úsáideoirí taobh amuigh de na grúpaí sin in ann a gcuid boird féin a chruthú, ach beidh siad fós in ann oibriú ar bhoird atá roinnte leo.",
    "Enable federation" : "Cumasaigh cónaidhm",
    "Board actions" : "Gníomhartha an Bhoird",
    "Scroll sideways" : "Scrollaigh taobh",
    "Navigate between cards" : "Déan nascleanúint idir cártaí",
    "Close card details" : "Dún sonraí an chárta",
    "Search" : "Cuardach",
    "Show card filters" : "Taispeáin scagairí cártaí",
    "Clear card filters" : "Glan na scagairí cárta",
    "Show those shortcuts" : "Taispeáin na haicearraí sin",
    "Card actions" : "Gníomhartha cártaí",
    "Open card details" : "Oscail sonraí cárta",
    "Edit the card title" : "Cuir teideal an chárta in eagar",
    "Assign yourself to the current card" : "Sann tú féin don chárta reatha",
    "Archive/unarchive the current card" : "Cuir an cárta reatha i gcartlann/dí-cartlann",
    "Mark card as completed/not completed" : "Marcáil an cárta mar chríochnaithe/gan chríochnú",
    "Open card menu" : "Oscail roghchlár cártaí",
    "Currently present people" : "Daoine i láthair faoi láthair",
    "Supported prefixes: {prefixes}. Wrap phrases in double quotes." : "Réimíreanna a dtacaítear leo: {prefixes}. Cuir frásaí idir comharthaí athfhriotail dúbailte.",
    "Search cards" : "Cártaí cuardaigh",
    "Loading board" : "Bord á lódáil",
    "Board not found" : "Bord gan aimsiú",
    "Create a new list to add cards to this board" : "Cruthaigh liosta nua chun cártaí a chur leis an gclár seo",
    "Sharing" : "Roinnt",
    "Tags" : "Clibeanna",
    "Deleted items" : "Míreanna scriosta",
    "Activity" : "Gníomhaíocht",
    "Deleted lists" : "Liostaí scriosta",
    "Undo" : "Cealaigh",
    "Deleted cards" : "Cártaí scriosta",
    "Hours" : "Uaireanta",
    "Days" : "Laethanta",
    "Weeks" : "Seachtaine",
    "Months" : "Míonna",
    "No cards yet" : "Níl cártaí fós",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Socraigh dáta tosaigh agus dáta dlite ar do chártaí chun iad a fheiceáil ar chairt Gantt",
    "Cards without dates ({count})" : "Cártaí gan dátaí ({count})",
    "Failed to create share with {displayName}" : "Theip ar chomhroinnt le {displayName} a chruthú",
    "Are you sure you want to transfer the board {title} to {user}?" : "An bhfuil tú cinnte gur mhaith leat an clár {title} a aistriú go {user}?",
    "Transfer the board." : "Aistrigh an bord.",
    "Transfer" : "Aistriú",
    "The board has been transferred to {user}" : "Aistríodh an clár go {user}",
    "Failed to transfer the board to {user}" : "Theip ar an gclár a aistriú go {user}",
    "Board owner" : "Úinéir boird",
    "(Group)" : "(Grúpa)",
    "(Team)" : "(Foireann)",
    "(remote)" : "(iargúlta)",
    "Can edit" : "Is féidir eagarthóireacht a dhéanamh",
    "Can share" : "Is féidir a roinnt",
    "Can manage" : "Is féidir a bhainistiú",
    "Owner" : "Úinéir",
    "Delete" : "Scrios",
    "List deleted" : "Scriosadh an liosta",
    "Edit list title" : "Cuir teideal an liosta in eagar",
    "Archive all cards" : "Cartlann na cártaí go léir",
    "Unarchive all cards" : "Díchairtigh na cártaí go léir",
    "Do not set cards as \"done\"" : "Ná socraigh cártaí mar \"déanta\"",
    "Set cards as \"done\"" : "Socraigh cártaí mar \"déanta\"",
    "Delete list" : "Scrios an liosta",
    "Archive all cards in this list" : "Cuir gach cárta sa liosta seo i gcartlann",
    "Unarchive all cards in this list" : "Déan na cártaí go léir sa liosta seo a dhíchartlannú",
    "Add a new card" : "Cuir cárta nua leis",
    "Card name" : "Ainm cárta",
    "title and color value must be provided" : "ní mór teideal agus luach datha a sholáthar",
    "Edit" : "Cuir in eagar",
    "Add a new tag" : "Cuir clib nua leis",
    "Search boards" : "Cuardaigh boird",
    "Board name" : "Ainm an bhoird",
    "Members" : "Baill",
    "Assign to users/groups/team" : "Sann d'úsáideoirí/grúpaí/foireann",
    "Assign a user to this card…" : "Sann úsáideoir don chárta seo…",
    "Select a user to assign to this card…" : "Roghnaigh úsáideoir le sannadh don chárta seo…",
    "File to share" : "Comhad le roinnt",
    "Invalid path selected" : "Conair neamhbhailí roghnaithe",
    "Upload new files" : "Uaslódáil comhaid nua",
    "Share from Files" : "Comhroinn ó Chomhaid",
    "Pending share" : "Ar feitheamh sciar",
    "Add this attachment" : "Cuir an ceangaltán seo leis",
    "Show in Files" : "Taispeáin i gComhaid",
    "Download" : "Íoslódáil",
    "Remove attachment" : "Bain ceangaltán",
    "Delete Attachment" : "Scrios Ceangaltán",
    "Restore Attachment" : "Athchóirigh Ceangaltán",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Modhnaithe: {modifiedDate} ⸱ Cruthaithe: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Modhnaithe: {modifiedDate}\nCruthaithe: {createdDate}\nCruthaithe ag: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Modhnaithe: {modifiedDate}\nCruthaithe: {createdDate}",
    "The title cannot be empty." : "Ní féidir leis an teideal a bheith folamh.",
    "Cannot close unsaved card!" : "Ní féidir cárta neamhshábháilte a dhúnadh!",
    "Open in sidebar view" : "Oscail san amharc barra taoibh",
    "Open in bigger view" : "Oscail i radharc níos mó",
    "Comments" : "Tuairimí",
    "Failed to load comments" : "Theip ar lódáil nótaí",
    "No comments yet. Begin the discussion!" : "Níl trácht ar bith fós. Cuir tús leis an bplé!",
    "The comment cannot be empty." : "Ní féidir leis an nóta tráchta a bheith folamh.",
    "The comment cannot be longer than 1000 characters." : "Ní féidir leis an nóta tráchta a bheith níos faide ná 1000 carachtar.",
    "Submit" : "Cuir isteach",
    "Created:" : "Cruthaithe:",
    "In reply to" : "Mar fhreagra ar",
    "Cancel reply" : "Cealaigh freagra",
    "Reply" : "Freagra",
    "Update" : "Nuashonrú",
    "Assign dependent cards" : "Sannadh cártaí cleithiúnacha",
    "Mark as done" : "Marcáil mar a rinneadh",
    "Mark as not done" : "Marcáil mar nach bhfuil déanta",
    "Remove dependency" : "Bain spleáchas",
    "Add dependent card" : "Cuir cárta cleithiúnach leis",
    "Assign a dependent card…" : "Sannadh cárta cleithiúnach…",
    "Select a dependent card…" : "Roghnaigh cárta spleách…",
    "Write a description …" : "Scríobh cur síos…",
    "The description has been changed by another user." : "Tá an cur síos athraithe ag úsáideoir eile.",
    "Could not save description" : "Níorbh fhéidir an cur síos a shábháil",
    "Description" : "Cur síos",
    "(Unsaved)" : "(Gan sábháil)",
    "(Saving…)" : "(Shábháil…)",
    "Formatting help" : "Ionsáigh ó Chomhaid",
    "Edit description" : "Cuir cur síos in eagar",
    "View description" : "Féach ar an gcur síos",
    "Add Attachment" : "Cuir Ceangaltán leis",
    "Choose attachment" : "Roghnaigh ceangaltán",
    "Select Date" : "Roghnaigh Dáta",
    "Later today – {timeLocale}" : "Níos déanaí inniu - {timeLocale}",
    "Set due date for later today" : "Socraigh dáta dlite níos déanaí inniu",
    "Tomorrow – {timeLocale}" : "Amárach - {timeLocale}",
    "Set due date for tomorrow" : "Socraigh dáta dlite don lá amárach",
    "This weekend – {timeLocale}" : "An deireadh seachtaine seo - {timeLocale}",
    "Set due date for this weekend" : "Socraigh dáta dlite don deireadh seachtaine seo",
    "Next week – {timeLocale}" : "An tseachtain seo chugainn – {timeLocale}",
    "Set due date for next week" : "Socraigh dáta dlite don tseachtain seo chugainn",
    "Assign a due date to this card…" : "Sann dáta dlite don chárta seo…",
    "Set a due date" : "Socraigh dáta dlite",
    "Add due date" : "Cuir dáta dlite leis",
    "Choose a date" : "Roghnaigh dáta",
    "Remove due date" : "Bain an dáta dlite",
    "Due at:" : "Dlite ag:",
    "Not done" : "Ní dhearna",
    "Unarchive card" : "Cárta unarchive",
    "Archive card" : "Cárta cartlainne",
    "Assign a start date to this card…" : "Sannadh dáta tosaithe don chárta seo…",
    "Set a start date" : "Socraigh dáta tosaithe",
    "Add start date" : "Cuir dáta tosaithe leis",
    "Remove start date" : "Bain an dáta tosaithe",
    "Assign a tag to this card…" : "Sann clib don chárta seo…",
    "Select or create a tag…" : "Roghnaigh nó cruthaigh clib…",
    "Create a new tag:" : "Cruthaigh clib nua:",
    "(group)" : "(grúpa)",
    "{count} comments, {unread} unread" : "{count} nóta tráchta, {unread} neamhléite",
    "Todo items" : "Míreanna le déanamh",
    "Edit card title" : "Cuir teideal an chárta in eagar",
    "Open link" : "Oscail nasc",
    "Card deleted" : "Cárta scriosta",
    "Change card color" : "Athraigh dath an chárta",
    "Edit title" : "Cuir teideal in eagar",
    "Assign to me" : "Sann dom",
    "Unassign myself" : "Díshannadh mé féin",
    "Delete card" : "Scrios cárta",
    "All boards" : "Gach boird",
    "Archived boards" : "Boird chartlainne",
    "Shared with you" : "Roinnte leat",
    "Cancel edit" : "Cealaigh eagarthóireacht",
    "Save board" : "Sábháil bord",
    "Board {0} deleted" : "Scriosadh an bord {0}",
    "All cards" : "Gach cártaí",
    "Only assigned cards" : "Cártaí sannta amháin",
    "No reminder" : "Gan meabhrúchán",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "An bhfuil tú cinnte gur mhaith leat an clár {title} a scriosadh? Scriosfaidh sé seo sonraí uile an bhoird seo lena n-áirítear cártaí cartlainne.",
    "Delete the board?" : "An bhfuil fonn ort an clár a scriosadh?",
    "Are you sure you want to leave the board {title}?" : "An bhfuil tú cinnte gur mian leat an bord {title} a fhágáil?",
    "Leave the board?" : "An bord a fhágáil?",
    "Leave" : "Fág",
    "Failed to leave the board" : "Theip ar an mbord a fhágáil",
    "Exporting board..." : "Ag easpórtáil an bhoird...",
    "An error occurred" : "Tharla earráid",
    "Board details" : "Sonraí an Bhoird",
    "Edit board" : "Cuir bord in eagar",
    "Clone board" : "Clár clón",
    "Unarchive board" : "Bord gan chartlann",
    "Archive board" : "Bord cartlainne",
    "Export board" : "Bord easpórtála",
    "Turn on due date reminders" : "Cuir meabhrúcháin dáta dlite ar siúl",
    "Turn off due date reminders" : "Múch meabhrúcháin dáta dlite",
    "Remove as default board" : "Bain mar bhord réamhshocraithe",
    "Set as default board" : "Socraigh mar bhord réamhshocraithe",
    "Due date reminders" : "Meabhrúcháin dáta dlite",
    "Assigned cards" : "Cártaí sannta",
    "No notifications" : "Gan fógraí",
    "Delete board" : "Scrios an bord",
    "Leave board" : "Fág an bord",
    "Importing board..." : "Ag allmhairiú boird...",
    "Board imported successfully" : "D'éirigh le hiompórtáil an bhoird",
    "Import board" : "Bord allmhairithe",
    "Clone {boardTitle}" : "Clón {boardTitle}",
    "Clone cards" : "Cártaí clón",
    "Clone assignments" : "Tascanna Clón",
    "Clone labels" : "Lipéid Clón",
    "Clone due dates" : "Dátaí dlite Clón",
    "Advanced options" : "Ardroghanna",
    "Move all cards to the first list" : "Bog na cártaí go léir chuig an gcéad liosta",
    "Restore archived cards" : "Athchóirigh cártaí gcartlann",
    "Clone" : "Clón",
    "Export {boardTitle}" : "Easpórtáil {boardTitle}",
    "Export as JSON" : "Easpórtáil mar JSON",
    "Export as CSV" : "Easpórtáil mar CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Nóta: Ní thacaítear ach leis an bhformáid JSON le haghaidh iompórtála ar ais isteach san aip Deck.",
    "Export" : "Easpórtáil",
    "Loading filtered view" : "Amharc scagtha á lódáil",
    "Search for {searchQuery} in other boards" : "Cuardaigh le haghaidh {searchQuery} i gcláir eile",
    "Search for {searchQuery} in all boards" : "Cuardaigh {searchQuery} i ngach clár",
    "No results found" : "Níor aimsíodh aon torthaí",
    "Deck board {name}\n* Last modified on {lastMod}" : "Bord deic {name}\n* Athraithe go deireanach ar {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Cruthaithe ar {created}\n* Athraithe go deireanach ar {lastMod}\n* {nbAttachments} ceangaltán\n* {nbComments} nóta tráchta",
    "{nbCards} cards" : "cártaí {nbCards}",
    "Due on {date}" : "Dlite ar {date}",
    "{stack} in {board}" : "{stack} in {board}",
    "Click to expand description" : "Cliceáil chun cur síos a leathnú",
    "Click to expand comment" : "Cliceáil chun trácht a leathnú",
    "Create card" : "Cruthaigh cárta",
    "Create a new card" : "Cruthaigh cárta nua",
    "Card title" : "Teideal an chárta",
    "Creating the new card …" : "Cárta nua á chruthú…",
    "Card \"{card}\" was added to \"{board}\"" : "Cuireadh cárta \"{card}\" le \"{board}\"",
    "Open card" : "Cárta oscailte",
    "Close" : "Dún",
    "No upcoming cards" : "Uimh cártaí le teacht",
    "upcoming cards today" : "cártaí atá le teacht inniu",
    "upcoming cards tomorrow" : "cártaí atá le teacht amárach",
    "upcoming cards" : "cártaí atá le teacht",
    "New card" : "Cárta nua",
    "Link to a board" : "Nasc le bord",
    "Link to a card" : "Nasc le cárta",
    "Create a card" : "Cruthaigh cárta",
    "Message from {author} in {conversationName}" : "Teachtaireacht ó {author} in {conversationName}",
    "Something went wrong" : "Chuaigh rud eigin mícheart",
    "Failed to upload {name}" : "Theip ar uaslódáil {name}",
    "Maximum file size of {size} exceeded" : "Sáraíodh uasmhéid comhaid de {size}",
    "Assigned users" : "Úsáideoirí sannta",
    "Due date" : "Dáta dlite",
    "Created" : "Cruthaithe",
    "Modified" : "Athraithe",
    "Error creating the share" : "Earráid agus an sciar á cruthú",
    "Share with a Deck card" : "Comhroinn le cárta Deic",
    "Share {file} with a Deck card" : "Roinn {file} le cárta Deic",
    "Share" : "Comhroinn",
    "The card \"%s\" on \"%s\" has reached its due date." : "Tá an dáta dlite sroichte ag an gcárta \"%s\" ar \"%s\".",
    "Share board with a user, group or team …" : "Roinn an clár le húsáideoir, grúpa nó foireann…",
    "Searching for users, groups and teams …" : "Ag cuardach úsáideoirí, grúpaí agus foirne…",
    "No participants found" : "Níor aimsíodh rannpháirtithe ar bith",
    "Save" : "Sábháil",
    "seconds ago" : "soicind ó shin",
    "Today" : "Inniu",
    "Tomorrow" : "Amárach",
    "No due" : "Níl dlite"
},
"nplurals=5; plural=(n==1 ? 0 : n==2 ? 1 : n<7 ? 2 : n<11 ? 3 : 4);");
