OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "You have created a new board {board}",
    "{user} has created a new board {board}" : "{user} has created a new board {board}",
    "You have deleted the board {board}" : "You have deleted the board {board}",
    "{user} has deleted the board {board}" : "{user} has deleted the board {board}",
    "You have restored the board {board}" : "You have restored the board {board}",
    "{user} has restored the board {board}" : "{user} has restored the board {board}",
    "You have shared the board {board} with {acl}" : "You have shared the board {board} with {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} has shared the board {board} with {acl}",
    "You have removed {acl} from the board {board}" : "You have removed {acl} from the board {board}",
    "{user} has removed {acl} from the board {board}" : "{user} has removed {acl} from the board {board}",
    "You have renamed the board {before} to {board}" : "You have renamed the board {before} to {board}",
    "{user} has renamed the board {before} to {board}" : "{user} has renamed the board {before} to {board}",
    "You have archived the board {board}" : "You have archived the board {board}",
    "{user} has archived the board {before}" : "{user} has archived the board {before}",
    "You have unarchived the board {board}" : "You have unarchived the board {board}",
    "{user} has unarchived the board {before}" : "{user} has unarchived the board {before}",
    "You have created a new list {stack} on board {board}" : "You have created a new list {stack} on board {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} has created a new list {stack} on board {board}",
    "You have renamed list {before} to {stack} on board {board}" : "You have renamed list {before} to {stack} on board {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} has renamed list {before} to {stack} on board {board}",
    "You have deleted list {stack} on board {board}" : "You have deleted list {stack} on board {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} has deleted list {stack} on board {board}",
    "You have created card {card} in list {stack} on board {board}" : "You have created card {card} in list {stack} on board {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} has created card {card} in list {stack} on board {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "You have deleted card {card} in list {stack} on board {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} has deleted card {card} in list {stack} on board {board}",
    "You have renamed the card {before} to {card}" : "You have renamed the card {before} to {card}",
    "{user} has renamed the card {before} to {card}" : "{user} has renamed the card {before} to {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "You have added a description to card {card} in list {stack} on board {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} has added a description to card {card} in list {stack} on board {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "You have updated the description of card {card} in list {stack} on board {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} has updated the description of the card {card} in list {stack} on board {board}",
    "You have archived card {card} in list {stack} on board {board}" : "You have archived card {card} in list {stack} on board {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} has archived card {card} in list {stack} on board {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "You have unarchived card {card} in list {stack} on board {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} has unarchived card {card} in list {stack} on board {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "You have marked the card {card} as done in list {stack} on board {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} has marked card {card} as done in list {stack} on board {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "You have marked the card {card} as undone in list {stack} on board {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} has marked the card {card} as undone in list {stack} on board {board}",
    "You have removed the due date of card {card}" : "You have removed the due date of card {card}",
    "{user} has removed the due date of card {card}" : "{user} has removed the due date of card {card}",
    "You have set the due date of card {card} to {after}" : "You have set the due date of card {card} to {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} has set the due date of card {card} to {after}",
    "You have updated the due date of card {card} to {after}" : "You have updated the due date of card {card} to {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} has updated the due date of card {card} to {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "You have added the tag {label} to card {card} in list {stack} on board {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} has added the tag {label} to card {card} in list {stack} on board {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "You have removed the tag {label} from card {card} in list {stack} on board {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} has removed the tag {label} from card {card} in list {stack} on board {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "You have assigned {assigneduser} to card {card} on board {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} has assigned {assigneduser} to card {card} on board {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "You have unassigned {assigneduser} from card {card} on board {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} has unassigned {assigneduser} from card {card} on board {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "You have moved the card {card} from list {stackBefore} to {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} has moved the card {card} from list {stackBefore} to {stack}",
    "You have added the attachment {attachment} to card {card}" : "You have added the attachment {attachment} to card {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} has added the attachment {attachment} to card {card}",
    "You have updated the attachment {attachment} on card {card}" : "You have updated the attachment {attachment} on card {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} has updated the attachment {attachment} on card {card}",
    "You have deleted the attachment {attachment} from card {card}" : "You have deleted the attachment {attachment} from card {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} has deleted the attachment {attachment} from card {card}",
    "You have restored the attachment {attachment} to card {card}" : "You have restored the attachment {attachment} to card {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} has restored the attachment {attachment} to card {card}",
    "You have commented on card {card}" : "You have commented on card {card}",
    "{user} has commented on card {card}" : "{user} has commented on card {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Changes in the <strong>Deck app</strong>",
    "A <strong>board, list or card</strong> was changed" : "A <strong>board, list or card</strong> was changed",
    "A <strong>comment</strong> was created on a card" : "A <strong>comment</strong> was created on a card",
    "A <strong>card description</strong> has been changed" : "A <strong>card description</strong> has been changed",
    "The file was uploaded" : "The file was uploaded",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "The uploaded file exceeds the upload_max_filesize directive in php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form",
    "The file was only partially uploaded" : "The file was only partially uploaded",
    "No file was uploaded" : "No file was uploaded",
    "Missing a temporary folder" : "Missing a temporary folder",
    "Could not write file to disk" : "Could not write file to disk",
    "A PHP extension stopped the file upload" : "A PHP extension stopped the file upload",
    "No file uploaded or file size exceeds maximum of %s" : "No file uploaded or file size exceeds maximum of %s",
    "Invalid file type. Only JSON files are allowed." : "Invalid file type. Only JSON files are allowed.",
    "Invalid JSON data" : "Invalid JSON data",
    "Failed to import board" : "Failed to import board",
    "Cards due today" : "Cards due today",
    "Cards due tomorrow" : "Cards due tomorrow",
    "Upcoming cards" : "Upcoming cards",
    "Load more" : "Load more",
    "Welcome to Nextcloud Deck!" : "Welcome to Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "The card \"%s\" on \"%s\" has been assigned to you by %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} has assigned the card {deck-card} on {deck-board} to you.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "The card \"%1$s\" on \"%2$s\" has reached its due date.",
    "The card {deck-card} on {deck-board} has reached its due date." : "The card {deck-card} on {deck-board} has reached its due date.",
    "%s has mentioned you in a comment on \"%s\"." : "%s has mentioned you in a comment on \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} has mentioned you in a comment on {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "The board \"%s\" has been shared with you by %s.",
    "{user} has shared {deck-board} with you." : "{user} has shared {deck-board} with you.",
    "The remote board %1$s has been shared with you by %2$s" : "The remote board %1$s has been shared with you by %2$s",
    "Deck board" : "Deck board",
    "Owned by %1$s" : "Owned by %1$s",
    "Deck boards, cards and comments" : "Deck boards, cards and comments",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "From %1$s, in %2$s/%3$s, owned by %4$s",
    "Create a new deck card" : "Create a new deck card",
    "Card comments" : "Card comments",
    "%s on %s" : "%s on %s",
    "Deck boards and cards" : "Deck boards and cards",
    "No data was provided to create an attachment." : "No data was provided to create an attachment.",
    "Finished" : "Finished",
    "To review" : "To review",
    "Action needed" : "Action needed",
    "Later" : "Later",
    "copy" : "copy",
    "Read more inside" : "Read more inside",
    "Custom lists - click to rename!" : "Custom lists - click to rename!",
    "To Do" : "To Do",
    "In Progress" : "In Progress",
    "Done" : "Done",
    "1. Open to learn more about boards and cards" : "1. Open to learn more about boards and cards",
    "2. Drag cards left and right, up and down" : "2. Drag cards left and right, up and down",
    "3. Apply rich formatting and link content" : "3. Apply rich formatting and link content",
    "4. Share, comment and collaborate!" : "4. Share, comment and collaborate!",
    "Create your first card!" : "Create your first card!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s.",
    "Attachments" : "Attachments",
    "File" : "File",
    "date" : "date",
    "Card not found" : "Card not found",
    "Path is already shared with this card" : "Path is already shared with this card",
    "Invalid date, date format must be YYYY-MM-DD" : "Invalid date, date format must be YYYY-MM-DD",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "All boards owned by you including stacks, cards, labels, assignments, and comments",
    "Personal planning and team project organization" : "Personal planning and team project organisation",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck is a kanban style organisation tool aimed at personal planning and project organisation for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organisation\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organised",
    "Add board" : "Add board",
    "Card details" : "Card details",
    "Select the board to link to a project" : "Select the board to link to a project",
    "Search by board title" : "Search by board title",
    "Select board" : "Select board",
    "Move/copy card" : "Move/copy card",
    "Select a board" : "Select a board",
    "No lists available" : "No lists available",
    "Select a list" : "Select a list",
    "Move card" : "Move card",
    "Copy card" : "Copy card",
    "Select the card to link to a project" : "Select the card to link to a project",
    "Link to card" : "Link to card",
    "Select a card" : "Select a card",
    "Cancel" : "Cancel",
    "This board is read only" : "This board is read only",
    "Drop your files to upload" : "Drop your files to upload",
    "File already exists" : "File already exists",
    "A file with the name {filename} already exists." : "A file with the name {filename} already exists.",
    "Do you want to overwrite it?" : "Do you want to overwrite it?",
    "Overwrite file" : "Overwrite file",
    "Keep existing file" : "Keep existing file",
    "Add card" : "Add card",
    "Archived cards" : "Archived cards",
    "Add list" : "Add list",
    "List name" : "List name",
    "Clear search" : "Clear search",
    "Active filters" : "Active filters",
    "Apply filter" : "Apply filter",
    "Filter by tag" : "Filter by tag",
    "Filter by assigned user" : "Filter by assigned user",
    "Unassigned" : "Unassigned",
    "Filter by status" : "Filter by status",
    "Open and completed" : "Open and completed",
    "Open" : "Open",
    "Completed" : "Completed",
    "Filter by due date" : "Filter by due date",
    "Overdue" : "Overdue",
    "Next 24 hours" : "Next 24 hours",
    "Next 7 days" : "Next 7 days",
    "Next 30 days" : "Next 30 days",
    "No due date" : "No due date",
    "Clear filter" : "Clear filter",
    "View Modes" : "View Modes",
    "Toggle View Modes" : "Toggle View Modes",
    "Kanban view" : "Kanban view",
    "Gantt view" : "Gantt view",
    "Hide archived cards" : "Hide archived cards",
    "Show archived cards" : "Show archived cards",
    "Toggle compact mode" : "Toggle compact mode",
    "Hide card cover images" : "Hide card cover images",
    "Show card cover images" : "Show card cover images",
    "Open details" : "Open details",
    "Details" : "Details",
    "Deck settings" : "Deck settings",
    "General" : "General",
    "Use bigger card view" : "Use bigger card view",
    "Appearance" : "Appearance",
    "Show card ID badge" : "Show card ID badge",
    "Show boards in calendar/tasks" : "Show boards in calendar/tasks",
    "Admin settings" : "Admin settings",
    "Limit board creation to some groups" : "Limit board creation to some groups",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them.",
    "Enable federation" : "Enable federation",
    "Board actions" : "Board actions",
    "Scroll sideways" : "Scroll sideways",
    "Navigate between cards" : "Navigate between cards",
    "Close card details" : "Close card details",
    "Search" : "Search",
    "Show card filters" : "Show card filters",
    "Clear card filters" : "Clear card filters",
    "Show those shortcuts" : "Show those shortcuts",
    "Card actions" : "Card actions",
    "Open card details" : "Open card details",
    "Edit the card title" : "Edit the card title",
    "Assign yourself to the current card" : "Assign yourself to the current card",
    "Archive/unarchive the current card" : "Archive/unarchive the current card",
    "Mark card as completed/not completed" : "Mark card as completed/not completed",
    "Open card menu" : "Open card menu",
    "Currently present people" : "Currently present people",
    "Supported prefixes: {prefixes}. Wrap phrases in double quotes." : "Supported prefixes: {prefixes}. Wrap phrases in double quotes.",
    "Search cards" : "Search cards",
    "Loading board" : "Loading board",
    "Board not found" : "Board not found",
    "Create a new list to add cards to this board" : "Create a new list to add cards to this board",
    "Sharing" : "Sharing",
    "Tags" : "Tags",
    "Deleted items" : "Deleted items",
    "Activity" : "Activity",
    "Deleted lists" : "Deleted lists",
    "Undo" : "Undo",
    "Deleted cards" : "Deleted cards",
    "Hours" : "Hours",
    "Days" : "Days",
    "Weeks" : "Weeks",
    "Months" : "Months",
    "No cards yet" : "No cards yet",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Set a start date and due date on your cards to see them on the Gantt chart",
    "Cards without dates ({count})" : "Cards without dates ({count})",
    "Failed to create share with {displayName}" : "Failed to create share with {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Are you sure you want to transfer the board {title} to {user}?",
    "Transfer the board." : "Transfer the board.",
    "Transfer" : "Transfer",
    "The board has been transferred to {user}" : "The board has been transferred to {user}",
    "Failed to transfer the board to {user}" : "Failed to transfer the board to {user}",
    "Board owner" : "Board owner",
    "(Group)" : "(Group)",
    "(Team)" : "(Team)",
    "(remote)" : "(remote)",
    "Can edit" : "Can edit",
    "Can share" : "Can share",
    "Can manage" : "Can manage",
    "Owner" : "Owner",
    "Delete" : "Delete",
    "List deleted" : "List deleted",
    "Edit list title" : "Edit list title",
    "Archive all cards" : "Archive all cards",
    "Unarchive all cards" : "Unarchive all cards",
    "Do not set cards as \"done\"" : "Do not set cards as \"done\"",
    "Set cards as \"done\"" : "Set cards as \"done\"",
    "Delete list" : "Delete list",
    "Archive all cards in this list" : "Archive all cards in this list",
    "Unarchive all cards in this list" : "Unarchive all cards in this list",
    "Add a new card" : "Add a new card",
    "Card name" : "Card name",
    "title and color value must be provided" : "title and colour value must be provided",
    "Edit" : "Edit",
    "Add a new tag" : "Add a new tag",
    "Search boards" : "Search boards",
    "Board name" : "Board name",
    "Members" : "Members",
    "Assign to users/groups/team" : "Assign to users/groups/team",
    "Assign a user to this card…" : "Assign a user to this card…",
    "Select a user to assign to this card…" : "Select a user to assign to this card…",
    "File to share" : "File to share",
    "Invalid path selected" : "Invalid path selected",
    "Upload new files" : "Upload new files",
    "Share from Files" : "Share from Files",
    "Pending share" : "Pending share",
    "Add this attachment" : "Add this attachment",
    "Show in Files" : "Show in Files",
    "Download" : "Download",
    "Remove attachment" : "Remove attachment",
    "Delete Attachment" : "Delete Attachment",
    "Restore Attachment" : "Restore Attachment",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Modified: {modifiedDate} ⸱ Created: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Modified: {modifiedDate}\nCreated: {createdDate}",
    "The title cannot be empty." : "The title cannot be empty.",
    "Cannot close unsaved card!" : "Cannot close unsaved card!",
    "Open in sidebar view" : "Open in sidebar view",
    "Open in bigger view" : "Open in bigger view",
    "Comments" : "Comments",
    "Failed to load comments" : "Failed to load comments",
    "No comments yet. Begin the discussion!" : "No comments yet. Begin the discussion!",
    "The comment cannot be empty." : "The comment cannot be empty.",
    "The comment cannot be longer than 1000 characters." : "The comment cannot be longer than 1000 characters.",
    "Submit" : "Submit",
    "Created:" : "Created:",
    "In reply to" : "In reply to",
    "Cancel reply" : "Cancel reply",
    "Reply" : "Reply",
    "Update" : "Update",
    "Assign dependent cards" : "Assign dependent cards",
    "Mark as done" : "Mark as done",
    "Mark as not done" : "Mark as not done",
    "Remove dependency" : "Remove dependency",
    "Add dependent card" : "Add dependent card",
    "Assign a dependent card…" : "Assign a dependent card…",
    "Select a dependent card…" : "Select a dependent card…",
    "Write a description …" : "Write a description …",
    "The description has been changed by another user." : "The description has been changed by another user.",
    "Could not save description" : "Could not save description",
    "Description" : "Description",
    "(Unsaved)" : "(Unsaved)",
    "(Saving…)" : "(Saving…)",
    "Formatting help" : "Formatting help",
    "Edit description" : "Edit description",
    "View description" : "View description",
    "Add Attachment" : "Add Attachment",
    "Choose attachment" : "Choose attachment",
    "Select Date" : "Select Date",
    "Later today – {timeLocale}" : "Later today – {timeLocale}",
    "Set due date for later today" : "Set due date for later today",
    "Tomorrow – {timeLocale}" : "Tomorrow – {timeLocale}",
    "Set due date for tomorrow" : "Set due date for tomorrow",
    "This weekend – {timeLocale}" : "This weekend – {timeLocale}",
    "Set due date for this weekend" : "Set due date for this weekend",
    "Next week – {timeLocale}" : "Next week – {timeLocale}",
    "Set due date for next week" : "Set due date for next week",
    "Assign a due date to this card…" : "Assign a due date to this card…",
    "Set a due date" : "Set a due date",
    "Add due date" : "Add due date",
    "Choose a date" : "Choose a date",
    "Remove due date" : "Remove due date",
    "Due at:" : "Due at:",
    "Not done" : "Not done",
    "Unarchive card" : "Unarchive card",
    "Archive card" : "Archive card",
    "Assign a start date to this card…" : "Assign a start date to this card…",
    "Set a start date" : "Set a start date",
    "Add start date" : "Add start date",
    "Remove start date" : "Remove start date",
    "Assign a tag to this card…" : "Assign a tag to this card…",
    "Select or create a tag…" : "Select or create a tag…",
    "Create a new tag:" : "Create a new tag:",
    "(group)" : "(group)",
    "{count} comments, {unread} unread" : "{count} comments, {unread} unread",
    "Todo items" : "Todo items",
    "Edit card title" : "Edit card title",
    "Open link" : "Open link",
    "Card deleted" : "Card deleted",
    "Change card color" : "Change card colour",
    "Edit title" : "Edit title",
    "Assign to me" : "Assign to me",
    "Unassign myself" : "Unassign myself",
    "Delete card" : "Delete card",
    "All boards" : "All boards",
    "Archived boards" : "Archived boards",
    "Shared with you" : "Shared with you",
    "Cancel edit" : "Cancel edit",
    "Save board" : "Save board",
    "Board {0} deleted" : "Board {0} deleted",
    "All cards" : "All cards",
    "Only assigned cards" : "Only assigned cards",
    "No reminder" : "No reminder",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards.",
    "Delete the board?" : "Delete the board?",
    "Are you sure you want to leave the board {title}?" : "Are you sure you want to leave the board {title}?",
    "Leave the board?" : "Leave the board?",
    "Leave" : "Leave",
    "Failed to leave the board" : "Failed to leave the board",
    "Exporting board..." : "Exporting board...",
    "An error occurred" : "An error occurred",
    "Board details" : "Board details",
    "Edit board" : "Edit board",
    "Clone board" : "Clone board",
    "Unarchive board" : "Unarchive board",
    "Archive board" : "Archive board",
    "Export board" : "Export board",
    "Turn on due date reminders" : "Turn on due date reminders",
    "Turn off due date reminders" : "Turn off due date reminders",
    "Remove as default board" : "Remove as default board",
    "Set as default board" : "Set as default board",
    "Due date reminders" : "Due date reminders",
    "Assigned cards" : "Assigned cards",
    "No notifications" : "No notifications",
    "Delete board" : "Delete board",
    "Leave board" : "Leave board",
    "Importing board..." : "Importing board...",
    "Board imported successfully" : "Board imported successfully",
    "Import board" : "Import board",
    "Clone {boardTitle}" : "Clone {boardTitle}",
    "Clone cards" : "Clone cards",
    "Clone assignments" : "Clone assignments",
    "Clone labels" : "Clone labels",
    "Clone due dates" : "Clone due dates",
    "Advanced options" : "Advanced options",
    "Move all cards to the first list" : "Move all cards to the first list",
    "Restore archived cards" : "Restore archived cards",
    "Clone" : "Clone",
    "Export {boardTitle}" : "Export {boardTitle}",
    "Export as JSON" : "Export as JSON",
    "Export as CSV" : "Export as CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Note: Only the JSON format is supported for importing back into the Deck app.",
    "Export" : "Export",
    "Loading filtered view" : "Loading filtered view",
    "Search for {searchQuery} in other boards" : "Search for {searchQuery} in other boards",
    "Search for {searchQuery} in all boards" : "Search for {searchQuery} in all boards",
    "No results found" : "No results found",
    "Deck board {name}\n* Last modified on {lastMod}" : "Deck board {name}\n* Last modified on {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments",
    "{nbCards} cards" : "{nbCards} cards",
    "Due on {date}" : "Due on {date}",
    "{stack} in {board}" : "{stack} in {board}",
    "Click to expand description" : "Click to expand description",
    "Click to expand comment" : "Click to expand comment",
    "Create card" : "Create card",
    "Create a new card" : "Create a new card",
    "Card title" : "Card title",
    "Creating the new card …" : "Creating the new card …",
    "Card \"{card}\" was added to \"{board}\"" : "Card \"{card}\" was added to \"{board}\"",
    "Open card" : "Open card",
    "Close" : "Close",
    "No upcoming cards" : "No upcoming cards",
    "upcoming cards today" : "upcoming cards today",
    "upcoming cards tomorrow" : "upcoming cards tomorrow",
    "upcoming cards" : "upcoming cards",
    "New card" : "New card",
    "Link to a board" : "Link to a board",
    "Link to a card" : "Link to a card",
    "Create a card" : "Create a card",
    "Message from {author} in {conversationName}" : "Message from {author} in {conversationName}",
    "Something went wrong" : "Something went wrong",
    "Failed to upload {name}" : "Failed to upload {name}",
    "Maximum file size of {size} exceeded" : "Maximum file size of {size} exceeded",
    "Assigned users" : "Assigned users",
    "Due date" : "Due date",
    "Created" : "Created",
    "Modified" : "Modified",
    "Error creating the share" : "Error creating the share",
    "Share with a Deck card" : "Share with a Deck card",
    "Share {file} with a Deck card" : "Share {file} with a Deck card",
    "Share" : "Share",
    "The card \"%s\" on \"%s\" has reached its due date." : "The card \"%s\" on \"%s\" has reached its due date.",
    "Share board with a user, group or team …" : "Share board with a user, group or team …",
    "Searching for users, groups and teams …" : "Searching for users, groups and teams …",
    "No participants found" : "No participants found",
    "Save" : "Save",
    "seconds ago" : "seconds ago",
    "Today" : "Today",
    "Tomorrow" : "Tomorrow",
    "No due" : "No due"
},
"nplurals=2; plural=(n != 1);");
