OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "您創建了一個新面板 {board}",
    "{user} has created a new board {board}" : "{user} 創建了一個新面板 {board}",
    "You have deleted the board {board}" : "您刪除了面板 {board}",
    "{user} has deleted the board {board}" : "{user} 刪除了面板 {board}",
    "You have restored the board {board}" : "您已還原面板 {board}",
    "{user} has restored the board {board}" : "{user} 已還原面板 {board}",
    "You have shared the board {board} with {acl}" : "您已和 {acl} 分享面板 {board}",
    "{user} has shared the board {board} with {acl}" : "{user} 已和 {acl} 分享面板 {board}",
    "You have removed {acl} from the board {board}" : "您從面板 {board} 中移除了 {acl}",
    "{user} has removed {acl} from the board {board}" : "{user} 從面板 {board} 中移除了 {acl} ",
    "You have renamed the board {before} to {board}" : "您將面板 {before} 重命名為 {board}",
    "{user} has renamed the board {before} to {board}" : "{user} 將面板 {before} 重命名為 {board}",
    "You have archived the board {board}" : "您已將面板 {board} 封存",
    "{user} has archived the board {before}" : "{user} 已將面板 {before} 封存",
    "You have unarchived the board {board}" : "您撤銷了面板 {board} 的封存",
    "{user} has unarchived the board {before}" : "{user} 撤銷了面板 {before} 的封存",
    "You have created a new list {stack} on board {board}" : "您在面板 {board} 上創建了一張新清單 {stack} ",
    "{user} has created a new list {stack} on board {board}" : "{user} 在面板 {board} 上創建了一張新清單 {stack}",
    "You have renamed list {before} to {stack} on board {board}" : "您已將面板 {board} 上的清單 {before} 重命名為 {stack}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} 已將面板 {board} 上的清單 {before} 重命名為 {stack}",
    "You have deleted list {stack} on board {board}" : "您已刪除了面板 {board} 上的清單 {stack}",
    "{user} has deleted list {stack} on board {board}" : "{user} 已刪除了面板 {board} 上的清單 {stack}",
    "You have created card {card} in list {stack} on board {board}" : "您在面板 {board} 上的清單 {stack} 中創建了一張新卡片 {card}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} 在面板 {board} 上的清單 {stack} 裏創建了一張新卡片 {card}",
    "You have deleted card {card} in list {stack} on board {board}" : "您已刪除了面板 {board} 上清單 {stack} 裏的卡片 {card}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} 已刪除了面板 {board} 上清單 {stack} 裏的卡片 {card}",
    "You have renamed the card {before} to {card}" : "您將卡片 {before} 重命名為 {card}",
    "{user} has renamed the card {before} to {card}" : "{user} 將卡片 {before} 重命名為 {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "您已在面板 {board} 上清單 {stack} 裏的卡片 {card} 中添加了描述",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} 已在面板 {board} 上清單 {stack} 裏的卡片 {card} 中添加了描述",
    "You have updated the description of card {card} in list {stack} on board {board}" : "您已在面板 {board} 上清單 {stack} 裏的卡片 {card} 中更新了描述",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} 已在面板 {board} 上清單 {stack} 裏的卡片 {card} 中更新了描述",
    "You have archived card {card} in list {stack} on board {board}" : "您已將面板 {board} 上清單 {stack} 裏的卡片 {card} 封存",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} 已將面板 {board} 上清單 {stack} 裏的卡片 {card} 封存",
    "You have unarchived card {card} in list {stack} on board {board}" : "您已將面板 {board} 上清單 {stack} 裏的卡片 {card} 解除封存",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} 已將面板 {board} 上清單 {stack} 裏的卡片 {card} 解除封存",
    "You have marked the card {card} as done in list {stack} on board {board}" : "您已在佈告欄 {board} 上的列表 {stack} 標記卡片 {card} 為已完成",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} 已在佈告欄 {board} 上的列表 {stack} 標記卡片 {card} 為已完成",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "您已在佈告欄 {board} 上的列表 {stack} 標記卡片 {card} 為未完成",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} 已在佈告欄 {board} 上的列表 {stack} 標記卡片 {card} 為未完成",
    "You have removed the due date of card {card}" : "您移除了到期的卡片 {card}",
    "{user} has removed the due date of card {card}" : "{user} 移除了到期的卡片 {card}",
    "You have set the due date of card {card} to {after}" : "您已將卡片 {card} 的到期日設置為 {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} 已將卡片 {card} 的到期日設置為 {after}",
    "You have updated the due date of card {card} to {after}" : "您已將卡片 {card} 的到期日更新為 {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} 已將卡片 {card} 的到期日更新為 {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "您已將標籤 {label} 添加到面板 {board} 上清單 {stack} 中的卡片 {card} 裏",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} 已將標籤 {label} 添加到面板 {board} 上清單 {stack} 中的卡片 {card} 裡",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "您已從面板 {board} 上清單 {stack} 中的卡片 {card} 裏刪除了標籤 {label}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} 已從面板 {board} 上清單 {stack} 中的卡片 {card} 裏刪除了標籤 {label}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "您將面板 {board} 上的卡片 {card} 指派給了 {assigneduser}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} 將面板 {board} 上的卡片 {card} 指派給了 {assigneduser}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "您對 {assigneduser} 取消了面板 {board} 上的卡片 {card} 指派",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} 對 {assigneduser} 取消了面板 {board} 上的卡片 {card} 指派",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "您已將卡片 {card} 從清單 {stackBefore} 移至 {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} 已將卡片 {card} 從清單 {stackBefore} 移至 {stack}",
    "You have added the attachment {attachment} to card {card}" : "您為卡片 {card} 添加了附件 {attachment}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} 為卡片 {card} 添加了附件 {attachment}",
    "You have updated the attachment {attachment} on card {card}" : "您更新了卡片 {card} 中的附件 {attachment}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} 更新了卡 {card} 上的附件 {attachment}",
    "You have deleted the attachment {attachment} from card {card}" : "您刪除了卡片 {card} 中的附件 {attachment}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} 已從卡 {card} 中刪除附件 {attachment}",
    "You have restored the attachment {attachment} to card {card}" : "您恢復了卡片 {card} 中的附件 {attachment}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} 恢復了卡片 {card} 中的附件 {attachment}",
    "You have commented on card {card}" : "您意見了卡片 {card}",
    "{user} has commented on card {card}" : "{user} 意見了卡片 {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "<strong>Deck 應用程式</strong>中的改變",
    "A <strong>board, list or card</strong> was changed" : "<strong>面板、清單或卡片</strong>已更新",
    "A <strong>comment</strong> was created on a card" : "卡片上創建了一個 <strong>意見</strong>",
    "A <strong>card description</strong> has been changed" : "<strong>卡片描述</strong>已更新",
    "The file was uploaded" : "檔案已上傳",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "上傳的檔案大小超過 php.ini 當中 upload_max_filesize 選項的限制",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "上傳的檔案大小超過 HTML 表單中 MAX_FILE_SIZE 的限制",
    "The file was only partially uploaded" : "檔案僅部分上傳",
    "No file was uploaded" : "沒有檔案被上傳",
    "Missing a temporary folder" : "找不到暫存資料夾",
    "Could not write file to disk" : "無法將檔案寫入磁碟",
    "A PHP extension stopped the file upload" : "某個 PHP 擴充功能終止檔案的上傳",
    "No file uploaded or file size exceeds maximum of %s" : "沒有上傳檔案或檔案大小超過 %s 的最大值",
    "Invalid file type. Only JSON files are allowed." : "無效的檔案類型。僅允許 JSON 檔案。",
    "Invalid JSON data" : "無效的 JSON 數據",
    "Failed to import board" : "無法導入面板",
    "Cards due today" : "於今日到期的卡片",
    "Cards due tomorrow" : "於明日到期的卡片",
    "Upcoming cards" : "快將到期的卡片",
    "Load more" : "載入更多",
    "Welcome to Nextcloud Deck!" : "歡迎來到 Nextcloud Deck！",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "%s 已將 “%s” 中的卡片 “%s” 指派給您。",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} 已將 {deck-board} 中的卡片 {deck-card} 指派給您。",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "\"%1$s\" 中的卡片 \"%2$s\" 已到期。",
    "The card {deck-card} on {deck-board} has reached its due date." : "deck-board} 中的卡片{deck-card} 已到期。",
    "%s has mentioned you in a comment on \"%s\"." : "%s 在 “%s” 的意見中提到了您。",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} 在 {deckcard} 的意見中提到了您。",
    "The board \"%s\" has been shared with you by %s." : "面板 \"%s\" 已由 %s 分享給您。",
    "{user} has shared {deck-board} with you." : "{user} 分享了面板 {deck-board} 給您。",
    "The remote board %1$s has been shared with you by %2$s" : "遠端面板「%1$s」已由 %2$s 分享給您。",
    "Deck board" : "Deck 面板",
    "Owned by %1$s" : "由 %1$s 擁有",
    "Deck boards, cards and comments" : "Deck 面板、卡片和評論",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "來自 %1$s，位於 %2$s/%3$s，由 %4$s 擁有",
    "Create a new deck card" : "建立新 deck 卡片",
    "Card comments" : "卡片意見",
    "%s on %s" : "%s上%s",
    "Deck boards and cards" : "Deck 面板及卡片",
    "No data was provided to create an attachment." : "未能提供數據以創建附件",
    "Finished" : "完成",
    "To review" : "待審閱",
    "Action needed" : "需要採取行動",
    "Later" : "稍後處理",
    "copy" : "複製",
    "Read more inside" : "您可以在裡面深入了解更多",
    "Custom lists - click to rename!" : "自訂清單 - 點擊以重新命名！",
    "To Do" : "待辦",
    "In Progress" : "進行中",
    "Done" : "完成",
    "1. Open to learn more about boards and cards" : "1. 打開以了解更多有關面板和卡片的信息",
    "2. Drag cards left and right, up and down" : "2. 拖動卡片向左和向右、向上和向下",
    "3. Apply rich formatting and link content" : "3. 應用豐富的格式和連結內容",
    "4. Share, comment and collaborate!" : "4. 分享、評論和協作！",
    "Create your first card!" : "創建您的第一張卡片！",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "此評論超過%s個字符。\n作為附件添加到名稱為 %s 的卡片上。\n可於此 URL  存取：%s。",
    "Attachments" : "附件",
    "File" : "檔案",
    "date" : "日期",
    "Card not found" : "未找到卡片",
    "Path is already shared with this card" : "已和這張卡片分享了路徑",
    "Invalid date, date format must be YYYY-MM-DD" : "無效的日期，需為 YYYY-MM-DD 格式",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "你擁有的所有看板，包括堆疊、卡片、標籤、任務分配及留言",
    "Personal planning and team project organization" : "個人規劃和團隊項目組織",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck是一種看板式組織工具，旨在針對與Nextcloud集成的團隊進行個人計劃和項目組織。\n\n\n- 📥 添加您的任務到 card 及將之整理好\n- 📄 用 Markdown 寫下額外的註釋分享分享分享\n- 🔖 分配標籤以更好地組織您的工作\n- 👥 與您的團隊、朋友或家人分享\n- 📎 附加檔案並將其嵌入到您的 Markdown 描述中 \n- 💬 使用意見與您的團隊討論\n- ⚡ 在活動流中跟踪更改\n- 🚀 讓您的專案井井有條",
    "Add board" : "添加面板",
    "Card details" : "卡片詳情",
    "Select the board to link to a project" : "選擇要連結到一個項目的面板",
    "Search by board title" : "通過標題搜索面板",
    "Select board" : "選擇面板",
    "Move/copy card" : "移動/複製卡片",
    "Select a board" : "選擇一個面板",
    "No lists available" : "無清單可用",
    "Select a list" : "選擇一個列表",
    "Move card" : "移動卡片",
    "Copy card" : "複製卡片",
    "Select the card to link to a project" : "選擇要連結到一個項目的卡片",
    "Link to card" : "連結到卡片",
    "Select a card" : "選擇卡片",
    "Cancel" : "取消",
    "This board is read only" : "此面板是唯讀的",
    "Drop your files to upload" : "拖放您的檔案以上傳",
    "File already exists" : "檔案已存在",
    "A file with the name {filename} already exists." : "名稱為 {filename} 的檔案已存在。",
    "Do you want to overwrite it?" : "您確定要覆蓋嗎？",
    "Overwrite file" : "覆蓋檔案",
    "Keep existing file" : "保持已存在的檔案",
    "Add card" : "添加卡片",
    "Archived cards" : "已存檔卡片",
    "Add list" : "添加清單",
    "List name" : "清單名稱",
    "Clear search" : "清除搜尋",
    "Active filters" : "有效的過濾器",
    "Apply filter" : "應用過濾器",
    "Filter by tag" : "以標籤過濾",
    "Filter by assigned user" : "以指派用戶過濾",
    "Unassigned" : "未指派",
    "Filter by status" : "以狀態過濾",
    "Open and completed" : "開啟並完成",
    "Open" : "開啟",
    "Completed" : "已完成",
    "Filter by due date" : "以到期日過濾",
    "Overdue" : "逾期",
    "Next 24 hours" : "未來24小時",
    "Next 7 days" : "未來7曰",
    "Next 30 days" : "未來30曰",
    "No due date" : "無到期日",
    "Clear filter" : "清空過濾器",
    "View Modes" : "檢視模式",
    "Toggle View Modes" : "切換檢視模式",
    "Kanban view" : "Kanban 檢視",
    "Gantt view" : "甘特圖檢視",
    "Hide archived cards" : "隱藏已存檔的卡片",
    "Show archived cards" : "顯示已存檔的卡片",
    "Toggle compact mode" : "切換簡潔模式",
    "Hide card cover images" : "隱藏卡片封面圖像",
    "Show card cover images" : "顯示卡片封面圖像",
    "Open details" : "開啟細節",
    "Details" : "詳情",
    "Deck settings" : "看板設定",
    "General" : "一般",
    "Use bigger card view" : "使用較大的卡片視圖",
    "Appearance" : "外觀",
    "Show card ID badge" : "顯示卡片 ID 徽章",
    "Show boards in calendar/tasks" : "在日曆/任務中顯示面板",
    "Admin settings" : "管理員設定",
    "Limit board creation to some groups" : "限制面板只能由某些群組創建",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "這些群組之外的用戶將無法創建自己的面板，但仍然可以在與他們共享的面板上工作。",
    "Enable federation" : "啟用聯邦",
    "Board actions" : "面板操作",
    "Scroll sideways" : "橫向滾動",
    "Navigate between cards" : "在卡片間移動",
    "Close card details" : "關閉卡片詳細資訊",
    "Search" : "搜尋",
    "Show card filters" : "顯示卡片篩選條件",
    "Clear card filters" : "清除卡片篩選條件",
    "Show those shortcuts" : "顯示這些快捷鍵",
    "Card actions" : "卡片操作",
    "Open card details" : "打開卡片細節",
    "Edit the card title" : "編輯卡片標題",
    "Assign yourself to the current card" : "將自己指派給目前卡片",
    "Archive/unarchive the current card" : "封存/解除封存目前卡片",
    "Mark card as completed/not completed" : "將卡片標記為完成/未完成",
    "Open card menu" : "打開卡片選項單",
    "Currently present people" : "目前在場人仕",
    "Supported prefixes: {prefixes}. Wrap phrases in double quotes." : "支援的前綴：{prefixes}。請將短語用雙引號括起來。",
    "Search cards" : "搜尋卡片",
    "Loading board" : "正在加載面板",
    "Board not found" : "未找到面板",
    "Create a new list to add cards to this board" : "創建一張新清單來添加卡片到這個看板",
    "Sharing" : "正在分享",
    "Tags" : "標籤",
    "Deleted items" : "已刪除項目",
    "Activity" : "活動",
    "Deleted lists" : "已刪除的清單",
    "Undo" : "撤消",
    "Deleted cards" : "已刪除卡片",
    "Hours" : "小時",
    "Days" : "天",
    "Weeks" : "星期",
    "Months" : "月",
    "No cards yet" : "尚無卡片",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "在卡片設定開始日期及到期日期，以便於甘特圖中顯示",
    "Cards without dates ({count})" : "沒有日期的卡片（{count}）",
    "Failed to create share with {displayName}" : "無法為 {displayName} 創建分享",
    "Are you sure you want to transfer the board {title} to {user}?" : "您確定要將面板 {title} 轉讓給 {user} 嗎？",
    "Transfer the board." : "轉移面板。",
    "Transfer" : "轉移",
    "The board has been transferred to {user}" : "面板已轉讓給 {user}",
    "Failed to transfer the board to {user}" : "未能將面板轉移給 {user}",
    "Board owner" : "面板板主",
    "(Group)" : "（群組）",
    "(Team)" : "（團隊）",
    "(remote)" : "﹙遠端﹚",
    "Can edit" : "可以編輯",
    "Can share" : "可以分享",
    "Can manage" : "可以管理",
    "Owner" : "所有者",
    "Delete" : "刪除",
    "List deleted" : "清單已被刪除",
    "Edit list title" : "編輯清單標題",
    "Archive all cards" : "封存所有卡片",
    "Unarchive all cards" : "解除封存所有卡片",
    "Do not set cards as \"done\"" : "請勿將卡片標記為「已完成」",
    "Set cards as \"done\"" : "設定卡片為「已完成」",
    "Delete list" : "刪除清單",
    "Archive all cards in this list" : "封存此清單內的所有卡片",
    "Unarchive all cards in this list" : "解除封存所有此列表中的卡片",
    "Add a new card" : "添加一張新卡片",
    "Card name" : "卡片名稱",
    "title and color value must be provided" : "必須提供標題和顏色值",
    "Edit" : "編輯",
    "Add a new tag" : "添加新標籤",
    "Search boards" : "搜尋看板",
    "Board name" : "面板名",
    "Members" : "成員",
    "Assign to users/groups/team" : "分配給用戶／群組／團隊",
    "Assign a user to this card…" : "將此卡片指派給用戶...",
    "Select a user to assign to this card…" : "選擇一個用戶賦值給此卡片...",
    "File to share" : "要分享的檔案",
    "Invalid path selected" : "選中了無效的路徑",
    "Upload new files" : "上傳新檔案",
    "Share from Files" : "從檔案進行分享",
    "Pending share" : "等待分享",
    "Add this attachment" : "添加此附件",
    "Show in Files" : "顯示在檔案中",
    "Download" : "下載",
    "Remove attachment" : "移除附件",
    "Delete Attachment" : "刪除附件",
    "Restore Attachment" : "恢復附件",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "修改時間：{modifiedDate} ⸱ 創建時間：{createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "修改時間：{modifiedDate}\n創建時間：{createdDate}\n創建者：{owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "修改時間：{modifiedDate}\n創建時間：{createdDate}",
    "The title cannot be empty." : "標題不能為空。",
    "Cannot close unsaved card!" : "無法關閉無法儲存的卡片！",
    "Open in sidebar view" : "在側邊欄視圖中打開",
    "Open in bigger view" : "在較大視圖中打開",
    "Comments" : "意見",
    "Failed to load comments" : "無法加載評論",
    "No comments yet. Begin the discussion!" : "尚無意見，開始討論吧！",
    "The comment cannot be empty." : "意見不能為空。",
    "The comment cannot be longer than 1000 characters." : "意見不能超過 1000 個字符。",
    "Submit" : "遞交",
    "Created:" : "建立：",
    "In reply to" : "回覆",
    "Cancel reply" : "取消回覆",
    "Reply" : "回覆",
    "Update" : "更新",
    "Assign dependent cards" : "指派相依卡片",
    "Mark as done" : "標記為完成",
    "Mark as not done" : "標記為未完成",
    "Remove dependency" : "移除相依關係",
    "Add dependent card" : "新增相依卡片",
    "Assign a dependent card…" : "指派相依卡片…",
    "Select a dependent card…" : "選擇相依卡片…",
    "Write a description …" : "寫一段描述",
    "The description has been changed by another user." : "描述已被另一個用戶修改。",
    "Could not save description" : "無法保存描述",
    "Description" : "描述",
    "(Unsaved)" : "（未保存的）",
    "(Saving…)" : "（保存中...）",
    "Formatting help" : "格式化幫助",
    "Edit description" : "編輯描述",
    "View description" : "查看描述",
    "Add Attachment" : "添加附件",
    "Choose attachment" : "選擇附件",
    "Select Date" : "選擇日期",
    "Later today – {timeLocale}" : "今天稍後 – {timeLocale}",
    "Set due date for later today" : "設定今天稍後的到期日",
    "Tomorrow – {timeLocale}" : "明日 – {timeLocale}",
    "Set due date for tomorrow" : "設定截止日期為明日",
    "This weekend – {timeLocale}" : "本週末 – {timeLocale}",
    "Set due date for this weekend" : "設定本週末的到期日",
    "Next week – {timeLocale}" : "下星期 – {timeLocale}",
    "Set due date for next week" : "設定下星期的到期日",
    "Assign a due date to this card…" : "為此卡片指派到期日 …",
    "Set a due date" : "設置到期日",
    "Add due date" : "添加到期日",
    "Choose a date" : "選擇日期",
    "Remove due date" : "移除到期日",
    "Due at:" : "到期日：",
    "Not done" : "未完成",
    "Unarchive card" : "取消對卡片的封存",
    "Archive card" : "封存卡片",
    "Assign a start date to this card…" : "為此卡片設定開始日期 …",
    "Set a start date" : "設定開始日期",
    "Add start date" : "新增開始日期",
    "Remove start date" : "移除開始日期",
    "Assign a tag to this card…" : "為該卡片分配標籤…",
    "Select or create a tag…" : "選擇或創建標籤 ...",
    "Create a new tag:" : "創建新標籤：",
    "(group)" : "（群組）",
    "{count} comments, {unread} unread" : "{count} 條評論，{unread} 條未讀",
    "Todo items" : "待辦事項",
    "Edit card title" : "編輯卡片標題",
    "Open link" : "打開連結",
    "Card deleted" : "卡片被刪除",
    "Change card color" : "更新卡片顏色",
    "Edit title" : "編輯標題",
    "Assign to me" : "指派給我",
    "Unassign myself" : "自己解除指派",
    "Delete card" : "刪除卡片",
    "All boards" : "全部面板",
    "Archived boards" : "已封存面板",
    "Shared with you" : "收到的分享",
    "Cancel edit" : "取消編輯",
    "Save board" : "保存面板",
    "Board {0} deleted" : "面板 {0} 已被刪除",
    "All cards" : "所有卡片",
    "Only assigned cards" : "僅指派了的卡片",
    "No reminder" : "無提醒",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "你確定你要刪除 {title} 面板嗎？面板內所有數據(包括已封存的卡片)都將因此被刪除。",
    "Delete the board?" : "是否刪除面板？",
    "Are you sure you want to leave the board {title}?" : "您確定您想要離開看板 {title} 嗎？",
    "Leave the board?" : "離開面板？",
    "Leave" : "離開",
    "Failed to leave the board" : "無法離開面板",
    "Exporting board..." : "正在匯出面板 ...",
    "An error occurred" : "發生錯誤",
    "Board details" : "面板詳情",
    "Edit board" : "編輯面板",
    "Clone board" : "複製面板",
    "Unarchive board" : "取消對面板的封存",
    "Archive board" : "封存面板",
    "Export board" : "匯出面板",
    "Turn on due date reminders" : "開啟到期日提醒",
    "Turn off due date reminders" : "關閉到期日提醒",
    "Remove as default board" : "移除默認面板",
    "Set as default board" : "設定為默認面板",
    "Due date reminders" : "到期日提醒",
    "Assigned cards" : "分配的卡片",
    "No notifications" : "無通知",
    "Delete board" : "刪除面板",
    "Leave board" : "離開面板",
    "Importing board..." : "正在導入面板 ...",
    "Board imported successfully" : "面板成功導入",
    "Import board" : "導入面板",
    "Clone {boardTitle}" : "複製 {boardTitle}",
    "Clone cards" : "複製 cards",
    "Clone assignments" : "複製任務分配",
    "Clone labels" : "複製標籤",
    "Clone due dates" : "複製期日期",
    "Advanced options" : "進階選項",
    "Move all cards to the first list" : "將所有卡片移動到第一個清單",
    "Restore archived cards" : "還原已封存的卡片",
    "Clone" : "複製",
    "Export {boardTitle}" : "匯出 {boardTitle}",
    "Export as JSON" : "匯出為 JSON",
    "Export as CSV" : "匯出為 CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "注意：僅支援 JSON 格式以便重新導入到 Deck 應用程式。",
    "Export" : "匯出",
    "Loading filtered view" : "正在加載已過濾視圖",
    "Search for {searchQuery} in other boards" : "在其他看板中搜尋 {searchQuery}",
    "Search for {searchQuery} in all boards" : "在所有面板中搜索 {searchQuery}",
    "No results found" : "未找到結果",
    "Deck board {name}\n* Last modified on {lastMod}" : "Deck 面板 {name}\n* 最後修改於  {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* 創建於 {created}\n* 最後修改於 {lastMod}\n* {nbAttachments} 個附件\n* {nbComments} 則留言",
    "{nbCards} cards" : "{nbCards} 張卡片",
    "Due on {date}" : "於 {date} 到期",
    "{stack} in {board}" : "{board} 中的 {stack}",
    "Click to expand description" : "點擊展開描述",
    "Click to expand comment" : "點擊展開評論",
    "Create card" : "建立卡片",
    "Create a new card" : "建立新卡片",
    "Card title" : "卡片標題",
    "Creating the new card …" : "正在建立新卡片...",
    "Card \"{card}\" was added to \"{board}\"" : "\"{card}\" 已添加到 \"{board}\"",
    "Open card" : "打開卡片",
    "Close" : "關閉",
    "No upcoming cards" : "沒有快將到期的卡片",
    "upcoming cards today" : "今天接下來的卡片",
    "upcoming cards tomorrow" : "明日的即將到來的卡片",
    "upcoming cards" : "快將到期的卡片",
    "New card" : "新卡片",
    "Link to a board" : "連結到面板",
    "Link to a card" : "連結到卡片",
    "Create a card" : "建立卡片",
    "Message from {author} in {conversationName}" : "來自 {conversationName} 中的 {author} 之消息",
    "Something went wrong" : "發生錯誤了",
    "Failed to upload {name}" : "上傳 {name} 失敗",
    "Maximum file size of {size} exceeded" : "檔案大小 {size} 超出最大限制",
    "Assigned users" : "指派的用戶",
    "Due date" : "到期日期",
    "Created" : "建立於",
    "Modified" : "修改於",
    "Error creating the share" : "創建分享出錯",
    "Share with a Deck card" : "用一張 Deck 卡片進行分享 ",
    "Share {file} with a Deck card" : "用一張 Deck 卡片分享 {file}",
    "Share" : "分享",
    "The card \"%s\" on \"%s\" has reached its due date." : "\"%s\" 中的卡片 \"%s\" 已到期。",
    "Share board with a user, group or team …" : "與用戶、群組或團隊分享面板 …",
    "Searching for users, groups and teams …" : "搜尋使用者、群組與團隊 …",
    "No participants found" : "未找到參與者",
    "Save" : "保存",
    "seconds ago" : "幾秒前",
    "Today" : "今日",
    "Tomorrow" : "明日",
    "No due" : "沒有到期的"
},
"nplurals=1; plural=0;");
