OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Vde. creou o novo taboleiro {board}",
    "{user} has created a new board {board}" : "{user}  creou o novo taboleiro {board}",
    "You have deleted the board {board}" : "Vde. eliminou o taboleiro {board}",
    "{user} has deleted the board {board}" : "{user} eliminou o taboleiro {board}",
    "You have restored the board {board}" : "Vde. restaurou o taboleiro {board}",
    "{user} has restored the board {board}" : "{user} restaurou o taboleiro {board}",
    "You have shared the board {board} with {acl}" : "Vde. compartiu o taboleiro {board} con {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} compartiu o taboleiro {board} con {acl}",
    "You have removed {acl} from the board {board}" : "Vde. retirou a {acl} do taboleiro {board}",
    "{user} has removed {acl} from the board {board}" : "{user} retirou a {acl} do taboleiro {board}",
    "You have renamed the board {before} to {board}" : "Vde. renomeou o taboleiro {before} como {board}",
    "{user} has renamed the board {before} to {board}" : "{user} renomeou o taboleiro {before} como {board}",
    "You have archived the board {board}" : "Vde. arquivou o taboleiro {board}",
    "{user} has archived the board {before}" : "{user} arquivou o taboleiro {before}",
    "You have unarchived the board {board}" : "Vde. desarquivou o taboleiro {board}",
    "{user} has unarchived the board {before}" : "{user} desarquivou o taboleiro {before}",
    "You have created a new list {stack} on board {board}" : "Vde. creou unha nova lista {stack} no taboleiro {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} creou unha nova lista {stack} no taboleiro {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Vde. renomeou a lista {before} no taboleiro {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} renomeou a lista {before} como {stack} no taboleiro {board}",
    "You have deleted list {stack} on board {board}" : "Vde. eliminou a lista {stack} do taboleiro {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} eliminou a lista {stack} do taboleiro {board}",
    "You have created card {card} in list {stack} on board {board}" : "Vde. creou a tarxeta {card} na lista {stack} no taboleiro {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} creou a tarxeta {card} na lista {stack} no taboleiro {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Vde. eliminou a tarxeta {card} na lista {stack} no taboleiro {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} eliminou a tarxeta {card} na lista {stack} no taboleiro {board}",
    "You have renamed the card {before} to {card}" : "Vde. renomeou a tarxeta {before} a {card}",
    "{user} has renamed the card {before} to {card}" : "{user} renomeou a tarxeta {before} a {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Vde. engadiu a descrición á tarxeta {card} na lista {stack} no taboleiro {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} engadiu a descrición á tarxeta {card} na lista {stack} no taboleiro {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Vde. actualizou a descrición da tarxeta {card} na lista {stack} no taboleiro {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} actualizou a descrición da tarxeta {card} na lista {stack} no taboleiro {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Vde. arquivou a tarxeta {card} na lista {stack} no taboleiro {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} arquivou a tarxeta {card} na lista {stack} no taboleiro {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Vde. desarquivou a tarxeta {card} na lista {stack} no taboleiro {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} desarquivou a tarxeta {card} na lista {stack} no taboleiro {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Vde. marcou a tarxeta {card} como feita na lista {stack} no taboleiro {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} marcou a tarxeta {card} como feita na lista {stack} no taboleiro {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Vde. marcou a tarxeta {card} como sen facer na lista {stack} no taboleiro {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} marcou a tarxeta {card} como sen facer na lista {stack} no taboleiro {board}",
    "You have removed the due date of card {card}" : "Vde. retirou a caducidade da tarxeta {card}",
    "{user} has removed the due date of card {card}" : "{user} retirou a caducidade da tarxeta {card}",
    "You have set the due date of card {card} to {after}" : "Vde. definiu a caducidade da tarxeta {card} a {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} definiu a caducidade da tarxeta {card} a {after}",
    "You have updated the due date of card {card} to {after}" : "Vde. actualizou a caducidade da tarxeta {card} a {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} actualizou a caducidade da tarxeta {card} a {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Vde. engadiu a etiqueta {label} á tarxeta {card} na lista {stack} no taboleiro {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} engadiu a etiqueta {label} á tarxeta {card} na lista {stack} no taboleiro {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Vde. retirou a etiqueta {label} da tarxeta {card} na lista {stack} no taboleiro {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} retirou a etiqueta {label} da tarxeta {card} na lista {stack} no taboleiro {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Vde. asignou a {assigneduser} á tarxeta {card} no taboleiro {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} asignou a {assigneduser} á tarxeta {card} no taboleiro {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Vde. desasignou a {assigneduser} da tarxeta {card} no taboleiro {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} desasignou a {assigneduser} da tarxeta {card} no taboleiro {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Vde. moveu a tarxeta {card} desde a lista {stackBefore} cara a lista {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} moveu a tarxeta {card} desde a lista {stackBefore} cara a lista {stack}",
    "You have added the attachment {attachment} to card {card}" : "Vde. engadiu o anexo  {attachment} na tarxeta {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} engadiu o anexo {attachment} na tarxeta {card}",
    "You have updated the attachment {attachment} on card {card}" : "Vde. actualizou o anexo  {attachment} na tarxeta {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} actualizou o anexo {attachment} na tarxeta {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Vde. eliminou o anexo  {attachment} da tarxeta {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} eliminou o anexo {attachment} da tarxeta {card}",
    "You have restored the attachment {attachment} to card {card}" : "Vde. restaurou o anexo  {attachment} na tarxeta {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} restaurou o anexo {attachment} da tarxeta {card}",
    "You have commented on card {card}" : "Vde. comentou na tarxeta {card}",
    "{user} has commented on card {card}" : "{user} comentou na tarxeta {card}",
    "Deck" : "Gabeta",
    "Changes in the <strong>Deck app</strong>" : "Cambios na <strong>aplicación Gabeta</strong>",
    "A <strong>board, list or card</strong> was changed" : "Cambiou un <strong>taboleiro, lista ou tarxeta</strong>",
    "A <strong>comment</strong> was created on a card" : "Creouse un <strong>comentario</strong> nunha tarxeta",
    "A <strong>card description</strong> has been changed" : "Cambiou a <strong>descrición dunha tarxeta</strong>",
    "The file was uploaded" : "O ficheiro foi enviado",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "O ficheiro enviado excede a directiva indicada por upload_max_filesize de php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "O ficheiro enviado excede da directiva MAX_FILE_SIZE especificada no formulario HTML",
    "The file was only partially uploaded" : "O ficheiro só foi parcialmente enviado",
    "No file was uploaded" : "Non se enviou ningún ficheiro",
    "Missing a temporary folder" : "Falta un cartafol temporal",
    "Could not write file to disk" : "Non foi posíbel escribir o ficheiro no disco",
    "A PHP extension stopped the file upload" : "Unha extensión PHP detivo o envío de ficheiros",
    "No file uploaded or file size exceeds maximum of %s" : "Non se enviou ningún ficheiro ou o tamaño do ficheiro supera o máximo de %s",
    "Invalid file type. Only JSON files are allowed." : "O tipo de ficheiro non é válido. Só se permiten ficheiros JSON.",
    "Invalid JSON data" : "Datos JSON non válidos",
    "Failed to import board" : "Produciuse un fallo ao importar o taboleiro",
    "Cards due today" : "Tarxetas para hoxe",
    "Cards due tomorrow" : "Tarxetas para mañá",
    "Upcoming cards" : "Tarxetas próximas",
    "Load more" : "Cargar máis",
    "Welcome to Nextcloud Deck!" : "Dámoslle a benvida á Gabeta de Nextcloud",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "A tarxeta «%s» en «%s» foi asignada a Vde. por %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} asignoulle a tarxeta {deck-card} en {deck-board} a Vde.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "A tarxeta «%1$s» en «%2$s» acadou a súa data de caducidade.",
    "The card {deck-card} on {deck-board} has reached its due date." : "A tarxeta {deck-card} en {deck-board} chegou á súa data de caducidade.",
    "%s has mentioned you in a comment on \"%s\"." : "%s mencionouno a Vde. nun comentario en «%s».",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} mencionouno a Vde. nun comentario en {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "O taboleiro «%s» foi compartido con Vde. por %s.",
    "{user} has shared {deck-board} with you." : "{user} compartiu {deck-board} con vostede.",
    "The remote board %1$s has been shared with you by %2$s" : "O taboleiro remoto %1$s foi compartido con Vde. por %2$s",
    "Deck board" : "Taboleiro da Gabeta",
    "Owned by %1$s" : "Propiedade de %1$s",
    "Deck boards, cards and comments" : "Taboleiros da Gabeta, tarxetas e comentarios",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "De %1$s, en %2$s/%3$s, propiedade de %4$s",
    "Create a new deck card" : "Crear unha nova tarxeta de Gabeta",
    "Card comments" : "Comentarios da tarxeta",
    "%s on %s" : "%s en %s",
    "Deck boards and cards" : "Taboleiros da Gabeta e tarxetas",
    "No data was provided to create an attachment." : "Non se forneceu ningún dato para crear un anexo.",
    "Finished" : "Rematado",
    "To review" : "Para revisar",
    "Action needed" : "Necesita unha acción",
    "Later" : "Após",
    "copy" : "copiar",
    "Read more inside" : "Ler máis dentro",
    "Custom lists - click to rename!" : "Listas personalizadas, prema para cambiar o nome!",
    "To Do" : "Tarefas pendentes",
    "In Progress" : "En proceso",
    "Done" : "Feito",
    "1. Open to learn more about boards and cards" : "1. Abra para saber máis sobre taboleiros e tarxetas",
    "2. Drag cards left and right, up and down" : "2. Arrastre as tarxetas cara á esquerda e á dereita, cara arriba e abaixo",
    "3. Apply rich formatting and link content" : "3. Aplique un formato mellorado e ligue a contidos",
    "4. Share, comment and collaborate!" : "4. Comparta, comente e colabore!",
    "Create your first card!" : "Crear a primeira tarxeta:",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Este comentario ten máis de %s caracteres.\nEngadido como anexo á tarxeta co nome %s.\nAccesíbel no URL: %s.",
    "Attachments" : "Anexos",
    "File" : "Ficheiro",
    "date" : "data",
    "Card not found" : "Non se atopou a tarxeta",
    "Path is already shared with this card" : "A ruta xa está compartida con esta tarxeta",
    "Invalid date, date format must be YYYY-MM-DD" : "Data incorrecta, o formato da data debe ser AAAA-MM-DD",
    "Personal planning and team project organization" : "Planificación persoal e organización de proxectos de equipo",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Gabeta é unha ferramenta de organización de estilo kanban dirixida a planificación persoal e organización de proxectos para equipos integrados con Nextcloud. \n\n\n-- 📥 Engada as súas tarefas ás tarxetas e fagas ordenadas\n- 📄 Escriba notas adicionais en Markdown\n- 🔖 Asigne etiquetas para unha mellor organización\n- 👥 Comparta co seu equipo, amigos ou a súa familia\n- 📎 Anexe ficheiros e integreos na súa descrición de Markdown\n- 💬 Debata co seu equipo usando os comentarios\n- ⚡ Faga un seguimento dos cambios no fluxo de actividade\n- 🚀 Teña o seu proxecto organizado",
    "Add board" : "Engadir taboleiro",
    "Card details" : "Detalles da tarxeta",
    "Select the board to link to a project" : "Seleccione o taboleiro para ligar a un proxecto",
    "Search by board title" : "Buscar polo título do taboleiro",
    "Select board" : "Seleccionar taboleiro",
    "Move/copy card" : "Mover/copiar a tarxeta",
    "Select a board" : "Seleccione un taboleiro",
    "No lists available" : "Non hai listas dispoñíbeis",
    "Select a list" : "Seleccionar unha lista",
    "Move card" : "Mover a tarxeta",
    "Copy card" : "Copiar a tarxeta",
    "Select the card to link to a project" : "Seleccione a tarxeta para ligar a un proxecto",
    "Link to card" : "Ligazón á tarxeta",
    "Select a card" : "Seleccione unha tarxeta",
    "Cancel" : "Cancelar",
    "This board is read only" : "Este taboleiro é só de lectura",
    "Drop your files to upload" : "Arrastre os seus ficheiros para envialos",
    "File already exists" : "O ficheiro xa existe",
    "A file with the name {filename} already exists." : "Xa existe un ficheiro co nome {filename}.",
    "Do you want to overwrite it?" : "Quere sobrescribilo?",
    "Overwrite file" : "Sobrescribir o ficheiro",
    "Keep existing file" : "Conservar o ficheiro existente",
    "Add card" : "Engadir tarxeta",
    "Archived cards" : "Tarxetas arquivadas",
    "Add list" : "Engadir unha lista",
    "List name" : "Nome da lista",
    "Clear search" : "Limpar a busca",
    "Active filters" : "Filtros activos",
    "Apply filter" : "Aplicar filtro",
    "Filter by tag" : "Filtrar pola etiqueta",
    "Filter by assigned user" : "Filtrar polo usuario asignado",
    "Unassigned" : "Sen asignar",
    "Filter by status" : "Filtrar polo estado",
    "Open and completed" : "Aberto ou rematado",
    "Open" : "Aberto",
    "Completed" : "Completado",
    "Filter by due date" : "Filtrar pola data de caducidade",
    "Overdue" : "Caducado",
    "Next 24 hours" : "Próximas 24 horas",
    "Next 7 days" : "Próximos 7 días",
    "Next 30 days" : "Próximos 30 días",
    "No due date" : "Sen data de caducidade",
    "Clear filter" : "Limpar o filtro",
    "View Modes" : "Modos de vista",
    "Toggle View Modes" : "Alternar os modos de vista",
    "Hide archived cards" : "Agochar as tarxetas arquivadas",
    "Show archived cards" : "Amosar as tarxetas arquivadas",
    "Toggle compact mode" : "Alternar o modo compacto",
    "Hide card cover images" : "Agochar as imaxes da portada da tarxeta",
    "Show card cover images" : "Amosar as imaxes da portada da tarxeta",
    "Open details" : "Abrir detalles",
    "Details" : "Detalles",
    "Deck settings" : "Axustes da Gabeta",
    "General" : "Xeral",
    "Use bigger card view" : "Usa a vista de tarxeta máis grande",
    "Appearance" : "Aparencia",
    "Show card ID badge" : "Amosar o distintivo de identificación da tarxeta",
    "Show boards in calendar/tasks" : "Amosar taboleiros no calendario/tarefas",
    "Admin settings" : "Axustes de administración",
    "Limit board creation to some groups" : "Limitar a creación do taboleiro a algúns grupos",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Os usuarios alleos a eses grupos non poderán crear os seus propios taboleiros, mais aínda poderán traballar en taboleiros que se compartiron con eles.",
    "Enable federation" : "Activar a federación",
    "Board actions" : "Accións de taboleiro",
    "Scroll sideways" : "Desprazamento lateral",
    "Navigate between cards" : "Navega entre as tarxetas",
    "Close card details" : "Pechar os detalles da tarxeta",
    "Search" : "Busca",
    "Show card filters" : "Amosar os filtros de tarxeta",
    "Clear card filters" : "Limpar os filtros de tarxeta",
    "Show those shortcuts" : "Amosar eses atallos",
    "Card actions" : "Accións de tarxeta",
    "Open card details" : "Abrir os detalles da tarxeta",
    "Edit the card title" : "Editar o título da tarxeta",
    "Assign yourself to the current card" : "Atribúase a tarxeta a Vde. mesmo",
    "Archive/unarchive the current card" : "Arquivar/desarquivar a tarxeta actual",
    "Mark card as completed/not completed" : "Marcar a tarxeta como completada/sen completar",
    "Open card menu" : "Abrir o menú da tarxeta",
    "Currently present people" : "Persoas presentes na actualidade",
    "Loading board" : "Cargando o taboleiro",
    "Board not found" : "Non se atopou o taboleiro",
    "Create a new list to add cards to this board" : "Crear unha nova lista para engadir tarxetas a este taboleiro",
    "Sharing" : "Compartindo",
    "Tags" : "Etiquetas",
    "Deleted items" : "Elementos eliminados",
    "Activity" : "Actividade",
    "Deleted lists" : "Listas eliminadas",
    "Undo" : "Desfacer",
    "Deleted cards" : "Eliminar tarxetas",
    "Hours" : "Horas",
    "Days" : "Días",
    "Weeks" : "Semanas",
    "Months" : "Meses",
    "No cards yet" : "Aínda non hai ningunha tarxeta",
    "Failed to create share with {displayName}" : "Produciuse un fallo ao crear o uso compartido con {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Confirma que quere transferir o taboleiro {title} a {user}?",
    "Transfer the board." : "Transferir o taboleiro.",
    "Transfer" : "Transferencia",
    "The board has been transferred to {user}" : "O taboleiro foi transferido a {user}",
    "Failed to transfer the board to {user}" : "Produciuse un fallo ao transferir o taboleiro a {user}",
    "Board owner" : "Propietariio do taboleiro",
    "(Group)" : "(Grupo)",
    "(Team)" : "(Equipo)",
    "(remote)" : "(remoto)",
    "Can edit" : "Pode editar",
    "Can share" : "Pode compartir",
    "Can manage" : "Pode xestionar",
    "Owner" : "Propietario",
    "Delete" : "Eliminar",
    "List deleted" : "Lista eliminada",
    "Edit list title" : "Editar o título da lista",
    "Archive all cards" : "Arquivar todas as tarxetas",
    "Unarchive all cards" : "Desarquivar todas as tarxetas",
    "Delete list" : "Eliminar lista",
    "Archive all cards in this list" : "Arquivar todas as tarxetas desta lista",
    "Unarchive all cards in this list" : "Desarquivar todas as tarxetas desta lista",
    "Add a new card" : "Engadir unha nova tarxeta",
    "Card name" : "Nome da tarxeta",
    "title and color value must be provided" : "debe indicar o título e o valor da cor",
    "Edit" : "Editar",
    "Add a new tag" : "Engadir unha nova etiqueta",
    "Board name" : "Nome do taboleiro",
    "Members" : "Membros",
    "Assign to users/groups/team" : "Asignar a usuarios/grupos/equipos",
    "Assign a user to this card…" : "Asignar un usuario a esta tarxeta…",
    "Select a user to assign to this card…" : "Seleccionar un usuario para asignarlle a esta tarxeta…",
    "File to share" : "Ficheiro para compartir",
    "Invalid path selected" : "Seleccionou unha ruta incorrecta.",
    "Upload new files" : "Enviar novos ficheiros",
    "Share from Files" : "Compartir desde «Ficheiros»",
    "Pending share" : "Compartición pendente",
    "Add this attachment" : "Engadir este anexo",
    "Show in Files" : "Amosar en Ficheiros",
    "Download" : "Descargar",
    "Remove attachment" : "Retirar o anexo",
    "Delete Attachment" : "Eliminar o anexo",
    "Restore Attachment" : "Restaurar o anexo",
    "The title cannot be empty." : "O título non pode estar baleiro.",
    "Cannot close unsaved card!" : "Non é posíbel pechar unha tarxeta non gardada!",
    "Open in sidebar view" : "Abrir na vista da barra lateral",
    "Open in bigger view" : "Abrir nunha vista máis grande",
    "Comments" : "Comentarios",
    "Failed to load comments" : "Produciuse un fallo ao cargar os comentarios",
    "No comments yet. Begin the discussion!" : "Aínda non hai comentarios. Comece o debate!",
    "The comment cannot be empty." : "O comentario non pode estar baleiro",
    "The comment cannot be longer than 1000 characters." : "O comentario non pode ter máis de 1000 caracteres.",
    "Submit" : "Enviar",
    "Created:" : "Creado:",
    "In reply to" : "En resposta a",
    "Cancel reply" : "Cancelar a resposta",
    "Reply" : "Responder",
    "Update" : "Actualizar",
    "Mark as done" : "Marcar como feito",
    "Mark as not done" : "Marcar como sen facer",
    "Write a description …" : "Escriba unha descrición…",
    "Could not save description" : "Non foi posíbel gardar a descrición",
    "Description" : "Descrición",
    "(Unsaved)" : "(Sen gardar)",
    "(Saving…)" : "(Gardando…)",
    "Formatting help" : "Axuda de formatado",
    "Edit description" : "Editar a descrición",
    "View description" : "Ver a descrición",
    "Add Attachment" : "Engadir un anexo",
    "Choose attachment" : "Escoller o anexo",
    "Select Date" : "Seleccione a data",
    "Later today – {timeLocale}" : "Hoxe máis tarde – {timeLocale}",
    "Set due date for later today" : "Definir a data de remate para máis tarde hoxe",
    "Tomorrow – {timeLocale}" : "Mañá – {timeLocale}",
    "Set due date for tomorrow" : "Definir a data de remate para mañá",
    "This weekend – {timeLocale}" : "Esta semana – {timeLocale}",
    "Set due date for this weekend" : "Definir a data de remate para esta fin de semana",
    "Next week – {timeLocale}" : "Semana seguinte – {timeLocale}",
    "Set due date for next week" : "Definir a data de remate para a semana seguinte",
    "Assign a due date to this card…" : "Asignar unha data de caducidade a esta tarxeta…",
    "Set a due date" : "Definir a data de caducidade",
    "Add due date" : "Engadir a data de remate",
    "Choose a date" : "Escoller a data",
    "Remove due date" : "Retirar a data de caducidade",
    "Due at:" : "Rematar o:",
    "Not done" : "Sen facer",
    "Unarchive card" : "Desarquivar a tarxeta",
    "Archive card" : "Arquivar a tarxeta",
    "Assign a tag to this card…" : "Asignar unha etiqueta a esta tarxeta…",
    "Select or create a tag…" : "Seleccionar ou crear unha etiqueta…",
    "Create a new tag:" : "Crear unha nova etiqueta:",
    "(group)" : "(grupo)",
    "{count} comments, {unread} unread" : "{count} comentarios, {unread} sen ler",
    "Todo items" : "Elementos pendentes",
    "Edit card title" : "Editar o título da tarxeta",
    "Open link" : "Abrir a ligazón",
    "Card deleted" : "Tarxeta eliminada",
    "Edit title" : "Editar o título",
    "Assign to me" : "Asignarme",
    "Unassign myself" : "Desasignarme",
    "Delete card" : "Eliminar tarxeta",
    "All boards" : "Todos os taboleiros",
    "Archived boards" : "Taboleiros arquivados",
    "Shared with you" : "Compartido con Vde.",
    "Cancel edit" : "Cancelar a edición",
    "Save board" : "Gardar o taboleiro",
    "Board {0} deleted" : "Eliminouse o taboleiro {0}",
    "All cards" : "Todas as tarxeta",
    "Only assigned cards" : "Só as tarxetas asignadas",
    "No reminder" : "Non hai lembretes",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Confirma que quere eliminar o taboleiro {title}? Isto eliminará todos os datos deste taboleiro, incluídas as tarxetas arquivadas.",
    "Delete the board?" : "Eliminar o taboleiro?",
    "Are you sure you want to leave the board {title}?" : "Confirma que quere abandonar o taboleiro {title}?",
    "Leave the board?" : "Abandonar o taboleiro?",
    "Leave" : "Abandonar",
    "Failed to leave the board" : "Produciuse un erro ao abandonar o taboleiro",
    "Exporting board..." : "Exportando o taboleiro…",
    "An error occurred" : "Produciuse un erro",
    "Board details" : "Detalles do taboleiro",
    "Edit board" : "Editar taboleiro",
    "Clone board" : "Clonar taboleiro",
    "Unarchive board" : "Desarquivar taboleiro",
    "Archive board" : "Arquivar taboleiro",
    "Export board" : "Exportar taboleiro",
    "Turn on due date reminders" : "Activar os lembretes de data de caducidade",
    "Turn off due date reminders" : "Desctivar os lembretes de data de caducidade",
    "Remove as default board" : "Retirar como taboleiro predeterminado",
    "Set as default board" : "Definir como taboleiro predeterminado",
    "Due date reminders" : "Lembretes de data de caducidade",
    "Assigned cards" : "Tarxetas asignadas",
    "No notifications" : "Non hai notificacións",
    "Delete board" : "Eliminar taboleiro",
    "Leave board" : "Abandonar o taboleiro",
    "Importing board..." : "Importando o taboleiro…",
    "Board imported successfully" : "O taboleiro foi importado correctamente",
    "Import board" : "Importar taboleiro",
    "Clone {boardTitle}" : "Clonar {boardTitle}",
    "Clone cards" : "Clonar tarxetas",
    "Clone assignments" : "Clonar as asignacións",
    "Clone labels" : "Clonar as etiquetas",
    "Clone due dates" : "Clonar as datas de caducidade",
    "Advanced options" : "Opcións avanzadas",
    "Move all cards to the first list" : "Mover todas as tarxetas á primeira lista",
    "Restore archived cards" : "Restaurar as tarxetas arquivadas",
    "Clone" : "Clonar",
    "Export {boardTitle}" : "Exportar {boardTitle}",
    "Export as JSON" : "Exportar como JSON",
    "Export as CSV" : "Exportar como CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Nota: Só se admite o formato JSON para importar de novo á aplicación Gabeta.",
    "Export" : "Exportar",
    "Loading filtered view" : "Cargando a vista filtrada",
    "Search for {searchQuery} in other boards" : "Buscar {searchQuery} noutros taboleiros",
    "Search for {searchQuery} in all boards" : "Buscar {searchQuery} en todos os taboleiros",
    "No results found" : "Non se atopou ningún resultado",
    "Deck board {name}\n* Last modified on {lastMod}" : "Taboleiro da Gabeta {name}\n* Última modificación en {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Creado o {created}\n* Última modificación o {lastMod}\n* {nbAttachments} anexos\n* {nbComments} comentarios",
    "{nbCards} cards" : "{nbCards} tarxetas",
    "Due on {date}" : "Caduca o {date}",
    "{stack} in {board}" : "{stack} en {board}",
    "Click to expand description" : "Prema para despregar a descrición",
    "Click to expand comment" : "Prema para despregar o comentario",
    "Create card" : "Crear tarxeta",
    "Create a new card" : "Crear unha nova tarxeta",
    "Card title" : "Título da tarxeta",
    "Creating the new card …" : "Creando unha nova tarxeta…",
    "Card \"{card}\" was added to \"{board}\"" : "A tarxeta «{card}» foi engdida a «{board}»",
    "Open card" : "Abrir a tarxeta",
    "Close" : "Pechar",
    "No upcoming cards" : "Non hai tarxetas próximas",
    "upcoming cards today" : "próximas tarxetas de hoxe",
    "upcoming cards tomorrow" : "próximas tarxetas de mañá",
    "upcoming cards" : "tarxetas próximas",
    "New card" : "Nova tarxeta",
    "Link to a board" : "Ligazón a un taboleiro",
    "Link to a card" : "Ligazón a unha tarxeta",
    "Create a card" : "Crear unha tarxeta",
    "Message from {author} in {conversationName}" : "Mensaxe de {author} en {conversationName}",
    "Something went wrong" : "Algo foi mal",
    "Failed to upload {name}" : "Produciuse un fallo ao enviar {name}",
    "Maximum file size of {size} exceeded" : "Excedeuse o tamaño máximo de ficheiro de {size}",
    "Assigned users" : "Usuarios asignados",
    "Due date" : "Data de remate",
    "Created" : "Creado",
    "Modified" : "Modificado",
    "Error creating the share" : "Produciuse un erro ao crear a compartición",
    "Share with a Deck card" : "Compartir cunha tarxeta de Gabeta",
    "Share {file} with a Deck card" : "Compartir {file} cunha tarxeta de Gabeta",
    "Share" : "Compartir",
    "The card \"%s\" on \"%s\" has reached its due date." : "A tarxeta «%s» en «%s» acadou a súa data de caducidade.",
    "Share board with a user, group or team …" : "Compartir taboleiro cun usuario, grupo ou equipo…",
    "Searching for users, groups and teams …" : "Buscando por usuarios, grupos e equipos…",
    "No participants found" : "Non se atopou ningún participante",
    "Save" : "Gardar",
    "seconds ago" : "segundos atrás",
    "Today" : "Hoxe",
    "Tomorrow" : "Mañá",
    "No due" : "Sen caducidade"
},
"nplurals=2; plural=(n != 1);");
