OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Stvorili ste novu ploču {board}",
    "{user} has created a new board {board}" : "{user} je stvorio novu ploču {board}",
    "You have deleted the board {board}" : "Izbrisali ste ploču {board}",
    "{user} has deleted the board {board}" : "{user} je izbrisao ploču {board}",
    "You have restored the board {board}" : "Vratili ste ploču {board}",
    "{user} has restored the board {board}" : "{user} je vratio ploču {board}",
    "You have shared the board {board} with {acl}" : "Podijelili ste ploču {board} s {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} je podijelio ploču {board} s {acl}",
    "You have removed {acl} from the board {board}" : "Uklonili ste {acl} s ploče {board}",
    "{user} has removed {acl} from the board {board}" : "{user} je uklonio {acl} s ploče {board}",
    "You have renamed the board {before} to {board}" : "Preimenovali ste ploču {before} u {board}",
    "{user} has renamed the board {before} to {board}" : "{user} je preimenovao ploču {before} u {board}",
    "You have archived the board {board}" : "Arhivirali ste ploču {board}",
    "{user} has archived the board {before}" : "{user} je arhivirao ploču {before}",
    "You have unarchived the board {board}" : "Dearhivirali ste ploču {board}",
    "{user} has unarchived the board {before}" : "{user} je dearhivirao ploču {before}",
    "You have created a new list {stack} on board {board}" : "Stvorili ste novi popis {stack} na ploči {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} je stvorio novi popis {stack} na ploči {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Preimenovali ste popis {before} u {stack} na ploči {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} je preimenovao popis {before} u {stack} na ploči {board}",
    "You have deleted list {stack} on board {board}" : "Izbrisali ste popis {stack} na ploči {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} je izbrisao popis {stack} na ploči {board}",
    "You have created card {card} in list {stack} on board {board}" : "Stvorili ste karticu {card} u popisu {stack} na ploči {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} je stvorio karticu {card} u popisu {stack} na ploči {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Izbrisali ste karticu {card} u popisu {stack} na ploči {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} je izbrisao karticu {card} u popisu {stack} na ploči {board}",
    "You have renamed the card {before} to {card}" : "Preimenovali ste karticu {before} u {card}",
    "{user} has renamed the card {before} to {card}" : "{user} je preimenovao karticu {before} u {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Dodali ste opis kartici {card} u popisu {stack} na ploči {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} je dodao opis kartici {card} u popisu {stack} na ploči {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Ažurirali ste opis kartice {card} u popisu {stack} na ploči {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} je ažurirao opis kartice {card} u popisu {stack} na ploči {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Arhivirali ste karticu {card} u popisu {stack} na ploči {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} je arhivirao karticu {card} u popisu {stack} na ploči {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Dearhivirali ste karticu {card} u popisu {stack} na ploči {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} je dearhivirao karticu {card} u popisu {stack} na ploči {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Na ploči {board} u popisu {stack} označili ste karticu {card} kao dovršenu",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} je na ploči {board} u popisu {stack} označio karticu {card} kao dovršenu",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Na ploči {board} u popisu {stack} označili ste karticu {card} kao nedovršenu",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} je na ploči {board} u popisu {stack} označio karticu {card} kao nedovršenu",
    "You have removed the due date of card {card}" : "Uklonili ste datum dospijeća kartice {card}",
    "{user} has removed the due date of card {card}" : "{user} je uklonio datum dospijeća kartice {card}",
    "You have set the due date of card {card} to {after}" : "Postavili ste datum dospijeća kartice {card} na {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} je postavio datum dospijeća kartice {card} na {after}",
    "You have updated the due date of card {card} to {after}" : "Ažurirali ste datum dospijeća kartice {card} na {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} je ažurirao datum dospijeća kartice {card} na {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Dodali ste oznaku {label} na karticu {card} u popisu {stack} na ploči {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} je dodao oznaku {label} na karticu {card} u popisu {stack} na ploči {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Uklonili ste oznaku {label} s kartice {card} u popisu {stack} na ploči {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} je uklonio oznaku {label} s kartice {card} u popisu {stack} na ploči {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Dodijelili ste {assigneduser} kartici {card} na ploči {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} je dodijelio {assigneduser} kartici {card} na ploči {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Uklonili ste {assigneduser} s kartice {card} na ploči {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} je uklonio {assigneduser} s kartice {card} na ploči {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Premjestili ste karticu {card} iz popisa {stackBefore} u {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} je premjestio karticu {card} iz popisa {stackBefore} u {stack}",
    "You have added the attachment {attachment} to card {card}" : "Dodali ste privitak {attachment} kartici {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} je dodao privitak {attachment} kartici {card}",
    "You have updated the attachment {attachment} on card {card}" : "Ažurirali ste privitak {attachment} na kartici {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} je ažurirao privitak {attachment} na kartici {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Izbrisali ste privitak {attachment} s kartice {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} je izbrisao privitak {attachment} s kartice {card}",
    "You have restored the attachment {attachment} to card {card}" : "Vratili ste privitak {attachment} na karticu {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} je vratio privitak {attachment} na karticu {card}",
    "You have commented on card {card}" : "Komentirali ste karticu {card}",
    "{user} has commented on card {card}" : "{user} je komentirao karticu {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Promjene u <strong>aplikaciji Deck</strong>",
    "A <strong>board, list or card</strong> was changed" : "Promijenjena je <strong>ploča, popis ili kartica</strong>",
    "A <strong>comment</strong> was created on a card" : "Na kartici je stvoren <strong>komentar</strong>",
    "A <strong>card description</strong> has been changed" : "Promijenjen je <strong>opis kartice</strong>",
    "The file was uploaded" : "Datoteka je otpremljena",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Otpremljena datoteka premašuje postavku upload_max_filesize u datoteci php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Otpremljena datoteka premašuje postavku MAX_FILE_SIZE koja je navedena u obrascu HTML-a",
    "The file was only partially uploaded" : "Datoteka je samo djelomično otpremljena",
    "No file was uploaded" : "Nije otpremljena nijedna datoteka",
    "Missing a temporary folder" : "Nedostaje privremena mapa",
    "Could not write file to disk" : "Nije moguće zapisati datoteku na disk",
    "A PHP extension stopped the file upload" : "Proširenje PHP-a zaustavilo je otpremanje datoteke",
    "No file uploaded or file size exceeds maximum of %s" : "Nijedna datoteka nije otpremljena ili veličina datoteke premašuje maksimalnu veličinu od %s",
    "Invalid file type. Only JSON files are allowed." : "Neispravna vrsta datoteke. Dopuštene su samo JSON datoteke.",
    "Invalid JSON data" : "Neispravni JSON podaci",
    "Failed to import board" : "Nije uspjelo uvesti ploču",
    "Cards due today" : "Kartice s rokom danas",
    "Cards due tomorrow" : "Kartice s rokom sutra",
    "Upcoming cards" : "Nadolazeće kartice",
    "Load more" : "Učitaj više",
    "Welcome to Nextcloud Deck!" : "Dobro došli u Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Karticu „%s” na „%s” dodijelio vam je %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} vam je dodijelio karticu {deck-card} na {deck-board}.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Kartica \"%1$s\" na \"%2$s\" dosegnula je rok.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Kartica {deck-card} na {deck-board} dosegla je datum dospijeća.",
    "%s has mentioned you in a comment on \"%s\"." : "%s vas spominje u komentaru na „%s”.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} vas je spomenuo u komentaru na {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Ploču „%s” je s vama podijelio %s.",
    "{user} has shared {deck-board} with you." : "{user} s vama dijeli {deck-board}.",
    "The remote board %1$s has been shared with you by %2$s" : "Udaljena ploča %1$s podijeljena je s vama od strane %2$s.",
    "Deck board" : "Deck ploča",
    "Owned by %1$s" : "Vlasnik: %1$s",
    "Deck boards, cards and comments" : "Deck ploče, kartice i komentari",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Od %1$s, u %2$s/%3$s, vlasnik: %4$s",
    "Create a new deck card" : "Stvori novu Deck karticu",
    "Card comments" : "Komentari o kartici",
    "%s on %s" : "%s na %s",
    "Deck boards and cards" : "Deck ploče i kartice",
    "No data was provided to create an attachment." : "Nema podataka za izradu privitka.",
    "Finished" : "Završeno",
    "To review" : "Provjeriti",
    "Action needed" : "Potrebna radnja",
    "Later" : "Kasnije",
    "copy" : "kopiraj",
    "Read more inside" : "Pročitajte više unutra",
    "Custom lists - click to rename!" : "Prilagođeni popisi – kliknite za preimenovanje!",
    "To Do" : "Zadatak",
    "In Progress" : "U tijeku",
    "Done" : "Gotovo",
    "1. Open to learn more about boards and cards" : "1. Otvorite da biste saznali više o pločama i karticama",
    "2. Drag cards left and right, up and down" : "2. Povlačite kartice lijevo i desno, gore i dolje",
    "3. Apply rich formatting and link content" : "3. Primijenite bogato oblikovanje i povežite sadržaj",
    "4. Share, comment and collaborate!" : "4. Dijelite, komentirajte i surađujte!",
    "Create your first card!" : "Stvorite svoju prvu karticu!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Ovaj komentar ima više od %s znakova.\nDodan je kao privitak kartici naziva %s.\nDostupno na URL-u: %s.",
    "Attachments" : "Privici",
    "File" : "Datoteka",
    "date" : "datum",
    "Card not found" : "Kartica nije pronađena",
    "Path is already shared with this card" : "Put je već podijeljen s ovom karticom",
    "Invalid date, date format must be YYYY-MM-DD" : "Nevažeći datum, oblik datuma mora biti GGGG-MM-DD",
    "Personal planning and team project organization" : "Osobno planiranje i organizacija timskih projekata",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck je alat za organizaciju u kanban stilu namijenjen osobnom planiranju i organizaciji projekata za timove, integriran s Nextcloudom.\n\n\n- 📥 Dodajte zadatke na kartice i posložite ih po redoslijedu\n- 📄 Zapišite dodatne bilješke u Markdownu\n- 🔖 Dodijelite oznake za još bolju organizaciju\n- 👥 Dijelite sa svojim timom, prijateljima ili obitelji\n- 📎 Priložite datoteke i ugradite ih u opis na Markdownu\n- 💬 Raspravljajte s timom putem komentara\n- ⚡ Pratite promjene u tijeku aktivnosti\n- 🚀 Organizirajte svoj projekt",
    "Add board" : "Dodaj ploču",
    "Card details" : "Pojedinosti o kartici",
    "Select the board to link to a project" : "Odaberite ploču za povezivanje s projektom",
    "Search by board title" : "Traži po naslovu ploče",
    "Select board" : "Odaberi ploču",
    "Move/copy card" : "Premjesti/kopiraj karticu",
    "Select a board" : "Odaberite ploču",
    "No lists available" : "Nema dostupnih popisa",
    "Select a list" : "Odaberi popis",
    "Move card" : "Premjesti karticu",
    "Copy card" : "Kopiraj karticu",
    "Select the card to link to a project" : "Odaberite karticu za povezivanje s projektom",
    "Link to card" : "Poveznica na karticu",
    "Select a card" : "Odaberite karticu",
    "Cancel" : "Odustani",
    "This board is read only" : "Ova je ploča samo za čitanje",
    "Drop your files to upload" : "Ispustite datoteke za otpremu",
    "File already exists" : "Datoteka već postoji",
    "A file with the name {filename} already exists." : "Datoteka s nazivom {filename} već postoji.",
    "Do you want to overwrite it?" : "Želite li prepisati?",
    "Overwrite file" : "Prepiši datoteku",
    "Keep existing file" : "Zadrži postojeću datoteku",
    "Add card" : "Dodaj karticu",
    "Archived cards" : "Arhivirane kartice",
    "Add list" : "Dodaj popis",
    "List name" : "Naziv popisa",
    "Clear search" : "Izbriši pretragu",
    "Active filters" : "Aktivni filteri",
    "Apply filter" : "Primijeni filtar",
    "Filter by tag" : "Filtriraj prema oznaci",
    "Filter by assigned user" : "Filtriraj prema dodijeljenom korisniku",
    "Unassigned" : "Nedodijeljeno",
    "Filter by status" : "Filtriraj prema statusu",
    "Open and completed" : "Otvoreno i dovršeno",
    "Open" : "Otvori",
    "Completed" : "Završeno",
    "Filter by due date" : "Filtriraj prema datumu dospijeća",
    "Overdue" : "Kasni",
    "Next 24 hours" : "Sljedeća 24 sata",
    "Next 7 days" : "Sljedećih 7 dana",
    "Next 30 days" : "Sljedećih 30 dana",
    "No due date" : "Nema datuma dospijeća",
    "Clear filter" : "Ukloni filtar",
    "View Modes" : "Načini prikaza",
    "Toggle View Modes" : "Uključi/isključi načine prikaza",
    "Hide archived cards" : "Sakrij arhivirane kartice",
    "Show archived cards" : "Prikaži arhivirane kartice",
    "Toggle compact mode" : "Prebaci u kompaktni način rada",
    "Hide card cover images" : "Sakrij naslovne slike kartica",
    "Show card cover images" : "Prikaži naslovne slike kartica",
    "Open details" : "Otvori pojedinosti",
    "Details" : "Pojedinosti",
    "Deck settings" : "Postavke Decka",
    "General" : "Općenito",
    "Use bigger card view" : "Prikaži veće kartice",
    "Appearance" : "Izgled",
    "Show card ID badge" : "Prikaži značku s ID-jem kartice",
    "Show boards in calendar/tasks" : "Prikaži ploče u kalendaru/zadacima",
    "Admin settings" : "Administratorske postavke",
    "Limit board creation to some groups" : "Ograniči stvaranje ploča na neke grupe",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Korisnici izvan tih grupa neće moći stvarati vlastite ploče, ali će i dalje moći raditi na pločama koje su im podijeljene.",
    "Enable federation" : "Omogući udruženja",
    "Board actions" : "Radnje ploče",
    "Scroll sideways" : "Pomicanje bočno",
    "Navigate between cards" : "Kretanje između kartica",
    "Close card details" : "Zatvori detalje kartice",
    "Search" : "Traži",
    "Show card filters" : "Prikaži filtre kartica",
    "Clear card filters" : "Očisti filtre kartica",
    "Show those shortcuts" : "Prikaži te prečace",
    "Card actions" : "Radnje kartice",
    "Open card details" : "Otvori detalje kartice",
    "Edit the card title" : "Uredi naslov kartice",
    "Assign yourself to the current card" : "Dodijeli sebe trenutačnoj kartici",
    "Archive/unarchive the current card" : "Arhiviraj/dearhiviraj trenutačnu karticu",
    "Mark card as completed/not completed" : "Označi karticu kao dovršenu/nedovršenu",
    "Open card menu" : "Otvori izbornik kartice",
    "Currently present people" : "Trenutačno prisutne osobe",
    "Loading board" : "Učitavanje ploče",
    "Board not found" : "Ploča nije pronađena",
    "Create a new list to add cards to this board" : "Stvorite novi popis kako biste dodali kartice na ovu ploču",
    "Sharing" : "Dijeljenje",
    "Tags" : "Oznake",
    "Deleted items" : "Izbrisane stavke",
    "Activity" : "Aktivnost",
    "Deleted lists" : "Izbrisani popisi",
    "Undo" : "Poništi",
    "Deleted cards" : "Izbrisane kartice",
    "Hours" : "Sati",
    "Weeks" : "Tjedana",
    "Months" : "Mjeseci",
    "No cards yet" : "Još nema kartica",
    "Failed to create share with {displayName}" : "Dijeljenje s {displayName} nije uspjelo",
    "Are you sure you want to transfer the board {title} to {user}?" : "Jeste li sigurni da želite prenijeti ploču {title} na korisnika {user}?",
    "Transfer the board." : "Prenesi ploču.",
    "Transfer" : "Prijenos",
    "The board has been transferred to {user}" : "Ploča je prenesena na korisnika {user}",
    "Failed to transfer the board to {user}" : "Prenošenje ploče na korisnika {user} nije uspjelo",
    "Board owner" : "Vlasnik ploče",
    "(Group)" : "(Grupa)",
    "(Team)" : "(Tim)",
    "(remote)" : "(na daljinu)",
    "Can edit" : "Uređivanje moguće",
    "Can share" : "Dijeljenje moguće",
    "Can manage" : "Upravljanje moguće",
    "Owner" : "Vlasnik",
    "Delete" : "Izbriši",
    "List deleted" : "Popis je izbrisan",
    "Edit list title" : "Uredi naslov popisa",
    "Archive all cards" : "Arhiviraj sve kartice",
    "Unarchive all cards" : "Dearhiviraj sve kartice",
    "Do not set cards as \"done\"" : "Ne označavaj kartice kao \"gotove\"",
    "Set cards as \"done\"" : "Označi kartice kao \"gotove\"",
    "Delete list" : "Izbriši popis",
    "Archive all cards in this list" : "Arhiviraj sve kartice s ovog popisa",
    "Unarchive all cards in this list" : "Dearhiviraj sve kartice u ovom popisu",
    "Add a new card" : "Dodaj novu karticu",
    "Card name" : "Naziv kartice",
    "title and color value must be provided" : "potrebno je odabrati naziv i vrijednost boje",
    "Edit" : "Uredi",
    "Add a new tag" : "Dodaj novu oznaku",
    "Board name" : "Naziv ploče",
    "Members" : "Članovi",
    "Assign to users/groups/team" : "Dodijeli korisnicima/grupama/timu",
    "Assign a user to this card…" : "Dodijeli korisnika ovoj kartici...",
    "Select a user to assign to this card…" : "Odaberite korisnika kojem želite dodijeliti ovu karticu…",
    "File to share" : "Datoteka za dijeljenje",
    "Invalid path selected" : "Odabran nevažeći put",
    "Upload new files" : "Otpremi nove datoteke",
    "Share from Files" : "Dijeli iz datoteka",
    "Pending share" : "Dijeljenje na čekanju",
    "Add this attachment" : "Dodajte ovaj privitak",
    "Show in Files" : "Prikaži u datotekama",
    "Download" : "Preuzmi",
    "Remove attachment" : "Ukloni privitak",
    "Delete Attachment" : "Izbriši privitak",
    "Restore Attachment" : "Vrati privitak",
    "The title cannot be empty." : "Naslov ne može biti prazan.",
    "Cannot close unsaved card!" : "Nije moguće zatvoriti nespremljenu karticu!",
    "Open in sidebar view" : "Otvori u bočnom prikazu",
    "Open in bigger view" : "Otvori u većem prikazu",
    "Comments" : "Komentari",
    "Failed to load comments" : "Učitavanje komentara nije uspjelo",
    "No comments yet. Begin the discussion!" : "Nema komentara. Započnite raspravu!",
    "The comment cannot be empty." : "Komentar ne može biti prazan.",
    "The comment cannot be longer than 1000 characters." : "Komentar ne može duži od 1000 znakova.",
    "Submit" : "Šalji",
    "Created:" : "Stvoreno:",
    "In reply to" : "Kao odgovor",
    "Cancel reply" : "Prekini odgovor",
    "Reply" : "Odgovori",
    "Update" : "Ažuriraj",
    "Mark as done" : "Označi kao završeno",
    "Mark as not done" : "Označi kao nedovršeno",
    "Write a description …" : "Napišite opis...",
    "The description has been changed by another user." : "Opis je promijenio drugi korisnik.",
    "Could not save description" : "Nije moguće spremiti opis",
    "Description" : "Opis",
    "(Unsaved)" : "(Nije spremljeno)",
    "(Saving…)" : "(Spremanje…)",
    "Formatting help" : "Pomoć pri oblikovanju",
    "Edit description" : "Uredi opis",
    "View description" : "Prikaži opis",
    "Add Attachment" : "Dodaj privitak",
    "Choose attachment" : "Odaberi privitak",
    "Select Date" : "Odaberi datum",
    "Later today – {timeLocale}" : "Kasnije danas – {timeLocale}",
    "Set due date for later today" : "Postavi rok za kasnije danas",
    "Tomorrow – {timeLocale}" : "Sutra – {timeLocale}",
    "Set due date for tomorrow" : "Postavi rok za sutra",
    "This weekend – {timeLocale}" : "Ovaj vikend – {timeLocale}",
    "Set due date for this weekend" : "Postavi rok za ovaj vikend",
    "Next week – {timeLocale}" : "Sljedeći tjedan – {timeLocale}",
    "Set due date for next week" : "Postavi rok za sljedeći tjedan",
    "Assign a due date to this card…" : "Dodijelite rok ovoj kartici…",
    "Set a due date" : "Postavi datum dospijeća",
    "Add due date" : "Dodaj rok",
    "Choose a date" : "Odaberite datum",
    "Remove due date" : "Ukloni datum dospijeća",
    "Due at:" : "Rok u:",
    "Not done" : "Nije dovršeno",
    "Unarchive card" : "Dearhiviraj karticu",
    "Archive card" : "Arhiviraj karticu",
    "Assign a start date to this card…" : "Dodijelite datum početka ovoj kartici....",
    "Set a start date" : "Postavi datum početka",
    "Add start date" : "Dodaj datum početka",
    "Remove start date" : "Ukloni datum početka",
    "Assign a tag to this card…" : "Dodijeli oznaku ovoj kartici...",
    "Select or create a tag…" : "Odaberite ili stvorite oznaku…",
    "Create a new tag:" : "Stvori novu oznaku:",
    "(group)" : "(grupa)",
    "{count} comments, {unread} unread" : "{count} komentara, {unread} nepročitanih",
    "Todo items" : "Stavke zadataka",
    "Edit card title" : "Uredi naslov kartice",
    "Open link" : "Otvori poveznicu",
    "Card deleted" : "Kartica je izbrisana",
    "Edit title" : "Uredi naslov",
    "Assign to me" : "Dodijeli meni",
    "Unassign myself" : "Ukloni sebe",
    "Delete card" : "Izbriši karticu",
    "All boards" : "Sve ploče",
    "Archived boards" : "Arhivirane ploče",
    "Shared with you" : "Podijeljeno s vama",
    "Cancel edit" : "Otkaži uređivanje",
    "Save board" : "Spremi ploču",
    "Board {0} deleted" : "Ploča {0} je izbrisana",
    "All cards" : "Sve kartice",
    "Only assigned cards" : "Samo dodijeljene kartice",
    "No reminder" : "Nema podsjetnika",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Jeste li sigurni da želite izbrisati ploču {title}? Time će se izbrisati svi podaci ove ploče, uključujući arhivirane kartice.",
    "Delete the board?" : "Želite li izbrisati ploču?",
    "Are you sure you want to leave the board {title}?" : "Jeste li sigurni da želite napustiti ploču {title}?",
    "Leave the board?" : "Napustiti ploču?",
    "Leave" : "Napusti",
    "Failed to leave the board" : "Napuštanje ploče nije uspjelo",
    "Exporting board..." : "Izvoz ploče…",
    "An error occurred" : "Došlo je do pogreške",
    "Board details" : "Pojedinosti o ploči",
    "Edit board" : "Uredi ploču",
    "Clone board" : "Kloniraj ploču",
    "Unarchive board" : "Dearhiviraj ploču",
    "Archive board" : "Arhiviraj ploču",
    "Export board" : "Izvezi ploču",
    "Turn on due date reminders" : "Uključi podsjetnike datuma dospijeća",
    "Turn off due date reminders" : "Isključi podsjetnike datuma dospijeća",
    "Remove as default board" : "Ukloni kao zadanu ploču",
    "Set as default board" : "Postavi kao zadanu ploču",
    "Due date reminders" : "Podsjetnici datuma dospijeća",
    "Assigned cards" : "Dodijeljene kartice",
    "No notifications" : "Nema obavijesti",
    "Delete board" : "Izbriši ploču",
    "Leave board" : "Napusti ploču",
    "Importing board..." : "Uvoz ploče…",
    "Board imported successfully" : "Ploča je uspješno uvezena",
    "Import board" : "Uvezi ploču",
    "Clone {boardTitle}" : "Kloniraj {boardTitle}",
    "Clone cards" : "Kloniraj kartice",
    "Clone assignments" : "Kloniraj dodjele",
    "Clone labels" : "Kloniraj oznake",
    "Clone due dates" : "Kloniraj rokove",
    "Advanced options" : "Napredne opcije",
    "Move all cards to the first list" : "Premjesti sve kartice u prvi popis",
    "Restore archived cards" : "Vrati arhivirane kartice",
    "Clone" : "Kloniraj",
    "Export {boardTitle}" : "Izvezi {boardTitle}",
    "Export as JSON" : "Izvezi kao JSON",
    "Export as CSV" : "Izvezi kao CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Napomena: Za ponovni uvoz u aplikaciju Deck podržan je samo JSON format.",
    "Export" : "Izvoz",
    "Loading filtered view" : "Učitavanje filtriranog prikaza",
    "Search for {searchQuery} in other boards" : "Pretraži {searchQuery} u drugim pločama",
    "Search for {searchQuery} in all boards" : "Traži {searchQuery} na svim pločama",
    "No results found" : "Nema rezultata",
    "Deck board {name}\n* Last modified on {lastMod}" : "Deck ploča {name}\n* Zadnja izmjena: {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Stvoreno: {created}\n* Zadnja izmjena: {lastMod}\n* Privitaka: {nbAttachments}\n* Komentara: {nbComments}",
    "{nbCards} cards" : "{nbCards} kartica",
    "Due on {date}" : "Rok: {date}",
    "{stack} in {board}" : "{stack} u {board}",
    "Click to expand description" : "Kliknite za proširenje opisa",
    "Click to expand comment" : "Kliknite za proširenje komentara",
    "Create card" : "Stvori karticu",
    "Create a new card" : "Stvori novu karticu",
    "Card title" : "Naslov kartice",
    "Creating the new card …" : "Stvaranje nove kartice…",
    "Card \"{card}\" was added to \"{board}\"" : "Kartica „{card}” je dodana na „{board}”",
    "Open card" : "Otvori karticu",
    "Close" : "Zatvori",
    "No upcoming cards" : "Nema nadolazećih kartica",
    "upcoming cards today" : "nadolazeće kartice danas",
    "upcoming cards tomorrow" : "nadolazeće kartice sutra",
    "upcoming cards" : "nadolazeće kartice",
    "New card" : "Nova kartica",
    "Link to a board" : "Poveznica na ploču",
    "Link to a card" : "Poveznica na karticu",
    "Create a card" : "Stvori karticu",
    "Message from {author} in {conversationName}" : "Poruka od {author} u {conversationName}",
    "Something went wrong" : "Nešto je pošlo po krivu",
    "Failed to upload {name}" : "Neuspješno otpremanje {name}",
    "Maximum file size of {size} exceeded" : "Prekoračena je maksimalna veličina datoteke od {size}",
    "Assigned users" : "Dodijeljeni korisnici",
    "Due date" : "Datum dospijeća",
    "Created" : "Stvoreno",
    "Modified" : "Promijenjeno",
    "Error creating the share" : "Pogreška pri stvaranju dijeljenja",
    "Share with a Deck card" : "Dijeli s Deck karticom",
    "Share {file} with a Deck card" : "Dijeli {file} s Deck karticom",
    "Share" : "Dijeli",
    "The card \"%s\" on \"%s\" has reached its due date." : "Kartica „%s” na „%s” dosegla je datum dospijeća.",
    "Share board with a user, group or team …" : "Dijeli ploču s korisnikom, grupom ili timom …",
    "Searching for users, groups and teams …" : "Pretraživanje korisnika, grupa i timova …",
    "No participants found" : "Nije pronađen nijedan sudionik",
    "Save" : "Spremi",
    "seconds ago" : "prije nekoliko sekundi",
    "Today" : "Danas",
    "Tomorrow" : "Sutra",
    "No due" : "Nema nezavršenih"
},
"nplurals=3; plural=n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2;");
