(self.webpackChunkdeck=self.webpackChunkdeck||[]).push([["node_modules_vue-click-outside_index_js"],{25276(e,t){function n(e){return"function"==typeof e.value||(console.warn("[Vue-click-outside:] provided expression",e.expression,"is not a function."),!1)}function c(e){return void 0!==e.componentInstance&&e.componentInstance.$isServer}e.exports={bind:function(e,t,u){if(!n(t))return;function o(t){if(u.context){var n=t.path||t.composedPath&&t.composedPath();n&&n.length>0&&n.unshift(t.target),e.contains(t.target)||function(e,t){if(!e||!t)return!1;for(var n=0,c=t.length;n<c;n++)try{if(e.contains(t[n]))return!0;if(t[n].contains(e))return!1}catch(e){return!1}return!1}(u.context.popupItem,n)||e.__vueClickOutside__.callback(t)}}e.__vueClickOutside__={handler:o,callback:t.value};const i="ontouchstart"in document.documentElement?"touchstart":"click";!c(u)&&document.addEventListener(i,o)},update:function(e,t){n(t)&&(e.__vueClickOutside__.callback=t.value)},unbind:function(e,t,n){const u="ontouchstart"in document.documentElement?"touchstart":"click";!c(n)&&e.__vueClickOutside__&&document.removeEventListener(u,e.__vueClickOutside__.handler),delete e.__vueClickOutside__}}}}]);
//# sourceMappingURL=deck-node_modules_vue-click-outside_index_js.js.map?v=960c53c392b428059e7c