OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Vous avez créé un nouveau tableau {board}",
    "{user} has created a new board {board}" : "{user} a créé un nouveau tableau {board}",
    "You have deleted the board {board}" : "Vous avez supprimé le tableau {board}",
    "{user} has deleted the board {board}" : "{user} a supprimé le tableau {board}",
    "You have restored the board {board}" : "Vous avez restauré le tableau {board}",
    "{user} has restored the board {board}" : "{user} a restauré le tableau {board}",
    "You have shared the board {board} with {acl}" : "Vous avez partagé le tableau {board} avec {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} a partagé le tableau {board} en {acl}",
    "You have removed {acl} from the board {board}" : "Vous avez supprimé {acl} du tableau {board}",
    "{user} has removed {acl} from the board {board}" : "{user} a supprimé {acl} du tableau {board}",
    "You have renamed the board {before} to {board}" : "Vous avez renommé le tableau {before} en {board}",
    "{user} has renamed the board {before} to {board}" : "{user} a renommé le tableau {before} en {board}",
    "You have archived the board {board}" : "Vous avez archivé le tableau {board}",
    "{user} has archived the board {before}" : "{user} a archivé le tableau {before}",
    "You have unarchived the board {board}" : "Vous avez désarchivé le tableau {board}",
    "{user} has unarchived the board {before}" : "{user} a désarchivé le tableau {before}",
    "You have created a new list {stack} on board {board}" : "Vous avez créé une nouvelle liste {stack} sur le tableau {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} a créé une nouvelle liste {stack} sur le tableau {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Vous avez renommé la liste {before} en {stack} sur le tableau {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} a renommé la liste {before} en {stack} sur le tableau {board}",
    "You have deleted list {stack} on board {board}" : "Vous avez supprimé la liste {stack} sur le tableau {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} a supprimé la liste {stack} sur le tableau {board}",
    "You have created card {card} in list {stack} on board {board}" : "Vous avez créé la carte {card} dans la liste {stack} du tableau {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} a créé la carte {card} dans la liste {stack} du tableau {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Vous avez supprimé la carte {card} de la liste {stack} du tableau {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} a supprimé la carte {card} de la liste {stack} du tableau {board}",
    "You have renamed the card {before} to {card}" : "Vous avez renommé la carte {before} en {card}",
    "{user} has renamed the card {before} to {card}" : "{user} a renommé la carte {before} en {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Vous avez ajouté une description à la carte {card} de la liste {stack} du tableau {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} a ajouté une description à la carte {card} de la liste {stack} du tableau {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Vous avez mis à jour la description de la carte {card} de la liste {stack} du tableau {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} a mis à jour la description de la carte {card} de la liste {stack} du tableau {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Vous avez archivé la carte {card} de la liste {stack} du tableau {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} a archivé la carte {card} de la liste {stack} du tableau {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Vous avez désarchivé la carte {card} de la liste {stack} du tableau {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} a désarchivé la carte {card} de la liste {stack} du tableau {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Vous avez marqué la carte {card} comme faite dans la liste {stack} sur le tableau {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} a marqué la carte {card} comme faite dans la liste {stack} sur le tableau {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Vous avez marqué la carte {card} comme non faite dans la liste {stack} sur le tableau {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} a marqué la carte {card} comme non faite dans la liste {stack} sur le tableau {board}",
    "You have removed the due date of card {card}" : "Vous avez supprimé la date d'échéance de la carte {card}",
    "{user} has removed the due date of card {card}" : "{user} a supprimé la date d'échéance de la carte {card}",
    "You have set the due date of card {card} to {after}" : "Vous avez établi la date d'échéance de la carte {card} au {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} a établi la date d'échéance de la carte {card} à {after}",
    "You have updated the due date of card {card} to {after}" : "Vous avez mis à jour la date d'échéance de la carte {card} au {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} a mis à jour la date d'échéance de la carte {card} à {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Vous avez ajouté l'étiquette {label} à la carte {card} de la liste {stack} du tableau {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} a ajouté l'étiquette {label} à la carte {card} de la liste {stack} du tableau {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Vous avez supprimé l'étiquette {label} de la carte {card} de la liste {stack} du tableau {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} a supprimé l'étiquette {label} de la carte {card} de la liste {stack} du tableau {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Vous avez affecté {assigneduser} à la carte {card} du tableau {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} a assigné {assigneduser} à la carte {card} du tableau {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Vous avez retiré {assigneduser} de la carte {card} du tableau {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} a retiré {assigneduser} de la carte {card} du tableau {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Vous avez déplacé la carte {card} de la liste {stackBefore} vers {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} a déplacé la carte {card} de la liste {stackBefore} vers {stack}",
    "You have added the attachment {attachment} to card {card}" : "Vous avez ajouté la pièce jointe {attachment} à la carte {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} a ajouté la pièce jointe {attachment} à la carte {card}",
    "You have updated the attachment {attachment} on card {card}" : "Vous avez mis à jour la pièce jointe {attachment} de la carte {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} a mis à jour la pièce jointe {attachment} de la carte {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Vous avez supprimé la pièce jointe {attachment} de la carte {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} a supprimé la pièce jointe {attachment} de la carte {card}",
    "You have restored the attachment {attachment} to card {card}" : "Vous avez restauré la pièce jointe {attachment} de la carte {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} a restauré la pièce jointe {attachment} de la carte {card}",
    "You have commented on card {card}" : "Vous avez commenté la carte {card}",
    "{user} has commented on card {card}" : "{user} a commenté la carte {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Modifications dans <strong>Deck app</strong>",
    "A <strong>board, list or card</strong> was changed" : "Un(e) <strong>tableau, liste ou carte</strong> a été modifié(e)",
    "A <strong>comment</strong> was created on a card" : "Un  <strong>commentaire</strong> a été créé sur une carte",
    "A <strong>card description</strong> has been changed" : "La <strong>description de la carte</strong> a été modifiée",
    "The file was uploaded" : "Le fichier a été envoyé",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Le fichier envoyé dépasse la valeur upload_max_filesize spécifiée dans php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Le fichier envoyé dépasse la valeur MAX_FILE_SIZE spécifiée dans le formulaire HTML",
    "The file was only partially uploaded" : "Le fichier n'a été que partiellement envoyé",
    "No file was uploaded" : "Aucun fichier n'a été envoyé",
    "Missing a temporary folder" : "Absence de dossier temporaire",
    "Could not write file to disk" : "Impossible d'écrire le fichier sur le disque",
    "A PHP extension stopped the file upload" : "Une extension PHP a arrêté l'envoi du fichier",
    "No file uploaded or file size exceeds maximum of %s" : "Aucun fichier envoyé ou la taille du fichier dépasse le maximum de %s",
    "Invalid file type. Only JSON files are allowed." : "Type de fichier non valide. Seuls les fichiers JSON sont autorisés.",
    "Invalid JSON data" : "Données JSON non valides",
    "Failed to import board" : "Échec de l'importation du tableau",
    "Cards due today" : "Cartes pour aujourd'hui",
    "Cards due tomorrow" : "Cartes pour demain",
    "Upcoming cards" : "Prochaines cartes",
    "Load more" : "Charger davantage",
    "Welcome to Nextcloud Deck!" : "Bienvenue dans Nextcloud Deck !",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "La carte \"%s\" de \"%s\" vous a été assignée par %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} vous a attribué la carte {deck-card} du tableau {deck-board}.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "La carte \"%1$s\" sur \"%2$s\" est arrivée à échéance.",
    "The card {deck-card} on {deck-board} has reached its due date." : "La carte {deck-card} sur {deck-board} a atteint sa date d'échéance.",
    "%s has mentioned you in a comment on \"%s\"." : "%s vous a mentionné dans un commentaire sur \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} vous a cité dans un commentaire sur {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Le tableau \"%s\" a été partagé avec vous par %s.",
    "{user} has shared {deck-board} with you." : "{user} vous a partagé le tableau {deck-board}.",
    "The remote board %1$s has been shared with you by %2$s" : "Le tableau distant %1$s a été partagé avec vous par %2$s",
    "Deck board" : "Tableau Deck",
    "Owned by %1$s" : "Assignée à %1$s",
    "Deck boards, cards and comments" : "Tableaux, cartes et commentaires de Deck",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "De %1$s, dans %2$s / %3$s, appartenant à %4$s",
    "Create a new deck card" : "Créer une nouvelle carte Deck",
    "Card comments" : "Commentaires des cartes",
    "%s on %s" : "%s sur %s",
    "Deck boards and cards" : "Tableaux et cartes de Deck",
    "No data was provided to create an attachment." : "Aucune donnée n'a été fournie pour créer une pièce jointe.",
    "Finished" : "Terminé",
    "To review" : "À réviser",
    "Action needed" : "Action requise",
    "Later" : "Plus tard",
    "copy" : "copie",
    "Read more inside" : "Lire la suite à l'intérieur",
    "Custom lists - click to rename!" : "Listes personnalisées - cliquez pour renommer !",
    "To Do" : "À faire",
    "In Progress" : "En cours",
    "Done" : "Fait",
    "1. Open to learn more about boards and cards" : "1. Ouvrez pour en apprendre davantage sur les tableaux et les cartes",
    "2. Drag cards left and right, up and down" : "2. Faites glisser les cartes vers la gauche et la droite, vers le haut et le bas",
    "3. Apply rich formatting and link content" : "3. Appliquer un formatage riche et lier le contenu",
    "4. Share, comment and collaborate!" : "4. Partagez, commentez et collaborez !",
    "Create your first card!" : "Créez votre première carte !",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Ce commentaire comporte plus de %s caractères.\nAjouté en pièce jointe à la carte intitulée %s.\nAccessible depuis l'URL: %s.",
    "Attachments" : "Pièces jointes",
    "File" : "Fichier",
    "date" : "date",
    "Card not found" : "Carte introuvable",
    "Path is already shared with this card" : "Le chemin est déjà partagé avec cette carte",
    "Invalid date, date format must be YYYY-MM-DD" : "Date non valable, le format de la date doit être AAAA-MM-JJ",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Tous vos tableaux, incluant les colonnes, les cartes, les étiquettes, les attributions aux utilisateurs et les commentaires",
    "Personal planning and team project organization" : "Agenda personnel et organisation projet",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck est un outil d'organisation de style kanban destiné à la planification personnelle et à l'organisation de projets pour les équipes intégrées à Nextcloud.\n\n\n- 📥 Ajoutez vos tâches à des cartes et organisez-les\n- 📄 Écrivez des notes supplémentaires en Markdown\n- 🔖 Affectez des étiquettes pour une organisation encore meilleure\n- 👥 Partagez avec votre équipe, vos amis ou votre famille\n- 📎 Joignez des fichiers et utilisez-les dans vos descriptions en Markdown\n- 💬 Échangez avec votre équipe grâce aux commentaires\n- ⚡ Gardez un œil sur les modifications dans le flux d'activité\n- 🚀 Organisez vos projets",
    "Add board" : "Ajouter un tableau",
    "Card details" : "Détails de la carte",
    "Select the board to link to a project" : "Sélection le tableau pour faire le lien avec un projet.",
    "Search by board title" : "Rechercher par titre du tableau",
    "Select board" : "Sélectionner le tableau",
    "Move/copy card" : "Déplacer/copier la carte",
    "Select a board" : "Sélectionner un tableau",
    "No lists available" : "Aucune liste disponible",
    "Select a list" : "Sélectionner une liste",
    "Move card" : "Déplacer la carte",
    "Copy card" : "Copier la carte",
    "Select the card to link to a project" : "Sélectionner la carte à relier à un projet",
    "Link to card" : "Relier à une carte",
    "Select a card" : "Sélectionnez une carte",
    "Cancel" : "Annuler",
    "This board is read only" : "Ce tableau est en lecture seule",
    "Drop your files to upload" : "Glissez vos fichiers pour les téléverser",
    "File already exists" : "Le fichier existe déjà",
    "A file with the name {filename} already exists." : "Un fichier avec le nom {filename} existe déjà.",
    "Do you want to overwrite it?" : "Souhaitez-vous l'écraser ?",
    "Overwrite file" : "Remplacer le fichier",
    "Keep existing file" : "Conserver le fichier existant",
    "Add card" : "Ajouter une carte",
    "Archived cards" : "Cartes archivées",
    "Add list" : "Ajouter une liste",
    "List name" : "Nom de la liste",
    "Clear search" : "Effacer la recherche",
    "Active filters" : "Filtres actifs",
    "Apply filter" : "Filtrer",
    "Filter by tag" : "Filtrer par étiquette",
    "Filter by assigned user" : "Filtrer par utilisateur",
    "Unassigned" : "Non attribué",
    "Filter by status" : "Filtrer par statut",
    "Open and completed" : "Ouvert et terminé",
    "Open" : "Ouvert",
    "Completed" : "Terminé",
    "Filter by due date" : "Filtrer par échéance",
    "Overdue" : "En retard",
    "Next 24 hours" : "prochaines 24 heures",
    "Next 7 days" : "7 prochains jours",
    "Next 30 days" : "30 prochains jours",
    "No due date" : "Sans échéance",
    "Clear filter" : "Supprimer le filtre",
    "View Modes" : "Modes d'affichage",
    "Toggle View Modes" : "Basculer les modes d'affichage",
    "Kanban view" : "Vue Kanban",
    "Gantt view" : "Vue Gantt",
    "Hide archived cards" : "Masquer les cartes archivées",
    "Show archived cards" : "Afficher les cartes archivées",
    "Toggle compact mode" : "Activer/Désactiver l'affichage compact",
    "Hide card cover images" : "Masquer les images de couverture de la carte",
    "Show card cover images" : "Afficher les images de couverture de la carte",
    "Open details" : "Ouvrir les détails",
    "Details" : "Détails",
    "Deck settings" : "Paramètres de Deck",
    "General" : "Général",
    "Use bigger card view" : "Utiliser la vue large des cartes",
    "Appearance" : "Apparence",
    "Show card ID badge" : "Afficher l'ID de la carte",
    "Show boards in calendar/tasks" : "Afficher les tableaux dans les agendas/tâches",
    "Admin settings" : "Paramètres d'administration",
    "Limit board creation to some groups" : "Limiter la création de tableaux à certains groupes",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Les utilisateurs qui ne font pas partie de ces groupes ne pourront pas créer leurs propres tableaux, mais pourront toujours travailler sur les tableaux qui ont été partagés avec eux.",
    "Enable federation" : "Activer la fédération",
    "Board actions" : "Actions sur le tableau",
    "Scroll sideways" : "Défilement latéral",
    "Navigate between cards" : "Naviguer entre les cartes",
    "Close card details" : "Fermer les détails de la carte",
    "Search" : "Rechercher",
    "Show card filters" : "Montrer les filtres de carte",
    "Clear card filters" : "Effacer les filtres de carte",
    "Show those shortcuts" : "Montrer ces raccourcis",
    "Card actions" : "Actions sur les cartes",
    "Open card details" : "Ouvrir les détails de la carte",
    "Edit the card title" : "Modifier le titre de la carte",
    "Assign yourself to the current card" : "M’assigner à la carte active",
    "Archive/unarchive the current card" : "(Dés)Archiver la carte active",
    "Mark card as completed/not completed" : "Marquer la carte (non) terminée",
    "Open card menu" : "Ouvrir le menu de la carte",
    "Currently present people" : "Personnes actuellement présentes",
    "Search cards" : "Rechercher des cartes",
    "Loading board" : "Chargement du tableau…",
    "Board not found" : "Tableau non trouvé",
    "Create a new list to add cards to this board" : "Créer une nouvelle liste pour ajouter des cartes à ce tableau",
    "Sharing" : "Partage",
    "Tags" : "Étiquettes",
    "Deleted items" : "Éléments supprimés",
    "Activity" : "Activité",
    "Deleted lists" : "Listes supprimées",
    "Undo" : "Annuler",
    "Deleted cards" : "Cartes supprimées",
    "Hours" : "Heures",
    "Days" : "Jours",
    "Weeks" : "Semaines",
    "Months" : "Mois",
    "No cards yet" : "Aucune carte pour l'instant",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Définissez une date de départ et une date d'échéance sur vos cartes pour les voir dans le diagramme de Gantt",
    "Cards without dates ({count})" : "Cartes sans dates ({count})",
    "Failed to create share with {displayName}" : "Échec de la création du partage avec {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Êtes-vous sûr de vouloir transférer le tableau {title} à {user} ?",
    "Transfer the board." : "Transférer le tableau",
    "Transfer" : "Transférer",
    "The board has been transferred to {user}" : "Le tableau a été transféré à {user}",
    "Failed to transfer the board to {user}" : "Échec du transfert du tableau à {user}",
    "Board owner" : "Propriétaire du tableau",
    "(Group)" : "(Groupe)",
    "(Team)" : "(Équipe)",
    "(remote)" : "(distant)",
    "Can edit" : "Peut modifier",
    "Can share" : "Peut partager",
    "Can manage" : "Peut gérer",
    "Owner" : "Propriétaire",
    "Delete" : "Supprimer",
    "List deleted" : "Liste supprimée",
    "Edit list title" : "Modifier le titre de la liste",
    "Archive all cards" : "Archiver toutes les cartes",
    "Unarchive all cards" : "Désarchiver toutes les cartes",
    "Do not set cards as \"done\"" : "Ne pas définir les cartes comme étant « terminées »",
    "Set cards as \"done\"" : "Définir les cartes comme étant « terminées »",
    "Delete list" : "Supprimer la liste",
    "Archive all cards in this list" : "Archiver toutes les cartes de cette liste",
    "Unarchive all cards in this list" : "Désarchiver toutes les cartes de cette liste",
    "Add a new card" : "Ajouter une nouvelle carte",
    "Card name" : "Nom de la carte",
    "title and color value must be provided" : "Le titre et la couleur doivent être renseignés",
    "Edit" : "Modifier",
    "Add a new tag" : "Ajouter une nouvelle étiquette",
    "Search boards" : "Rechercher des tableaux",
    "Board name" : "Nom du tableau",
    "Members" : "Membres",
    "Assign to users/groups/team" : "Attribuer à des utilisateurs / groupes / équipes",
    "Assign a user to this card…" : "Affecter cette carte à un utilisateur…",
    "Select a user to assign to this card…" : "Sélectionner un utilisateur auquel assigner la carte...",
    "File to share" : "Fichier à partager",
    "Invalid path selected" : "Chemin sélectionné non valide",
    "Upload new files" : "Téléverser de nouveaux fichiers",
    "Share from Files" : "Choisir depuis Fichiers",
    "Pending share" : "Partage en attente",
    "Add this attachment" : "Ajouter cette pièce jointe",
    "Show in Files" : "Afficher dans Fichiers",
    "Download" : "Télécharger",
    "Remove attachment" : "Supprimer la pièce jointe",
    "Delete Attachment" : "Supprimer la pièce jointe",
    "Restore Attachment" : "Restaurer la pièce jointe",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Modifié : {modifiedDate} ⸱ Créé : {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Modifié : {modifiedDate}\nCréé : {createdDate}\nCréé par : {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Modifié : {modifiedDate}\nCréé : {createdDate}",
    "The title cannot be empty." : "Le titre ne peut pas être vide.",
    "Cannot close unsaved card!" : "Impossible de fermer une carte non enregistrée !",
    "Open in sidebar view" : "Ouvrir dans le panneau latéral",
    "Open in bigger view" : "Ouvrir dans la vue large",
    "Comments" : "Commentaires",
    "Failed to load comments" : "Échec du chargement des commentaires",
    "No comments yet. Begin the discussion!" : "Aucun commentaire pour l'instant. Commencez la discussion !",
    "The comment cannot be empty." : "Un commentaire ne peut pas être vide.",
    "The comment cannot be longer than 1000 characters." : "Un commentaire est limité à 1 000 caractères.",
    "Submit" : "Envoyer",
    "Created:" : "Créée le :",
    "In reply to" : "En réponse à",
    "Cancel reply" : "Annuler la réponse",
    "Reply" : "Répondre",
    "Update" : "Mettre à jour",
    "Assign dependent cards" : "Désigner des cartes dépendantes",
    "Mark as done" : "Marquer comme fait",
    "Mark as not done" : "Marquer comme non fait",
    "Remove dependency" : "Supprimer la dépendance",
    "Add dependent card" : "Ajouter une carte dépendante",
    "Assign a dependent card…" : "Désigner une carte dépendante...",
    "Select a dependent card…" : "Sélectionner une carte dépendante...",
    "Write a description …" : "Écrire une description…",
    "The description has been changed by another user." : "La description a été modifiée par un autre utilisateur.",
    "Could not save description" : "Impossible d'enregistrer la description",
    "Description" : "Description",
    "(Unsaved)" : "(Non enregistré)",
    "(Saving…)" : "(Enregistrement...)",
    "Formatting help" : "Aide sur la mise en forme",
    "Edit description" : "Modifier la description",
    "View description" : "Afficher la description",
    "Add Attachment" : "Ajouter une pièce jointe",
    "Choose attachment" : "Choisir une pièce jointe",
    "Select Date" : "Sélectionnez une date",
    "Later today – {timeLocale}" : "Plus tard aujourd'hui – {timeLocale}",
    "Set due date for later today" : "Définir une date d'échéance pour plus tard dans la journée",
    "Tomorrow – {timeLocale}" : "Demain – {timeLocale}",
    "Set due date for tomorrow" : "Définir une date d'échéance pour demain",
    "This weekend – {timeLocale}" : "Ce week-end – {timeLocale}",
    "Set due date for this weekend" : "Définir une date d'échéance pour ce week-end",
    "Next week – {timeLocale}" : "Semaine prochaine – {timeLocale}",
    "Set due date for next week" : "Définir une date d'échéance pour la semaine prochaine",
    "Assign a due date to this card…" : "Attribuez une date d'échéance à cette carte...",
    "Set a due date" : "Définir une date d'échéance",
    "Add due date" : "Ajouter une date d'échéance",
    "Choose a date" : "Sélectionner une date",
    "Remove due date" : "Retirer la date d'échéance",
    "Due at:" : "Dû au :",
    "Not done" : "Non fait",
    "Unarchive card" : "Sortir la carte des archives",
    "Archive card" : "Archiver la carte",
    "Assign a start date to this card…" : "Assigner une date de départ à cette carte...",
    "Set a start date" : "Définir une date de départ",
    "Add start date" : "Ajouter une date de départ",
    "Remove start date" : "Supprimer la date de départ",
    "Assign a tag to this card…" : "Associer une étiquette à cette carte…",
    "Select or create a tag…" : "Sélectionner ou créer une étiquette",
    "Create a new tag:" : "Créer une nouvelle étiquette :",
    "(group)" : "(groupe)",
    "{count} comments, {unread} unread" : "{count} commentaires, {unread} non lus",
    "Todo items" : "À faire",
    "Edit card title" : "Modifier le titre de la carte",
    "Open link" : "Ouvrir le lien",
    "Card deleted" : "Carte supprimée",
    "Change card color" : "Changer la couleur de la carte",
    "Edit title" : "Modifier le titre",
    "Assign to me" : "Me l'affecter",
    "Unassign myself" : "Me désaffecter",
    "Delete card" : "Supprimer la carte",
    "All boards" : "Tous les tableaux",
    "Archived boards" : "Tableaux archivés",
    "Shared with you" : "Partagés avec vous",
    "Cancel edit" : "Annuler les modifications",
    "Save board" : "Sauvegarder le tableau",
    "Board {0} deleted" : "Tableau {0} supprimé",
    "All cards" : "Toutes les cartes",
    "Only assigned cards" : "Uniquement les cartes assignées",
    "No reminder" : "Aucun rappel",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Êtes-vous sûr de vouloir supprimer le tableau {title} ? Cela supprimera toutes les données de ce tableau, y compris les cartes archivées.",
    "Delete the board?" : "Supprimer le tableau ?",
    "Are you sure you want to leave the board {title}?" : "Êtes-vous sûr de vouloir quitter le tableau {title} ?",
    "Leave the board?" : "Quitter le tableau ?",
    "Leave" : "Quitter",
    "Failed to leave the board" : "Impossible de quitter le tableau",
    "Exporting board..." : "Exportation de la carte...",
    "An error occurred" : "Une erreur est survenue",
    "Board details" : "Détails du tableau",
    "Edit board" : "Renommer le tableau",
    "Clone board" : "Dupliquer le tableau",
    "Unarchive board" : "Désarchiver le tableau",
    "Archive board" : "Archiver le tableau",
    "Export board" : "Exporter le tableau",
    "Turn on due date reminders" : "Activer les rappels",
    "Turn off due date reminders" : "Désactiver les rappels",
    "Remove as default board" : "Retirer comme tableau par défaut",
    "Set as default board" : "Définir comme tableau par défaut",
    "Due date reminders" : "Rappels",
    "Assigned cards" : "Cartes assignées",
    "No notifications" : "Aucune notification",
    "Delete board" : "Supprimer le tableau",
    "Leave board" : "Quitter le tableau",
    "Importing board..." : "Importation du tableau...",
    "Board imported successfully" : "Carte importée avec succès",
    "Import board" : "Importer un tableau",
    "Clone {boardTitle}" : "Cloner {boardTitle}",
    "Clone cards" : "Dupliquer les cartes",
    "Clone assignments" : "Cloner les affectations",
    "Clone labels" : "Cloner les étiquettes",
    "Clone due dates" : "Cloner les dates d'échéance",
    "Advanced options" : "Options avancées",
    "Move all cards to the first list" : "Déplacer toutes les cartes vers la première liste",
    "Restore archived cards" : "Restaurer les cartes archivées",
    "Clone" : "Dupliquer",
    "Export {boardTitle}" : "Exporter {boardTitle}",
    "Export as JSON" : "Exporter au format JSON",
    "Export as CSV" : "Exporter en CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Remarque : seul le format JSON est pris en charge pour la réimportation dans l'application Deck.",
    "Export" : "Exporter",
    "Loading filtered view" : "Chargement de la vue filtrée",
    "Search for {searchQuery} in other boards" : "Rechercher {searchQuery} dans les autres tableaux",
    "Search for {searchQuery} in all boards" : "Recherche de {searchQuery} dans tous les tableaux",
    "No results found" : "Aucun résultat",
    "Deck board {name}\n* Last modified on {lastMod}" : "Tableau Deck {name}\n* Dernière modification le {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Créée le {created}\n* Dernière modification le {lastMod}\n* {nbAttachments} pièces jointes\n* {nbComments} commentaires",
    "{nbCards} cards" : "{nbCards} cartes",
    "Due on {date}" : "Échéance le {date}",
    "{stack} in {board}" : "{stack} dans {board}",
    "Click to expand description" : "Cliquer pour visualiser la description",
    "Click to expand comment" : "Cliquer pour déplier le commentaire",
    "Create card" : "Créer la carte",
    "Create a new card" : "Créer une nouvelle carte",
    "Card title" : "Titre de la carte",
    "Creating the new card …" : "Création de la nouvelle carte…",
    "Card \"{card}\" was added to \"{board}\"" : "La carte \"{card}\" a été ajoutée au tableau  \"{board}\"",
    "Open card" : "Ouvrir la carte",
    "Close" : "Fermer",
    "No upcoming cards" : "Pas de cartes à venir",
    "upcoming cards today" : "cartes à venir aujourd'hui",
    "upcoming cards tomorrow" : "cartes à venir demain",
    "upcoming cards" : "cartes à venir",
    "New card" : "Nouvelle carte",
    "Link to a board" : "Relier à un tableau",
    "Link to a card" : "Relier à une carte",
    "Create a card" : "Créer une carte",
    "Message from {author} in {conversationName}" : "Message de {author} dans {conversationName}",
    "Something went wrong" : "Quelque chose s'est mal passé",
    "Failed to upload {name}" : "Échec d'envoi de {name}",
    "Maximum file size of {size} exceeded" : "Taille de fichier maximale de {size} dépassée",
    "Assigned users" : "Utilisateurs assignés",
    "Due date" : "Date d'échéance",
    "Created" : "Créée",
    "Modified" : "Modifiée",
    "Error creating the share" : "Erreur lors de la création du partage",
    "Share with a Deck card" : "Partager avec une carte Deck",
    "Share {file} with a Deck card" : "Partager {file} avec une carte Deck ",
    "Share" : "Partager",
    "The card \"%s\" on \"%s\" has reached its due date." : "La date d'échéance de la carte \"%s\" de \"%s\" a été atteinte.",
    "Share board with a user, group or team …" : "Partager le tableau avec un utilisateur, un groupe ou une équipe...",
    "Searching for users, groups and teams …" : "Recherche d'utilisateurs, de groupes et d'équipes...",
    "No participants found" : "Aucun participant trouvé",
    "Save" : "Enregistrer",
    "seconds ago" : "à l'instant",
    "Today" : "Aujourd'hui",
    "Tomorrow" : "Demain",
    "No due" : "Sans échéance"
},
"nplurals=3; plural=(n == 0 || n == 1) ? 0 : n != 0 && n % 1000000 == 0 ? 1 : 2;");
