OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Направили сте нову таблу {board}",
    "{user} has created a new board {board}" : "{user} направи нову таблу {board}",
    "You have deleted the board {board}" : "Обрисали сте таблу {board}",
    "{user} has deleted the board {board}" : "{user} је обрисао таблу {board}",
    "You have restored the board {board}" : "Повратили сте таблу {board}",
    "{user} has restored the board {board}" : "{user} је повратио таблу {board}",
    "You have shared the board {board} with {acl}" : "Поделили сте таблу {board} са {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} је поделио таблу {board} са {acl}",
    "You have removed {acl} from the board {board}" : "Уклонили сте {acl} са табле {board}",
    "{user} has removed {acl} from the board {board}" : "{user} је уклонио {acl} са табле {board}",
    "You have renamed the board {before} to {board}" : "Преименовали сте таблу {before} у {board}",
    "{user} has renamed the board {before} to {board}" : "{user} је преименовао таблу {before} у {board}",
    "You have archived the board {board}" : "Архивирали сте таблу {board}",
    "{user} has archived the board {before}" : "{user} је архивирао таблу {before}",
    "You have unarchived the board {board}" : "Вратили сте таблу {board} из архиве",
    "{user} has unarchived the board {before}" : "{user} је вратио таблу {before} из архиве",
    "You have created a new list {stack} on board {board}" : "Направили сте нови списак {stack} на табли {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} је направио нови списак {stack} на табли {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Преименовали сте списак {before} у {stack} на табли {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} је преименовао списак {before} у {stack} на табли {board}",
    "You have deleted list {stack} on board {board}" : "Обрисали сте списак {stack} са табле {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} је обрисао списак {stack} са табле {board}",
    "You have created card {card} in list {stack} on board {board}" : "Направили сте картицу {card} на списку {stack} у табли {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} је направио картицу {card} на списку {stack} у табли {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Обрисали сте картицу {card} са списка {stack} у табли {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} је обрисао картицу {card} са списка {stack} у табли {board}",
    "You have renamed the card {before} to {card}" : "Преименовали сте картицу {before} као картицу {card}",
    "{user} has renamed the card {before} to {card}" : "{user} је преименовао картицу {before} као картицу {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Додали сте опис картици {card} са списка {stack} на табли {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} је додао опис картици {card} са списка {stack} на табли {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Ажурирали сте опис картице {card} са списка {stack} на табли {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "Корисник {user} је ажурирао опис картице {card} са списка {stack} на табли {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Архивирали сте картицу {card} са списка {stack} на табли {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} је архивирао картицу {card} са списка {stack} на табли {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Вратили сте из архиве картицу {card} са списка {stack} у табли {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} је вратио из архиве картицу {card} са списка {stack} у табли {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Картицу {card} на списку {stack} у табли {board} сте означили као завршену",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "Корисник {user} је картицу {card} на списку {stack} у табли {board} означио као завршену",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Уклонили сте ознаку да је завршена картица {card} на списку {stack} у табли {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "Корисник {user} је уклонио ознаку да је завршена картица {card} на списку {stack} у табли {board}",
    "You have removed the due date of card {card}" : "Уклонили сте рок са картице {card}",
    "{user} has removed the due date of card {card}" : "{user} је уклонио рок са картице {card}",
    "You have set the due date of card {card} to {after}" : "Поставили сте рок {after} на картици {card}",
    "{user} has set the due date of card {card} to {after}" : "{user} је поставио рок {after} на картици {card}",
    "You have updated the due date of card {card} to {after}" : "Ажурирали сте рок на {after} на картици {card}",
    "{user} has updated the due date of card {card} to {after}" : "{user} је ажурирао рок на {after} на картици {card}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Додали сте ознаку {label} на картицу {card} са списка {stack} у табли {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} је додао ознаку {label} на картицу {card} са списка {stack} у табли {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Уклонили сте ознаку {label} са картице {card} са списка {stack} у табли {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} је уклонио ознаку {label} са картице {card} са списка {stack} у табли {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Доделили сте корисника {assigneduser} картици {card} са табле {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} је доделио корисника {assigneduser} картици {card} са табле {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Раздужили сте корисника {assigneduser} с картице {card} на табли {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} раздужи корисника {assigneduser} с картице {card} на табли {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Померили сте картицу {card} са списка {stackBefore} на списак {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} је померио картицу {card} са списка {stackBefore} на списак {stack}",
    "You have added the attachment {attachment} to card {card}" : "Додали сте прилог {attachment} на картицу {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} је додао прилог {attachment} на картицу {card}",
    "You have updated the attachment {attachment} on card {card}" : "Ажурирали сте прилог {attachment} на картици {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} је ажурирао прилог {attachment} на картици {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Обрисали сте прилог {attachment} са картице {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} је обрисао прилог {attachment} са картице {card}",
    "You have restored the attachment {attachment} to card {card}" : "Повратили сте прилог {attachment} на картицу {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} је повратио прилог {attachment} на картицу {card}",
    "You have commented on card {card}" : "Оставили сте коментар на картици {card}",
    "{user} has commented on card {card}" : "{user} је оставио коментар на картици {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Измене у <strong>Deck апликацији</strong>",
    "A <strong>board, list or card</strong> was changed" : "Измењена је <strong>табла, листа или картица</strong>",
    "A <strong>comment</strong> was created on a card" : "Направљен је <strong>коментар</strong> на картици",
    "A <strong>card description</strong> has been changed" : "Промењен је <strong>опис картице</strong>",
    "The file was uploaded" : "Фајл је отпремљен",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Фајл за отпремање премашује upload_max_filesize директиву у php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Фајл за отпремање премашује MAX_FILE_SIZE директиву наведену у HTML форми",
    "The file was only partially uploaded" : "Фајл је парцијално отпремљен",
    "No file was uploaded" : "Ниједан фајл није отпремљен",
    "Missing a temporary folder" : "Недостаје привремена фасцикла",
    "Could not write file to disk" : "Не могу да пишем на диск",
    "A PHP extension stopped the file upload" : "PHP екстензија је зауставила отпремање фајла",
    "No file uploaded or file size exceeds maximum of %s" : "Ниједан фајл није отпремљен или величина фајла премашује максимум од %s",
    "Invalid file type. Only JSON files are allowed." : "Неисправан тип фајла. Дозвољени су само JSON фајлови.",
    "Invalid JSON data" : "Неисправни JSON подаци",
    "Failed to import board" : "Није успео увоз табле",
    "Cards due today" : "Картице које доспевају данас",
    "Cards due tomorrow" : "Картице које доспецају сутра",
    "Upcoming cards" : "Предстојеће картице",
    "Load more" : "Учитај још",
    "Welcome to Nextcloud Deck!" : "Добродошли у Nextcloud шпил!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Корисник %s Вам је доделио картицу „%s“ са табле „%s“.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} вам је доделио картицу {deck-card} на {deck-board}.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Картица „%1$s” на „%2$s” је достигла свој рок за завршетак.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Картица {deck-card} на {deck-board} је дошла достигла датум када треба да се реши.",
    "%s has mentioned you in a comment on \"%s\"." : "%s Вас је поменуо у коментару на „%s“.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} вас је поменуо у коментару на {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Корисник „%s“ је поделио са Вама таблу „%s“.",
    "{user} has shared {deck-board} with you." : "{user} је са вама поделио {deck-board}.",
    "Deck board" : "Табла Шпила",
    "Owned by %1$s" : "Власник је %1$s",
    "Deck boards, cards and comments" : "Табле шпилова, картице и коментари",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Од %1$s, у %2$s/%3$s, власник је %4$s",
    "Create a new deck card" : "Креирај нову картицу шпила",
    "Card comments" : "Коментари картица",
    "%s on %s" : "%s на %s",
    "Deck boards and cards" : "Табле шпилова и картице",
    "No data was provided to create an attachment." : "Нису дати подаци за креирање прилога.",
    "Finished" : "Завршено",
    "To review" : "Треба прегледати",
    "Action needed" : "Потребна акција",
    "Later" : "После",
    "copy" : "копирај",
    "Read more inside" : "Прочитајте више",
    "Custom lists - click to rename!" : "Произвољне листе – кликните да промените име!",
    "To Do" : "Обавезе",
    "In Progress" : "Напредује",
    "Done" : "Готово",
    "1. Open to learn more about boards and cards" : "1. Отворите да сазнате више о таблама и картицама",
    "2. Drag cards left and right, up and down" : "2. Превлачите картице лево десно, горе и доле",
    "3. Apply rich formatting and link content" : "3. Примените богато форматирање и линкујте садржај",
    "4. Share, comment and collaborate!" : "4. Делите, коментаришите и сарађујте!",
    "Create your first card!" : "Креирајте своју прву картицу!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Овај коментар има више од %s карактера.\nДодат је као прилог картици под именом %s.\nДоступно је на URL адреси: %s.",
    "Attachments" : "Прилози",
    "File" : "Фајл",
    "date" : "датум",
    "Card not found" : "Картица није нађена",
    "Path is already shared with this card" : "Путања се већ дели са овом картицом",
    "Invalid date, date format must be YYYY-MM-DD" : "Неисправан датум, формат датума мора бити ГГГГ-ММ-ДД",
    "Personal planning and team project organization" : "Лични планер и организатор тимског пројекта",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Шпил је организациони алат у канбан стилу који је намењен личном планирању и организацији пројекта у тимовима интегрисаним са Nextcloud.\n\n\n- 📥 Додајте своје задатке у картице и поређајте их по редоследу\n- 📄 Напишите додатне белешке употребом Markdown синтаксе\n- 🔖 Доделите ознаке у циљу још боље организације\n- 👥 Делите са својим тимом, пријатељима или породицом\n- 📎 Прикачите фајлове и уградите их у свој Markdown опис\n- 💬 Дискутујте са својим тимом користећи коментаре\n- ⚡ Пратите измене у току активности\n- 🚀 Организујте свој пројекат",
    "Add board" : "Додај таблу",
    "Card details" : "Детаљи картице",
    "Select the board to link to a project" : "Одаберите таблу да је повежете са пројектом",
    "Search by board title" : "Претражи по наслову табле",
    "Select board" : "Одаберите таблу",
    "Move/copy card" : "Премести/копирај картицу",
    "Select a board" : "Изаберите таблу",
    "No lists available" : "Нема доступних спискова",
    "Select a list" : "Одабери списак",
    "Move card" : "Премести картицу",
    "Copy card" : "Копирај картицу",
    "Select the card to link to a project" : "Изаберите картицу да повежете на пројекат",
    "Link to card" : "Повежи са картицом",
    "Select a card" : "Изаберите картицу",
    "Cancel" : "Одустани",
    "This board is read only" : "Табла је само за читање",
    "Drop your files to upload" : "Отпустите фајлове за отпремање",
    "File already exists" : "Фајл већ постоји",
    "A file with the name {filename} already exists." : "Фајл {filename} већ постоји.",
    "Do you want to overwrite it?" : "Да ли желите да га препишете?",
    "Overwrite file" : "Препиши фајл",
    "Keep existing file" : "Задржи постојећи фајл",
    "Add card" : "Додај картицу",
    "Archived cards" : "Архивиране картице",
    "Add list" : "Додај списак",
    "List name" : "Назив листе",
    "Clear search" : "Обриши претрагу",
    "Active filters" : "Активни филтери",
    "Apply filter" : "Примени филтер",
    "Filter by tag" : "Филтрирај по ознаци",
    "Filter by assigned user" : "Филтрирај по додељеном кориснику",
    "Unassigned" : "Недодељен",
    "Filter by status" : "Филтрирај по статусу",
    "Open and completed" : "Оторени и завршени",
    "Open" : "Отвори",
    "Completed" : "Завршено",
    "Filter by due date" : "Филтрирај по року",
    "Overdue" : "Прошао рок",
    "Next 24 hours" : "Наредна 24 сата",
    "Next 7 days" : "Следећих 7 дана",
    "Next 30 days" : "Следећих 30 дана",
    "No due date" : "Без рока",
    "Clear filter" : "Очисти филтер",
    "View Modes" : "Режими приказа",
    "Toggle View Modes" : "Пребацуј режиме приказа",
    "Hide archived cards" : "Сакриј архивиране картице",
    "Show archived cards" : "Прикажи архивиране картице",
    "Toggle compact mode" : "Укључи/искључи компактни режим",
    "Hide card cover images" : "Сакриј слике омота картица",
    "Show card cover images" : "Прикажи слике омота картица",
    "Open details" : "Отвори детаље",
    "Details" : "Детаљи",
    "Deck settings" : "Поставке Шпила",
    "General" : "Опште",
    "Use bigger card view" : "Користи већи приказ картице",
    "Appearance" : "Изглед",
    "Show card ID badge" : "Прикажи беџ ID картице",
    "Show boards in calendar/tasks" : "Прикажи табле у календару/задацима",
    "Admin settings" : "Админ подешавања",
    "Limit board creation to some groups" : "Ограничи креирање табле на одређене групе",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Корисници који не припадају овим групама неће моћи да креирају своје табле, али ће и даље моћи да раде на таблама које су подељене са њима.",
    "Board actions" : "Акције табле",
    "Scroll sideways" : "Скролуј бочно",
    "Navigate between cards" : "Кретање по картицама",
    "Close card details" : "Затвори детаљи картице",
    "Search" : "Претрага",
    "Show card filters" : "Прикажи филтере картице",
    "Clear card filters" : "Уклони филтере картице",
    "Show those shortcuts" : "Прикажи те пречице",
    "Card actions" : "Акције картице",
    "Open card details" : "Отвори детаље картице",
    "Edit the card title" : "Уреди наслов картице",
    "Assign yourself to the current card" : "Доделите себе текућој картици",
    "Archive/unarchive the current card" : "Архивирај/уклони из архиве текућу картицу",
    "Mark card as completed/not completed" : "Означи картицу као завршену/незавршену",
    "Open card menu" : "Отвори мени картице",
    "Currently present people" : "Тренутно присутне особе",
    "Loading board" : "Учитавам таблу",
    "Board not found" : "Табла није нађена",
    "Create a new list to add cards to this board" : "Направите нови списак да додате картице на ову таблу",
    "Sharing" : "Дељење",
    "Tags" : "Ознаке",
    "Deleted items" : "Обрисане ставке",
    "Activity" : "Активност",
    "Deleted lists" : "Обрисани спискови",
    "Undo" : "Опозови",
    "Deleted cards" : "Обрисане картице",
    "Hours" : "Сати",
    "Days" : "Дана",
    "Weeks" : "Недеља",
    "Months" : "Месеци",
    "No cards yet" : "Још нема картица",
    "Failed to create share with {displayName}" : "Грешка у прављењу дељења са {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Да ли сте сигурни да кориснику {user} пренесете таблу {title}?",
    "Transfer the board." : "Пренос табле.",
    "Transfer" : "Пренеси",
    "The board has been transferred to {user}" : "Табла је пренета кориснику {user}",
    "Failed to transfer the board to {user}" : "Није успео пренос табле кориснику {user}",
    "Board owner" : "Власник табле",
    "(Group)" : "(група)",
    "(Team)" : "(Тим)",
    "(remote)" : "(удаљено)",
    "Can edit" : "Може да мења",
    "Can share" : "Може да дели",
    "Can manage" : "Може да управља",
    "Owner" : "Власник",
    "Delete" : "Избриши",
    "List deleted" : "Листа обрисана",
    "Edit list title" : "Уреди наслов листе",
    "Archive all cards" : "Архивирај све картице",
    "Unarchive all cards" : "Врати све картице из архиве",
    "Delete list" : "Обриши списак",
    "Archive all cards in this list" : "Архивирај све картице са овог списка",
    "Unarchive all cards in this list" : "Враћа из архиве све картице у овој листи",
    "Add a new card" : "Додај нову картицу",
    "Card name" : "Име картице",
    "title and color value must be provided" : "морају се дати вредности за наслов и боју",
    "Edit" : "Измени",
    "Add a new tag" : "Додај нову ознаку",
    "Board name" : "Име табле",
    "Members" : "Чланови",
    "Assign to users/groups/team" : "Додели корисницима/групама/тиму",
    "Assign a user to this card…" : "Додели корисника овој картици…",
    "Select a user to assign to this card…" : "Изаберите корисника који ће се доделити овој картици",
    "File to share" : "Фајл за дељење",
    "Invalid path selected" : "Одабрана неисправна путања",
    "Upload new files" : "Отпреми нове фајлове",
    "Share from Files" : "Подели из Фајлова",
    "Pending share" : "Дељење на чекању",
    "Add this attachment" : "Додај овај прилог",
    "Show in Files" : "Прикажи у Фајловима",
    "Download" : "Преузми",
    "Remove attachment" : "Уклони прилог",
    "Delete Attachment" : "Обриши прилог",
    "Restore Attachment" : "Поврати прилог",
    "The title cannot be empty." : "Наслов не може бити празан.",
    "Cannot close unsaved card!" : "Не може да се затвори несачувана картица!",
    "Open in sidebar view" : "Отвори у бочном прегледу",
    "Open in bigger view" : "Отвори на већем приказу",
    "Comments" : "Коментари",
    "Failed to load comments" : "Није успело учитавање коментара",
    "No comments yet. Begin the discussion!" : "Нема још коментара. Започните дискусију!",
    "The comment cannot be empty." : "Коментар не може да буде празан.",
    "The comment cannot be longer than 1000 characters." : "Коментар не може да има преко 1000 карактера.",
    "Submit" : "Пошаљи",
    "Created:" : "Направљен:",
    "In reply to" : "Као одговор на",
    "Cancel reply" : "Откажи одговор",
    "Reply" : "Одговори",
    "Update" : "Ажурирај",
    "Mark as done" : "Означи као завршен",
    "Mark as not done" : "Означи да није завршена",
    "Write a description …" : "Напишите опис ...",
    "Could not save description" : "Не може да се сачува опис",
    "Description" : "Опис",
    "(Unsaved)" : "(несачуван)",
    "(Saving…)" : "(чувам…)",
    "Formatting help" : "Помоћ око форматирања",
    "Edit description" : "Измени опис",
    "View description" : "Погледај опис",
    "Add Attachment" : "Додај прилог",
    "Choose attachment" : "Одабери прилог",
    "Select Date" : "Одаберите датум",
    "Later today – {timeLocale}" : "Касније данас – {timeLocale}",
    "Set due date for later today" : "Постави датум доспећа на касније данас",
    "Tomorrow – {timeLocale}" : "Сутра – {timeLocale}",
    "Set due date for tomorrow" : "Постави датум доспећа на сутра",
    "This weekend – {timeLocale}" : "Овог викенда – {timeLocale}",
    "Set due date for this weekend" : "Постави датум доспећа на овај викенд",
    "Next week – {timeLocale}" : "Наредне недеље – {timeLocale}",
    "Set due date for next week" : "Постави датум доспећа на наредну седмицу",
    "Assign a due date to this card…" : "Додели крајњи рок овој картици...",
    "Set a due date" : "Постави рок",
    "Add due date" : "Додај датум доспећа",
    "Choose a date" : "Изаберите датум",
    "Remove due date" : "Уклони рок",
    "Due at:" : "Рок завршетка:",
    "Not done" : "Није завршено",
    "Unarchive card" : "Врати картицу из архиве",
    "Archive card" : "Архивирај картицу",
    "Assign a tag to this card…" : "Додели ознаку овој картици…",
    "Select or create a tag…" : "Изаберите или креирајте ознаку",
    "Create a new tag:" : "Креирај нову ознаку:",
    "(group)" : "(група)",
    "{count} comments, {unread} unread" : "{count} коментара, {unread} није прочитано",
    "Todo items" : "Ставке обавеза",
    "Edit card title" : "Уреди наслов картице",
    "Open link" : "Отвори линк",
    "Card deleted" : "Картица обрисана",
    "Edit title" : "Измени наслов",
    "Assign to me" : "Додели мени",
    "Unassign myself" : "Склони са мене",
    "Delete card" : "Обриши картицу",
    "All boards" : "Све табле",
    "Archived boards" : "Архивиране табле",
    "Shared with you" : "Дељено са Вама",
    "Cancel edit" : "Поништи измену",
    "Save board" : "Сачувај таблу",
    "Board {0} deleted" : "Табла {0} обрисана",
    "All cards" : "Све картице",
    "Only assigned cards" : "Само додељене картице",
    "No reminder" : "Нема подсетника",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Да ли сте сигурни да желите да обришете таблу {title}? Ово ће да обрише све податке на овој табли, укључијући и архивиране картице.",
    "Delete the board?" : "Обрисати таблу?",
    "Leave" : "Напуштање",
    "Exporting board..." : "Табла се извози",
    "An error occurred" : "Догодила се грешка",
    "Board details" : "Детаљи табле",
    "Edit board" : "Измени таблу",
    "Clone board" : "Клонирај таблу",
    "Unarchive board" : "Врати таблу из архиве",
    "Archive board" : "Архивирај таблу",
    "Export board" : "Извези таблу",
    "Turn on due date reminders" : "Укључи подсетнике о року",
    "Turn off due date reminders" : "Искључи подсетнике о року",
    "Due date reminders" : "Подсетници о року",
    "Assigned cards" : "Додељене картице",
    "No notifications" : "Нема обавештења",
    "Delete board" : "Избриши таблу",
    "Importing board..." : "Табла се увози",
    "Board imported successfully" : "Табла је успешно увезена",
    "Import board" : "Увези таблу",
    "Clone {boardTitle}" : "Клонирај {boardTitle}",
    "Clone cards" : "Клонирај картице",
    "Clone assignments" : "Клонирај доделе",
    "Clone labels" : "Клонирај ознаке",
    "Clone due dates" : "Клонирај датуме завршетка",
    "Advanced options" : "Напредне опције",
    "Move all cards to the first list" : "Премести све картице на прву листу",
    "Restore archived cards" : "Врати архивиране картице",
    "Clone" : "Клонирај",
    "Export {boardTitle}" : "Извези {boardTitle}",
    "Export as JSON" : "Извези као JSON",
    "Export as CSV" : "Извези као CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Напомена: само JSON формат се подржава за увоз назад у апликацију Шпил.",
    "Export" : "Извези",
    "Loading filtered view" : "Учитам филтрирани преглед",
    "Search for {searchQuery} in other boards" : "Претражи {searchQuery} у осталим таблама",
    "Search for {searchQuery} in all boards" : "Тражи се {searchQuery} у свим таблама",
    "No results found" : "Нема пронађених резултата",
    "Deck board {name}\n* Last modified on {lastMod}" : "Шпил табла {name}\n* Последњи пут измењена дана {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Креирано дана {created}\n* Последњи пут измењено дана {lastMod}\n* {nbAttachments} прилога\n* {nbComments} коментара",
    "{nbCards} cards" : "{nbCards} картица",
    "Due on {date}" : "Рок је {date}",
    "{stack} in {board}" : "{stack} у {board}",
    "Click to expand description" : "Кликните да проширите опис",
    "Click to expand comment" : "Кликните да проширите коментар",
    "Create card" : "Направите картицу",
    "Create a new card" : "Креирај нову картицу",
    "Card title" : "Наслов картицњ",
    "Creating the new card …" : "Креира се нова картица ...",
    "Card \"{card}\" was added to \"{board}\"" : "Картица „{card}” је додта на „{board}",
    "Open card" : "Отвори картицу",
    "Close" : "Затвори",
    "No upcoming cards" : "Нема предстојећих картица",
    "upcoming cards today" : "предстојеће картице данас",
    "upcoming cards tomorrow" : "предстојеће картице сутра",
    "upcoming cards" : "предстојеће картице",
    "New card" : "Нова картица",
    "Link to a board" : "Веза ка табли",
    "Link to a card" : "Веза ка картици",
    "Create a card" : "Креирај картицу",
    "Message from {author} in {conversationName}" : "Порука од {author} у {conversationName}",
    "Something went wrong" : "Нешто је пошло наопако",
    "Failed to upload {name}" : "Није успело отпремање {name}",
    "Maximum file size of {size} exceeded" : "Премашена максимална величина фајла од {size}",
    "Assigned users" : "Додељени корисници",
    "Due date" : "Рок",
    "Created" : "Направљен",
    "Modified" : "Измењен",
    "Error creating the share" : "Грешка при прављењу дељења",
    "Share with a Deck card" : "Дели са Шпил картицом",
    "Share {file} with a Deck card" : "Дели {file} са Шпил картицом",
    "Share" : "Подели",
    "The card \"%s\" on \"%s\" has reached its due date." : "Картици „%s“ на табли „%s“ је истекао рок.",
    "Share board with a user, group or team …" : "Делите таблу са корисником, групом или тимом…",
    "Searching for users, groups and teams …" : "Траже се корисници, групе и тимови…",
    "No participants found" : "Нема нађених учесника",
    "Save" : "Сачувај",
    "seconds ago" : "пре неколико секунди",
    "Today" : "Данас",
    "Tomorrow" : "сутра",
    "No due" : "Нема рокова"
},
"nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2);");
