OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Ви створили нову дошку {board}",
    "{user} has created a new board {board}" : "{user} створив(-ла) нову дошку {board}",
    "You have deleted the board {board}" : "Ви вилучили дошку {board}",
    "{user} has deleted the board {board}" : "{user} вилучив дошку {board}",
    "You have restored the board {board}" : "Ви відновили дошку {board}",
    "{user} has restored the board {board}" : "{user} відновив дошку {board}",
    "You have shared the board {board} with {acl}" : "Ви поділилися дошкою {board} з {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} поділив(-ла-)ся дошкою {board} з {acl}",
    "You have removed {acl} from the board {board}" : "Ви вилучили {acl} з дошки {board}",
    "{user} has removed {acl} from the board {board}" : "{user} вилучив {acl} з дошки {board}",
    "You have renamed the board {before} to {board}" : "Ви перейменували дошку з {before} на {board}",
    "{user} has renamed the board {before} to {board}" : "{user} змінив(-ла) назву дошки {before} на {board}",
    "You have archived the board {board}" : "Ви заархівували дошку {board}",
    "{user} has archived the board {before}" : "{user} заархівував дошку {before}",
    "You have unarchived the board {board}" : "Ви розархівували дошку {board}",
    "{user} has unarchived the board {before}" : "{user} розархівував дошку {before}",
    "You have created a new list {stack} on board {board}" : "Ви створили новий список {stack} на дошці {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} створив(-ла) новий список {stack} на дошці {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Ви перейменували список {before} на {stack} на дошці {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} змінив(-ла) назву списку {before} на {stack} на дошці {board}",
    "You have deleted list {stack} on board {board}" : "Ви видучили список {stack} з дошки {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} вилучив(-ла) список {stack} на дошці {board}",
    "You have created card {card} in list {stack} on board {board}" : "Ви створили картку {card} у списку {stack} на дошці {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} створив(-ла) картку {card} у списку {stack} на дошці {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Ви вилучили картку {card} зі списку {stack} на дошці {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} вилучив(-ла) картку {card} у списку {stack} на дошці {board}",
    "You have renamed the card {before} to {card}" : "Ви перейменували картку {before} на {card}",
    "{user} has renamed the card {before} to {card}" : "{user} змінив(-ла) назву картки {before} на {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Ви додали опис картки {card} у списку {stack} на дошці {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} додав(-ла) опис до картки {card} у списку {stack} на дошці {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Ви оновили опис картки {card} у списку {stack} на дошці {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} оновив(-ла) опис до картки {card} у списку {stack} на дошці {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Ви архівували картку {card} у списку {stack} на дошці {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} архівув(-ла) картку {card} у списку {stack} на дошці {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Ви розархівували картку {card} у списку {stack} на дошці {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} розархівував(-ла) картку {card} у списку {stack} на дошці {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Ви позначили як виконану картку {card} у списку {stack} на дошці {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} позначив(-ла) як виконану картку {card} у списку {stack} на дошці {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Ви позначили як не виконану картку {card} у списку {stack} на дошці {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} позначив(-ла) як не виконану картку {card} у списку {stack} на дошці {board}",
    "You have removed the due date of card {card}" : "Ви вилучили дату завершення картки {card}",
    "{user} has removed the due date of card {card}" : "{user} вилучив(-ла) дату завершення картки {card}",
    "You have set the due date of card {card} to {after}" : "Ви встановили дату завершення картки {card} на {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} встановив(-ла) дату завершення картки {card} на {after}",
    "You have updated the due date of card {card} to {after}" : "Ви оновили дату завершення картки {card} на {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} оновив(-ла) дату завершення картки {card} на {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Ви додали мітку {label} до картки {card} в списку {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} додав(-ла) мітку  {label} до картки {card} у списку {stack} на дошці {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Ви вилучили мітку {label} з картки {card} у списку {stack} на дошці {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} вилучив(-ла) мітку {label} з картки {card} у списку {stack} на дошці {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Ви призначили {assigneduser} до картки {card} на дошці {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} призначив {assigneduser} на картку {card} на дошці {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Ви скасували призначення {assigneduser} з картки {card} на дошці {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} скасував призначення {assigneduser} з карти {card} на дошці {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Ви перемістили картку {card} зі списку {stackBefore} до {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} перемістив картку {card} зі списку {stackBefore} до {stack}",
    "You have added the attachment {attachment} to card {card}" : "Ви додали вкладення {attachment} до картки {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} додав вкладення {attachment} до картки {card}",
    "You have updated the attachment {attachment} on card {card}" : "Ви оновили вкладення {attachment} на картці {card}",
    "{user} has updated the attachment {attachment} on card {card}" : " {user}оновив вкладення {attachment} на картці {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Ви вилучили вкладення {attachment} з картки {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} вилучив(-ла) вкладення {attachment} з картки {card}",
    "You have restored the attachment {attachment} to card {card}" : "Ви відновили вкладення {attachment} до картки {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} відновив вкладення {attachment} до картки {card}",
    "You have commented on card {card}" : "Ви залишили коментар до картки {card}",
    "{user} has commented on card {card}" : "{user} прокоментував картку {card}",
    "Deck" : "Колода",
    "Changes in the <strong>Deck app</strong>" : "Зміни у застосунку <strong>Колода</strong>",
    "A <strong>board, list or card</strong> was changed" : "Змінено <strong>таблицю, список або картку</strong>",
    "A <strong>comment</strong> was created on a card" : "Створено <strong>коментар</strong> на картці",
    "A <strong>card description</strong> has been changed" : "Змінено <strong>опис картки</strong>",
    "The file was uploaded" : "Файл завантажено",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Файл для завантаження перевищує параметр upload_max_filesize у php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Розмір завантаженого файлу перевищує значення MAX_FILE_SIZE, яке було зазначено у HTML формі",
    "The file was only partially uploaded" : "Файл завантажено частково",
    "No file was uploaded" : "Жодного файлу не було завантажено",
    "Missing a temporary folder" : "Відсутній каталог тимчасових файлів",
    "Could not write file to disk" : "Неможливо записати файл на диск",
    "A PHP extension stopped the file upload" : "Розширення PHP призупинило завантаження файлу",
    "No file uploaded or file size exceeds maximum of %s" : "Немає завантажених файлів або розмір файлу перевищує максимум %s",
    "Invalid file type. Only JSON files are allowed." : "Неправильний тип файлу. Допускаються тільки файли у форматі JSON.",
    "Invalid JSON data" : "Неправильні дані JSON",
    "Failed to import board" : "Не вдалося імпортувати дошку",
    "Cards due today" : "Картки повинні бути оплачені сьогодні",
    "Cards due tomorrow" : "Картки на завтра",
    "Upcoming cards" : "Очікують на виконання",
    "Load more" : "Більше",
    "Welcome to Nextcloud Deck!" : "Ласкаво просимо до Колоди для Nextcloud!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Картка \"%s\" на \"%s\" була призначена вам %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} призначив вам карту {deck-card} на {deck-board}.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Досягнуто кінцеву дату у картці \"%1$s\" із \"%2$s\".",
    "The card {deck-card} on {deck-board} has reached its due date." : "Термін дії карти {deck-card} на {deck-board} закінчився.",
    "%s has mentioned you in a comment on \"%s\"." : "%s згадав вас у коментарі до \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} згадав вас у коментарі до {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Дошку \"%s\" з вами поділився %s.",
    "{user} has shared {deck-board} with you." : "{user} поділився з вами {deck-board}.",
    "The remote board %1$s has been shared with you by %2$s" : "%2$s поділив(ла)ся з тобою віддаленою дошкою %1$s",
    "Deck board" : "Дошка колоди",
    "Owned by %1$s" : "Належить %1$s",
    "Deck boards, cards and comments" : "Дошки колод, карти та коментарі",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "З %1$s, в %2$s/%3$s, у власності %4$s",
    "Create a new deck card" : "Створіть нову карту колоди",
    "Card comments" : "Коментарі до картки",
    "%s on %s" : "%s на %s",
    "Deck boards and cards" : "Колода та карти",
    "No data was provided to create an attachment." : "Для створення вкладення не було надано жодних даних.",
    "Finished" : "Завершено",
    "To review" : "На перегляд",
    "Action needed" : "Потребує дій",
    "Later" : "Пізніше",
    "copy" : "копіювати",
    "Read more inside" : "Читайте більше всередині",
    "Custom lists - click to rename!" : "Користувацькі списки - натисніть, щоб перейменувати!",
    "To Do" : "Зробити",
    "In Progress" : "В процесі",
    "Done" : "Готово",
    "1. Open to learn more about boards and cards" : "1. Відкриті, щоб дізнатися більше про дошки та картки",
    "2. Drag cards left and right, up and down" : "2. Перетягування карток вліво і вправо, вгору і вниз",
    "3. Apply rich formatting and link content" : "3. Застосовуйте багате форматування та посилання на вміст",
    "4. Share, comment and collaborate!" : "4. Діліться, коментуйте та співпрацюйте!",
    "Create your first card!" : "Створіть свою першу листівку!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Цей коментар містить більше %s символів.\nДодано як вкладення до картки з іменем %s.\nДоступний за URL: %s.",
    "Attachments" : "Вкладення",
    "File" : "Файл",
    "date" : "дата",
    "Card not found" : "Картку не знайдено",
    "Path is already shared with this card" : "Шлях вже є спільним з цією карткою",
    "Invalid date, date format must be YYYY-MM-DD" : "Недійсна дата, формат дати має бути РРРР-ММ-ДД",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Всі ваші дошки, включно стеки, картки, ярлики, призначення та коментарі",
    "Personal planning and team project organization" : "Особисте планування та організація командних проектів",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck - це інструмент організації в стилі канбан, призначений для особистого планування та організації проектів для команд, інтегрованих з Nextcloud.\n\n\n- 📥 Додавайте свої завдання на картки та впорядковуйте їх\n- 📄 Записуйте додаткові нотатки в Markdown\n- 🔖 Призначайте мітки для ще кращої організації\n- 👥 Діліться з командою, друзями чи родиною\n- Прикріплюйте файли та вбудовуйте їх в опис Markdown\n- 💬 Обговорюйте зі своєю командою за допомогою коментарів\n- ⚡ Відстежуйте зміни в потоці активностей\n- 🚀 Організуйте свій проект",
    "Add board" : "Додати дошку",
    "Card details" : "Деталі картки",
    "Select the board to link to a project" : "Виберіть дошку для прив'зки до проєкту",
    "Search by board title" : "Шукати за назвою дошки",
    "Select board" : "Вибрати дошку",
    "Move/copy card" : "Картка переміщення/копіювання",
    "Select a board" : "Вибрати дошку",
    "No lists available" : "Списки відсутні",
    "Select a list" : "Виберіть список",
    "Move card" : "Пересунути картку",
    "Copy card" : "Копіювальна картка",
    "Select the card to link to a project" : "Виберіть картку для прив'язки до проєкту",
    "Link to card" : "Прив'язати до картки",
    "Select a card" : "Вибрати картку",
    "Cancel" : "Скасувати",
    "This board is read only" : "Дошка тільки для читання",
    "Drop your files to upload" : "Перетягнути файли для завантаження",
    "File already exists" : "Файл вже існує",
    "A file with the name {filename} already exists." : "Файл {filename} вже існує.",
    "Do you want to overwrite it?" : "Дійсно перезаписати файл?",
    "Overwrite file" : "Перезаписати файл",
    "Keep existing file" : "Зберегти поточний файл",
    "Add card" : "Додати картку",
    "Archived cards" : "Архівні картки",
    "Add list" : "Додати список",
    "List name" : "Назва списку",
    "Clear search" : "Очистити пошук",
    "Active filters" : "Активні фільтри",
    "Apply filter" : "Швидкий фільтр",
    "Filter by tag" : "Відібрати за мітками",
    "Filter by assigned user" : "Відібрати за призначеним користувачем",
    "Unassigned" : "Не призначено",
    "Filter by status" : "Фільтр за статусом",
    "Open and completed" : "Відкрито та завершено",
    "Open" : "Відкрито",
    "Completed" : "Виконано",
    "Filter by due date" : "Відібрати за датою завершення",
    "Overdue" : "Протерміновано",
    "Next 24 hours" : "Наступні 24 години",
    "Next 7 days" : "Наступні 7 днів",
    "Next 30 days" : "Наступні 30 днів",
    "No due date" : "Без дати завершення",
    "Clear filter" : "Очистити фільтр",
    "View Modes" : "Режими перегляду",
    "Toggle View Modes" : "Перемикання режимів перегляду",
    "Kanban view" : "Подання Канбан",
    "Gantt view" : "Подання Ґант",
    "Hide archived cards" : "Приховати архівні картки",
    "Show archived cards" : "Показати архівні картки",
    "Toggle compact mode" : "Перемкнути компактний вигляд",
    "Hide card cover images" : "Приховати зображення обкладинок карток",
    "Show card cover images" : "Показати зображення обкладинки картки",
    "Open details" : "Показати деталі",
    "Details" : "Деталі",
    "Deck settings" : "Налаштування",
    "General" : "Загальне",
    "Use bigger card view" : "Режим перегляду зі збільшеними картками",
    "Appearance" : "Вигляд",
    "Show card ID badge" : "Показувати бейдж з ідентифікаційною карткою",
    "Show boards in calendar/tasks" : "Показувати дошки в календарі та завданнях",
    "Admin settings" : "Налаштування адміністратора",
    "Limit board creation to some groups" : "Обмежити створення дошки для деяких груп",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Користувачі поза цими групами не зможуть створювати власні дошки, але зможуть працювати на дошках, до яких їм надано спільний доступ.",
    "Enable federation" : "Увімкнути функцію об'єднаних хмар",
    "Board actions" : "Дії правління",
    "Scroll sideways" : "Прокрутити вбік",
    "Navigate between cards" : "Перехід між картками",
    "Close card details" : "Закрити дані картки",
    "Search" : "Пошук",
    "Show card filters" : "Показати фільтри карток",
    "Clear card filters" : "Очистити фільтри карток",
    "Show those shortcuts" : "Показувати скорочення клавіш",
    "Card actions" : "Дії з карткою",
    "Open card details" : "Відкрити реквізити картки",
    "Edit the card title" : "Редагувати назву картки",
    "Assign yourself to the current card" : "Приєднайте себе до поточної картки",
    "Archive/unarchive the current card" : "Архівувати/розархівувати поточну картку",
    "Mark card as completed/not completed" : "Позначити картку як завершену/не завершену",
    "Open card menu" : "Відкрити меню картки",
    "Currently present people" : "Наразі присутні люди",
    "Supported prefixes: {prefixes}. Wrap phrases in double quotes." : "Підтримувані префікси: {prefixes}. Огорніть вирази подвійними лапками.",
    "Search cards" : "Пошук карток",
    "Loading board" : "Завантаження дошки",
    "Board not found" : "Дошку не знайдено",
    "Create a new list to add cards to this board" : "Створіть список щоб додати картки на цю дошку",
    "Sharing" : "Поділитися",
    "Tags" : "Мітки",
    "Deleted items" : "Вилучені елементи",
    "Activity" : "Події",
    "Deleted lists" : "Вилучені списки",
    "Undo" : "Скасувати",
    "Deleted cards" : "Вилучені картки",
    "Hours" : "Години",
    "Days" : "Дні",
    "Weeks" : "Тижні",
    "Months" : "Місяці",
    "No cards yet" : "Ще немає жодної картки",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Зазначте у картці дату початку та завершення, щоби ці дані було показано на діаграмі Ґанта ",
    "Cards without dates ({count})" : "Карток без дати ({count})",
    "Failed to create share with {displayName}" : "Не вдалося створити спільний доступ з {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Ви впевнені, що хочете перенести дошку {title} користувачеві {user}?",
    "Transfer the board." : "Перенесіть дошку.",
    "Transfer" : "Передати",
    "The board has been transferred to {user}" : "Дошка була передана {user}",
    "Failed to transfer the board to {user}" : "Не вдалося передати дошку користувачеві {user}",
    "Board owner" : "Власник дошки",
    "(Group)" : "(група)",
    "(Team)" : "(Команда)",
    "(remote)" : "(віддалено)",
    "Can edit" : "Може редагувати",
    "Can share" : "Can share",
    "Can manage" : "Може керувати",
    "Owner" : "Власник",
    "Delete" : "Вилучити",
    "List deleted" : "Список вилучено",
    "Edit list title" : "Редагувати заголовок списку",
    "Archive all cards" : "Архівувати всі картки",
    "Unarchive all cards" : "Розархівувати всі картки",
    "Do not set cards as \"done\"" : "Не встановлювати статус \"зроблено\" на картках",
    "Set cards as \"done\"" : "Встановити статус картки \"зроблено\"",
    "Delete list" : "Вилучити список",
    "Archive all cards in this list" : "Архівуйте всі картки в цьому списку",
    "Unarchive all cards in this list" : "Розархівувати всі картки в цьому списку",
    "Add a new card" : "Додати нову картку",
    "Card name" : "Назва картки",
    "title and color value must be provided" : "потрібно зазначити назву та колір",
    "Edit" : "Редагувати",
    "Add a new tag" : "Додати нову мітку",
    "Search boards" : "Пошук дошок",
    "Board name" : "Назва дошки",
    "Members" : "Учасники",
    "Assign to users/groups/team" : "Призначити користувачам/групам/команді",
    "Assign a user to this card…" : "Призначити користувачеві цю картку",
    "Select a user to assign to this card…" : "Виберіть користувача, якого слід призначити на цю картку...",
    "File to share" : "Виберіть файл для надання доступу",
    "Invalid path selected" : "Вибрано неправильний шлях",
    "Upload new files" : "Додати файл",
    "Share from Files" : "Відкрити Файли",
    "Pending share" : "Очікуваний спільний ресурс",
    "Add this attachment" : "Долучити вкладення",
    "Show in Files" : "Показати в Файлах",
    "Download" : "Звантажити",
    "Remove attachment" : "Вилучити",
    "Delete Attachment" : "Забрати вкладення",
    "Restore Attachment" : "Відновити вкладення",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Змінено: {modifiedDate} ⸱ Створено: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Змінено: {modifiedDate}\nСтворено: {createdDate}\nКим створено: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Змінено: {modifiedDate}\nСтворено: {createdDate}",
    "The title cannot be empty." : "Заголовок не може бути порожнім.",
    "Cannot close unsaved card!" : "Неможливо закрити незбережену картку!",
    "Open in sidebar view" : "Відкрити у вигляді бічної панелі",
    "Open in bigger view" : "Відкрити в більшому розмірі",
    "Comments" : "Коментарі",
    "Failed to load comments" : "Не вдалося завантажити коментарі",
    "No comments yet. Begin the discussion!" : "Коментарів немає, почніть обговорення!",
    "The comment cannot be empty." : "Коментар не може бути порожнім.",
    "The comment cannot be longer than 1000 characters." : "Коментар не може бути довшим за 1000 символів.",
    "Submit" : "Відправити",
    "Created:" : "Створено:",
    "In reply to" : "У відповідь",
    "Cancel reply" : "Скасувати відповідь",
    "Reply" : "Відповісти",
    "Update" : "Оновлення",
    "Assign dependent cards" : "Призначити пов'язані картки",
    "Mark as done" : "Позначити як виконане",
    "Mark as not done" : "Позначити як не виконано",
    "Remove dependency" : "Вилучити залежності",
    "Add dependent card" : "Додати пов'язану картку",
    "Assign a dependent card…" : "Призначити пов'язану картку...",
    "Select a dependent card…" : "Виберіть пов'язану картку...",
    "Write a description …" : "Додайте опис ...",
    "The description has been changed by another user." : "Опис змінено іншим користувачем. ",
    "Could not save description" : "Не вдалося зберегти опис",
    "Description" : "Опис",
    "(Unsaved)" : "(Не збережено)",
    "(Saving…)" : "(Зберігання...)",
    "Formatting help" : "Допомога",
    "Edit description" : "Редагувати опис",
    "View description" : "Переглянути опис",
    "Add Attachment" : "Долучити вкладення",
    "Choose attachment" : "Вибрати вкладення",
    "Select Date" : "Вкажіть дату",
    "Later today – {timeLocale}" : "Пізніше сьогодні - {timeLocale}",
    "Set due date for later today" : "Встановіть термін завершення на пізніше сьогодні",
    "Tomorrow – {timeLocale}" : "Завтра - {timeLocale}",
    "Set due date for tomorrow" : "Встановіть дедлайн на завтра",
    "This weekend – {timeLocale}" : "На цих вихідних - {timeLocale}",
    "Set due date for this weekend" : "Встановіть термін виконання на ці вихідні",
    "Next week – {timeLocale}" : "Наступного тижня - {timeLocale}",
    "Set due date for next week" : "Встановіть термін виконання на наступний тиждень",
    "Assign a due date to this card…" : "Призначте цій картці термін завершення...",
    "Set a due date" : "Встановити дату завершення",
    "Add due date" : "Додати термін завершення",
    "Choose a date" : "Виберіть дату",
    "Remove due date" : "Вилучити кінцеву дату",
    "Due at:" : "На часі:",
    "Not done" : "Ще не все.",
    "Unarchive card" : "Розархівувати картку",
    "Archive card" : "Архівувати картку",
    "Assign a start date to this card…" : "Призначити дату початку цій картці...",
    "Set a start date" : "Встановити дату початку",
    "Add start date" : "Додати дату початку",
    "Remove start date" : "Вилучити дату початку",
    "Assign a tag to this card…" : "Додати мітку до цієї картки",
    "Select or create a tag…" : "Виберіть або створіть мітку ...",
    "Create a new tag:" : "Створіть нову мітку:",
    "(group)" : "(група)",
    "{count} comments, {unread} unread" : "{count} коментарів, {unread} непрочитаних",
    "Todo items" : "Пункти завдань",
    "Edit card title" : "Редагувати заголовок картки",
    "Open link" : "Відкрити посилання",
    "Card deleted" : "Картку вилучено",
    "Change card color" : "Змінити колір картки",
    "Edit title" : "Редагувати",
    "Assign to me" : "Призначити мені",
    "Unassign myself" : "Зняти з себе повноваження.",
    "Delete card" : "Вилучити картку",
    "All boards" : "Усі дошки",
    "Archived boards" : "Архівні дошки",
    "Shared with you" : "Вам надано доступ",
    "Cancel edit" : "Скасувати редагування",
    "Save board" : "Зберегти дошку",
    "Board {0} deleted" : "Дошку {0} вилучено",
    "All cards" : "Всі карти",
    "Only assigned cards" : "Тільки призначені картки",
    "No reminder" : "Відсутні нагадування",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Ви впевнені, що хочете вилучити дошку {title}? Це призведе до повного вилучення всіх даних цієї дошки, включно архівні картки.",
    "Delete the board?" : "Вилучити дошку?",
    "Are you sure you want to leave the board {title}?" : "Дійсно залишити дошку {title}?",
    "Leave the board?" : "Залишити дошку?",
    "Leave" : "Залишити",
    "Failed to leave the board" : "Не вдалося залишити дошку",
    "Exporting board..." : "Експортна дошка...",
    "An error occurred" : "Виникла помилка",
    "Board details" : "Деталі дошки",
    "Edit board" : "Перейменувати",
    "Clone board" : "Копіювати дошку",
    "Unarchive board" : "Розархівувати дошку",
    "Archive board" : "Архівувати дошку",
    "Export board" : "Дошка експорту",
    "Turn on due date reminders" : "Нагадування про термін виконання",
    "Turn off due date reminders" : "Вимкнути нагадування про терміни виконання",
    "Remove as default board" : "Вилучити типову дошку",
    "Set as default board" : "Встановити типовою дошкою",
    "Due date reminders" : "Нагадування про терміни виконання",
    "Assigned cards" : "Призначені картки",
    "No notifications" : "Немає сповіщень",
    "Delete board" : "Вилучити дошку",
    "Leave board" : "Залишити дошку",
    "Importing board..." : "Імпортування дошки...",
    "Board imported successfully" : "Дошку успішно імпортовано",
    "Import board" : "Імпорт дошки",
    "Clone {boardTitle}" : "Копіювати {boardTitle}",
    "Clone cards" : "Копіювати картки",
    "Clone assignments" : "Призначення копій",
    "Clone labels" : "Етикетки копій",
    "Clone due dates" : "Копіювати дати завершення",
    "Advanced options" : "Розширені налаштування",
    "Move all cards to the first list" : "Перемістіть усі картки до першого списку",
    "Restore archived cards" : "Відновлення заархівованих карток",
    "Clone" : "Копіювати",
    "Export {boardTitle}" : "Експорт {boardTitle}",
    "Export as JSON" : "Експорт як JSON",
    "Export as CSV" : "Експорт у CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Примітка: Дані для імпорту до застосунку Колоди підтримуються лише у форматі JSON.",
    "Export" : "Експорт",
    "Loading filtered view" : "Завантаження відфільтрованого перегляду",
    "Search for {searchQuery} in other boards" : "Шукати {searchQuery} на інших дошках",
    "Search for {searchQuery} in all boards" : "Шукати {searchQuery} на всіх дошках оголошень",
    "No results found" : "Не знайдено жодного результату",
    "Deck board {name}\n* Last modified on {lastMod}" : "Колода {name}\n* Востаннє змінено на {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Створено на {created}\n* Востаннє змінено на {lastMod}\n* {nbAttachments} вкладення\n* {nbComments} коментарі",
    "{nbCards} cards" : "{nbCards} картки",
    "Due on {date}" : "До {date}",
    "{stack} in {board}" : "{stack} в {board}",
    "Click to expand description" : "Натисніть, щоб розгорнути опис",
    "Click to expand comment" : "Натисніть, щоб розгорнути коментар",
    "Create card" : "Створити картку",
    "Create a new card" : "Створити нову картку",
    "Card title" : "Заголовок картки",
    "Creating the new card …" : "Створення нової картки ...",
    "Card \"{card}\" was added to \"{board}\"" : "Картку \"{card}\" додано до \"{board}\"",
    "Open card" : "Відкрити картку",
    "Close" : "Закрити",
    "No upcoming cards" : "Немає майбутніх завдань",
    "upcoming cards today" : "найближчі картки на сьогодні",
    "upcoming cards tomorrow" : "найближчі картки завтра",
    "upcoming cards" : "очікують на виконання",
    "New card" : "Нова картка",
    "Link to a board" : "Прив'язати до дошки",
    "Link to a card" : "Прив'язати до картки",
    "Create a card" : "Створити картку",
    "Message from {author} in {conversationName}" : "Повідомлення від {author} у {conversationName}",
    "Something went wrong" : "От халепа!",
    "Failed to upload {name}" : "Не вдалося завантажити {name}",
    "Maximum file size of {size} exceeded" : "Досягнуто максимальний розмір файлу {size}",
    "Assigned users" : "Призначені користувачі",
    "Due date" : "Дата виконання",
    "Created" : "Створено",
    "Modified" : "Змінено",
    "Error creating the share" : "Помилка створення спільного доступу",
    "Share with a Deck card" : "Поділітися з карткою колоди",
    "Share {file} with a Deck card" : "Поділітися {file} з карткою з колоди",
    "Share" : "Спільний доступ",
    "The card \"%s\" on \"%s\" has reached its due date." : "Термін дії картки \"%s\" на \"%s\" закінчився.",
    "Share board with a user, group or team …" : "Спільний доступ до дошки з користувачем, групою або командою ...",
    "Searching for users, groups and teams …" : "Пошук користувачів, груп і команд ...",
    "No participants found" : "Учасників не знайдено",
    "Save" : "Зберегти",
    "seconds ago" : "секунд тому",
    "Today" : "Сьогодні",
    "Tomorrow" : "Завтра",
    "No due" : "Без призначеної дати"
},
"nplurals=4; plural=(n % 1 == 0 && n % 10 == 1 && n % 100 != 11 ? 0 : n % 1 == 0 && n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 12 || n % 100 > 14) ? 1 : n % 1 == 0 && (n % 10 ==0 || (n % 10 >=5 && n % 10 <=9) || (n % 100 >=11 && n % 100 <=14 )) ? 2: 3);");
