OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "شما یک بورد جدید به نام {board} ایجاد کرده‌اید",
    "{user} has created a new board {board}" : "{user} یک بورد جدید به نام {board} ایجاد کرده است",
    "You have deleted the board {board}" : "شما بورد {board} را حذف کرده‌اید",
    "{user} has deleted the board {board}" : "{user} بورد {board} را حذف کرده است",
    "You have restored the board {board}" : "شما بورد {board} را بازیابی کرده‌اید",
    "{user} has restored the board {board}" : "{user} بورد {board} را بازیابی کرده است",
    "You have shared the board {board} with {acl}" : "شما بورد {board} را با {acl} به اشتراک گذاشته‌اید",
    "{user} has shared the board {board} with {acl}" : "{user} بورد {board} را با {acl} به اشتراک گذاشته است",
    "You have removed {acl} from the board {board}" : "شما {acl} را از بورد {board} حذف کرده‌اید",
    "{user} has removed {acl} from the board {board}" : "{user} {acl} را از بورد {board} حذف کرده است",
    "You have renamed the board {before} to {board}" : "شما نام بورد را از {before} به {board} تغییر داده‌اید",
    "{user} has renamed the board {before} to {board}" : "{user} نام بورد را از {before} به {board} تغییر داده است",
    "You have archived the board {board}" : "شما بورد {board} را بایگانی کرده‌اید",
    "{user} has archived the board {before}" : "{user} بورد {before} را بایگانی کرده است",
    "You have unarchived the board {board}" : "شما بورد {board} را از بایگانی خارج کرده‌اید",
    "{user} has unarchived the board {before}" : "{user} بورد {before} را از بایگانی خارج کرده است",
    "You have created a new list {stack} on board {board}" : "شما یک لیست جدید به نام {stack} در بورد {board} ایجاد کرده‌اید",
    "{user} has created a new list {stack} on board {board}" : "{user} یک لیست جدید به نام {stack} در بورد {board} ایجاد کرده است",
    "You have renamed list {before} to {stack} on board {board}" : "شما نام لیست را از {before} به {stack} در بورد {board} تغییر داده‌اید",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} نام لیست را از {before} به {stack} در بورد {board} تغییر داده است",
    "You have deleted list {stack} on board {board}" : "شما لیست {stack} را در بورد {board} حذف کرده‌اید",
    "{user} has deleted list {stack} on board {board}" : "{user} لیست {stack} را در بورد {board} حذف کرده است",
    "You have created card {card} in list {stack} on board {board}" : "شما کارت {card} را در لیست {stack} در بورد {board} ایجاد کرده‌اید",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} کارت {card} را در لیست {stack} در بورد {board} ایجاد کرده است",
    "You have deleted card {card} in list {stack} on board {board}" : "شما کارت {card} را در لیست {stack} در بورد {board} حذف کرده‌اید",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} کارت {card} را در لیست {stack} در بورد {board} حذف کرده است",
    "You have renamed the card {before} to {card}" : "شما نام کارت را از {before} به {card} تغییر داده‌اید",
    "{user} has renamed the card {before} to {card}" : "{user} نام کارت را از {before} به {card} تغییر داده است",
    "You have added a description to card {card} in list {stack} on board {board}" : "شما توضیحی به کارت {card} در لیست {stack} در بورد {board} اضافه کرده‌اید",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} توضیحی به کارت {card} در لیست {stack} در بورد {board} اضافه کرده است",
    "You have updated the description of card {card} in list {stack} on board {board}" : "شما توضیحات کارت {card} را در لیست {stack} در بورد {board} به‌روزرسانی کرده‌اید",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} توضیحات کارت {card} را در لیست {stack} در بورد {board} به‌روزرسانی کرده است",
    "You have archived card {card} in list {stack} on board {board}" : "شما کارت {card} را در لیست {stack} در بورد {board} بایگانی کرده‌اید",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} کارت {card} را در لیست {stack} در بورد {board} بایگانی کرده است",
    "You have unarchived card {card} in list {stack} on board {board}" : "شما کارت {card} را در لیست {stack} در بورد {board} از بایگانی خارج کرده‌اید",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} کارت {card} را در لیست {stack} در بورد {board} از بایگانی خارج کرده است",
    "You have marked the card {card} as done in list {stack} on board {board}" : "شما کارت {card} را در لیست {stack} در بورد {board} به عنوان انجام شده علامت زده‌اید",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} کارت {card} را در لیست {stack} در بورد {board} به عنوان انجام شده علامت زده است",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "شما کارت {card} را در لیست {stack} در بورد {board} به عنوان انجام نشده علامت زده‌اید",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} کارت {card} را در لیست {stack} در بورد {board} به عنوان انجام نشده علامت زده است",
    "You have removed the due date of card {card}" : "شما تاریخ سررسید کارت {card} را حذف کرده‌اید",
    "{user} has removed the due date of card {card}" : "{user} تاریخ سررسید کارت {card} را حذف کرده است",
    "You have set the due date of card {card} to {after}" : "شما تاریخ سررسید کارت {card} را به {after} تنظیم کرده‌اید",
    "{user} has set the due date of card {card} to {after}" : "{user} تاریخ سررسید کارت {card} را به {after} تنظیم کرده است",
    "You have updated the due date of card {card} to {after}" : "شما تاریخ سررسید کارت {card} را به {after} به‌روزرسانی کرده‌اید",
    "{user} has updated the due date of card {card} to {after}" : "{user} تاریخ سررسید کارت {card} را به {after} به‌روزرسانی کرده است",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "شما برچسب {label} را به کارت {card} در لیست {stack} در بورد {board} اضافه کرده‌اید",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} برچسب {label} را به کارت {card} در لیست {stack} در بورد {board} اضافه کرده است",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "شما برچسب {label} را از کارت {card} در لیست {stack} در بورد {board} حذف کرده‌اید",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} برچسب {label} را از کارت {card} در لیست {stack} در بورد {board} حذف کرده است",
    "You have assigned {assigneduser} to card {card} on board {board}" : "شما کاربر {assigneduser} را به کارت {card} در بورد {board} اختصاص داده‌اید",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "کاربر {user} کاربر {assigneduser} را به کارت {card} در بورد {board} اختصاص داده است",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "شما کاربر {assigneduser} را از کارت {card} در بورد {board} حذف کرده‌اید",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "کاربر {user} کاربر {assigneduser} را از کارت {card} در بورد {board} حذف کرده است",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "شما کارت {card} را از لیست {stackBefore} به {stack} منتقل کرده‌اید",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "کاربر {user} کارت {card} را از لیست {stackBefore} به {stack} منتقل کرده است",
    "You have added the attachment {attachment} to card {card}" : "شما پیوست {attachment} را به کارت {card} اضافه کرده‌اید",
    "{user} has added the attachment {attachment} to card {card}" : "کاربر {user} پیوست {attachment} را به کارت {card} اضافه کرده است",
    "You have updated the attachment {attachment} on card {card}" : "شما پیوست {attachment} را در کارت {card} به‌روزرسانی کرده‌اید",
    "{user} has updated the attachment {attachment} on card {card}" : "کاربر {user} پیوست {attachment} را در کارت {card} به‌روزرسانی کرده است",
    "You have deleted the attachment {attachment} from card {card}" : "شما پیوست {attachment} را از کارت {card} حذف کرده‌اید",
    "{user} has deleted the attachment {attachment} from card {card}" : "کاربر {user} پیوست {attachment} را از کارت {card} حذف کرده است",
    "You have restored the attachment {attachment} to card {card}" : "شما پیوست {attachment} را به کارت {card} بازگردانده‌اید",
    "{user} has restored the attachment {attachment} to card {card}" : "کاربر {user} پیوست {attachment} را به کارت {card} بازگردانده است",
    "You have commented on card {card}" : "شما روی کارت {card} نظر داده‌اید",
    "{user} has commented on card {card}" : "کاربر {user} روی کارت {card} نظر داده است",
    "Deck" : "کارتابل",
    "Changes in the <strong>Deck app</strong>" : "تغییرات در <strong>برنامه کارتابل</strong>",
    "A <strong>board, list or card</strong> was changed" : "یک <strong>بورد، لیست یا کارت</strong> تغییر کرد",
    "A <strong>comment</strong> was created on a card" : "یک <strong>نظر</strong> روی یک کارت ایجاد شد",
    "A <strong>card description</strong> has been changed" : "یک <strong>توضیحات کارت</strong> تغییر کرده است",
    "The file was uploaded" : "فایل آپلود شد",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "فایل آپلود شده از دستور upload_max_filesize در php.ini فراتر می‌رود",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "فایل آپلود شده از دستور MAX_FILE_SIZE که در فرم HTML مشخص شده بود فراتر می‌رود",
    "The file was only partially uploaded" : "فایل فقط به صورت جزئی آپلود شد",
    "No file was uploaded" : "هیچ فایلی آپلود نشد",
    "Missing a temporary folder" : "پوشه موقت وجود ندارد",
    "Could not write file to disk" : "نمی‌توان فایل را روی دیسک نوشت",
    "A PHP extension stopped the file upload" : "یک افزونه PHP آپلود فایل را متوقف کرد",
    "No file uploaded or file size exceeds maximum of %s" : "هیچ فایلی آپلود نشد یا اندازه فایل از حداکثر %s فراتر می‌رود",
    "Invalid file type. Only JSON files are allowed." : "نوع فایل نامعتبر است. فقط فایل‌های JSON مجاز هستند.",
    "Invalid JSON data" : "داده JSON نامعتبر است",
    "Failed to import board" : "وارد کردن بورد ناموفق بود",
    "Cards due today" : "کارت‌های موعد امروز",
    "Cards due tomorrow" : "کارت‌های موعد فردا",
    "Upcoming cards" : "کارت‌های آینده",
    "Load more" : "بارگذاری بیشتر",
    "Welcome to Nextcloud Deck!" : "به کارتابل نکست‌کلود خوش آمدید!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "کارت \"%s\" در \"%s\" توسط %s به شما اختصاص داده شده است.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "کاربر {user} کارت {کارتابل-card} را در {کارتابل-board} به شما اختصاص داده است.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "کارت \"%1$s\" در \"%2$s\" به تاریخ سررسید خود رسیده است.",
    "The card {deck-card} on {deck-board} has reached its due date." : "کارت {کارتابل-card} در {کارتابل-board} به تاریخ سررسید خود رسیده است.",
    "%s has mentioned you in a comment on \"%s\"." : "%s در یک نظر روی \"%s\" از شما نام برده است.",
    "{user} has mentioned you in a comment on {deck-card}." : "کاربر {user} در یک نظر روی {کارتابل-card} از شما نام برده است.",
    "The board \"%s\" has been shared with you by %s." : "بورد \"%s\" توسط %s با شما به اشتراک گذاشته شده است.",
    "{user} has shared {deck-board} with you." : "کاربر {user} بورد {کارتابل-board} را با شما به اشتراک گذاشته است.",
    "The remote board %1$s has been shared with you by %2$s" : "بورد راه دور %1$s توسط %2$s با شما به اشتراک گذاشته شده است",
    "Deck board" : "بورد کارتابل",
    "Owned by %1$s" : "متعلق به %1$s",
    "Deck boards, cards and comments" : "بوردها، کارت‌ها و نظرات کارتابل",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "از %1$s، در %2$s/%3$s، متعلق به %4$s",
    "Create a new deck card" : "ایجاد کارت جدید کارتابل",
    "Card comments" : "نظرات کارت",
    "%s on %s" : "%s در %s",
    "Deck boards and cards" : "بردها و کارت‌های کارتابل",
    "No data was provided to create an attachment." : "داده‌ای برای ایجاد پیوست ارائه نشد.",
    "Finished" : "تمام شده",
    "To review" : "برای بررسی",
    "Action needed" : "نیاز به اقدام",
    "Later" : "بعداً",
    "copy" : "کپی",
    "Read more inside" : "داخل را بیشتر بخوانید",
    "Custom lists - click to rename!" : "لیست‌های سفارشی - برای تغییر نام کلیک کنید!",
    "To Do" : "انجام دادنی",
    "In Progress" : "در حال انجام",
    "Done" : "انجام شده",
    "1. Open to learn more about boards and cards" : "۱. برای آشنایی بیشتر با بردها و کارت‌ها باز کنید",
    "2. Drag cards left and right, up and down" : "۲. کارت‌ها را به چپ و راست، بالا و پایین بکشید",
    "3. Apply rich formatting and link content" : "۳. قالب‌بندی پیشرفته و پیوند محتوا اعمال کنید",
    "4. Share, comment and collaborate!" : "۴. اشتراک‌گذاری، نظر دهید و همکاری کنید!",
    "Create your first card!" : "اولین کارت خود را ایجاد کنید!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "این نظر بیش از %s کاراکتر دارد.",
    "Attachments" : "پیوست‌ها",
    "File" : "فایل",
    "date" : "تاریخ",
    "Card not found" : "کارت یافت نشد",
    "Path is already shared with this card" : "مسیر قبلاً با این کارت به اشتراک گذاشته شده است",
    "Invalid date, date format must be YYYY-MM-DD" : "تاریخ نامعتبر، فرمت تاریخ باید YYYY-MM-DD باشد",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "تمام بردهای متعلق به شما شامل استک‌ها، کارت‌ها، برچسب‌ها، انتساب‌ها و نظرات",
    "Personal planning and team project organization" : "برنامه‌ریزی شخصی و سازماندهی پروژه تیمی",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "برگه‌دان یک ابزار سازمان‌دهی سبک کانبان است که هدف آن برنامه‌ریزی شخصی و سازمان‌دهی پروژه برای گروه‌های مرتبط با Nextcloud است.\n\n\n- 📥 وظایف خود را به برگه‌ها اضافه کنید و آن‌ها را مرتب کنید\n- 📄 یادداشت‌های اضافی را در Markdown یادداشت کنید\n- 🔖 برچسب‌هایی را برای سازمان‌دهی بهتر اختصاص دهید\n- 👥 با گروه، دوستان یا خانوادهٔ خود به اشتراک بگذارید\n- 📎 فایل‌ها را ضمیمه کنید و آنها را در توضیحات Markdown خود قرار دهید\n- 💬 با استفاده از نظرات با تیم خود بحث کنید\n- ⚡ تغییرات در جریان فعالیت را پیگیری کنید\n- 🚀 پروژه خود را سازمان‌دهی کنید",
    "Add board" : "افزودن برد",
    "Card details" : "جزئیات کارت",
    "Select the board to link to a project" : "برد مورد نظر برای پیوند به پروژه را انتخاب کنید",
    "Search by board title" : "جستجو بر اساس عنوان برد",
    "Select board" : "انتخاب برد",
    "Move/copy card" : "جابجایی/کپی کارت",
    "Select a board" : "یک برد انتخاب کنید",
    "No lists available" : "هیچ لیستی موجود نیست",
    "Select a list" : "یک لیست انتخاب کنید",
    "Move card" : "جابجایی کارت",
    "Copy card" : "کپی کارت",
    "Select the card to link to a project" : "کارت مورد نظر برای پیوند به پروژه را انتخاب کنید",
    "Link to card" : "پیوند به کارت",
    "Select a card" : "یک کارت انتخاب کنید",
    "Cancel" : "لغو",
    "This board is read only" : "این برد فقط خواندنی است",
    "Drop your files to upload" : "فایل‌های خود را برای بارگذاری رها کنید",
    "File already exists" : "فایل از قبل وجود دارد",
    "A file with the name {filename} already exists." : "فایلی با نام {filename} از قبل وجود دارد.",
    "Do you want to overwrite it?" : "آیا می‌خواهید آن را بازنویسی کنید؟",
    "Overwrite file" : "بازنویسی فایل",
    "Keep existing file" : "نگه‌داشتن فایل موجود",
    "Add card" : "افزودن کارت",
    "Archived cards" : "کارت‌های بایگانی‌شده",
    "Add list" : "افزودن لیست",
    "List name" : "نام لیست",
    "Clear search" : "پاک کردن جستجو",
    "Active filters" : "فیلترهای فعال",
    "Apply filter" : "اعمال فیلتر",
    "Filter by tag" : "فیلتر بر اساس برچسب",
    "Filter by assigned user" : "فیلتر بر اساس کاربر تعیین‌شده",
    "Unassigned" : "تعیین‌نشده",
    "Filter by status" : "فیلتر بر اساس وضعیت",
    "Open and completed" : "باز و تکمیل‌شده",
    "Open" : "باز",
    "Completed" : "تکمیل‌شده",
    "Filter by due date" : "فیلتر بر اساس تاریخ سررسید",
    "Overdue" : "عقب‌افتاده",
    "Next 24 hours" : "۲۴ ساعت آینده",
    "Next 7 days" : "۷ روز آینده",
    "Next 30 days" : "۳۰ روز آینده",
    "No due date" : "بدون تاریخ سررسید",
    "Clear filter" : "پاک کردن فیلتر",
    "View Modes" : "حالت‌های نمایش",
    "Toggle View Modes" : "تغییر حالت نمایش",
    "Kanban view" : "نمای Kanban",
    "Gantt view" : "نمای Gantt",
    "Hide archived cards" : "مخفی‌سازی کارت‌های بایگانی‌شده",
    "Show archived cards" : "نمایش کارت‌های بایگانی‌شده",
    "Toggle compact mode" : "تغییر حالت فشرده",
    "Hide card cover images" : "مخفی‌سازی تصاویر جلد کارت",
    "Show card cover images" : "نمایش تصاویر جلد کارت",
    "Open details" : "باز کردن جزئیات",
    "Details" : "جزئیات",
    "Deck settings" : "تنظیمات کارتابل",
    "General" : "عمومی",
    "Use bigger card view" : "استفاده از نمای بزرگ‌تر کارت",
    "Appearance" : "ظاهر",
    "Show card ID badge" : "نمایش نشان شناسه کارت",
    "Show boards in calendar/tasks" : "نمایش بوردها در تقویم/وظایف",
    "Admin settings" : "تنظیمات مدیر",
    "Limit board creation to some groups" : "محدود کردن ایجاد بورد به برخی گروه‌ها",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "کاربران خارج از آن گروه‌ها نمی‌توانند بوردهای خود را ایجاد کنند، اما همچنان می‌توانند روی بوردهایی که با آن‌ها به اشتراک گذاشته شده کار کنند.",
    "Enable federation" : "فعال‌سازی فدراسیون",
    "Board actions" : "اقدامات بورد",
    "Scroll sideways" : "پیمایش به پهلو",
    "Navigate between cards" : "حرکت بین کارت‌ها",
    "Close card details" : "بستن جزئیات کارت",
    "Search" : "جستجو",
    "Show card filters" : "نمایش فیلترهای کارت",
    "Clear card filters" : "پاک کردن فیلترهای کارت",
    "Show those shortcuts" : "نمایش میانبرهای موجود",
    "Card actions" : "عملیات کارت",
    "Open card details" : "باز کردن جزئیات کارت",
    "Edit the card title" : "ویرایش عنوان کارت",
    "Assign yourself to the current card" : "اختصاص خود به کارت فعلی",
    "Archive/unarchive the current card" : "بایگانی/خروج از بایگانی کارت فعلی",
    "Mark card as completed/not completed" : "علامت‌گذاری کارت به عنوان تکمیل‌شده/تکمیل‌نشده",
    "Open card menu" : "باز کردن منوی کارت",
    "Currently present people" : "افراد حاضر در حال حاضر",
    "Loading board" : "در حال بارگذاری برد",
    "Board not found" : "برد پیدا نشد",
    "Create a new list to add cards to this board" : "یک لیست جدید برای افزودن کارت به این برد ایجاد کنید",
    "Sharing" : "اشتراک‌گذاری",
    "Tags" : "برچسب‌ها",
    "Deleted items" : "موارد حذف‌شده",
    "Activity" : "فعالیت‌ها",
    "Deleted lists" : "لیست‌های حذف‌شده",
    "Undo" : "بازگردانی",
    "Deleted cards" : "کارت‌های حذف‌شده",
    "Hours" : "ساعت‌ها",
    "Days" : "روزها",
    "Weeks" : "هفته‌ها",
    "Months" : "ماه‌ها",
    "No cards yet" : "هنوز کارتی وجود ندارد",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "برای مشاهده کارت‌ها در نمودار گانت، تاریخ شروع و سررسید را روی کارت‌های خود تنظیم کنید",
    "Cards without dates ({count})" : "کارت‌های بدون تاریخ ({count})",
    "Failed to create share with {displayName}" : "ایجاد اشتراک با {displayName} انجام نشد",
    "Are you sure you want to transfer the board {title} to {user}?" : "آیا مطمئن هستید که می‌خواهید برد {title} را به {user} منتقل کنید؟",
    "Transfer the board." : "انتقال برد.",
    "Transfer" : "انتقال",
    "The board has been transferred to {user}" : "برد به {user} منتقل شد",
    "Failed to transfer the board to {user}" : "انتقال برد به {user} انجام نشد",
    "Board owner" : "مالک برد",
    "(Group)" : "(گروه)",
    "(Team)" : "(تیم)",
    "(remote)" : "(از راه دور)",
    "Can edit" : "قابلیت ویرایش",
    "Can share" : "قابلیت اشتراک‌گذاری",
    "Can manage" : "قابلیت مدیریت",
    "Owner" : "مالک",
    "Delete" : "حذف",
    "List deleted" : "لیست حذف شد",
    "Edit list title" : "ویرایش عنوان لیست",
    "Archive all cards" : "بایگانی همه کارت‌ها",
    "Unarchive all cards" : "خروج همه کارت‌ها از بایگانی",
    "Do not set cards as \"done\"" : "کارت‌ها را به عنوان «انجام‌شده» تنظیم نکن",
    "Set cards as \"done\"" : "کارت‌ها را به عنوان «انجام‌شده» تنظیم کن",
    "Delete list" : "حذف لیست",
    "Archive all cards in this list" : "بایگانی همه کارت‌های این لیست",
    "Unarchive all cards in this list" : "خارج کردن همه کارت‌ها از بایگانی در این لیست",
    "Add a new card" : "افزودن کارت جدید",
    "Card name" : "نام کارت",
    "title and color value must be provided" : "عنوان و مقدار رنگ باید ارائه شود",
    "Edit" : "ویرایش",
    "Add a new tag" : "افزودن برچسب جدید",
    "Board name" : "نام بورد",
    "Members" : "اعضا",
    "Assign to users/groups/team" : "اختصاص به کاربران/گروه‌ها/تیم",
    "Assign a user to this card…" : "اختصاص یک کاربر به این برگه…",
    "Select a user to assign to this card…" : "یک کاربر برای تخصیص به این کارت انتخاب کنید…",
    "File to share" : "فایل برای اشتراک‌گذاری",
    "Invalid path selected" : "مسیر نامعتبر انتخاب شد",
    "Upload new files" : "بارگذاری فایل‌های جدید",
    "Share from Files" : "اشتراک‌گذاری از فایل‌ها",
    "Pending share" : "اشتراک در انتظار",
    "Add this attachment" : "افزودن این پیوست",
    "Show in Files" : "نمایش در فایل‌ها",
    "Download" : "دانلود",
    "Remove attachment" : "حذف پیوست",
    "Delete Attachment" : "حذف پیوست",
    "Restore Attachment" : "بازیابی پیوست",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "تغییر‌یافته: {modifiedDate} ⸱ ایجاد‌شده: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "تغییر یافته: {modifiedDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "تغییر یافته: {modifiedDate}",
    "The title cannot be empty." : "عنوان نمی‌تواند خالی باشد.",
    "Cannot close unsaved card!" : "نمی‌توان کارت ذخیره‌نشده را بست!",
    "Open in sidebar view" : "باز کردن در نمای کناری",
    "Open in bigger view" : "باز کردن در نمای بزرگ‌تر",
    "Comments" : "نظرات",
    "Failed to load comments" : "بارگذاری نظرات ناموفق بود",
    "No comments yet. Begin the discussion!" : "هنوز نظری وجود ندارد. بحث را شروع کنید!",
    "The comment cannot be empty." : "نظر نمی‌تواند خالی باشد.",
    "The comment cannot be longer than 1000 characters." : "نظر نمی‌تواند بیش از ۱۰۰۰ کاراکتر باشد.",
    "Submit" : "ارسال",
    "Created:" : "ایجاد شده:",
    "In reply to" : "در پاسخ به",
    "Cancel reply" : "لغو پاسخ",
    "Reply" : "پاسخ",
    "Update" : "به‌روزرسانی",
    "Assign dependent cards" : "اختصاص کارت‌های وابسته",
    "Mark as done" : "علامت‌گذاری به عنوان انجام شده",
    "Mark as not done" : "علامت‌گذاری به عنوان انجام نشده",
    "Remove dependency" : "حذف وابستگی",
    "Add dependent card" : "افزودن کارت وابسته",
    "Assign a dependent card…" : "تخصیص یک کارت وابسته…",
    "Select a dependent card…" : "یک کارت وابسته انتخاب کنید…",
    "Write a description …" : "توضیح بنویس…",
    "The description has been changed by another user." : "توضیحات توسط کاربر دیگری تغییر یافته است.",
    "Could not save description" : "ذخیره توضیحات ممکن نشد",
    "Description" : "توضیحات",
    "(Unsaved)" : "(ذخیره نشده)",
    "(Saving…)" : "(در حال ذخیره...)",
    "Formatting help" : "راهنمای قالب‌بندی",
    "Edit description" : "ویرایش توضیحات",
    "View description" : "مشاهده توضیحات",
    "Add Attachment" : "افزودن پیوست",
    "Choose attachment" : "انتخاب پیوست",
    "Select Date" : "انتخاب تاریخ",
    "Later today – {timeLocale}" : "امروز دیرتر – {timeLocale}",
    "Set due date for later today" : "تنظیم مهلت برای امروز بعداً",
    "Tomorrow – {timeLocale}" : "فردا – {timeLocale}",
    "Set due date for tomorrow" : "تنظیم مهلت برای فردا",
    "This weekend – {timeLocale}" : "این آخر هفته – {timeLocale}",
    "Set due date for this weekend" : "تنظیم مهلت برای این آخر هفته",
    "Next week – {timeLocale}" : "هفتهٔ آینده – {timeLocale}",
    "Set due date for next week" : "تنظیم مهلت برای هفته آینده",
    "Assign a due date to this card…" : "تعیین مهلت برای این کارت…",
    "Set a due date" : "تنظیم مهلت",
    "Add due date" : "افزودن مهلت",
    "Choose a date" : "انتخاب تاریخ",
    "Remove due date" : "حذف مهلت",
    "Due at:" : "مهلت در:",
    "Not done" : "انجام نشده",
    "Unarchive card" : "خارج کردن کارت از بایگانی",
    "Archive card" : "بایگانی کارت",
    "Assign a start date to this card…" : "تعیین تاریخ شروع برای این کارت…",
    "Set a start date" : "تنظیم تاریخ شروع",
    "Add start date" : "افزودن تاریخ شروع",
    "Remove start date" : "حذف تاریخ شروع",
    "Assign a tag to this card…" : "یک برچسب به این برگه اختصاص دهید…",
    "Select or create a tag…" : "یک برچسب انتخاب یا ایجاد کنید…",
    "Create a new tag:" : "ایجاد برچسب جدید:",
    "(group)" : "(گروه)",
    "{count} comments, {unread} unread" : "{count} نظر، {unread} خوانده نشده",
    "Todo items" : "موارد انجام دادنی",
    "Edit card title" : "ویرایش عنوان کارت",
    "Open link" : "باز کردن پیوند",
    "Card deleted" : "کارت حذف شد",
    "Change card color" : "تغییر رنگ کارت",
    "Edit title" : "ویرایش عنوان",
    "Assign to me" : "اختصاص به من",
    "Unassign myself" : "لغو اختصاص خودم",
    "Delete card" : "حذف کارت",
    "All boards" : "همه بوردها",
    "Archived boards" : "بوردهای بایگانی شده",
    "Shared with you" : "اشتراک‌گذاری شده با شما",
    "Cancel edit" : "لغو ویرایش",
    "Save board" : "ذخیره بورد",
    "Board {0} deleted" : "تخته {0} حذف شد",
    "All cards" : "همه کارت‌ها",
    "Only assigned cards" : "فقط کارت‌های تعیین‌شده",
    "No reminder" : "بدون یادآوری",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "آیا مطمئن هستید که می‌خواهید تخته {title} را حذف کنید؟ این کار تمام داده‌های این تخته از جمله کارت‌های بایگانی‌شده را پاک می‌کند.",
    "Delete the board?" : "تخته حذف شود؟",
    "Are you sure you want to leave the board {title}?" : "آیا مطمئن هستید که می‌خواهید تخته {title} را ترک کنید؟",
    "Leave the board?" : "تخته ترک شود؟",
    "Leave" : "ترک",
    "Failed to leave the board" : "ترک تخته انجام نشد",
    "Exporting board..." : "در حال خروجی گرفتن از تخته...",
    "An error occurred" : "خطایی رخ داد",
    "Board details" : "جزئیات تخته",
    "Edit board" : "ویرایش تخته",
    "Clone board" : "کپی کردن تخته",
    "Unarchive board" : "خارج کردن تخته از بایگانی",
    "Archive board" : "بایگانی تخته",
    "Export board" : "خروجی گرفتن از تخته",
    "Turn on due date reminders" : "فعال‌سازی یادآوری مهلت‌ها",
    "Turn off due date reminders" : "غیرفعال‌سازی یادآوری مهلت‌ها",
    "Remove as default board" : "حذف به‌عنوان تخته پیش‌فرض",
    "Set as default board" : "تنظیم به‌عنوان تخته پیش‌فرض",
    "Due date reminders" : "یادآوری مهلت‌ها",
    "Assigned cards" : "کارت‌های تعیین‌شده",
    "No notifications" : "بدون اعلان",
    "Delete board" : "حذف تخته",
    "Leave board" : "ترک تخته",
    "Importing board..." : "در حال وارد کردن تخته...",
    "Board imported successfully" : "تخته با موفقیت وارد شد",
    "Import board" : "وارد کردن تخته",
    "Clone {boardTitle}" : "کپی {boardTitle}",
    "Clone cards" : "کپی کارت‌ها",
    "Clone assignments" : "کپی انتساب‌ها",
    "Clone labels" : "کپی برچسب‌ها",
    "Clone due dates" : "کپی مهلت‌ها",
    "Advanced options" : "گزینه‌های پیشرفته",
    "Move all cards to the first list" : "انتقال همه کارت‌ها به اولین لیست",
    "Restore archived cards" : "بازیابی کارت‌های بایگانی‌شده",
    "Clone" : "کپی",
    "Export {boardTitle}" : "خروجی {boardTitle}",
    "Export as JSON" : "خروجی به‌صورت JSON",
    "Export as CSV" : "خروجی به‌صورت CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "توجه: فقط فرمت JSON برای وارد کردن مجدد به برنامه کارتابلپشتیبانی می‌شود.",
    "Export" : "خروجی",
    "Loading filtered view" : "در حال بارگذاری نمای فیلترشده",
    "Search for {searchQuery} in other boards" : "جستجوی {searchQuery} در سایر تخته‌ها",
    "Search for {searchQuery} in all boards" : "جستجوی {searchQuery} در همه تخته‌ها",
    "No results found" : "نتیجه‌ای یافت نشد",
    "Deck board {name}\n* Last modified on {lastMod}" : "تخته نکست‌کلود {name}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* ایجاد شده در {created}",
    "{nbCards} cards" : "{nbCards} کارت",
    "Due on {date}" : "موعد سررسید: {date}",
    "{stack} in {board}" : "{stack} در {board}",
    "Click to expand description" : "برای مشاهده توضیحات کلیک کنید",
    "Click to expand comment" : "برای مشاهده نظر کلیک کنید",
    "Create card" : "ایجاد کارت",
    "Create a new card" : "ایجاد کارت جدید",
    "Card title" : "عنوان کارت",
    "Creating the new card …" : "ساختن برگهٔ جدید ...",
    "Card \"{card}\" was added to \"{board}\"" : "کارت «{card}» به «{board}» اضافه شد",
    "Open card" : "باز کردن کارت",
    "Close" : "بستن",
    "No upcoming cards" : "کارت آتی وجود ندارد",
    "upcoming cards today" : "کارت‌های امروز",
    "upcoming cards tomorrow" : "کارت‌های فردا",
    "upcoming cards" : "کارت‌های آتی",
    "New card" : "کارت جدید",
    "Link to a board" : "پیوند به یک بورد",
    "Link to a card" : "پیوند به یک کارت",
    "Create a card" : "ایجاد یک کارت",
    "Message from {author} in {conversationName}" : "پیام از {author} در {conversationName}",
    "Something went wrong" : "خطایی رخ داد",
    "Failed to upload {name}" : "بارگذاری {name} ناموفق بود",
    "Maximum file size of {size} exceeded" : "حداکثر حجم فایل {size} بیش از حد مجاز است",
    "Assigned users" : "کاربران تعیین‌شده",
    "Due date" : "تاریخ سررسید",
    "Created" : "ایجاد شده",
    "Modified" : "ویرایش شده",
    "Error creating the share" : "خطا در ایجاد اشتراک‌گذاری",
    "Share with a Deck card" : "اشتراک‌گذاری با یک کارت کارتابل",
    "Share {file} with a Deck card" : "اشتراک‌گذاری {file} با یک کارت کارتابل",
    "Share" : "اشتراک‌گذاری",
    "The card \"%s\" on \"%s\" has reached its due date." : "کارت «%s» در «%s» به تاریخ سررسید خود رسیده است.",
    "Share board with a user, group or team …" : "هم‌رسانی تخته با یک کاربر، گروه یا تیم …",
    "Searching for users, groups and teams …" : "در حال جستجوی کاربران، گروه‌ها و تیم‌ها …",
    "No participants found" : "هیچ شرکت‌کننده‌ای یافت نشد",
    "Save" : "ذخیره",
    "seconds ago" : "ثانیه پیش",
    "Today" : "امروز",
    "Tomorrow" : "فردا",
    "No due" : "بدون سررسید"
},
"nplurals=2; plural=(n > 1);");
