OC.L10N.register(
    "deck",
    {
    "The file was uploaded" : "Նիշքը վերբերռնված է",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Վերբեռնած նիշքը գերազանցում է upload_max_filesize սահմանված php.ini֊ում",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Վերբեռնած նիշքը գերազանցում է MAX_FILE_SIZE, որը սահմանված է HTML ձևաթղթում",
    "The file was only partially uploaded" : "Նիշքի մի մասն է վերբեռնված",
    "No file was uploaded" : "Ոչ մի նիշք չի վերնբեռնվել",
    "Missing a temporary folder" : "Բացակայում է ժամանակավոր պանակը",
    "Could not write file to disk" : "Չհաջողվեց գրառել նիշքը սկավառակի վրա",
    "A PHP extension stopped the file upload" : "PHP֊ի ընդլայնումն կանգնեցրեց նիշքի վերբեռնումը",
    "Done" : "Done",
    "Cancel" : "ընդհատել",
    "Completed" : "Ավարտվեց",
    "Details" : "Մանրամասներ",
    "General" : "Ընդհանուր",
    "Search" : "Search",
    "Activity" : "Գործունեություն",
    "Hours" : "Ժամ",
    "Can edit" : "Can edit",
    "Can share" : "Can share",
    "Owner" : "Owner",
    "Delete" : "հեռացնել",
    "Edit" : "մշակել",
    "Download" : "Ներբեռնել",
    "Comments" : "Կարծիքներ",
    "Created:" : "Ստեղծված.",
    "Update" : "Թարմացնել",
    "Description" : "Նկարագրություն",
    "Shared with you" : "Shared with you",
    "Export" : "Արտահանում",
    "Close" : "Փակել",
    "Modified" : "Փոփոխված",
    "Share" : "Կիսվել",
    "Save" : "Պահպանել",
    "seconds ago" : "վրկ. առաջ",
    "Today" : "այսօր"
},
"nplurals=2; plural=(n != 1);");
