OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Vytvorili ste novú nástenku {board}",
    "{user} has created a new board {board}" : "Používateľ {user} vytvoril novú nástenku {board}",
    "You have deleted the board {board}" : "Vymazali ste nástenku {board}",
    "{user} has deleted the board {board}" : "Používateľ {user} vymazal nástenku {board}",
    "You have restored the board {board}" : "Obnovili ste nástenku {board}",
    "{user} has restored the board {board}" : "Používateľ {user} obnovil nástenku {board}",
    "You have shared the board {board} with {acl}" : "Sprístupnili ste nástenku {board} s {acl}",
    "{user} has shared the board {board} with {acl}" : "Používateľ {user} sprístupnil nástenku {board} pre {acl}",
    "You have removed {acl} from the board {board}" : "Odobrali ste {acl} z nástenky {board}",
    "{user} has removed {acl} from the board {board}" : "Používateľ {user} odobral {acl} z nástenky {board}",
    "You have renamed the board {before} to {board}" : "Premenovali ste nástenku {before} na {board}",
    "{user} has renamed the board {before} to {board}" : "Používateľ {user} premenoval nástenku {before} na {board}",
    "You have archived the board {board}" : "Archivovali ste nástenku {board}",
    "{user} has archived the board {before}" : "Používateľ {user} archivoval nástenku {board}",
    "You have unarchived the board {board}" : "Zrušili ste archiváciu nástenky {board}",
    "{user} has unarchived the board {before}" : "Používateľ {user} zrušil archiváciu nástenky {before}",
    "You have created a new list {stack} on board {board}" : "Vytvorili ste nový zoznam {stack} na nástenke {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} vytvoril nový zoznam {stack} na nástenke {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Premenovali ste zoznam {before} na {stack} na nástenke {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} premenoval zoznam {before} na {stack} na nástenke {board}",
    "You have deleted list {stack} on board {board}" : "Odstránili ste {stack} z nástenky {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} odstránil zoznam {stack} z nástenky {board}",
    "You have created card {card} in list {stack} on board {board}" : "Vytvorili ste kartu {card} v zozname {stack} na nástenke {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} vytvoril kartu {card} v zozname {stack} na nástenke {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Odstránili ste kartu {card} zo zoznamu {stack} na nástenke {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} odstránil kartu {card} zo zoznamu {stack} na nástenke {board}",
    "You have renamed the card {before} to {card}" : "Premenovali ste kartu {before} na {card}",
    "{user} has renamed the card {before} to {card}" : "Používateľ {user} premenoval kartu {before} na {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Pridali ste popis na kartu {card} v zozname {stack} na nástenke {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} pridal popis na kartu {card} v zozname {stack} na nástenke {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Aktualizovali ste popis na karte {card} v zozname {stack} na nástenke {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} aktualizoval popis na karte {card} v zozname {stack} na nástenke {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Archivovali ste kartu {card} v zozname {stack} na nástenke {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} archivoval kartu {card} v zozname {stack} na nástenke {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Zrušili ste archiváciu karty {card} v zozname {stack} na nástenke {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} zrušil archiváciu karty {card} v zozname {stack} na nástenke {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Označili ste kartu {card} ako vykonanú v zozname {stack} na  {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} označil kartu {card} ako hotovú v zozname {stack} na nástenke {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "V zozname {stack} na {board} ste označili kartu {card} ako zrušenú",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} označil kartu {card} ako nedokončenú v zozname {stack} na {board}",
    "You have removed the due date of card {card}" : "Odobrali ste termín dokončenia z karty {card}",
    "{user} has removed the due date of card {card}" : "Používateľ {user} odobral termín dokončenia z karty {card}",
    "You have set the due date of card {card} to {after}" : "Nastavili ste termín dokončenia pre kartu {card} na {after}",
    "{user} has set the due date of card {card} to {after}" : "Používateľ {user} nastavil termín dokončenia karty {card} na {after}",
    "You have updated the due date of card {card} to {after}" : "Zmenili ste termín dokončenia karty {card} na {after}",
    "{user} has updated the due date of card {card} to {after}" : "Používateľ {user} aktualizoval termín dokončenia karty {card} na {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Pridali ste štítok {label} ku karte {card} v zozname {stack} na nástenke {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} pridal štítok {label} ku karte {card} v zozname {stack} na nástenke {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Odstránili ste štítok {label} karty {card} v zozname {stack} na nástenke {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} odstránil štítok {label} karty {card} v zozname {stack} na nástenke {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Priradili ste používateľa {assigneduser} ku karte {card} na nástenke {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "Používateľ {user} priradil používateľa {assigneduser} ku karte {card} na nástenke {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Vyradili ste priradenie používateľa {assigneduser} z karty {card} na nástenke {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "Používateľ {user} vyradil používateľa {assigneduser} z karty {card} na nástenke {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Presunuli ste kartu {card} zo zoznamu {stackBefore} na {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} presunul kartu {card} zo zoznamu {stackBefore} na {stack}",
    "You have added the attachment {attachment} to card {card}" : "Pridali ste prílohu {attachment} ku karte {card}",
    "{user} has added the attachment {attachment} to card {card}" : "Používateľ {user} pridal prílohu {attachment} ku karte {card}",
    "You have updated the attachment {attachment} on card {card}" : "Aktualizovali ste prílohu {attachment} na karte {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "Používateľ {user} aktualizoval prílohu {attachment} na karte {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Vymazali ste prílohu {attachment} na karte {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "Používateľ {user} vymazal prílohu {attachment} na karte {card}",
    "You have restored the attachment {attachment} to card {card}" : "Obnovili ste prílohu {attachment} na karte {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "Používateľ {user} obnovil prílohu {attachment} na karte {card}",
    "You have commented on card {card}" : "Komentovali ste kartu {card}",
    "{user} has commented on card {card}" : "{user} komentoval(a) kartu {card}",
    "Deck" : "Nástenka",
    "Changes in the <strong>Deck app</strong>" : "Zmeny v apke <strong>Nástenka</strong>",
    "A <strong>board, list or card</strong> was changed" : "<strong>zoznam, alebo karta</strong> boli zmenené",
    "A <strong>comment</strong> was created on a card" : "Na karte bol vytvorený <strong>komentár</strong>. ",
    "A <strong>card description</strong> has been changed" : "<strong>popis karty</strong> bol zmenený",
    "The file was uploaded" : "Súbor bol nahraný",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Nahraný súbor prekročil limit nastavený v upload_max_filesize v súbore php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Ukladaný súbor prekračuje nastavenie MAX_FILE_SIZE z volieb HTML formulára.",
    "The file was only partially uploaded" : "Súbor bol nahraný iba čiastočne",
    "No file was uploaded" : "Nenahral sa žiadny súbor",
    "Missing a temporary folder" : "Chýba priečinok pre dočasné súbory",
    "Could not write file to disk" : "Nepodarilo sa zapísať súbor na disk",
    "A PHP extension stopped the file upload" : "Rozšírenie PHP zastavilo nahrávanie súboru",
    "No file uploaded or file size exceeds maximum of %s" : "Nenahral sa žiadny súbor alebo jeho veľkosť presiahla %s",
    "Invalid file type. Only JSON files are allowed." : "Neplatný typ súboru. Povolené sú iba súbory JSON.",
    "Invalid JSON data" : "Neplatné JSON dáta",
    "Failed to import board" : "Importovanie boardu zlyhalo",
    "Cards due today" : "Karty platné dodnes",
    "Cards due tomorrow" : "Karty platné do zajtra",
    "Upcoming cards" : "Nadchádzajúce karty",
    "Load more" : "Načítať viac",
    "Welcome to Nextcloud Deck!" : "Vitajte v Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Používateľ %s vám priradil kartu „%s“ na „%s“.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} vám priradil kartu {deck-card} na {deck-board}.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Karta \"%1$s\" na \"%2$s\" dosiahla svoj dátum splatnosti.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Karta {deck-card} na {deck-board} dosiahla termín dokončenia.",
    "%s has mentioned you in a comment on \"%s\"." : "Používateľ %s vás zmienil v komentári ku „%s“.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} vás spomenul v komentári v {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Používateľ %s vám sprístupnil nástenku \"%s\".",
    "{user} has shared {deck-board} with you." : "{user} zdieľa {deck-board} s vami.",
    "The remote board %1$s has been shared with you by %2$s" : "Vzdialená nástenka %1$s bola zdieľaná s vami od %2$s",
    "Deck board" : "Palubná doska",
    "Owned by %1$s" : "Vlastní %1$s",
    "Deck boards, cards and comments" : "Nástenky Deck, karty a komentáre",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Od %1$s, v %2$s/%3$s, vlastnené %4$s",
    "Create a new deck card" : "Vytvoriť novú kartu deck",
    "Card comments" : "Komentáre karty",
    "%s on %s" : "%s na %s",
    "Deck boards and cards" : "Nástenky Deck a karty",
    "No data was provided to create an attachment." : "Neboli poskytnuté dáta pre vytvorenie prílohy.",
    "Finished" : "Ukončené",
    "To review" : "Na posúdenie",
    "Action needed" : "Potrebná akcia",
    "Later" : "Neskôr",
    "copy" : "kopírovať",
    "Read more inside" : "Prečítajte si viac vo vnútri",
    "Custom lists - click to rename!" : "Vlastné zoznamy - kliknite pre premenovanie!",
    "To Do" : "Úlohy",
    "In Progress" : "Prebieha",
    "Done" : "Hotové",
    "1. Open to learn more about boards and cards" : "1. Otvorte a dozviete sa viac o nástenkách a kartách",
    "2. Drag cards left and right, up and down" : "2. Potiahnite karty doľava a doprava, nahor a nadol",
    "3. Apply rich formatting and link content" : "3. Použite bohaté formátovanie a pripojte obsah",
    "4. Share, comment and collaborate!" : "4. Zdieľajte, komentujte a spoluprecujte!",
    "Create your first card!" : "Vytvorte vašu prvú kartu!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Tento komentár má viac než %s znakov.\nPridané ako príloha ku karte s názvom %s.\nDostupné na URL: %s.",
    "Attachments" : "Prílohy",
    "File" : "Súbor",
    "date" : "dátum",
    "Card not found" : "Karta nebola nájdená",
    "Path is already shared with this card" : "Cesta je už zdieľaná s touto kartou",
    "Invalid date, date format must be YYYY-MM-DD" : "Neplatný dátum, formát musí byť v tvare YYYY-MM-DD",
    "Personal planning and team project organization" : "Osobné plánovanie a organizácia tímových projektov",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck je organizačný nástroj typu kanban zameraný na osobné plánovanie a organizáciu projektov pre tímy integrované s Nextcloud.\n\n\n- 📥 Pridajte svoje úlohy na karty a usporiadajte ich\n- 📄 Zapíšte si ďalšie poznámky v Markdown\n- 🔖 Priraďte štítky pre ešte lepšiu organizáciu\n- 👥 Zdieľajte so svojím tímom, priateľmi alebo rodinou\n- 📎 Pripojte súbory a vložte ich do popisu Markdown\n- 💬 Diskutujte so svojím tímom pomocou komentárov\n- ⚡ Sledujte zmeny v prúde aktivít\n- 🚀 Zorganizujte si svoj projekt",
    "Add board" : "Pridať nástenku",
    "Card details" : "Podrobnosti o karte",
    "Select the board to link to a project" : "Vybrať nástenku na vytvorenie odkazu ku projektu",
    "Search by board title" : "Vyhľadávanie podľa názvu nástenky",
    "Select board" : "Vybrať nástenku",
    "Move/copy card" : "Presunúť/Kopírovať kartu",
    "Select a board" : "Vyberte nástenku",
    "No lists available" : "Nie sú k dispozícii žiadne zoznamy",
    "Select a list" : "Vybrať zoznam",
    "Move card" : "Presunúť kartu",
    "Copy card" : "Kopírovať kartu",
    "Select the card to link to a project" : "Vyberte kartu, ktorú chcete prepojiť s projektom",
    "Link to card" : "Prepojiť s kartou",
    "Select a card" : "Vyberte kartu",
    "Cancel" : "Zrušiť",
    "This board is read only" : "Táto nástenka je iba na čítanie",
    "Drop your files to upload" : "Pretiahnite sem súbory, ktoré chcete nahrať",
    "File already exists" : "Súbor už existuje",
    "A file with the name {filename} already exists." : "Súbor s názvom {filename} už existuje.",
    "Do you want to overwrite it?" : "Chcete ho prepísať?",
    "Overwrite file" : "Prepísať súbor",
    "Keep existing file" : "Ponechať existujúci súbor",
    "Add card" : "Pridať kartu",
    "Archived cards" : "Archivované karty",
    "Add list" : "Pridať zoznam",
    "List name" : "Názov zoznamu",
    "Clear search" : "Vymazať hľadanie",
    "Active filters" : "Aktívne filtre",
    "Apply filter" : "Uplatniť filter",
    "Filter by tag" : "Filtrovať podľa štítkov",
    "Filter by assigned user" : "Filtrovať podľa priradeného používateľa",
    "Unassigned" : "Nepriradený",
    "Filter by status" : "Filtrovať podľa stavu",
    "Open and completed" : "Otvorené a dokončené",
    "Open" : "Otvoriť",
    "Completed" : "Dokončené",
    "Filter by due date" : "Filtrovať podľa termínu",
    "Overdue" : "Po termíne",
    "Next 24 hours" : "Ďalších 24 hodín",
    "Next 7 days" : "Následujúcich 7 dní",
    "Next 30 days" : "Následujúcich 30 dní",
    "No due date" : "Bez termínu",
    "Clear filter" : "Vyčistiť filter",
    "View Modes" : "Režimy zobrazenia",
    "Toggle View Modes" : "Prepnúť režimy zobrazenia",
    "Hide archived cards" : "Skryť archivované karty",
    "Show archived cards" : "Zobraziť archivované karty",
    "Toggle compact mode" : "Vyp/zap. kompaktný režim",
    "Hide card cover images" : "Skryť obrázky obalu karty",
    "Show card cover images" : "Zobraziť obrázky obalu karty",
    "Open details" : "Otvoriť detaily",
    "Details" : "Podrobnosti",
    "Deck settings" : "Nastavenia paluby",
    "General" : "Všeobecné",
    "Use bigger card view" : "Použiť väčšie zobrazenie karty",
    "Appearance" : "Vzhľad",
    "Show card ID badge" : "Ukázať ID karty",
    "Show boards in calendar/tasks" : "Zobrazovať nástenky v kalendári/úlohách",
    "Admin settings" : "Nastavenia administrátora",
    "Limit board creation to some groups" : "Obmedziť vytváranie nástenky pre konkrétne skupiny",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Užívatelia mimo týchto skupín si nebudú môcť vytvárať vlastné nástenky, ale stále budú môcť pracovať na nástenkách, ktoré s nimi boli zdieľané.",
    "Enable federation" : "Povoliť federáciu",
    "Board actions" : "Akcie nástenky",
    "Scroll sideways" : "Rolovať do strán",
    "Navigate between cards" : "Navigovať medzi kartami",
    "Close card details" : "Zatvoriť podrobnosti o karte",
    "Search" : "Hľadať",
    "Show card filters" : "Zobraziť filtre karty",
    "Clear card filters" : "Vymazať filtre kariet",
    "Show those shortcuts" : "Zobraziť klávesové skratky",
    "Card actions" : "Akcie karty",
    "Open card details" : "Otvoriť podrobnosti o karte",
    "Edit the card title" : "Upraviť názov karty",
    "Assign yourself to the current card" : "Priraďte seba k aktuálnej karte",
    "Archive/unarchive the current card" : "Archivovať/zrušiť archiváciu aktuálnej karty",
    "Mark card as completed/not completed" : "Označiť kartu ako dokončenú/nedokončenú",
    "Open card menu" : "Otvoriť menu karty",
    "Currently present people" : "Momentálne prítomný ľudia",
    "Loading board" : "Načítanie nástenky",
    "Board not found" : "Nástenka nebola nájdená",
    "Create a new list to add cards to this board" : "Pre pridanie kariet na túto tabuľu vytvorte nový zoznam",
    "Sharing" : "Zdieľanie",
    "Tags" : "Štítky",
    "Deleted items" : "Zmazané položky",
    "Activity" : "Aktivita",
    "Deleted lists" : "Vymazané zoznamy",
    "Undo" : "Odvolať",
    "Deleted cards" : "Odstránené karty",
    "Hours" : "Hodiny",
    "Days" : "Dni",
    "Weeks" : "Týždňov",
    "Months" : "Mesiacov",
    "No cards yet" : "Zatiaľ žiadne karty",
    "Failed to create share with {displayName}" : "Nepodarilo sa vytvoriť zdieľanie pre {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Naozaj chcete preniesť nástenku {title} na užívateľa {user}?",
    "Transfer the board." : "Prenos nástenky.",
    "Transfer" : "Prenos",
    "The board has been transferred to {user}" : "Nástenka bola presunutá na {user}",
    "Failed to transfer the board to {user}" : "Chyba pri presune nástenky na {user}",
    "Board owner" : "Vlastník nástenky",
    "(Group)" : "(Skupina)",
    "(Team)" : "(Tím)",
    "(remote)" : "(vzdialené)",
    "Can edit" : "Môže upravovať",
    "Can share" : "Môže sprístupniť",
    "Can manage" : "Môže spravovať",
    "Owner" : "Vlastník",
    "Delete" : "Zmazať",
    "List deleted" : "Zoznam bol vymazaný",
    "Edit list title" : "Upraviť názov zoznamu",
    "Archive all cards" : "Archivovať všetky karty",
    "Unarchive all cards" : "Zrušiť archiváciu všetkých kariet",
    "Do not set cards as \"done\"" : "Nenastaviť karty ako „dokončené“",
    "Set cards as \"done\"" : "Nastaviť karty ako „dokončené“",
    "Delete list" : "Vymazať zoznam",
    "Archive all cards in this list" : "Archivovať všetky karty v tomto zozname",
    "Unarchive all cards in this list" : "Zrušiť archiváciu všetkých kariet v tomto zozname",
    "Add a new card" : "Pridať novú kartu",
    "Card name" : "Názov karty",
    "title and color value must be provided" : "je potrebné zadať nadpis a vybrať farbu",
    "Edit" : "Upraviť",
    "Add a new tag" : "Pridať nový štítok",
    "Board name" : "Názov nástenky",
    "Members" : "Členovia",
    "Assign to users/groups/team" : "Priradiť k užívateľom/skupinám/tímom",
    "Assign a user to this card…" : "Priradiť používateľa k tejto karte…",
    "Select a user to assign to this card…" : "Vyberte užívateľa, ktorého chcete priradiť k tejto karte…",
    "File to share" : "Súbor na zdieľanie",
    "Invalid path selected" : "Bola vybraná neplatná cesta",
    "Upload new files" : "Nahrať nové súbory",
    "Share from Files" : "Zdieľať zo Súborov",
    "Pending share" : "Čakajúce zdieľanie",
    "Add this attachment" : "Pridať túto prílohu",
    "Show in Files" : "Zobraziť v Súboroch",
    "Download" : "Sťahovanie",
    "Remove attachment" : "Odstrániť prílohu",
    "Delete Attachment" : "Odstrániť prílohu",
    "Restore Attachment" : "Obnoviť prílohu",
    "The title cannot be empty." : "Nadpis nemôže byť prázdny.",
    "Cannot close unsaved card!" : "Nie je možné zatvoriť neuloženú kartu!",
    "Open in sidebar view" : "Otvoriť v bočnom paneli",
    "Open in bigger view" : "Otvoriť vo väčšom zobrazení",
    "Comments" : "Komentáre",
    "Failed to load comments" : "Nepodarilo sa načítať komentáre",
    "No comments yet. Begin the discussion!" : "Zatiaľ bez komentárov. Začnite diskusiu!",
    "The comment cannot be empty." : "Komentár nemôže byť prázdny.",
    "The comment cannot be longer than 1000 characters." : "Komentár nemôže byť dlhší ako 1000 znakov.",
    "Submit" : "Odoslať",
    "Created:" : "Vytvorené:",
    "In reply to" : "Odpoveď na",
    "Cancel reply" : "Zrušiť odpoveď",
    "Reply" : "Odpoveď",
    "Update" : "Aktualizovať",
    "Mark as done" : "Označiť ako dokončené",
    "Mark as not done" : "Označiť ako nedokončené",
    "Write a description …" : "Napíšte popis...",
    "The description has been changed by another user." : "Popis bol zmenený iným používateľom.",
    "Could not save description" : "Nepodarilo sa uložiť popis",
    "Description" : "Popis",
    "(Unsaved)" : "(Neuložené)",
    "(Saving…)" : "(Ukladanie…)",
    "Formatting help" : "Pomoc pri formátovaní",
    "Edit description" : "Upraviť popis",
    "View description" : "Zobraziť popis",
    "Add Attachment" : "Pridať prílohu",
    "Choose attachment" : "Vybrať prílohu",
    "Select Date" : "Vybrať dátum",
    "Later today – {timeLocale}" : "Neskôr dnes - {timeLocale}",
    "Set due date for later today" : "Nastaviť termín dokončenia na neskôr dnes ",
    "Tomorrow – {timeLocale}" : "Zajtra – {timeLocale}",
    "Set due date for tomorrow" : "Nastaviť termín dokončenia na zajtra",
    "This weekend – {timeLocale}" : "Tento víkend – {timeLocale}",
    "Set due date for this weekend" : "Nastaviť termín dokončenia na tento víkend",
    "Next week – {timeLocale}" : "Budúci týždeň – {timeLocale}",
    "Set due date for next week" : "Nastaviť termín dokončenia na budúci týždeň",
    "Assign a due date to this card…" : "Priradiť tejto karte termín…",
    "Set a due date" : "Nastaviť termín",
    "Add due date" : "Pridať termín",
    "Choose a date" : "Vyberte dátum",
    "Remove due date" : "Odstrániť termín dokončenia",
    "Due at:" : "Termín dokončenia:",
    "Not done" : "Neurobené",
    "Unarchive card" : "Zrušiť archiváciu karty",
    "Archive card" : "Archivovať kartu",
    "Assign a start date to this card…" : "Priraďiť tejto karte dátum začiatku…",
    "Set a start date" : "Nastaviť dátum začiatku",
    "Add start date" : "Pridať dátum začatia",
    "Remove start date" : "Odstrániť dátum začatia",
    "Assign a tag to this card…" : "Tejto karte priradiť štítok…",
    "Select or create a tag…" : "Vybrať alebo vytvoriť štítok...",
    "Create a new tag:" : "Vytvoriť nový štítok:",
    "(group)" : "(skupina)",
    "{count} comments, {unread} unread" : "{count} komentárov, {unread} neprečítaných",
    "Todo items" : "Položky úloh",
    "Edit card title" : "Upraviť názov karty",
    "Open link" : "Otvoriť odkaz",
    "Card deleted" : "Karta bola vymazaná",
    "Edit title" : "Premenovať",
    "Assign to me" : "Priradiť mne",
    "Unassign myself" : "Zrušiť priradenie mne",
    "Delete card" : "Zmazať kartu",
    "All boards" : "Všetky nástenky",
    "Archived boards" : "Archivované nástenky",
    "Shared with you" : "Vám sprístupnené",
    "Cancel edit" : "Zrušiť upravovanie",
    "Save board" : "Uložiť tabuľu",
    "Board {0} deleted" : "Nástenka {0} vymazaná",
    "All cards" : "Všetky karty",
    "Only assigned cards" : "Len priradené karty",
    "No reminder" : "Žiadna pripomienka",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Naozaj chcete chcete vymazať nástenku {title}? Toto zmaže všetky údaje o tejto nástenke vrátane archivovaných kariet.",
    "Delete the board?" : "Vymazať nástenku?",
    "Are you sure you want to leave the board {title}?" : "Ste si istí, že chcete opustiť nástenku {title}?",
    "Leave the board?" : "Opustiť nástenku?",
    "Leave" : "Odísť",
    "Failed to leave the board" : "Nepodarilo sa opustiť nástenku",
    "Exporting board..." : "Exportovanie nástenky...",
    "An error occurred" : "Vyskytla sa chyba",
    "Board details" : "Podrobnosti o nástenke",
    "Edit board" : "Upraviť nástenku",
    "Clone board" : "Duplikovať nástenku",
    "Unarchive board" : "Zrušiť archiváciu nástenky",
    "Archive board" : "Archivovať nástenku",
    "Export board" : "Exportovať nástenku",
    "Turn on due date reminders" : "Zapnúť pripomienky na termín dokončenia",
    "Turn off due date reminders" : "Vypnúť pripomienky na termín dokončenia",
    "Remove as default board" : "Odstrániť ako predvolenú nástenku",
    "Set as default board" : "Nastaviť ako predvolenú nástenku",
    "Due date reminders" : "Pripomienky termínu dokončenia",
    "Assigned cards" : "Priradené karty",
    "No notifications" : "Žiadne upozornenia",
    "Delete board" : "Vymazať nástenku",
    "Leave board" : "Opustiť nástenku",
    "Importing board..." : "Importovanie nástenky...",
    "Board imported successfully" : "Nástenka bola úspešne nainmportovaná",
    "Import board" : "Importovať nástenku",
    "Clone {boardTitle}" : "Klonovať {boardTitle}",
    "Clone cards" : "Duplikovať karty",
    "Clone assignments" : "Klonovať úlohu",
    "Clone labels" : "Klonovať štítok",
    "Clone due dates" : "Klonovať termín dokončenia",
    "Advanced options" : "Pokročilé možnosti",
    "Move all cards to the first list" : "Presunúť všetky karty na prvý zoznam",
    "Restore archived cards" : "Obnoviť archivované karty",
    "Clone" : "Duplikovať",
    "Export {boardTitle}" : "Exportovať {boardTitle}",
    "Export as JSON" : "Exportovať ako JSON",
    "Export as CSV" : "Exportovať do CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Poznámka: Na import späť do aplikácie Deck je podporovaný iba formát JSON.",
    "Export" : "Export",
    "Loading filtered view" : "Načítavanie filtrovaného pohľadu",
    "Search for {searchQuery} in other boards" : "Vyhľadať {searchQuery}na iných nástenkách",
    "Search for {searchQuery} in all boards" : "Vyhľadať {searchQuery} na všetkých plochách",
    "No results found" : "Neboli nájdené žiadne výsledky",
    "Deck board {name}\n* Last modified on {lastMod}" : "Nástenka Deck {name}\n* Naposledy upravené {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Vytvorené dňa {created}\n* Naposledy upravené {lastMod}\n* {nbAttachments} príloh\n* Komentáre: {nbComments}",
    "{nbCards} cards" : "{nbCards} kariet",
    "Due on {date}" : "Platné do {date}",
    "{stack} in {board}" : "{stack} v {board}",
    "Click to expand description" : "Kliknite pre rozšírenie popisu",
    "Click to expand comment" : "Kliknite pre zobrazenie komentára",
    "Create card" : "Vytvoriť kartu",
    "Create a new card" : "Vytvoriť novú kartu",
    "Card title" : "Meno karty",
    "Creating the new card …" : "Vytváram novú kartu ...",
    "Card \"{card}\" was added to \"{board}\"" : "Karta \"{card}\" bola pridaná na \"{board}\"",
    "Open card" : "Otvoriť kartu",
    "Close" : "Zatvoriť",
    "No upcoming cards" : "Žiadne nadchádzajúce karty",
    "upcoming cards today" : "Nadchádzajúce karty na dnes",
    "upcoming cards tomorrow" : "Nadchádzajúce karty na zajtra",
    "upcoming cards" : "nadchádzajúce karty",
    "New card" : "Nová karta",
    "Link to a board" : "Odkaz na nástenku",
    "Link to a card" : "Prepojiť s kartou",
    "Create a card" : "Vytvoriť kartu",
    "Message from {author} in {conversationName}" : "Správa od {author} v {conversationName}",
    "Something went wrong" : "Niečo sa pokazilo",
    "Failed to upload {name}" : "Zlyhalo nahrávanie {name}",
    "Maximum file size of {size} exceeded" : "Bola prekročená maximálna veľkosť súboru {size}",
    "Assigned users" : "Priradený užívatelia",
    "Due date" : "Termín dokončenia",
    "Created" : "Vytvorené",
    "Modified" : "Upravené",
    "Error creating the share" : "Chyba pri vytváraní zdieľania",
    "Share with a Deck card" : "Zdieľať s kartou Deck",
    "Share {file} with a Deck card" : "Zdieľať {file} s kartou Deck",
    "Share" : "Zdieľať",
    "The card \"%s\" on \"%s\" has reached its due date." : "Karta \"%s\" na nástenke \"%s\" dosiahla termín dokončenia.",
    "Share board with a user, group or team …" : "Zdieľanie nástenky s užívateľom, skupinou alebo tímom …",
    "Searching for users, groups and teams …" : "Vyhľadávam užívateľov, skupiny a tímy ...",
    "No participants found" : "Nenašli sa žiadni účastníci",
    "Save" : "Uložiť",
    "seconds ago" : "pred niekoľkými sekundami",
    "Today" : "Dnes",
    "Tomorrow" : "Zajtra",
    "No due" : "Žiadny termín dokončenia"
},
"nplurals=4; plural=(n % 1 == 0 && n == 1 ? 0 : n % 1 == 0 && n >= 2 && n <= 4 ? 1 : n % 1 != 0 ? 2: 3);");
