OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Þú hefur búið til nýja borðið {board}",
    "{user} has created a new board {board}" : "{user} hefur búið til nýja borðið {board}",
    "You have deleted the board {board}" : "Þú hefur eytt borðinu {board}",
    "{user} has deleted the board {board}" : "{user} hefur eytt borðinu {board}",
    "You have restored the board {board}" : "Þú hefur endurheimt borðið {board}",
    "{user} has restored the board {board}" : "{user} hefur endurheimt borðið {board}",
    "You have shared the board {board} with {acl}" : "Þú hefur deilt borðinu {board} með {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} hefur deilt borðinu {board} með {acl}",
    "You have removed {acl} from the board {board}" : "Þú fjarlægðir {acl} af borðinu {board}",
    "{user} has removed {acl} from the board {board}" : "{user} fjarlægði {acl} af borðinu {board}",
    "You have renamed the board {before} to {board}" : "Þú hefur endurnefnt borðið {before} sem {board}",
    "{user} has renamed the board {before} to {board}" : "{user} hefur endurnefnt borðið {before} sem {board}",
    "You have archived the board {board}" : "Þú settir borðið {board} í geymslu",
    "{user} has archived the board {before}" : "{user} setti borðið {before} í geymslu",
    "You have unarchived the board {board}" : "Þú tókst borðið {board} úr geymslu",
    "{user} has unarchived the board {before}" : "{user} tók borðið {before} úr geymslu",
    "You have created a new list {stack} on board {board}" : "Þú hefur búið til nýjan lista {stack} á borðinu {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} hefur búið til nýjan lista {stack} á borðinu {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Þú hefur endurnefnt listann {before} sem {stack} á borðinu {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} hefur endurnefnt listann {before} sem {stack} á borðinu {board}",
    "You have deleted list {stack} on board {board}" : "Þú hefur eytt listanum {stack} á borðinu {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} hefur eytt listanum {stack} á borðinu {board}",
    "You have created card {card} in list {stack} on board {board}" : "Þú hefur búið til spjaldið {card} í listanum {stack} á borðinu {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} hefur búið til spjaldið {card} í listanum {stack} á borðinu {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Þú eyddir spjaldinu {card} í listanum {stack} á borðinu {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} eyddi spjaldinu {card} í listanum {stack} á borðinu {board}",
    "You have renamed the card {before} to {card}" : "Þú hefur endurnefnt spjaldið {before} sem {card}",
    "{user} has renamed the card {before} to {card}" : "{user} hefur endurnefnt spjaldið {before} sem {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Þú bættir við lýsingu á spjaldinu {card} í listanum {stack} á borðinu {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} bætti við lýsingu á spjaldinu {card} í listanum {stack} á borðinu {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Þú uppfærðir lýsingu á spjaldinu {card} í listanum {stack} á borðinu {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} uppfærði lýsingu á spjaldinu {card} í listanum {stack} á borðinu {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Þú settir í geymslu spjaldið {card} í listanum {stack} á borðinu {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} setti í geymslu spjaldið {card} í listanum {stack} á borðinu {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Þú tókst úr geymslu spjaldið {card} í listanum {stack} á borðinu {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} tók úr geymslu spjaldið {card} í listanum {stack} á borðinu {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Þú hefur merkt spjaldið {card} sem lokið í listanum {stack} á borðinu {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} hefur merkt spjaldið {card} sem lokið í listanum {stack} á borðinu {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Þú hefur merkt spjaldið {card} sem ólokið í listanum {stack} á borðinu {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} hefur merkt spjaldið {card} sem ólokið í listanum {stack} á borðinu {board}",
    "You have removed the due date of card {card}" : "Þú fjarlægðir gildistíma af spjaldinu {card}",
    "{user} has removed the due date of card {card}" : "{user} fjarlægði gildistíma af spjaldinu {card}",
    "You have set the due date of card {card} to {after}" : "Þú settir gildistíma spjaldsins {card} á {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} setti gildistíma spjaldsins {card} á {after}",
    "You have updated the due date of card {card} to {after}" : "Þú uppfærðir gildistíma spjaldsins {card} á {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} uppfærði gildistíma spjaldsins {card} á {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Þú bættir við merkingunni {label} af spjaldinu {card} í listanum {stack} á borðinu {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} bætti við merkingunni {label} af spjaldinu {card} í listanum {stack} á borðinu {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Þú hefur fjarlægt merkinguna {label} af spjaldinu {card} í listanum {stack} á borðinu {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} fjarlægði merkinguna {label} af spjaldinu {card} í listanum {stack} á borðinu {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Þú úthlutaðir {assigneduser} á spjaldið {card} á borðinu {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} úthlutaði {assigneduser} á spjaldið {card} á borðinu {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Þú hefur afturkallað úthlutun {assigneduser} á spjaldið {card} á borðinu {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} hefur afturkallað úthlutun {assigneduser} á spjaldið {card} á borðinu {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Þú færðir spjaldið {card} úr listanum {stackBefore} yfir í {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} færði spjaldið {card} úr listanum {stackBefore} yfir í {stack}",
    "You have added the attachment {attachment} to card {card}" : "Þú bættir viðhenginu {attachment} við spjaldið {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} bætti viðhenginu {attachment} við spjaldið {card}",
    "You have updated the attachment {attachment} on card {card}" : "Þú uppfærðir viðhengið {attachment} á spjaldinu {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} uppfærði viðhengið {attachment} á spjaldinu {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Þú eyddir viðhenginu {attachment} af spjaldinu {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} eyddi viðhenginu {attachment} af spjaldinu {card}",
    "You have restored the attachment {attachment} to card {card}" : "Þú endurheimtir viðhengið {attachment} á spjaldinu {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} endurheimti viðhengið {attachment} á spjaldinu {card}",
    "You have commented on card {card}" : "Þú settir inn athugasemd við spjaldið {card}",
    "{user} has commented on card {card}" : "{user} setti inn athugasemd við spjaldið {card}",
    "Deck" : "Deck-spjaldaforrit",
    "Changes in the <strong>Deck app</strong>" : "Breytingar í <strong>Deck-spjaldaforritinu</strong>",
    "A <strong>board, list or card</strong> was changed" : "<strong>Borði, lista eða spjaldi</strong> hefur verið breytt",
    "A <strong>comment</strong> was created on a card" : "<strong>Athugasemd</strong> var gerð á spjaldi",
    "A <strong>card description</strong> has been changed" : "<strong>Lýsingu á spjaldi</strong> hefur verið breytt",
    "The file was uploaded" : "Skráin var send inn",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Innsend skrá er stærri en upload_max stillingin í php.ini:",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Innsenda skráin er stærri en MAX_FILE_SIZE sem skilgreint er í HTML sniðinu.",
    "The file was only partially uploaded" : "Einungis hluti af innsendri skrá skilaði sér",
    "No file was uploaded" : "Engin skrá skilaði sér",
    "Missing a temporary folder" : "Vantar bráðabirgðamöppu",
    "Could not write file to disk" : "Tókst ekki að skrifa skrá á disk.",
    "A PHP extension stopped the file upload" : "PHP-viðbót stöðvaði innsendingu skráar",
    "No file uploaded or file size exceeds maximum of %s" : "Engin innsend skrá eða að skráarstærð fór fram úr hámarksstæðinni %s",
    "Cards due today" : "Spjöld sem á að ljúka í dag",
    "Cards due tomorrow" : "Spjöld sem á að ljúka á morgun",
    "Upcoming cards" : "Spjöld á næstunni",
    "Load more" : "Hlaða inn meira",
    "Welcome to Nextcloud Deck!" : "Velkomin í Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Spjaldinu \"%s\" á \"%s\" hefur verið úthlutað til þín af %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} hefur úthlutað spjaldinu {deck-card} á {deck-board} til þín.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Spjaldið {deck-card} á {deck-board} er komið fram yfir lokadagsetningu.",
    "%s has mentioned you in a comment on \"%s\"." : "%s minntist á þig í athugasemd við \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} minntist á þig í athugasemd við {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Borðinu \"%s\" hefur verið deilt með þér af %s.",
    "{user} has shared {deck-board} with you." : "{user} deildi {deck-board} með þér.",
    "Deck board" : "Deck-borð",
    "Owned by %1$s" : "Í eigu notandans %1$s",
    "Deck boards, cards and comments" : "Deck-borð, spjöld og athugasemdir",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Frá %1$s, í %2$s/%3$s, í eigu notandans %4$s",
    "Create a new deck card" : "Búa til nýtt spjald",
    "Card comments" : "Athugasemdir spjalds",
    "%s on %s" : "%s á %s",
    "Deck boards and cards" : "Deck-borð og spjöld",
    "No data was provided to create an attachment." : "Engin gögn voru gefin til að útbúa viðhengi.",
    "Finished" : "Lokið",
    "To review" : "Til að yfirfara",
    "Action needed" : "Aðgerðar krafist",
    "Later" : "Seinna",
    "copy" : "afrita",
    "Read more inside" : "Lesa meira inni í",
    "Custom lists - click to rename!" : "Sérsniðnir listar - smelltu til að endurnefna!",
    "To Do" : "Verkefnalisti",
    "In Progress" : "Í vinnslu",
    "Done" : "Lokið",
    "1. Open to learn more about boards and cards" : "1. Opnaðu til að læra meira um borð og spjöld",
    "2. Drag cards left and right, up and down" : "2. Dragðu spjöld hægri vinstri og upp niður",
    "3. Apply rich formatting and link content" : "3. Settu sniðinn texta og efni í tenglum",
    "4. Share, comment and collaborate!" : "4. Deildu, gerðu athugasemdir og eigðu í samstarfi!",
    "Create your first card!" : "Útbúðu fyrsta spjaldið þitt!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Þessi athugasemd er lengri en %s stafir.\nBætt við sem viðhengi á spjald með heitinu %s.\nAðgengileg á slóðinni: %s.",
    "Attachments" : "Viðhengi",
    "Card not found" : "Spjald fannst ekki",
    "Path is already shared with this card" : "Slóðinni er þegar deilt með þessu spjaldi",
    "Invalid date, date format must be YYYY-MM-DD" : "Ógild dagsetning, dagsetningasniðið verður að vera ÁÁÁÁ-MM-DD",
    "Personal planning and team project organization" : "Persónuleg áætlanagerð og skipulag verkefnisvinnu",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Dekk (e. Deck) er skipulagningartól í kanban-stíl sem ætlað er fyrir verkefna- og persónuupplýsingastjórnun hópvinnuteyma innan Nextcloud.\n\n\n- 📥 Settu verkefnin þín á spjöld of raðaðu þeim á ýmsa vegu\n- 📄 Skrifaðu niður minnispunkta í Markdown-kóða\n- 🔖 Úthlutaðu merkjum/skýringum til að bæta skipulagninguna\n- 👥 Deildu með vinnuhópnum, vinum eða fjölskyldu\n- 📎 Hengdu við skrár og ívefðu þær í lýsinguna með Markdown-kóða\n- 💬 Ræddu málin við hópinn þinn með athugasemdum\n- ⚡ Haltu utan um breytingar í virknistreyminu\n- 🚀 Haltu verkefnunum þínum skipulögðum",
    "Add board" : "Bæta við borði",
    "Card details" : "Nánar um spjald",
    "Select the board to link to a project" : "Veldu borðið sem á að tengja við verkefnið",
    "Search by board title" : "Leita eftir titli borðs",
    "Select board" : "Veldu borð",
    "Move/copy card" : "Færa/afrita spjald",
    "Select a board" : "Veldu borð",
    "No lists available" : "Engir listar tiltækir",
    "Select a list" : "Veldu lista",
    "Move card" : "Færa spjald",
    "Copy card" : "Afrita spjald",
    "Select the card to link to a project" : "Veldu spjaldið sem á að tengja við verkefnið",
    "Link to card" : "Tengill á spjald",
    "Select a card" : "Veldu spjald",
    "Cancel" : "Hætta við",
    "This board is read only" : "Þetta spjald er ritvarið",
    "Drop your files to upload" : "Slepptu skrám til að senda þær inn",
    "File already exists" : "Skrá er þegar til",
    "A file with the name {filename} already exists." : "Skrá með heitinu {filename} er þegar til",
    "Do you want to overwrite it?" : "Viltu skrifa yfir það?",
    "Overwrite file" : "Skrifa yfir skrá",
    "Keep existing file" : "Halda fyrirliggjandi skrá",
    "Add card" : "Bæta við spjaldi",
    "Archived cards" : "Spjöld í geymslu",
    "Add list" : "Bæta við lista",
    "List name" : "Heiti á lista",
    "Clear search" : "Hreinsa leit",
    "Active filters" : "Virkar síur",
    "Apply filter" : "Beita síu",
    "Filter by tag" : "Sía eftir merki",
    "Filter by assigned user" : "Sía eftir úthlutuðum notanda",
    "Unassigned" : "Ekki úthlutað",
    "Filter by status" : "Sía eftir stöðu",
    "Open and completed" : "Opið og frágengið",
    "Open" : "Opna",
    "Completed" : "Lokið",
    "Filter by due date" : "Sía eftir lokadagsetningu",
    "Overdue" : "Fram yfir tímamörk",
    "Next 24 hours" : "Næstu 24 klukkustundir",
    "Next 7 days" : "Næstu 7 daga",
    "Next 30 days" : "Næstu 30 daga",
    "No due date" : "Enginn lokadagur",
    "Clear filter" : "Hreinsa síu",
    "View Modes" : "Skoðunarhamir",
    "Toggle View Modes" : "Víxla milli skoðunarhama",
    "Hide archived cards" : "Fela spjöld í geymslu",
    "Show archived cards" : "Sýna spjöld í geymslu",
    "Toggle compact mode" : "Víxla þjöppuðum ham af/á",
    "Hide card cover images" : "Fela kápumyndir spjalda",
    "Show card cover images" : "Birta kápumyndir spjalda",
    "Open details" : "Opna nánari upplýsingar",
    "Details" : "Details",
    "Deck settings" : "Stillingar Deck",
    "General" : "Almennt",
    "Use bigger card view" : "Nota stærri spjaldasýn",
    "Appearance" : "Útlit",
    "Show card ID badge" : "Sýna auðkennismerki spjalds",
    "Show boards in calendar/tasks" : "Birta borð í dagatali/verkefnum",
    "Limit board creation to some groups" : "Takmarka gerð borða við suma hópa",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Notendur sem tilheyra ekki þessum hópum eru útilokaðir frá því að útbúa sín eigin borð, en munu samt geta áfram unnið með borð sem hefur verið deilt til þeirra.",
    "Board actions" : "Aðgerðir borðs",
    "Scroll sideways" : "Skruna til hliðar",
    "Navigate between cards" : "Flakka á milli spjalda",
    "Close card details" : "Loka nánari upplýsingum um spjald",
    "Search" : "Search",
    "Show card filters" : "Sýna síur spjalds",
    "Clear card filters" : "Hreinsa síur spjalds",
    "Show those shortcuts" : "Sýna þessa flýtilykla",
    "Card actions" : "Aðgerðir spjalds",
    "Open card details" : "Opna nánari upplýsingar um spjald",
    "Edit the card title" : "Breyta titli spjalds",
    "Assign yourself to the current card" : "Úthluta þér núverandi spjaldi",
    "Archive/unarchive the current card" : "Setja/taka spjaldið úr geymslu",
    "Mark card as completed/not completed" : "Merkja spjaldið sem frágengið/ólokið",
    "Open card menu" : "Opna valmynd spjalds",
    "Currently present people" : "Viðstatt fólk",
    "Loading board" : "Hleð inn borði",
    "Board not found" : "Borð fannst ekki",
    "Create a new list to add cards to this board" : "Búðu til nýjan lista til að bæta spjöldum á þetta borð",
    "Sharing" : "Deiling",
    "Tags" : "Merki",
    "Deleted items" : "Eydd atriði",
    "Activity" : "Virkni",
    "Deleted lists" : "Eyddi listum",
    "Undo" : "Afturkalla",
    "Deleted cards" : "Eydd spjöld",
    "Hours" : "klukkustundir",
    "Days" : "Dagar",
    "Weeks" : "Vikur",
    "Months" : "Mánuðir",
    "No cards yet" : "Engin spjöld ennþá",
    "Failed to create share with {displayName}" : "Gat ekki búið til sameign með {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Ertu viss um að þú viljir yfirfæra borðið {title} til {user}?",
    "Transfer the board." : "Flytja borðið.",
    "Transfer" : "Færa",
    "The board has been transferred to {user}" : "Borðið hefur verið yfirfært til {user}",
    "Failed to transfer the board to {user}" : "Mistókst að yfirfæra borðið til {user}",
    "Board owner" : "Eigandi borðs",
    "(Group)" : "(Hópur)",
    "(Team)" : "(Teymi)",
    "(remote)" : "(fjartengt)",
    "Can edit" : "Getur breytt",
    "Can share" : "Getur deilt",
    "Can manage" : "Gerur sýslað með",
    "Owner" : "Eigandi",
    "Delete" : "Eyða",
    "List deleted" : "Lista eytt",
    "Edit list title" : "Breyta titli lista",
    "Archive all cards" : "Setja öll spjöld í geymslu",
    "Unarchive all cards" : "Taka öll spjöld úr geymslu",
    "Delete list" : "Eyða lista",
    "Archive all cards in this list" : "Setja öll spjöld á þessum lista í geymslu",
    "Unarchive all cards in this list" : "Taka öll spjöld á þessum lista úr geymslu",
    "Add a new card" : "Bæta við nýju spjaldi",
    "Card name" : "Heiti spjalds",
    "title and color value must be provided" : "titill og litgildi verða að vera til staðar",
    "Edit" : "Breyta",
    "Add a new tag" : "Bæta við nýju merki",
    "Board name" : "Heiti borðs",
    "Members" : "Meðlimir",
    "Assign to users/groups/team" : "Úthluta á notendur/hópa/teymi",
    "Assign a user to this card…" : "Úthluta notanda á þetta spjald",
    "Select a user to assign to this card…" : "Veldu notanda sem úthluta á þessu spjaldi…",
    "File to share" : "Skrá til að deila",
    "Invalid path selected" : "Ógild slóð valin",
    "Upload new files" : "Senda inn nýjar skrár",
    "Share from Files" : "Deila úr skrám",
    "Pending share" : "Sameign í bið",
    "Add this attachment" : "Bæta við þessu viðhengi",
    "Show in Files" : "Sýna í skrám",
    "Download" : "Sækja",
    "Remove attachment" : "Fjarlægja viðhengi",
    "Delete Attachment" : "Eyða viðhengi",
    "Restore Attachment" : "Endurheimta viðhengi",
    "The title cannot be empty." : "Titill getur ekki verið auður.",
    "Open in sidebar view" : "Opna í hliðarspjaldssýn",
    "Open in bigger view" : "Opna í stærri sýn",
    "Comments" : "Athugasemdir",
    "Failed to load comments" : "Mistókst að hlaða inn athugasemdum",
    "No comments yet. Begin the discussion!" : "Engar athugasemdir ennþá. Byrjaðu umræðuna!",
    "The comment cannot be empty." : "Athugasemdin má ekki vera tóm.",
    "The comment cannot be longer than 1000 characters." : "Athugasemdin má ekki vera lengri en 1000 stafir.",
    "Submit" : "Senda inn",
    "Created:" : "Búið til:",
    "In reply to" : "Sem svar til",
    "Cancel reply" : "Hætta við svar",
    "Reply" : "Svara",
    "Update" : "Uppfæra",
    "Mark as done" : "Merkja sem lokið",
    "Mark as not done" : "Merkja sem ólokið",
    "Write a description …" : "Skrifaðu lýsingu …",
    "Description" : "Lýsing",
    "(Unsaved)" : "(Óvistað)",
    "(Saving…)" : "(Vista…)",
    "Formatting help" : "Hjálp við að sníða",
    "Edit description" : "Breyta lýsingu",
    "View description" : "Skoða lýsingu",
    "Add Attachment" : "Bæta við viðhengi",
    "Choose attachment" : "Veldu viðhengi",
    "Select Date" : "Veldu dag",
    "Later today – {timeLocale}" : "Síðar í dag – {timeLocale}",
    "Set due date for later today" : "Setja skil á seinna í dag",
    "Tomorrow – {timeLocale}" : "Á morgun – {timeLocale}",
    "Set due date for tomorrow" : "Setja skil á morgun",
    "This weekend – {timeLocale}" : "Þessa helgi – {timeLocale}",
    "Set due date for this weekend" : "Setja skil á þessa helgi",
    "Next week – {timeLocale}" : "Í næstu viku – {timeLocale}",
    "Set due date for next week" : "Setja skil á í næstu viku",
    "Assign a due date to this card…" : "Úthluta skiladagsetningu á þetta spjald…",
    "Set a due date" : "Setja lokadag",
    "Add due date" : "Bæta við skiladegi",
    "Choose a date" : "Veldu dagsetningu",
    "Remove due date" : "Fjarlægja skiladag",
    "Due at:" : "Á að ljúka klukkan:",
    "Not done" : "Ekki lokið",
    "Unarchive card" : "Taka spjald úr geymslu",
    "Archive card" : "Setja spjald í geymslu",
    "Assign a tag to this card…" : "Úthluta merki á þetta spjald",
    "Select or create a tag…" : "Veldu eða búðu til merki…",
    "Create a new tag:" : "Búa til nýtt merki:",
    "(group)" : "(hópur)",
    "{count} comments, {unread} unread" : "{count} athugasemdir, {unread} ólesnar",
    "Todo items" : "Atriði á verkefnalista",
    "Edit card title" : "Breyta titli spjalds",
    "Open link" : "Opna tengil",
    "Card deleted" : "Spjaldi eytt",
    "Edit title" : "Breyta titli",
    "Assign to me" : "Úthluta mér",
    "Unassign myself" : "Aftengja úthlutun sjálfs mín",
    "Delete card" : "Eyða spjaldi",
    "All boards" : "Öll borð",
    "Archived boards" : "Borð í geymslu",
    "Shared with you" : "Deilt með þér",
    "Cancel edit" : "Hætta við breytingar",
    "Save board" : "Vista borð",
    "Board {0} deleted" : "Borðinu {0} eytt",
    "All cards" : "Öll spjöld",
    "Only assigned cards" : "Aðeins úthlutuð spjöld",
    "No reminder" : "Engin áminning",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Ertu viss um að þú viljir eyða borðinu {title}?. Þetta mun eyða öllum gögnum í þessu borði, meðal annars spjöldum í geymslu.",
    "Delete the board?" : "Eyða borðinu?",
    "An error occurred" : "Villa kom upp",
    "Board details" : "Nánar um borð",
    "Edit board" : "Breyta borði",
    "Clone board" : "Klóna borð",
    "Unarchive board" : "Taka borð úr geymslu",
    "Archive board" : "Setja borð í geymslu",
    "Export board" : "Flytja út borð",
    "Turn on due date reminders" : "Kveikja á áminningum um skiladagsetningar",
    "Turn off due date reminders" : "Slökkva á áminningum um skiladagsetningar",
    "Due date reminders" : "Áminningar um skiladagsetningar",
    "Assigned cards" : "Úthlutuð spjöld",
    "No notifications" : "Engar tilkynningar",
    "Delete board" : "Eyða borði",
    "Clone {boardTitle}" : "Klóna {boardTitle}",
    "Clone cards" : "Klóna spjöld",
    "Clone assignments" : "Klóna úthlutanir",
    "Clone labels" : "Klóna merkingar",
    "Clone due dates" : "Klóna skiladaga",
    "Advanced options" : "Ítarlegir valkostir",
    "Move all cards to the first list" : "Færa öll spjöld á fyrsta listann",
    "Restore archived cards" : "Endurheimta spjöld úr geymslu",
    "Clone" : "Klóna",
    "Export" : "Flytja út",
    "Loading filtered view" : "Hleð inn síaðri sýn",
    "Search for {searchQuery} in all boards" : "Leita að {searchQuery} í öllum borðum",
    "No results found" : "Engar niðurstöður fundust",
    "Deck board {name}\n* Last modified on {lastMod}" : "Deck-borð {name}\n* Síðast breytt {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Búið til {created}\n* Síðast breytt {lastMod}\n* {nbAttachments} viðhengi\n* {nbComments} athugasemdir",
    "{nbCards} cards" : "{nbCards} spjöld",
    "Due on {date}" : "Skilist {date}",
    "{stack} in {board}" : "{stack} á {board}",
    "Click to expand description" : "Smelltu til að fella út lýsingu",
    "Click to expand comment" : "Smelltu til að fella út athugasemd",
    "Create card" : "Búa til spjald",
    "Create a new card" : "Búa til nýtt spjald",
    "Card title" : "Titill á spjaldi",
    "Creating the new card …" : "Bý til nýja spjaldið …",
    "Card \"{card}\" was added to \"{board}\"" : "Spjaldinu \"{card}\" var bætt á \"{board}\"",
    "Open card" : "Opna spjald",
    "Close" : "Loka",
    "No upcoming cards" : "Engin spjöld á næstunni",
    "upcoming cards today" : "spjöld væntanleg í dag",
    "upcoming cards tomorrow" : "spjöld væntanleg á morgun",
    "upcoming cards" : "væntanleg spjöld",
    "New card" : "Nýtt spjald",
    "Link to a board" : "Tengill við borð",
    "Link to a card" : "Tengja við spjald",
    "Create a card" : "Búa til spjald",
    "Message from {author} in {conversationName}" : "Skilaboð frá {author} í {conversationName}",
    "Something went wrong" : "Eitthvað fór úrskeiðis",
    "Failed to upload {name}" : "Mistókst að senda inn {name}",
    "Maximum file size of {size} exceeded" : "Fór yfir hámarks skráarstærð {size}",
    "Due date" : "Skiladagur",
    "Created" : "Búið til þann",
    "Modified" : "Breytt",
    "Error creating the share" : "Villa við að búa til sameignina",
    "Share with a Deck card" : "Deila með Deck-spjaldi",
    "Share {file} with a Deck card" : "Deila {file} með Deck-spjaldi",
    "Share" : "Deila",
    "The card \"%s\" on \"%s\" has reached its due date." : "Spjaldið \"%s\" á \"%s\" er komið fram yfir lokadagsetningu.",
    "Share board with a user, group or team …" : "Deila borði með notanda, hóp eða teymi …",
    "Searching for users, groups and teams …" : "Leita að notendum, hópum og teymum …",
    "No participants found" : "No participants found",
    "Save" : "Vista",
    "seconds ago" : "sekúndum síðan",
    "Today" : "Í dag",
    "Tomorrow" : "Á morgun",
    "No due" : "Enginn skilatími"
},
"nplurals=2; plural=(n % 10 != 1 || n % 100 == 11);");
