OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Ați creat panoul {board}",
    "{user} has created a new board {board}" : "{user} a creat panoul {board}",
    "You have deleted the board {board}" : "Ați șters panoul {board}",
    "{user} has deleted the board {board}" : "{user} a șters panoul {board}",
    "You have restored the board {board}" : "Ați restaurat panoul {board}",
    "{user} has restored the board {board}" : "{user} a restaurat panoul {board}",
    "You have shared the board {board} with {acl}" : "Ați partajat panoul {board} cu {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} a partajat panoul {board} cu {acl}",
    "You have removed {acl} from the board {board}" : "Ați eliminat {acl} din panou {board}",
    "{user} has removed {acl} from the board {board}" : "{user} a eliminat {acl} din panoul {board}",
    "You have renamed the board {before} to {board}" : "Ați redenumit panoul {before} în {board}",
    "{user} has renamed the board {before} to {board}" : "{user} a redenumit panoul {before} în {board}",
    "You have archived the board {board}" : "Ați arhivat panoul {board}",
    "{user} has archived the board {before}" : "{user} a arhivat panoul {before}",
    "You have unarchived the board {board}" : "Ați dezarhivat panoul {board}",
    "{user} has unarchived the board {before}" : "{user} a dezarhivat panoul {before}",
    "You have created a new list {stack} on board {board}" : "Ați creat o nouă listă {stack} în panoul {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} a creat o nouă listă {stack} în panoul {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Ați redenumit lista {before} în {stack} în panoul {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} a redenumit lista {before} în {stack} în panoul {board}",
    "You have deleted list {stack} on board {board}" : "Ați șters lista {stack} în panoul {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} a șters lista {stack} în panoul {board}",
    "You have created card {card} in list {stack} on board {board}" : "Ați creat cardul {card} în lista {stack} în panoul {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} a creat cardul {card} în lista {stack} în panoul {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Ați șters cardul {card} în lista {stack} în panoul {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} a șters cardul {card} în lista {stack} în panoul {board}",
    "You have renamed the card {before} to {card}" : "Ați redenumit cardul {before} în {card}",
    "{user} has renamed the card {before} to {card}" : "{user} a redenumit cardul {before} în {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Ați adăugat o descriere cardului {card} în lista {stack} în panoul {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} a adăugat o descriere cardului {card} în lista {stack} în panoul {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Ați actualizat descrierea cardului {card} în lista {stack} în panoul {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} a actualizat descrierea cardului {card} în lista {stack} în panoul {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Ați arhivat cardul {card} în lista {stack} în panoul {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} a arhivat cardul {card} în lista {stack} în panoul {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Ați dezarhivat cardul {card} în lista {stack} în panoul {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} a dezarhivat cardul {card} în lista {stack} în panoul {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Ați marcat cardul {card} ca finalizat în lista {stack} în panoul {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} a marcat cardul {card} ca finalizat în lista {stack} în panoul {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Ați marcat cardul {card} ca nefinalizat în lista {stack} în panoul {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} a marcat cardul {card} ca nefinalizat în lista {stack} în panoul {board}",
    "You have removed the due date of card {card}" : "Ați eliminat scadența cardului {card}",
    "{user} has removed the due date of card {card}" : "{user} a eliminat scadența cardului {card}",
    "You have set the due date of card {card} to {after}" : "Ați setat scadența cardului {card} la {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} a setat scadența cardului {card} la {after}",
    "You have updated the due date of card {card} to {after}" : "Ați actualizat scadența cardului {card} la {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} a actualizat scadența cardului {card} la {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "A'i adăugat eticheta {label} cardului {card} în lista {stack} în panoul {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} a adăugat eticheta {label} cardului {card} în lista {stack} în panoul {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Ați șters eticheta {label} cardului {card} în lista {stack} în panoul {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} a șters eticheta {label} cardului {card} în lista {stack} în panoul {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Ați desemnat pe {assigneduser} pentru cardul {card} în panoul {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} a desemnat pe {assigneduser} pentru card {card} în panoul {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Ați dezasociat pe {assigneduser} de la cardul {card} în panoul {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} a dezasociat {assigneduser} de la cardul {card} în panoul {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Ați mutat cardul {card} din lista {stackBefore} în {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} a mutat cardul {card} din lista {stackBefore} în {stack}",
    "You have added the attachment {attachment} to card {card}" : "Ați adăugat atașamentul {attachment} la cardul {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} a adăugat atașamentul {attachment} la cardul {card}",
    "You have updated the attachment {attachment} on card {card}" : "Ați actualizat atașamentul {attachment} la cardul {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} a actualizat atașamentul {attachment} la cardul {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Ați șters atașamentul {attachment} cardului {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} a șters atașamentul {attachment} cardului {card}",
    "You have restored the attachment {attachment} to card {card}" : "Ați restaurat atașamentul {attachment} cardului {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} a restaurat atașamentul {attachment} cardului {card}",
    "You have commented on card {card}" : "Ați adăugat comentarii la cardul {card}",
    "{user} has commented on card {card}" : "{user} a adăugat comentarii la cardul {card}",
    "Changes in the <strong>Deck app</strong>" : "Modificări în <strong>Aplicația Deck</strong>",
    "A <strong>board, list or card</strong> was changed" : "A fost modificat un <strong>panou, listă sau card</strong>",
    "A <strong>comment</strong> was created on a card" : "A fost creat un <strong>comentariu</strong> la un card",
    "A <strong>card description</strong> has been changed" : "A fost modificată o <strong>descriere de card</strong>",
    "The file was uploaded" : "Fișierul a fost încărcat",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Fișierul încărcat depășește directiva upload_max_filesize din php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Fișierul încărcat depășește directiva MAX_FILE_SIZE specificată în formularul HTML",
    "The file was only partially uploaded" : "Fișierul a fost încărcat doar parțial",
    "No file was uploaded" : "Nu a fost încărcat niciun fișier",
    "Missing a temporary folder" : "Lipsește un dosar temporar",
    "Could not write file to disk" : "Nu s-a putut scrie fișierul pe disc",
    "A PHP extension stopped the file upload" : "O extensie PHP a oprit încărcarea fișierelor",
    "No file uploaded or file size exceeds maximum of %s" : "Niciun fișier pentru încărcare sau s-a depășit maximul de %s",
    "Cards due today" : "Carduri scadente azi",
    "Cards due tomorrow" : "Carduri scadente mâine",
    "Upcoming cards" : "Carduri viitoare",
    "Load more" : "Încarcă mai mult",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Cardul \"%s\" din \"%s\" v-a fost desemnat de %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} v-a desemnat pentru cardul {deck-card} din {deck-board}.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Cardul {deck-card} din {deck-board} a ajuns la scadență.",
    "%s has mentioned you in a comment on \"%s\"." : "%s v-a menționat într-un comentariu la \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} v-a menționat într-un comentariu la {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Panoul \"%s\" a fost partajat de %s.",
    "{user} has shared {deck-board} with you." : "{user} a partajat {deck-board} cu tine.",
    "Deck board" : "Panou Deck",
    "Owned by %1$s" : "Proprietar %1$s",
    "Deck boards, cards and comments" : "Panouri, carduri și comentarii",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "De la %1$s, în %2$s/%3$s, proprietar %4$s",
    "Create a new deck card" : "Creează un nou card",
    "Card comments" : "Comentarii card",
    "%s on %s" : "%s pe %s",
    "Deck boards and cards" : "Panouri și carduri",
    "No data was provided to create an attachment." : "Nu s-au furnizat date pentru crearea unui atașament.",
    "Finished" : "Finalizat",
    "To review" : "De revizuit",
    "Action needed" : "Acțiune necesară",
    "Later" : "Mai târziu",
    "copy" : "copiază",
    "To Do" : "De Făcut",
    "In Progress" : "În curs de desfășurare",
    "Done" : "Realizat",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Comentariul are mai mult de %s caractere.\nA fost adăugat ca atașament la card cu numele %s.\nAccesibil la URL: %s.",
    "Attachments" : "Atașamente ",
    "File" : "Fișier",
    "Card not found" : "Card negăsit ",
    "Path is already shared with this card" : "Calea este deja partajată cu acest card",
    "Invalid date, date format must be YYYY-MM-DD" : "Dată invalidă, formatul trebuie să fie AAAA-LL-ZZ",
    "Personal planning and team project organization" : "Planificare personală și organizarea proiectelor",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck este un utilitar de organizare stil kanban care are ca scop planificarea personală șiorganizarea proiectelor de echipă în Nextcloud.\n\n\n- 📥 Adăugați sarcinile la carduri și ordonați-le\n- 📄 Scrieți note suplimentare în Markdown\n- 🔖 Asociați etichete pentru o mai bună organizare\n- 👥 Partajați cu echipa, prietenii și familia\n- 📎 Atașați fișiere și referiți-le în descrierea Markdown\n- 💬 Discutați cu echipa folosind comentarii\n- ⚡ Urmăriți modificările din fluxul de activitate\n- 🚀 Organizați-vă proiectul",
    "Add board" : "Adaugă panou",
    "Card details" : "Detaliile cardului",
    "Select the board to link to a project" : "Selectați un panou pentru asociere la un proiect",
    "Search by board title" : "Căutare după titlul panoului",
    "Select board" : "Selectează panoul",
    "Select a board" : "Selectați un panou",
    "No lists available" : "Nu este disponibilă nicio listă",
    "Select a list" : "Selectează o listă",
    "Move card" : "Mută card",
    "Select the card to link to a project" : "Selectați un card pentru asociere la un proiect",
    "Link to card" : "Link la card",
    "Select a card" : "Selectează un card",
    "Cancel" : "Anulează",
    "This board is read only" : "Panoul este în citire",
    "Drop your files to upload" : "Trageți fișierele pentru încărcare",
    "File already exists" : "Fișierul există deja",
    "A file with the name {filename} already exists." : "Un fișier cu numele {filename} există deja.",
    "Do you want to overwrite it?" : "Îl suprascrieți?",
    "Overwrite file" : "Suprascrie fișier",
    "Keep existing file" : "Păstrează fișierul existent",
    "Add card" : "Adaugă card",
    "Archived cards" : "Carduri arhivate",
    "Add list" : "Adaugă listă ",
    "List name" : "Nume listă",
    "Active filters" : "Filtre active",
    "Apply filter" : "Aplică filtru",
    "Filter by tag" : "Filtrare după etichetă",
    "Filter by assigned user" : "Filtrează după utilizatorul atribuit",
    "Unassigned" : "Nealocat",
    "Open" : "Deschide",
    "Completed" : "Terminat",
    "Filter by due date" : "Filtrează după data scadentă",
    "Overdue" : "Termen depășit",
    "Next 24 hours" : "Următoarele 24 de ore",
    "Next 7 days" : "Următorele 7 zile",
    "Next 30 days" : "Următorele 30 zile",
    "No due date" : "Fără dată limită",
    "Clear filter" : "Șterge filtrul",
    "View Modes" : "Moduri de vizualizare",
    "Toggle View Modes" : "Comuta mod vizualizare",
    "Hide archived cards" : "Ascunde carduri arhivate",
    "Show archived cards" : "Arată carduri arhivate",
    "Toggle compact mode" : "Comuta la modul compact",
    "Hide card cover images" : "Ascunde coperta cardurilor",
    "Show card cover images" : "Arată coperta cardurilor",
    "Open details" : "Deschide detaliile",
    "Details" : "Detalii",
    "Deck settings" : "Setări Deck",
    "General" : "General",
    "Use bigger card view" : "Mărește cardul",
    "Appearance" : "Aspect",
    "Show card ID badge" : "Arată ID-ul cardului",
    "Show boards in calendar/tasks" : "Arată panourile în Calendar / Sarcini",
    "Limit board creation to some groups" : "Permite crearea panourilor doar anumitor grupuri",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Utilizatorii din afara acestor grupuri nu vor putea să-și creeze propriile panouri, dar vor putea să acceseze panourile care le-au fost partajate",
    "Search" : "Căutare",
    "Show those shortcuts" : "Afișează aceste scurtături",
    "Currently present people" : "Persoane prezenta acum",
    "Loading board" : "Se încarcă panoul",
    "Board not found" : "Panou inexistent",
    "Create a new list to add cards to this board" : "Creează o listă pentru adăugare carduri la acest panou",
    "Sharing" : "Partajare",
    "Tags" : "Etichete",
    "Deleted items" : "Elemente șterse",
    "Activity" : "Activitate",
    "Deleted lists" : "Liste șterse",
    "Undo" : "Anulează ultima acțiune",
    "Deleted cards" : "Carduri șterse",
    "No cards yet" : "Nu există carduri încă ",
    "Failed to create share with {displayName}" : "Nu s-a creat partajarea cu  {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Sigur transferați panoul {title} la {user}?",
    "Transfer the board." : "Transferă panoul.",
    "Transfer" : "Transfer",
    "The board has been transferred to {user}" : "Panoul a fost transferat la {user}",
    "Failed to transfer the board to {user}" : "Eroare la transferul panoului la {user}",
    "Board owner" : "Proprietar panou",
    "(Group)" : "(Grup)",
    "(remote)" : "(la distanță)",
    "Can edit" : "Poate edita",
    "Can share" : "Can share",
    "Can manage" : "Poate administra",
    "Owner" : "Proprietar",
    "Delete" : "Șterge",
    "List deleted" : "Lista a fost ștearsă",
    "Edit list title" : "Editare titlu listă",
    "Archive all cards" : "Arhivează toate cardurile",
    "Unarchive all cards" : "Dezarhivează toate cardurile",
    "Delete list" : "Șterge lista",
    "Archive all cards in this list" : "Arhivează toate cardurile din listă",
    "Unarchive all cards in this list" : "Dezarhivează toate cardurile din listă",
    "Add a new card" : "Adaugă un card nou",
    "Card name" : "Nume card",
    "title and color value must be provided" : "trebuie furnizate titlul și culoarea ",
    "Edit" : "Editează",
    "Add a new tag" : "Adaugă o etichetă",
    "Board name" : "Nume panou",
    "Members" : "Membri",
    "Assign a user to this card…" : "Desemnează un utilizator pentru acest card ...",
    "File to share" : "Fișier de partajat",
    "Invalid path selected" : "Calea selectată este invalidă",
    "Upload new files" : "Încarcă noi fișiere",
    "Share from Files" : "Partajează din Fișiere",
    "Pending share" : "Partajare în așteptare",
    "Add this attachment" : "Adaugă acest atașament",
    "Show in Files" : "Arată în Fișiere",
    "Download" : "Descărcare",
    "Remove attachment" : "Elimină atașamentul",
    "Delete Attachment" : "Șterge atașamentul",
    "Restore Attachment" : "Restaurează atașamentul",
    "The title cannot be empty." : "Titlul nu poate fi gol.",
    "Open in sidebar view" : "Deschide în vederea laterală",
    "Open in bigger view" : "Deschide în vedere mărită",
    "Comments" : "Comentarii",
    "Failed to load comments" : "Eroare la încărcarea comentariilor",
    "No comments yet. Begin the discussion!" : "Niciun comentariu încă. Începeți discuția!",
    "The comment cannot be empty." : "Comentariul nu poate fi gol",
    "The comment cannot be longer than 1000 characters." : "Comentariul nu poate avea mai mult de 1000 de caractere.",
    "Submit" : "Trimite",
    "Created:" : "Creat:",
    "In reply to" : "Ca răspuns la",
    "Cancel reply" : "Anulează răspunsul",
    "Reply" : "Răspunde",
    "Update" : "Actualizare",
    "Mark as done" : "Marchează ca realizat",
    "Mark as not done" : "Marchează ca realizat",
    "Write a description …" : "Scrieți o descriere ...",
    "Description" : "Descriere",
    "(Unsaved)" : "(Nesalvat)",
    "(Saving…)" : "(Se salvează…)",
    "Formatting help" : "Ajutor pentru formatare",
    "Edit description" : "Editează descriere",
    "View description" : "Vedeți descierea",
    "Add Attachment" : "Adaugă atașament",
    "Choose attachment" : "Selectați atașamentul",
    "Select Date" : "Selectează data",
    "Later today – {timeLocale}" : "Azi, mai târziu – {timeLocale}",
    "Set due date for later today" : "Setează scadența pentru azi, mai târziu",
    "Tomorrow – {timeLocale}" : "Mâine – {timeLocale}",
    "Set due date for tomorrow" : "Setează scadența pentru mâine",
    "This weekend – {timeLocale}" : "În acest weekend – {timeLocale}",
    "Set due date for this weekend" : "Setează scadența pentru acest weekend",
    "Next week – {timeLocale}" : "Săptămâna viitoare– {timeLocale}",
    "Set due date for next week" : "Setează scadența pentru săptămâna viitoare",
    "Assign a due date to this card…" : "Specificați o scadență pentru card ...",
    "Set a due date" : "Setați o scadență",
    "Add due date" : "Adăugați o scadență",
    "Choose a date" : "Alegeți o dată",
    "Remove due date" : "Șterge scadența",
    "Due at:" : "Scadență la:",
    "Unarchive card" : "Dezarhivează cardul",
    "Archive card" : "Arhivează card",
    "Assign a tag to this card…" : "Asociați o eticheta acestui card ...",
    "Create a new tag:" : "Creează o etichetă:",
    "(group)" : "(grup)",
    "{count} comments, {unread} unread" : "{count} comentarii, {unread} necitite",
    "Todo items" : "Elemente de executat",
    "Edit card title" : "Editează titlul cardului",
    "Card deleted" : "Cardul a fost șters",
    "Edit title" : "Editează titlul",
    "Assign to me" : "Atribuie-mi mie",
    "Unassign myself" : "Dezasociază-mi-l",
    "Delete card" : "Șterge card",
    "All boards" : "Toate panourile",
    "Archived boards" : "Panouri arhivate",
    "Shared with you" : "Partajat cu tine",
    "Cancel edit" : "Anulați editarea",
    "Board {0} deleted" : "Panoul {0} a fost șters",
    "All cards" : "Toate cardurile",
    "Only assigned cards" : "Doar cardurile repartizate",
    "No reminder" : "Fără mementouri",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Sunteți sigur că doriți să ștergeți panoul {title} ? Se vor șterge toate datele din panou, inclusiv cardurile arhivate.",
    "Delete the board?" : "Ștergeți panoul?",
    "Leave" : "Părăsește",
    "An error occurred" : "A apărut o eraore",
    "Board details" : "Detalii panou",
    "Edit board" : "Editează panou",
    "Clone board" : "Clonează panou",
    "Unarchive board" : "Dezarhivează panou",
    "Archive board" : "Arhivează panou",
    "Export board" : "Exportă panou",
    "Turn on due date reminders" : "Activează mementouri pentru scadență",
    "Turn off due date reminders" : "Dezctivează mementourile pentru scadență",
    "Due date reminders" : "Mementouri scadențe",
    "Assigned cards" : "Carduri repartizate",
    "No notifications" : "Nu sunt notificări",
    "Delete board" : "Șterge panou",
    "Clone cards" : "Clonează carduri",
    "Advanced options" : "Opțiuni avansate",
    "Clone" : "Clonează",
    "Export" : "Exportă",
    "Loading filtered view" : "Se încarcă vizualizarea filtrată",
    "Search for {searchQuery} in all boards" : "Căutare {searchQuery} în toate panourile",
    "No results found" : "Nu s-a găsit nimic",
    "Deck board {name}\n* Last modified on {lastMod}" : "Panoul Deck {name}\n* Ultima modificare în {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Creat în {created}\n* Ultima modificare în {lastMod}\n* {nbAttachments} atașamente\n* {nbComments} comentarii",
    "{nbCards} cards" : "{nbCards} carduri",
    "Due on {date}" : "Scadență în {date}",
    "{stack} in {board}" : "{stack} în {board}",
    "Click to expand description" : "Click pentru a extinde descrierea",
    "Click to expand comment" : "Click pentru extindere comentariu",
    "Create card" : "Creează card",
    "Create a new card" : "Creează un card",
    "Card title" : "Titlu card",
    "Creating the new card …" : "Se creează un nou card ...",
    "Card \"{card}\" was added to \"{board}\"" : "Cardul \"{card}\" a fost adăugat la \"{board}\"",
    "Open card" : "Deschide cardul",
    "Close" : "Închide",
    "No upcoming cards" : "Niciun card următor",
    "upcoming cards today" : "cardurile următoare astăzi",
    "upcoming cards tomorrow" : "cardurile următoare de mâine",
    "upcoming cards" : "carduri care urmează",
    "New card" : "Card nou",
    "Link to a board" : "Legătură la panou",
    "Link to a card" : "Legătură la card",
    "Create a card" : "Creează un card",
    "Message from {author} in {conversationName}" : "Mesaj de la {author} în {conversationName}",
    "Something went wrong" : "A apărut o eroare",
    "Failed to upload {name}" : "Eroare la încărcarea {name}",
    "Maximum file size of {size} exceeded" : "A fost depășită mărimea maximă de {size}",
    "Due date" : "Data scadenței",
    "Created" : "A fost creat",
    "Modified" : "Modificat",
    "Error creating the share" : "Eroare la crearea partajării",
    "Share with a Deck card" : "Partajează cu un card Deck",
    "Share {file} with a Deck card" : "Partajează {file} cu un card Deck",
    "Share" : "Partajează",
    "The card \"%s\" on \"%s\" has reached its due date." : "Cardul \"%s\" din \"%s\" a ajuns la scadență.",
    "No participants found" : "Nu s-au găsit participanți",
    "Save" : "Salvează",
    "seconds ago" : "secunde în urmă",
    "Today" : "Azi",
    "Tomorrow" : "Mâine",
    "No due" : "Fără scadență"
},
"nplurals=3; plural=(n==1?0:(((n%100>19)||((n%100==0)&&(n!=0)))?2:1));");
