/**
 * SPDX-FileCopyrightText: 2019 Nextcloud GmbH and Nextcloud contributors
 * SPDX-License-Identifier: AGPL-3.0-or-later
 */
.resource-type-deck img {
	opacity: 0.4 !important;
}

.resource-type-deck:hover img {
	opacity: 0.7 !important;
}
