OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Vytvořili jste tabuli {board}",
    "{user} has created a new board {board}" : "{user} vytvořil(a) novou tabuli {board}",
    "You have deleted the board {board}" : "Smazali jste tabuli {board}",
    "{user} has deleted the board {board}" : "{user} smazal(a) tabuli {board}",
    "You have restored the board {board}" : "Obnovili jste tabuli {board}",
    "{user} has restored the board {board}" : "{user} obnovil(a) tabuli {board}",
    "You have shared the board {board} with {acl}" : "Nasdíleli jste tabuli {board} s {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} nasdílel(a) tabuli {board} s {acl}",
    "You have removed {acl} from the board {board}" : "Odebrali jste {acl} z tabule {board}",
    "{user} has removed {acl} from the board {board}" : "{user} odebral(a) {acl} z tabule {board}",
    "You have renamed the board {before} to {board}" : "Přejmenovali jste tabuli {before} na {board}",
    "{user} has renamed the board {before} to {board}" : "{user} přejmenoval(a) tabuli {before} na {board}",
    "You have archived the board {board}" : "Archivovali jste tabuli {board}",
    "{user} has archived the board {before}" : "{user} archivoval(a) tabuli {before}",
    "You have unarchived the board {board}" : "Vrátili jste zpět z archivace tabuli {board}",
    "{user} has unarchived the board {before}" : "{user} vrátil(a) tabuli {before} zpět z archivace",
    "You have created a new list {stack} on board {board}" : "Vytvořili jste nový sloupec {stack} na tabuli {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} vytvořil(a) nový sloupec {stack} na tabuli {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Přejmenovali jste sloupec {before} na {stack} na tabuli {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} přejmenoval(a) sloupec {before} na {stack} na tabuli {board}",
    "You have deleted list {stack} on board {board}" : "Smazali jste sloupec {stack} na tabuli {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} smazal(a) sloupec {stack} na tabuli {board}",
    "You have created card {card} in list {stack} on board {board}" : "Vytvořili jste kartu {card} ve sloupci {stack} na tabuli {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} vytvořil(a) kartu {card} ve sloupci {stack} na tabuli {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Smazali jste kartu {card} ve sloupci {stack} na tabuli {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} smazal(a) kartu {card} ve sloupci {board} na tabuli {board}",
    "You have renamed the card {before} to {card}" : "Přejmenovali jste kartu {before} na {card}",
    "{user} has renamed the card {before} to {card}" : "{user} přejmenoval(a) kartu {before} na {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Přidali jste popis ke kartě {card} ve sloupci {stack} na tabuli {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} přidal(a) popis ke kartě {card} ve sloupci {stack} na tabuli {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Aktualizovali jste popis karty {card} ve sloupci {stack} na tabuli {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} aktualizoval(a) popis karty {card} ve sloupci {stack} na tabuli {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Archivovali kartu {card} ve sloupci {stack} na tabuli {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} archivoval(a) kartu {card} ve sloupci {stack} na tabuli {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Zrušili jste archivacii karty {card} ve sloupci {stack} na tabuli {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} zrušil(a) archivaci karty {card} ve sloupci {stack} na tabuli {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Označili jste kartu {card} jako splněnou v seznamu {stack} na tabuli {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} označil(a) {card} jako splněnou {stack} na tabuli {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Zrušili jste označení karty {card} jako splněné v seznamu {stack} na tabuli {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} zrušil(a) označení {card} jako splněné {stack} na tabuli {board}",
    "You have removed the due date of card {card}" : "Odebrali jste termín u karty {card}",
    "{user} has removed the due date of card {card}" : "{user} odebral(a) termín u karty {card}",
    "You have set the due date of card {card} to {after}" : "Nastavili jste termín u karty {card} na {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} nastavil(a) termín u karty {card} na {after}",
    "You have updated the due date of card {card} to {after}" : "Změnili jste termín u karty {card} na {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} aktualizoval(a) termín u karty {card} na {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Přidali jste štítek {label} kartě {card} ve sloupci {stack} na tabuli {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} přidal(a) štítek {label} ke kartě {card} ve sloupci {stack} na tabuli {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Odebrali jste štítek {label} z karty {card} ve sloupci {stack} na tabuli {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} odebral(a) štítek {label} z karty {card} ve sloupci {stack} na tabuli {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Přiřadili jste {assigneduser} ke kartě {card} na tabuli {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} přiřadil(a) {assigneduser} ke kartě {card} na tabuli {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Zrušili jste přiřazení {assigneduser} u karty {card} na tabuli {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} zrušil(a) přiřazení {assigneduser} z karty {card} na tabuli {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Přesunuli jste kartu {card} ze sloupce {stackBefore} do {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} přesunul(a) kartu {card} ze sloupce {stackBefore} do {stack}",
    "You have added the attachment {attachment} to card {card}" : "Přidali jste přílohu {attachment} ke kartě {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} přidal(a) přílohu {attachment} ke kartě {card}",
    "You have updated the attachment {attachment} on card {card}" : "Aktualizovali jste přílohu {attachment} u karty {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} aktualizoval(a) přílohu {attachment} u karty {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Smazali jste přílohu {attachment} u karty {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} smazal(a) přílohu {attachment} u karty {card}",
    "You have restored the attachment {attachment} to card {card}" : "Obnovili jste přílohu {attachment} ke kartě {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} obnovil(a) přílohu {attachment} ke kartě {card}",
    "You have commented on card {card}" : "Přidali jste komentář na kartě {card}",
    "{user} has commented on card {card}" : "{user} přidal(a) komentář na kartě {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Změny v <strong>aplikaci Deck</strong>",
    "A <strong>board, list or card</strong> was changed" : "<strong>tabule, seznam nebo karta</strong> byla změněna",
    "A <strong>comment</strong> was created on a card" : "Na kartě byl vytvořen <strong>komentář</strong>",
    "A <strong>card description</strong> has been changed" : "<strong>Popis karty</strong> byl změněn",
    "The file was uploaded" : "Soubor byl nahrán",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Velikost nahrávaného souboru překračuje limit nastavení direktivou upload_max_filesize v php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Nahrávaný soubor přesáhl svou velikostí hodnotu direktivy MAX_FILE_SIZE, určenou v HTML formuláři",
    "The file was only partially uploaded" : "Soubor byl nahrán pouze z části",
    "No file was uploaded" : "Nebyl nahrán žádný soubor",
    "Missing a temporary folder" : "Chybí složka pro dočasné soubory",
    "Could not write file to disk" : "Soubor se nedaří se zapsat na úložiště",
    "A PHP extension stopped the file upload" : "PHP rozšíření zastavilo nahrávání souboru.",
    "No file uploaded or file size exceeds maximum of %s" : "Nebyl nahrán žádný soubor nebo jeho velikost přesáhla %s",
    "Invalid file type. Only JSON files are allowed." : "Neplatný typ souboru. Povolené jsou pouze JSON soubory.",
    "Invalid JSON data" : "Neplatná JSON data",
    "Failed to import board" : "Tabuli se nepodařilo naimportovat",
    "Cards due today" : "Karty, kterým končí termín dnes",
    "Cards due tomorrow" : "Karty, které mají termín zítra",
    "Upcoming cards" : "Nadcházející karty",
    "Load more" : "Načíst více",
    "Welcome to Nextcloud Deck!" : "Vítejte v Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Kartu „%s“ na „%s“ vám přiřadil(a) %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} vám přiřadil(a) kartu {deck-card} na tabuli {deck-board}.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "U karty „%1$s“ z tabule „%2$s“ nastalo plánované datum dokončení.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Karta {deck-card} na tabuli {deck-board} dosáhla data plánovaného dokončení.",
    "%s has mentioned you in a comment on \"%s\"." : "%s vás zmínil(a) v komentáři k „%s“.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} vás zmínil(a) v komentáři na kartě {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Uživatel %s vám nasdílel(a) tabuli „%s“.",
    "{user} has shared {deck-board} with you." : "{user} vám nasdílel(a) tabuli {deck-board}.",
    "The remote board %1$s has been shared with you by %2$s" : "Vzdálená tabule %1$s vám byla nasdílena uživatelem %2$s",
    "Deck board" : "Tabule aplikace Deck",
    "Owned by %1$s" : "Vlastní %1$s",
    "Deck boards, cards and comments" : "Tabule, karty a komentáře aplikace Deck",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Od %1$s, v %2$s/%3$s, vlastní %4$s",
    "Create a new deck card" : "Vytvořit novou kartu v Deck",
    "Card comments" : "Komentáře v kartách",
    "%s on %s" : "%s na %s",
    "Deck boards and cards" : "Tabule a karty aplikace Deck",
    "No data was provided to create an attachment." : "Nebyla poskytnuta žádná data pro vytvoření přílohy.",
    "Finished" : "Dokončeno",
    "To review" : "K revizi",
    "Action needed" : "Nutná akce",
    "Later" : "Později",
    "copy" : "kopie",
    "Read more inside" : "Více se dočtete uvnitř",
    "Custom lists - click to rename!" : "Uživatelsky určené seznamy – kliknutím přejmenujte!",
    "To Do" : "Úkoly",
    "In Progress" : "Probíhá",
    "Done" : "Hotovo",
    "1. Open to learn more about boards and cards" : "1. Otevřete a zjistěte víc o tabulích a kartách",
    "2. Drag cards left and right, up and down" : "2. Přetahujte karty doleva a doprava, nahoru a dolů",
    "3. Apply rich formatting and link content" : "3. Uplatnit formátování a obsah odkazu",
    "4. Share, comment and collaborate!" : "4. Sdílejte, komentujte a spolupracujte!",
    "Create your first card!" : "Vytvořte svou první kartu!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Tento komentář je delší než %s znaků.\nPřidán jako příloha ke kartě nazvané %s.\nDostupné na URL: %s.",
    "Attachments" : "Přílohy",
    "File" : "Soubor",
    "date" : "datum",
    "Card not found" : "Karta nebyla nalezena",
    "Path is already shared with this card" : "Popis umístění už je touto kartou sdíleno",
    "Invalid date, date format must be YYYY-MM-DD" : "Neplatné datum – je třeba, aby formát data byl RRRR-MM-DD",
    "Personal planning and team project organization" : "Osobní plánování a organizování týmového projektu",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck je nástroj cílený na osobní nebo projektové plánování týmů v Kanban stylu, napojený do Nextcloud.\n\n\n- 📥 Zadávejte a uspořádávejte své úkoly do karet\n- 📄 Zapisujte si dodatečné poznámky (v Markdown) \n- 🔖 Přiřazujte štítky pro ještě lepší uspořádání\n- 👥 Sdílejte se svým týmem, přáteli nebo rodinou\n- 📎 Připojte soubory a vložte je do (Markdown) popisu \n- 💬 Diskutujte se svým týmem prostřednictvím komentářů\n- ⚡ Udržte si přehled o změnách prostřednictvím sledování proudu aktivit\n- 🚀 Dostaňte svůj projekt pod kontrolu",
    "Add board" : "Přidat tabuli",
    "Card details" : "Podrobnosti o kartě",
    "Select the board to link to a project" : "Vyberte tabuli kterou propojit s projektem",
    "Search by board title" : "Hledat podle názvu tabule",
    "Select board" : "Vybrat tabuli",
    "Move/copy card" : "Přesunout/zkopírovat kartu",
    "Select a board" : "Vybrat tabuli",
    "No lists available" : "Nejsou k dispozici žádné seznamy",
    "Select a list" : "Vyberte sloupec",
    "Move card" : "Přesunout kartu",
    "Copy card" : "Zkopírovat kartu",
    "Select the card to link to a project" : "Vyberte kartu kterou propojit s projektem",
    "Link to card" : "Propojit s kartou",
    "Select a card" : "Vybrat kartu",
    "Cancel" : "Storno",
    "This board is read only" : "Tato tabule je pouze pro čtení",
    "Drop your files to upload" : "Přetáhněte sem soubory, které chcete nahrát",
    "File already exists" : "Soubor už existuje",
    "A file with the name {filename} already exists." : "Soubor nazvaný {filename} už existuje.",
    "Do you want to overwrite it?" : "Chcete položku přepsat?",
    "Overwrite file" : "Přepsat soubor",
    "Keep existing file" : "Ponechat existující soubor",
    "Add card" : "Přidat kartu",
    "Archived cards" : "Archivované karty",
    "Add list" : "Přidat seznam",
    "List name" : "Název seznamu",
    "Clear search" : "Vyčistit hledání",
    "Active filters" : "Zapnuté filtry",
    "Apply filter" : "Uplatnit filtr",
    "Filter by tag" : "Filtrovat podle štítku",
    "Filter by assigned user" : "Filtrovat podle uživatele, který je úkolem pověřen",
    "Unassigned" : "Nepřiřazeno",
    "Filter by status" : "Filtrovat podle stavu",
    "Open and completed" : "Otevřít a dokončeno",
    "Open" : "Otevřít",
    "Completed" : "Dokončeno",
    "Filter by due date" : "Filtrovat podle termínu",
    "Overdue" : "Po termínu",
    "Next 24 hours" : "Následujících 24 hodin",
    "Next 7 days" : "Příštích 7 dnů",
    "Next 30 days" : "Příštích 30 dnů",
    "No due date" : "Žádný termín",
    "Clear filter" : "Vyčistit filtr",
    "View Modes" : "Zobrazit režimy",
    "Toggle View Modes" : "Přepnout mezi režimy zobrazení",
    "Hide archived cards" : "Skrýt archivované karty",
    "Show archived cards" : "Zobrazit archivované karty",
    "Toggle compact mode" : "Vyp/zap. kompaktní režim",
    "Hide card cover images" : "Skrýt obrázky přebalu karet",
    "Show card cover images" : "Zobrazit obrázky přebalu karet",
    "Open details" : "Otevřít podrobnosti",
    "Details" : "Podrobnosti",
    "Deck settings" : "Nastavení pro Deck",
    "General" : "Obecné",
    "Use bigger card view" : "Použít větší zobrazení karet",
    "Appearance" : "Vzhled",
    "Show card ID badge" : "Zobrazit odznáček s identifikátorem karty",
    "Show boards in calendar/tasks" : "Zobrazit tabule v kalendáři/úkolech",
    "Admin settings" : "Nastavení pro správu",
    "Limit board creation to some groups" : "Umožnit vytváření tabulí pouze vybraným skupinám",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Uživatelé, kteří nejsou členy těchto skupin nebudou moci vytvářet své vlastní tabule, ale pořád budou moci pracovat na tabulích, které jim byly nasdíleny.",
    "Enable federation" : "Zapnout federování",
    "Board actions" : "Akce ohledně tabule",
    "Scroll sideways" : "Posunout na strany",
    "Navigate between cards" : "Pohyb mezi kartami",
    "Close card details" : "Zavřít podrobnosti o kartě",
    "Search" : "Hledat",
    "Show card filters" : "Zobrazit filtry karty",
    "Clear card filters" : "Vyčistit filtry karty",
    "Show those shortcuts" : "Zobrazit tyto zkratky",
    "Card actions" : "Akce ohledně karty",
    "Open card details" : "Otevřít podrobnosti o kartě",
    "Edit the card title" : "Upravit nadpis karty",
    "Assign yourself to the current card" : "Přiřadit stávající kartu sobě",
    "Archive/unarchive the current card" : "Archivovat / zrušit archivaci stávající karty",
    "Mark card as completed/not completed" : "Označit kartu jako (ne)dokončenou",
    "Open card menu" : "Otevřít nabídku karty",
    "Currently present people" : "Nyní přítomní lidé",
    "Loading board" : "Načítání tabule",
    "Board not found" : "Tabule nenalezena",
    "Create a new list to add cards to this board" : "Pro přidání karet na tuto tabuli vytvořte nový seznam",
    "Sharing" : "Sdílení",
    "Tags" : "Štítky",
    "Deleted items" : "Smazané položky",
    "Activity" : "Aktivita",
    "Deleted lists" : "Smazané seznamy",
    "Undo" : "Vrátit zpět",
    "Deleted cards" : "Smazané karty",
    "Hours" : "Hodiny",
    "Days" : "Dny",
    "Weeks" : "Týdnů",
    "Months" : "Měsíců",
    "No cards yet" : "Zatím žádné karty",
    "Failed to create share with {displayName}" : "Nepodařilo se vytvořit sdílení s {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Opravdu chcete předat vlastnictví tabule {title} uživateli {user}?",
    "Transfer the board." : "Předat vlastnictví tabule.",
    "Transfer" : "Předat vlastnictví",
    "The board has been transferred to {user}" : "Vlastnictví tabule bylo předáno uživateli {user}",
    "Failed to transfer the board to {user}" : "Nepodařilo se předat vlastnictví tabule uživateli {user}",
    "Board owner" : "Vlastník tabule",
    "(Group)" : "(Skupina)",
    "(Team)" : "(Tým)",
    "(remote)" : "(vzdálené)",
    "Can edit" : "Může upravovat",
    "Can share" : "Může sdílet",
    "Can manage" : "Může spravovat",
    "Owner" : "Vlastník",
    "Delete" : "Smazat",
    "List deleted" : "Seznam smazán",
    "Edit list title" : "Upravit nadpis seznamu",
    "Archive all cards" : "Archivovat všechny karty",
    "Unarchive all cards" : "Zrušit archivaci všech karet",
    "Do not set cards as \"done\"" : "Nenastavovat karty jako „dokončeno“",
    "Set cards as \"done\"" : "Nastavit karty jako „hotovo“",
    "Delete list" : "Smazat seznam",
    "Archive all cards in this list" : "Archivovat všechny karty v tomto seznamu",
    "Unarchive all cards in this list" : "Zrušit archivaci všech karet v tomto seznamu",
    "Add a new card" : "Přidat novou kartu",
    "Card name" : "Název karty",
    "title and color value must be provided" : "je třeba zadat nadpis a zvolit barvu",
    "Edit" : "Upravit",
    "Add a new tag" : "Přidat nový štítek",
    "Board name" : "Název tabule",
    "Members" : "Členové",
    "Assign to users/groups/team" : "Přiřadit uživatelům/skupinám/týmům",
    "Assign a user to this card…" : "Přiřadit uživatele k této kartě …",
    "Select a user to assign to this card…" : "Vyberte uživatele, kterému tuto kartu přiřadit …",
    "File to share" : "Soubor ke sdílení",
    "Invalid path selected" : "Je vybrán neplatný popis umístění",
    "Upload new files" : "Nahrát nové soubory",
    "Share from Files" : "Sdílet ze Souborů",
    "Pending share" : "Čekající sdílení",
    "Add this attachment" : "Přidat tuto přílohu",
    "Show in Files" : "Zobrazit v Souborech",
    "Download" : "Stáhnout",
    "Remove attachment" : "Odebrat přílohu",
    "Delete Attachment" : "Smazat přílohu",
    "Restore Attachment" : "Obnovit přílohu",
    "The title cannot be empty." : "Nadpis je třeba vyplnit.",
    "Cannot close unsaved card!" : "Nebylo možné zavřít neuloženou kartu!",
    "Open in sidebar view" : "Otevřít v zobrazení v postranním panelu",
    "Open in bigger view" : "Otevřít ve větším zobrazení",
    "Comments" : "Komentáře",
    "Failed to load comments" : "Načtení komentářů se nezdařilo",
    "No comments yet. Begin the discussion!" : "Zatím bez komentářů. Zahajte diskuzi!",
    "The comment cannot be empty." : "Komentář je třeba vyplnit.",
    "The comment cannot be longer than 1000 characters." : "Délka komentáře může být nejvýše 1 000 znaků.",
    "Submit" : "Odeslat",
    "Created:" : "Vytvořeno:",
    "In reply to" : "V odpověď na",
    "Cancel reply" : "Zrušit odpověď",
    "Reply" : "Odpovědět",
    "Update" : "Aktualizovat",
    "Mark as done" : "Označit jako hotové",
    "Mark as not done" : "Označit jako nehotové",
    "Remove dependency" : "Odebrat závislost",
    "Write a description …" : "Zadejte popis …",
    "Could not save description" : "Nebylo možné uložit popis",
    "Description" : "Popis",
    "(Unsaved)" : "(Neuloženo)",
    "(Saving…)" : "(Ukládání …)",
    "Formatting help" : "Nápověda k formátování",
    "Edit description" : "Upravit popis",
    "View description" : "Zobrazit popis",
    "Add Attachment" : "Přidat přílohu",
    "Choose attachment" : "Zvolte přílohu",
    "Select Date" : "Vybrat datum",
    "Later today – {timeLocale}" : "Později dnes – {timeLocale}",
    "Set due date for later today" : "Nastavit termín na dnešek",
    "Tomorrow – {timeLocale}" : "Zítra – {timeLocale}",
    "Set due date for tomorrow" : "Nastavit termín na zítřek",
    "This weekend – {timeLocale}" : "Tento víkend – {timeLocale}",
    "Set due date for this weekend" : "Nastavit termín na tento týden",
    "Next week – {timeLocale}" : "Příští týden – {timeLocale}",
    "Set due date for next week" : "Nastavit termín na příští týden",
    "Assign a due date to this card…" : "Přiřadit této kartě termín …",
    "Set a due date" : "Vybrat termín",
    "Add due date" : "Přidat termín",
    "Choose a date" : "Zvolit datum",
    "Remove due date" : "Odstranit termín",
    "Due at:" : "Termín v:",
    "Not done" : "Nehotové",
    "Unarchive card" : "Zrušit archivaci karty",
    "Archive card" : "Archivovat kartu",
    "Assign a tag to this card…" : "Přiřadit této kartě štítek …",
    "Select or create a tag…" : "Vyberte nebo vytvořte štítek …",
    "Create a new tag:" : "Vytvořit nový štítek:",
    "(group)" : "(skupina)",
    "{count} comments, {unread} unread" : "{count} komentářů, {unread} nepřečtených",
    "Todo items" : "Položky k udělání",
    "Edit card title" : "Upravit nadpis karty",
    "Open link" : "Otevřít okaz",
    "Card deleted" : "Karta smazána",
    "Edit title" : "Upravit nadpis",
    "Assign to me" : "Přiřadit mě",
    "Unassign myself" : "Zrušit přiřazení sobě",
    "Delete card" : "Smazat kartu",
    "All boards" : "Všechny tabule",
    "Archived boards" : "Archivované tabule",
    "Shared with you" : "Sdíleno s vámi",
    "Cancel edit" : "Zrušit úpravu",
    "Save board" : "Uložit tabuli",
    "Board {0} deleted" : "Tabule {0} smazána",
    "All cards" : "Všechny karty",
    "Only assigned cards" : "Pouze přiřazené karty",
    "No reminder" : "Bez připomínky",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Opravdu chcete tabuli {title} smazat? Toto smaže veškerá data této tabule, včetně archivovaných karet.",
    "Delete the board?" : "Smazat tabuli?",
    "Are you sure you want to leave the board {title}?" : "Opravdu chcete opustit tabuli {title}?",
    "Leave the board?" : "Opustit tabuli?",
    "Leave" : "Opustit",
    "Failed to leave the board" : "Opuštění tabule se nezdařilo",
    "Exporting board..." : "Exportování tabule …",
    "An error occurred" : "Došlo k chybě",
    "Board details" : "Podrobnosti o tabuli",
    "Edit board" : "Upravit tabuli",
    "Clone board" : "Klonovat tabuli",
    "Unarchive board" : "Vrátit tabuli zpět z archivu",
    "Archive board" : "Archivovat tabuli",
    "Export board" : "Exportovat tabuli",
    "Turn on due date reminders" : "Zapnout upomínky termínů",
    "Turn off due date reminders" : "Vypnout upomínky termínů",
    "Remove as default board" : "Odebrat jako výchozí tabuli",
    "Set as default board" : "Nastavit jako výchozí tabuli",
    "Due date reminders" : "Upomínky termínů",
    "Assigned cards" : "Přiřazené karty",
    "No notifications" : "Žádná upozornění",
    "Delete board" : "Smazat tabuli",
    "Leave board" : "Opustit tabuli",
    "Importing board..." : "Importování tabule …",
    "Board imported successfully" : "Tabule úspěšně naimportována",
    "Import board" : "Naimportovat tabuli",
    "Clone {boardTitle}" : "Naklonovat {boardTitle}",
    "Clone cards" : "Klonovat karty",
    "Clone assignments" : "Přiřazení klonu",
    "Clone labels" : "Popisky klonu",
    "Clone due dates" : "Termíny klonu",
    "Advanced options" : "Pokročilé předvolby",
    "Move all cards to the first list" : "Přesunout všechny karty na první seznam",
    "Restore archived cards" : "Obnovit archivované karty",
    "Clone" : "Klonovat",
    "Export {boardTitle}" : "Exportovat {boardTitle}",
    "Export as JSON" : "Exportovat jako JSON",
    "Export as CSV" : "Exportovat jako CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Pozn.: Pro import zpět do aplikace Deck je podporován pouze formát JSON.",
    "Export" : "Exportovat",
    "Loading filtered view" : "Načítání filtrovaného pohledu",
    "Search for {searchQuery} in other boards" : "Hledat {searchQuery} v ostatních tabulích",
    "Search for {searchQuery} in all boards" : "Hledat {searchQuery} na všech tabulích",
    "No results found" : "Nenalezeny žádné výsledky",
    "Deck board {name}\n* Last modified on {lastMod}" : "Deck karta {name}\n* Naposledy změněno {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Vytvořeno {created}\n* Naposledy upraveno {lastMod}\n* {nbAttachments} příloh\n* {nbComments} komentářů",
    "{nbCards} cards" : "{nbCards} karet",
    "Due on {date}" : "Termín {date}",
    "{stack} in {board}" : "{stack} v {board}",
    "Click to expand description" : "Popis rozbalíte kliknutím",
    "Click to expand comment" : "Komentář rozbalíte kliknutím",
    "Create card" : "Vytvořit kartu",
    "Create a new card" : "Vytvořit novou kartu",
    "Card title" : "Název karty",
    "Creating the new card …" : "Vytváření nové karty …",
    "Card \"{card}\" was added to \"{board}\"" : "Karta „{card}“ byla přidána na „{board}“",
    "Open card" : "Otevřít kartu",
    "Close" : "Zavřít",
    "No upcoming cards" : "Žádné nadcházející karty",
    "upcoming cards today" : "nadcházející karty na dnešek",
    "upcoming cards tomorrow" : "nadcházející karty na zítřek",
    "upcoming cards" : "nadcházející karty",
    "New card" : "Nová karta",
    "Link to a board" : "Propojit s tabulí",
    "Link to a card" : "Propojit s kartou",
    "Create a card" : "Vytvořit kartu",
    "Message from {author} in {conversationName}" : "Zpráva od {author} v {conversationName}",
    "Something went wrong" : "Něco se pokazilo",
    "Failed to upload {name}" : "Nepodařilo se nahrát {name}",
    "Maximum file size of {size} exceeded" : "Překročena nejvyšší umožněná velikost souboru {size}",
    "Assigned users" : "Přiřazení uživatelé",
    "Due date" : "Datum termínu",
    "Created" : "Vytvořeno",
    "Modified" : "Změněno",
    "Error creating the share" : "Chyba při vytváření sdílení",
    "Share with a Deck card" : "Sdílet s kartou aplikace Deck",
    "Share {file} with a Deck card" : "Sdílet {file} s kartou aplikace Deck",
    "Share" : "Sdílet",
    "The card \"%s\" on \"%s\" has reached its due date." : "U karty „%s“ z tabule „%s“ nastalo plánované datum dokončení.",
    "Share board with a user, group or team …" : "Sdílet tabuli s uživatelem, skupinou nebo týmem …",
    "Searching for users, groups and teams …" : "Hledání v uživatelích, skupinách a týmech …",
    "No participants found" : "Nenalezeni žádní účastníci",
    "Save" : "Uložit",
    "seconds ago" : "před několika sekundami",
    "Today" : "Dnes",
    "Tomorrow" : "Zítra",
    "No due" : "Žádný termín"
},
"nplurals=4; plural=(n == 1 && n % 1 == 0) ? 0 : (n >= 2 && n <= 4 && n % 1 == 0) ? 1: (n % 1 != 0 ) ? 2 : 3;");
