OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Umetengeneza bodi mpya {board}",
    "{user} has created a new board {board}" : "{user} ametengeneza bodi mpya {board}",
    "You have deleted the board {board}" : "Umefuta bodi {board}",
    "{user} has deleted the board {board}" : "{user} amefuta bodi {board}",
    "You have restored the board {board}" : "Umefifadhi bodi upya {board}",
    "{user} has restored the board {board}" : "{user} amehifadhi bodi upya {board}",
    "You have shared the board {board} with {acl}" : "Umeshirikisha bodi{board} kwa {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} ameshirikisha bodi {board} kwa {acl}",
    "You have removed {acl} from the board {board}" : "Umeondoa {acl} kutoka katika bodi {board}",
    "{user} has removed {acl} from the board {board}" : "{user}ameondoa {acl}kutoka katika bodi{board}",
    "You have renamed the board {before} to {board}" : "Umeita jina jipya bodi {before} kwa {board}",
    "{user} has renamed the board {before} to {board}" : "{user} ameita jina jipya bodi {before} kwa {board}",
    "You have archived the board {board}" : "Umefikia malengo ya bodi {board}",
    "{user} has archived the board {before}" : "{user} amefikia malengo ya bodi {before}",
    "You have unarchived the board {board}" : "Hujafikia malengo ya bodi {board}",
    "{user} has unarchived the board {before}" : "{user}hajafikia malengo ya bodi {before}",
    "You have created a new list {stack} on board {board}" : "Umetengeneza orodha mpya {stack} katika bodi {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} ametengeneza orodha mpya {stack} katika bodi {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Umeiita orodha jina jipya {before} kwa {stack} katika bodi {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user}ameiita orodha jina jipya {before}kwa {stack} katika bodi {board}",
    "You have deleted list {stack} on board {board}" : "Umefuta orodha {stack} katika bodi {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} amefuta orodha {stack} katika bodi {board}",
    "You have created card {card} in list {stack} on board {board}" : "Umetengeneza kadi {card} katka orodha {stack} juu ya bodi {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} ametengeneza kaadi {card} katika orodha {stack} juu ya bodi {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Umefuta kadi {card}katika orodha {stack} kwenye bodi {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} amefuta kadi {card} katika orodha {stack} juu ya bodi {board}",
    "You have renamed the card {before} to {card}" : "Umeiita kadi jina jipya {before}kwa {card} ",
    "{user} has renamed the card {before} to {card}" : " {user}ameiita kadi jina jipya {before} hadi {card} ",
    "You have added a description to card {card} in list {stack} on board {board}" : "Umeongeza maelezo katika kadi {card}katika orodha {stack}juu ya bodi {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} ameongeza maelezo katika kadi {card}katika orodha {stack} juu ya bodi {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Umesasisha maelezo ya kadi{card} katika orodha {stack}juu ya bodi {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} amesasisha maelezo ya kadi{card} katika orodha {stack}juu ya bodi {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Umefikia malengo ya kadi {card} katika orodha {stack}juu ya bodi{board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user}amefikia malengo ya kadi {card}katika orodha{stack}juu ya bodi{board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Una kadi ambayo haijafikia malengo {card} katika orodha {stack} juu ya bodi {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user}ana kadi ambayo haijafikia malengo {card} katika orodha{stack}juu ya bodi {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Una kadi iliyotiwa alama {card}kama orodha iliyofanyika {stack}juu ya bodi {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} ana kadi iliyotiwa alama{card} kama orodha iliyofanyika {stack}juu ya bodi {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Umeitia alama kadi {card}kama orodha ambayo haikufanyika{stack} juu ya bodi {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} ameitia alama kadi{card} kama orodha ambayo haikufanyika{stack} juu ua bodi {board}",
    "You have removed the due date of card {card}" : "Umeondoa tarehe ya mwisho ya kadi{card}",
    "{user} has removed the due date of card {card}" : "{user}ameondoa tarehe ya mwisho ya kadi {card}",
    "You have set the due date of card {card} to {after}" : "Umepangilia tarehe ya mwisho ya kadi {card}hadi{after}",
    "{user} has set the due date of card {card} to {after}" : "{user} amepangilia tarehe ya mwisho ya kadi{card} hadi {after}",
    "You have updated the due date of card {card} to {after}" : "Umesasisha tarehe ya mwisho ya kadi {card} hadi {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} amesasisha tarehe ya mwisho ya kadi{card} hadi {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Umeongeza lebo{label}kwenye kadi{card}katika orodha{stack} juu ya bodi {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user}ameongeza lebo{label} kwenye kadi{card} katika orodha {stack}juu ya bodi {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Umeondoa lebo {label}kutoka katika kadi{card} katika orodha {stack}juu ya bodi {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user}ameondoa lebo{label}kutoka katika kadi{card}katika orodha{stack}juu ya bodi{board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Umekabidhi {assigneduser}kwenda katika kadi {card}juu ya bodi{board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} amekabidhi {assigneduser}kwenye kadi{card} juu ya bodi {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Umeondoa ukabithi {assigneduser}kutoka katika kadi{card}juu ya bodi {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user}ameondoa ukabidhi wa {assigneduser}kutoka katika kadi {card}juu ya bodi {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Umeondoa kadi{card}kutoka katika orodha {stackBefore}mpaka{stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user}ameondoa kadi {card} kutoka katika orodha {stackBefore}hadi {stack}",
    "You have added the attachment {attachment} to card {card}" : "Umeongeza kiambatanisho{attachment} katika kadi {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} ameongeza kiambatanisho {attachment} katika kadi {card}",
    "You have updated the attachment {attachment} on card {card}" : "Umesasisha kiambatanisho {attachment} katika kadi {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user}amesasisha kiambatanisho {attachment}katika kadi {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Umefuta kiambatanisho {attachment}kutoka kadi {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} amefuta kiambatanisho {attachment}kutoka kadi {card}",
    "You have restored the attachment {attachment} to card {card}" : "Umehifadhi kiambatanisho {attachment} katika kadi {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user}amehifadhi kiambatanisho {attachment} katika kadi {card}",
    "You have commented on card {card}" : "Umetoa maoni kwa kadi{card}",
    "{user} has commented on card {card}" : "{user}ametoa maoni kwa kadi {card}",
    "Deck" : "Deki",
    "Changes in the <strong>Deck app</strong>" : "Mabadiliko katika 1 app ya deki 1",
    "A <strong>board, list or card</strong> was changed" : "1 bodi, orodha au kadi 1 ilibadilishwa",
    "A <strong>comment</strong> was created on a card" : "1 maoni 1 yalitengenezwa katika kadi",
    "A <strong>card description</strong> has been changed" : "1 maelezo ya kadi 1 yamebadilika",
    "The file was uploaded" : "Faili lilipakiwa",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Faili lililopakiwa linazidi kiwango cha juu cha ukubwa wa faili linalielekea katika php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Faili iliyopakiliwa imezidi kiwango cha ukubwa wa faili iliyoelekezwa maalum katika fomu ya HTML",
    "The file was only partially uploaded" : "Faili lilipakiwa kwa sehemu ndogo tu",
    "No file was uploaded" : "Hakuna faili lililopakiwa",
    "Missing a temporary folder" : "Imekosa faili la muda",
    "Could not write file to disk" : "Haikuweza kuandika faili kwenye disk",
    "A PHP extension stopped the file upload" : "Uongezaji wa PHP umesimamisha upakiaji wa faili",
    "No file uploaded or file size exceeds maximum of %s" : "Hakuna faili lililopakiwa au faili lilizidi kiwango cha juu cha %s",
    "Invalid file type. Only JSON files are allowed." : "Aina ya faili si halisi. Mafaili ya JSON tu yanaruhusiwa",
    "Invalid JSON data" : "Data za JSON si halisi",
    "Failed to import board" : "Imeshindwa kuagiza bodi",
    "Cards due today" : "Kadi zitalipwa leo",
    "Cards due tomorrow" : "Kadi zitalipwa kesho",
    "Upcoming cards" : "Kadi zinazofuata",
    "Load more" : "Pakia zaidi",
    "Welcome to Nextcloud Deck!" : "Karibu katika deki ya Nextcloud",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Kadi \"%s\" katika \"%s\" imekabithiwa kwako na %s",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} amekabithi kadi {deck-card}juu ya {deck-board} kwako",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Kadi \"%1$s\" kwenye \"%2$s\" imefikia tarehe yake ya kukamilisha.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Kadi {deck-card}katika {deck-board}imefikia tarehe yake ya mwisho",
    "%s has mentioned you in a comment on \"%s\"." : "%s amekutaja kwenye maoni katika \"%s\"",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} amekutaja katika maoni katika {deck-card}",
    "The board \"%s\" has been shared with you by %s." : "Bodi \"%s imekushirikisha %s",
    "{user} has shared {deck-board} with you." : "{user} ameshirikisha {deck-board} kwako",
    "Deck board" : "Bodi ya deck",
    "Owned by %1$s" : "Inamilikiwa na %1$s",
    "Deck boards, cards and comments" : "Bodi za deki, kadi na maoni",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Kutoka %1$skatika%2$s/%3$s inayomilikiwa na %4$s",
    "Create a new deck card" : "Kadi mpya ya deki imetengenezwa",
    "Card comments" : "Maoni ya kadi",
    "%s on %s" : "%sjuu ya %s",
    "Deck boards and cards" : "Bodi za deki na kadi",
    "No data was provided to create an attachment." : "Hakuna data iliyotolewa kutengeneza kiambatanisho",
    "Finished" : "Imemalizika",
    "To review" : "Kuangalia",
    "Action needed" : "Matendo yanahitajika",
    "Later" : "Baadaye",
    "copy" : "Nakili",
    "Read more inside" : "Soma ndani zaidi",
    "Custom lists - click to rename!" : "Orodha za wateja-bonyeza kuandika jina upya",
    "To Do" : "Kufanya",
    "In Progress" : "Katika mwendelezo",
    "Done" : "Imefanyika",
    "1. Open to learn more about boards and cards" : "1. Fungua kujifunza zaidi kuhusu bodi na kadi",
    "2. Drag cards left and right, up and down" : "2. Buruza kadi kushoto na kulia, juu na chini",
    "3. Apply rich formatting and link content" : "3. Omba uumbizaji kamilifu na kiungo ",
    "4. Share, comment and collaborate!" : "4. Shirikisha, toa maoni na shirikiana",
    "Create your first card!" : "Tengeneza kadi yako ya kwanza",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Maoni haya yana vibambo zaidi ya %s\nVimeongezwa kama kiambatanisho kwenye kadi na jina%s\nInapatikana katika URL%s",
    "Attachments" : "Viambatanisho",
    "File" : "Faili",
    "date" : "tarehe",
    "Card not found" : "Kadi haipatikani",
    "Path is already shared with this card" : "Njia tayari imeshirikishwa na kadi hii",
    "Invalid date, date format must be YYYY-MM-DD" : "Tarehe batili, umbizo la tarehe lazima liwe YYYY-MM-DD",
    "Personal planning and team project organization" : "Mpango wa kibinafsi na mradi wa timu ya shirika",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck ni chombo cha mtindo wa shirika cha kanban kilicholenga katika mpango binafsi na mradi wa shirika kwa timu zilizounganishwa na Nextcloud.\n\n\n-📥Ongeza kazi zako kwenye kadi na zipange katika mpangilio\n-📄Andika chini nukuu za nyongeza katika alama ya chini\n-🔖 Wekw lebo kwa shirika bora zaidi\n- 👥 Shirikisha timu yako, marafiki au familia\n- 📎 Ambatanisha faili na uzipachike katika maelezo yako ya alama\n- 💬 Jadiliana na timu yako kwa kutumia maoni\n- ⚡  Fuatilia mabadiliko katika mtiririko wa shughuli\n- 🚀 Panga mradi wako",
    "Add board" : "Ongeza bodi",
    "Card details" : "Maeleza ya kadi",
    "Select the board to link to a project" : "Chagua bodi ya kuunga kwenye mradi",
    "Search by board title" : "Chagua bodi kwa kichwa cha habari",
    "Select board" : "Chagua bodi",
    "Move/copy card" : "Ondoa/nakili kadi",
    "Select a board" : "Chagua bodi",
    "No lists available" : "Hakuna orodha inayopatikana",
    "Select a list" : "Chagua orodha",
    "Move card" : "Ondoa kadi",
    "Copy card" : "Nakili kadi",
    "Select the card to link to a project" : "Chagua kadi kuunganisha na mradi",
    "Link to card" : "Unga kwenye kadi",
    "Select a card" : "Chagua kadi",
    "Cancel" : "Ghairi",
    "This board is read only" : "Bodi hii inasomwa tu",
    "Drop your files to upload" : "Dondosha mafaili yako kupakia",
    "File already exists" : "Faili lipo tayari",
    "A file with the name {filename} already exists." : "Faili lenye jina {filename}limetokea tayari",
    "Do you want to overwrite it?" : "Unahitaji kulibatilisha?",
    "Overwrite file" : "Batilisha faili",
    "Keep existing file" : "Weka faili liendelee kuwepo",
    "Add card" : "Ongeza kadi",
    "Archived cards" : "Kadi iliyohifadhiwa",
    "Add list" : "Ongeza orodha",
    "List name" : "Orodhesha majina",
    "Clear search" : "Futa utafutaji",
    "Active filters" : "Vichujio vinavyotumika",
    "Apply filter" : "Tumia kichujio",
    "Filter by tag" : "Chuja kwa lebo ",
    "Filter by assigned user" : "Chuja kulingana na mtumiaji aleyekabidhiwa",
    "Unassigned" : "Hakukabidhiwa",
    "Filter by status" : "Chuja kwa wadhifa",
    "Open and completed" : "Iliyofunguliwa na kukamilika",
    "Open" : "Fungua",
    "Completed" : "Kamilika",
    "Filter by due date" : "Chuja kwa tarehe ya mwisho",
    "Overdue" : "Imepitwa na wakati",
    "Next 24 hours" : "Masaa 24 yajayo",
    "Next 7 days" : "Siku 7 zijazo",
    "Next 30 days" : "Siku 30 zijazo",
    "No due date" : "Hakuna tarehe ya mwisho",
    "Clear filter" : "Futa kichujio",
    "View Modes" : "Njia za kutazama",
    "Toggle View Modes" : "Geuza njia za kutazama",
    "Hide archived cards" : "Ficha kadi zilizohifadhiwa",
    "Show archived cards" : "Onesha kadi zilizohifadhiwa",
    "Toggle compact mode" : "Geuza hali ya makubaliano",
    "Hide card cover images" : "Ficha picha ya jalada la kadi",
    "Show card cover images" : "Onesha picha ya jalada la kadi",
    "Open details" : "Fungua maelezo",
    "Details" : "Maelezo ya kina",
    "Deck settings" : "Mipangilio ya deck",
    "General" : "Kuu",
    "Use bigger card view" : "Tumia mwonekano mkubwa wa kadi",
    "Appearance" : "Mwonekano",
    "Show card ID badge" : "Onesha beji ya kitambulisho cha kadi",
    "Show boards in calendar/tasks" : "Onesha bodi katika kalenda/shughuli",
    "Admin settings" : "Mipangilio ya msimamizi",
    "Limit board creation to some groups" : "Zuia matengenezo ya bodi kwenye makundi kadhaa",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Watumiaji nje ya makundi hayo hawataweza kutengeneza bodi zao wenyewe, lakini bado wataweza kufanya kazi juu ya bodi ambazo zimeshirikishwa na wao.",
    "Board actions" : "Matendo ya bodi",
    "Scroll sideways" : "Tembeza pembeni",
    "Navigate between cards" : "Nenda kati ya kadi",
    "Close card details" : "Funga maelezo ya kadi",
    "Search" : "Tafuta",
    "Show card filters" : "Onesha vichungi vya kadi",
    "Clear card filters" : "Futa vichungi vya kadi",
    "Show those shortcuts" : "Onesha mikato hiyo",
    "Card actions" : "Matendo ya kadi",
    "Open card details" : "Fungua maelezo ya kadi",
    "Edit the card title" : "Hariri kichwa cha habari cha kadi",
    "Assign yourself to the current card" : "Jisajili kwenye kadi ya sasa",
    "Archive/unarchive the current card" : "Hifadhi/ondoa kadi ya sasa kwenye kumbukumbu",
    "Mark card as completed/not completed" : "Weka alama kadi kama iliyokamilika/haikukamilika",
    "Open card menu" : "Fungua menyu ya kadi",
    "Currently present people" : "Watu wa sasa hivi",
    "Loading board" : "Bodi inapakia",
    "Board not found" : "Bodi haipatikani",
    "Create a new list to add cards to this board" : "Tengeneza orodha mpya kuongeza kadi kwenye bodi hii",
    "Sharing" : "inashirikisha",
    "Tags" : "Maoni",
    "Deleted items" : "Vifungu vilivyofutwa",
    "Activity" : "Shughuli",
    "Deleted lists" : "Orodha zilizofutwa",
    "Undo" : "Tendua",
    "Deleted cards" : "Kadi zilizifutwa",
    "Hours" : "Masaa",
    "Days" : "Siku",
    "Weeks" : "Wiki",
    "Months" : "Miezi",
    "No cards yet" : "Bado hakuna kadi",
    "Failed to create share with {displayName}" : "Imeshindwa kutengeneza ushiriki na {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Una uhakika unahutaji kuhamisha bodi {title} kwenda {user}",
    "Transfer the board." : "Hamisha bodi",
    "Transfer" : "Hamisha",
    "The board has been transferred to {user}" : "Bodi imehamishwa kwenda {user}",
    "Failed to transfer the board to {user}" : "Imeshindwa kuhamisha bodi kwenda {user}",
    "Board owner" : "Mmiliki wa bodi",
    "(Group)" : "(Kundi)",
    "(Team)" : "(Timu)",
    "(remote)" : "(mbali)",
    "Can edit" : "Inaweza kuhariri",
    "Can share" : "Inaweza kushiikisha",
    "Can manage" : "Inaweza kusimamia",
    "Owner" : "Mmiliki",
    "Delete" : "Futa",
    "List deleted" : "Orodha imefutwa",
    "Edit list title" : "Hariri kichwa cha orodha",
    "Archive all cards" : "Hifadhi kadi zote",
    "Unarchive all cards" : "Usihifadhi kadi zote",
    "Delete list" : "Futa orodha",
    "Archive all cards in this list" : "Hifadhi kadi zote katika orodha hii",
    "Unarchive all cards in this list" : "Usihifadhi kadi zote katika orodha hii",
    "Add a new card" : "Ongeza kadi mpya",
    "Card name" : "Jina la kadi",
    "title and color value must be provided" : "Kichwa cha habari na thamani ya rangi lazima igawiwe",
    "Edit" : "Hariri",
    "Add a new tag" : "Ongeza maoni mapya",
    "Board name" : "Jina la bodi",
    "Members" : "Wanachama",
    "Assign to users/groups/team" : "Kabidhi kwa watumiaji/makundi/timu",
    "Assign a user to this card…" : "Mkabithi mtumiaji kwa kadi hii",
    "Select a user to assign to this card…" : "Chagua mtumiaji wa kumkabithi kwa kadi hii",
    "File to share" : "Faili la kushirikisha",
    "Invalid path selected" : "Njia iliyochaguliwa si halali",
    "Upload new files" : "Pakia mafaili mapya",
    "Share from Files" : "Shirikisha kutoka katika mafaili",
    "Pending share" : "Ushirikishaji unasubiri",
    "Add this attachment" : "Ongeza kiambatanisho hiki",
    "Show in Files" : "Onesha katika mafaili",
    "Download" : "Pakua",
    "Remove attachment" : "Ondoa kiambatanisho",
    "Delete Attachment" : "Futa kiambatanisho",
    "Restore Attachment" : "Hifadhi kiambatanisho upya",
    "The title cannot be empty." : "Kichwa cha habari hakiwezi kuwa tupu",
    "Cannot close unsaved card!" : "Haiwezi kufunga kadi ambayo haijahifadhiwa!",
    "Open in sidebar view" : "Fungua katika mwonekano wa utepe",
    "Open in bigger view" : "Fungua katika mwonekeno mkubwa",
    "Comments" : "Maoni",
    "Failed to load comments" : "Imeshindwa kupakia maoni",
    "No comments yet. Begin the discussion!" : "Hakuna maoni bado. Anza mjadala",
    "The comment cannot be empty." : "Maoni hayawezi kuwa tupu",
    "The comment cannot be longer than 1000 characters." : "Maoni hayawezi kuwa marefu zaidi ya vibambo 1000",
    "Submit" : "Wasilisha",
    "Created:" : "Imetengenezwa:",
    "In reply to" : "Kwa kujibu",
    "Cancel reply" : "Ghairi jibu",
    "Reply" : "Jibu",
    "Update" : "Sasisha",
    "Mark as done" : "Weka alama kama iliyotendeka",
    "Mark as not done" : "Weka alama kama haijatendeka",
    "Write a description …" : "Andika maelezo",
    "Could not save description" : "Haikuweza kuhifadhi maelezo",
    "Description" : "Maelezo",
    "(Unsaved)" : "(Isiyihifadhiwa)",
    "(Saving…)" : "(Uhifadhi....)",
    "Formatting help" : "Uasidizi wa uambizaji",
    "Edit description" : "Hariri maelezo",
    "View description" : "Tazama maelezo",
    "Add Attachment" : "Ongeza kiambatanisho",
    "Choose attachment" : "Chagua kiambatanisho",
    "Select Date" : "Chagua tarehe",
    "Later today – {timeLocale}" : "Baadaye leo-{timeLocale}",
    "Set due date for later today" : "Pangilia tarehe ya mwisho baadaye leo",
    "Tomorrow – {timeLocale}" : "Kesho-{timeLocale}",
    "Set due date for tomorrow" : "Pangilia tarehe ya mwisho kwa kesho",
    "This weekend – {timeLocale}" : "Wikendi-{timeLocale}",
    "Set due date for this weekend" : "Pangilia tarehe ya mwisho kwa wiki hii",
    "Next week – {timeLocale}" : "Wiki ijayo-{timeLocale}",
    "Set due date for next week" : "Pangilia tarehe ya mwisho kwa wiki ijayo",
    "Assign a due date to this card…" : "Kabidhi tarehe ya mwisho katika kadi hii",
    "Set a due date" : "Pangilia tarehe ya mwisho",
    "Add due date" : "Ongeza tarehe ya mwisho",
    "Choose a date" : "Chagua tarehe",
    "Remove due date" : "Ondoa tarehe ya mwisho",
    "Due at:" : "Mwisho katika",
    "Not done" : "Haijatendeka",
    "Unarchive card" : "Kadi ambayo haijahifadhiwa",
    "Archive card" : "Hifadhi kadi",
    "Assign a tag to this card…" : "Kabidhi maoni katika kadi hii",
    "Select or create a tag…" : "Chagua au tengeneza maoni",
    "Create a new tag:" : "Tengeneza oni jipya",
    "(group)" : "(kundi)",
    "{count} comments, {unread} unread" : "{count}maoni, {unread}hayajasomwa",
    "Todo items" : "Vitu vya kufanya",
    "Edit card title" : "Hariri kichwa cha habari cha kadi",
    "Open link" : "Fungua kiungio",
    "Card deleted" : "Kadi imefutwa",
    "Edit title" : "Hariri kichwa cha habari",
    "Assign to me" : "Kabidhi kwangu",
    "Unassign myself" : "Sijajikabidhi mwenyewe",
    "Delete card" : "Futa kadi",
    "All boards" : "Bodi zote",
    "Archived boards" : "Bodi zilizohifadhiwa",
    "Shared with you" : "Imeshirikiwa na wewe",
    "Cancel edit" : "Sistisha uhariri",
    "Save board" : "Hifadhi bodi",
    "Board {0} deleted" : "Bodi {0}futa",
    "All cards" : "Kadi zote",
    "Only assigned cards" : "Kadi zilizosajiliwa tu",
    "No reminder" : "Hakuna ukumbushaji",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Una uhakika unataka kufuta bodi {title}? Hii itafuta data zote za bodi ikijumuisha kadi zilizohifadhiwa",
    "Delete the board?" : "Futa bodi?",
    "Are you sure you want to leave the board {title}?" : "Je, una uhakika unataka kuondoka kwenye ubao wa {title}?",
    "Leave the board?" : "Ungependa kuondoka kwenye ubao?",
    "Leave" : "Ondoka",
    "Failed to leave the board" : "Imeshindwa kuondoka kwenye ubao",
    "Exporting board..." : "Safirisha bodi...",
    "An error occurred" : "Hitilafu imetokea",
    "Board details" : "Maelezo ya bodi",
    "Edit board" : "Hariri bodi",
    "Clone board" : "Zalisha bodi",
    "Unarchive board" : "Bodi isiyohifadhiwa",
    "Archive board" : "Hifadhi bodi",
    "Export board" : "Safirisha bodi",
    "Turn on due date reminders" : "Washa ukumbushaji wa tarehe kikomo",
    "Turn off due date reminders" : "Zima ukumbushaji wa tarehe kikomo",
    "Due date reminders" : "Ukumbushaji wa tarehe kikomo",
    "Assigned cards" : "Kadi zilizokabidhiwa",
    "No notifications" : "Hakuna taarifa",
    "Delete board" : "Futa bodi",
    "Leave board" : "Ondoka kwenye ubao",
    "Importing board..." : "Inaingiza bodi....",
    "Board imported successfully" : "Bodi imeingizwa kikamilifu",
    "Import board" : "Ingiza bodi",
    "Clone {boardTitle}" : "Zalisha {boardTitle}",
    "Clone cards" : "Zalisha kadi",
    "Clone assignments" : "Zalisha kazi",
    "Clone labels" : "Zalisha lebo",
    "Clone due dates" : "Zalisha tarehe kikomo",
    "Advanced options" : "Chaguzi za juu",
    "Move all cards to the first list" : "Hamishia kadi zote kwenye orodha ya kwanza",
    "Restore archived cards" : "Hifadhi upya kadi zilizohifadhiwa",
    "Clone" : "Zalisha",
    "Export {boardTitle}" : "Agiza {boardTitle}",
    "Export as JSON" : "Safirisha kama JSON",
    "Export as CSV" : "Safirisha kama CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Nukuu: Umbizo la JSON pekee ndilo linalotumika kuleta tena kwenye app ya Deck",
    "Export" : "Safirisha",
    "Loading filtered view" : "Inapakia mwonekano uliochujwa",
    "Search for {searchQuery} in other boards" : "Tafuta {searchQuery} katika vibao vingine",
    "Search for {searchQuery} in all boards" : "Tafuta kwa {searchQuery}katika bodi zote",
    "No results found" : "Hakuna matokeo yaliyopatikana",
    "Deck board {name}\n* Last modified on {lastMod}" : "Bodi ya deki{name}\nMaboresho ya mwisho katika {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "*Imetengenezwa kwenye {created}\nImeboreshwa katika {lastMod}\n{nbAttachments}viambatanisho\n{nbComments}maoni",
    "{nbCards} cards" : "{nbCards}kadi",
    "Due on {date}" : "Ukomo katika {date}",
    "{stack} in {board}" : "{stack} katika {board}",
    "Click to expand description" : "Bonyeza kupanua maelezo",
    "Click to expand comment" : "Bonyeza kupanua maoni",
    "Create card" : "Tengeneza kadi",
    "Create a new card" : "Tengeneza kadi mpya ",
    "Card title" : "Kichwa cha habari cha kadi",
    "Creating the new card …" : "Inatengeneza kadi mpya",
    "Card \"{card}\" was added to \"{board}\"" : "Kadi \"{card}\" iliongezwa katika \"{board}\"",
    "Open card" : "Fungua kadi",
    "Close" : "Funga",
    "No upcoming cards" : "Hakuna kadi zijazo",
    "upcoming cards today" : "Kadi zijazo leo",
    "upcoming cards tomorrow" : "Kadi zijazo kesho",
    "upcoming cards" : "Kadi zijazo",
    "New card" : "Kadi mpya",
    "Link to a board" : "Unga kwenye bodi",
    "Link to a card" : "Unga kwenye kadi",
    "Create a card" : "Tengeneza kadi",
    "Message from {author} in {conversationName}" : "Ujumbe kutoka {author}katika{conversationName}",
    "Something went wrong" : "Kitu fulani kimeenda vibaya",
    "Failed to upload {name}" : "Imeshindwa kupakia {name}",
    "Maximum file size of {size} exceeded" : "Kiwango cha juu cha ukubwa wa faili {size}kimezidi",
    "Assigned users" : "Watumiaji waliokabidhiwa",
    "Due date" : "Tarehe ya ukomo",
    "Created" : "Imetengenezwa",
    "Modified" : "Boresha",
    "Error creating the share" : "Hitilafu katika kutengeneza ushiriki",
    "Share with a Deck card" : "Shirikisha na kadi za deck",
    "Share {file} with a Deck card" : "Shirikisha {file}na kadi za deck",
    "Share" : "Shirikisha",
    "The card \"%s\" on \"%s\" has reached its due date." : "Kadi \"%s\" katika \"%s\" imefikia tarehe yake ya mwisho",
    "Share board with a user, group or team …" : "Shirikisha bodi na watumiaji, kundi au timu",
    "Searching for users, groups and teams …" : "Inatafuta watumiaji, vikundi na timu",
    "No participants found" : "Hakuna mshiriki aliyepatikana",
    "Save" : "Hifadhi",
    "seconds ago" : "sukunde zilizopita",
    "Today" : "Leo",
    "Tomorrow" : "Kesho",
    "No due" : "Hakuna ukomo"
},
"nplurals=2; plural=(n != 1);");
