OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Sa oled koostanud uue tahvli: {board}",
    "{user} has created a new board {board}" : "{user} on koostanud uue tahvli: {board}",
    "You have deleted the board {board}" : "Sa oled kustutanud tahvli: {board}",
    "{user} has deleted the board {board}" : "{user} on kustutanud tahvli: {board}",
    "You have restored the board {board}" : "Sa oled taastanud tahvli: {board}",
    "{user} has restored the board {board}" : "{user} on taastanud tahvli: {board}",
    "You have shared the board {board} with {acl}" : "Sa oled jaganud {board} tahvlit: {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} on jaganud {board} tahvlit: {acl}",
    "You have removed {acl} from the board {board}" : "Sa oled eemaldanud {acl} ligipääsu tahvlilt {board}",
    "{user} has removed {acl} from the board {board}" : "{user} on eemaldanud {acl} ligipääsu tahvlilt {board}",
    "You have renamed the board {before} to {board}" : "Sa oled muutnud tahvli nime: „{before}“ → „{board}“",
    "{user} has renamed the board {before} to {board}" : "{user} on muutnud tahvli nime: „{before}“ → „{board}“",
    "You have archived the board {board}" : "Sa oled {board} tahvli arhiveerinud",
    "{user} has archived the board {before}" : "{user} on {before} tahvli arhiveerinud",
    "You have unarchived the board {board}" : "Sa oled „{board}“ tahvli arhiivist eemaldanud",
    "{user} has unarchived the board {before}" : "{user} on „{before}“ tahvli arhiivist eemaldanud",
    "You have created a new list {stack} on board {board}" : "Sa oled „{board}“ tahvlile lisanud uue loendi „{stack}“",
    "{user} has created a new list {stack} on board {board}" : "{user} on „{board}“ tahvlil koostanud uue loendi „{stack}“ ",
    "You have renamed list {before} to {stack} on board {board}" : "Sa oled muutnud „{board}“ tahvli loendi nime: „{before}“ → „{stack}“",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} on muutnud „{board}“ tahvli loendi nime: „{before}“ → „{stack}“",
    "You have deleted list {stack} on board {board}" : "Sa oled „{board}“ tahvlilt kustutanud „{stack}“ loendi",
    "{user} has deleted list {stack} on board {board}" : "{user}on „{board}“ tahvlilt kustutanud „{stack}“ loendi",
    "You have created card {card} in list {stack} on board {board}" : "Sa oled „{board}“ tahvli „{stack}“ loendisse lisanud „{card}“ kaardi",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} on „{board}“ tahvli „{stack}“ loendisse lisanud „{card}“ kaardi",
    "You have deleted card {card} in list {stack} on board {board}" : "Sa oled „{board}“ tahvli „{stack}“ loendist kustutanud „{card}“ kaardi",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} on „{board}“ tahvli „{stack}“ loendist kustutanud „{card}“ kaardi",
    "You have renamed the card {before} to {card}" : "Sa oled muutnud kaardi nime: „{before}“ → „{card}“",
    "{user} has renamed the card {before} to {card}" : "{user} on muutnud kaardi nime: „{before}“ → „{card}“",
    "You have added a description to card {card} in list {stack} on board {board}" : "Sa oled „{board}“ tahvli „{stack}“ loendi „{card}“ kaardile lisanud kirjelduse",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} on „{board}“ tahvli „{stack}“ loendi „{card}“ kaardile lisanud kirjelduse",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Sa oled „{board}“ tahvli „{stack}“ loendi „{card}“ kaardi kirjeldust uuendanud",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} on „{board}“ tahvli „{stack}“ loendi „{card}“ kaardi kirjeldust uuendanud",
    "You have archived card {card} in list {stack} on board {board}" : "Sa oled „{board}“ tahvli „{stack}“ loendi „{card}“ kaardi arhiveerinud",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} on „{board}“ tahvli „{stack}“ loendi „{card}“ kaardi arhiveerinud",
    "You have unarchived card {card} in list {stack} on board {board}" : "Sa oled „{board}“ tahvli „{stack}“ loendi „{card}“ kaardi arhiveerist eemaldanud",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} on „{board}“ tahvli „{stack}“ loendi „{card}“ kaardi arhiivist eemaldanud",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Sa oled märkinud {board} tahvli {stack} loendi {card} kaardi tehtuks",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} on märkinud {board} tahvli {stack} loendi {card} kaardi tehtuks",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Sa oled märkinud {board} tahvli {stack} loendi {card} kaardi mittetehtuks",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} on märkinud {board} tahvli {stack} loendi {card} kaardi mittetehtuks",
    "You have removed the due date of card {card}" : "Sina oled eemaldanud „{card}“ kaardi tähtaja",
    "{user} has removed the due date of card {card}" : "{user} on eemaldanud „{card}“ kaardi tähtaja",
    "You have set the due date of card {card} to {after}" : "Sina oled määranud „{card}“ kaardi tähtajaks {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} on määranud „{card}“ kaardi tähtajaks {after}",
    "You have updated the due date of card {card} to {after}" : "Sina oled muutnud „{card}“ kaardi tähtajaks {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} on muutnud „{card}“ kaardi tähtajaks {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Sa oled „{board}“ tahvli „{stack}“ loendi „{card}“ kaardile lisanud „{label}“ sildi",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} on „{board}“ tahvli „{stack}“ loendi „{card}“ kaardile lisanud „{label}“ sildi",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Sina oled „{board}“ tahvli „{stack}“ loendi „{card}“ kaardilt eemaldanud „{label}“ sildi",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} on „{board}“ tahvli „{stack}“ loendi „{card}“ kaardilt eemaldanud „{label}“ sildi",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Sa oled määranud {board} tahvli {card} kaardi täitjaks {assigneduser}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} on määranud {board} tahvli {card} kaardi täitjaks {assigneduser}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Sa oled {board} tahvli {card} kaardi täitja rollist eemaldanud kasutaja {assigneduser}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} on  {board} tahvli {card} kaardi täitja rollist eemaldanud kasutaja {assigneduser}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Sina oled tõstnud {card} kaardi ühest loendist teise: {stackBefore} → {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} on tõstnud {card} kaardi ühest loendist teise: {stackBefore} → {stack}",
    "You have added the attachment {attachment} to card {card}" : "Sa oled lisanud „{card}“ kaardile „{attachment}“ manuse",
    "{user} has added the attachment {attachment} to card {card}" : "{user} on lisanud „{card}“ kaardile „{attachment}“ manuse",
    "You have updated the attachment {attachment} on card {card}" : "Sina oled uuendanud „{card}“ kaardil „{attachment}“ manust",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} on uuendanud „{card}“ kaardi „{attachment}“ manust",
    "You have deleted the attachment {attachment} from card {card}" : "Sina oled kustutanud „{card}“ kaardilt „{attachment}“ manuse",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} on kustutanud „{card}“ kaardilt „{attachment}“ manuse",
    "You have restored the attachment {attachment} to card {card}" : "Sa oled taastanud „{card}“ kaardile „{attachment}“ manuse",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} on taastanud „{card}“ kaardi „{attachment}“ manuse",
    "You have commented on card {card}" : "Sina oled kommenteerinud „{card}“ kaarti",
    "{user} has commented on card {card}" : "{user} on kommenteerinud „{card}“ kaarti",
    "Deck" : "Kanban",
    "Changes in the <strong>Deck app</strong>" : "Muudatused <strong>Kanbani rakenduses</strong>",
    "A <strong>board, list or card</strong> was changed" : "<strong>Tahvel, loend või kaart</strong> on muutunud",
    "A <strong>comment</strong> was created on a card" : "Kaardile on lisatud<strong>kommentaar</strong>",
    "A <strong>card description</strong> has been changed" : "<strong>Kaardi kirjeldus</strong> on muutunud",
    "The file was uploaded" : "Fail laaditi üles",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Üleslaetud fail on suurem, kui php.ini failis määratud upload_max_filesize",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Üleslaetud fail on suurem, kui MAX_FILE_SIZE atribuut, mis seadistati HTML vormis",
    "The file was only partially uploaded" : "Fail laaditi üles ainult osaliselt",
    "No file was uploaded" : "Ühtegi faili ei laaditud üles",
    "Missing a temporary folder" : "Ajutine kaust on puudu",
    "Could not write file to disk" : "Faili kirjutamine kettale ei õnnestunud",
    "A PHP extension stopped the file upload" : "PHP laiendus peatas faili üleslaadimise",
    "No file uploaded or file size exceeds maximum of %s" : "Faili ei laaditud üles või selle suurus ületab maksimaalse %s",
    "Invalid file type. Only JSON files are allowed." : "Vigane failitüüp. Vaid JSON-failid on lubatud.",
    "Invalid JSON data" : "Vigased andmed JSON-is",
    "Failed to import board" : "Tahvli importimine ei õnnestunud",
    "Cards due today" : "Kaardid, mille tähtaeg on täna",
    "Cards due tomorrow" : "Kaardid, mille tähtaeg on homme",
    "Upcoming cards" : "Tulevased kaardid",
    "Load more" : "Laadi veel",
    "Welcome to Nextcloud Deck!" : "Tere tulemast kasutama kanbani rakendust Nextcloudi Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "„%s“ kaart „%s“ tahvlil on määratud sulle täitmiseks %s poolt.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} on andnud „{deck-board}“ tahvli „{deck-card}“ kaardi sulle täitmiseks",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Tähtaeg on käes: „%1$s“ kaart, „%2$s“ tahvel",
    "The card {deck-card} on {deck-board} has reached its due date." : "Tähtaeg on käes: „{deck-card}“ kaart, „{deck-board}“ tahvel",
    "%s has mentioned you in a comment on \"%s\"." : "%s mainis sind „%s“ kommentaaris.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} on sind maininud „{deck-card}“ tahvli kommentaaris.",
    "The board \"%s\" has been shared with you by %s." : "%s on sinuga jaganud „%s“ tahvlit.",
    "{user} has shared {deck-board} with you." : "{user} on „{deck-board}“ tahvlit sinuga jaganud",
    "The remote board %1$s has been shared with you by %2$s" : "%2$s on sinuga jaganud „%1$s“ tahvlit kaugseadmes.",
    "Deck board" : "Kanbani tahvel",
    "Owned by %1$s" : "Omanik: %1$s",
    "Deck boards, cards and comments" : "Kanbani tahvlid, kaardid ja kommentaarid",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Kasutajalt „%1$s“ „%2$s“ tahvli „%3$s“ loendis, mille omanik on „%4$s“",
    "Create a new deck card" : "Koosta uus kanbani kaart",
    "Card comments" : "Kaardi kommentaarid",
    "%s on %s" : "%s %s-l",
    "Deck boards and cards" : "Kanbani tahvlid ja kaardid",
    "No data was provided to create an attachment." : "Polnud andmeid, millest luua manust.",
    "Finished" : "Lõpetatud",
    "To review" : "Ülevaatamiseks",
    "Action needed" : "Tegevus on vajalik",
    "Later" : "Hiljem",
    "copy" : "koopia",
    "Read more inside" : "Loe lisateavet järgnevast",
    "Custom lists - click to rename!" : "Kohandatud loendid - klõpsi nime muutmiseks!",
    "To Do" : "Tegemiseks",
    "In Progress" : "Töös",
    "Done" : "Valmis",
    "1. Open to learn more about boards and cards" : "1. Ava lugemaks lisateavet tahvlite ja kaartide kohta",
    "2. Drag cards left and right, up and down" : "2. Lohista kaarte vasakule ja paremale ning üles ja alla",
    "3. Apply rich formatting and link content" : "3. Kasuta teksti vormindamise ja linkide lisamise võimalust",
    "4. Share, comment and collaborate!" : "4. Jaga, kommenteeri ning osale ühistegevuses!",
    "Create your first card!" : "Koosta oma esimene kaart!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Sellel kommentaari pikkus on üle %s tähemärgi.\nTa on kaardile lisatud „%s“ nimega manusena.\nVõrguaadress: %s.",
    "Attachments" : "Manused",
    "File" : "Fail",
    "date" : "kuupäev",
    "Card not found" : "Kaarti ei leidu",
    "Path is already shared with this card" : "Asukoht on juba selle kaardiga jagatud",
    "Invalid date, date format must be YYYY-MM-DD" : "Vigane kuupäev, formaat peab olema YYYY-MM-DD",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Kõik tahvlid, mille omanik sa oled, sealhulgas loendid, kaardid, sildid, ülesannete jagamised ja kommentaarid.",
    "Personal planning and team project organization" : "Projektijuhtimise lahendus sulle ja sinu tiimile",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Kanbani rakendus „Deck“ on Nextcloudi lõimitud haldustarvik, mis sobib nii sinu oma tegemiste, kui tiimide ülesannete haldamiseks.\n\n\n- 📥 Lisa oma ülesanded kaartidele ja sea nad õigesse järjestusse\n- 📄 Koosta Markdown-vormingus märkmeid\n- 🔖 Tõhusama halduse nimel kasuta silte\n- 👥 Jaga oma tiimiga, sõpradega või perega\n- 📎 Lisa manuseid ja kasutades Markdown-vormingut lõimi neid teksti\n- 💬 Kasutades kommentaare saad vestelda oma tiimi liikmetega\n- ⚡ Tegevuste voost näed kõiki toimunud muudatusi\n- 🚀 Sinu projekt saab hallatud parimal viisil",
    "Add board" : "Lisa tahvel",
    "Card details" : "Kaardi üksikasjad",
    "Select the board to link to a project" : "Vali projektiga seotav tahvel",
    "Search by board title" : "Otsi tahvli pealkirja alusel",
    "Select board" : "Vali tahvel",
    "Move/copy card" : "Teisalda või kopeeri kaart",
    "Select a board" : "Vali tahvel",
    "No lists available" : "Loendeid pole saadaval",
    "Select a list" : "Vali loend",
    "Move card" : "Teisalda kaart",
    "Copy card" : "Kopeeri kaart",
    "Select the card to link to a project" : "Vali kaart, mida projektis viidata",
    "Link to card" : "Lisa link kaardile",
    "Select a card" : "Vali kaart",
    "Cancel" : "Katkesta",
    "This board is read only" : "See tahvel on vaid loetav",
    "Drop your files to upload" : "Üleslaadimiseks lohista failid siia",
    "File already exists" : "Fail on juba olemas",
    "A file with the name {filename} already exists." : "„{filename}“ nimega fail on juba olemas.",
    "Do you want to overwrite it?" : "Kas sa sooviksid ta üle kirjutada?",
    "Overwrite file" : "Kirjuta fail üle",
    "Keep existing file" : "Säilita olemasolev fail",
    "Add card" : "Lisa kaart",
    "Archived cards" : "Arhiveeritud kaardid",
    "Add list" : "Lisa loend",
    "List name" : "Nimekirja nimi",
    "Clear search" : "Tühjenda otsing",
    "Active filters" : "Aktiivsed filtrid",
    "Apply filter" : "Rakenda filtrit",
    "Filter by tag" : "Filtreeri sildi alusel",
    "Filter by assigned user" : "Filtreeri täitja alusel",
    "Unassigned" : "Täitja on määramata",
    "Filter by status" : "Filtreeri oleku järgi",
    "Open and completed" : "Avatud ja lõpetatud",
    "Open" : "Avatud",
    "Completed" : "Lõpetatud",
    "Filter by due date" : "Filtreeri tähtaja alusel",
    "Overdue" : "Üle tähtaja",
    "Next 24 hours" : "Järgmise 24 tunni jooksul",
    "Next 7 days" : "Järgmise 7 päeva jooksul",
    "Next 30 days" : "Järgmise 30 päeva jooksul",
    "No due date" : "Tähtaega pole",
    "Clear filter" : "Tühjenda filter",
    "View Modes" : "Vaadete liigid",
    "Toggle View Modes" : "Lülita vaadete liike sisse/välja",
    "Kanban view" : "Kanbani vaade",
    "Gantt view" : "Gantti vaade",
    "Hide archived cards" : "Peida arhiveeritud kaardid",
    "Show archived cards" : "Näita arhiveeritud kaarte",
    "Toggle compact mode" : "Lülita kompaktne vaade sisse/välja",
    "Hide card cover images" : "Peida kaardi kaanepildid",
    "Show card cover images" : "Näita kaardi kaanepilte",
    "Open details" : "Ava üksikasjad",
    "Details" : "Üksikasjad",
    "Deck settings" : "Kanbani rakenduse seadistused",
    "General" : "Üldine",
    "Use bigger card view" : "Kasuta suuremat kaardivaadet",
    "Appearance" : "Välimus",
    "Show card ID badge" : "Näita silti kaardi tunnusega",
    "Show boards in calendar/tasks" : "Näita tahvleid kalendrite ja ülesannete vaates",
    "Admin settings" : "Peakasutaja seadistused",
    "Limit board creation to some groups" : "Piira tahvlite loomise õigus mõnede gruppidega",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Kasutajad, kes pole nende gruppide liikmed, ei saa oma tahvleid luua, kuid saavad töötada tahvlitega, mida neile on jagatud.",
    "Enable federation" : "Lülita födereerimine (liitpilve kasutamine) sisse",
    "Board actions" : "Tegevused tahvliga",
    "Scroll sideways" : "Külitsi kerimine",
    "Navigate between cards" : "Liigu kaartide vahel",
    "Close card details" : "Sulge kaardi üksikasjad",
    "Search" : "Otsi",
    "Show card filters" : "Näita kaartide filtreid",
    "Clear card filters" : "Tühjenda kaartide filtrid",
    "Show those shortcuts" : "Näita neid otseteid",
    "Card actions" : "Tegevused kaardiga",
    "Open card details" : "Ava kaardi üksikasjad",
    "Edit the card title" : "Muuda kaardi pealkirja",
    "Assign yourself to the current card" : "Anna see kaart endale täitmiseks",
    "Archive/unarchive the current card" : "Lisa see kaart arhiivi või eemalda sealt",
    "Mark card as completed/not completed" : "Märgi lõpetatuks/pooleli olevaks",
    "Open card menu" : "Ava kaardi menüü",
    "Currently present people" : "Hetkel kohalolevad inimesed",
    "Supported prefixes: {prefixes}. Wrap phrases in double quotes." : "Toetatud prefiksid: {prefixes}. Otsitavad fraasid pane jutumärkide vahele.",
    "Search cards" : "Otsi kaarte",
    "Loading board" : "Laadin tahvlit",
    "Board not found" : "Tahvlit ei leidu",
    "Create a new list to add cards to this board" : "Kaartide lisamiseks sellele tahvlile koosta uus loend",
    "Sharing" : "Jagamine",
    "Tags" : "Sildid",
    "Deleted items" : "Kustutatud kirjed",
    "Activity" : "Tegevus",
    "Deleted lists" : "Kustutatud loendid",
    "Undo" : "Tühista",
    "Deleted cards" : "Kustutatud kaardid",
    "Hours" : "Tunnid",
    "Days" : "Päevad",
    "Weeks" : "Nädalad",
    "Months" : "Kuud",
    "No cards yet" : "Kaarte veel pole",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Määra oma kaartidele algus- ja lõppkupäevad ning näed neid Gantti diagrammil",
    "Cards without dates ({count})" : "Ilma kuupäevata kaardid ({count})",
    "Failed to create share with {displayName}" : "Jagamine ei õnnestunud kasutajaga {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Kas oled kindel et tahad „{title}“ tahvli omandi üle anda kasutajale „{user}“?",
    "Transfer the board." : "Anna tahvli omand üle.",
    "Transfer" : "Anna omand üle.",
    "The board has been transferred to {user}" : "Tahvli omand on üle antud kasutajale {user}",
    "Failed to transfer the board to {user}" : "Tahvli omandi üleandmine kasutajale {user} ei õnnestunud",
    "Board owner" : "Tahvli omanik",
    "(Group)" : "(Grupp)",
    "(Team)" : "(Tiim)",
    "(remote)" : "(kaugseade)",
    "Can edit" : "Võib muuta",
    "Can share" : "Võib jagada",
    "Can manage" : "Võib hallata",
    "Owner" : "Omanik",
    "Delete" : "Kustuta",
    "List deleted" : "Loend on kustutatud",
    "Edit list title" : "Muuda loendi pealkirja",
    "Archive all cards" : "Arhiveeri kõik kaardid",
    "Unarchive all cards" : "Eemalda kõik kaardid arhiivist",
    "Do not set cards as \"done\"" : "Ära märgi kaarte olekusse „Tehtud“",
    "Set cards as \"done\"" : "Märgi kaardid olekusse „Tehtud“",
    "Delete list" : "Kustuta loend",
    "Archive all cards in this list" : "Arhiveeri kõik selle loendi kaardid",
    "Unarchive all cards in this list" : "Eemalda kõik selle loendi kaardid arhiivist",
    "Add a new card" : "Lisa uus kaart",
    "Card name" : "Kaardi nimi",
    "title and color value must be provided" : "Minimaalselt on vajalik pealkiri ja kaardi värv",
    "Edit" : "Muuda",
    "Add a new tag" : "Lisa uus silt",
    "Search boards" : "Otsi tahvleid",
    "Board name" : "Tahvli nimi",
    "Members" : "Liikmed",
    "Assign to users/groups/team" : "Anna täitmiseks kasutajale/grupile/tiimile",
    "Assign a user to this card…" : "Anna kaart kasutajale täitmiseks…",
    "Select a user to assign to this card…" : "Vali kasutaja, kellele sa tahad selle kaardi ülesandeks anda…",
    "File to share" : "Jagatav fail",
    "Invalid path selected" : "Vigane asukoht on valitud",
    "Upload new files" : "Laadi uued failid üles",
    "Share from Files" : "Jaga failirakendusest",
    "Pending share" : "Ootel jagamine",
    "Add this attachment" : "Lisa see manus",
    "Show in Files" : "Näita failirakenduses",
    "Download" : "Laadi alla",
    "Remove attachment" : "Eemalda manus",
    "Delete Attachment" : "Kustuta manus",
    "Restore Attachment" : "Taasta manus",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Muudetud: {modifiedDate} ⸱ Loodud: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Muudetud: {modifiedDate}\nLoodud: {createdDate}\nLooja: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Muudetud: {modifiedDate}\nLoodud: {createdDate}",
    "The title cannot be empty." : "Pealkiri ei saa olla tühi.",
    "Cannot close unsaved card!" : "Salvestamata kaarti ei saa sulgeda!",
    "Open in sidebar view" : "Ava külgriba vaates",
    "Open in bigger view" : "Ava suuremas vaates",
    "Comments" : "Kommentaarid",
    "Failed to load comments" : "Kommentaaride laadimine ei õnnestunud",
    "No comments yet. Begin the discussion!" : "Kommentaare veel pole. Asu aga keskustelema!",
    "The comment cannot be empty." : "Kommentaar ei saa olla tühi.",
    "The comment cannot be longer than 1000 characters." : "Kommentaar ei või olla pikem, kui 1000 tähemärki",
    "Submit" : "Saada",
    "Created:" : "Loodud:",
    "In reply to" : "Vastuseks:",
    "Cancel reply" : "Katkesta vastamine",
    "Reply" : "Vasta",
    "Update" : "Uuenda",
    "Assign dependent cards" : "Määra sõltuvaid kaarte",
    "Mark as done" : "Märgi tehtuks",
    "Mark as not done" : "Märgi mittetehtuks",
    "Remove dependency" : "Eemalda sõltuvus",
    "Add dependent card" : "Lisa sõltuv kaart",
    "Assign a dependent card…" : "Määra sõltuv kaart…",
    "Select a dependent card…" : "Vali sõltuv kaart…",
    "Write a description …" : "Koosta kirjeldus…",
    "The description has been changed by another user." : "Teine kasutaja on muutnud kirjeldust.",
    "Could not save description" : "Kirjelduse salvestamine ei õnnestunud",
    "Description" : "Kirjeldus",
    "(Unsaved)" : "(Salvestamata)",
    "(Saving…)" : "(Salvestan…)",
    "Formatting help" : "Abiteave vorminduse kohta",
    "Edit description" : "Muuda kirjeldust",
    "View description" : "Vaata kirjeldust",
    "Add Attachment" : "Lisa manus",
    "Choose attachment" : "Vali manus",
    "Select Date" : "Vali kuupäev",
    "Later today – {timeLocale}" : "Täna hiljem – {timeLocale}",
    "Set due date for later today" : "Määra tähtaeg tänaseks hilisemaks ajaks",
    "Tomorrow – {timeLocale}" : "Homme – {timeLocale}",
    "Set due date for tomorrow" : "Määra tähtaeg homseks",
    "This weekend – {timeLocale}" : "Sel nädalavahetusel – {timeLocale}",
    "Set due date for this weekend" : "Määra tähtaeg selleks nädalavahetuseks",
    "Next week – {timeLocale}" : "Järgmisel nädalal – {timeLocale}",
    "Set due date for next week" : "Määra tähtaeg järgmiseks nädalaks",
    "Assign a due date to this card…" : "Määra sellele kaardile tähtaeg…",
    "Set a due date" : "Määra tähtaeg",
    "Add due date" : "Lisa tähtaeg",
    "Choose a date" : "Vali kuupäev",
    "Remove due date" : "Eemalda tähtaeg",
    "Due at:" : "Tähtaeg:",
    "Not done" : "Pole valmis",
    "Unarchive card" : "Eemalda kaart arhiivist",
    "Archive card" : "Arhiveeri kaart",
    "Assign a start date to this card…" : "Määra sellele kaardile alguskuupäev…",
    "Set a start date" : "Määratle alguskuupäev",
    "Add start date" : "Lisa alguskuupäev",
    "Remove start date" : "Eemalda alguskuupäev",
    "Assign a tag to this card…" : "Lisa sellele kaardile silt…",
    "Select or create a tag…" : "Vali või koosta silt…",
    "Create a new tag:" : "Lisa uus silt:",
    "(group)" : "(grupp)",
    "{count} comments, {unread} unread" : "{count} kommentaari, neist {unread} lugemata",
    "Todo items" : "Ülesanded",
    "Edit card title" : "Muuda kaardi pealkirja",
    "Open link" : "Ava link",
    "Card deleted" : "Kaart on kustutatud",
    "Change card color" : "Muuda kaardi värvi",
    "Edit title" : "Muuda pealkirja",
    "Assign to me" : "Anna täitmiseks mulle",
    "Unassign myself" : "Võta endalt see ülesanne ära",
    "Delete card" : "Kustuta kaart",
    "All boards" : "Kõik tahvlid",
    "Archived boards" : "Arhiveeritud tahvlid",
    "Shared with you" : "Sinuga jagatud",
    "Cancel edit" : "Loobu muutmisest",
    "Save board" : "Salvesta tahvel",
    "Board {0} deleted" : "„{0}“ tahvel on kustutatud",
    "All cards" : "Kõik kaardid",
    "Only assigned cards" : "Vaid kaardid, millel on täitja olemas",
    "No reminder" : "Meeldetuletust pole",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Kas oled kindel, et soovid „{title}“ tahvli kustutada. Sellega saavad kustutatud kõik selle tahvli andmed, sealhulgas arhiveeritud kaardid.",
    "Delete the board?" : "Kas kustutad selle tahvli?",
    "Are you sure you want to leave the board {title}?" : "Kas sa oled kindel, et soovid „{title}“ tahvlilt lahkuda?",
    "Leave the board?" : "Kas lahkud tahvlilt?",
    "Leave" : "Lahku",
    "Failed to leave the board" : "Tahvlilt lahkumine ei õnnestunud",
    "Exporting board..." : "Ekspordin tahvlit...",
    "An error occurred" : "Tekkis viga",
    "Board details" : "Tahvli üksikasjad",
    "Edit board" : "Muuda tahvlit",
    "Clone board" : "Tee tahvlist koopia",
    "Unarchive board" : "Eemalda tahvel arhiivist",
    "Archive board" : "Arhiveeri tahvel",
    "Export board" : "Ekspordi tahvel",
    "Turn on due date reminders" : "Kasuta tähtaja meeldetuletusi",
    "Turn off due date reminders" : "Ära kasuta tähtaja meeldetuletusi",
    "Remove as default board" : "Lõpeta olek vaikimisi tahvlina",
    "Set as default board" : "Määra vaikimisi tahvliks",
    "Due date reminders" : "Tähtaja meeldetuletused",
    "Assigned cards" : "Kaardid, millel on täitja olemas",
    "No notifications" : "Märguandeid pole",
    "Delete board" : "Kustuta tahvel",
    "Leave board" : "Lahku tahvlilt",
    "Importing board..." : "Impordin tahvlit...",
    "Board imported successfully" : "Tahvli importimine õnnestus",
    "Import board" : "Impordi tahvel",
    "Clone {boardTitle}" : "Tee tahvlist koopia: {boardTitle}",
    "Clone cards" : "Tee kaartidest koopia",
    "Clone assignments" : "Tee ülesannetest koopia",
    "Clone labels" : "Tee siltidest koopia",
    "Clone due dates" : "Tee tähtaegadest koopia",
    "Advanced options" : "Täpsemad valikud",
    "Move all cards to the first list" : "Tõsta kõik kaardid esimesse loendisse",
    "Restore archived cards" : "Taasta arhiveeritud kaardid",
    "Clone" : "Tee koopia",
    "Export {boardTitle}" : "Ekspordi „{boardTitle}“",
    "Export as JSON" : "Ekspordi JSON-failina",
    "Export as CSV" : "Ekspordi CSV-failina",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Märkus: vaid JSON-vorming on toetatud tagasi importimiseks siia kanbani-rakendusse.",
    "Export" : "Ekspordi",
    "Loading filtered view" : "Laadin filtreeritud vaadet",
    "Search for {searchQuery} in other boards" : "Otsi muudelt tahvlitelt: „{searchQuery}“",
    "Search for {searchQuery} in all boards" : "Otsi kõikidelt tahvlitelt: „{searchQuery}“",
    "No results found" : "Otsingutulemusi ei leidu",
    "Deck board {name}\n* Last modified on {lastMod}" : "Kanbani tahvel {name}\n* Viimati muudetud {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Loodud {created}\n* Viimati muudetud {lastMod}\n* {nbAttachments} manus(t)\n* {nbComments} kommentaar(i)",
    "{nbCards} cards" : "{nbCards} kaarti",
    "Due on {date}" : "Tähtaeg: {date}",
    "{stack} in {board}" : "„{stack}“ kasutaja „{board}“ kõnes",
    "Click to expand description" : "Klõpsi kogu kirjelduse vaatamiseks",
    "Click to expand comment" : "Klõpsi kogu kommentaari vaatamiseks",
    "Create card" : "Koosta kaart",
    "Create a new card" : "Koosta uus Kanbani kaart",
    "Card title" : "Kaardi pealkiri",
    "Creating the new card …" : "Uus kaart on loomisel...",
    "Card \"{card}\" was added to \"{board}\"" : "„{card}“ ülesanne on lisatud „{board}“ tahvlile",
    "Open card" : "Ava kaart",
    "Close" : "Sulge",
    "No upcoming cards" : "Lähiajal täitmisele kuuluvaid kaarte pole",
    "upcoming cards today" : "täna täitmisele kuuluvad kaardid",
    "upcoming cards tomorrow" : "homme täitmisele kuuluvad kaardid",
    "upcoming cards" : "järgmised täitmisele kuuluvad kaardid",
    "New card" : "Uus kaart",
    "Link to a board" : "Lisa link tahvlile",
    "Link to a card" : "Lisa link kaardile",
    "Create a card" : "Koosta kaart",
    "Message from {author} in {conversationName}" : "Sõnum kasutajalt „{author}“ „{conversationName}“ vestluses",
    "Something went wrong" : "Vaat kus lops, midagi läks valesti",
    "Failed to upload {name}" : "„{name}“ üleslaadimine ei õnnestunud",
    "Maximum file size of {size} exceeded" : "Failide maksimaalne lubatud suurus {size} on ületatud",
    "Assigned users" : "Ülesannetega kasutajad",
    "Due date" : "Tähtaeg",
    "Created" : "Loodud",
    "Modified" : "Muudetud",
    "Error creating the share" : "Viga jaosmeedia loomisel",
    "Share with a Deck card" : "Jaga kanbani kaardiga",
    "Share {file} with a Deck card" : "Jaga „{file}“ faili kanbani kaardiga",
    "Share" : "Jaga",
    "The card \"%s\" on \"%s\" has reached its due date." : "Tähtaeg on käes: „%s“ kaart, „%s“ tahvel",
    "Share board with a user, group or team …" : "Jaga tahvlit kasutaja, grupi või tiimidega…",
    "Searching for users, groups and teams …" : "Otsin kasutajaid, gruppe või tiime…",
    "No participants found" : "Osalejaid ei leidu",
    "Save" : "Salvesta",
    "seconds ago" : "sekundit tagasi",
    "Today" : "Täna",
    "Tomorrow" : "Homme",
    "No due" : "Tähtaega pole"
},
"nplurals=2; plural=(n != 1);");
