OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Has creado el tablero nuevo {board}",
    "{user} has created a new board {board}" : "{user} ha creado el tablero nuevo {board}",
    "You have deleted the board {board}" : "Has eliminado el tablero {board}",
    "{user} has deleted the board {board}" : "{user} ha eliminado el tablero {board}",
    "You have restored the board {board}" : "Has restaurado el tablero {board}",
    "{user} has restored the board {board}" : "{user} ha restaurado el tablero {board}",
    "You have shared the board {board} with {acl}" : "Has compartido el tablero {board} con {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} ha compartido el tablero {board} con {acl}",
    "You have removed {acl} from the board {board}" : "Has eliminado a {acl} del tablero {board}",
    "{user} has removed {acl} from the board {board}" : "{user} ha elimiando a {acl} del tablero {board}",
    "You have renamed the board {before} to {board}" : "Has renombrado el tablero {before} como {board}",
    "{user} has renamed the board {before} to {board}" : "{user} ha renombrado el tablero {before} como {board}",
    "You have archived the board {board}" : "Has archivado el tablero {board}",
    "{user} has archived the board {before}" : "{user} ha archivado el tablero {board}",
    "You have unarchived the board {board}" : "Has desarchivado el tablero {board}",
    "{user} has unarchived the board {before}" : "{user} ha desarchivado el tablero {before}",
    "You have created a new list {stack} on board {board}" : "Has creado una nueva lista {stack} del tablero {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} ha creado una nueva lista {stack} del tablero {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Has renombrado la lista {before} a {stack} del tablero {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} ha renombrado la lista {before} a {stack} del tablero {board}",
    "You have deleted list {stack} on board {board}" : "Has eliminado la lista {stack} del tablero {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} ha eliminado la lista {stack} del tablero {board}",
    "You have created card {card} in list {stack} on board {board}" : "Has creado la tarjeta {card} en la lista {stack} del tablero {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} ha creado la tarjeta {card} en la lista {stack} del tablero {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Has eliminado la tarjeta {card}  en la lista {stack} del tablero {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} ha borrado la tarjeta {card} en la lista {stack} del tablero {board}",
    "You have renamed the card {before} to {card}" : "Has renombrado la tarjeta {before} como {card}",
    "{user} has renamed the card {before} to {card}" : "{user} ha renombrado la tarjeta {before} como {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Has añadido una descripción a la tarjeta {card} en la lista {stack} del tablero {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} ha añadido una descripción a la tarjeta {card} en la lista {stack} del tablero {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Has actualizado la descripción de {card} en la lista {stack} del tablero {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} ha actualizado la descripción de la tarjeta {card} en la lista {stack} del tablero {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Has archivado la tarjeta {card} en la lista {stack} del tablero {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} ha archivado la tarjeta {card} en la lista {stack} del tablero {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Has desarchivado la tarjeta {card} en la lista {stack} del tablero {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} ha desarchivado la tarjeta {card} en la lista {stack} del tablero {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Ud. ha marcado la tarjeta {card} como completada en la lista {stack} del tablero {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} ha marcado la tarjeta {card} como completada en la lista {stack} del tablero {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Ud. ha marcado la tarjeta {card} como no completada en la lista {stack} del tablero {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} ha marcado la tarjeta {card} como no completada en la lista {stack} del tablero {board}",
    "You have removed the due date of card {card}" : "Has eliminado la fecha de vencimiento de {card}",
    "{user} has removed the due date of card {card}" : "{user} ha eliminado la fecha de vencimiento de {card}",
    "You have set the due date of card {card} to {after}" : "Has establecido la fecha de vencimiento de {card} como {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} ha establecido la fecha de vencimiento de {card} como {after}",
    "You have updated the due date of card {card} to {after}" : "Has actualizado la fecha de vencimiento de {card} para {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} ha actualizado la fecha de vencimiento de {card} para {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Has añadido la etiqueta {label} a la tarjeta {card} en la lista {stack} del tablero {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} ha añadido la etiqueta {label} a la tarjeta {card} en la lista {stack} del tablero {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Has eliminado la etiqueta {label} de la tarjeta {card} en la lista {stack} del tablero {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} ha elimininado la etiqueta {label} de la tarjeta{card} en la lista {stack} del tablero {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Has asignado a {assigneduser} para {card} en {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} ha asignado a {assigneduser} para {card} en {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Has desasignado a {assigneduser} para {card} en {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} ha asignado a {assigneduser} para {card} en {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Has movido la tarjeta {card} de la lista {stackBefore} al tablero {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} ha movido la tarjeta {card} de la lista {stackBefore} al tablero {stack}",
    "You have added the attachment {attachment} to card {card}" : "Has añadido el adjunto {attachment} en {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} ha añadido el adjunto {attachment} en {card}",
    "You have updated the attachment {attachment} on card {card}" : "Has actualizado el adjunto [attachment] en {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} ha actualizado el adjunto {attachment} en la tarjeta {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Has eliminado el adjunto {attachment} de {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} ha eliminado el adjunto {attachment} de la tarjeta {card}",
    "You have restored the attachment {attachment} to card {card}" : "Has restaurado el adjunto {attachment} a {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} ha restaurado el adjunto {attachment} a {card}",
    "You have commented on card {card}" : "Has comentado en la tarjeta {card}",
    "{user} has commented on card {card}" : "{user} ha comentado en {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Cambios en la <strong>app Deck</strong>",
    "A <strong>board, list or card</strong> was changed" : "Un <strong>tablero, lista o tarjeta</strong> fue cambiado/a",
    "A <strong>comment</strong> was created on a card" : "Un <strong>comentario</strong> ha sido creado en una tarjeta",
    "A <strong>card description</strong> has been changed" : "Una <strong>descripción de tarjeta</strong> ha sido cambiada",
    "The file was uploaded" : "Se ha subido el archivo",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "El archivo subido excede la directiva upload_max_filesize en php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "El archivo subido excede la directiva MAX_FILE-SIZE directive que se especificó en el formulario web",
    "The file was only partially uploaded" : "El archivo se ha subido solo parcialmente",
    "No file was uploaded" : "No se ha subido ningún archivo",
    "Missing a temporary folder" : "Falta una carpeta temporal",
    "Could not write file to disk" : "No se ha podido escribir el archivo al disco",
    "A PHP extension stopped the file upload" : "Una extensión de PHP ha detenido la subida del archivo",
    "No file uploaded or file size exceeds maximum of %s" : "No se ha subido ningún archivo, o el tamaño del archivo excede el máximo de %s",
    "Invalid file type. Only JSON files are allowed." : "Tipo de archivo inválido. Solo se permiten archivos JSON",
    "Invalid JSON data" : "Datos JSON inválidos",
    "Failed to import board" : "Fallo al importar tablero",
    "Cards due today" : "Tarjetas que vencen hoy",
    "Cards due tomorrow" : "Tarjetas que vencen mañana",
    "Upcoming cards" : "Próximas tarjetas",
    "Load more" : "Cargar más",
    "Welcome to Nextcloud Deck!" : "¡Bienvenido a Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "La tarjeta \"%s\" en \"%s\" te ha sido asignada por %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} te ha asigando la tarjeta {deck-card} de {deck-board} a ti.",
    "The card {deck-card} on {deck-board} has reached its due date." : "La tarjeta {deck-card} en {deck-board} ha alcanzado su fecha de caducidad.",
    "%s has mentioned you in a comment on \"%s\"." : "%s te ha mencionado en un comentario en \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} te ha mencionado en un comentario de {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "El tablero \"%s\" ha sido compartido contigo por %s.",
    "{user} has shared {deck-board} with you." : "{user} ha compartido {deck-board} contigo.",
    "Deck board" : "Tablero Deck",
    "Owned by %1$s" : "Propiedad de %1$s",
    "Deck boards, cards and comments" : "Tableros Deck, tarjetas y comentarios",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "De %1$s, en %2$s/%3$s, propiedad de %4$s",
    "Create a new deck card" : "Crear una nueva tarjeta de tablero",
    "Card comments" : "Comentarios en tarjetas",
    "%s on %s" : "%s en %s",
    "Deck boards and cards" : "Tableros Deck y tarjetas",
    "No data was provided to create an attachment." : "No se proporcionaron datos para crear un adjunto",
    "Finished" : "Finalizado",
    "To review" : "Para revisar",
    "Action needed" : "Acción necesaria",
    "Later" : "Después",
    "copy" : "copiar",
    "Read more inside" : "Lea más, adentro",
    "Custom lists - click to rename!" : "Listas personalizadas - ¡Haga clic para renombrar!",
    "To Do" : "Por hacer",
    "In Progress" : "En progreso",
    "Done" : "Hecho",
    "1. Open to learn more about boards and cards" : "1. Abra aquí para aprender más sobre los tableros y las tarjetas",
    "2. Drag cards left and right, up and down" : "2. Arrastre las tarjetas a la izquierda y derecha, arriba y abajo",
    "3. Apply rich formatting and link content" : "3. Aplique formato enriquecido y contenido a los enlaces",
    "4. Share, comment and collaborate!" : "4. ¡Comparta, comente y colabore!",
    "Create your first card!" : "¡Cree su primera tarjeta!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Este comentario tiene más de %s caracteres.\nSe ha añadido como adjunto a la tarjeta con el nombre %s.\nAccesible en la URL: %s.",
    "Attachments" : "Adjuntos",
    "File" : "Archivo",
    "date" : "fecha",
    "Card not found" : "Tarjeta no encontrada",
    "Path is already shared with this card" : "La ruta ya se ha compartido con esta tarjeta",
    "Invalid date, date format must be YYYY-MM-DD" : "Fecha no válida, el formato de las fechas debe ser AAAA-MM-DD",
    "Personal planning and team project organization" : "Planificación personal y organización de proyecto de equipo",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck es una herramienta de organización de estilo kanban dirigida a la planificación personal y la organización de proyectos para equipos, integrada a Nextcloud.\n\n\n- 📥 Agregue sus tareas a las tarjetas y póngalas en orden.\n- 📄 Escriba notas adicionales en Markdown\n- 🔖 Asigne etiquetas para una organización mejor\n- 👥 Comparta con su equipo, amigos o familia.\n- 📎 Adjunte archivos e incruste los mismos en su descripción Markdown\n- 💬 Discuta con su equipo usando comentarios.\n- ⚡ Mantenga un registro de los cambios en el flujo de actividad\n- 🚀 Organice su proyecto",
    "Add board" : "Añadir tablero",
    "Card details" : "Detalles de la tarjeta",
    "Select the board to link to a project" : "Selecciona el tablero que enlazar a un proyecto",
    "Search by board title" : "Buscar por título de tablero",
    "Select board" : "Selecciona tablero",
    "Move/copy card" : "Mover/copiar tarjeta",
    "Select a board" : "Selecciona un tablero",
    "No lists available" : "No hay listas disponibles",
    "Select a list" : "Seleccionar una lista",
    "Move card" : "Mover tarjeta",
    "Copy card" : "Copiar tarjeta",
    "Select the card to link to a project" : "Selecciona la tarjeta que enlazar a un proyecto",
    "Link to card" : "Enlace a tarjeta",
    "Select a card" : "Seleccionar una tarjeta",
    "Cancel" : "Cancelar",
    "This board is read only" : "Este tablero es de sólo lectura",
    "Drop your files to upload" : "Arrastra tus archivos para subir",
    "File already exists" : "El archivo ya existe",
    "A file with the name {filename} already exists." : "Ya existe un archivo con el nombre {filename}.",
    "Do you want to overwrite it?" : "¿Desea sobreescribirlo?",
    "Overwrite file" : "Sobreescribir archivo",
    "Keep existing file" : "Mantener el archivo existente",
    "Add card" : "Añadir tarjeta",
    "Archived cards" : "Tarjetas archivadas",
    "Add list" : "Agregar lista",
    "List name" : "Nombre de la lista",
    "Clear search" : "Limpiar búsqueda",
    "Active filters" : "Filtros activos",
    "Apply filter" : "Aplicar filtro",
    "Filter by tag" : "Filtrar por etiqueta",
    "Filter by assigned user" : "Filtrar por usuario asignado",
    "Unassigned" : "No asignado",
    "Filter by status" : "Filtrar por estado",
    "Open and completed" : "Abiertos y completados",
    "Open" : "Abrir",
    "Completed" : "Completado",
    "Filter by due date" : "Filtrar por fecha de finalización",
    "Overdue" : "Demorado",
    "Next 24 hours" : "Próximas 24 horas",
    "Next 7 days" : "Siguientes 7 días",
    "Next 30 days" : "Siguientes 30 días",
    "No due date" : "Sin fecha de vencimiento",
    "Clear filter" : "Borrar filtro",
    "View Modes" : "Modos de visualización",
    "Toggle View Modes" : "Alternar modos de visualización",
    "Hide archived cards" : "Ocultar tarjetas archivadas",
    "Show archived cards" : "Mostrar tarjetas archivadas",
    "Toggle compact mode" : "Mostrar/ocultar modo compacto",
    "Hide card cover images" : "Ocultar la imagen de portada de las tarjetas",
    "Show card cover images" : "Mostrar la imagen de portada de las tarjetas",
    "Open details" : "Abrir detalles",
    "Details" : "Detalles",
    "Deck settings" : "Configuración del tablero",
    "General" : "General",
    "Use bigger card view" : "Usar vista de tarjeta mayor",
    "Appearance" : "Apariencia",
    "Show card ID badge" : "Mostrar insignia de la ID de tarjeta",
    "Show boards in calendar/tasks" : "Mostrar tableros en calendario/tareas",
    "Admin settings" : "Opciones de Admin",
    "Limit board creation to some groups" : "Limitar la creación de tableros a algunos grupos",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Los usuarios fuera de estos grupos no podrán crear sus propios tableros, sin embargo, podrán trabajar en tableros que han sido compartidos con ellos.",
    "Board actions" : "Acciones del tablero",
    "Scroll sideways" : "Desplazarse lateralmente",
    "Navigate between cards" : "Navegar entre tarjetas",
    "Close card details" : "Cerrar detalles de la tarjeta",
    "Search" : "Buscar",
    "Show card filters" : "Mostrar filtros de tarjeta",
    "Clear card filters" : "Limpiar filtros de tarjeta",
    "Show those shortcuts" : "Mostrar estos atajos",
    "Card actions" : "Acciones de tarjeta",
    "Open card details" : "Abrir detalles de la tarjeta",
    "Edit the card title" : "Editar el título de la tarjeta",
    "Assign yourself to the current card" : "Asignarme a mí mismo a la tarjeta actual",
    "Archive/unarchive the current card" : "Archivar/desarchivar la tarjeta actual",
    "Mark card as completed/not completed" : "Marcar tarjeta como completada/no completada",
    "Open card menu" : "Abrir menú de tarjetas",
    "Currently present people" : "Personas presentes actualmente",
    "Loading board" : "Cargando tablero",
    "Board not found" : "Tablero no encontrado",
    "Create a new list to add cards to this board" : "Crea una lista nueva para añadir tarjetas a este tablero",
    "Sharing" : "Compartir",
    "Tags" : "Etiquetas",
    "Deleted items" : "Elementos eliminados",
    "Activity" : "Actividad",
    "Deleted lists" : "Listas eliminadas",
    "Undo" : "Deshacer",
    "Deleted cards" : "Cartas eliminadas",
    "Hours" : "Horas",
    "Days" : "Días",
    "Weeks" : "Semanas",
    "Months" : "Meses",
    "No cards yet" : "Todavía no hay tarjetas",
    "Failed to create share with {displayName}" : "Fallo al crear el recurso compartido denominado {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "¿Está seguro de que quiere transferir el tablero {title} a {user}?",
    "Transfer the board." : "Transfiera el tablero.",
    "Transfer" : "Transferir",
    "The board has been transferred to {user}" : "El tablero ha sido transferido a {user}",
    "Failed to transfer the board to {user}" : "Fallo al transferir el tablero a {user}",
    "Board owner" : "Propietario del tablero",
    "(Group)" : "(Grupo)",
    "(Team)" : "(Equipo)",
    "(remote)" : "(remoto)",
    "Can edit" : "Puede editar",
    "Can share" : "Puede compartir",
    "Can manage" : "Puede gestionar",
    "Owner" : "Propietario",
    "Delete" : "Eliminar",
    "List deleted" : "Lista borrada",
    "Edit list title" : "Editar el título de la lista",
    "Archive all cards" : "Archivar todas las tarjetas",
    "Unarchive all cards" : "Desarchivar todas las tarjetas",
    "Delete list" : "Eliminar lista",
    "Archive all cards in this list" : "Archivar todas las tarjetas en esta lista",
    "Unarchive all cards in this list" : "Desarchivar todas las tarjetas en esta lista",
    "Add a new card" : "Añadir una nueva tarjeta",
    "Card name" : "Nombre de la tarjeta",
    "title and color value must be provided" : "Se debe indicar un valor para título y color ",
    "Edit" : "Editar",
    "Add a new tag" : "Añade una nueva etiqueta",
    "Board name" : "Nombre del tablero",
    "Members" : "Miembros",
    "Assign to users/groups/team" : "Asignar a usuarios/grupos/equipo",
    "Assign a user to this card…" : "Asignar un usuario a esta tarjeta...",
    "Select a user to assign to this card…" : "Seleccione un usuario para asignar esta tarjeta…",
    "File to share" : "Archivo a compartir",
    "Invalid path selected" : "Ruta seleccionada no válida",
    "Upload new files" : "Subir nuevos archivos",
    "Share from Files" : "Compartir desde Archivos",
    "Pending share" : "Recurso compartido pendiente",
    "Add this attachment" : "Añadir este adjunto",
    "Show in Files" : "Mostrar en Archivos",
    "Download" : "Descargar",
    "Remove attachment" : "Quitar adjunto",
    "Delete Attachment" : "Borrar adjunto",
    "Restore Attachment" : "Restaurar Adjunto",
    "The title cannot be empty." : "El título no puede estar vacío.",
    "Cannot close unsaved card!" : "¡No se puede cerrar esta tarjeta no guardada!",
    "Open in sidebar view" : "Abrir en vista de barra lateral",
    "Open in bigger view" : "Abrir en vista más grande",
    "Comments" : "Comentarios",
    "Failed to load comments" : "Fallo al cargar los comentarios",
    "No comments yet. Begin the discussion!" : "Todavía no hay comentarios. ¡Comienza la discusión!",
    "The comment cannot be empty." : "El comentario no puede estar vacío.",
    "The comment cannot be longer than 1000 characters." : "El comentario no puede tener más de 1000  caracteres.",
    "Submit" : "Enviar",
    "Created:" : "Creado:",
    "In reply to" : "En respuesta a",
    "Cancel reply" : "Cancelar respuesta",
    "Reply" : "Responder",
    "Update" : "Actualizar",
    "Mark as done" : "Marcar como completado",
    "Mark as not done" : "Marcar como no completado",
    "Write a description …" : "Escribe una descripción...",
    "Could not save description" : "No se pudo guardar la descripción",
    "Description" : "Descripción",
    "(Unsaved)" : "(Sin salvar)",
    "(Saving…)" : "(Guardando...)",
    "Formatting help" : "Ayuda de formato",
    "Edit description" : "Editar descripción",
    "View description" : "Ver descripción",
    "Add Attachment" : "Añadir adjunto",
    "Choose attachment" : "Escoger adjunto",
    "Select Date" : "Seleccione la fecha",
    "Later today – {timeLocale}" : "Hoy, más tarde – {timeLocale}",
    "Set due date for later today" : "Establecer hoy como fecha de vencimiento, más tarde",
    "Tomorrow – {timeLocale}" : "Mañana – {timeLocale}",
    "Set due date for tomorrow" : "Establecer mañana como fecha de vencimiento",
    "This weekend – {timeLocale}" : "Este fin de semana – {timeLocale}",
    "Set due date for this weekend" : "Establecer este fin de semana como fecha de vencimiento",
    "Next week – {timeLocale}" : "Próxima semana – {timeLocale}",
    "Set due date for next week" : "Establecer la próxima semana como fecha de vencimiento",
    "Assign a due date to this card…" : "Asignar una fecha de vencimiento a esta tarjeta…",
    "Set a due date" : "Fijar una fecha límite",
    "Add due date" : "Añadir fecha de vencimiento",
    "Choose a date" : "Elija una fecha",
    "Remove due date" : "Eliminar fecha límite",
    "Due at:" : "Vence el:",
    "Not done" : "No está finalizado",
    "Unarchive card" : "Desarchivar tarjeta",
    "Archive card" : "Archivar tarjeta",
    "Assign a tag to this card…" : "Asignar una etiqueta a esta tarjeta...",
    "Select or create a tag…" : "Seleccionar o crear una etiqueta…",
    "Create a new tag:" : "Crear una nueva etiqueta:",
    "(group)" : "(grupo)",
    "{count} comments, {unread} unread" : "{count} comentarios, {unread} sin leer",
    "Todo items" : "Ítems pendientes",
    "Edit card title" : "Editar el título de la tarjeta",
    "Open link" : "Abrir enlace",
    "Card deleted" : "Tarjeta borrada",
    "Edit title" : "Editar título",
    "Assign to me" : "Asignarme a mí",
    "Unassign myself" : "Desasignarme a mí mismo",
    "Delete card" : "Eliminar tarjeta",
    "All boards" : "Todos los tableros",
    "Archived boards" : "Tableros archivados",
    "Shared with you" : "Compartido contigo",
    "Cancel edit" : "Cacelar edición",
    "Save board" : "Guardar tablero",
    "Board {0} deleted" : "Tablero {0} eliminado",
    "All cards" : "Todas las tarjetas",
    "Only assigned cards" : "Sólo las tarjetas asignadas",
    "No reminder" : "No hay recordatorio",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "¿Está seguro de querer eliminar el tablero {title}? Esto eliminará todos los datos del tablero incluyendo las tarjetas archivadas.",
    "Delete the board?" : "¿Borrar el tablero?",
    "Leave" : "Abandonar",
    "Exporting board..." : "Exportando tablero...",
    "An error occurred" : "Ocurrió un error",
    "Board details" : "Detalles del tablero",
    "Edit board" : "Editar tablero",
    "Clone board" : "Clonar tablero",
    "Unarchive board" : "Desarchivar tablero",
    "Archive board" : "Archivar tablero",
    "Export board" : "Exportar tablero",
    "Turn on due date reminders" : "Activar recordatorios de la fecha de vencimiento",
    "Turn off due date reminders" : "Desactivar recordatorios de la fecha de vencimiento",
    "Due date reminders" : "Recordatorios de la fecha de vencimiento",
    "Assigned cards" : "Tarjetas asignadas",
    "No notifications" : "No hay notificaciones",
    "Delete board" : "Eliminar tablero",
    "Importing board..." : "Importando tablero...",
    "Board imported successfully" : "Se importó el tablero exitosamente",
    "Import board" : "Importar tablero",
    "Clone {boardTitle}" : "Clonar {boardTitle}",
    "Clone cards" : "Clonar tarjetas",
    "Clone assignments" : "Clonar asignaciones",
    "Clone labels" : "Clonar etiquetas",
    "Clone due dates" : "Clonar fechas de vencimiento",
    "Advanced options" : "Opciones avanzadas",
    "Move all cards to the first list" : "Mover todas las tarjetas a la primera lista",
    "Restore archived cards" : "Restaurar tarjetas archivadas",
    "Clone" : "Clonar",
    "Export {boardTitle}" : "Exportar {boardTitle}",
    "Export as JSON" : "Exportar como JSON",
    "Export as CSV" : "Exportar como CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Nota: Solo el formato JSON es soportar al importar de vuelta en la app Deck.",
    "Export" : "Exportar",
    "Loading filtered view" : "Cargando vista filtrada",
    "Search for {searchQuery} in other boards" : "Buscar {searchQuery} en otros tableros",
    "Search for {searchQuery} in all boards" : "Buscar {searchQuery} en todos los tableros",
    "No results found" : "No se encontraron resultados",
    "Deck board {name}\n* Last modified on {lastMod}" : "Tablero Deck {name}\n* Se modificó por última vez el {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Creado en {created}\n* Se modificó por última vez el {lastMod}\n* {nbAttachments} adjuntos\n* {nbComments} comentarios",
    "{nbCards} cards" : "{nbCards} tarjetas",
    "Due on {date}" : "Vence en {date}",
    "{stack} in {board}" : "{stack} en {board}",
    "Click to expand description" : "Haga clic para expandir la descripción",
    "Click to expand comment" : "Haga clic para expandir el comentario",
    "Create card" : "Crear tarjeta",
    "Create a new card" : "Crear una tarjeta nueva",
    "Card title" : "Título de la tarjeta",
    "Creating the new card …" : "Creando una tarjeta nueva …",
    "Card \"{card}\" was added to \"{board}\"" : "La tarjeta \"{card}\" fue añadida a \"{board}\"",
    "Open card" : "Abrir tarjeta",
    "Close" : "Cerrar",
    "No upcoming cards" : "No hay tarjetas próximas",
    "upcoming cards today" : "próximas tarjetas hoy",
    "upcoming cards tomorrow" : "próximas tarjetas mañana",
    "upcoming cards" : "próximas tarjetas",
    "New card" : "Nueva tarjeta",
    "Link to a board" : "Enlace a un tablero",
    "Link to a card" : "Enlace a una tarjeta",
    "Create a card" : "Crear una tarjeta",
    "Message from {author} in {conversationName}" : "Mensaje de {author} en {conversationName}",
    "Something went wrong" : "Algo ha ido mal",
    "Failed to upload {name}" : "Error al subir {name}",
    "Maximum file size of {size} exceeded" : "Tamaño máximo de archivo de {size} excedido",
    "Assigned users" : "Usuarios asignados",
    "Due date" : "Fecha de vencimiento",
    "Created" : "Creado",
    "Modified" : "Modificado",
    "Error creating the share" : "Error creando el recurso compartido",
    "Share with a Deck card" : "Compartir con una tarjeta de Deck",
    "Share {file} with a Deck card" : "Compartir {file} con una tarjeta de Deck",
    "Share" : "Compartir",
    "The card \"%s\" on \"%s\" has reached its due date." : "La tarjeta \"%s\" en \"%s\" ha alcanzado su fecha límite.",
    "Share board with a user, group or team …" : "Compartir tablero con un usuario, grupo o equipo …",
    "Searching for users, groups and teams …" : "Buscando usuarios, grupos y equipos …",
    "No participants found" : "No se encontraron participantes",
    "Save" : "Guardar",
    "seconds ago" : "hace unos segundos",
    "Today" : "Hoy",
    "Tomorrow" : "Mañana",
    "No due" : "Sin finalizar"
},
"nplurals=3; plural=n == 1 ? 0 : n != 0 && n % 1000000 == 0 ? 1 : 2;");
