<?php

/**
 * SPDX-FileCopyrightText: 2016 Nextcloud GmbH and Nextcloud contributors
 * SPDX-License-Identifier: AGPL-3.0-or-later
 */

namespace OCA\Deck\AppInfo;

/**
 * Additional autoloader registration, e.g. registering composer autoloaders
 */
require_once __DIR__ . '/../vendor/autoload.php';
