OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Създадохте ново табло {board}",
    "{user} has created a new board {board}" : "{user}  създаде ново табло {board}",
    "You have deleted the board {board}" : "Изтрихте таблото {board}",
    "{user} has deleted the board {board}" : "{user} изтри таблото {board}",
    "You have restored the board {board}" : "Възстановихте таблото {board}",
    "{user} has restored the board {board}" : "{user}  възстанови таблото {board}",
    "You have shared the board {board} with {acl}" : "Споделихте таблото {board} с {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} сподели таблото {board} с {acl}",
    "You have removed {acl} from the board {board}" : "Премахнахте {acl} от таблото {board}",
    "{user} has removed {acl} from the board {board}" : "{user} премахна {acl} от таблото {board}",
    "You have renamed the board {before} to {board}" : "Преименувахте таблото от {before} на {board}",
    "{user} has renamed the board {before} to {board}" : "{user} преименува таблото от {before} на {board}",
    "You have archived the board {board}" : "Архивирахте таблото {board}",
    "{user} has archived the board {before}" : "{user}  архивира таблото {board}",
    "You have unarchived the board {board}" : "Разархивирали сте таблото {board}",
    "{user} has unarchived the board {before}" : "{user} е разархивирал таблото {before}",
    "You have created a new list {stack} on board {board}" : "Създадохте нов списък {stack} на таблото  {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} създаде нов списък {stack} на таблото  {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Преименувахте списъка {before} на {stack} на таблото {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} преименува списъка {before} на {stack} на таблото {board}",
    "You have deleted list {stack} on board {board}" : "Изтрихте списък {stack} на таблото {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} изтри списък {stack} на таблото {board}",
    "You have created card {card} in list {stack} on board {board}" : "Създали сте карта {card} в списък {stack} на таблото {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} създаде карта {card} в списък {stack} на таблото {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Изтрили сте карта {card} в списък {stack} на таблото {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} изтри карта {card} в списък {stack} на таблото {board}",
    "You have renamed the card {before} to {card}" : "Преименувахте картата {before} на {card}",
    "{user} has renamed the card {before} to {card}" : "{user} преименува картата {before} на {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Добавили сте описание към карта {card} в списък {stack} на таблото {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} добави описание към карта {card} в списък {stack} на таблото {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : " Актуализирали сте описанието към карта {card} в списък {stack} на таблото {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : " {user} актуализира описанието към карта {card} в списък {stack} на таблото {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Архивирали сте карта {card} в списък {stack} на борда {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user}  архивира карта {card} в списък {stack} на борда {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Разархивирали сте карта {card} в списък {stack} на борда {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user}  разархивира карта {card} в списък {stack} на борда {board}",
    "You have removed the due date of card {card}" : "Премахнахте датата на падежа на картата {card}",
    "{user} has removed the due date of card {card}" : "{user} премахна датата на падежа на картата {card}",
    "You have set the due date of card {card} to {after}" : "Задали сте датата на падежа на картата {card} на {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} зададе дата на падежа на картата {card} на {after}",
    "You have updated the due date of card {card} to {after}" : "Актуализирали сте датата на падежа на картата {card} на {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} актуализира датата на падежа на картата {card} на {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Добавихте етикет {label} към карта {card} в списък {stack} на таблото {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} добави етикет {label} към карта {card} в списък {stack} натаблото {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Премахнахте етикет {label} към карта {card} в списък {stack} на таблото {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} премахна етикет {label} към карта {card} в списък {stack} на таблото {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Присвоили сте {assigneduser} на карта {card} на борда {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} присвои {assigneduser} на карта {card} на борда {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Имате неопределен {assigneduser} на карта {card} на борда {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} има неопределен {assigneduser} на карта {card} на борда {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Преместихте картата {card} от списък {stackBefore} в {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} премести картата {card} от списък {stackBefore} в {stack}",
    "You have added the attachment {attachment} to card {card}" : "Добавихте прикачения файл {attachment} към карта {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} добави прикачения файл {attachment} към карта {card}",
    "You have updated the attachment {attachment} on card {card}" : " Актуализирали сте прикачения файл {attachment} към карта {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} актуализира прикачения файл {attachment} към карта {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Изтрихте прикачения файл {attachment} от карта {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} изтри прикачения файл {attachment} към карта {card}",
    "You have restored the attachment {attachment} to card {card}" : "Възстановили сте прикачения файл {attachment} към карта {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} възстанови прикачения файл {attachment} към карта {card}",
    "You have commented on card {card}" : "Коментирали сте карта {card}",
    "{user} has commented on card {card}" : "{user}  коментира карта {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Промени в <strong>приложението Deck</strong>",
    "A <strong>comment</strong> was created on a card" : "На картата е създаден <strong>коментар</strong>",
    "The file was uploaded" : "Файлът е качен",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Размерът на каченият файл надвишава максималния размер определен от upload_max_filesize в php.ini:",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Размерът на файла надвишава максималния размер определен от MAX_FILE_SIZE в HTML формата.",
    "The file was only partially uploaded" : "Файлът е качен частично",
    "No file was uploaded" : "Нито един файл не е качен",
    "Missing a temporary folder" : "Липсва временна папка",
    "Could not write file to disk" : " Файлът не можа да бъде записан на диск",
    "A PHP extension stopped the file upload" : "PHP разширение спря качването на файла",
    "No file uploaded or file size exceeds maximum of %s" : "Няма качен файл или размерът на файла надвишава максимума от %s",
    "Cards due today" : "Карти изтичащи днес",
    "Cards due tomorrow" : "Карти изтичащи утре",
    "Upcoming cards" : "Предстоящи карти",
    "Load more" : "Зареждане на още",
    "Welcome to Nextcloud Deck!" : "Добре дошли в Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Картата \"%s\" на \"%s\" Ви е била възложена от %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} е възложил картата {deck-card} на {deck-board} на Вас.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Картата {deck-card} на {deck-board} е достигнала датата на падежа си.",
    "%s has mentioned you in a comment on \"%s\"." : "%s те спомена в коментар за “%s”",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} те спомена в коментар за {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Таблото \"%s\" е споделено с вас от%s.",
    "{user} has shared {deck-board} with you." : "{user} сподели {deck-board} с Вас.",
    "Deck board" : "Deck табло",
    "Owned by %1$s" : "Притежаван от %1$s",
    "Deck boards, cards and comments" : "Табла, карти и коментари",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "От %1$s, в %2$s/%3$s, притежание на %4$s",
    "Card comments" : "Коментари на карти",
    "%s on %s" : "%s на %s",
    "Deck boards and cards" : "Табла и карти",
    "No data was provided to create an attachment." : "Не бяха предоставени данни за създаване на прикачен файл.",
    "Finished" : "Готово",
    "To review" : "За преглед",
    "Action needed" : "Необходимо е действие",
    "Later" : "По-късно",
    "copy" : "Копиране",
    "To Do" : "Задачи",
    "Done" : "Завършено",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Този коментар има повече от %s знака.\nДобавено като прикачен файл към картата с име %s.\nДостъпно на URL: %s.",
    "Attachments" : "Прикачени файлове",
    "File" : "Файл",
    "Card not found" : "Катртата не е намерена",
    "Path is already shared with this card" : "Пътят вече е споделен с тази карта",
    "Invalid date, date format must be YYYY-MM-DD" : "Невалидна дата, форматът е различен от ГГГГ-ММ-ДД",
    "Personal planning and team project organization" : "Лично планиране и организация на екипни проекти",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck е инструмент за организация в стил kanban, насочен към лично планиране и организация на проекти за екипи, интегрирани с Nextcloud.\n\n\n- 📥 Добавете задачите си към карти и ги подредете\n- 📄 Запишете допълнителни бележки в  markdown формат\n- Присвояване на етикети за още по-добра организация\n- 👥 Споделете с вашия екип, приятели или семейство\n- 📎Прикачете файлове и ги вградете във вашето описание за маркиране\n- 💬Обсъдете с вашия екип, като използвате коментари\n- ⚡ Проследявайте промените в потока от дейности\n- 🚀 Организирайте проекта си",
    "Add board" : "Добави табло",
    "Card details" : "Подробности за картата",
    "Select the board to link to a project" : "Изберете таблото, което да свържете към проект",
    "Search by board title" : "Търсене по заглавие на таблото",
    "Select board" : "Избор на табло",
    "Select a board" : "Избор на табло",
    "No lists available" : "Няма налична списъци.",
    "Select a list" : "Избор на списък",
    "Move card" : "Преместване на карта",
    "Select the card to link to a project" : "Изберете картата, която да свържете към проект",
    "Link to card" : "Връзка към карта",
    "Select a card" : "Избор на карта",
    "Cancel" : "Отказ",
    "This board is read only" : "Таблото е само за четене",
    "Drop your files to upload" : "Пуснете вашите файлове, за да ги качите",
    "File already exists" : "Файлът вече съществува",
    "A file with the name {filename} already exists." : "Вече съществува файл с името {filename}.",
    "Do you want to overwrite it?" : "Искате ли да го презапишете?",
    "Overwrite file" : "Презапиши файла",
    "Keep existing file" : "Запазване на съществуващ файл",
    "Add card" : "Добави карта",
    "Archived cards" : "Архивирани карти",
    "Add list" : "Добави списък",
    "List name" : "Име на списък",
    "Clear search" : "Изчисти търсенето",
    "Active filters" : "Актижни филтри",
    "Apply filter" : "Приложи филтър",
    "Filter by tag" : "Филтрирай по маркер",
    "Filter by assigned user" : "Филтриране по назначен потребител",
    "Unassigned" : "Неприсвоен",
    "Open" : "Отвори",
    "Completed" : "Завършен",
    "Filter by due date" : "Филтриране по крайна дата",
    "Overdue" : "Просрочен",
    "Next 24 hours" : "Следващите 24 часа",
    "Next 7 days" : "Следващите 7 дни",
    "Next 30 days" : "Следващите 30 дни",
    "No due date" : "Няма крайна дата",
    "Clear filter" : "Изчстиване на филтър",
    "Hide archived cards" : "Скриване на архивирани карти",
    "Show archived cards" : "Показване на архивирани карти",
    "Toggle compact mode" : "Превключване на компактен режим",
    "Open details" : "Отваряне на подробности",
    "Details" : "Подробности",
    "Deck settings" : "Настройки на платформата",
    "General" : "Общи",
    "Use bigger card view" : "Използва по-голям изглед на картата",
    "Appearance" : "Изглед",
    "Show card ID badge" : "Показване на обозначение за самоличност на картата",
    "Show boards in calendar/tasks" : "Показване на таблата в календар / задачи",
    "Search" : "Търсене",
    "Show those shortcuts" : "Показване тези преки пътища",
    "Currently present people" : "Хора присъстващи в момента",
    "Loading board" : "Зареждане на табло",
    "Board not found" : "Таблото не е намерено",
    "Create a new list to add cards to this board" : "Създайте нов списък, за да добавите карти към това табло",
    "Sharing" : "Споделяне",
    "Tags" : "Етикети",
    "Deleted items" : "Изтрити елементи",
    "Activity" : "Активност",
    "Deleted lists" : "Изтрити списъци",
    "Undo" : "Отмяна",
    "Deleted cards" : "Изтрити карти",
    "Hours" : "Часа",
    "Weeks" : "Седмици",
    "Months" : "Месеци",
    "No cards yet" : "Все още няма карти",
    "Failed to create share with {displayName}" : "Създаването на споделяне с {displayName} не бе успешно",
    "Are you sure you want to transfer the board {title} to {user}?" : "Сигурни ли сте че искате да прехвърлите таблото {title} на {user}?",
    "Transfer the board." : "Прехвърлете таблото.",
    "Transfer" : "Прехвърляне",
    "The board has been transferred to {user}" : "Таблото беше прехвърлено на {user}",
    "Failed to transfer the board to {user}" : "Неуспешно прехвърляне на таблото на {user}",
    "Board owner" : "Собственик на табло",
    "(Group)" : "(Група)",
    "(remote)" : "(отдалечен)",
    "Can edit" : "Може да редактира",
    "Can share" : "Може да споделя",
    "Can manage" : "Може да управлява",
    "Owner" : "Собственик",
    "Delete" : "Изтриване",
    "List deleted" : "Списъкът е изтрит",
    "Edit list title" : "Редактиране на заглавието на списъка",
    "Archive all cards" : "Архивира всички карти",
    "Unarchive all cards" : "Разархивиране на всички карти",
    "Delete list" : "Изтрива списък",
    "Archive all cards in this list" : "Архивира всички карти в този списък",
    "Unarchive all cards in this list" : "Разархивиране всички карти в този списък",
    "Add a new card" : "Добави нова карта",
    "Card name" : "Име на карта",
    "title and color value must be provided" : "трябва да се предоставят заглавие и стойност на цвета",
    "Edit" : "Редакция",
    "Add a new tag" : "Добавяне на нов етикет",
    "Board name" : "Име на табло",
    "Members" : "Членове",
    "Assign a user to this card…" : "Зачисляване на потребител на тази карта ...",
    "File to share" : "Файл за споделяне",
    "Invalid path selected" : "Избран е невалиден път ",
    "Upload new files" : "Качи нови файлове",
    "Share from Files" : "Споделяне от Файлове",
    "Pending share" : "Чакащо споделяне",
    "Add this attachment" : "Добавете този прикачен файл",
    "Show in Files" : "Показване във файлове",
    "Download" : "Изтегляне",
    "Remove attachment" : "Премахване на прикачен файл",
    "Delete Attachment" : "Изтриване на прикачен файл",
    "Restore Attachment" : "Възстановяване на прикачен файл",
    "The title cannot be empty." : "Заглавието не може да бъде празно.",
    "Open in sidebar view" : "Отваряне в изгледа на страничната лента",
    "Open in bigger view" : "Отваря се в по-голям изглед",
    "Comments" : "Коментари",
    "Failed to load comments" : "Неуспешно зареждане на коментари",
    "No comments yet. Begin the discussion!" : "Все още няма коментари. Започнете дискусията!",
    "The comment cannot be empty." : "Коментарът не може да бъде празен.",
    "The comment cannot be longer than 1000 characters." : "Коментарът не може да бъде по-дълъг от 1000 знака.",
    "Submit" : "Изпращане",
    "Created:" : "Създадено:",
    "In reply to" : "В отговор на",
    "Cancel reply" : "Отказ на отговор",
    "Reply" : "Отговори",
    "Update" : "Обновяване",
    "Mark as done" : "Маркирай като готово",
    "Write a description …" : "Напишете описание ...",
    "Description" : "Описание",
    "(Unsaved)" : " (Незаписан)",
    "(Saving…)" : "(Записване...)",
    "Formatting help" : "Помощ за форматиране",
    "Edit description" : "Редактиране на описанието",
    "View description" : "Преглед на описанието",
    "Add Attachment" : "Добавяне на прикачен файл",
    "Choose attachment" : "Избор на прикачен файл",
    "Select Date" : "Изберете дата",
    "Later today – {timeLocale}" : "По-късно днес - {timeLocale}",
    "Tomorrow – {timeLocale}" : "Утре - {timeLocale}",
    "This weekend – {timeLocale}" : "Този уикенд - {timeLocale}",
    "Next week – {timeLocale}" : "Следващата седмица - {timeLocale}",
    "Set a due date" : "Задаване на крайна дата",
    "Remove due date" : "Премахни крайната дата",
    "Not done" : "Незавършен",
    "Unarchive card" : "Разархивиране на карта",
    "Archive card" : "Архивиране на  карта",
    "Assign a tag to this card…" : "Присвояване на етикет на тази карта ...",
    "(group)" : "(група)",
    "{count} comments, {unread} unread" : "{count} коментари, {unread} непрочетени",
    "Todo items" : "Елементи на задача за изпълнение",
    "Edit card title" : "Редактиране на заглавието на карта",
    "Card deleted" : "Картата е изтрита",
    "Edit title" : "Редактиране на заглавието",
    "Assign to me" : "Зачисляване към мен",
    "Unassign myself" : "Отмяна на зачисляването към мен",
    "Delete card" : "Изтриване на карта",
    "All boards" : "Всички табла",
    "Archived boards" : "Архивирани табла",
    "Shared with you" : "Споделени с вас",
    "Cancel edit" : "Отказ на редактиране",
    "Board {0} deleted" : "Табло {0} изтрито",
    "All cards" : "Всички карти",
    "Only assigned cards" : "Само зачислените карти",
    "No reminder" : "Няма напомняне",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Сигурни ли сте, че искате да изтриете таблото {title}? Това ще изтрие всички данни на това табло, включително архивираните карти.",
    "Delete the board?" : "Изтриване на таблото?",
    "An error occurred" : "Възникна грешка",
    "Board details" : "Подробности за табло",
    "Edit board" : "Редактиране на табло",
    "Clone board" : " Клониране на табло",
    "Unarchive board" : "Разархивиране натабло",
    "Archive board" : "Архивиране на табло",
    "Export board" : "Експортиране на табло",
    "Turn on due date reminders" : "Включва напомнянията за краен срок",
    "Turn off due date reminders" : "Изключва напомнянията за краен срок",
    "Due date reminders" : " Напомняния за краен срок",
    "Assigned cards" : "Зачислени карти",
    "No notifications" : "Няма известия",
    "Delete board" : "Изтриване на табло",
    "Clone cards" : " Клониране на карти",
    "Advanced options" : "Допълнителни опции",
    "Clone" : "Клониране",
    "Export as CSV" : "Експортиране като CSV файл",
    "Export" : "Експорт /изнасям/",
    "Loading filtered view" : "Зареждане на филтриран изглед",
    "Search for {searchQuery} in all boards" : "Търсене на {searchQuery} във всички табла",
    "No results found" : "Няма намерени резултати",
    "Deck board {name}\n* Last modified on {lastMod}" : "Табло {name}\n* Последна промяна на {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Създаден на {created}\n* Последна промяна на {lastMod} \n* {nbAttachments} прикачени файлове \n* {nbComments} коментара",
    "{nbCards} cards" : "{nbCards} карти",
    "Due on {date}" : "Краен срок до {date}",
    "{stack} in {board}" : "{stack} в {board}",
    "Click to expand description" : "Кликване за разширяване на описанието",
    "Click to expand comment" : "Кликване за разширяване на коментара",
    "Create card" : "Създаване на карта",
    "Create a new card" : "Създаване на нова карта",
    "Card title" : "Заглавие на карта",
    "Creating the new card …" : "Създаване на новата карта ...",
    "Card \"{card}\" was added to \"{board}\"" : "Карта \"{card}\" беше добавена към \"{board}\"",
    "Open card" : "Отваряне на карта",
    "Close" : "Затваряне",
    "No upcoming cards" : "Няма предстоящи карти",
    "upcoming cards" : "предстоящи карти",
    "New card" : "Нова карта",
    "Link to a board" : "Линк към табло",
    "Link to a card" : "Линк към карта",
    "Create a card" : "Създаване на карта",
    "Message from {author} in {conversationName}" : "Съобщение от {author} в {conversationName}",
    "Something went wrong" : "Нещо се обърка",
    "Failed to upload {name}" : " Неуспешно качване на {name}",
    "Maximum file size of {size} exceeded" : "Максималният размер на файла от {size} е надвишен",
    "Due date" : "Крайна дата",
    "Created" : "Създаден",
    "Modified" : "Промяна",
    "Error creating the share" : "Грешка при създаване на споделяне",
    "Share with a Deck card" : "Споделяне с Deck карта",
    "Share {file} with a Deck card" : "Споделяне {file} с Deck карта",
    "Share" : "Споделяне",
    "The card \"%s\" on \"%s\" has reached its due date." : "Картата \"%s\" on \"%s\" е достигнала датата на падежа си.",
    "No participants found" : "Няма намерени участници",
    "Save" : "Запазване",
    "seconds ago" : "преди секунди",
    "Today" : "Днес",
    "Tomorrow" : "Утре",
    "No due" : "Не се дължи"
},
"nplurals=2; plural=(n != 1);");
