OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Ustvarite novo zbirko {board}",
    "{user} has created a new board {board}" : "{user} ustvari novo zbirko {board}",
    "You have deleted the board {board}" : "Izbrišete zbirko {board}",
    "{user} has deleted the board {board}" : "{user} izbriše zbirko {board}",
    "You have restored the board {board}" : "Obnovite izbrisano zbirko {board}",
    "{user} has restored the board {board}" : "{user} obnovi izbrisano zbirko {board}",
    "You have shared the board {board} with {acl}" : "Omogočite souporabo zbirke {board} z uporabnikom {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} omogoči souporabo zbirke {board} z uporabnikom {acl}",
    "You have removed {acl} from the board {board}" : "Odstranite uporabnika {acl} z zbirke {board}",
    "{user} has removed {acl} from the board {board}" : "{user} odstrani uporabnika {acl} z zbirke {board}",
    "You have renamed the board {before} to {board}" : "Zbirko {before} preimenujete v {board}",
    "{user} has renamed the board {before} to {board}" : "{user} preimenuje zbirko {before} v {board}",
    "You have archived the board {board}" : "Arhivirate zbirko {board}",
    "{user} has archived the board {before}" : "{user} arhivira zbirko {board}",
    "You have unarchived the board {board}" : "Povrnete zbirko {board} iz arhiva",
    "{user} has unarchived the board {before}" : "{user} povrne zbirko {board} iz arhiva",
    "You have created a new list {stack} on board {board}" : "Ustvarite nov seznam nalog {stack} v zbirki {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} ustvari v zbirki {board} nov seznam nalog {stack}",
    "You have renamed list {before} to {stack} on board {board}" : "Preimenujete seznam nalog v zbirki {board} iz {before} v {stack}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} je preimenoval seznam nalog v zbirki {board} iz {before} v {stack}",
    "You have deleted list {stack} on board {board}" : "Izbrišete seznam nalog {stack} v zbirki {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} izbriše seznam nalog {stack} v zbirki {board}",
    "You have created card {card} in list {stack} on board {board}" : "Ustvarite nalogo {card} v seznamu nalog {stack} zbirke {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} ustvari nalogo {card} v seznamu nalog {stack} zbirke {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Izbrišete nalogo {card} v seznamu nalog {stack} zbirke {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} izbriše nalogo {card} v seznamu nalog {stack} zbirke {board}",
    "You have renamed the card {before} to {card}" : "Preimenujete nalogo {before} v {card}",
    "{user} has renamed the card {before} to {card}" : "{user} preimenuje nalogo {before} v {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Dodate opis k nalogi {card} v seznamu nalog {stack} zbirke {board}.",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} doda opis k nalogi {card} v seznamu nalog {stack} zbirke {board}.",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Posodobite opis naloge {card} v seznamu nalog {stack} zbirke {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} posodobi opis naloge {card} v seznamu nalog {stack} zbirke {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Arhivirate nalogo {card} v seznamu nalog {stack} zbirke {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} arhivira nalogo {card} v seznamu nalog {stack} zbirke {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Iz arhiva povrnete nalogo {card} v seznamu nalog {stack} zbirke {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} iz arhiva povrne nalogo {card} v seznamu nalog {stack} zbirke {board}",
    "You have removed the due date of card {card}" : "Odstranite datum preteka z naloge {card}",
    "{user} has removed the due date of card {card}" : "{user} odstrani datum preteka z naloge {card}",
    "You have set the due date of card {card} to {after}" : "Nastavite datum preteka naloge {card} na {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} nastavi datum preteka naloge {card} na {after}",
    "You have updated the due date of card {card} to {after}" : "Posodobite datum preteka naloge {card} na {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} posodobi datum preteka naloge {card} na {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Dodate oznako {label} k nalogi {card} seznama nalog {stack} v zbirki {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} doda oznako {label} k nalogi {card} seznama nalog {stack} v zbirki {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Odstranite oznako {label} z naloge {card} seznama nalog {stack} v zbirki {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} odstrani oznako {label} z naloge {card} seznama nalog {stack} v zbirki {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Dodelite nalogo {card} v zbirki {board} → {assigneduser}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} dodeli nalogo {card} v zbirki {board} → {assigneduser}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Odvzamete nalogo {card} v zbirki {board} ✕ {assigneduser}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} odvzame nalogo {card} v zbirki {board} ✕ {assigneduser}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Premaknete nalogo {card} s seznama nalog {stackBefore} v {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} premakne nalogo {card} s seznama nalog {stackBefore} v {stack}",
    "You have added the attachment {attachment} to card {card}" : "Dodate prilogo {attachment} k nalogi {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} doda prilogo {attachment} k nalogi {card}",
    "You have updated the attachment {attachment} on card {card}" : "Posodobite prilogo {attachment} v nalogi {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} posodobi prilogo {attachment} v nalogi {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Izbišete prilogo {attachment} v nalogi {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} izbriše prilogo {attachment} iz naloge {card}",
    "You have restored the attachment {attachment} to card {card}" : "Obnovite prilogo {attachment} k nalogi {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} obnovi prilogo {attachment} naloge {card}",
    "You have commented on card {card}" : "Vpišete opombo k nalogi {card}",
    "{user} has commented on card {card}" : "{user} vpiše opombo k nalogi {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Spremembe v programu <strong>Deck</strong>",
    "A <strong>comment</strong> was created on a card" : "Nalogi je dodana nova <strong>opomba</strong>",
    "The file was uploaded" : "Datoteka je uspešno poslana.",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Poslana datoteka presega določilo največje dovoljene velikosti v datoteki php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Poslana datoteka presega velikost, ki je določena kot omejitev MAX_FILE_SIZE v zapisu HTML.",
    "The file was only partially uploaded" : "Datoteka ni bila poslana v celoti",
    "No file was uploaded" : "Datoteka ni bila poslana",
    "Missing a temporary folder" : "Manjka začasna mapa",
    "Could not write file to disk" : "Ni mogoče zapisati datoteke na disk",
    "A PHP extension stopped the file upload" : "Razširitev PHP onemogoča pošiljanje datoteke",
    "No file uploaded or file size exceeds maximum of %s" : "Ali ni poslane datoteke ali pa velikost datoteke presega omejitev %s",
    "Upcoming cards" : "Prihajajoče naloge",
    "Load more" : "Naloži več",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Nalogo »%s« na »%s« vam dodeli %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} vam dodeli nalogo »{deck-card}« na »{deck-board}«.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Naloga »{deck-card} « na »{deck-board}« je dosegla datum preteka.",
    "%s has mentioned you in a comment on \"%s\"." : "%s vas omeni v opombi na »%s«.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} vas omeni v opombi na »{deck-card}«.",
    "The board \"%s\" has been shared with you by %s." : "Zbirko »%s« vam dodeli %s.",
    "{user} has shared {deck-board} with you." : "{user} vam omogoča souporabo zbirke {deck-board}.",
    "Deck board" : "Zbirka Deck",
    "Card comments" : "Opombe k nalogi",
    "%s on %s" : "%s v nalogi %s",
    "No data was provided to create an attachment." : "Ni podanih podatkov za ustvarjanje priloge.",
    "Finished" : "Končano",
    "To review" : "Za pregled",
    "Action needed" : "Čaka na odziv",
    "Later" : "Kasneje",
    "copy" : "kopija",
    "To Do" : "Naloge",
    "Done" : "Končane naloge",
    "Attachments" : "Priloge",
    "File" : "Datoteka",
    "Card not found" : "Naloge ni mogoče najti",
    "Path is already shared with this card" : "Pot je že povezana s to nalogo",
    "Invalid date, date format must be YYYY-MM-DD" : "Neveljaven zapis časa; biti mora v zapisu YYYY-MM-DD",
    "Personal planning and team project organization" : "Osebno načrtovanje nalog in organizacija skupinskih projektov",
    "Add board" : "Dodaj zbirko",
    "Card details" : "Podrobnosti naloge",
    "Select the board to link to a project" : "Izberite zbirko za povezavo s projektom",
    "Search by board title" : "Išči po imenu zbirke",
    "Select board" : "Izbor zbirke",
    "Select a board" : "Izbor zbirke",
    "No lists available" : "Ni razpoložljivih seznamov",
    "Select a list" : "Izbor seznama",
    "Move card" : "Premakni nalogo",
    "Select the card to link to a project" : "Izbor naloge za povezavo do projekta",
    "Link to card" : "Poveži nalogo",
    "Select a card" : "Izbor naloge",
    "Cancel" : "Prekliči",
    "This board is read only" : "Zbirka je označena le za branje",
    "Drop your files to upload" : "Spustite datoteke za pošiljanje v oblak",
    "File already exists" : "Datoteka s tem imenom že obstaja.",
    "A file with the name {filename} already exists." : "Datoteka z imenom {filename} že obstaja.",
    "Do you want to overwrite it?" : "Ali želite predmet prepisati?",
    "Overwrite file" : "Prepiši datoteko",
    "Keep existing file" : "Ohrani obstoječe datoteke",
    "Add card" : "Dodaj nalogo",
    "Archived cards" : "Arhivirane naloge",
    "Add list" : "Dodaj seznam",
    "List name" : "Ime seznama",
    "Clear search" : "Počisti iskanje",
    "Active filters" : "Dejavni filtri",
    "Apply filter" : "Uveljavi filter",
    "Filter by tag" : "Filtriraj po oznaki",
    "Filter by assigned user" : "Filtriraj po dodeljenem uporabniku",
    "Unassigned" : "Nedodeljeno",
    "Filter by status" : "Filtri po stanju",
    "Open and completed" : "Odprto in končano",
    "Open" : "Odpri",
    "Completed" : "Končano",
    "Filter by due date" : "Filtriraj po datumu preteka",
    "Overdue" : "Preteklo",
    "Next 24 hours" : "Naslednjih 24 ur",
    "Next 7 days" : "Naslednjih 7 dni",
    "Next 30 days" : "Naslednjih 30 dni",
    "No due date" : "Ni datuma preteka",
    "Clear filter" : "Počisti filter",
    "View Modes" : "Načini pogleda",
    "Toggle View Modes" : "Preklopi načine pogleda",
    "Hide archived cards" : "Skrij arhivirane naloge",
    "Show archived cards" : "Prikaži arhivirane naloge",
    "Toggle compact mode" : "Preklopi način prikaza",
    "Open details" : "Odpri podrobnosti",
    "Details" : "Podrobnosti",
    "General" : "Splošno",
    "Use bigger card view" : "Uporabi povečan pogled nalog",
    "Appearance" : "Videz",
    "Show boards in calendar/tasks" : "Pokaži zbirke v koledarjem in med opravili",
    "Scroll sideways" : "Zdrsni bočno",
    "Navigate between cards" : "Krmarjenje med nalogami",
    "Close card details" : "Zapri podrobnosti kartice",
    "Search" : "Poišči",
    "Show card filters" : "Pokaži filtre kartice",
    "Clear card filters" : "Počisti filtre kartice",
    "Show those shortcuts" : "Pokaži tipkovne bližnjice",
    "Card actions" : "Dejanja naloge",
    "Open card details" : "Odpri podrobnosti naloge",
    "Edit the card title" : "Uredi naslov naloge",
    "Loading board" : "Poteka nalaganje zbirke",
    "Board not found" : "Zbirke ni mogoče najti",
    "Create a new list to add cards to this board" : "Ustvari nov seznam in doda kartice v zbirko",
    "Sharing" : "Souporaba",
    "Tags" : "Oznake",
    "Deleted items" : "Izbrisani predmeti",
    "Activity" : "Dejavnost",
    "Deleted lists" : "Izbrisani seznami",
    "Undo" : "Razveljavi",
    "Deleted cards" : "Izbrisane naloge",
    "Hours" : "Ure",
    "Days" : "Dnevi",
    "Weeks" : "tednov",
    "Months" : "mesecev",
    "No cards yet" : "Ni še vpisanih nalog",
    "Failed to create share with {displayName}" : "Souporaba z uporabnikom {displayName} je spodletela",
    "Transfer" : "Prenos",
    "Board owner" : "Lastnik zbirke",
    "(Group)" : "(Skupina)",
    "(remote)" : "(oddaljeno) ",
    "Can edit" : "Lahko ureja",
    "Can share" : "Lahko omogoči souporabo",
    "Can manage" : "Lahko upravlja",
    "Owner" : "Lastnik",
    "Delete" : "Izbriši",
    "List deleted" : "Seznam je izbrisan",
    "Archive all cards" : "Arhiviraj vse kartice",
    "Delete list" : "Izbriši seznam",
    "Archive all cards in this list" : "Arhiviraj vse kartice tega seznama",
    "Add a new card" : "Dodaj novo nalogo",
    "Card name" : "Ime kartice",
    "title and color value must be provided" : "navedena morata biti podatka naslova in barve",
    "Edit" : "Uredi",
    "Add a new tag" : "Dodaj novo oznako",
    "Board name" : "Ime zbirke",
    "Members" : "Člani",
    "Assign a user to this card…" : "Dodeli uporabnika k nalogi ...",
    "File to share" : "Datoteka za souporabo",
    "Invalid path selected" : "Izbrana je neveljavna pot",
    "Upload new files" : "Pošlji nove datoteke",
    "Share from Files" : "Poveži iz oblaka",
    "Add this attachment" : "Dodaj prilogo",
    "Show in Files" : "Poveži iz oblaka",
    "Download" : "Prejmi",
    "Remove attachment" : "Odstrani prilogo",
    "Delete Attachment" : "Izbriši prilogo",
    "Restore Attachment" : "Obnovi prilogo",
    "The title cannot be empty." : "Polje naslova ne sme biti prazno.",
    "Open in sidebar view" : "Odpri v bočnem pogledu",
    "Open in bigger view" : "Odpri v povečanem pogledu",
    "Comments" : "Opombe",
    "Failed to load comments" : "Nalaganje opomb je spodletelo.",
    "No comments yet. Begin the discussion!" : "Ni še odzivov, bodite prvi!",
    "The comment cannot be empty." : "Polje opombe ne sme biti prazno.",
    "The comment cannot be longer than 1000 characters." : "Opomba ne sme biti daljša od 1000 znakov.",
    "Submit" : "Pošlji",
    "Created:" : "Ustvarjeno:",
    "In reply to" : "V odgovor",
    "Cancel reply" : "Prekliči odgovor",
    "Reply" : "Odgovori",
    "Update" : "Posodobi",
    "Mark as done" : "Označi kot končano",
    "Mark as not done" : "Označi kot Ni končano",
    "Write a description …" : "Vpišite opis ...",
    "Description" : "Opis",
    "(Unsaved)" : "(neshranjeno)",
    "(Saving…)" : "(shranjevanje ...)",
    "Formatting help" : "Pomoč pri oblikovanju",
    "Edit description" : "Uredi opis",
    "View description" : "Pokaži opis",
    "Add Attachment" : "Dodaj prilogo",
    "Choose attachment" : "Izbor priloge",
    "Select Date" : "Izbor datuma",
    "Later today – {timeLocale}" : "Danes – {timeLocale}",
    "Set due date for later today" : "Nastavi datum preteka na danes kasneje",
    "Tomorrow – {timeLocale}" : "Jutri – {timeLocale}",
    "Set due date for tomorrow" : "Nastavi datum preteka na jutri",
    "This weekend – {timeLocale}" : "Ta teden – {timeLocale}",
    "Set due date for this weekend" : "Nastavi datum preteka na ta teden",
    "Next week – {timeLocale}" : "Naslednji teden – {timeLocale}",
    "Set due date for next week" : "Nastavi datum preteka na naslednji teden",
    "Set a due date" : "Nastavi datum preteka",
    "Add due date" : "Dodaj datum preteka",
    "Choose a date" : "Izbor datuma",
    "Remove due date" : "Odstrani datum preteka",
    "Due at:" : "Poteče:",
    "Not done" : "Ni končano",
    "Unarchive card" : "Povrni nalogo",
    "Archive card" : "Arhiviraj nalogo",
    "Assign a tag to this card…" : "Dodeli oznako nalogi ...",
    "Select or create a tag…" : "Izberi ali ustvari oznako ...",
    "Create a new tag:" : "Ustvari novo oznako:",
    "(group)" : "(skupina)",
    "{count} comments, {unread} unread" : "{count} sporočil, {unread} neprebranih",
    "Todo items" : "Naloge",
    "Edit card title" : "Uredi naslov naloge",
    "Card deleted" : "Naloga je izbrisana",
    "Edit title" : "Uredi naslov",
    "Assign to me" : "Nalogo dodeli meni",
    "Unassign myself" : "Odstrani me z naloge",
    "Delete card" : "Izbriši nalogo",
    "All boards" : "Vse zbirke",
    "Archived boards" : "Arhivirane zbirke",
    "Shared with you" : "V souporabi z vami",
    "Cancel edit" : "Prekliči urejanje",
    "Board {0} deleted" : "Zbirka {0} je izbrisana",
    "All cards" : "Vse naloge",
    "Only assigned cards" : "Le dodeljene naloge",
    "No reminder" : "Ni vpisanih opomnikov",
    "Delete the board?" : "Ali želite izbrisati zbirko?",
    "An error occurred" : "Prišlo je do napake.",
    "Board details" : "Podrobnosti zbirke",
    "Edit board" : "Uredi zbirko",
    "Clone board" : "Kloniraj zbirko",
    "Unarchive board" : "Povrni zbirko",
    "Archive board" : "Arhiviraj zbirko",
    "Turn on due date reminders" : "Omogoči opomnike o datumu zapadlosti ",
    "Turn off due date reminders" : "Onemogoči opomnike o datumu zapadlosti",
    "Due date reminders" : "Opomniki o datumu zapadlosti",
    "Assigned cards" : "Dodeljene naloge",
    "No notifications" : "Ni obvestil",
    "Delete board" : "Izbriši zbirko",
    "Clone cards" : "Kloniraj nalogo",
    "Advanced options" : "Napredne možnosti",
    "Clone" : "Kloniraj",
    "Export as CSV" : "Izvozi v zapis CSV",
    "Export" : "Izvozi",
    "Loading filtered view" : "Poteka nalaganje filtriranega pogleda",
    "Search for {searchQuery} in all boards" : "Poišči {searchQuery} med vsemi zbirkami nalog.",
    "No results found" : "Ni najdenih zadetkov",
    "Due on {date}" : "Poteče {date}",
    "{stack} in {board}" : "{stack} v {board}",
    "Click to expand comment" : "Kliknite za razširitev opombe",
    "Create card" : "Ustvari nalogo",
    "Create a new card" : "Ustvari novo nalogo",
    "Card title" : "Naslov naloge",
    "Creating the new card …" : "Poteka ustvarjanje nove naloge ...",
    "Card \"{card}\" was added to \"{board}\"" : "Naloga »{card}« je dodana v zbirko »{board}«.",
    "Open card" : "Odpri nalogo",
    "Close" : "Zapri",
    "No upcoming cards" : "Ni prihajajočih nalog",
    "upcoming cards" : "prihajajoče naloge",
    "New card" : "Nova naloga",
    "Link to a board" : "Povezava do zbirke",
    "Link to a card" : "Povezava do naloge",
    "Create a card" : "Ustvari nalogo",
    "Message from {author} in {conversationName}" : "{author} objavi sporočilo v pogovoru {conversationName}",
    "Something went wrong" : "Prišlo je do napake ...",
    "Failed to upload {name}" : "Pošiljanje {name} je spodletelo",
    "Maximum file size of {size} exceeded" : "Omejitev velikosti datoteke {size} je prekoračena.",
    "Due date" : "Datum preteka",
    "Created" : "Ustvarjeno",
    "Modified" : "Spremenjeno",
    "Error creating the share" : "Napaka ustvarjanja mesta souporabe",
    "Share with a Deck card" : "Poveži z nalogo Deck",
    "Share {file} with a Deck card" : "Poveži datoteko {file} z nalogo Deck",
    "Share" : "Souporaba",
    "The card \"%s\" on \"%s\" has reached its due date." : "Naloga »%s« na »%s« je dosegla datum preteka.",
    "No participants found" : "Ni zaznanih udeleženev",
    "Save" : "Shrani",
    "seconds ago" : "pred nekaj sekundami",
    "Today" : "Danes",
    "Tomorrow" : "Jutri",
    "No due" : "Ni datuma preteka"
},
"nplurals=4; plural=(n%100==1 ? 0 : n%100==2 ? 1 : n%100==3 || n%100==4 ? 2 : 3);");
