OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Loit uuden taulun {board}",
    "{user} has created a new board {board}" : "{user} loi uuden taulun {board}",
    "You have deleted the board {board}" : "Poistit taulun {board}",
    "{user} has deleted the board {board}" : "{user} poisti taulun {board}",
    "You have restored the board {board}" : "Palautit taulun {board}",
    "{user} has restored the board {board}" : "{user} palautti taulun {board}",
    "You have shared the board {board} with {acl}" : "Jaoit taulun {board} ryhmälle {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} jakoi taulun {board} ryhmälle {acl}",
    "You have removed {acl} from the board {board}" : "Poistit ryhmän {acl} taulun {board} jaosta",
    "{user} has removed {acl} from the board {board}" : "{user} poisti {acl}-ryhmän taululta {board}",
    "You have renamed the board {before} to {board}" : "Asetit taulun {before} uudeksi nimeksi {board}",
    "{user} has renamed the board {before} to {board}" : "{user} asetti taulun {before} uudeksi nimeksi {board}",
    "You have archived the board {board}" : "Arkistoit taulun {board}",
    "{user} has archived the board {before}" : "{user} arkistoi taulun {before}",
    "You have unarchived the board {board}" : "Palautit taulun {board} arkistosta ",
    "{user} has unarchived the board {before}" : "{user} palautti taulun {before} arkistosta",
    "You have created a new list {stack} on board {board}" : "Olet lisännyt listan {stack} taululle {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} on lisännyt uuden listan {stack} taululle {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Muutit listan {before} nimeksi {stack} taululla {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} muutti listan {before} nimeksi {stack} taululla {board}",
    "You have deleted list {stack} on board {board}" : "Olet poistanut listan {stack} taululta {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} on poistanut listan {stack} taululta {board}",
    "You have created card {card} in list {stack} on board {board}" : "Olet lisännyt kortin {card} listaan {stack} taululla {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} on lisännyt kortin {card} listaan {stack} taululla {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Olet poistanut kortin {card} listasta {stack} taululla {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} on poistanut kortin {card} listasta {stack} taululla {board}",
    "You have renamed the card {before} to {card}" : "Muutit kortin {before} uudeksi nimeksi {card}",
    "{user} has renamed the card {before} to {card}" : "{user} muutti kortin {before} uudeksi nimeksi {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Olet lisännyt kuvauksen kortille {card} listalla {stack} taululla {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} on lisännyt kuvauksen kortille {card} listalla {stack} taululla {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Olet päivittänyt kortin {card} kuvauksen listalla {stack} taululla {board}",
    "You have removed the due date of card {card}" : "Poistit eräpäivän kortilta {card}",
    "{user} has removed the due date of card {card}" : "{user} poisti eräpäivän kortilta {card}",
    "You have set the due date of card {card} to {after}" : "Asetit kortille {card} eräpäivän {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} asetti kortille {card} eräpäivän {after}",
    "You have updated the due date of card {card} to {after}" : "Päivitit kortin {card} eräpäiväksi {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} päivitti kortin {card} eräpäiväksi {after}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Asetit kortin {card} käyttäjän {assigneduser} tehtäväksi taululla {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} asetti kortin {card} käyttäjän {assigneduser} tehtäväksi taululla {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Poistit kortin {card} käyttäjän {assigneduser} tehtävistä taululla {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} poisti kortin {card} käyttäjän {assigneduser} tehtävistä taululla {board}",
    "You have added the attachment {attachment} to card {card}" : "Lisäsit liitteen {attachment} kortille {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} lisäsi liitteen {attachment} kortille {card}",
    "You have updated the attachment {attachment} on card {card}" : "Päivitit kortin {card} liitettä {attachment}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} päivitti kortin {card} liitteen {attachment}",
    "You have deleted the attachment {attachment} from card {card}" : "Poistit liitteen {attachment} kortilta {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} poisti liitteen {attachment} kortilta {card}",
    "You have restored the attachment {attachment} to card {card}" : "Palautit liitteen {attachment} kortille {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} palautti liitteen {attachment} kortille {card}",
    "You have commented on card {card}" : "Kommentoit korttia {card}",
    "{user} has commented on card {card}" : "{user} kommentoi korttia {card}",
    "Deck" : "Pakka",
    "Changes in the <strong>Deck app</strong>" : "Muutokset <strong>Pakka-sovelluksessa</strong>",
    "A <strong>comment</strong> was created on a card" : "<strong>Kommentti</strong> luotiin kortille",
    "The file was uploaded" : "Tiedosto lähetettiin",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Lähetetty tiedosto ylittää php.ini-tiedostossa määritellyn upload_max_filesize -säännön ",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Lähetettävän tiedoston enimmäiskoko ylittää HTML-lomakkeessa määritellyn MAX_FILE_SIZE-säännön",
    "The file was only partially uploaded" : "Tiedosto lähetettiin vain osittain",
    "No file was uploaded" : "Tiedostoa ei lähetetty",
    "Missing a temporary folder" : "Väliaikaistiedostojen kansio puuttuu",
    "Could not write file to disk" : "Tiedostoa ei voitu kirjoittaa levylle",
    "A PHP extension stopped the file upload" : "PHP:n laajennus esti tiedoston lähettämisen",
    "No file uploaded or file size exceeds maximum of %s" : "Tiedostoa ei lähetetty tai sen koko ylitti maksimin %s",
    "Upcoming cards" : "Tulevat kortit",
    "Load more" : "Lataa lisää",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Kortti \"%s\" taululla \"%s\" on asetettu sinulle käyttäjän %s toimesta.",
    "%s has mentioned you in a comment on \"%s\"." : "%s mainitsi sinut kommentissa kortilla \"%s\".",
    "The board \"%s\" has been shared with you by %s." : "Taulu \"%s\" on jaettu kanssasi käyttäjän %s toimesta.",
    "{user} has shared {deck-board} with you." : "{user} on jakanut taulun {deck-board} kanssasi.",
    "Deck board" : "Deck-taulu",
    "Create a new deck card" : "Luo uusi kortti",
    "Card comments" : "Kortin kommentit",
    "Finished" : "Valmistunut",
    "To review" : "Arvosteltavana",
    "Action needed" : "Toimia vaaditaan",
    "Later" : "Myöhemmin",
    "copy" : "kopioi",
    "To Do" : "Tehtävä",
    "Done" : "Valmis",
    "Attachments" : "Liitteet",
    "File" : "Tiedosto",
    "Card not found" : "Korttia ei löydy",
    "Path is already shared with this card" : "Polku on jo jaettu tämän kortin kanssa",
    "Invalid date, date format must be YYYY-MM-DD" : "Virheellinen päivä, päivän muodon tulee olla YYYY-MM-DD",
    "Add board" : "Lisää taulu",
    "Card details" : "Näytä kortin sisältö",
    "Select the board to link to a project" : "Valitse projektiin linkitettävä taulu",
    "Search by board title" : "Hae taulun otsikon mukaan",
    "Select board" : "Valitse taulu",
    "Select a board" : "Valitse taulu",
    "No lists available" : "Ei listoja saatavilla",
    "Select a list" : "Valitse lista ",
    "Move card" : "Siirrä kortti",
    "Select the card to link to a project" : "Valitse projektiin linkitettävä kortti",
    "Link to card" : "Linkitä korttiin",
    "Select a card" : "Valitse kortti",
    "Cancel" : "Peruuta",
    "This board is read only" : "Tämä taulu on kirjoitussuojattu",
    "Drop your files to upload" : "Pudota tiedostot lähettääksesi ne",
    "File already exists" : "Tiedosto on jo olemassa",
    "A file with the name {filename} already exists." : "Tiedosto nimellä {filename} on jo olemassa.",
    "Do you want to overwrite it?" : "Haluatko ylikirjoittaa sen?",
    "Overwrite file" : "Päällekirjoita tiedosto",
    "Keep existing file" : "Pidä nykyinen tiedosto",
    "Add card" : "Lisää kortti",
    "Archived cards" : "Arkistoidut kortit",
    "Add list" : "Lisää lista",
    "List name" : "Listan nimi",
    "Clear search" : "Tyhjennä haku",
    "Active filters" : "Aktiiviset suodattimet",
    "Apply filter" : "Toteuta suodatus",
    "Filter by tag" : "Suodata tunnisteen perusteella",
    "Filter by assigned user" : "Suodata määritetyn käyttäjän perusteella",
    "Unassigned" : "Määrittämätön",
    "Filter by status" : "Suodata tilan perusteella",
    "Open and completed" : "Avoimet ja valmiit",
    "Open" : "Avoimet",
    "Completed" : "Valmiit",
    "Filter by due date" : "Suodata määräpäivän perusteella",
    "Overdue" : "Myöhässä",
    "Next 24 hours" : "Seuraavat 24 tuntia",
    "Next 7 days" : "Seuraavat 7 päivää",
    "Next 30 days" : "Seuraavat 30 päivää",
    "No due date" : "Ei eräpäivää",
    "Clear filter" : "Tyhjennä suodatin",
    "Hide archived cards" : "Piilota arkistoidut kortit",
    "Show archived cards" : "Näytä arkistoidut kortit",
    "Toggle compact mode" : "Käytä kompaktia tilaa",
    "Hide card cover images" : "Piilota korttien kansikuvat",
    "Show card cover images" : "Näytä korttien kansikuvat",
    "Open details" : "Avaa yksityiskohdat",
    "Details" : "Tiedot",
    "Deck settings" : "Pakan asetukset",
    "General" : "Yleiset",
    "Appearance" : "Ulkoasu",
    "Show boards in calendar/tasks" : "Näytä taulut kalenterissa ja tehtävissä",
    "Search" : "Etsi",
    "Show those shortcuts" : "Näytä pikanäppäimet",
    "Mark card as completed/not completed" : "Merkitse kortti valmiiksi/avoimeksi",
    "Loading board" : "Ladataan taulua",
    "Board not found" : "Taulua ei löydy",
    "Create a new list to add cards to this board" : "Lisää uusi lista lisätäksesi kortteja tälle taululle",
    "Sharing" : "Jakaminen",
    "Tags" : "Tunnisteet",
    "Deleted items" : "Poistetut tietueet",
    "Activity" : "Tapahtumat",
    "Deleted lists" : "Poistetut listat",
    "Undo" : "Kumoa",
    "Deleted cards" : "Poistetut kortit",
    "Hours" : "Tunnit",
    "Days" : "Päivää",
    "Weeks" : "Viikkoa",
    "Months" : "Kuukautta",
    "No cards yet" : "Ei kortteja vielä",
    "Failed to create share with {displayName}" : "Jaon luonti epäonnistui kohteen {displayName} kanssa",
    "Are you sure you want to transfer the board {title} to {user}?" : "Haluatko varmasti siirtää taulun {title} käyttäjälle {user}?",
    "Transfer" : "Siirrä",
    "Board owner" : "Taulun omistaja",
    "(Group)" : "(Ryhmä)",
    "(remote)" : "(etä)",
    "Can edit" : "Voi muokata",
    "Can share" : "Voi jakaa",
    "Can manage" : "Voi hallita",
    "Owner" : "Omistaja",
    "Delete" : "Poista",
    "List deleted" : "Lista poistettu",
    "Archive all cards" : "Arkistoi kaikki kortit",
    "Delete list" : "Poista lista",
    "Archive all cards in this list" : "Arkistoi kaikki kortit tässä listassa",
    "Add a new card" : "Lisää uusi kortti",
    "Card name" : "Kortin nimi",
    "title and color value must be provided" : "tunnisteella on oltava nimi ja väri",
    "Edit" : "Muokkaa",
    "Add a new tag" : "Lisää uusi tunniste",
    "Board name" : "Taulun nimi",
    "Members" : "Jäsenet",
    "File to share" : "Jaettava tiedosto",
    "Invalid path selected" : "Virheellinen polku valittu",
    "Upload new files" : "Lähetä uusia tiedostoja",
    "Share from Files" : "Jaa tiedostoista",
    "Pending share" : "Odottava jako",
    "Add this attachment" : "Lisää tämä liite",
    "Download" : "Lataa",
    "Remove attachment" : "Poista liite",
    "Delete Attachment" : "Poista liite",
    "Restore Attachment" : "Palauta liite",
    "The title cannot be empty." : "Otsikko ei voi olla tyhjä.",
    "Open in sidebar view" : "Avaa sivupalkkinäkymään",
    "Comments" : "Kommentit",
    "Failed to load comments" : "Kommenttien lataaminen epäonnistui",
    "No comments yet. Begin the discussion!" : "Ei vielä kommentteja. Aloita keskustelu!",
    "The comment cannot be empty." : "Kommentti ei voi olla tyhjä.",
    "The comment cannot be longer than 1000 characters." : "Kommentin on oltava alle 1000 merkkiä pitkä.",
    "Submit" : "Lähetä",
    "Created:" : "Luotu:",
    "In reply to" : "Vastauksena",
    "Cancel reply" : "Peru vastaus",
    "Reply" : "Vastaa",
    "Update" : "Päivitä",
    "Mark as done" : "Merkitse valmiiksi",
    "Write a description …" : "Kirjoita kuvaus…",
    "Description" : "Kuvaus",
    "(Unsaved)" : "(Tallentamaton)",
    "(Saving…)" : "(Tallennetaan…)",
    "Formatting help" : "Muotoiluapu",
    "Edit description" : "Muokkaa kuvausta",
    "View description" : "Näytä kuvaus",
    "Add Attachment" : "Lisää liite",
    "Choose attachment" : "Valitse liite",
    "Select Date" : "Valitse päivä",
    "Later today – {timeLocale}" : "Myöhemmin tänään – {timeLocale}",
    "Tomorrow – {timeLocale}" : "Huomenna – {timeLocale}",
    "This weekend – {timeLocale}" : "Tämä viikonloppu – {timeLocale}",
    "Next week – {timeLocale}" : "Ensi viikko – {timeLocale}",
    "Set a due date" : "Aseta eräpäivä",
    "Choose a date" : "Valitse päivämäärä",
    "Remove due date" : "Poista eräpäivä",
    "Unarchive card" : "Poista kortti arkistosta",
    "Archive card" : "Arkistoi kortti",
    "Assign a tag to this card…" : "Lisää kortille tunniste...",
    "Create a new tag:" : "Luo uusi tunniste:",
    "(group)" : "(ryhmä)",
    "{count} comments, {unread} unread" : "{count} kommenttia, {unread} lukematonta",
    "Edit card title" : "Muokkaa kortin otsikkoa",
    "Open link" : "Avaa linkki",
    "Card deleted" : "Kortti poistettu",
    "Edit title" : "Muokkaa otsikkoa",
    "Assign to me" : "Määritä minulle",
    "Unassign myself" : "Poista määritys minulta",
    "Delete card" : "Poista kortti",
    "All boards" : "Kaikki taulut",
    "Archived boards" : "Arkistoidut taulut",
    "Shared with you" : "Jaettu kanssasi",
    "Cancel edit" : "Peruuta muokkaus",
    "Board {0} deleted" : "Taulu {0} poistettu",
    "All cards" : "Kaikki kortit",
    "No reminder" : "Ei muistutusta",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Haluatko varmasti poistaa taulun {title}? Tämä poistaa kaikki taulun tiedot, mukaan lukien arkistoidut kortit.",
    "Delete the board?" : "Poistetaanko tämä taulu?",
    "Leave" : "Poistu",
    "An error occurred" : "Tapahtui virhe",
    "Board details" : "Taulun tiedot",
    "Edit board" : "Muokkaa taulua",
    "Clone board" : "Monista taulu",
    "Unarchive board" : "Kumoa taulun arkistointi",
    "Archive board" : "Arkistoi taulu",
    "Export board" : "Vie taulu",
    "No notifications" : "Ei ilmoituksia",
    "Delete board" : "Poista taulu",
    "Clone cards" : "Kloonaa kortteja",
    "Advanced options" : "Lisävalinnat",
    "Clone" : "Kloonaa",
    "Export as CSV" : "Vie CSV:nä",
    "Export" : "Vie",
    "Search for {searchQuery} in all boards" : "Hae ehtoja {searchQuery} kaikilta tauluilta",
    "No results found" : "Ei tuloksia",
    "{nbCards} cards" : "{nbCards} korttia    ",
    "Due on {date}" : "Määräpäivä {date}",
    "Click to expand description" : "Laajenna kuvausta ",
    "Click to expand comment" : "Napsauta laajentaaksesi kommentin",
    "Create card" : "Luo kortti",
    "Create a new card" : "Luo uusi kortti",
    "Card title" : "Kortin otsikko",
    "Creating the new card …" : "Luodaan uutta korttia…",
    "Card \"{card}\" was added to \"{board}\"" : "Kortti \"{card}\" lisättiin taululle \"{board}\"",
    "Open card" : "Avaa kortti",
    "Close" : "Sulje",
    "No upcoming cards" : "Ei tulevia kortteja",
    "upcoming cards" : "tulevat kortit",
    "New card" : "Uusi kortti",
    "Link to a board" : "Linkki taululle",
    "Link to a card" : "Linkitä korttiin",
    "Create a card" : "Luo kortti",
    "Something went wrong" : "Jokin meni vikaan",
    "Failed to upload {name}" : "Ei voitu lähettää {name}",
    "Maximum file size of {size} exceeded" : "Tiedoston enimmäiskoko {size} ylitetty",
    "Due date" : "Eräpäivä",
    "Created" : "Luotu",
    "Modified" : "Muokattu",
    "Error creating the share" : "Virhe jakoa luotaessa",
    "Share" : "Jaa",
    "The card \"%s\" on \"%s\" has reached its due date." : "Kortin \"%s\" on \"%s\" eräpäivä on tullut vastaan.",
    "Share board with a user, group or team …" : "Jaa taulu käyttäjän, ryhmän tai tiimin kanssa...",
    "No participants found" : "Ei osallistujia löydetty",
    "Save" : "Tallenna",
    "seconds ago" : "sekuntia sitten",
    "Today" : "Tänään",
    "Tomorrow" : "Huomenna"
},
"nplurals=2; plural=(n != 1);");
