OC.L10N.register(
    "deck",
    {
    "You have renamed the board {before} to {board}" : "Tu pārdēvēji dēli {before} par {board}",
    "{user} has renamed the board {before} to {board}" : "{user} pārdēvēja dēli {before} par {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Tu pārdēvēji dēļa {board} sarakstu {before} par {stack}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} pārdēvēja dēļa {board} sarakstu {before} par {stack}",
    "You have renamed the card {before} to {card}" : "Tu pārdēvēji kartīti {before} par {card}",
    "{user} has renamed the card {before} to {card}" : "{user} pārdēvēja kartīti {before} par {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Tu pievienoji kartītes {card} aprakstu dēļa {board} sarakstā {stack}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} pievienoja kartītes {card} aprakstu dēļa {board} sarakstā {stack}",
    "The file was uploaded" : "Datne tika augšupielādēta",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Augšupielādētā datne pārsniedz php.ini norādīto upload_max_filesize vērtību",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Augšupielādētā datne pārsniedz MAX_FILE_SIZE norādi, kas ir norādīta HTML formā",
    "The file was only partially uploaded" : "Datne tika augšupielādēta tikai daļēji",
    "No file was uploaded" : "Neviena datne netika augšupielādēta",
    "Missing a temporary folder" : "Trūkst pagaidu mapes",
    "Could not write file to disk" : "Nevarēja ierakstīt datni diskā",
    "A PHP extension stopped the file upload" : "PHP paplašinājums apturēja datnes augšupielādi",
    "Cards due today" : "Kartītes, kuru laiks pienāk šodien",
    "Load more" : "Ielādēt vairāk",
    "Finished" : "Pabeigts",
    "Action needed" : "Nepieciešama darbība",
    "Later" : "Vēlāk",
    "copy" : "kopēt",
    "Custom lists - click to rename!" : "Pielāgoti saraksti - klikšķināt, lai pārdēvētu.",
    "Done" : "Pabeigts",
    "Attachments" : "Pielikumi",
    "File" : "Datne",
    "Invalid date, date format must be YYYY-MM-DD" : "Nederīgs datums, datuma formātam jābūt YYYY-MM-DD",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck ir kanban veida organizēšanas rīks, kas paredzēts personīgai plānošanai un projektu orgenizēšanai komandām, kas ir iekļauts Nextcloud.\n\n\n- 📥 Uzdevumu pievienošana kartītēm un to sakārtošana\n- 📄 Papildu piezīmes var pierakstīt ar Markdown\n- 🔖 Vēl labākai pārraudzībai var piešķirt iezīmes\n- 👥 Koplietošana ar savu komandu, draugiem vai ģimeni\n- 📎 Datņu pievienošana un iekļaušana Markdown aprakstā\n- 💬 Apspriešana ar savu komandu ar piebildēm\n- ⚡ Izmaiņu izsekošana darbību plūsmā\n- 🚀 Sava projekta sakārtošana",
    "Cancel" : "Atcelt",
    "File already exists" : "Datne jau pastāv",
    "Do you want to overwrite it?" : "Vai tu gribi pārrakstīt to?",
    "Add list" : "Pievienot sarakstu…",
    "Clear search" : "Notīrīt meklēšanu",
    "Open" : "Atvērt",
    "Completed" : "Pabeigts",
    "Clear filter" : "Notīrīt atlasi",
    "Hide archived cards" : "Paslēpt arhivētās kartes",
    "Show archived cards" : "Rādīt arhivētās kartes",
    "Open details" : "Atvērt informāciju",
    "Details" : "Informācija",
    "General" : "Vispārīgi",
    "Appearance" : "Izskats",
    "Search" : "Meklēt",
    "Sharing" : "Koplietošana",
    "Tags" : "Birkas",
    "Activity" : "Darbības",
    "Undo" : "Atsaukt",
    "Hours" : "Stundas",
    "Days" : "Dienas",
    "Weeks" : "Nedēļas",
    "Months" : "Mēniši",
    "Board owner" : "Dēļa īpašnieks",
    "(Group)" : "(Grupa)",
    "Can edit" : "Var labot",
    "Can share" : "Var koplietot",
    "Owner" : "Īpašnieks",
    "Delete" : "Izdzēst",
    "Edit" : "Labot",
    "Members" : "Dalībnieki",
    "File to share" : "Kopīgojamā datne",
    "Invalid path selected" : "Atlasīts nederīgs ceļš",
    "Download" : "Lejupielādēt",
    "Open in sidebar view" : "Atvērt sānjoslas skatā",
    "Comments" : "Piebildes",
    "Submit" : "Iesniegt",
    "Created:" : "Izveidots:",
    "Reply" : "Atbildēt",
    "Update" : "Atjaunināt",
    "Description" : "Apraksts",
    "Select Date" : "Atlasīt datumu",
    "Later today – {timeLocale}" : "Vēlāk šodien – {timeLocale}",
    "Set due date for later today" : "Iestatīt beigu datumu vēlāk šodienā",
    "(group)" : "(grupa)",
    "Open link" : "Atvērt saiti",
    "Shared with you" : "Koplietots ar tevi",
    "No reminder" : "Nav atgādinājuma",
    "An error occurred" : "Gadījās kļūda",
    "No notifications" : "Nav paziņojumu",
    "Advanced options" : "Papildu iespējas",
    "Export" : "Izgūt",
    "Deck board {name}\n* Last modified on {lastMod}" : "Kavas dēlis {name}\n* Pēdējoreiz izmainīts {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Izveidots {created}\n* Pēdējoreiz izmainīts {lastMod}\n* {nbAttachments} pielikumi\n* {nbComments} piebildes",
    "Close" : "Aizvērt",
    "Due date" : "Izpildes datums",
    "Created" : "Izveidots",
    "Modified" : "Mainīts",
    "Share" : "Koplietot",
    "Searching for users, groups and teams …" : "Meklē lietotājus, kopas un komandas …",
    "Save" : "Saglabāt",
    "seconds ago" : "pirms vairākām sekundēm",
    "Today" : "Šodien",
    "Tomorrow" : "Rīt"
},
"nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n != 0 ? 1 : 2);");
