"use strict";(self.webpackChunkdeck=self.webpackChunkdeck||[]).push([["node_modules_nextcloud_dialogs_dist_chunks_index-BMbtc3xh_mjs"],{83731(e,d,s){s.d(d,{FilePickerVue:()=>_});const _=(0,s(85471).$V)((()=>Promise.all([s.e("vendors-node_modules_nextcloud_dialogs_node_modules_nextcloud_files_dist_index_mjs-node_modul-06376e"),s.e("vendors-node_modules_nextcloud_dialogs_dist_chunks_FilePicker-JKNLPCbR_mjs")]).then(s.bind(s,56876))))}}]);
//# sourceMappingURL=deck-node_modules_nextcloud_dialogs_dist_chunks_index-BMbtc3xh_mjs.js.map?v=a64bef285858d61924ed