OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Sie haben das neue Board {board} erstellt",
    "{user} has created a new board {board}" : "{user} hat das neue Board {board} erstellt",
    "You have deleted the board {board}" : "Sie haben das Board {board} gelöscht",
    "{user} has deleted the board {board}" : "{user} hat das Board {board} gelöscht",
    "You have restored the board {board}" : "Sie haben das Board {board} wiederhergestellt",
    "{user} has restored the board {board}" : "{user} hat das Board {board} wiederhergestellt",
    "You have shared the board {board} with {acl}" : "Sie haben das Board {board} mit {acl} geteilt",
    "{user} has shared the board {board} with {acl}" : "{user} hat das Board {board} mit {acl} geteilt",
    "You have removed {acl} from the board {board}" : "Sie haben {acl} vom Board {board} entfernt",
    "{user} has removed {acl} from the board {board}" : "{user} hat {acl} vom Board {board} entfernt",
    "You have renamed the board {before} to {board}" : "Sie haben das Board {before} in {board} umbenannt",
    "{user} has renamed the board {before} to {board}" : "{user} hat das Board {before} in {board} umbenannt",
    "You have archived the board {board}" : "Sie haben das Board {board} archiviert",
    "{user} has archived the board {before}" : "{user} hat das Board {before} archiviert",
    "You have unarchived the board {board}" : "Sie haben das Board {board} dearchiviert",
    "{user} has unarchived the board {before}" : "{user} hat das Board {before} dearchiviert",
    "You have created a new list {stack} on board {board}" : "Sie haben die neue Liste {stack} auf dem Board {board} erstellt",
    "{user} has created a new list {stack} on board {board}" : "{user} hat die neue Liste {stack} auf dem Board {board} erstellt",
    "You have renamed list {before} to {stack} on board {board}" : "Sie haben die Liste {before} auf dem Board {board} in {stack} umbenannt",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} hat die Liste {before} in {stack} auf dem Board {board} umbenannt",
    "You have deleted list {stack} on board {board}" : "Sie haben die Liste {stack} auf dem Board {board} gelöscht",
    "{user} has deleted list {stack} on board {board}" : "{user} hat die Liste {stack} aus dem Board {board} gelöscht",
    "You have created card {card} in list {stack} on board {board}" : "Sie haben die Karte {card} in der Liste {stack} auf dem Board {board} erstellt",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} hat die Karte {card} in der Liste {stack} auf dem Board {board} erstellt",
    "You have deleted card {card} in list {stack} on board {board}" : "Sie haben die Karte {card} in der Liste {stack} auf dem Board {board} gelöscht",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} hat die Karte {card} in der Liste {stack} auf dem Board {board} gelöscht",
    "You have renamed the card {before} to {card}" : "Sie haben die Karte {before} in {card} umbenannt",
    "{user} has renamed the card {before} to {card}" : "{user} hat die Karte {before} in {card} umbenannt",
    "You have added a description to card {card} in list {stack} on board {board}" : "Sie haben der Karte {card} in der Liste {stack} auf dem Board {board} eine Beschreibung hinzugefügt",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} hat der Karte {card} in der Liste {stack} auf dem Board {board} eine Beschreibung hinzugefügt",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Sie haben die Beschreibung der Karte {card} in der Liste {stack} auf dem Board {board} aktualisiert",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} hat die Beschreibung der Karte {card} in der Liste {stack} auf dem Board {board} aktualisiert",
    "You have archived card {card} in list {stack} on board {board}" : "Sie haben die Karte {card} in der Liste {stack} auf dem Board {board} archiviert",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} hat die Karte {card} in der Liste {stack} auf dem Board {board} archiviert",
    "You have unarchived card {card} in list {stack} on board {board}" : "Sie haben die Karte {card} in der Liste {stack} auf dem Board {board} dearchiviert",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} hat die Karte {card} in der Liste {stack} auf dem Board {board} dearchiviert",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Sie haben die Karte {card} in der Liste {stack} im Board {board} als erledigt markiert",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} hat die Karte {card} in der Liste {stack} im Board {board} als erledigt markiert",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Sie haben die Karte {card} in der Liste {stack} im Board {board} als unerledigt markiert",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} hat die Karte {card} in der Liste {stack} im Board {board} als unerledigt markiert",
    "You have removed the due date of card {card}" : "Sie haben das Fälligkeitsdatum der Karte {card} entfernt",
    "{user} has removed the due date of card {card}" : "{user} hat das Fälligkeitsdatum der Karte {card} entfernt",
    "You have set the due date of card {card} to {after}" : "Sie haben das Fälligkeitsdatum der Karte {card} auf {after} gesetzt",
    "{user} has set the due date of card {card} to {after}" : "{user} hat das Fälligkeitsdatum der Karte {card} auf {after} gesetzt",
    "You have updated the due date of card {card} to {after}" : "Sie haben das Fälligkeitsdatum der Karte {card} auf {after} aktualisiert",
    "{user} has updated the due date of card {card} to {after}" : "{user} hat das Fälligkeitsdatum der Karte {card} auf {after} aktualisiert",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Sie haben der Karte {card} in der Liste {stack} auf dem Board {board} das Schlagwort {label} hinzugefügt",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} hat der Karte {card} in der Liste {stack} auf dem Board {board} das Schlagwort {label} hinzugefügt",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Sie haben von der Karte {card} in der Liste {stack} auf dem Board {board} das Schlagwort {label} entfernt",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} hat von der Karte {card} in der Liste {stack} auf dem Board {board} das Schlagwort {label} entfernt",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Sie haben {assigneduser} der Karte {card} auf dem Board {board} zugewiesen",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} hat {assigneduser} der Karte {card} auf dem Board {board} zugewiesen",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Sie haben die Zuweisung von {assigneduser} zur Karte {card} auf dem Board {board} aufgehoben",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} hat die Zuweisung von {assigneduser} zur Karte {card} auf dem Board {board} aufgehoben",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Sie haben die Karte {card} von der Liste {stackBefore} nach {stack} verschoben",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} hat die Karte {card} von der Liste {stackBefore} nach {stack} verschoben",
    "You have added the attachment {attachment} to card {card}" : "Sie haben den Anhang {attachment} zur Karte {card} hinzugefügt",
    "{user} has added the attachment {attachment} to card {card}" : "{user} hat den Anhang {attachment} zur Karte {card} hinzugefügt",
    "You have updated the attachment {attachment} on card {card}" : "Sie haben den Anhang {attachment} der Karte {card} aktualisiert",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} hat den Anhang {attachment} der Karte {card} aktualisiert",
    "You have deleted the attachment {attachment} from card {card}" : "Sie haben den Anhang {attachment} von der Karte {card} gelöscht",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} hat den Anhang {attachment} von Karte {card} gelöscht",
    "You have restored the attachment {attachment} to card {card}" : "Sie haben den Anhang {attachment} der Karte {card} wiederhergestellt",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} hat den Anhang {attachment} der Karte {card} wiederhergestellt",
    "You have commented on card {card}" : "Sie haben die Karte {card} kommentiert",
    "{user} has commented on card {card}" : "{user} hat die Karte {card} kommentiert",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Änderungen in der <strong>Deck-App</strong>",
    "A <strong>board, list or card</strong> was changed" : "Ein <strong>Bord, eine Liste oder Karte</strong> wurde geändert",
    "A <strong>comment</strong> was created on a card" : "Ein <strong>Kommentar</strong> zu einer Karte wurde erstellt",
    "A <strong>card description</strong> has been changed" : "Eine <strong>Kartenbeschreibung</strong> wurde geändert",
    "The file was uploaded" : "Die Datei wurde hochgeladen",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Die hochgeladene Datei überschreitet die upload_max_filesize-Vorgabe in php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Die Datei ist größer als die MAX_FILE_SIZE-Vorgabe, die im HTML-Formular angegeben ist",
    "The file was only partially uploaded" : "Die Datei konnte nur teilweise hochgeladen werden",
    "No file was uploaded" : "Es wurde keine Datei hochgeladen",
    "Missing a temporary folder" : "Kein temporärer Ordner vorhanden",
    "Could not write file to disk" : "Die Datei konnte nicht auf die Festplatte geschrieben werden",
    "A PHP extension stopped the file upload" : "Eine PHP-Erweiterung hat das Hochladen der Datei gestoppt",
    "No file uploaded or file size exceeds maximum of %s" : "Keine Datei hochgeladen oder die Dateigröße überschreitet %s",
    "Invalid file type. Only JSON files are allowed." : "Ungültiger Dateityp. Nur JSON-Dateien sind zulässig.",
    "Invalid JSON data" : "Ungültige JSON-Daten",
    "Failed to import board" : "Board konnte nicht importiert werden",
    "Cards due today" : "Karten die heute fällig sind",
    "Cards due tomorrow" : "Karten die morgen fällig sind",
    "Upcoming cards" : "Anstehende Karten",
    "Load more" : "Mehr laden",
    "Welcome to Nextcloud Deck!" : "Willkommen bei Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Die Karte \"%s\" auf \"%s\" wurde Ihnen von %s zugewiesen.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} hat Ihnen die Karte {deck-card} auf {deck-board} zugewiesen.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Die Karte \"%1$s\" in \"%2$s\" ist überfällig.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Die Karte {deck-card} in {deck-board} ist überfällig.",
    "%s has mentioned you in a comment on \"%s\"." : "%s hat Sie in einem Kommentar zu \"%s\" erwähnt.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} hat Sie in einem Kommentar zu {deck-card} erwähnt.",
    "The board \"%s\" has been shared with you by %s." : "Das Board \"%s\" wurde von %s mit Ihnen geteilt.",
    "{user} has shared {deck-board} with you." : "{user} hat das Board {deck-board} mit Ihnen geteilt.",
    "The remote board %1$s has been shared with you by %2$s" : "Das Remote-Board %1$s wurde von %2$s mit Ihnen geteilt",
    "Deck board" : "Deck-Board",
    "Owned by %1$s" : "Eigentum von %1$s",
    "Deck boards, cards and comments" : "Deck-Boards, Karten und Kommentare",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Von %1$s, in %2$s / %3$s, Eigentum von %4$s",
    "Create a new deck card" : "Neue Deck-Karte erstellen",
    "Card comments" : "Kartenkommentare",
    "%s on %s" : "%s von %s",
    "Deck boards and cards" : "Deck-Boards und Karten",
    "No data was provided to create an attachment." : "Es wurden keine Daten zum Erstellen eines Anhangs bereitgestellt.",
    "Finished" : "Abgeschlossen",
    "To review" : "Zu überprüfen",
    "Action needed" : "Handlung erforderlich",
    "Later" : "Später",
    "copy" : "Kopie",
    "Read more inside" : "Mehr erfahren",
    "Custom lists - click to rename!" : "Benutzerdefinierte Listen - Zum Umbenennen klicken!",
    "To Do" : "Offen",
    "In Progress" : "In Bearbeitung",
    "Done" : "Erledigt",
    "1. Open to learn more about boards and cards" : "1. Öffnen, um mehr über Boards und Karten zu erfahren",
    "2. Drag cards left and right, up and down" : "2. Karten nach links und rechts, oben und unten ziehen",
    "3. Apply rich formatting and link content" : "3. Umfangreiche Formatierung und Linkinhalte",
    "4. Share, comment and collaborate!" : "4. Teilen, kommentieren und mitarbeiten!",
    "Create your first card!" : "Erstellen Sie Ihre erste Karte!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Dieser Kommentar hat mehr als %s Zeichen.\nAls Anhang der Karte mit dem Namen %s hinzugefügt.\nZugriff über die URL: %s.",
    "Attachments" : "Anhänge",
    "File" : "Datei",
    "date" : "Datum",
    "Card not found" : "Karte nicht gefunden",
    "Path is already shared with this card" : "Pfad wurde bereits mit dieser Karte geteilt",
    "Invalid date, date format must be YYYY-MM-DD" : "Ungültiges Datum, zulässiges Datumsformat: JJJJ-MM-TT",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Alle Boards, die Ihnen gehören, einschließlich Stapeln, Karten, Etiketten, Zuordnungen und Kommentaren",
    "Personal planning and team project organization" : "Persönliche Planung und Teamprojektorganisation",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck ist ein Organisationstool im Kanban-Stil für die persönliche Planung und Projektorganisation von Teams, die in Nextcloud integriert sind.\n\n\n- 📥 Fügen Sie Ihre Aufgaben zu den Karten hinzu und ordnen Sie diese\n- 📄 Zusätzliche Hinweise in der Abschrift notieren\n- 🔖 Zuweisen von Schlagworten für noch bessere Organisation\n- 👥 Teilen Sie mit Ihrem Team, Ihren Freunden oder Ihrer Familie\n- 📎 Fügen Sie Dateien hinzu und verwende diese in Ihren Markdown-Beschreibungen\n- 💬 Diskutieren Sie mit Ihrem Team mit Kommentaren\n- ⚡ Behalten Sie Überblick über Änderungen mit dem Aktivitäten-Stream\n- 🚀 Organisieren Sie Ihr Projekt",
    "Add board" : "Board hinzufügen",
    "Card details" : "Karten-Details",
    "Select the board to link to a project" : "Wählen Sie ein Board aus, um dieses mit einem Projekt zu verknüpfen",
    "Search by board title" : "Nach einem Board suchen",
    "Select board" : "Board auswählen",
    "Move/copy card" : "Karte verschieben/kopieren",
    "Select a board" : "Ein Board auswählen",
    "No lists available" : "Keine Listen verfügbar",
    "Select a list" : "Eine Liste auswählen",
    "Move card" : "Karte verschieben",
    "Copy card" : "Karte kopieren",
    "Select the card to link to a project" : "Wählen Sie die Karte aus, um diese mit einem Projekt zu verknüpfen",
    "Link to card" : "Mit einer Karte verknüpfen",
    "Select a card" : "Eine Karte auswählen",
    "Cancel" : "Abbrechen",
    "This board is read only" : "Dieses Board ist schreibgeschützt",
    "Drop your files to upload" : "Dateien zum Hochladen hineinziehen",
    "File already exists" : "Datei bereits vorhanden",
    "A file with the name {filename} already exists." : "Eine Datei mit diesem Namen {filename} existiert bereits.",
    "Do you want to overwrite it?" : "Möchten Sie überschreiben?",
    "Overwrite file" : "Datei überschreiben",
    "Keep existing file" : "Existierende Datei behalten",
    "Add card" : "Karte hinzufügen",
    "Archived cards" : "Archivierte Karten",
    "Add list" : "Liste hinzufügen",
    "List name" : "Listenname",
    "Clear search" : "Suche löschen",
    "Active filters" : "Aktive Filter",
    "Apply filter" : "Filter anwenden",
    "Filter by tag" : "Nach Schlagwort filtern",
    "Filter by assigned user" : "Nach zugewiesenem Benutzer filtern",
    "Unassigned" : "Nicht zugewiesen",
    "Filter by status" : "Nach Status filtern",
    "Open and completed" : "Offene und abgeschlossene",
    "Open" : "Offene",
    "Completed" : "Fertiggestellt",
    "Filter by due date" : "Nach Fälligkeitsdatum filtern",
    "Overdue" : "Überfällig",
    "Next 24 hours" : "Die nächsten 24 Stunden",
    "Next 7 days" : "Die nächsten 7 Tage",
    "Next 30 days" : "Die nächsten 30 Tage",
    "No due date" : "Kein Fälligkeitsdatum",
    "Clear filter" : "Filter zurücksetzen",
    "View Modes" : "Ansichtsmodi",
    "Toggle View Modes" : "Ansichtsmodi wechseln",
    "Kanban view" : "Kanban Ansicht",
    "Gantt view" : "Gantt Ansicht",
    "Hide archived cards" : "Archivierte Karten ausblenden",
    "Show archived cards" : "Archivierte Karten anzeigen",
    "Toggle compact mode" : "Kompaktmodus umschalten",
    "Hide card cover images" : "Kartencoverbilder ausblenden",
    "Show card cover images" : "Kartencoverbilder anzeigen",
    "Open details" : "Details öffnen",
    "Details" : "Details",
    "Deck settings" : "Deck-Einstellungen",
    "General" : "Allgemein",
    "Use bigger card view" : "Größere Kartenansicht verwenden",
    "Appearance" : "Aussehen",
    "Show card ID badge" : "Abzeichen mit Karten-ID zeigen",
    "Show boards in calendar/tasks" : "Board in Kalender/Aufgaben anzeigen",
    "Admin settings" : "Administrationseinstellungen",
    "Limit board creation to some groups" : "Boarderstellung auf einige Gruppen beschränken",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Benutzer außerhalb dieser Gruppen können keine eigenen Boards erstellen, aber weiterhin an Boards arbeiten, die mit ihnen geteilt wurden.",
    "Enable federation" : "Federation aktivieren",
    "Board actions" : "Board-Aktionen",
    "Scroll sideways" : "Seitwärts scrollen",
    "Navigate between cards" : "Zwischen Karten navigieren",
    "Close card details" : "Karten-Details schließen",
    "Search" : "Suche",
    "Show card filters" : "Kartenfilter anzeigen",
    "Clear card filters" : "Kartenfilter löschen",
    "Show those shortcuts" : "Diese Tastaturkürzel anzeigen",
    "Card actions" : "Kartenaktionen",
    "Open card details" : "Kartendetails öffnen",
    "Edit the card title" : "Kartentitel bearbeiten",
    "Assign yourself to the current card" : "Weisen Sie sich der aktuellen Karte zu",
    "Archive/unarchive the current card" : "Die aktuelle Karte archivieren/dearchivieren",
    "Mark card as completed/not completed" : "Karte als abgeschlossen/nicht abgeschlossen markieren",
    "Open card menu" : "Kartenmenü öffnen",
    "Currently present people" : "Aktuell Anwesende",
    "Supported prefixes: {prefixes}. Wrap phrases in double quotes." : "Unterstützte Präfixe: {prefixes}. Phrasen in doppelte Anführungszeichen setzen.",
    "Search cards" : "Karten suchen",
    "Loading board" : "Lade Board",
    "Board not found" : "Board nicht gefunden",
    "Create a new list to add cards to this board" : "Erstellen Sie eine neue Liste, um diesem Board Karten hinzuzufügen",
    "Sharing" : "Teilen",
    "Tags" : "Schlagworte",
    "Deleted items" : "Gelöschte Objekte",
    "Activity" : "Aktivität",
    "Deleted lists" : "Gelöschte Listen",
    "Undo" : "Rückgängig",
    "Deleted cards" : "Gelöschte Karten",
    "Hours" : "Stunden",
    "Days" : "Tage",
    "Weeks" : "Wochen",
    "Months" : "Monate",
    "No cards yet" : "Bislang keine Karten",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Ein Start- und Zieldatum auf Ihren Karten setzen, um dieses in der Gantt-Ansicht zu anzuzeigen",
    "Cards without dates ({count})" : "Karten ohne Datum ({count})",
    "Failed to create share with {displayName}" : "Freigabe mit dem Namen {displayName} konnte nicht erstellt werden",
    "Are you sure you want to transfer the board {title} to {user}?" : "Möchten Sie das Board {title} auf {user} übertragen?",
    "Transfer the board." : "Board übertragen.",
    "Transfer" : "Übertragen",
    "The board has been transferred to {user}" : "Das Board wurde auf {user} übertragen",
    "Failed to transfer the board to {user}" : "Das Board konnte nicht auf {user} übertragen werden",
    "Board owner" : "Board-Besitzer",
    "(Group)" : "(Gruppe)",
    "(Team)" : "(Team)",
    "(remote)" : "(extern)",
    "Can edit" : "kann bearbeiten",
    "Can share" : "kann teilen",
    "Can manage" : "kann verwalten",
    "Owner" : "Besitzer",
    "Delete" : "Löschen",
    "List deleted" : "Liste gelöscht",
    "Edit list title" : "Listentitel bearbeiten",
    "Archive all cards" : "Alle Karten archivieren",
    "Unarchive all cards" : "Alle Karten dearchivieren",
    "Do not set cards as \"done\"" : "Karten nicht als \"erledigt\" markieren",
    "Set cards as \"done\"" : "Karten als \"erledigt\" markieren",
    "Delete list" : "Liste löschen",
    "Archive all cards in this list" : "Alle Karten in dieser Liste archivieren",
    "Unarchive all cards in this list" : "Alle Karten in dieser Liste dearchivieren",
    "Add a new card" : "Neue Karte hinzufügen",
    "Card name" : "Kartenname",
    "title and color value must be provided" : "Titel und Farbwert müssen angegeben werden",
    "Edit" : "Bearbeiten",
    "Add a new tag" : "Neues Schlagwort hinzufügen",
    "Search boards" : "Boards suchen",
    "Board name" : "Boardname",
    "Members" : "Mitglieder",
    "Assign to users/groups/team" : "Benutzern/Gruppen/Teams zuweisen",
    "Assign a user to this card…" : "Diese Karte einem Benutzer zuweisen …",
    "Select a user to assign to this card…" : "Wählen Sie einen Benutzer aus, der dieser Karte zugewiesen werden soll …",
    "File to share" : "Zu teilende Datei",
    "Invalid path selected" : "Ungültiger Pfad ausgewählt",
    "Upload new files" : "Neue Dateien hochladen",
    "Share from Files" : "Aus Dateien heraus teilen",
    "Pending share" : "Ausstehende Freigabe",
    "Add this attachment" : "Diesen Anhang anhängen",
    "Show in Files" : "In Dateien anzeigen",
    "Download" : "Herunterladen",
    "Remove attachment" : "Anhang entfernen",
    "Delete Attachment" : "Anhang löschen",
    "Restore Attachment" : "Anhang wiederherstellen",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Geändert: {modifiedDate} ⸱ Erstellt: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Geändert: {modifiedDate}\nErstellt: {createdDate}\nErstellt von: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Geändert: {modifiedDate}\nErstellt: {createdDate}",
    "The title cannot be empty." : "Der Titel darf nicht leer sein.",
    "Cannot close unsaved card!" : "Nicht gespeicherte Karte kann nicht geschlossen werden!",
    "Open in sidebar view" : "In Seitenleiste öffnen",
    "Open in bigger view" : "In größerer Ansicht öffnen",
    "Comments" : "Kommentare",
    "Failed to load comments" : "Kommentare konnten nicht geladen werden",
    "No comments yet. Begin the discussion!" : "Bislang keine Kommentare. Beginnen Sie die Diskussion!",
    "The comment cannot be empty." : "Der Kommentar darf nicht leer sein.",
    "The comment cannot be longer than 1000 characters." : "Der Kommentar darf nicht länger als 1000 Zeichen sein.",
    "Submit" : "Übermitteln",
    "Created:" : "Erstellt:",
    "In reply to" : "Als Antwort auf",
    "Cancel reply" : "Antwort abbrechen",
    "Reply" : "Antworten",
    "Update" : "Aktualisieren",
    "Assign dependent cards" : "Abhängige Karten zuweisen",
    "Mark as done" : "Als erledigt markieren",
    "Mark as not done" : "Als unerledigt markieren",
    "Remove dependency" : "Abhängigkeit entfernen",
    "Add dependent card" : "Abhängige Karte hinzufügen",
    "Assign a dependent card…" : "Eine abhängige Karte zuweisen …",
    "Select a dependent card…" : "Eine abhängige Karte auswählen …",
    "Write a description …" : "Beschreibung schreiben …",
    "The description has been changed by another user." : "Die Beschreibung wurde von einem anderen Benutzer geändert.",
    "Could not save description" : "Beschreibung konnte nicht gespeichert werden",
    "Description" : "Beschreibung",
    "(Unsaved)" : "(nicht gespeichert)",
    "(Saving…)" : "(Speichere …)",
    "Formatting help" : "Formatierungshilfe",
    "Edit description" : "Beschreibung bearbeiten",
    "View description" : "Beschreibung anzeigen",
    "Add Attachment" : "Anhang anhängen",
    "Choose attachment" : "Anhang auswählen",
    "Select Date" : "Datum auswählen",
    "Later today – {timeLocale}" : "Später heute – {timeLocale}",
    "Set due date for later today" : "Fälligkeitsdatum auf später heute setzen",
    "Tomorrow – {timeLocale}" : "Morgen – {timeLocale}",
    "Set due date for tomorrow" : "Fälligkeitsdatum auf morgen setzen",
    "This weekend – {timeLocale}" : "Dieses Wochenende – {timeLocale}",
    "Set due date for this weekend" : "Fälligkeitsdatum auf kommendes Wochenende setzen",
    "Next week – {timeLocale}" : "Nächste Woche – {timeLocale}",
    "Set due date for next week" : "Fälligkeitsdatum auf nächste Woche setzen",
    "Assign a due date to this card…" : "Dieser Karte ein Fälligkeitsdatum zuweisen …",
    "Set a due date" : "Ein Ablaufdatum setzen",
    "Add due date" : "Fälligkeitsdatum hinzufügen",
    "Choose a date" : "Datum auswählen",
    "Remove due date" : "Fälligkeitsdatum entfernen",
    "Due at:" : "Fällig am:",
    "Not done" : "Nicht erledigt",
    "Unarchive card" : "Karte dearchivieren",
    "Archive card" : "Karte archivieren",
    "Assign a start date to this card…" : "Dieser Karte ein Anfangsdatum zuweisen …",
    "Set a start date" : "Anfangsdatum setzen",
    "Add start date" : "Anfangsdatum hinzufügen",
    "Remove start date" : "Anfangsdatum entfernen",
    "Assign a tag to this card…" : "Dieser Karte ein Schlagwort zuweisen …",
    "Select or create a tag…" : "Schlagwort auswählen oder eins erstellen …",
    "Create a new tag:" : "Neues Schlagwort erstellen:",
    "(group)" : "(Gruppe)",
    "{count} comments, {unread} unread" : "{count} Kommentare, {unread} ungelesen",
    "Todo items" : "Aufgaben-Elemente",
    "Edit card title" : "Kartentitel bearbeiten",
    "Open link" : "Link öffnen",
    "Card deleted" : "Karte gelöscht",
    "Change card color" : "Kartenfarbe ändern",
    "Edit title" : "Titel bearbeiten",
    "Assign to me" : "Mir zuweisen",
    "Unassign myself" : "Nicht mehr mir zuweisen",
    "Delete card" : "Karte löschen",
    "All boards" : "Alle Boards",
    "Archived boards" : "Archivierte Boards",
    "Shared with you" : "Mit Ihnen geteilt",
    "Cancel edit" : "Bearbeiten abbrechen",
    "Save board" : "Board sichern",
    "Board {0} deleted" : "Board {0} gelöscht",
    "All cards" : "Alle Karten",
    "Only assigned cards" : "Nur zugewiesene Karten",
    "No reminder" : "Keine Erinnerung",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Möchten Sie das Board {title} löschen? Dadurch werden alle Daten dieses Boards einschließlich archivierter Karten gelöscht.",
    "Delete the board?" : "Das Board löschen?",
    "Are you sure you want to leave the board {title}?" : "Möchten Sie das Board {title} verlassen?",
    "Leave the board?" : "Das Board verlassen?",
    "Leave" : "Verlassen",
    "Failed to leave the board" : "Das Board konnte nicht verlassen werden",
    "Exporting board..." : "Exportiere Board …",
    "An error occurred" : "Es ist ein Fehler aufgetreten",
    "Board details" : "Board-Details",
    "Edit board" : "Board bearbeiten",
    "Clone board" : "Board klonen",
    "Unarchive board" : "Board dearchivieren",
    "Archive board" : "Board archivieren",
    "Export board" : "Board exportieren",
    "Turn on due date reminders" : "Fälligkeitserinnerungen einschalten",
    "Turn off due date reminders" : "Fälligkeitserinnerungen ausschalten",
    "Remove as default board" : "Nicht mehr als Standardboard verwenden",
    "Set as default board" : "Als Standardboard verwenden",
    "Due date reminders" : "Fälligkeitserinnerungen",
    "Assigned cards" : "Zugewiesene Karten",
    "No notifications" : "Keine Benachrichtigungen",
    "Delete board" : "Board löschen",
    "Leave board" : "Board verlassen",
    "Importing board..." : "Importiere Board …",
    "Board imported successfully" : "Board importiert",
    "Import board" : "Board importieren",
    "Clone {boardTitle}" : "{boardTitle} klonen",
    "Clone cards" : "Karten klonen",
    "Clone assignments" : "Zuordnungen klonen",
    "Clone labels" : "Etiketten klonen",
    "Clone due dates" : "Fälligkeitsdaten klonen",
    "Advanced options" : "Erweiterte Optionen",
    "Move all cards to the first list" : "Alle Karten in die erste Liste verschieben",
    "Restore archived cards" : "Archivierte Karten wiederherstellen",
    "Clone" : "Klonen",
    "Export {boardTitle}" : "{boardTitle} exportieren",
    "Export as JSON" : "Als JSON exportieren",
    "Export as CSV" : "Als CSV exportieren",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Hinweis: Für den Rückimport in die Deck-App wird nur das JSON-Format unterstützt.",
    "Export" : "Exportieren",
    "Loading filtered view" : "Lade gefilterte Ansicht",
    "Search for {searchQuery} in other boards" : "Suche nach {searchQuery} in anderen Boards",
    "Search for {searchQuery} in all boards" : "Suche nach {searchQuery} in allen Boards",
    "No results found" : "Keine Ergebnisse gefunden",
    "Deck board {name}\n* Last modified on {lastMod}" : "Deck-Board {name}\n* Zuletzt geändert am {lastMod} ",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Erstellt am {created}\n* Zuletzt geändert am {lastMod}\n* {nbAttachments} Anhänge\n* {nbComments} Kommentare",
    "{nbCards} cards" : "{nbCards} Karten",
    "Due on {date}" : "Fällig am {date}",
    "{stack} in {board}" : "{stack} auf {board}",
    "Click to expand description" : "Klicken, um die Beschreibung zu erweitern",
    "Click to expand comment" : "Klicken, um den Kommentar zu erweitern",
    "Create card" : "Karte erstellen",
    "Create a new card" : "Neue Karte erstellen",
    "Card title" : "Titel der Karte",
    "Creating the new card …" : "Neue Karte wird erstellt …",
    "Card \"{card}\" was added to \"{board}\"" : "Karte \"{card}\" wurde \"{board}\" hinzugefügt",
    "Open card" : "Karte öffnen",
    "Close" : "Schließen",
    "No upcoming cards" : "Keine anstehenden Karten",
    "upcoming cards today" : "Heute anstehende Karten",
    "upcoming cards tomorrow" : "Morgen anstehende Karten",
    "upcoming cards" : "Anstehende Karten",
    "New card" : "Neue Karte",
    "Link to a board" : "Mit einem Board verknüpfen",
    "Link to a card" : "Mit einer Karte verknüpfen",
    "Create a card" : "Eine Karte erstellen",
    "Message from {author} in {conversationName}" : "Nachricht von {author} in {conversationName}",
    "Something went wrong" : "Etwas ist schiefgelaufen",
    "Failed to upload {name}" : "{name} konnte nicht hochgeladen werden",
    "Maximum file size of {size} exceeded" : "Maximale Dateigröße von {size} überschritten",
    "Assigned users" : "Zugewiesene Benutzer",
    "Due date" : "Fälligkeitsdatum",
    "Created" : "Erstellt",
    "Modified" : "Geändert",
    "Error creating the share" : "Fehler beim Erstellen der Freigabe",
    "Share with a Deck card" : "Mit einer Deck-Karte teilen",
    "Share {file} with a Deck card" : "{file} mit einer Deck-Karte teilen",
    "Share" : "Freigeben",
    "The card \"%s\" on \"%s\" has reached its due date." : "Die Karte \"%s\" in \"%s\" ist überfällig.",
    "Share board with a user, group or team …" : "Board mit einem Benutzer, einer Gruppe oder einem Team teilen …",
    "Searching for users, groups and teams …" : "Suche nach Benutzern, Gruppen und Teams …",
    "No participants found" : "Keine Teilnehmer gefunden",
    "Save" : "Speichern",
    "seconds ago" : "Gerade eben",
    "Today" : "Heute",
    "Tomorrow" : "Morgen",
    "No due" : "Kein Fälligkeitsdatum"
},
"nplurals=2; plural=(n != 1);");
