<?php return array(
    'root' => array(
        'name' => 'nextcloud/deck',
        'pretty_version' => 'v1.18.4',
        'version' => '1.18.4.0',
        'reference' => '46faf8ddaff4972dbd073b0ef101ce149e714e6a',
        'type' => 'project',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => false,
    ),
    'versions' => array(
        'bamarni/composer-bin-plugin' => array(
            'pretty_version' => '1.9.0',
            'version' => '1.9.0.0',
            'reference' => 'f6af6be0534fad47b38b34311e7d6148a67b5587',
            'type' => 'composer-plugin',
            'install_path' => __DIR__ . '/../bamarni/composer-bin-plugin',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'justinrainbow/json-schema' => array(
            'pretty_version' => 'v6.7.2',
            'version' => '6.7.2.0',
            'reference' => '6fea66c7204683af437864e7c4e7abf383d14bc0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../justinrainbow/json-schema',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'marc-mabe/php-enum' => array(
            'pretty_version' => 'v4.7.2',
            'version' => '4.7.2.0',
            'reference' => 'bb426fcdd65c60fb3638ef741e8782508fda7eef',
            'type' => 'library',
            'install_path' => __DIR__ . '/../marc-mabe/php-enum',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nextcloud/deck' => array(
            'pretty_version' => 'v1.18.4',
            'version' => '1.18.4.0',
            'reference' => '46faf8ddaff4972dbd073b0ef101ce149e714e6a',
            'type' => 'project',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
