OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Вы создали доску «{board}»",
    "{user} has created a new board {board}" : "{user} создал(а) доску «{board}»",
    "You have deleted the board {board}" : "Вы удалили доску «{board}»",
    "{user} has deleted the board {board}" : "{user} удалил(а) доску «{board}»",
    "You have restored the board {board}" : "Вы восстановили из корзины доску «{board}»",
    "{user} has restored the board {board}" : "{user} восстановил(а) доску «{board}» из корзины",
    "You have shared the board {board} with {acl}" : "Вы предоставили {acl} доступ к рабочей доске «{board}»",
    "{user} has shared the board {board} with {acl}" : "{user} предоставил(а) {acl} доступ к доске «{board}»",
    "You have removed {acl} from the board {board}" : "Вы закрыли {acl} доступ к рабочей доске «{board}»",
    "{user} has removed {acl} from the board {board}" : "{user} закрыл(а) {acl} доступ к рабочей доске «{board}»",
    "You have renamed the board {before} to {board}" : "Вы переименовали рабочую доску из «{before}» в «{board}»",
    "{user} has renamed the board {before} to {board}" : "{user} переименовал(а) рабочую доску из «{before}» в «{board}»",
    "You have archived the board {board}" : "Вы переместили доску «{board}» в архив",
    "{user} has archived the board {before}" : "{user} переместил(а) доску «{board}» в архив",
    "You have unarchived the board {board}" : "Вы восстановили доску «{board}» из архива",
    "{user} has unarchived the board {before}" : "{user} восстановил(а) доску «{before}» из архива",
    "You have created a new list {stack} on board {board}" : "Вы создали колонку «{stack}» на доске «{board}»",
    "{user} has created a new list {stack} on board {board}" : "{user} создал(а) колонку «{stack}» на доске «{board}»",
    "You have renamed list {before} to {stack} on board {board}" : "Вы переименовали колонку «{before}» в «{stack}» на доске «{board}»",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} переименовал(а) колонку «{before}» в «{stack}» на доске «{board}»",
    "You have deleted list {stack} on board {board}" : "Вы удалили колонку «{stack}» на доске «{board}»",
    "{user} has deleted list {stack} on board {board}" : "{user} удалил(а) колонку «{stack}» на доски «{board}»",
    "You have created card {card} in list {stack} on board {board}" : "Вы создали карточку «{card}» в колонке «{stack}» на доске «{board}»",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} создал(а) карточку «{card}» в колонке «{stack}» на доске «{board}»",
    "You have deleted card {card} in list {stack} on board {board}" : "Вы удалили карточку «{card}» в колонке «{stack}» на доске «{board}»",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} удалил(а) карточку «{card}» в колонке «{stack}» на доске «{board}»",
    "You have renamed the card {before} to {card}" : "Вы переименовали карточку из «{before}» в «{card}»",
    "{user} has renamed the card {before} to {card}" : "{user} переименовал(а) карточку из «{before}» в «{card}»",
    "You have added a description to card {card} in list {stack} on board {board}" : "Вы добавили описание карточки «{card}» в колонке «{stack}» на доске «{board}»",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} добавил(а) описание карточки «{card}» в колонке «{stack}» на доске «{board}»",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Вы изменили описание карточки «{card}» в колонке «{stack}» на доске «{board}»",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} изменил(а) описание карточки «{card}» в колонке «{stack}» на доске «{board}»",
    "You have archived card {card} in list {stack} on board {board}" : "Вы переместили в архив карточку «{card}» из колонки «{stack}» на доске «{board}»",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} переместил(а) в архив карточку «{card}» из колонки «{stack}» на доске «{board}»",
    "You have unarchived card {card} in list {stack} on board {board}" : "Вы восстановили из архива карточку «{card}» в колонку «{stack}» на доске «{board}»",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} восстановил(а) из архива карточку «{card}» в колонку «{stack}» на доске «{board}»",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Вы отметили как завершённую карточку «{card}» в колонке «{stack}» на доске «{board}»",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} отметил(а) как завершённую карточку «{card}» в колонке «{stack}» на доске «{board}»",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Вы отметили как незавершённую карточку «{card}» в колонке «{stack}» на доске «{board}»",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} отметил(а) как незавершённую карточку «{card}» в колонке «{stack}» на доске «{board}»",
    "You have removed the due date of card {card}" : "Вы удалили дедлайн карточки «{card}»",
    "{user} has removed the due date of card {card}" : "{user} удалил(а) срок исполнения по карточке «{card}»",
    "You have set the due date of card {card} to {after}" : "Вы задали срок исполнения по карточке «{card}» до {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} задал(а) срок исполнения по карточке «{card}» до {after}",
    "You have updated the due date of card {card} to {after}" : "Вы изменили срок исполнения по карточке «{card}» до {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} изменил(а) срок исполнения по карточке «{card}» до {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Вы добавили метку «{label}» карточке «{card}» в колонке «{stack}» на доске «{board}»",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} добавил(а) метку «{label}» карточке «{card}» в колонке «{stack}» на доске «{board}»",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Вы убрали метку «{label}» с карточки «{card}» в колонке «{stack}» на доске «{board}»",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} убрал(а) метку «{label}» с карточки «{card}» в колонке «{stack}» на доске «{board}»",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Вы назначили пользователю {assigneduser} карточку «{card}» с доски «{board}»",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} назначил(а) пользователю {assigneduser} карточку «{card}» с доски «{board}»",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Вы отменили назначение пользователю {assigneduser} карточки «{card}» с доски «{board}»",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} отменил(а) назначение пользователю {assigneduser} карточки «{card}» с доски «{board}»",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Вы переместили карточку «{card}» из колонки «{stackBefore}» в «{stack}»",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} переместил(а) карточку «{card}» из колонки «{stackBefore}» в «{stack}»",
    "You have added the attachment {attachment} to card {card}" : "Вы добавили вложение «{attachment}» в карточку «{card}»",
    "{user} has added the attachment {attachment} to card {card}" : "{user} добавил(а) вложение «{attachment}» в карточку «{card}»",
    "You have updated the attachment {attachment} on card {card}" : "Вы изменили вложение «{attachment}» в карточке «{card}»",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} изменил(а) вложение «{attachment}» в карточке «{card}»",
    "You have deleted the attachment {attachment} from card {card}" : "Вы удалили вложение «{attachment}» из карточки «{card}»",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} удалил(а) вложение «{attachment}» из карточки «{card}»",
    "You have restored the attachment {attachment} to card {card}" : "Вы восстановили вложение «{attachment}» в карточке «{card}»",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} восстановил(а) из корзины файл «{attachment}» в карточке «{card}»",
    "You have commented on card {card}" : "Вы добавили комментарий к карточке «{card}»",
    "{user} has commented on card {card}" : "{user} добавил(а) комментарий к карточке «{card}»",
    "Deck" : "Карточки",
    "Changes in the <strong>Deck app</strong>" : "Изменения в <strong>приложении Карточки</strong>",
    "A <strong>board, list or card</strong> was changed" : "<strong>Доска, колонка или карточка</strong> были изменены",
    "A <strong>comment</strong> was created on a card" : "Добавлен <strong>комментарий</strong> к карточке",
    "A <strong>card description</strong> has been changed" : "<strong>Описание карточки</strong> изменено",
    "The file was uploaded" : "Файл загружен",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Размер загружаемого файла превышает предел, заданный в параметре «upload_max_filesize» в файле php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Размер загруженного файла превышает заданное в HTML-форме значение параметра «MAX_FILE_SIZE»",
    "The file was only partially uploaded" : "Файл получен частично",
    "No file was uploaded" : "Файл не получен",
    "Missing a temporary folder" : "Отсутствует временный каталог",
    "Could not write file to disk" : "Невозможно записать файл на диск",
    "A PHP extension stopped the file upload" : "Загрузка файла была прервана модулем расширений PHP",
    "No file uploaded or file size exceeds maximum of %s" : "Файл не получен или его размер превышает %s",
    "Invalid file type. Only JSON files are allowed." : "Неверный тип файла. Разрешены только файлы JSON.",
    "Invalid JSON data" : "Неверные данные JSON",
    "Failed to import board" : "Не удалось импортировать доску",
    "Cards due today" : "Завершающиеся сегодня",
    "Cards due tomorrow" : "Завершающиеся завтра",
    "Upcoming cards" : "Ожидающие выполнения",
    "Load more" : "Загрузить больше",
    "Welcome to Nextcloud Deck!" : "Добро пожаловать в Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "Вам назначена карточка «%s» с доски «%s»  пользователем %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} назначил Вам карточку {deck-card} из {deck-board}.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "Дедлайн карточки \"%1$s\" на \"%2$s\" истёк.",
    "The card {deck-card} on {deck-board} has reached its due date." : "Сегодня дедлайн карточки {deck-card} в {deck-board}.",
    "%s has mentioned you in a comment on \"%s\"." : "Вы были упомянуты пользователем %s в комментарии «%s».",
    "{user} has mentioned you in a comment on {deck-card}." : "Вы были упомянуты пользователем {user} в комментарии {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "Вам предоставлен доступ к рабочей доске «%s» пользователем %s.",
    "{user} has shared {deck-board} with you." : "{user} предоставил(а) вам доступ к {deck-board}.",
    "The remote board %1$s has been shared with you by %2$s" : " Вам предоставлен доступ к удалённой доске %1$s пользователем %2$s",
    "Deck board" : "Доска",
    "Owned by %1$s" : "Владелец: %1$s",
    "Deck boards, cards and comments" : "Доски, карточки и комментарии",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Из %1$s, в %2$s/%3$s, принадлежит %4$s",
    "Create a new deck card" : "Создать новую карточку на доске",
    "Card comments" : "Комментарии к карточке",
    "%s on %s" : "%s на %s",
    "Deck boards and cards" : "Доски и карточки",
    "No data was provided to create an attachment." : "Отсутствуют данные для создания вложения.",
    "Finished" : "Завершено",
    "To review" : "Проверить",
    "Action needed" : "Требуется действие",
    "Later" : "Позже",
    "copy" : "Копировать",
    "Read more inside" : "Подробнее читайте внутри",
    "Custom lists - click to rename!" : "Пользовательские колонки — нажмите, чтобы переименовать!",
    "To Do" : "Задачи",
    "In Progress" : "Выполянется",
    "Done" : "Выполненные",
    "1. Open to learn more about boards and cards" : "1. Откройте, чтобы узнать больше о досках и карточках.",
    "2. Drag cards left and right, up and down" : "2. Перетаскивайте карточки влево и вправо, вверх и вниз.",
    "3. Apply rich formatting and link content" : "3. Примените расширенное форматирование и свяжите контент",
    "4. Share, comment and collaborate!" : "4. Делитесь, комментируйте и сотрудничайте!",
    "Create your first card!" : "Создайте свою первую карточку!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Этот комментарий содержит более %s символов.\nКомментарий добавлен к карточке «%s» в виде вложения.\nПостоянный адрес этого комментария: %s.",
    "Attachments" : "Вложения",
    "File" : "Файл",
    "date" : "дата",
    "Card not found" : "Карточка не найдена",
    "Path is already shared with this card" : "Этот путь уже опубликован в карточке",
    "Invalid date, date format must be YYYY-MM-DD" : "Некорректный формат даты, дата должна быть в формате YYYY-MM-DD",
    "All boards owned by you including stacks, cards, labels, assignments, and comments" : "Все доски, принадлежащие вам, включая колонки, карточки, метки, задания и комментарии.",
    "Personal planning and team project organization" : "Личное планирование и управление проектами для команд",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Карточки — приложение для Nextcloud, предоставляющее возможности личного планирования и управления проектами для рабочих групп в стиле канбан.\n\n\n- 📥 Создавайте карточки задач и упорядочивайте их;\n- 📄 Добавляйте описания задач с возможностью использовать формат разметки «markdown»;\n- 🔖 Присваивайте ярлыки для ещё лучшей организации;\n- 👥 Делитесь со своей командой, друзьями или семьей;\n- 📎 Вкладывайте файлы в виде ссылок в описании, используя формат разметки «Markdown»;\n- 💬 Обсуждайте в комментариях;\n- ⚡ Отслеживайте изменения в ленте событий;\n- 🚀 Сделайте свой проект более организованным.",
    "Add board" : "Добавить доску",
    "Card details" : "Свойства карточки",
    "Select the board to link to a project" : "Выберите доску, чтобы связать с проектом",
    "Search by board title" : "Поиск по заголовкам досок",
    "Select board" : "Выбор доски",
    "Move/copy card" : "Переместить/копировать карточку",
    "Select a board" : "Выберите доску",
    "No lists available" : "Нет ни одной колонки",
    "Select a list" : "Выберите колонку",
    "Move card" : "Переместить карточку",
    "Copy card" : "Копировать карточку",
    "Select the card to link to a project" : "Выберите карточку для связывания с проектом",
    "Link to card" : "Ссылка на карточку",
    "Select a card" : "Выберите карточку",
    "Cancel" : "Отменить",
    "This board is read only" : "Эта доска доступна только для чтения",
    "Drop your files to upload" : "Для загрузки файлов, перетащите их сюда",
    "File already exists" : "Файл уже существует",
    "A file with the name {filename} already exists." : "Файл «{filename}» уже существует.",
    "Do you want to overwrite it?" : "Перезаписать?",
    "Overwrite file" : "Перезаписать",
    "Keep existing file" : "Сохранить имеющийся",
    "Add card" : "Создать карточку",
    "Archived cards" : "Карточки в архиве",
    "Add list" : "Добавить колонку",
    "List name" : "Название колонки",
    "Clear search" : "Очистить поиск",
    "Active filters" : "Активные фильтры",
    "Apply filter" : "Применить фильтр",
    "Filter by tag" : "Отбор по меткам:",
    "Filter by assigned user" : "Отбор по назначению:",
    "Unassigned" : "Без назначения",
    "Filter by status" : "Отбор по состоянию:",
    "Open and completed" : "Открытые или завершённые",
    "Open" : "Открытые",
    "Completed" : "Завершённые",
    "Filter by due date" : "Отбор по сроку исполнения:",
    "Overdue" : "Просроченные",
    "Next 24 hours" : "Ближайшие 24 часа",
    "Next 7 days" : "Ближайшие 7 дней",
    "Next 30 days" : "Ближайшие 30 дней",
    "No due date" : "Без дедлайна",
    "Clear filter" : "Сбросить фильтр",
    "View Modes" : "Режимы просмотра",
    "Toggle View Modes" : "Выбор режима просмотра",
    "Kanban view" : "Kanban вид",
    "Gantt view" : "Gantt вид",
    "Hide archived cards" : "Скрыть архивные карточки",
    "Show archived cards" : "Показывать архивные карточки",
    "Toggle compact mode" : "Выбор компактного или обычного режима просмотра",
    "Hide card cover images" : "Скрыть обложки карточек",
    "Show card cover images" : "Показать обложки карточек",
    "Open details" : "Открыть подробности",
    "Details" : "Свойства",
    "Deck settings" : "Параметры карточек",
    "General" : "Основные",
    "Use bigger card view" : "Режим просмотра с увеличенными карточками",
    "Appearance" : "Внешний вид",
    "Show card ID badge" : "Показать идентификатор карточки",
    "Show boards in calendar/tasks" : "Показывать карточки в календаре и задачах",
    "Admin settings" : "Настройки администратора",
    "Limit board creation to some groups" : "Разрешить создавать доски только участниками определённых групп",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Создание собственных рабочих досок пользователям, не входящим в заданные группы, будет заблокировано. Тем не менее такие пользователи смогут продолжить работать с общими досками, к которым у них есть доступ. ",
    "Enable federation" : "Включить федерацию",
    "Board actions" : "Управление доской",
    "Scroll sideways" : "Боковая прокрутка",
    "Navigate between cards" : "Перемещение между карточками",
    "Close card details" : "Закрыть свойства карточки",
    "Search" : "Поиск",
    "Show card filters" : "Открыть фильтр карточек",
    "Clear card filters" : "Очистить фильтр карточек",
    "Show those shortcuts" : "Показать ярлыки",
    "Card actions" : "Управление карточками",
    "Open card details" : "Открыть свойства карточки",
    "Edit the card title" : "Изменить заголовок карточки",
    "Assign yourself to the current card" : "Назначить открытую карточку себе",
    "Archive/unarchive the current card" : "Переместить карточку в архив или вернуть из архива",
    "Mark card as completed/not completed" : "Изменить отметку выполнении карточки",
    "Open card menu" : "Открыть меню управления карточкой",
    "Currently present people" : "Присутствующие в настоящее время люди",
    "Loading board" : "Загрузка доски",
    "Board not found" : "Доска не найдена",
    "Create a new list to add cards to this board" : "Создайте колонку чтобы добавить карточки на эту доску",
    "Sharing" : "Общий доступ",
    "Tags" : "Метки",
    "Deleted items" : "Корзина",
    "Activity" : "Активность",
    "Deleted lists" : "Удалённые колонки",
    "Undo" : "Отменить",
    "Deleted cards" : "Карточки в корзине",
    "Hours" : "Часы",
    "Days" : "Дней",
    "Weeks" : "Недель",
    "Months" : "Месяц(ев)",
    "No cards yet" : "Ещё нет ни одной карточки",
    "Set a start date and due date on your cards to see them on the Gantt chart" : "Установите дату начала и дедлайн на своих карточках, чтобы увидеть их на диаграмме Ганта",
    "Cards without dates ({count})" : "Карточки без дат ({count})",
    "Failed to create share with {displayName}" : "Не удалось предоставить общий доступ для {displayName}",
    "Are you sure you want to transfer the board {title} to {user}?" : "Вы уверены, что хотите передать доску {title} пользователю {user}?",
    "Transfer the board." : "Перенесите доску.",
    "Transfer" : "Передача",
    "The board has been transferred to {user}" : "Доска была передана пользователю {user}",
    "Failed to transfer the board to {user}" : "Не удалось передать доску пользователю {user}",
    "Board owner" : "Владелец доски",
    "(Group)" : "(Группа)",
    "(Team)" : "(Команда)",
    "(remote)" : "(удалённо)",
    "Can edit" : "Разрешить редактировать",
    "Can share" : "Разрешить делиться с другими",
    "Can manage" : "Разрешить изменять",
    "Owner" : "Владелец",
    "Delete" : "Удалить",
    "List deleted" : "Колонка удалена",
    "Edit list title" : "Изменить название колонки",
    "Archive all cards" : "Переместить все карточки в архив",
    "Unarchive all cards" : "Восстановить все карточки из архива",
    "Do not set cards as \"done\"" : "Не устанавливать статус карточек как «выполнено»",
    "Set cards as \"done\"" : "Установить статус карточек как «выполнено»",
    "Delete list" : "Удалить колонку",
    "Archive all cards in this list" : "Переместить в архив все карточки колонки",
    "Unarchive all cards in this list" : "Восстановить из архива все карточки колонки",
    "Add a new card" : "Создать карточку",
    "Card name" : "Название карточки",
    "title and color value must be provided" : "необходимо указать цвет и название",
    "Edit" : "Редактировать",
    "Add a new tag" : "Добавить метку",
    "Board name" : "Название доски",
    "Members" : "Участники",
    "Assign to users/groups/team" : "Назначить пользователям, группам или командам",
    "Assign a user to this card…" : "Назначить карточку пользователю…",
    "Select a user to assign to this card…" : "Выберите пользователя, которому будет назначена карточка…",
    "File to share" : "Файл для публикации",
    "Invalid path selected" : "Выбран некорректный путь",
    "Upload new files" : "Загрузить новые файлы",
    "Share from Files" : "Опубликовать из приложения «Файлы»",
    "Pending share" : "Ожидающие публикации",
    "Add this attachment" : "Добавить это вложение",
    "Show in Files" : "Показать в приложении «Файлы»",
    "Download" : "Скачать",
    "Remove attachment" : "Убрать вложение",
    "Delete Attachment" : "Удалить вложение",
    "Restore Attachment" : "Восстановить вложение",
    "Modified: {modifiedDate} ⸱ Created: {createdDate}" : "Изменено: {modifiedDate} ⸱ Дата создания: {createdDate}",
    "Modified: {modifiedDate}\nCreated: {createdDate}\nCreated by: {owner}" : "Изменено: {modifiedDate}\nДата создания: {createdDate}\nСоздано пользователем: {owner}",
    "Modified: {modifiedDate}\nCreated: {createdDate}" : "Изменено: {modifiedDate}\nДата создания: {createdDate}",
    "The title cannot be empty." : "Название не может быть пустым",
    "Cannot close unsaved card!" : "Невозможно закрыть несохранённую карточку!",
    "Open in sidebar view" : "Открыть в боковой панели",
    "Open in bigger view" : "Открыть режиме подробного просмотра",
    "Comments" : "Комментарии",
    "Failed to load comments" : "Сбой при загрузке комментариев",
    "No comments yet. Begin the discussion!" : "Комментарии отсутствуют, начните обсуждение!",
    "The comment cannot be empty." : "Комментарий не может быть пустым.",
    "The comment cannot be longer than 1000 characters." : "Комментарий не может быть длиннее 1 000 символов.",
    "Submit" : "Отправить ответ",
    "Created:" : "Создание:",
    "In reply to" : "В ответ",
    "Cancel reply" : "Отменить ответ",
    "Reply" : "Ответить",
    "Update" : "Обновить",
    "Assign dependent cards" : "Назначить дочерние карточки (подзадачи)",
    "Mark as done" : "Отметить как выполненную",
    "Mark as not done" : "Отметить как невыполненную",
    "Remove dependency" : "Удалить зависимость",
    "Add dependent card" : "Добавить дочернюю карточку (подзадачу)",
    "Assign a dependent card…" : "Назначить дочернюю карточку (подзадачу)...",
    "Select a dependent card…" : "Выбрать дочернюю карточку (подзадачу)…",
    "Write a description …" : "Добавьте описание...",
    "The description has been changed by another user." : "Описание изменено другим пользователем.",
    "Could not save description" : "Не удалось сохранить описание",
    "Description" : "Описание",
    "(Unsaved)" : "(Не сохранен)",
    "(Saving…)" : "(Сохранение...)",
    "Formatting help" : "Справка по форматированию",
    "Edit description" : "Редактировать описание",
    "View description" : "Просмотреть описание",
    "Add Attachment" : "Добавить вложение",
    "Choose attachment" : "Выберите вложение",
    "Select Date" : "Выберите дату",
    "Later today – {timeLocale}" : "Позднее сегодня – {timeLocale}",
    "Set due date for later today" : "Задать дедлайн на сегодня позднее",
    "Tomorrow – {timeLocale}" : "Завтра – {timeLocale}",
    "Set due date for tomorrow" : "Задать дедлайн на завтра",
    "This weekend – {timeLocale}" : "Эти выходные – {timeLocale}",
    "Set due date for this weekend" : "Задать дедлайн на ближайшие выходные",
    "Next week – {timeLocale}" : "Следующая неделя – {timeLocale}",
    "Set due date for next week" : "Задать дедлайн на следующую неделю",
    "Assign a due date to this card…" : "Назначить дедлайн карточки...",
    "Set a due date" : "Установить дедлайн",
    "Add due date" : "Добавить дедлайн",
    "Choose a date" : "Выбрать дату…",
    "Remove due date" : "Удалить дедлайн",
    "Due at:" : "Дедлайн:",
    "Not done" : "Незавершённые",
    "Unarchive card" : "Восстановить из архива",
    "Archive card" : "Переместить в архив",
    "Assign a start date to this card…" : "Назначьте этой карточке дату начала…",
    "Set a start date" : "Установите дату начала",
    "Add start date" : "Добавить дату начала",
    "Remove start date" : "Удалить дату начала",
    "Assign a tag to this card…" : "Назначить метку этой карточке…",
    "Select or create a tag…" : "Выбрать или создать метку…",
    "Create a new tag:" : "Создать метку:",
    "(group)" : "(группа)",
    "{count} comments, {unread} unread" : "{count} комментариев, {unread} непрочитано",
    "Todo items" : "Элементы списка задач",
    "Edit card title" : "Изменить заголовок карточки",
    "Open link" : "Открыть ссылку",
    "Card deleted" : "Карточка удалена",
    "Change card color" : "Изменить цвет карточки",
    "Edit title" : "Редактировать заголовок",
    "Assign to me" : "Назначить себе",
    "Unassign myself" : "Отказаться от назначения",
    "Delete card" : "Удалить",
    "All boards" : "Все доски",
    "Archived boards" : "Архив досок",
    "Shared with you" : "Предоставленные вам",
    "Cancel edit" : "Отменить правку",
    "Save board" : "Сохранить доску",
    "Board {0} deleted" : "Доска {0} удалена",
    "All cards" : "Для всех карточек",
    "Only assigned cards" : "Только для назначенных карточек",
    "No reminder" : "Не напоминать",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Подтвердите удаление доски «{title}»; это действие приведёт к удалению также все данных, принадлежащих этой доске, включая карточки в архиве.",
    "Delete the board?" : "Удалить доску?",
    "Are you sure you want to leave the board {title}?" : "Вы уверены, что хотите покинуть доску {title}?",
    "Leave the board?" : "Покинуть доску?",
    "Leave" : "Выйти",
    "Failed to leave the board" : "Не удалось покинуть доску",
    "Exporting board..." : "Экспорт доски...",
    "An error occurred" : "Произошла ошибка",
    "Board details" : "Свойства доски",
    "Edit board" : "Редактировать",
    "Clone board" : "Скопировать доску",
    "Unarchive board" : "Восстановить доску из архива",
    "Archive board" : "Переместить доску в архив",
    "Export board" : "Экспортировать доску",
    "Turn on due date reminders" : "Включить напоминания о дедлайне",
    "Turn off due date reminders" : "Отключить напоминания о дедлайне",
    "Remove as default board" : "Удалить как доску по умолчанию",
    "Set as default board" : "Установить как доску по умолчанию",
    "Due date reminders" : "Напоминания о дедлайне",
    "Assigned cards" : "Назначенные карточки",
    "No notifications" : "Без уведомлений",
    "Delete board" : "Удалить доску",
    "Leave board" : "Покинуть доску",
    "Importing board..." : "Импорт доски...",
    "Board imported successfully" : "Доска успешно импортирована",
    "Import board" : "Импортировать доску",
    "Clone {boardTitle}" : "Клонировать {boardTitle}",
    "Clone cards" : "Скопировать карточку",
    "Clone assignments" : "Клонировать назнечение карточек",
    "Clone labels" : "Клонировать метки",
    "Clone due dates" : "Клонировать дедлайны",
    "Advanced options" : "Дополнительные настройки",
    "Move all cards to the first list" : "Переместить все карточки в первую колонку",
    "Restore archived cards" : "Восстановить заархивированные карточки",
    "Clone" : "Скопировать",
    "Export {boardTitle}" : "Экспортировать {boardTitle}",
    "Export as JSON" : "Экспорт как JSON",
    "Export as CSV" : "Экспортировать в файл CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Примечание. Для обратного импорта в приложение Deck поддерживается только формат JSON.",
    "Export" : "Экспорт",
    "Loading filtered view" : "Применяется фильтр",
    "Search for {searchQuery} in other boards" : "Искать {searchQuery} на других досках",
    "Search for {searchQuery} in all boards" : "Искать {searchQuery} на всех досках",
    "No results found" : "Результаты отсутствуют",
    "Deck board {name}\n* Last modified on {lastMod}" : "Доска «{name}»\n* Последнее изменение: {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Создано {created}\n* Последнее изменение {lastMod}\n* {nbAttachments} вложений\n* {nbComments} комментариев",
    "{nbCards} cards" : "карточек: {nbCards}",
    "Due on {date}" : "Дедлайн: {date}",
    "{stack} in {board}" : "«{stack}» с доски «{board}»",
    "Click to expand description" : "Нажмите, чтобы развернуть описание",
    "Click to expand comment" : "Нажмите, чтобы развернуть комментарии",
    "Create card" : "Создать карточку",
    "Create a new card" : "Создать карточку",
    "Card title" : "Заголовок карточки",
    "Creating the new card …" : "Создание новой карточки...",
    "Card \"{card}\" was added to \"{board}\"" : "Карточка «{card}» была добавлена в «{board}»",
    "Open card" : "Открыть карточку",
    "Close" : "Закрыть",
    "No upcoming cards" : "Отсутствуют карточки, ожидающие выполнения",
    "upcoming cards today" : "карточки, которые должны быть выполнены сегодня",
    "upcoming cards tomorrow" : "карточки, которые должны быть выполнены завтра",
    "upcoming cards" : "карточки, ожидающие выполнения",
    "New card" : "Новая карточка",
    "Link to a board" : "Ссылка на доску",
    "Link to a card" : "Ссылка на карточку",
    "Create a card" : "Создать карточку",
    "Message from {author} in {conversationName}" : "Сообщение от {author} из беседы «{conversationName}»",
    "Something went wrong" : "Что-то пошло не так",
    "Failed to upload {name}" : "Не удалось передать на сервер {name}",
    "Maximum file size of {size} exceeded" : "Превышен максимально разрешённый размер файла в  {size}",
    "Assigned users" : "Назначенные пользователи",
    "Due date" : "Дедлайн",
    "Created" : "Создано",
    "Modified" : "Изменено",
    "Error creating the share" : "Не удалось опубликовать",
    "Share with a Deck card" : "Опубликовать в приложении Карточки",
    "Share {file} with a Deck card" : "Опубликовать «{file}» в приложении Карточки",
    "Share" : "Опубликовать",
    "The card \"%s\" on \"%s\" has reached its due date." : "Сегодня дата исполнения по карточке «%s» в «%s» .",
    "Share board with a user, group or team …" : "Открыть доступ к доске пользователю, группе или команде…",
    "Searching for users, groups and teams …" : "Поиск пользователей, групп и команд…",
    "No participants found" : "Не удалось найти ни одного участника",
    "Save" : "Сохранить",
    "seconds ago" : "несколько секунд назад",
    "Today" : "Сегодня",
    "Tomorrow" : "Завтра",
    "No due" : "Без дедлайна"
},
"nplurals=4; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<12 || n%100>14) ? 1 : n%10==0 || (n%10>=5 && n%10<=9) || (n%100>=11 && n%100<=14)? 2 : 3);");
