OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Та шинэ {board} самбар үүсгэлээ",
    "{user} has created a new board {board}" : "{user} шинэ {board} самбар үүсгэлээ",
    "You have deleted the board {board}" : "Та {board} самбарыг устгалаа",
    "{user} has deleted the board {board}" : "{user} {board} самбарыг устгалаа",
    "You have restored the board {board}" : "Та {board} самбарыг сэргээлээ",
    "{user} has restored the board {board}" : "{user} {board} самбарыг сэргээлээ",
    "You have shared the board {board} with {acl}" : "Та {board} самбарыг {acl}-тай хуваалцлаа",
    "{user} has shared the board {board} with {acl}" : "{user} {board} самбарыг {acl}-тай хуваалцлаа",
    "You have removed {acl} from the board {board}" : "Та {acl}-г {board} самбараас хаслаа",
    "{user} has removed {acl} from the board {board}" : "{user} {acl}-г {board} самбараас хаслаа",
    "You have renamed the board {before} to {board}" : "Та {before} самбарыг {board} болгон өөрчиллөө",
    "{user} has renamed the board {before} to {board}" : "{user} {before} самбарыг {board} болгон өөрчиллөө",
    "You have archived the board {board}" : "Та {board} самбарыг архивлалаа",
    "{user} has archived the board {before}" : "{user} {before} самбарыг архивлалаа",
    "You have unarchived the board {board}" : "Та {board} самбарыг архиваас гаргалаа",
    "{user} has unarchived the board {before}" : "{user} {before} самбарыг архиваас гаргалаа",
    "You have created a new list {stack} on board {board}" : "Та {board} самбарт шинэ {stack} жагсаалт үүсгэлээ",
    "{user} has created a new list {stack} on board {board}" : "{user} {board} самбарт шинэ {stack} жагсаалт үүсгэлээ",
    "You have renamed list {before} to {stack} on board {board}" : "Та {board} самбарт {before} жагсаалтыг {stack} болгон өөрчиллөө",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} {board} самбарт {before} жагсаалтыг {stack} болгон өөрчиллөө",
    "You have deleted list {stack} on board {board}" : "Та {board} самбараас {stack} жагсаалтыг устгалаа",
    "{user} has deleted list {stack} on board {board}" : "{user} {board} самбараас {stack} жагсаалтыг устгалаа",
    "You have created card {card} in list {stack} on board {board}" : "Та {board} самбарын {stack} жагсаалтад {card} карт үүсгэлээ",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} {board} самбарын {stack} жагсаалтад {card} карт үүсгэлээ",
    "You have deleted card {card} in list {stack} on board {board}" : "Та {board} самбарын {stack} жагсаалтаас {card} картыг устгалаа",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} {board} самбарын {stack} жагсаалтаас {card} картыг устгалаа",
    "You have renamed the card {before} to {card}" : "Та {before} картыг {card} болгон өөрчиллөө",
    "{user} has renamed the card {before} to {card}" : "{user} {before} картыг {card} болгон өөрчиллөө",
    "You have added a description to card {card} in list {stack} on board {board}" : "Та {board} самбарын {stack} жагсаалт дахь {card} картанд тайлбар нэмсэн",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} нь {board} самбарын {stack} жагсаалт дахь {card} картад тайлбар нэмлээ",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Та {board} самбарын {stack} жагсаалт дахь {card} картын тайлбарыг шинэчиллээ",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} нь {board} самбарын {stack} жагсаалт дахь {card} картын тайлбарыг шинэчиллээ",
    "You have archived card {card} in list {stack} on board {board}" : "Та {board} самбарын {stack} жагсаалтын {card} картыг архивлалаа",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} {board} самбарын {stack} жагсаалтын {card} картыг архивлалаа",
    "You have unarchived card {card} in list {stack} on board {board}" : "Та {board} самбарын {stack} жагсаалтын {card} картыг архиваас гаргалаа",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} {board} самбарын {stack} жагсаалтын {card} картыг архиваас гаргалаа",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Та {board} самбарын {stack} жагсаалт дахь {card} картыг дууссан гэж тэмдэглэсэн",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} {board} самбарын {stack} жагсаалтын {card} картыг дууссан гэж тэмдэглэлээ",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Та {board} самбарын {stack} жагсаалт дахь {card} картыг дуусаагүй гэж тэмдэглэсэн",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} {board} самбарын {stack} жагсаалт дахь {card} картыг дуусаагүй гэж тэмдэглэсэн",
    "You have removed the due date of card {card}" : "Та {card} картын дуусах огноог хаслаа",
    "{user} has removed the due date of card {card}" : "{user} {card} картын дуусах огноог хаслаа",
    "You have set the due date of card {card} to {after}" : "Та {card} картын дуусах огноог {after} болгож тохируулалаа",
    "{user} has set the due date of card {card} to {after}" : "{user} {card} картын дуусах огноог {after} болгож тохируулалаа",
    "You have updated the due date of card {card} to {after}" : "Та {card} картын дуусах огноог {after} болгож шинэчиллээ",
    "{user} has updated the due date of card {card} to {after}" : "{user} {card} картын дуусах огноог {after} болгож шинэчиллээ",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Та {board} самбарын {stack} жагсаалт дахь {card} картад {label} шошго нэмлээ",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} нь {board} самбарын {stack} жагсаалт дахь {card} картад {label} шошго нэмлээ",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Та {board} самбарын {stack} жагсаалт дахь {card} картаас {label} шошгыг хаслаа",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} нь {board} самбарын {stack} жагсаалт дахь {card} картаас {label} шошгыг хаслаа",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Та {board} самбарын {card} картанд {assigneduser}-г оноолоо",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} {board} самбарын {card} картанд {assigneduser}-г оноолоо",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Та {board} самбарын {card} картнаас {assigneduser}-г хаслаа",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} {board} самбарын {card} картнаас {assigneduser}-г хаслаа",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Та {card} картыг {stackBefore} жагсаалтаас {stack} руу зөөлөө",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} {card} картыг {stackBefore} жагсаалтаас {stack} руу зөөлөө",
    "You have added the attachment {attachment} to card {card}" : "Та {card} картанд {attachment} хавсралт нэмлээ",
    "{user} has added the attachment {attachment} to card {card}" : "{user} {card} картанд {attachment} хавсралт нэмлээ",
    "You have updated the attachment {attachment} on card {card}" : "Та {card} картын {attachment} хавсралтыг шинэчиллээ",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} {card} картын {attachment} хавсралтыг шинэчиллээ",
    "You have deleted the attachment {attachment} from card {card}" : "Та {card} картнаас {attachment} хавсралтыг устгалаа",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} {card} картнаас {attachment} хавсралтыг устгалаа",
    "You have restored the attachment {attachment} to card {card}" : "Та {card} картын {attachment} хавсралтыг сэргээлээ",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} {card} картын {attachment} хавсралтыг сэргээлээ",
    "You have commented on card {card}" : "Та {card} картанд сэтгэгдэл бичлээ",
    "{user} has commented on card {card}" : "{user} {card} картанд сэтгэгдэл бичлээ",
    "Deck" : "–ê–∂–ª—ã–Ω —Ç–∞–ª–±–∞—Ä",
    "Changes in the <strong>Deck app</strong>" : "<strong>Deck апп</strong>-н өөрчлөлтүүд",
    "A <strong>board, list or card</strong> was changed" : "<strong>Самбар, жагсаалт эсвэл карт</strong> өөрчлөгдлөө",
    "A <strong>comment</strong> was created on a card" : "Картанд <strong>сэтгэгдэл</strong> үүсгэгдлээ",
    "A <strong>card description</strong> has been changed" : "<strong>Картын тайлбар</strong> өөрчлөгдлөө",
    "The file was uploaded" : "Файл байршуулагдлаа",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Байршуулсан файл php.ini дахь upload_max_filesize хязгаараас хэтэрсэн",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Байршуулсан файл нь HTML маягтад заасан MAX_FILE_SIZE хязгаараас хэтэрсэн байна",
    "The file was only partially uploaded" : "Файл зөвхөн хэсэгчлэн байршуулагдсан",
    "No file was uploaded" : "Файл байршуулагдаагүй",
    "Missing a temporary folder" : "Түр хавтас олдохгүй байна",
    "Could not write file to disk" : "Файлыг диск рүү бичиж чадсангүй",
    "A PHP extension stopped the file upload" : "PHP өргөтгөл файл байршуулалтыг зогсоосон",
    "No file uploaded or file size exceeds maximum of %s" : "Файл байршуулаагүй эсвэл файлын хэмжээ %s хамгийн ихээс хэтэрсэн",
    "Invalid file type. Only JSON files are allowed." : "Буруу файлын төрөл. Зөвхөн JSON файл зөвшөөрөгдөнө.",
    "Invalid JSON data" : "Буруу JSON өгөгдөл",
    "Failed to import board" : "Самбар импортлох амжилтгүй болсон",
    "Cards due today" : "Өнөөдөр дуусах картууд",
    "Cards due tomorrow" : "Маргааш дуусах картууд",
    "Upcoming cards" : "Удахгүй болох картууд",
    "Load more" : "Цааш ачаалах",
    "Welcome to Nextcloud Deck!" : "Nextcloud Deck-д тавтай морил!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "\"%s\" дээрх \"%s\" картыг %s танд оноолоо.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} {deck-board} дээрх {deck-card} картыг танд оноолоо.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "\"%2$s\" дээрх \"%1$s\" картын хугацаа дууслаа.",
    "The card {deck-card} on {deck-board} has reached its due date." : "{deck-board} дээрх {deck-card} картын хугацаа дууслаа.",
    "%s has mentioned you in a comment on \"%s\"." : "%s таныг \"%s\" дээрх сэтгэгдэлд дурдлаа.",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} таныг {deck-card} дээрх сэтгэгдэлд дурдлаа.",
    "The board \"%s\" has been shared with you by %s." : "\"%s\" самбарыг %s тантай хуваалцлаа.",
    "{user} has shared {deck-board} with you." : "{user} {deck-board}-г тантай хуваалцлаа.",
    "Deck board" : "Deck самбар",
    "Owned by %1$s" : "%1$s эзэмшилтэй",
    "Deck boards, cards and comments" : "Deck самбар, карт болон сэтгэгдлүүд",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "%1$s-аас, %2$s/%3$s дотор, %4$s эзэмшилтэй",
    "Create a new deck card" : "Шинэ deck карт үүсгэх",
    "Card comments" : "Картын сэтгэгдлүүд",
    "%s on %s" : "%s %s дээр",
    "Deck boards and cards" : "Deck самбар болон картууд",
    "No data was provided to create an attachment." : "Хавсралт үүсгэх өгөгдөл байхгүй.",
    "Finished" : "–î—É—É—Å–≥–∞—Å–∞–Ω",
    "To review" : "–î–∞—Ö–∏–Ω —Ö—è–Ω–∞—Ö",
    "Action needed" : "“Æ–π–ª–¥—ç–ª —à–∞–∞—Ä–¥–ª–∞–≥–∞—Ç–∞–π",
    "Later" : "–•–æ–π—à–ª—É—É–ª–∞—Ö",
    "copy" : "–•—É—É–ª–∞—Ö ",
    "Read more inside" : "Дэлгэрэнгүйг дотроос уншина уу",
    "Custom lists - click to rename!" : "Тохируулсан жагсаалтууд - нэр өөрчлөхийн тулд дарна уу!",
    "To Do" : "Хийх зүйлс",
    "In Progress" : "Явагдаж байна",
    "Done" : "–•–∏–π—Å—ç–Ω",
    "1. Open to learn more about boards and cards" : "1. Самбар болон картын талаар илүү ихийг мэдэхийн тулд нээнэ үү",
    "2. Drag cards left and right, up and down" : "2. Картыг зүүн, баруун, дээш, доош чирнэ үү",
    "3. Apply rich formatting and link content" : "3. Баялаг форматлах болон холбоостой агуулга ашиглана уу",
    "4. Share, comment and collaborate!" : "4. Хуваалцаж, сэтгэгдэл бичиж, хамтран ажиллана уу!",
    "Create your first card!" : "Эхний картаа үүсгээрэй!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Энэ сэтгэгдэл %s-ээс илүү тэмдэгт агуулж байна.\n%s нэртэй картад хавсралт болгон нэмсэн.\nURL хаяг: %s.",
    "Attachments" : "–•–∞–≤—Å—Ä–∞–ª—Ç",
    "File" : "Файл",
    "date" : "огноо",
    "Card not found" : "Карт олдсонгүй",
    "Path is already shared with this card" : "Зам аль хэдийн энэ карттай хуваалцагдсан",
    "Invalid date, date format must be YYYY-MM-DD" : "Огноо буруу, огнооны формат YYYY-MM-DD байх ёстой",
    "Personal planning and team project organization" : "Хувийн төлөвлөлт болон багийн төслийн зохион байгуулалт",
    "Add board" : "Самбар нэмэх",
    "Card details" : "Картын дэлгэрэнгүй",
    "Select the board to link to a project" : "Төсөлтэй холбох самбарыг сонгоно уу",
    "Search by board title" : "Самбарын гарчгаар хайх",
    "Select board" : "Самбар сонгох",
    "Move/copy card" : "Карт зөөх/хуулах",
    "Select a board" : "Самбар сонгох",
    "No lists available" : "Жагсаалт байхгүй",
    "Select a list" : "Жагсаалт сонгох",
    "Move card" : "Карт зөөх",
    "Copy card" : "Карт хуулах",
    "Select the card to link to a project" : "Төсөлтэй холбох картыг сонгоно уу",
    "Link to card" : "Картанд холбох",
    "Select a card" : "Карт сонгох",
    "Cancel" : "–±–æ–ª–∏—É–ª–∞—Ö",
    "This board is read only" : "Энэ самбар зөвхөн унших горимд байна",
    "Drop your files to upload" : "Файлуудаа байршуулахын тулд энд тавина уу",
    "File already exists" : "Файл аль хэдийн байна",
    "A file with the name {filename} already exists." : "{filename} нэртэй файл аль хэдийн байна.",
    "Do you want to overwrite it?" : "Дарж бичихийг хүсэж байна уу?",
    "Overwrite file" : "Файл дарж бичих",
    "Keep existing file" : "Одоогийн файлыг хадгалах",
    "Add card" : "Карт нэмэх",
    "Archived cards" : "Архивлагдсан картууд",
    "Add list" : "Жагсаалт нэмэх",
    "List name" : "Жагсаалтын нэр",
    "Clear search" : "Хайлт арилгах",
    "Active filters" : "Идэвхтэй шүүлтүүрүүд",
    "Apply filter" : "Шүүлтүүр хэрэгжүүлэх",
    "Filter by tag" : "Тагаар шүүх",
    "Filter by assigned user" : "Оноогдсон хэрэглэгчээр шүүх",
    "Unassigned" : "Оноогдоогүй",
    "Filter by status" : "Төлөвөөр шүүх",
    "Open and completed" : "Нээлттэй болон дууссан",
    "Open" : "–ù—ç—ç—Ö",
    "Completed" : "–ì“Ø–π—Ü—ç—Ç–≥—ç–≥–¥—Å—ç–Ω",
    "Filter by due date" : "Дуусах огноогоор шүүх",
    "Overdue" : "Хугацаа хэтэрсэн",
    "Next 24 hours" : "Дараагийн 24 цаг",
    "Next 7 days" : "Дараагийн 7 өдөр",
    "Next 30 days" : "Дараагийн 30 өдөр",
    "No due date" : "Дуусах огноо байхгүй",
    "Clear filter" : "Шүүлтүүр арилгах",
    "View Modes" : "Харах горимууд",
    "Toggle View Modes" : "Харах горим сэлгэх",
    "Hide archived cards" : "–ê—Ä—Ö–∏–≤–ª–∞–≥–¥—Å–∞–Ω –∫–∞—Ä—Ç—É—É–¥—ã–≥ –Ω—É—É—Ö",
    "Show archived cards" : "–ê—Ä—Ö–∏–≤–ª–∞–≥–¥—Å–∞–Ω –∫–∞—Ä—Ç—É—É–¥—ã–≥ —Ö–∞—Ä–∞—Ö",
    "Toggle compact mode" : "Шахмал горим сэлгэх",
    "Hide card cover images" : "Картын нүүр зургийг нуух",
    "Show card cover images" : "Картын нүүр зургийг харуулах",
    "Open details" : "Дэлгэрэнгүйг нээх",
    "Details" : "–î—ç–ª–≥—ç—Ä—ç–Ω–≥“Ø–π",
    "Deck settings" : "Deck тохиргоо",
    "General" : "–µ—Ä”©–Ω—Ö–∏–π",
    "Use bigger card view" : "Том картын харагдац ашиглах",
    "Appearance" : "Гадаад төрх",
    "Show card ID badge" : "Картын ID тэмдэг харуулах",
    "Show boards in calendar/tasks" : "Календар/даалгаварт самбар харуулах",
    "Admin settings" : "Админ тохиргоо",
    "Limit board creation to some groups" : "Самбар үүсгэхийг зарим бүлэгт хязгаарлах",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Тэдгээр бүлгүүдэд хамаарахгүй хэрэглэгчид өөрийн самбар үүсгэх боломжгүй боловч тэдэнд хуваалцсан самбаруудад ажиллах боломжтой хэвээр байна.",
    "Enable federation" : "Холбооныг идэвхжүүлэх",
    "Board actions" : "Самбарын үйлдлүүд",
    "Scroll sideways" : "Хажуу тийш гүйлгэх",
    "Navigate between cards" : "Картуудын хооронд шилжих",
    "Close card details" : "Картын дэлгэрэнгүй хаах",
    "Search" : "–•–∞–π—Ö",
    "Show card filters" : "Картын шүүлтүүр харуулах",
    "Clear card filters" : "Картын шүүлтүүр цэвэрлэх",
    "Show those shortcuts" : "Эдгээр товчлолыг харуулах",
    "Card actions" : "Картын үйлдлүүд",
    "Open card details" : "Картын дэлгэрэнгүй нээх",
    "Edit the card title" : "Картын гарчиг засах",
    "Assign yourself to the current card" : "Одоогийн картанд өөрийгөө оноох",
    "Archive/unarchive the current card" : "Одоогийн картыг архивлах/архиваас гаргах",
    "Mark card as completed/not completed" : "Картыг дууссан/дуусаагүй гэж тэмдэглэх",
    "Open card menu" : "Картын цэс нээх",
    "Currently present people" : "Одоо байгаа хүмүүс",
    "Loading board" : "Самбар ачааллаж байна",
    "Board not found" : "Самбар олдсонгүй",
    "Create a new list to add cards to this board" : "Энэ самбарт карт нэмэхийн тулд шинэ жагсаалт үүсгэнэ үү",
    "Sharing" : "–¢“Ø–≥—ç—ç—Ö",
    "Tags" : "Tag-“Ø“Ø–¥",
    "Deleted items" : "Устгагдсан зүйлс",
    "Activity" : "“Æ–π–ª–¥–ª“Ø“Ø–¥",
    "Deleted lists" : "Устгагдсан жагсаалтууд",
    "Undo" : "–±—É—Ü–∞—Ö",
    "Deleted cards" : "Устгагдсан картууд",
    "Hours" : "Цаг",
    "Days" : "Хоног",
    "Months" : "Сар",
    "Failed to create share with {displayName}" : "{displayName}-тай хуваалцалт үүсгэж чадсангүй",
    "Are you sure you want to transfer the board {title} to {user}?" : "{title} самбарыг {user} руу шилжүүлэхийг хүсэж байна уу?",
    "Transfer the board." : "Самбарыг шилжүүлэх.",
    "Transfer" : "Шилжүүлэх",
    "The board has been transferred to {user}" : "Самбар {user} руу шилжүүлэгдлээ",
    "Failed to transfer the board to {user}" : "Самбарыг {user} руу шилжүүлж чадсангүй",
    "Board owner" : "Самбарын эзэн",
    "(Group)" : "(Бүлэг)",
    "(Team)" : "(Баг)",
    "(remote)" : "(алсын)",
    "Can edit" : "Can edit",
    "Can share" : "Can share",
    "Can manage" : "Удирдах боломжтой",
    "Owner" : "–≠–∑—ç–º—à–∏–≥—á",
    "Delete" : "–£—Å—Ç–≥–∞—Ö",
    "List deleted" : "Жагсаалт устгагдлаа",
    "Edit list title" : "Жагсаалтын гарчиг засах",
    "Archive all cards" : "Бүх картыг архивлах",
    "Unarchive all cards" : "Бүх картыг архиваас гаргах",
    "Delete list" : "Жагсаалт устгах",
    "Archive all cards in this list" : "Энэ жагсаалтын бүх картыг архивлах",
    "Unarchive all cards in this list" : "Энэ жагсаалтын бүх картыг архиваас гаргах",
    "Add a new card" : "Шинэ карт нэмэх",
    "Card name" : "Картын нэр",
    "title and color value must be provided" : "гарчиг болон өнгийн утга өгөх ёстой",
    "Edit" : "–ó–∞—Å–≤–∞—Ä–ª–∞—Ö",
    "Add a new tag" : "Шинэ таг нэмэх",
    "Board name" : "Самбарын нэр",
    "Members" : "–ì–∏—à“Ø“Ø–¥",
    "Assign to users/groups/team" : "Хэрэглэгч/бүлэг/багт оноох",
    "File to share" : "Хуваалцах файл",
    "Invalid path selected" : "Буруу зам сонгосон",
    "Upload new files" : "Шинэ файл байршуулах",
    "Share from Files" : "Файлуудаас хуваалцах",
    "Pending share" : "Хүлээгдэж буй хуваалцалт",
    "Add this attachment" : "Энэ хавсралтыг нэмэх",
    "Show in Files" : "Файлуудад харуулах",
    "Download" : "–¢–∞—Ç–∞—Ö",
    "Remove attachment" : "Хавсралт устгах",
    "Delete Attachment" : "Хавсралт устгах",
    "Restore Attachment" : "Хавсралт сэргээх",
    "The title cannot be empty." : "Гарчиг хоосон байж болохгүй.",
    "Cannot close unsaved card!" : "Хадгалаагүй картыг хааж болохгүй!",
    "Open in sidebar view" : "Хажуугийн самбарт нээх",
    "Open in bigger view" : "Том харагдацад нээх",
    "Comments" : "–°—ç—Ç–≥—ç–≥–¥–ª“Ø“Ø–¥",
    "Failed to load comments" : "Сэтгэгдлүүд ачаалж чадсангүй",
    "No comments yet. Begin the discussion!" : "Сэтгэгдэл байхгүй байна. Хэлэлцүүлгийг эхлүүлнэ үү!",
    "The comment cannot be empty." : "Сэтгэгдэл хоосон байж болохгүй.",
    "The comment cannot be longer than 1000 characters." : "Сэтгэгдэл 1000 тэмдэгтээс урт байж болохгүй.",
    "Submit" : "–º—ç–¥—ç–≥–¥—ç—Ö",
    "Created:" : "“Æ“Ø—Å–≥—ç—Å—ç–Ω:",
    "In reply to" : "Хариулж байна",
    "Cancel reply" : "Хариулт болих",
    "Reply" : "—Ö–∞—Ä–∏—É–ª—Ç",
    "Update" : "–®–∏–Ω—ç—á–ª—ç—Ö",
    "Mark as done" : "Дууссан гэж тэмдэглэх",
    "Mark as not done" : "Дуусаагүй гэж тэмдэглэх",
    "Could not save description" : "Тайлбар хадгалж чадсангүй",
    "Description" : "–¢–∞–π–ª–±–∞—Ä",
    "(Unsaved)" : "(Хадгалаагүй)",
    "Formatting help" : "Форматлалын тусламж",
    "Edit description" : "Тайлбар засах",
    "View description" : "Тайлбар харах",
    "Add Attachment" : "Хавсралт нэмэх",
    "Choose attachment" : "Хавсралт сонгох",
    "Select Date" : "Огноо сонгох",
    "Set due date for later today" : "Өнөөдөр дараа дуусах огноо тохируулах",
    "Set due date for tomorrow" : "Маргааш дуусах огноо тохируулах",
    "Set due date for this weekend" : "Энэ амралтын өдөр дуусах огноо тохируулах",
    "Set due date for next week" : "Дараа долоо хоног дуусах огноо тохируулах",
    "Set a due date" : "Дуусах огноо тохируулах",
    "Add due date" : "Дуусах огноо нэмэх",
    "Choose a date" : "Огноо сонгох",
    "Remove due date" : "Дуусах огноо устгах",
    "Due at:" : "Дуусах огноо:",
    "Not done" : "Дуусаагүй",
    "Unarchive card" : "Картыг архиваас гаргах",
    "Archive card" : "Карт архивлах",
    "Create a new tag:" : "Шинэ таг үүсгэх:",
    "(group)" : "(–±“Ø–ª—ç–≥)",
    "{count} comments, {unread} unread" : "{count} сэтгэгдэл, {unread} уншаагүй",
    "Todo items" : "Хийх зүйлс",
    "Edit card title" : "Картын гарчиг засах",
    "Open link" : "Холбоос нээх",
    "Card deleted" : "Карт устгагдлаа",
    "Edit title" : "Гарчиг засах",
    "Assign to me" : "Надад оноох",
    "Unassign myself" : "Өөрийгөө хасах",
    "Delete card" : "Карт устгах",
    "All boards" : "Бүх самбар",
    "Archived boards" : "Архивлагдсан самбарууд",
    "Shared with you" : "—Ç–∞–Ω—Ç–∞–π —Ö—É–≤–∞–∞–ª—Ü—Å–∞–Ω",
    "Cancel edit" : "Засварлахыг болих",
    "Save board" : "Самбар хадгалах",
    "Board {0} deleted" : "{0} самбар устгагдлаа",
    "All cards" : "Бүх карт",
    "Only assigned cards" : "Зөвхөн оноогдсон картууд",
    "No reminder" : "Сануулга байхгүй",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "{title} самбарыг устгахдаа итгэлтэй байна уу? Энэ нь архивлагдсан картуудыг оролцуулаад самбарын бүх өгөгдлийг устгана.",
    "Delete the board?" : "Самбарыг устгах уу?",
    "Are you sure you want to leave the board {title}?" : "{title} самбараас гарахыг хүсэж байна уу?",
    "Leave the board?" : "Самбараас гарах уу?",
    "Leave" : "Орхих",
    "Failed to leave the board" : "Самбараас гарж чадсангүй",
    "Exporting board..." : "Самбарыг экспортлож байна...",
    "An error occurred" : "Алдаа гарлаа",
    "Board details" : "Самбарын дэлгэрэнгүй",
    "Edit board" : "Самбар засах",
    "Clone board" : "Самбар хуулбарлах",
    "Unarchive board" : "Самбарыг архиваас гаргах",
    "Archive board" : "Самбар архивлах",
    "Export board" : "Самбар экспортлох",
    "Turn on due date reminders" : "Дуусах огнооны сануулга идэвхжүүлэх",
    "Turn off due date reminders" : "Дуусах огнооны сануулга унтраах",
    "Due date reminders" : "Дуусах огнооны сануулгууд",
    "Assigned cards" : "Оноогдсон картууд",
    "No notifications" : "–ú—ç–¥—ç–≥–¥—ç–ª –±–∞–π—Ö–≥“Ø–π",
    "Delete board" : "Самбар устгах",
    "Leave board" : "Самбараас гарах",
    "Importing board..." : "Самбарыг импортлож байна...",
    "Board imported successfully" : "Самбар амжилттай импортлогдлоо",
    "Import board" : "Самбар импортлох",
    "Clone {boardTitle}" : "{boardTitle} хуулбарлах",
    "Clone cards" : "Картууд хуулбарлах",
    "Clone assignments" : "Оноолтууд хуулбарлах",
    "Clone labels" : "Шошгонууд хуулбарлах",
    "Clone due dates" : "Дуусах огноонууд хуулбарлах",
    "Advanced options" : "–ë—É—Å–∞–¥ —Å–æ–Ω–≥–æ–ª—Ç—É—É–¥",
    "Move all cards to the first list" : "Бүх картыг эхний жагсаалтад зөөх",
    "Restore archived cards" : "Архивлагдсан картуудыг сэргээх",
    "Clone" : "Хуулбарлах",
    "Export {boardTitle}" : "{boardTitle} экспортлох",
    "Export as JSON" : "JSON болгон экспортлох",
    "Export as CSV" : "CSV болгон экспортлох",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Тэмдэглэл: Deck програмд дахин импортлоход зөвхөн JSON форматыг дэмждэг.",
    "Export" : "–≠–∫—Å–ø–æ—Ä—Ç",
    "Loading filtered view" : "Шүүгдсэн харагдацыг ачааллаж байна",
    "Search for {searchQuery} in other boards" : "Бусад самбаруудаас {searchQuery} хайх",
    "Search for {searchQuery} in all boards" : "Бүх самбаруудаас {searchQuery} хайх",
    "No results found" : "Үр дүн олдсонгүй",
    "Deck board {name}\n* Last modified on {lastMod}" : "Deck самбар: {name}\n* Сүүлд өөрчлөгдсөн: {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Үүсгэсэн: {created}\n* Сүүлд өөрчлөгдсөн: {lastMod}\n* {nbAttachments} хавсралт\n* {nbComments} сэтгэгдэл",
    "{nbCards} cards" : "{nbCards} карт",
    "Due on {date}" : "{date}-д дуусна",
    "{stack} in {board}" : "{board} дахь {stack}",
    "Click to expand description" : "Тайлбарыг дэлгэхийн тулд дарна уу",
    "Click to expand comment" : "Сэтгэгдлийг дэлгэхийн тулд дарна уу",
    "Create card" : "Карт үүсгэх",
    "Create a new card" : "Шинэ карт үүсгэх",
    "Card title" : "Картын гарчиг",
    "Card \"{card}\" was added to \"{board}\"" : "\"{card}\" карт \"{board}\"-д нэмэгдлээ",
    "Open card" : "Карт нээх",
    "Close" : "‚Äì‚Ä¢‚Äì‚àû‚Äì‚àû‚Äî√ñ",
    "No upcoming cards" : "Удахгүй болох карт байхгүй",
    "upcoming cards today" : "өнөөдөр удахгүй болох картууд",
    "upcoming cards tomorrow" : "маргааш удахгүй болох картууд",
    "upcoming cards" : "удахгүй болох картууд",
    "New card" : "Шинэ карт",
    "Link to a board" : "Самбартай холбох",
    "Link to a card" : "Карттай холбох",
    "Create a card" : "Карт үүсгэх",
    "Message from {author} in {conversationName}" : "{conversationName} дахь {author}-ийн мессеж",
    "Something went wrong" : "Ямар нэг зүйл буруу болсон",
    "Failed to upload {name}" : "{name} байршуулж чадсангүй",
    "Maximum file size of {size} exceeded" : "Файлын хамгийн их хэмжээ {size}-с хэтэрлээ",
    "Assigned users" : "Оноогдсон хэрэглэгчид",
    "Due date" : "Дуусах огноо",
    "Created" : "“Æ“Ø—Å–≥—ç—Å—ç–Ω",
    "Modified" : "”®”©—Ä—á–ª”©–≥–¥—Å”©–Ω",
    "Error creating the share" : "Хуваалцалт үүсгэхэд алдаа гарлаа",
    "Share with a Deck card" : "Deck карттай хуваалцах",
    "Share {file} with a Deck card" : "{file}-г Deck карттай хуваалцах",
    "Share" : "–¢“Ø–≥—ç—ç—Ö",
    "The card \"%s\" on \"%s\" has reached its due date." : "\"%s\" дээрх \"%s\" картын хугацаа дууслаа.",
    "No participants found" : "Оролцогч олдсонгүй",
    "Save" : "–•–∞–¥–≥–∞–ª–∞—Ö",
    "seconds ago" : "—Ö–æ—ë—Ä–¥–∞—Ö—å ”©–º–Ω”©",
    "Today" : "”©–Ω”©”©–¥”©—Ä",
    "Tomorrow" : "–º–∞—Ä–≥–∞–∞—à",
    "No due" : "Дуусах огноо байхгүй"
},
"nplurals=2; plural=(n != 1);");
