<?php

declare(strict_types=1);

/**
 * SPDX-FileCopyrightText: 2022 Nextcloud GmbH and Nextcloud contributors
 * SPDX-License-Identifier: AGPL-3.0-or-later
 */

namespace OCA\Deck;

class NotifyPushEvents {
	public const DeckBoardUpdate = 'deck_board_update';
	public const DeckCardUpdate = 'deck_card_update';
}
