OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Ha creado un nuevo tablero {board}",
    "{user} has created a new board {board}" : "{user} ha creado un nuevo tablero {board}",
    "You have deleted the board {board}" : "Eliminó el tablero {board}",
    "{user} has deleted the board {board}" : "{user} eliminó el tablero {board}",
    "You have restored the board {board}" : "Restauró el tablero {board}",
    "{user} has restored the board {board}" : "{user} restaruó el tablero {board}",
    "You have shared the board {board} with {acl}" : "Compartió el tablero {board} con {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} compartió el tablero {board} con {acl}",
    "You have removed {acl} from the board {board}" : "Eliminó {acl} del tablero {board}",
    "{user} has removed {acl} from the board {board}" : "{user} removió {acl} del tablero {board}",
    "You have renamed the board {before} to {board}" : "Renombró el tablero {before} a {board}",
    "{user} has renamed the board {before} to {board}" : "{user} renombró el tablero {before} a {board}",
    "You have archived the board {board}" : "Archivó el tablero {board}",
    "{user} has archived the board {before}" : "{user} archivó el tablero {before}",
    "You have unarchived the board {board}" : "Desarchivó el tablero {board}",
    "{user} has unarchived the board {before}" : "{user} desarchivó el tablero {before}",
    "You have created a new list {stack} on board {board}" : "Creó una nueva lista {stack} en el tablero {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} creó una nueva lista {stack} en el tablero {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Renombró la lista {before} a {stack} en el tablero {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} renombró la lista {before} a {stack} en el tablero {board}",
    "You have deleted list {stack} on board {board}" : "Eliminó la lista {stack} en el tablero {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} eliminó la lista {stack} en el tablero {board}",
    "You have created card {card} in list {stack} on board {board}" : "Creó la tarjeta {card} en la lista {stack} en el tablero {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} creó la tarjeta {card} en la lista {stack} en el tablero {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Eliminó la tarjeta {card} de la lista {stack} en el tablero {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} eliminó la tarjeta {card} de la lista {stack} en el tablero {board}",
    "You have renamed the card {before} to {card}" : "Renombró la tarjeta {before} a {card}",
    "{user} has renamed the card {before} to {card}" : "{user} renombró la tarjeta {before} a {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Añadió una descripción a la tarjeta {card} de la lista {stack} en el tablero {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} añadió una descripción a la tarjeta {card} de la lista {stack} en el tablero {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Actualizó la descripción de la tarjeta {card} de la lista {stack} en el tablero {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} actualizó la descripción de la tarjeta {card} de la lista {stack} en el tablero {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Archivó la tarjeta {card} de la lista {stack} en el tablero {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} archivó la tarjeta {card} de la lista {stack} en el tablero {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Desarchivó la tarjeta {card} de la lista {stack} en el tablero {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} desarchivó la tarjeta {card} de la lista {stack} en el tablero {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Marcó la tarjeta {card} como completada de la lista {stack} en el tablero {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} marcó la tarjeta {card} como completada de la lista {stack} en el tablero {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Marcó la tarjeta {card} como no completada de la lista {stack} en el tablero {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} marcó la tarjeta {card} como no completada de la lista {stack} en el tablero {board}",
    "You have removed the due date of card {card}" : "Eliminó la fecha límite de la tarjeta {card}",
    "{user} has removed the due date of card {card}" : "{user} eliminó la fecha límite de la tarjeta {card}",
    "You have set the due date of card {card} to {after}" : "Estableció la fecha límite de la tarjeta {card} como {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} estableció la fecha límite de la tarjeta {card} como {after}",
    "You have updated the due date of card {card} to {after}" : "Actualizó la fecha límite de la tarjeta {card} como {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} actualizó la fecha límite de la tarjeta {card} como {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Añadió la etiqueta {label} a la tarjeta {card} de la lista {stack} en el tablero {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} añadió la etiqueta {label} a la tarjeta {card} de la lista {stack} en el tablero {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Eliminó la etiqueta {label} de la tarjeta {card} de la lista {stack} en el tablero {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} eliminó la etiqueta {label} de la tarjeta {card} de la lista {stack} en el tablero {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Asignó a {assigneduser} la tarjeta {card} en el tablero {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} asignó a {assigneduser} la tarjeta {card} en el tablero {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Quitó a {assigneduser} de la tarjeta {card} en el tablero {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} quitó a {assigneduser} de la tarjeta {card} en el tablero {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Movió la tarjeta {card} de la lista {stackBefore} a {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} movió la tarjeta {card} de la lista {stackBefore} a {stack}",
    "You have added the attachment {attachment} to card {card}" : "Añadió el adjunto {attachment} a la tarjeta {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} añadió el adjunto {attachment} a la tarjeta {card}",
    "You have updated the attachment {attachment} on card {card}" : "Actualizó el adjunto {attachment} de la tarjeta {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} actualizó el adjunto {attachment} de la tarjeta {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Eliminó el adjunto {attachment} de la tarjeta {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} eliminó el adjunto {attachment} de la tarjeta {card}",
    "You have restored the attachment {attachment} to card {card}" : "Restauró el adjunto {attachment} de la tarjeta {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} restauró el adjunto {attachment} de la tarjeta {card}",
    "You have commented on card {card}" : "Comentó en la tarjeta {card}",
    "{user} has commented on card {card}" : "{user} comentó en la tarjeta {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Cambios en la aplicación <strong>Deck</strong>",
    "A <strong>board, list or card</strong> was changed" : "Un <strong>tablero, lista o tarjeta</strong> cambió",
    "A <strong>comment</strong> was created on a card" : "Se creó un <strong>comentario</strong> en una tarjeta",
    "A <strong>card description</strong> has been changed" : "Se cambió una <strong>descripción de tarjeta</strong>",
    "The file was uploaded" : "El archivo fue cargado",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "El archivo cargado excede el valor establecido en la directiva upload_max_filesize en el archivo php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "El archivo cargado excede el valor especificado de la directiva MAX_FILE_SIZE en la forma de HTML",
    "The file was only partially uploaded" : "El archivo sólo fue cargado parcialmente",
    "No file was uploaded" : "No se cargó el archivo",
    "Missing a temporary folder" : "Falta una carpeta temporal",
    "Could not write file to disk" : "No fue posible escribir a disco",
    "A PHP extension stopped the file upload" : "Una extensión de PHP detuvo la carga del archivo",
    "No file uploaded or file size exceeds maximum of %s" : "No se ha cargado ningún archivo o el tamaño del archivo excede el máximo de %s",
    "Cards due today" : "Tarjetas para hoy",
    "Cards due tomorrow" : "Tarjetas para mañana",
    "Upcoming cards" : "Tarjetas próximas",
    "Load more" : "Cargar más",
    "The board \"%s\" has been shared with you by %s." : "El tablero \"%s\" ha sido compartido contigo por %s.",
    "Deck board" : "Tablero de Deck",
    "%s on %s" : "%s en %s",
    "Finished" : "Terminado",
    "To review" : "Para revisar",
    "Action needed" : "Acción requerida",
    "Later" : "Después",
    "copy" : "copiar",
    "To Do" : "Pendiente",
    "In Progress" : "En proceso",
    "Done" : "Terminado",
    "Attachments" : "Adjuntos",
    "File" : "Archivo",
    "Card not found" : "No se encontró la tarjeta",
    "Invalid date, date format must be YYYY-MM-DD" : "La fecha es inválida, por favor sigue el formato AAAA-MM-DD",
    "Add board" : "Añadir pizarra",
    "Move card" : "Mover tarjeta",
    "Cancel" : "Cancelar",
    "Drop your files to upload" : "Arrastre sus archivos para cargar",
    "File already exists" : "El archivo ya existe",
    "Do you want to overwrite it?" : "¿Deseas sobre escribirlo?",
    "Add card" : "Agregar tarjeta",
    "Archived cards" : "Tarjetas archivadas",
    "Add list" : "Añadir lista",
    "List name" : "Nombre de lista",
    "Clear search" : "Limpiar búsqueda",
    "Active filters" : "Filtros activos",
    "Filter by tag" : "Filtrar por etiqueta",
    "Unassigned" : "No asignado",
    "Open" : "Abrir",
    "Completed" : "Completado",
    "Overdue" : "Con retraso",
    "Next 7 days" : "Próximos 7 días",
    "Next 30 days" : "Próximos 30 días",
    "No due date" : "Sin fecha de vencimiento",
    "Clear filter" : "Limpiar filtro",
    "Hide archived cards" : "Ocultar tarjetas archivadas",
    "Show archived cards" : "Mostrar tarjetas archivadas",
    "Open details" : "Abrir detalles",
    "Details" : "Detalles",
    "General" : "General",
    "Appearance" : "Apariencia",
    "Search" : "Buscar",
    "Sharing" : "Compartiendo",
    "Tags" : "Etiquetas",
    "Activity" : "Actividad",
    "Undo" : "Deshacer",
    "Hours" : "Horas",
    "Days" : "Días",
    "Weeks" : "Semanas",
    "Months" : "Meses",
    "No cards yet" : "Aún no hay tarjetas",
    "Failed to create share with {displayName}" : "Fallo al crear el recurso compartido denominado {displayName}",
    "Transfer" : "Transferir",
    "(Group)" : "(Grupo)",
    "(remote)" : "(remoto)",
    "Can edit" : "Puede editar",
    "Can share" : "Puede compartir",
    "Can manage" : "Puede gestionar",
    "Owner" : "Dueño",
    "Delete" : "Borrar",
    "Delete list" : "Eliminar lista",
    "Edit" : "Editar",
    "Members" : "Miembros",
    "Invalid path selected" : "Ruta seleccionada no válida.",
    "Upload new files" : "Cargar nuevos archivos",
    "Share from Files" : "Compartir desde Archivos",
    "Show in Files" : "Mostrar en Archivos",
    "Download" : "Descargar",
    "Comments" : "Comentarios",
    "Submit" : "Enviar",
    "Created:" : "Creado:",
    "Reply" : "Responder",
    "Update" : "Actualizar",
    "Mark as done" : "Marcar como hecho",
    "Description" : "Descripción",
    "Formatting help" : "Ayuda de formato",
    "Edit description" : "Editar descripción",
    "Remove due date" : "Eliminar fecha de expiración",
    "Not done" : "No está finalizado",
    "Archive card" : "Archivar tarjeta",
    "(group)" : "(grupo)",
    "Delete card" : "Eliminar tarjeta",
    "Archived boards" : "Tableros archivados",
    "Shared with you" : "Compartido con usted",
    "Cancel edit" : "Cancelar edición",
    "No reminder" : "Sin recordatorio",
    "An error occurred" : "Ha ocurrido un error",
    "Board details" : "Detalles del tablero",
    "Edit board" : "Editar el tablero",
    "Clone board" : "Clonar pizarra",
    "Unarchive board" : "Desarchivar tablero",
    "Archive board" : "Archivar tablero",
    "No notifications" : "No hay notificaciones",
    "Delete board" : "Borrar tableros",
    "Clone cards" : "Clonar tarjetas",
    "Advanced options" : "Opciones avanzadas",
    "Clone" : "Clonar",
    "Export" : "Exportar",
    "Close" : "Cerrar",
    "Due date" : "Fecha de vencimiento",
    "Created" : "Creado",
    "Modified" : "Modificado",
    "Error creating the share" : "Error creando el recurso compartido",
    "Share" : "Compartir",
    "The card \"%s\" on \"%s\" has reached its due date." : "La tarjeta \"%s\" en \"%s\" ha alacanzado su fecha de entrega",
    "No participants found" : "No se encontraron participantes",
    "Save" : "Guardar",
    "seconds ago" : "hace segundos",
    "Today" : "Hoy",
    "Tomorrow" : "Mañana"
},
"nplurals=3; plural=n == 1 ? 0 : n != 0 && n % 1000000 == 0 ? 1 : 2;");
