OC.L10N.register(
    "deck",
    {
    "Deck" : "Tarxeteru",
    "The file was uploaded" : "Xubióse'l ficheru",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "El ficheru xubíu supera la direutiva «upload_max_filesize» del ficheru php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "El ficheru xubíu supera la direutiva «MAX_FILE_SIZE» que s'especificó nel formulariu HTML",
    "No file was uploaded" : "Nun se xubió nengún ficheru",
    "Missing a temporary folder" : "Falta una carpeta temporal",
    "Could not write file to disk" : "Nun se pudo escribir el ficheru nel discu",
    "A PHP extension stopped the file upload" : "Una estensión PHP paró la xuba de ficheros",
    "No file uploaded or file size exceeds maximum of %s" : "Nun se xubió nengún ficheru o'l tamañu del ficheru supera'l máximu de %s",
    "Upcoming cards" : "Tarxetes próximes",
    "Load more" : "Cargar más",
    "Deck board" : "Mesa de tarxetes",
    "%s on %s" : "%s n %s",
    "Action needed" : "Aición necesaria",
    "copy" : "copia",
    "Done" : "Fecho",
    "File" : "Ficheru",
    "Invalid date, date format must be YYYY-MM-DD" : "La data ye inválida, el so formatu ha ser AAAA-MM-DD",
    "Move card" : "Mover la tarxeta",
    "Cancel" : "Encaboxar",
    "Drop your files to upload" : "Suelta los ficheros pa xubilos",
    "File already exists" : "El ficheru yá esiste",
    "Add card" : "Amestar una tarxeta",
    "Archived cards" : "Tarxetes archivaes",
    "Add list" : "Amestar una llista",
    "List name" : "Nome de la llista",
    "Open" : "Abrir",
    "Completed" : "Completáu",
    "Next 24 hours" : "Les próximes 24 hores",
    "Next 7 days" : "Los próximos 7 díes",
    "Next 30 days" : "Los próximos 30 díes",
    "Open details" : "Abrir los detalles",
    "Details" : "Detalles",
    "General" : "Xeneral",
    "Appearance" : "Aspeutu",
    "Search" : "Guetar",
    "Sharing" : "Comparticiñon",
    "Tags" : "Etiquetes",
    "Deleted items" : "Elementos desaniciaos",
    "Activity" : "Actividá",
    "Deleted lists" : "Llistes desaniciaes",
    "Undo" : "Desfacer",
    "Deleted cards" : "Tarxetes desaniciaes",
    "Hours" : "Hores",
    "Days" : "Díes",
    "Weeks" : "Selmanes",
    "Months" : "Meses",
    "No cards yet" : "Nun hai nenguna tarxeta",
    "Transfer" : "Tresferir",
    "(Group)" : "(Grupu)",
    "(Team)" : "(Equipu)",
    "(remote)" : "(remotu)",
    "Can edit" : "Pue editar",
    "Owner" : "Propietariu",
    "Delete" : "Desaniciar",
    "List deleted" : "Desanicióse la llista",
    "Edit list title" : "Editar el títulu de la llista",
    "Archive all cards" : "Archivar toles carpetes",
    "Unarchive all cards" : "Desarchivar toles tarxetes",
    "Delete list" : "Desaniciar la llista",
    "Archive all cards in this list" : "Archivar toles tarxetes d'esta llista",
    "Unarchive all cards in this list" : "Desarchivar toles tarxetes d'esta llista",
    "Add a new card" : "Amestar una tarxeta nueva",
    "Card name" : "Nome de la tarxeta",
    "Edit" : "Editar",
    "Add a new tag" : "Amestar una etiqueta nueva",
    "Members" : "Miembros",
    "Invalid path selected" : "Seleicionóse un camín inválidu",
    "Show in Files" : "Amosar en Ficheros",
    "Download" : "Baxar",
    "The title cannot be empty." : "El títulu nun pue tar baleru",
    "Comments" : "Comentarios",
    "Failed to load comments" : "Nun se puen cargar los comentarios",
    "No comments yet. Begin the discussion!" : "Nun hai nengún comentariu. ¡Comienza'l discutiniu!",
    "The comment cannot be empty." : "El comentariu nun pue tar baleru.",
    "Submit" : "Unviar",
    "Created:" : "Creóse'l",
    "In reply to" : "En rempuesta a",
    "Reply" : "Responder",
    "Update" : "Anovar",
    "Mark as done" : "Marcar como fecha",
    "Description" : "Descripción",
    "(Unsaved)" : "(ensin guardar)",
    "(Saving…)" : "(Guardando…)",
    "Formatting help" : "Ayuda del formatu",
    "Edit description" : "Editar la descripción",
    "View description" : "Ver la descripción",
    "Tomorrow – {timeLocale}" : "Mañana – {timeLocale}",
    "This weekend – {timeLocale}" : "Esta fin de selmana – {timeLocale}",
    "Next week – {timeLocale}" : "La próxima selmana – {timeLocale}",
    "Archive card" : "Archivar la tarxeta",
    "(group)" : "(grupu)",
    "Edit title" : "Editar el títulu",
    "Delete card" : "Desaniciar la tarxeta",
    "Shared with you" : "Compartióse contigo",
    "Cancel edit" : "Anular la edición",
    "All cards" : "Toles tarxetes",
    "No reminder" : "Nun hai nengún recordatoriu",
    "Leave" : "Colar",
    "An error occurred" : "Prodúxose un error",
    "No notifications" : "Nun hai nengún avisu",
    "Clone cards" : "Clonar les tarxetes",
    "Advanced options" : "Opciones avanzaes",
    "Export" : "Esportar",
    "No results found" : "Nun s'atopó nengún resultáu",
    "Close" : "Zarrar",
    "Message from {author} in {conversationName}" : "Mensaxe de {author} en {conversationName}",
    "Due date" : "Data de fin",
    "Modified" : "Modificóse",
    "Error creating the share" : "Hebo un error al crear la compartición",
    "Share" : "Compartir",
    "Save" : "Guardar",
    "seconds ago" : "hai segundos",
    "Today" : "Güei",
    "Tomorrow" : "Mañana"
},
"nplurals=2; plural=(n != 1);");
