OC.L10N.register(
    "deck",
    {
    "You have created a new board {board}" : "Hai creato una nuova lavagna {board}",
    "{user} has created a new board {board}" : "{user} ha creato una nuova lavagna {board}",
    "You have deleted the board {board}" : "Hai eliminato la lavagna {board}",
    "{user} has deleted the board {board}" : "{user} ha eliminato la lavagna {board}",
    "You have restored the board {board}" : "Hai ripristinato la lavagna {board}",
    "{user} has restored the board {board}" : "{user} ha ripristinato la lavagna {board}",
    "You have shared the board {board} with {acl}" : "Hai condiviso la lavagna {board} con {acl}",
    "{user} has shared the board {board} with {acl}" : "{user} ha condiviso la lavagna {board} con {acl}",
    "You have removed {acl} from the board {board}" : "Hai rimosso {acl} dalla lavagna {board}",
    "{user} has removed {acl} from the board {board}" : "{user} ha rimosso {acl} dalla lavagna {board}",
    "You have renamed the board {before} to {board}" : "Hai rinominato la lavagna {before} in {board}",
    "{user} has renamed the board {before} to {board}" : "{user} ha rinominato la lavagna {before} in {board}",
    "You have archived the board {board}" : "Hai archiviato la lavagna {board}",
    "{user} has archived the board {before}" : "{user} ha archiviato la lavagna {before}",
    "You have unarchived the board {board}" : "Hai annullato l'archiviazione della lavagna {board}",
    "{user} has unarchived the board {before}" : "{user} ha annullato l'archiviazione della lavagna {board}",
    "You have created a new list {stack} on board {board}" : "Hai creato un nuovo elenco {stack} sulla lavagna {board}",
    "{user} has created a new list {stack} on board {board}" : "{user} ha creato un nuovo elenco {stack} sulla lavagna {board}",
    "You have renamed list {before} to {stack} on board {board}" : "Hai rinominato l'elenco {before} in {stack} sulla lavagna {board}",
    "{user} has renamed list {before} to {stack} on board {board}" : "{user} ha rinominato l'elenco {before} in {stack} sulla lavagna {board}",
    "You have deleted list {stack} on board {board}" : "Hai eliminato l'elenco {stack} sulla lavagna {board}",
    "{user} has deleted list {stack} on board {board}" : "{user} ha eliminato l'elenco {stack} sulla lavagna {board}",
    "You have created card {card} in list {stack} on board {board}" : "Hai creato la scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "{user} has created card {card} in list {stack} on board {board}" : "{user} ha creato la scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "You have deleted card {card} in list {stack} on board {board}" : "Hai eliminato la scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "{user} has deleted card {card} in list {stack} on board {board}" : "{user} ha creato la scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "You have renamed the card {before} to {card}" : "Hai rinominato la scheda {before} in {card}",
    "{user} has renamed the card {before} to {card}" : "{user} ha rinominato la scheda {before} in {card}",
    "You have added a description to card {card} in list {stack} on board {board}" : "Hai aggiunto una descrizione alla scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "{user} has added a description to card {card} in list {stack} on board {board}" : "{user} ha aggiunto una descrizione alla scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "You have updated the description of card {card} in list {stack} on board {board}" : "Hai aggiornato la descrizione della scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "{user} has updated the description of the card {card} in list {stack} on board {board}" : "{user} ha aggiornato la descrizione della scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "You have archived card {card} in list {stack} on board {board}" : "Hai archiviato la scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "{user} has archived card {card} in list {stack} on board {board}" : "{user} ha archiviato la scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "You have unarchived card {card} in list {stack} on board {board}" : "Hai annullato l'archiviazione della scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "{user} has unarchived card {card} in list {stack} on board {board}" : "{user} ha annullato l'archiviazione della scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "You have marked the card {card} as done in list {stack} on board {board}" : "Hai segnato la scheda {card} come fatta nell'elenco {stack} sulla lavagna {board}",
    "{user} has marked card {card} as done in list {stack} on board {board}" : "{user} ha segnato la scheda {card} come fatta nell'elenco {stack} sulla lavagna {board}",
    "You have marked the card {card} as undone in list {stack} on board {board}" : "Hai segnato la scheda {card} come non fatta nell'elenco {stack} sulla lavagna {board}",
    "{user} has marked the card {card} as undone in list {stack} on board {board}" : "{user} ha segnato la scheda {card} come non fatta nell'elenco {stack} sulla lavagna {board}",
    "You have removed the due date of card {card}" : "Hai rimosso la data di scadenza della scheda {card}",
    "{user} has removed the due date of card {card}" : "{user} ha rimosso la data di scadenza dalla scheda {card}",
    "You have set the due date of card {card} to {after}" : "Hai impostato la data di scadenza della scheda {card} a {after}",
    "{user} has set the due date of card {card} to {after}" : "{user} ha impostato la data di scadenza della scheda {card} a {after}",
    "You have updated the due date of card {card} to {after}" : "Hai aggiornato la data di scadenza della scheda {card} a {after}",
    "{user} has updated the due date of card {card} to {after}" : "{user} ha aggiornato la data di scadenza della scheda {card} a {after}",
    "You have added the tag {label} to card {card} in list {stack} on board {board}" : "Hai aggiunto l'etichetta {label} alla scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "{user} has added the tag {label} to card {card} in list {stack} on board {board}" : "{user} ha aggiunto l'etichetta {label} alla scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "You have removed the tag {label} from card {card} in list {stack} on board {board}" : "Hai rimosso l'etichetta {label} alla scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "{user} has removed the tag {label} from card {card} in list {stack} on board {board}" : "{user} ha rimosso l'etichetta {label} alla scheda {card} nell'elenco {stack} sulla lavagna {board}",
    "You have assigned {assigneduser} to card {card} on board {board}" : "Hai assegnato {assigneduser} alla scheda {card} sulla lavagna {board}",
    "{user} has assigned {assigneduser} to card {card} on board {board}" : "{user} ha assegnato {assigneduser} alla scheda {card} sulla lavagna {board}",
    "You have unassigned {assigneduser} from card {card} on board {board}" : "Hai annullato l'assegnazione di {assigneduser} da scheda {card} sulla lavagna {board}",
    "{user} has unassigned {assigneduser} from card {card} on board {board}" : "{user} ha annullato l'assegnazione di {assigneduser} dalla scheda {card} sulla lavagna {board}",
    "You have moved the card {card} from list {stackBefore} to {stack}" : "Hai spostato la scheda {card} dall'elenco {stackBefore} in {stack}",
    "{user} has moved the card {card} from list {stackBefore} to {stack}" : "{user} ha spostato la scheda {card} dall'elenco {stackBefore} in {stack}",
    "You have added the attachment {attachment} to card {card}" : "Hai aggiunto l'allegato {attachment} alla scheda {card}",
    "{user} has added the attachment {attachment} to card {card}" : "{user} ha aggiunto l'allegato {attachment} alla scheda {card}",
    "You have updated the attachment {attachment} on card {card}" : "Hai aggiornato l'allegato {attachment} sulla scheda {card}",
    "{user} has updated the attachment {attachment} on card {card}" : "{user} ha aggiornato l'allegato {attachment} nella scheda {card}",
    "You have deleted the attachment {attachment} from card {card}" : "Hai eliminato l'allegato {attachment} dalla scheda {card}",
    "{user} has deleted the attachment {attachment} from card {card}" : "{user} ha eliminato l'allegato {attachment} dalla scheda {card}",
    "You have restored the attachment {attachment} to card {card}" : "Hai ripristinato l'allegato {attachment} nella scheda {card}",
    "{user} has restored the attachment {attachment} to card {card}" : "{user} ha ripristinato l'allegato {attachment} nella scheda {card}",
    "You have commented on card {card}" : "Hai commentato sulla scheda {card}",
    "{user} has commented on card {card}" : "{user} ha commentato sulla scheda {card}",
    "Deck" : "Deck",
    "Changes in the <strong>Deck app</strong>" : "Modifiche nell'<strong>applicazione Deck</strong>",
    "A <strong>board, list or card</strong> was changed" : "Una <strong>lavagna, lista o scheda</strong> è stata modificata",
    "A <strong>comment</strong> was created on a card" : "Un <strong>commento</strong> è stato creato su una scheda",
    "A <strong>card description</strong> has been changed" : "Una <strong>descrizione di scheda</strong> è stata modificata",
    "The file was uploaded" : "Il file è stato caricato",
    "The uploaded file exceeds the upload_max_filesize directive in php.ini" : "Il file caricato supera la direttiva upload_max_filesize in php.ini",
    "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" : "Il file caricato supera la direttiva MAX_FILE_SIZE specificata nel modulo HTML",
    "The file was only partially uploaded" : "Il file è stato caricato solo parzialmente",
    "No file was uploaded" : "Non è stato caricato alcun file",
    "Missing a temporary folder" : "Manca una cartella temporanea",
    "Could not write file to disk" : "Impossibile scrivere il file su disco",
    "A PHP extension stopped the file upload" : "Un'estensione PHP ha fermato il caricamento del file",
    "No file uploaded or file size exceeds maximum of %s" : "Nessun file caricato o la dimensione del file supera il massimo di %s",
    "Invalid file type. Only JSON files are allowed." : "Tipo di file non valido. Sono consentiti solo file JSON.",
    "Invalid JSON data" : "Dati JSON non validi",
    "Failed to import board" : "Impossibile importare la lavagna",
    "Cards due today" : "Schede che scadono oggi",
    "Cards due tomorrow" : "Schede che scadono domani",
    "Upcoming cards" : "Prossime schede",
    "Load more" : "Carica altro",
    "Welcome to Nextcloud Deck!" : "Benvenuti su Nextcloud Deck!",
    "The card \"%s\" on \"%s\" has been assigned to you by %s." : "La scheda \"%s\" in \"%s\" ti è stata assegnata da %s.",
    "{user} has assigned the card {deck-card} on {deck-board} to you." : "{user} ti ha assegnato la scheda {deck-card} in {deck-board}.",
    "The card \"%1$s\" on \"%2$s\" has reached its due date." : "La scheda \"%1$s \" su \" %2$s\" ha raggiunto la data di scadenza.",
    "The card {deck-card} on {deck-board} has reached its due date." : "La scheda {deck-card} in {deck-board} ha raggiunto la sua data di scadenza.",
    "%s has mentioned you in a comment on \"%s\"." : "%s ti ha menzionato in un commento su \"%s\".",
    "{user} has mentioned you in a comment on {deck-card}." : "{user} ti ha menzionato in un commento su {deck-card}.",
    "The board \"%s\" has been shared with you by %s." : "La lavagna \"%s\" è stata condivisa con te da %s.",
    "{user} has shared {deck-board} with you." : "{user} ha condiviso {deck-board} con te.",
    "Deck board" : "Lavagna di Deck",
    "Owned by %1$s" : "Proprietario: %1$s",
    "Deck boards, cards and comments" : "Lavagne, schede e commenti di Deck",
    "From %1$s, in %2$s/%3$s, owned by %4$s" : "Da %1$s, in %2$s/%3$s, proprietario %4$s",
    "Create a new deck card" : "Crea una nuova scheda di Deck",
    "Card comments" : "Commenti",
    "%s on %s" : "%s su %s",
    "Deck boards and cards" : "Lavagne e schede di Deck",
    "No data was provided to create an attachment." : "Non è  stato fornito alcun dato per creare un allegato.",
    "Finished" : "Finito",
    "To review" : "Da revisionare",
    "Action needed" : "Azione necessaria",
    "Later" : "Più tardi",
    "copy" : "copia",
    "Read more inside" : "Leggi di più all'interno",
    "Custom lists - click to rename!" : "Elenchi personalizzati: fai clic per rinominarli!",
    "To Do" : "Da fare",
    "In Progress" : "In corso",
    "Done" : "Fatto",
    "1. Open to learn more about boards and cards" : "1. Apri per sapere di più su lavagne e schede",
    "2. Drag cards left and right, up and down" : "2. Trascina le schede a sinistra e destra, sopra e sotto",
    "3. Apply rich formatting and link content" : "3. Applica la formattazione avanzata e collega il contenuto",
    "4. Share, comment and collaborate!" : "4. Condividi, commenta e collabora!",
    "Create your first card!" : "Crea la tua prima scheda!",
    "This comment has more than %s characters.\nAdded as an attachment to the card with name %s.\nAccessible on URL: %s." : "Questo commento contiene più di %s caratteri.\nAggiunto come allegato alla scheda con nome %s.\nAccessibile su URL: %s.",
    "Attachments" : "Allegati",
    "File" : "File",
    "date" : "data",
    "Card not found" : "Scheda non trovata",
    "Path is already shared with this card" : "Il percorso è già condiviso con questa scheda",
    "Invalid date, date format must be YYYY-MM-DD" : "Data non valida, il formato della data deve essere AAAA-MM-GG",
    "Personal planning and team project organization" : "Pianificazione personale e organizzazione dei progetti di squadra",
    "Deck is a kanban style organization tool aimed at personal planning and project organization for teams integrated with Nextcloud.\n\n\n- 📥 Add your tasks to cards and put them in order\n- 📄 Write down additional notes in Markdown\n- 🔖 Assign labels for even better organization\n- 👥 Share with your team, friends or family\n- 📎 Attach files and embed them in your Markdown description\n- 💬 Discuss with your team using comments\n- ⚡ Keep track of changes in the activity stream\n- 🚀 Get your project organized" : "Deck è uno strumento di organizzazione in stile kanban destinato alla pianificazione personale e all'organizzazione di progetti per squadre integrato con Nextcloud.\n\n\n- 📥 Aggiungi le tue attività alle schede e tienile in ordine\n- 📄 Scrivi note aggiuntive in Markdown\n- 🔖 Assegna etichette per un'organizzazione ancora migliore\n- 👥 Condividi con la tua squadra, amici o famiglia\n- 📎 Allega file e integrali nella tua descrizione Markdown\n- 💬 Discuti con la tua squadra utilizzando i commenti\n- ⚡ Tieni traccia dei cambiamenti nel flusso delle attività\n- 🚀 Organizza il tuo progetto",
    "Add board" : "Aggiungi lavagna",
    "Card details" : "Dettagli scheda",
    "Select the board to link to a project" : "Seleziona la lavagna da collegare a un progetto",
    "Search by board title" : "Cerca per titolo della lavagna",
    "Select board" : "Seleziona lavagna",
    "Move/copy card" : "Muovi/copia scheda",
    "Select a board" : "Seleziona una lavagna",
    "No lists available" : "Nessun elenco disponibile",
    "Select a list" : "Seleziona un elenco",
    "Move card" : "Sposta scheda",
    "Copy card" : "Copia scheda",
    "Select the card to link to a project" : "Seleziona la scheda da collegare a un progetto",
    "Link to card" : " Collega una scheda",
    "Select a card" : "Seleziona una scheda",
    "Cancel" : "Annulla",
    "This board is read only" : "Questa lavagna è in sola lettura",
    "Drop your files to upload" : "Rilascia i tuoi file per caricarli",
    "File already exists" : "Il file esiste già",
    "A file with the name {filename} already exists." : "Un file con il nome {filename} esiste già.",
    "Do you want to overwrite it?" : "Vuoi sovrascriverlo?",
    "Overwrite file" : "Sovrascrivi file",
    "Keep existing file" : "Mantieni file esistente",
    "Add card" : "Aggiungi scheda",
    "Archived cards" : "Schede archiviate",
    "Add list" : "Aggiungi elenco",
    "List name" : "Nome elenco",
    "Clear search" : "Svuota ricerca",
    "Active filters" : "Filtri attivi",
    "Apply filter" : "Applica filtro",
    "Filter by tag" : "Filtra per etichette",
    "Filter by assigned user" : "Filtra per utente assegnato",
    "Unassigned" : "Non assegnato",
    "Filter by status" : "Filtra per stato",
    "Open and completed" : "Aperte e completate",
    "Open" : "Aperte",
    "Completed" : "Completate",
    "Filter by due date" : "Filtra per data di scadenza",
    "Overdue" : "Scaduto",
    "Next 24 hours" : "Prossime 24 ore",
    "Next 7 days" : "7 giorni successivi",
    "Next 30 days" : "30 giorni successivi",
    "No due date" : "Nessuna data di scadenza",
    "Clear filter" : "Pulisci il filtro",
    "View Modes" : "Modalità di visualizzazione",
    "Toggle View Modes" : "Cambia modalità di visualizzazione",
    "Hide archived cards" : "Nascondi le schede archiviate",
    "Show archived cards" : "Mostra le schede archiviate",
    "Toggle compact mode" : "Commuta la modalità compatta",
    "Hide card cover images" : "Nascondi immagini di copertina della scheda",
    "Show card cover images" : "Mostra immagini di copertina della scheda",
    "Open details" : "Apri i dettagli",
    "Details" : "Dettagli",
    "Deck settings" : "Impostazioni di Deck",
    "General" : "Generale",
    "Use bigger card view" : "Usa la vista con schede più grandi",
    "Appearance" : "Aspetto",
    "Show card ID badge" : "Mostra ID della scheda",
    "Show boards in calendar/tasks" : "Mostra le lavagne in calendario/attività",
    "Admin settings" : "Impostazioni di amministratore",
    "Limit board creation to some groups" : "Limita la creazione di lavagne ad alcuni gruppi",
    "Users outside of those groups will not be able to create their own boards, but will still be able to work on boards that have been shared with them." : "Gli utenti che non fanno parte di tali gruppi non potranno creare le proprie lavagne, ma saranno ancora in grado di lavorare su quelle che sono state condivise con loro.",
    "Board actions" : "Azioni della lavagna",
    "Scroll sideways" : "Scorri lateralmente",
    "Navigate between cards" : "Naviga tra le schede",
    "Close card details" : "Chiudi dettagli scheda",
    "Search" : "Cerca",
    "Show card filters" : "Mostra filtri scheda",
    "Clear card filters" : "Rimuovi filtri scheda",
    "Show those shortcuts" : "Mostra quelle scorciatoie",
    "Card actions" : "Azioni della scheda",
    "Open card details" : "Apri dettagli della scheda",
    "Edit the card title" : "Modifica titolo della scheda",
    "Assign yourself to the current card" : "Assegna te stesso alla scheda attuale",
    "Archive/unarchive the current card" : "Archivia/recupera la scheda attuale",
    "Mark card as completed/not completed" : "Segna la scheda come completata/non completata",
    "Open card menu" : "Apri menu della scheda",
    "Currently present people" : "Persone attualmente presenti",
    "Loading board" : "Caricamento lavagna",
    "Board not found" : "Lavagna non trovata",
    "Create a new list to add cards to this board" : "Crea un nuovo elenco per aggiungere schede a questa lavagna",
    "Sharing" : "Condivisione",
    "Tags" : "Etichette",
    "Deleted items" : "Elementi eliminati",
    "Activity" : "Attività",
    "Deleted lists" : "Elenchi eliminati",
    "Undo" : "Annulla",
    "Deleted cards" : "Schede eliminate",
    "Hours" : "Ore",
    "Days" : "Giorni",
    "Weeks" : "Settimane",
    "Months" : "Mesi",
    "No cards yet" : "Ancora nessuna scheda",
    "Failed to create share with {displayName}" : "Creazione della condivisione con {displayName} non riuscita",
    "Are you sure you want to transfer the board {title} to {user}?" : "Sei sicuro di voler trasferire la lavagna {title} a {user}?",
    "Transfer the board." : "Trasferisci la bacheca.",
    "Transfer" : "Trasferisci",
    "The board has been transferred to {user}" : "La lavagna è stata trasferita a {user}",
    "Failed to transfer the board to {user}" : "Trasferimento della lavagna a {user} fallito",
    "Board owner" : "Proprietario della lavagna",
    "(Group)" : "(Gruppo)",
    "(Team)" : "(Squadra)",
    "(remote)" : "(remota)",
    "Can edit" : "Può modificare",
    "Can share" : "Può condividere",
    "Can manage" : "Può gestire",
    "Owner" : "Proprietario",
    "Delete" : "Elimina",
    "List deleted" : "Elenco eliminato",
    "Edit list title" : "Modifica titolo elenco",
    "Archive all cards" : "Archivia tutte le schede",
    "Unarchive all cards" : "Recupera tutte le schede dall'archivio",
    "Delete list" : "Elimina elenco",
    "Archive all cards in this list" : "Archivia tutte le schede in questo elenco",
    "Unarchive all cards in this list" : "Recupera tutte le schede in questo elenco",
    "Add a new card" : "Aggiungi una nuova scheda",
    "Card name" : "Nome scheda",
    "title and color value must be provided" : "devono essere forniti il titolo e il valore del colore",
    "Edit" : "Modifica",
    "Add a new tag" : "Aggiungi una nuova etichetta",
    "Board name" : "Nome della lavagna",
    "Members" : "Membri",
    "Assign to users/groups/team" : "Assegna a utenti/gruppi/squadre",
    "Assign a user to this card…" : "Assegna un utente a questa scheda…",
    "Select a user to assign to this card…" : "Seleziona un utente da assegnare a questa scheda…",
    "File to share" : "File da condividere",
    "Invalid path selected" : "Percorso selezionato non valido",
    "Upload new files" : "Carica nuovi file",
    "Share from Files" : "Condividi da File",
    "Pending share" : "Condivisione in attesa",
    "Add this attachment" : "Aggiungi questo allegato",
    "Show in Files" : "Mostra in File",
    "Download" : "Scarica",
    "Remove attachment" : "Rimuovi allegato",
    "Delete Attachment" : "Elimina allegato",
    "Restore Attachment" : "Ripristina allegato",
    "The title cannot be empty." : "Il titolo non può essere vuoto.",
    "Cannot close unsaved card!" : "Impossibile chiudere la card non salvata!",
    "Open in sidebar view" : "Apri nella barra laterale",
    "Open in bigger view" : "Apri in una vista più grande",
    "Comments" : "Commenti",
    "Failed to load comments" : "Caricamento commenti non riuscito",
    "No comments yet. Begin the discussion!" : "Ancora nessun commento. Inizia la discussione.",
    "The comment cannot be empty." : "Il commento non può essere vuoto.",
    "The comment cannot be longer than 1000 characters." : "Il commento non può essere più lungo di 1000 caratteri.",
    "Submit" : "Invia",
    "Created:" : "Creato:",
    "In reply to" : "In risposta a",
    "Cancel reply" : "Annulla risposta",
    "Reply" : "Rispondi",
    "Update" : "Aggiorna",
    "Mark as done" : "Segna come fatto",
    "Mark as not done" : "Segna come non fatto",
    "Write a description …" : "Scrivi una descrizione…",
    "Could not save description" : "Impossibile salvare la descrizione",
    "Description" : "Descrizione",
    "(Unsaved)" : "(Non salvata)",
    "(Saving…)" : "(Salvataggio...)",
    "Formatting help" : "Guida di formattazione",
    "Edit description" : "Modifica descrizione",
    "View description" : "Visualizza descrizione",
    "Add Attachment" : "Aggiungi allegato",
    "Choose attachment" : "Scegli allegato",
    "Select Date" : "Seleziona data",
    "Later today – {timeLocale}" : "Oggi – {timeLocale}",
    "Set due date for later today" : "Imposta scadenza per oggi",
    "Tomorrow – {timeLocale}" : "Domani – {timeLocale}",
    "Set due date for tomorrow" : "Imposta scadenza per domani",
    "This weekend – {timeLocale}" : "Questo fine settimana – {timeLocale}",
    "Set due date for this weekend" : "Imposta scadenza per questo fine settimana",
    "Next week – {timeLocale}" : "Prossima settimana – {timeLocale}",
    "Set due date for next week" : "Imposta scadenza per la prossima settimana",
    "Assign a due date to this card…" : "Assegna una scadenza a questa scheda…",
    "Set a due date" : "Imposta una data di scadenza",
    "Add due date" : "Aggiungi data di scadenza",
    "Choose a date" : "Scegli una data",
    "Remove due date" : "Rimuovi data di scadenza",
    "Due at:" : "Scadenza:",
    "Not done" : "Non fatto",
    "Unarchive card" : "Recupera lavagna dall'archivio",
    "Archive card" : "Archivia scheda",
    "Assign a tag to this card…" : "Assegna un'etichetta a questa scheda…",
    "Select or create a tag…" : "Seleziona o crea un'etichetta…",
    "Create a new tag:" : "Crea una nuova etichetta:",
    "(group)" : "(gruppo)",
    "{count} comments, {unread} unread" : "{count} commenti, {unread} non letti",
    "Todo items" : "Elementi da fare",
    "Edit card title" : "Modifica titolo scheda",
    "Open link" : "Apri collegamento",
    "Card deleted" : "Scheda eliminata",
    "Edit title" : "Modifica titolo",
    "Assign to me" : "Assegna a me",
    "Unassign myself" : "Rimuovi assegnazione da me",
    "Delete card" : "Elimina scheda",
    "All boards" : "Tutte le lavagne",
    "Archived boards" : "Lavagne archiviate",
    "Shared with you" : "Condiviso con te",
    "Cancel edit" : "Annulla modifica",
    "Save board" : "Salva bacheca",
    "Board {0} deleted" : "Lavagna {0} eliminata",
    "All cards" : "Tutte le schede",
    "Only assigned cards" : "Solo le schede assegnate",
    "No reminder" : "Nessun promemoria",
    "Are you sure you want to delete the board {title}? This will delete all the data of this board including archived cards." : "Sei sicuro di voler eliminare la lavagna {title}? Questo eliminerà tutti i dati di questa lavagna incluse le schede archiviate.",
    "Delete the board?" : "Vuoi eliminare la lavagna?",
    "Are you sure you want to leave the board {title}?" : "Sei sicuro di voler lasciare la bacheca{title}?",
    "Leave the board?" : "Vuoi lasciare la bacheca?",
    "Leave" : "Lascia",
    "Failed to leave the board" : "Impossibile lasciare la bacheca",
    "Exporting board..." : "Esportazione della lavagna...",
    "An error occurred" : "Si è verificato un errore",
    "Board details" : "Dettagli lavagna",
    "Edit board" : "Modifica lavagna",
    "Clone board" : "Clona lavagna",
    "Unarchive board" : "Recupera lavagna dall'archivio",
    "Archive board" : "Archivia lavagna",
    "Export board" : "Esporta lavagna",
    "Turn on due date reminders" : "Attiva promemoria per le date di scadenza",
    "Turn off due date reminders" : "Disattiva promemoria per le date di scadenza",
    "Due date reminders" : "Promemoria delle date di scadenza",
    "Assigned cards" : "Schede assegnate",
    "No notifications" : "Nessuna notifica",
    "Delete board" : "Elimina lavagna",
    "Leave board" : "Lascia bacheca",
    "Importing board..." : "Importazione della lavagna...",
    "Board imported successfully" : "Lavagna importata con successo",
    "Import board" : "Importa lavagna",
    "Clone {boardTitle}" : "Clona {boardTitle}",
    "Clone cards" : "Clona schede",
    "Clone assignments" : "Assegnazioni di clonazione",
    "Clone labels" : "Clona etichette",
    "Clone due dates" : "Clona date di scadenza",
    "Advanced options" : "Opzioni avanzate",
    "Move all cards to the first list" : "Aggiungi le tue attività alle schede e mettile in ordine",
    "Restore archived cards" : "Ripristina le schede archiviate",
    "Clone" : "Clona",
    "Export {boardTitle}" : "Esporta {boardTitle}",
    "Export as JSON" : "Esporta come JSON",
    "Export as CSV" : "Esporta come CSV",
    "Note: Only the JSON format is supported for importing back into the Deck app." : "Nota: per l'importazione nell'applicazione Deck è supportato solo il formato JSON.",
    "Export" : "Esporta",
    "Loading filtered view" : "Caricamento della vista filtrata",
    "Search for {searchQuery} in other boards" : "Cerca {searchQuery} in altre bacheche",
    "Search for {searchQuery} in all boards" : "Risultato di ricerca per \"{search}\" in tutte le lavagne",
    "No results found" : "Nessun risultato trovato",
    "Deck board {name}\n* Last modified on {lastMod}" : "Lavagna {name}\n* Ultima modifica: {lastMod}",
    "* Created on {created}\n* Last modified on {lastMod}\n* {nbAttachments} attachments\n* {nbComments} comments" : "* Creata: {created}\n* Ultima modifica: {lastMod}\n* {nbAttachments} allegati\n* {nbComments} commenti",
    "{nbCards} cards" : "{nbCards} schede",
    "Due on {date}" : "Scadenza: {date}",
    "{stack} in {board}" : "{stack} in {board}",
    "Click to expand description" : "Fai clic per espandere la descrizione",
    "Click to expand comment" : "Fai clic per espandere il commento",
    "Create card" : "Crea scheda",
    "Create a new card" : "Crea una nuova scheda",
    "Card title" : "Titolo scheda",
    "Creating the new card …" : "Creazione della nuova scheda...",
    "Card \"{card}\" was added to \"{board}\"" : "La scheda \"{card}\" è stata aggiunta a \"{board}\"",
    "Open card" : "Apri scheda",
    "Close" : "Chiudi",
    "No upcoming cards" : "Nessuna scheda rimasta",
    "upcoming cards today" : "prossime schede oggi",
    "upcoming cards tomorrow" : "prossime schede domani",
    "upcoming cards" : "prossime schede",
    "New card" : "Nuova scheda",
    "Link to a board" : "Collega a una lavagna",
    "Link to a card" : " Collega a una scheda",
    "Create a card" : "Crea una scheda",
    "Message from {author} in {conversationName}" : "Messaggio da {author} in {conversationName}",
    "Something went wrong" : "Qualcosa non ha funzionato",
    "Failed to upload {name}" : "Caricamenti di {name} non riuscito",
    "Maximum file size of {size} exceeded" : "Dimensione massima dei file di {size} superata",
    "Assigned users" : "Utenti assegnati",
    "Due date" : "Data di scadenza",
    "Created" : "Creato il",
    "Modified" : "Modificato",
    "Error creating the share" : "Errore durante la creazione della condivisione",
    "Share with a Deck card" : "Condividi con una scheda di Deck",
    "Share {file} with a Deck card" : "Condividi {file} con una scheda di Deck",
    "Share" : "Condividi",
    "The card \"%s\" on \"%s\" has reached its due date." : "La scheda \"%s\" in \"%s\" ha raggiunto la sua data di scadenza.",
    "Share board with a user, group or team …" : "Condividi lavagna con un utente, gruppo o una squadra …",
    "Searching for users, groups and teams …" : "Ricerca di utenti, gruppi e squadre …",
    "No participants found" : "Nessun partecipante trovato",
    "Save" : "Salva",
    "seconds ago" : "secondi fa",
    "Today" : "Oggi",
    "Tomorrow" : "Domani",
    "No due" : "Nessuna scadenza"
},
"nplurals=3; plural=n == 1 ? 0 : n != 0 && n % 1000000 == 0 ? 1 : 2;");
